/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.action;

import java.util.Iterator;

import org.eclipse.core.runtime.Assert;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PlatformUI;

import com.sun.org.apache.xalan.internal.xsltc.DOM;

import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts.DoMitVerbLinieEditPart;
import de.bsvrz.buv.plugin.anlagendarstellung.editor.GraphicalEditorToView;
import de.bsvrz.buv.plugin.anlagendarstellung.views.AnzeigenEinstellenDialog;

/**
 * Action zur Anzeige der Auswahl fr Anzeigen der Sonderprogramme. 
 * Es sollen nur Daten im Kontext Trichter verndert werden.
 * @author Pittner
 *
 */
public class ActionKontextTrichterAnzeigeABC implements IWorkbenchWindowActionDelegate
{
  private IWorkbenchPart _workbenchPart;

  @Override
  public void dispose()
  {
    // TODO Auto-generated method stub
  }

  @Override
  public void init(IWorkbenchWindow window)
  {
    _workbenchPart = window.getActivePage().getActivePart();
  }

  @Override
  public void run(IAction action)
  {
    GraphicalViewer viewer = getViewer();
    if (viewer != null)
    {
      ISelection selection = viewer.getSelection();

      if (selection instanceof StructuredSelection)
      {
        String secId = "";
        StructuredSelection ss = (StructuredSelection)selection;
        Iterator it = ss.iterator();
        AnzeigeModelTypAbstract model = null;
        while (it.hasNext())
        {
          
          
          Object obj = it.next();
          if (obj instanceof DoMitVerbLinieEditPart)
          {
            DoModelMitVerbLinie doModel = ((DoMitVerbLinieEditPart)obj).getModel();
            if (doModel instanceof AnzeigeModelTypAbstract)
            {
              if (!secId.isEmpty())
              {
                secId += ";";
              }
              model = ((AnzeigeModelTypAbstract)doModel);
              secId += model.getSystemObject().getPid();
            }
          }
        }

        if (_workbenchPart instanceof GraphicalEditorToView)
        {
          GraphicalEditorToView view = (GraphicalEditorToView)_workbenchPart;
          String secondId = view.getViewSite().getSecondaryId();
          secId += "#" + secondId;
          
          AnzeigenEinstellenDialog dialog = new AnzeigenEinstellenDialog(PlatformUI.getWorkbench()
              .getActiveWorkbenchWindow().getShell());
          dialog.oeffne(view.getViewSite().getId(), secId);
        }
      }
    }
    else
    {
      System.out.println("Viewer ist null");
    }
  }

  @Override
  public void selectionChanged(IAction action, ISelection selection)
  {
  }

  /**
   * @return
   */
  protected IWorkbenchPart getWorkbenchPart()
  {
    return _workbenchPart;
  }

  /**
   * Gibt den {@link GraphicalViewer} des Workbench Parts zurck.
   * 
   * @return der Viewer oder <code>null</code>, wenn es keinen gibt.
   */
  protected GraphicalViewer getViewer()
  {
    return (GraphicalViewer)getWorkbenchPart().getAdapter(GraphicalViewer.class);
  }

  /**
   * Gibt den Command Stack zurck.
   * 
   * @return der Command Stack, niemals <code>null</code>.
   */
  protected CommandStack getCommandStack()
  {

    CommandStack stack = getViewer().getEditDomain().getCommandStack();

    Assert.isNotNull(stack, "Command Stack nicht verfgbar.");
    return stack;
  }

}
