/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.action;

import java.text.SimpleDateFormat;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Event;
import org.eclipse.ui.IActionDelegate2;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.PickerVertical;
import de.bsvrz.buv.plugin.anlagendarstellung.util.SituationsWrapperProvider;
import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.BaustellenWrapper;
import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.SituationsWrapper;
import de.bsvrz.buv.plugin.dobj.internal.DObjPlugin;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.bitctrl.modell.ObjektFactory;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.att.Zeitstempel;
import de.bsvrz.sys.funclib.bitctrl.modell.fachmodellglobal.attribute.AttStreckeMeter;
import de.bsvrz.sys.funclib.bitctrl.modell.systemmodellglobal.objekte.Benutzer;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Baustelle;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Stau;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Unfall;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.parameter.PdSituationsEigenschaften;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.parameter.PdSituationsEigenschaften.Daten;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.StreckenAbschnitt;

/**
 * Action zur Anzeige des Tooltip fr Situationen (Baustellen, Unflle, Staus) in der Anlagendarstellung
 * 
 * @author Pittner
 * 
 */
public class ActionSituationToolTip implements IWorkbenchWindowActionDelegate, IActionDelegate2
{

  private IWorkbenchPart _workbenchPart;

  private ClientDavInterface _connection = DaVVerbindung.getInstanz().getDaVVerbindung();
  
  private final static SimpleDateFormat _sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
   
  
  

  @Override
  public void dispose()
  {
    // TODO Auto-generated method stub
  }

  @Override
  public void init(IWorkbenchWindow window)
  {
    _workbenchPart = window.getActivePage().getActivePart();
  }

  @Override
  public void run(IAction action)
  {
  }

  @Override
  public void selectionChanged(IAction action, ISelection selection)
  {
  }

  /**
   * @return
   */
  protected IWorkbenchPart getWorkbenchPart()
  {
    return _workbenchPart;
  }

  /**
   * Gibt den {@link GraphicalViewer} des Workbench Parts zurck.
   * 
   * @return der Viewer oder <code>null</code>, wenn es keinen gibt.
   */
  protected GraphicalViewer getViewer()
  {
    return (GraphicalViewer)getWorkbenchPart().getAdapter(GraphicalViewer.class);
  }

  /**
   * 
   * @param workbenchPart
   */
  protected void setWorkbenchPart(IWorkbenchPart workbenchPart)
  {
    _workbenchPart = workbenchPart;
  }

  /**
   * Gibt den Command Stack zurck.
   * 
   * @return der Command Stack, niemals <code>null</code>.
   */
  protected CommandStack getCommandStack()
  {
    CommandStack stack = getViewer().getEditDomain().getCommandStack();
    Assert.isNotNull(stack, "Command Stack nicht verfgbar.");
    return stack;
  }

  @Override
  public void init(IAction action)
  {
    // TODO Auto-generated method stub

  }

  @Override
  public void runWithEvent(IAction action, Event event)
  {
    
//    System.out.println("ActionSituationToolTip.runWithEvent(): " + event);
    
    GraphicalViewer viewer = getViewer();
    Object obj = event.data;
    if (viewer != null && obj != null)
    {
      DoModel doModel = null;
      if (obj instanceof DoModel)
      {
        doModel = (DoModel)obj;
          
        if (doModel instanceof PickerVertical)
        {
          SystemObject so = doModel.getSystemObject();
          ObjektFactory objektFactory = DObjPlugin.getDefault().getObjektFactory();
          final SystemObjekt systemObjekt = objektFactory.getModellobjekt(doModel.getSystemObject());
          
          SituationsWrapper wrapper = null;
          
          String s = "";
          
          if (systemObjekt instanceof Baustelle)
          {              
            s += "Baustelle: " + so;
          }
          else if (systemObjekt instanceof Unfall)
          {              
            s += "Unfall: " + so;
          }
          else if (systemObjekt instanceof Stau)
          {              
            s += "Stau: " + so;                          
          }
          wrapper = SituationsWrapperProvider.getInstanz().getSituationsWrapper(so.getPid());
          
          if (wrapper != null){
            StreckenAbschnitt streckenAbschnitt = wrapper.getStreckenAbschnitt();
            double bkmStart = streckenAbschnitt.getStartBetriebsKilometer();                  
            double bkmEnde = streckenAbschnitt.getEndBetriebsKilometer();
            
            PdSituationsEigenschaften pd = wrapper.getSituation().getPdSituationsEigenschaften();
            Daten datum = pd.getDatum();
            
            String strlaenge = "undefiniert";
            AttStreckeMeter laenge = datum.getLaenge();
            if (laenge != null)
              strlaenge = "" + laenge.getValue();
            Benutzer benutzerReferenz = datum.getUrlasser().getBenutzerReferenz();
            String name = "";
            if (benutzerReferenz != null)
              name += benutzerReferenz.getName();
              
            Long dauer = wrapper.getDauer();
            Long dauerInMin = new Long(0);
            if (dauer != null && dauer > 0L){
                dauerInMin = dauer / (60 * 1000);
            }

            s += "\nBKM Start: " + bkmStart;
            s += "\nBKM Ende: " + bkmEnde;            
            s += "\nLnge (in Meter): " + strlaenge;
            s += "\nStartzeit : " + _sdf.format(wrapper.getStartZeit());
            s += "\nDauer (in Minuten): " + dauerInMin;
            s += "\nBenutzer: " + name;
                          
            EStructuralFeature feature = AnlagendarstellungPackage.Literals.PICKER_VERTICAL__DARSTELLUNG_UPDATE;
            final SetCommand cmd = new SetCommand(doModel, feature, s);
            getCommandStack().execute(cmd);                  
          }
        }   
      }
    }
  }
}
