/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.action;

import java.util.List;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.StreckenStationModel;
import de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts.StreckenStationEditPart;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.StreckenstationFigure;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Action zur Anzeige des Tooltips zum Aufbau einer Streckenstation
 * 
 * @author Pittner
 *
 */
public class ActionStreckenStationToolTip implements IWorkbenchWindowActionDelegate
{

  private IWorkbenchPart _workbenchPart;

  @Override
  public void dispose()
  {
    // TODO Auto-generated method stub
  }

  @Override
  public void init(IWorkbenchWindow window)
  {
    _workbenchPart = window.getActivePage().getActivePart();
  }

  @Override
  public void run(IAction action)
  {
    GraphicalViewer viewer = getViewer();
    if (viewer != null)
    {
      Point pt = Display.getCurrent().getCursorLocation();
      Point pt2 = viewer.getControl().toControl(pt);
      org.eclipse.draw2d.geometry.Point point = new org.eclipse.draw2d.geometry.Point(pt2);

      Object obj = viewer.findObjectAt(point);

      StreckenStationModel doModel = null;
      StreckenstationFigure figure = null;

      if (obj instanceof StreckenStationEditPart)
      {
        StreckenStationEditPart domep = (StreckenStationEditPart)obj;
        figure = (StreckenstationFigure)domep.getFigure();
        doModel = (StreckenStationModel)domep.getModel();

        SystemObject so = doModel.getSystemObject();

        if (so != null)
        {
          String s = "Sm: " + so.toString();
          ConfigurationObject co = (ConfigurationObject)so;
          NonMutableSet set = co.getNonMutableSet("Eak");
          if (set != null)
          {
            List<SystemObject> eaks = set.getElements();
            for (SystemObject so2 : eaks)
            {
              ConfigurationObject co2 = (ConfigurationObject)so2;
              s += "\n  Eak: " + so2;
              NonMutableSet set2 = co2.getNonMutableSet("De");
              if (set != null)
              {
                List<SystemObject> eaks2 = set2.getElements();
                for (SystemObject so3 : eaks2)
                {
                  s += "\n    De: " + so3;
                }
              }
            }
          }
          EStructuralFeature feature = AnlagendarstellungPackage.Literals.STRECKEN_STATION_MODEL__DARSTELLUNG_UPDATE;

          final SetCommand cmd = new SetCommand(doModel, feature, s);
          getCommandStack().execute(cmd);

        }
      }
    }
  }

  @Override
  public void selectionChanged(IAction action, ISelection selection)
  {
  }

  /**
   * @return
   */
  protected IWorkbenchPart getWorkbenchPart()
  {
    return _workbenchPart;
  }

  /**
   * Gibt den {@link GraphicalViewer} des Workbench Parts zurck.
   * 
   * @return der Viewer oder <code>null</code>, wenn es keinen gibt.
   */
  protected GraphicalViewer getViewer()
  {
    return (GraphicalViewer)getWorkbenchPart().getAdapter(GraphicalViewer.class);
  }

  /**
   * 
   * @param workbenchPart
   */
  protected void setWorkbenchPart(IWorkbenchPart workbenchPart)
  {
    _workbenchPart = workbenchPart;
  }

  /**
   * Gibt den Command Stack zurck.
   * 
   * @return der Command Stack, niemals <code>null</code>.
   */
  protected CommandStack getCommandStack()
  {
    CommandStack stack = getViewer().getEditDomain().getCommandStack();
    Assert.isNotNull(stack, "Command Stack nicht verfgbar.");
    return stack;
  }

}
