/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener;

import java.util.Iterator;
import java.util.concurrent.CopyOnWriteArrayList;

import de.bsvrz.dav.daf.main.ResultData;

/**
 * Die Klasse kapselt die Verwaltungsmethoden fr die Listener-Funktionalitt des IDaVDataUpdateListener. Diese Klasse
 * implemntiert das Singleton-Pattern und stellt dadurch sicher dass der Listener nur einmal Instanziiert wird.
 * 
 * 
 * @author Pittner
 * 
 */
public class DaVDataUpdateListener
{
  /**
   * Liste der {@link IDaVDataUpdateListener}, die ber einen neue DaV-Daten informiert werden mchten.
   */  
  private CopyOnWriteArrayList<IDaVDataUpdateListener> _daVDataUpdateListener = new CopyOnWriteArrayList<IDaVDataUpdateListener>();

  /**
   * Singleton-Instanz
   */
  private static DaVDataUpdateListener _instanz = null;

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static DaVDataUpdateListener getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new DaVDataUpdateListener();
    }
    return _instanz;
  }

  /**
   * Fgt einen Listener zur Liste der DaVDataUpdate-Listener hinzu.
   * 
   * @param listener
   */
  public void addDaVDataUpdateListener(IDaVDataUpdateListener listener)
  {
    if (!_daVDataUpdateListener.contains(listener))
    {
      _daVDataUpdateListener.add(listener);
    }
    else
    {
      System.err.println("Der Listener ist bereits in der Zuhrerliste vorhanden und wird nicht erneut hinzugefgt");
    }
  }

  /**
   * Benachrichtigt alle registrierten {@link IDaVDataUpdateListener} darber, dass neue Daten vorhanden sind.
   * 
   */
  public void notifyDaVDataUpdateListener(ResultData result)
  {

    DaVDataUpdateEvent event = new DaVDataUpdateEvent(this, result);

    Iterator<IDaVDataUpdateListener> it = _daVDataUpdateListener.iterator();
    while (it.hasNext())
    {
      IDaVDataUpdateListener listener = it.next();
      listener.daVDataUpdate(event);
    }
  }

  /**
   * Entfernt einen {@link IDaVDataUpdateListener} von der Liste der Zuhrer.
   * 
   * @param IDaVDataUpdateListener
   *          Zu entfernender {@link IDaVDataUpdateListener}.
   */
  public void removeDaVDataUpdateListener(IDaVDataUpdateListener listener)
  {
    if (_daVDataUpdateListener.contains(listener))
    {
      _daVDataUpdateListener.remove(listener);
    }
    else
    {
      System.err.println("Der Listener ist in der Zuhrerliste nicht vorhanden und wird somit nicht entfernt.");
    }
  }

}
