/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editpolicies.ResizableEditPolicy;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;

import de.bsvrz.buv.plugin.anlagendarstellung.AQModelBezeichnung;
import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.BezeichnungMQFigure;
import de.bsvrz.buv.plugin.dobj.model.DoModel;

/**
 * Implementierung fr einen {@link DoMitVerbLinieEditPart} der ein Model-Objekt 
 * vom Typ {@link AQModelBezeichnung} verwendet.
 * 
 * @author Pittner
 * 
 */
public class AQBezeichnungEditPart extends DoMitVerbLinieEditPart
{
  @Override
  protected BezeichnungMQFigure createFigure()
  {

    Point pos = getCastedModel().getLocation();
    Dimension size = getCastedModel().getSize();

    if (size == null)
      size = new Dimension(50, 20);

    String bez = getCastedModel().getBezeichnung();
    String kilo = getCastedModel().getKilometrierung();

    if (bez == null || bez.length() == 0)
      bez = "MQ 123";

    if (kilo == null || kilo.length() == 0)
      kilo = "km 1,0";

    RGB rgb = (RGB)getCastedModel().getBackground();

    BezeichnungMQFigure fig = new BezeichnungMQFigure(pos.x, pos.y, bez, kilo);

    if (rgb != null)
      fig.setBackgroundColor(new Color(null, rgb));

    addToolTipListener(fig);

    return fig;
  }

  @Override
  public void notifyChanged(Notification notification)
  {
    super.notifyChanged(notification);

    final Object notifier = notification.getNotifier();
    final int type = notification.getEventType();
    if (notifier instanceof AQModelBezeichnung)
    {
      final int featureID = notification.getFeatureID(DoModel.class);
      switch (type)
      {
        case Notification.SET:
          switch (featureID)
          {
            case AnlagendarstellungPackage.AQ_MODEL_BEZEICHNUNG__SIZE:
              Dimension size = (Dimension)notification.getNewValue();
              BezeichnungMQFigure figure = (BezeichnungMQFigure)getFigure();
              figure.getLabelBezeichnung().setSize(size.width, figure.getLabelBezeichnung().getSize().height);
              figure.getLabelKilometer().setSize(size.width, figure.getLabelKilometer().getSize().height);
              refresh();
              break;
            case AnlagendarstellungPackage.AQ_MODEL_BEZEICHNUNG__BACKGROUND:
              refresh();
              break;
          }

          break;
        default:
      }
    }

    if (AnlagendarstellungPackage.Literals.AQ_MODEL_BEZEICHNUNG__BEZEICHNUNG.equals(notification.getFeature()))
    {
      String bez = notification.getNewStringValue();
      BezeichnungMQFigure fig = (BezeichnungMQFigure)getFigure();
      fig.getLabelBezeichnung().setText(bez);
      getFigure().repaint();

    }
    else if (AnlagendarstellungPackage.Literals.AQ_MODEL_BEZEICHNUNG__KILOMETRIERUNG.equals(notification.getFeature()))
    {
      String kilo = notification.getNewStringValue();
      BezeichnungMQFigure fig = (BezeichnungMQFigure)getFigure();
      fig.getLabelKilometer().setText(kilo);
      getFigure().repaint();

    }
    else if (AnlagendarstellungPackage.Literals.AQ_MODEL_BEZEICHNUNG__KONTEXT.equals(notification.getFeature()))
    {

    }
    else if (AnlagendarstellungPackage.Literals.AQ_MODEL_BEZEICHNUNG__BACKGROUND.equals(notification.getFeature()))
    {

      Object obj = notification.getNewValue();

      if (obj instanceof RGB)
      {
        RGB rgb = (RGB)obj;
        getFigure().setBackgroundColor(new Color(null, rgb));
      }
      getFigure().repaint();
    }
    else if (AnlagendarstellungPackage.Literals.AQ_MODEL_BEZEICHNUNG__DARSTELLUNG_UPDATE.equals(notification
        .getFeature()))
    {
      Object object = notification.getNewValue();
      if (object instanceof String)
      {
        String s = (String)notification.getNewValue();
        getFigure().setToolTip(new Label(s));
      }
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
   */
  protected void refreshVisuals()
  {
    Point pos = getModel().getLocation();
    Dimension size = getCastedModel().getSize();

    getFigure().setLocation(pos);
    if (size != null)
      getFigure().setSize(size);
  }

  @Override
  protected void createEditPolicies()
  {
    super.createEditPolicies();
    installEditPolicy(EditPolicy.LAYOUT_ROLE, new ResizableEditPolicy());
  }

  /**
   * @return
   */
  protected AQModelBezeichnung getCastedModel()
  {
    AQModelBezeichnung model = null;
    if (getModel() instanceof AQModelBezeichnung)
    {
      model = (AQModelBezeichnung)getModel();
    }
    return model;
  }

}
