/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editpolicies.ResizableEditPolicy;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnschlussStelleModel;
import de.bsvrz.buv.plugin.anlagendarstellung.DetektorLVE;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnschlussStelleFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionVerkehrsLage;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.ILegendeBaustein2;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnschlussStelleTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.util.LegendeHelper;
import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.dobj.internal.DarstellungExtensionPoint;
import de.bsvrz.buv.plugin.dobj.model.ConfiguratedDoTyp;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.plugin.dobj.model.DoTyp;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.ILegendeBaustein;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.LegendeBaustein;
import de.bsvrz.dav.daf.main.ResultData;

/**
 * Implementierung fr einen {@link DoMitVerbLinieEditPart} der ein Model-Objekt 
 * vom Typ {@link AnschlussStelleModel} verwendet.
 * 
 * @author Pittner
 * 
 */
public class AnschlussStelleEditPart extends DoMitVerbLinieEditPart implements ILegendeBaustein2
{
  private ArrayList<ILegendeBaustein> legendenBausteine;

  @Override
  protected List getModelChildren()
  {
    if (getCastedModel().getKomponenten() != null && getCastedModel().getKomponenten().size() > 0)
    {
      return getCastedModel().getKomponenten();
    }
    else
    {
      ConfiguratedDoTyp doTyp = DarstellungExtensionPoint.getDoTyp("DetektorLveEditPart");
      Darstellung darstellung = (Darstellung)getViewer().getProperty(Darstellung.class.toString());
      EList<DoTyp> doTypen = darstellung.getDoTypen();
      boolean vorhanden = false;
      for (DoTyp typ : doTypen)
      {
        if (typ instanceof ConfiguratedDoTyp)
        {
          ConfiguratedDoTyp co = (ConfiguratedDoTyp)typ;
          if (co.getId().equals(doTyp.getId()))
          {
            vorhanden = true;
            doTyp = co;
          }
        }
      }

      DetektorLVE d1 = (DetektorLVE)doTyp.createModel();
      d1.setLocation(new Point(1, 10));
      getCastedModel().getKomponenten().add(d1);

      if (!vorhanden)
      {
        doTypen.add(doTyp);
      }

      return getCastedModel().getKomponenten();
    }
  }

  @Override
  protected IFigure createFigure()
  {
    Point pos = getModel().getLocation();
    Dimension size = getCastedModel().getSize();
    if (size == null)
    {
      size = new Dimension(25, 40);
      getCastedModel().setSize(size);
    }

    int staerke = getCastedModel().getStaerke();
    if (staerke < 1)
    {
      getCastedModel().setStaerke(10);
    }

    AnschlussStelleFigure fig = new AnschlussStelleFigure("", pos, size, getCastedModel().getStaerke());
    fig.setSize(size);

//    DoModelMitVerbLinie target = getModel().getTarget();
//
//    if (target != null)
//    {
////      target.setTarget(getModel());
//      createTargetConnection(getModel(), target);
//    }

    addToolTipListener(fig);

    return fig;
  }

  @Override
  public void notifyChanged(Notification notification)
  {
    super.notifyChanged(notification);

    final Object notifier = notification.getNotifier();
    final int type = notification.getEventType();

    AnschlussStelleFigure fig = (AnschlussStelleFigure)getFigure();

    if (notifier instanceof AnschlussStelleModel)
    {
      final int featureID = notification.getFeatureID(DoModel.class);

      switch (type)
      {
        case Notification.SET:
          switch (featureID)
          {
            case AnlagendarstellungPackage.FAHR_STREIFEN_MODEL_ABSTRACT__LOCATION:
              refresh();
              break;
            case AnlagendarstellungPackage.FAHR_STREIFEN_MODEL_ABSTRACT__SIZE:
              refresh();

              break;
          }

          break;
        default:
      }
    }

    if (AnlagendarstellungPackage.Literals.FAHR_STREIFEN_MODEL_ABSTRACT__KONTEXT.equals(notification.getFeature()))
    {
      Point pos = getModel().getLocation();
      Dimension size = getCastedModel().getSize();
      int kontext = notification.getNewIntValue();
      AnschlussStelleTyp typ = AnschlussStelleTyp.getTypeType(kontext);
      fig.createPointList(typ, pos, size, getCastedModel().getStaerke());
      refresh();
    }
    else if (AnlagendarstellungPackage.Literals.FAHR_STREIFEN_MODEL_ABSTRACT__DATA_UPDATE.equals(notification
        .getFeature()))
    {
      ResultData result = (ResultData)notification.getNewValue();
      dataUpdateAnschlussStelle(result);
    }
    else if (AnlagendarstellungPackage.Literals.FAHR_STREIFEN_MODEL_ABSTRACT__DARSTELLUNG_UPDATE.equals(notification
        .getFeature()))
    {
      Object object = notification.getNewValue();
      if (object instanceof String)
      {
        String s = (String)notification.getNewValue();
        getFigure().setToolTip(new Label(s));
      }
    }
    else if (AnlagendarstellungPackage.Literals.ANSCHLUSS_STELLE_MODEL__STAERKE.equals(notification.getFeature()))
    {
      Point pos = getModel().getLocation();
      Dimension size = getCastedModel().getSize();
      int staerke = notification.getNewIntValue();
      AnschlussStelleTyp typ = AnschlussStelleTyp.getTypeType(getCastedModel().getKontext());
      fig.createPointList(typ, pos, size, staerke);
      refresh();
    }
  }

  /**
   * We don't need to explicitly handle refresh visuals because the times when this needs to be done it is handled by
   * the table e.g. handleNameChange()
   */
  protected void refreshVisuals()
  {
    Point pos = getModel().getLocation();
    Dimension size = getCastedModel().getSize();

    if (size != null)
    {
      int kontext = getCastedModel().getKontext();
      AnschlussStelleTyp typ = AnschlussStelleTyp.getTypeType(kontext);
      AnschlussStelleFigure fig = (AnschlussStelleFigure)getFigure();
      fig.createPointList(typ, pos, size, getCastedModel().getStaerke());
    }

    getFigure().setLocation(pos);
    if (size != null)
      getFigure().setSize(size);

    List list = getChildren();
    if (list != null)
    {
      for (Object object : list)
      {
        if (object instanceof DetektorLveEditPart)
        {
          DetektorLveEditPart ep = (DetektorLveEditPart)object;
          ep.refresh();

        }
      }
    }

    getFigure().setBackgroundColor(ColorConstants.gray);
  }

  @Override
  protected void createEditPolicies()
  {
    super.createEditPolicies();
    installEditPolicy(EditPolicy.LAYOUT_ROLE, new ResizableEditPolicy());
  }

  /**
   * @return
   */
  protected AnschlussStelleModel getCastedModel()
  {
    AnschlussStelleModel model = null;
    if (getModel() instanceof AnschlussStelleModel)
    {
      model = (AnschlussStelleModel)getModel();
    }
    return model;
  }

  @Override
  public String getId()
  {
    // TODO Auto-generated method stub
    return "Fahrstreifen";
  }

  /** {@inheritDoc} */
  @Override
  public List<ILegendeBaustein> getBausteine() {
    if (legendenBausteine == null) {
      legendenBausteine = new ArrayList<ILegendeBaustein>();
      for (Object listObj : getDefinition())
      {
        legendenBausteine.addAll(LegendeHelper.erzeugeLegendeBausteine(listObj));        
      }
    }
    return legendenBausteine;
  }

  @Override
  public Image getIcon()
  {
    // TODO Auto-generated method stub
    return null;
  }

  @Override
  public String getText()
  {
    // TODO Auto-generated method stub
    return "Fahrstreifen (Verkehrslage MARZ)";
  }

  @Override
  public List getDefinition()
  {
    // TODO Auto-generated method stub
    List list = new ArrayList();
    list.add(LegendeDefinitionVerkehrsLage.class);
    return list;
  }

}
