/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.swt.graphics.Image;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeQuerschnittFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeQuerschnittLinksFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeQuerschnittRechtsFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.ILegendeBaustein2;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionAnzeigeQuerschnitt;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionAnzeigeQuerschnittProgramm;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionAnzeigeQuerschnittStatus;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionAnzeigeQuerschnittStatusBefehle;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeQuerschnittTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.util.LegendeHelper;
import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.dobj.internal.DarstellungExtensionPoint;
import de.bsvrz.buv.plugin.dobj.model.ConfiguratedDoTyp;
import de.bsvrz.buv.plugin.dobj.model.DoTyp;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.ILegendeBaustein;
import de.bsvrz.dav.daf.main.ResultData;

/**
 * Implementierung fr einen {@link DoMitVerbLinieEditPart} der ein Model-Objekt 
 * vom Typ {@link AnzeigeQuerschnittModelTypAbstract} verwendet.
 * 
 * @author Pittner
 * 
 */
public class AnzeigeQuerschnittTyp21A8EditPart extends DoMitVerbLinieEditPart implements ILegendeBaustein2
{
  protected AnzeigeQuerschnittLinksFigure _contentPane = null;
  private ArrayList<ILegendeBaustein> legendenBausteine;

  @Override
  public IFigure getContentPane()
  {
    // TODO Auto-generated method stub
    return _contentPane;
  }

  @Override
  protected List getModelChildren()
  {
    if (getCastedModel().getKomponenten() != null && getCastedModel().getKomponenten().size() > 0)
    {
      return getCastedModel().getKomponenten();
    }
    else
    {
      ConfiguratedDoTyp doTyp1 = DarstellungExtensionPoint.getDoTyp("AnzeigeTypAEditPart");
      ConfiguratedDoTyp doTyp2 = DarstellungExtensionPoint.getDoTyp("AnzeigeTypBEditPart");
      ConfiguratedDoTyp doTyp3 = DarstellungExtensionPoint.getDoTyp("AnzeigeTypC2EditPart");
      ConfiguratedDoTyp doTyp4 = DarstellungExtensionPoint.getDoTyp("AnzeigeTypDEditPart");
      ConfiguratedDoTyp doTyp5 = DarstellungExtensionPoint.getDoTyp("AnzeigeTypD2EditPart");
      ConfiguratedDoTyp doTyp6 = DarstellungExtensionPoint.getDoTyp("AnzeigeTypBlinkerEditPart");
      ConfiguratedDoTyp doTyp7 = DarstellungExtensionPoint.getDoTyp("AnzeigeTypTEditPart");

      Darstellung darstellung = (Darstellung)getViewer().getProperty(Darstellung.class.toString());
      EList<DoTyp> doTypen = darstellung.getDoTypen();

      boolean vorhanden1 = false;
      boolean vorhanden2 = false;
      boolean vorhanden3 = false;
      boolean vorhanden4 = false;
      boolean vorhanden5 = false;
      boolean vorhanden6 = false;
      boolean vorhanden7 = false;

      for (DoTyp typ : doTypen)
      {
        if (typ instanceof ConfiguratedDoTyp)
        {
          ConfiguratedDoTyp co = (ConfiguratedDoTyp)typ;
          if (co.getId().equals(doTyp1.getId()))
          {
            vorhanden1 = true;
            doTyp1 = co;
          }
          if (co.getId().equals(doTyp2.getId()))
          {
            vorhanden2 = true;
            doTyp2 = co;
          }
          if (co.getId().equals(doTyp3.getId()))
          {
            vorhanden3 = true;
            doTyp3 = co;
          }
          if (co.getId().equals(doTyp4.getId()))
          {
            vorhanden4 = true;
            doTyp4 = co;
          }
          if (co.getId().equals(doTyp5.getId()))
          {
            vorhanden5 = true;
            doTyp5 = co;
          }
          if (co.getId().equals(doTyp6.getId()))
          {
            vorhanden6 = true;
            doTyp6 = co;
          }
          if (co.getId().equals(doTyp7.getId()))
          {
            vorhanden7 = true;
            doTyp7 = co;
          }
        }
      }

      AnzeigeModelTypAbstract a1 = (AnzeigeModelTypAbstract)doTyp1.createModel();
      a1.setLocation(new Point(52, 17));
      getCastedModel().getKomponenten().add(a1);

      AnzeigeModelTypAbstract b1 = (AnzeigeModelTypAbstract)doTyp2.createModel();
      b1.setLocation(new Point(86, 17));
      getCastedModel().getKomponenten().add(b1);

      AnzeigeModelTypAbstract c1 = (AnzeigeModelTypAbstract)doTyp3.createModel();
      c1.setLocation(new Point(72, 51));
      getCastedModel().getKomponenten().add(c1);

      AnzeigeModelTypAbstract a2 = (AnzeigeModelTypAbstract)doTyp1.createModel();
      a2.setLocation(new Point(128, 17));
      getCastedModel().getKomponenten().add(a2);

      AnzeigeModelTypAbstract b2 = (AnzeigeModelTypAbstract)doTyp2.createModel();
      b2.setLocation(new Point(162, 17));
      getCastedModel().getKomponenten().add(b2);

      AnzeigeModelTypAbstract c2 = (AnzeigeModelTypAbstract)doTyp3.createModel();
      c2.setLocation(new Point(148, 51));
      getCastedModel().getKomponenten().add(c2);

      AnzeigeModelTypAbstract a3 = (AnzeigeModelTypAbstract)doTyp1.createModel();
      a3.setLocation(new Point(204, 17));
      getCastedModel().getKomponenten().add(a3);

      AnzeigeModelTypAbstract b3 = (AnzeigeModelTypAbstract)doTyp2.createModel();
      b3.setLocation(new Point(238, 17));
      getCastedModel().getKomponenten().add(b3);

      AnzeigeModelTypAbstract c3 = (AnzeigeModelTypAbstract)doTyp3.createModel();
      c3.setLocation(new Point(224, 51));
      getCastedModel().getKomponenten().add(c3);

      AnzeigeModelTypAbstract a4 = (AnzeigeModelTypAbstract)doTyp1.createModel();
      a4.setLocation(new Point(280, 17));
      getCastedModel().getKomponenten().add(a4);

      AnzeigeModelTypAbstract d = (AnzeigeModelTypAbstract)doTyp4.createModel();
      d.setLocation(new Point(72, 69));
      getCastedModel().getKomponenten().add(d);

      AnzeigeModelTypAbstract d2 = (AnzeigeModelTypAbstract)doTyp5.createModel();
      d2.setLocation(new Point(296, 51));
      getCastedModel().getKomponenten().add(d2);

      AnzeigeModelTypAbstract t1 = (AnzeigeModelTypAbstract)doTyp7.createModel();
      t1.setLocation(new Point(1, 35));
      getCastedModel().getKomponenten().add(t1);

      AnzeigeModelTypAbstract t2 = (AnzeigeModelTypAbstract)doTyp7.createModel();
      t2.setLocation(new Point(314, 35));
      getCastedModel().getKomponenten().add(t2);

      AnzeigeModelTypAbstract bl1 = (AnzeigeModelTypAbstract)doTyp6.createModel();
      bl1.setLocation(new Point(18, 17));
      getCastedModel().getKomponenten().add(bl1);

      AnzeigeModelTypAbstract bl2 = (AnzeigeModelTypAbstract)doTyp6.createModel();
      bl2.setLocation(new Point(330, 17));
      getCastedModel().getKomponenten().add(bl2);

      if (!vorhanden1)
      {
        doTypen.add(doTyp1);
      }
      if (!vorhanden2)
      {
        doTypen.add(doTyp2);
      }
      if (!vorhanden3)
      {
        doTypen.add(doTyp3);
      }
      if (!vorhanden4)
      {
        doTypen.add(doTyp4);
      }
      if (!vorhanden5)
      {
        doTypen.add(doTyp5);
      }
      if (!vorhanden6)
      {
        doTypen.add(doTyp6);
      }
      if (!vorhanden7)
      {
        doTypen.add(doTyp7);
      }

      return getCastedModel().getKomponenten();
    }
  }

  @Override
  protected AnzeigeQuerschnittFigure createFigure()
  {

    AnzeigeQuerschnittRechtsFigure rechts = null;
    AnzeigeQuerschnittFigure result = null;

    _contentPane = new AnzeigeQuerschnittLinksFigure(AnzeigeQuerschnittTyp.WVZ_A8_Typ21, 339);
    _contentPane.setSize(369, 97);
    rechts = new AnzeigeQuerschnittRechtsFigure(getCastedModel().getKontext());
    rechts.setSize(35, 97);

    result = new AnzeigeQuerschnittFigure(_contentPane, rechts);
    result.setSize(399, 97);

    result.switchLinksRechts(getCastedModel().getFahrtrichtung());

    String text = getCastedModel().getBezeichnung();
    if (text != null)
    {
      result.setExtraText(text);
    }

    addToolTipListener(result);

    return result;
  }

  @Override
  public void notifyChanged(Notification notification)
  {
    super.notifyChanged(notification);
    if (AnlagendarstellungPackage.Literals.ANZEIGE_QUERSCHNITT_MODEL_TYP_ABSTRACT__BEZEICHNUNG.equals(notification
        .getFeature()))
    {

      ((AnzeigeQuerschnittFigure)getFigure()).setExtraText(notification.getNewStringValue());
      getFigure().repaint();

    }
    else if (AnlagendarstellungPackage.Literals.ANZEIGE_QUERSCHNITT_MODEL_TYP_ABSTRACT__KONTEXT.equals(notification
        .getFeature()))
    {
    }
    else if (AnlagendarstellungPackage.Literals.ANZEIGE_QUERSCHNITT_MODEL_TYP_ABSTRACT__FAHRTRICHTUNG
        .equals(notification.getFeature()))
    {

      int fr = notification.getNewIntValue();

      AnzeigeQuerschnittFigure figure = ((AnzeigeQuerschnittFigure)getFigure());
      figure.switchLinksRechts(fr);

      EList<AnzeigeModelTypAbstract> anzeigen = getCastedModel().getKomponenten();

      if (getCastedModel().getFahrtrichtung() != 1)
      {
        // Typ A,B,C
        anzeigen.get(0).setLocation(new Point(52, 17));
        anzeigen.get(1).setLocation(new Point(86, 17));
        anzeigen.get(2).setLocation(new Point(72, 51));
        // Typ A,B,C
        anzeigen.get(3).setLocation(new Point(128, 17));
        anzeigen.get(4).setLocation(new Point(162, 17));
        anzeigen.get(5).setLocation(new Point(148, 51));
        // Typ A,B,C
        anzeigen.get(6).setLocation(new Point(204, 17));
        anzeigen.get(7).setLocation(new Point(238, 17));
        anzeigen.get(8).setLocation(new Point(224, 51));
        // Typ A
        anzeigen.get(9).setLocation(new Point(280, 17));
        // Typ D
        anzeigen.get(10).setLocation(new Point(72, 69));
        // Typ d2
        anzeigen.get(11).setLocation(new Point(293, 51));
        // Typ T
        anzeigen.get(12).setLocation(new Point(1, 35));
        anzeigen.get(13).setLocation(new Point(314, 35));
        // Blinker
        anzeigen.get(14).setLocation(new Point(18, 17));
        anzeigen.get(15).setLocation(new Point(330, 17));
      }
      else
      {
        int offset = 24;

        // Typ A,B,C
        anzeigen.get(9).setLocation(new Point(52, 19 + offset));
        anzeigen.get(7).setLocation(new Point(86, 19 + offset));
        anzeigen.get(8).setLocation(new Point(72, 1 + offset));
        // Typ A,B,C
        anzeigen.get(6).setLocation(new Point(128, 19 + offset));
        anzeigen.get(4).setLocation(new Point(162, 19 + offset));
        anzeigen.get(5).setLocation(new Point(148, 1 + offset));
        // Typ A,B,C
        anzeigen.get(3).setLocation(new Point(204, 19 + offset));
        anzeigen.get(1).setLocation(new Point(238, 19 + offset));
        anzeigen.get(2).setLocation(new Point(224, 1 + offset));
        // Typ A
        anzeigen.get(0).setLocation(new Point(280, 19 + offset));
        // Typ D
        anzeigen.get(10).setLocation(new Point(72, 1));
        // Typ d2
        anzeigen.get(11).setLocation(new Point(54, 1 + offset));
        // Typ T
        anzeigen.get(13).setLocation(new Point(1, 9));
        anzeigen.get(12).setLocation(new Point(314, 9));
        // Blinker
        anzeigen.get(15).setLocation(new Point(18, 59));
        anzeigen.get(14).setLocation(new Point(330, 59));

      }
    }
    else if (AnlagendarstellungPackage.Literals.ANZEIGE_QUERSCHNITT_MODEL_TYP_ABSTRACT__DATA_UPDATE.equals(notification
        .getFeature()))
    {
      ResultData result = (ResultData)notification.getNewValue();
      dataUpdateAnzeigeQuerschnittModelTypAbstract(result, getCastedModel());
    }
    else if (AnlagendarstellungPackage.Literals.ANZEIGE_QUERSCHNITT_MODEL_TYP_ABSTRACT__DARSTELLUNG_UPDATE
        .equals(notification.getFeature()))
    {
      String s = (String)notification.getNewValue();
      getFigure().setToolTip(new Label(s));
    }
  }

  /**
   * We don't need to explicitly handle refresh visuals because the times when this needs to be done it is handled by
   * the table e.g. handleNameChange()
   */
  protected void refreshVisuals()
  {
    Point pos = getModel().getLocation();
    Dimension dim = getFigure().getSize();

    Rectangle constraint = new Rectangle(pos, dim);

    if (constraint != null)
      ((GraphicalEditPart)getParent()).setLayoutConstraint(this, getFigure(), constraint);
    else
    {
      ((GraphicalEditPart)getParent()).setLayoutConstraint(this, getFigure(), new Rectangle(pos.x, pos.y, -1, -1));
    }

  }

  /**
   * @return
   */
  private AnzeigeQuerschnittModelTypAbstract getCastedModel()
  {

    AnzeigeQuerschnittModelTypAbstract model = null;

    if (getModel() instanceof AnzeigeQuerschnittModelTypAbstract)
    {
      model = (AnzeigeQuerschnittModelTypAbstract)getModel();
    }

    return model;
  }
  
  @Override
  public String getId()
  {
    // TODO Auto-generated method stub
    return getText();
  }

  /** {@inheritDoc} */
  @Override
  public List<ILegendeBaustein> getBausteine() {
    if (legendenBausteine == null) {
      legendenBausteine = new ArrayList<ILegendeBaustein>();
      for (Object listObj : getDefinition())
      {
        legendenBausteine.addAll(LegendeHelper.erzeugeLegendeBausteine(listObj));        
      }
    }
    return legendenBausteine;
  }

  @Override
  public Image getIcon()
  {
    // TODO Auto-generated method stub
    return null;
  }

  @Override
  public String getText()
  {
    // TODO Auto-generated method stub
    return "Anzeigequerschnitt (GF) A8";
  }

  @Override
  public List getDefinition()
  {
    // TODO Auto-generated method stub
//    List list = new ArrayList();
//    list.add(LegendeDefinitionAnzeigeQuerschnitt.class);
//    int kontext = 0;
//    kontext = getCastedModel().getKontext();
//    if (kontext == 0)
//      list.add(LegendeDefinitionAnzeigeQuerschnittStatus.class);
//    else if (kontext == 1)
//      list.add(LegendeDefinitionAnzeigeQuerschnittStatusBefehle.class);
//    list.add(LegendeDefinitionAnzeigeQuerschnittProgramm.class);
//    return list;
    return LegendeHelper.erzeugeAqLegendeDefinition(getCastedModel(), false);
  }
  
}
