/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.LayerConstants;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeQuerschnittFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeQuerschnittLinksFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeQuerschnittRechtsFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeQuerschnittRechtsFigureA14;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeQuerschnittTyp;
import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.dobj.internal.DarstellungExtensionPoint;
import de.bsvrz.buv.plugin.dobj.model.ConfiguratedDoTyp;
import de.bsvrz.buv.plugin.dobj.model.DoTyp;
import de.bsvrz.dav.daf.main.ResultData;

/**
 * Implementierung fr einen {@link DoMitVerbLinieEditPart} der ein Model-Objekt 
 * vom Typ {@link AnzeigeQuerschnittModelTypAbstract} verwendet.
 * 
 * @author Pittner
 * 
 */
public class AnzeigeQuerschnittTypA2_1sp2EditPart extends DoMitVerbLinieEditPart implements LayerConstants
{
  private int _offsetYWert = 17;
  
  private int _offsetXWert = 29;

  protected AnzeigeQuerschnittLinksFigure _contentPane = null;

  @Override
  public IFigure getContentPane()
  {
    // TODO Auto-generated method stub
    return _contentPane;
  }

  @Override
  protected List getModelChildren()
  {
    if (getCastedModel().getKomponenten() != null && getCastedModel().getKomponenten().size() > 0)
    {
      return getCastedModel().getKomponenten();
    }
    else
    {
      ConfiguratedDoTyp doTyp1 = DarstellungExtensionPoint.getDoTyp("AnzeigeTypAEditPart");
      ConfiguratedDoTyp doTyp2 = DarstellungExtensionPoint.getDoTyp("AnzeigeTypBEditPart");
      ConfiguratedDoTyp doTyp3 = DarstellungExtensionPoint.getDoTyp("AnzeigeTypCEditPart");

      Darstellung darstellung = (Darstellung)getViewer().getProperty(Darstellung.class.toString());
      EList<DoTyp> doTypen = darstellung.getDoTypen();

      boolean vorhanden1 = false;
      boolean vorhanden2 = false;
      boolean vorhanden3 = false;

      for (DoTyp typ : doTypen)
      {
        if (typ instanceof ConfiguratedDoTyp)
        {
          ConfiguratedDoTyp co = (ConfiguratedDoTyp)typ;
          if (co.getId().equals(doTyp1.getId()))
          {
            vorhanden1 = true;
            doTyp1 = co;
          }
          if (co.getId().equals(doTyp2.getId()))
          {
            vorhanden2 = true;
            doTyp2 = co;
          }
          if (co.getId().equals(doTyp3.getId()))
          {
            vorhanden3 = true;
            doTyp3 = co;
          }
        }
      }

      AnzeigeModelTypAbstract a1 = (AnzeigeModelTypAbstract)doTyp1.createModel();
      a1.setLocation(new Point(5 + _offsetXWert, 1 + _offsetYWert));
      getCastedModel().getKomponenten().add(a1);
      
//      AnzeigeModelTypAbstract a2 = (AnzeigeModelTypAbstract)doTyp1.createModel();
//      a2.setLocation(new Point(5 + _offsetXWert, 35 + _offsetYWert));
//      getCastedModel().getKomponenten().add(a2);

      AnzeigeModelTypAbstract b1 = (AnzeigeModelTypAbstract)doTyp2.createModel();
      b1.setLocation(new Point(1  + _offsetXWert, 35 + _offsetYWert));
      getCastedModel().getKomponenten().add(b1);
//
      AnzeigeModelTypAbstract c1 = (AnzeigeModelTypAbstract)doTyp3.createModel();
      c1.setLocation(new Point(1 + _offsetXWert, 35 + 36 + _offsetYWert));
      getCastedModel().getKomponenten().add(c1);

//      AnzeigeModelTypAbstract a2 = (AnzeigeModelTypAbstract)doTyp1.createModel();
//      a2.setLocation(new Point(77, 20 + _offsetYWert));
//      getCastedModel().getKomponenten().add(a2);
      
//      AnzeigeModelTypAbstract b2 = (AnzeigeModelTypAbstract)doTyp2.createModel();
//      b2.setLocation(new Point(77+34, 1 + _offsetYWert));
//      getCastedModel().getKomponenten().add(b2);
//
//      AnzeigeModelTypAbstract c2 = (AnzeigeModelTypAbstract)doTyp3.createModel();
//      c2.setLocation(new Point(77+34, 36 + _offsetYWert));
//      getCastedModel().getKomponenten().add(c2);
//      
//      AnzeigeModelTypAbstract a3 = (AnzeigeModelTypAbstract)doTyp1.createModel();
//      a3.setLocation(new Point(77+34+42, 20 + _offsetYWert));
//      getCastedModel().getKomponenten().add(a3);

      if (!vorhanden1)
      {
        doTypen.add(doTyp1);
      }
      if (!vorhanden2)
      {
        doTypen.add(doTyp2);
      }
      if (!vorhanden3)
      {
        doTypen.add(doTyp3);
      }

      return getCastedModel().getKomponenten();
    }
  }

  @Override
  protected AnzeigeQuerschnittFigure createFigure()
  {
    AnzeigeQuerschnittRechtsFigure rechts = null;

    _contentPane = new AnzeigeQuerschnittLinksFigure(AnzeigeQuerschnittTyp.WVZ_A2_Typ3, 3);
    _contentPane.setSize(70 + 32, 58 + 35 + _offsetYWert);
    rechts = new AnzeigeQuerschnittRechtsFigureA14(getCastedModel().getKontext());
    rechts.setSize(35, 58 + 35 + _offsetYWert);

    AnzeigeQuerschnittFigure result = new AnzeigeQuerschnittFigure(_contentPane, rechts);
    result.setSize(110 + 32, 58 + 35 + _offsetYWert);

    result.switchLinksRechts(getCastedModel().getFahrtrichtung());

    String text = getCastedModel().getBezeichnung();
    if (text != null)
    {
      result.setExtraText(text);
    }

    return result;
  }

  @Override
  public void notifyChanged(Notification notification)
  {
    super.notifyChanged(notification);
    if (AnlagendarstellungPackage.Literals.ANZEIGE_QUERSCHNITT_MODEL_TYP_ABSTRACT__BEZEICHNUNG.equals(notification
        .getFeature()))
    {
      ((AnzeigeQuerschnittFigure)getFigure()).setExtraText(notification.getNewStringValue());
      getFigure().repaint();
    }
    else if (AnlagendarstellungPackage.Literals.ANZEIGE_QUERSCHNITT_MODEL_TYP_ABSTRACT__KONTEXT.equals(notification
        .getFeature()))
    {

    }
    else if (AnlagendarstellungPackage.Literals.ANZEIGE_QUERSCHNITT_MODEL_TYP_ABSTRACT__FAHRTRICHTUNG
        .equals(notification.getFeature()))
    {
      int fr = notification.getNewIntValue();

      AnzeigeQuerschnittFigure figure = (AnzeigeQuerschnittFigure)getFigure();
      figure.switchLinksRechts(fr);

      EList<AnzeigeModelTypAbstract> anzeigen = getCastedModel().getKomponenten();

      if (getCastedModel().getFahrtrichtung() != 1)
      {
        anzeigen.get(0).setLocation(new Point(5 + _offsetXWert, 1 + _offsetYWert));
        anzeigen.get(1).setLocation(new Point(1 + _offsetXWert, 35 + _offsetYWert));
        anzeigen.get(2).setLocation(new Point(1 + _offsetXWert, 35 + 36 + _offsetYWert));
//        anzeigen.get(3).setLocation(new Point(77, 20 + _offsetYWert));
//        anzeigen.get(4).setLocation(new Point(77+34, 1 + _offsetYWert));
//        anzeigen.get(5).setLocation(new Point(77+34, 36 + _offsetYWert));
//        anzeigen.get(6).setLocation(new Point(77+34+42, 20 + _offsetYWert));
      }
      else
      {
//        anzeigen.get(0).setLocation(new Point(77+34+42, 1));
//        anzeigen.get(1).setLocation(new Point(77+34, 19));
//        anzeigen.get(2).setLocation(new Point(77+34, 1));
//        anzeigen.get(3).setLocation(new Point(77, 1));
//        anzeigen.get(4).setLocation(new Point(35, 19));
//        anzeigen.get(5).setLocation(new Point(35, 1));
        anzeigen.get(0).setLocation(new Point(5 + _offsetXWert, 38 + _offsetYWert));
        anzeigen.get(1).setLocation(new Point(1 + _offsetXWert, 4 + _offsetYWert));
        anzeigen.get(2).setLocation(new Point(1 + _offsetXWert, 1));
      }

      for (AnzeigeModelTypAbstract anzeige : anzeigen)
      {
        anzeige.setFahrtrichtung(fr);
      }

    }
    else if (AnlagendarstellungPackage.Literals.ANZEIGE_QUERSCHNITT_MODEL_TYP_ABSTRACT__DATA_UPDATE.equals(notification
        .getFeature()))
    {
      ResultData result = (ResultData)notification.getNewValue();
      dataUpdateAnzeigeQuerschnittModelTypAbstract(result, getCastedModel());     
    }
    else if (AnlagendarstellungPackage.Literals.ANZEIGE_QUERSCHNITT_MODEL_TYP_ABSTRACT__DARSTELLUNG_UPDATE
        .equals(notification.getFeature()))
    {
      String s = (String)notification.getNewValue();
      getFigure().setToolTip(new Label(s));
    }
  }

  /**
   * We don't need to explicitly handle refresh visuals because the times when this needs to be done it is handled by
   * the table e.g. handleNameChange()
   */
  protected void refreshVisuals()
  {
    Point pos = getModel().getLocation();
    Dimension dim = getFigure().getSize();

    Rectangle constraint = new Rectangle(pos, dim);

    if (constraint != null)
      ((GraphicalEditPart)getParent()).setLayoutConstraint(this, getFigure(), constraint);
    else
    {
      ((GraphicalEditPart)getParent()).setLayoutConstraint(this, getFigure(), new Rectangle(pos.x, pos.y, -1, -1));
    }
  }

  /**
   * @return
   */
  private AnzeigeQuerschnittModelTypAbstract getCastedModel()
  {
    AnzeigeQuerschnittModelTypAbstract model = null;
    if (getModel() instanceof AnzeigeQuerschnittModelTypAbstract)
    {
      model = (AnzeigeQuerschnittModelTypAbstract)getModel();
    }
    return model;
  }
}
