/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import java.util.ArrayList;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeElementFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeElementTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.NumberValue;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.IntegerValueState;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Implementierung fr einen {@link DoMitVerbLinieEditPart} der ein Model-Objekt vom Typ {@link AnzeigeModelTypAbstract}
 * verwendet. <br>
 * Hier wird die LSA mit Rot/Gelb auf DE-Ebene umgesetzt.
 * 
 * 
 * 
 * @author HofmannJ
 * 
 */
public class AnzeigeTypA81LsaEditPart extends AnzeigeTypAEditPart
{
  private final Dimension DIM = new Dimension(18, 33);

  /**
   * Flag legt fest, ob der DE-Fehlerstatus fr die Border herangezogen wird oder nicht.
   */
  private static final boolean USE_DE_FEHLER_STATUS_FOR_BORDER = false;

  private LineBorder _border;

  @Override
  protected AnzeigeElementFigure createFigure()
  {
    ClientDavInterface dav = DaVVerbindung.getInstanz().getDaVVerbindung();
    DataModel dataModel = dav.getDataModel();

    AnzeigeModelTypAbstract model = getCastedModel();

    setzteInhalteModell(model, dataModel);

    boolean isFrNord = getCastedModel().getFahrtrichtung() != 1 ? true : false;
    AnzeigeElementFigure aef = new AnzeigeElementFigure("Anzeige LSA", AnzeigeElementTyp.LSA_A81,
        AnzeigeInhaltTyp.DUNKEL, true, isFrNord);
    aef.setOpaque(true);
    aef.setSize(DIM.width, DIM.height);

    /*
     * Default-Border festlegen. ndert sich nicht an Hand des DE-Fehlerstatus auf Grund Flag
     * USE_DE_FEHLER_STATUS_FOR_BORDER.
     */
    _border = new LineBorder(ColorConstants.orange, 2);

    aef.setBorder(_border);

    addToolTipListener(aef);

    return aef;
  }

  @Override
  public void notifyChanged(Notification notification)
  {
    super.notifyChanged(notification);

    if (AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE.equals(notification.getFeature()))
    {
      Object newValue = notification.getNewValue();
      Object oldValue = notification.getOldValue();

      if (newValue instanceof AnzeigeInhaltTyp)
      {
        AnzeigeInhaltTyp inhalt = (AnzeigeInhaltTyp)newValue;
        inhaltUpdateAnzeigeModelTypAbstract(AnzeigeElementTyp.LSA_A81, inhalt);
      }
      else if (newValue instanceof ResultData)
      {
        ResultData result = (ResultData)newValue;

        int kontext = 0;
        if (getParent() != null)
        {
          Object parentModel = getParent().getModel();
          {
            if (parentModel instanceof AnzeigeQuerschnittModelTypAbstract)
            {
              AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)parentModel;
              kontext = aq.getKontext();
            }
          }
        }

        if (true)
        {
          if (result != null)
          {
            SystemObject so = result.getObject();

            /*
             * Auf das richtige Objekt prfen
             */
            if ((getModel() != null) && (getModel().getSystemObject() != null)
                && (getModel().getSystemObject().equals(so)))
            {
              DataDescription dd = result.getDataDescription();
              AttributeGroup atg = dd.getAttributeGroup();
              Aspect asp = dd.getAspect();

              if ((atg != null) && (asp != null))
              {
                if ((atg.getPid().equals("atg.tlsZltLsaSperranlage")) && (asp.getPid().equals("asp.tlsZustand")))
                {
                  if ((result.hasData()) && (result.getData() != null))
                  {
                    AnzeigeInhaltTyp anzInhTyp = null;

                    /*
                     * Daten auswerten
                     */
                    Data data = result.getData();
                    String status = data.getItem("Status").asTextValue().getText();

                    if (status.equalsIgnoreCase("LSA dunkel"))
                    {
                      anzInhTyp = AnzeigeInhaltTyp.DUNKEL_BI;
                    }
                    else if (status.equalsIgnoreCase("rot"))
                    {
                      anzInhTyp = AnzeigeInhaltTyp.LSA_A81_ROT;
                    }
                    else if (status.equalsIgnoreCase("gelb"))
                    {
                      anzInhTyp = AnzeigeInhaltTyp.LSA_A81_GELB;
                    }
                    else
                    {
                      anzInhTyp = AnzeigeInhaltTyp.LSA_A81_UNBEKANNT;
                    }

                    inhaltUpdateAnzeigeModelTypAbstract(AnzeigeElementTyp.LSA_A81, anzInhTyp);
                  }
                  else
                  {
                    /*
                     * Keine Daten vorhanden
                     */
                    inhaltUpdateAnzeigeModelTypAbstract(AnzeigeElementTyp.LSA_A81, AnzeigeInhaltTyp.LSA_A81_UNBEKANNT);
                  }
                }

                if (atg.getPid().equals("atg.tlsGloDeFehler"))
                {

                  if ((USE_DE_FEHLER_STATUS_FOR_BORDER) && (_border != null))
                  {
                    /*
                     * Fehlerstatus
                     */
                    if (result.hasData())
                    {
                      Data data = result.getData();

                      NumberValue svDEFehlerStatus = data.getScaledValue("DEFehlerStatus");
                      IntegerValueState state = svDEFehlerStatus.getState();
                      final String text = svDEFehlerStatus.getText();
                      String valueText = svDEFehlerStatus.getValueText();

                      if (text.equalsIgnoreCase("ok"))
                      {
                        _border.setColor(ColorConstants.green);
                      }
                      else
                      {
                        _border.setColor(ColorConstants.red);
                      }

                      getFigure().repaint();

                    }
                    else
                    {
                      _border.setColor(ColorConstants.red);
                      getFigure().repaint();
                    }
                  }
                }
              }
            }

          }
        }

        if (false)
        {
          if (result != null && result.hasData())
          {
            Data data = result.getData();
            SystemObject so = data.getItem("Eigenschaft").getItem("AnzeigeInhalt").asReferenceValue().getSystemObject();
            ClientDavInterface dav = DaVVerbindung.getInstanz().getDaVVerbindung();
            AttributeGroup atg2 = dav.getDataModel().getAttributeGroup("atg.wvzInhalt");

            // AnzeigeInhalt hat eine hoehere Prio, d.h. wenn ein
            // AnzeigeInhalt vorhanden ist,
            // wird er auch angezeigt
            if (so != null)
            {
              Data data2 = so.getConfigurationData(atg2);
              final String inhalt = data2.getItem("Bildinhalt").asTextValue().getValueText();
              AnzeigeInhaltTyp inhaltNeu = AnzeigeInhaltTyp.setBildInhalt(inhalt, result.getDataTime());
              inhaltUpdateAnzeigeModelTypAbstract(AnzeigeElementTyp.LSA_A81, inhaltNeu);
            }
            else
            {
              inhaltUpdateAnzeigeModelTypAbstract(AnzeigeElementTyp.LSA_A81, AnzeigeInhaltTyp.GESTOERT_BI);
            }
          }
          else
          {
            inhaltUpdateAnzeigeModelTypAbstract(AnzeigeElementTyp.LSA_A81, AnzeigeInhaltTyp.GESTOERT_BI);
          }
        }
      }

    }
  }

  @Override
  protected void setzteInhalteModell(AnzeigeModelTypAbstract model, DataModel dataModel)
  {
    /*
     * Hier wird bestimmt, welche Ausprgungen angenommen werden knnen.
     */
    ArrayList<String> list = new ArrayList<String>();
    list.add("rot");
    list.add("gelb");
    list.add("Dunkel");
    list.add("unbekannt");
    EList<String> elist = new BasicEList<String>(list);
    getCastedModel().setInhalte1(elist);
    getCastedModel().setInhalte2(elist);
  }
}
