/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.draw2d.Border;
import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.MouseEvent;
import org.eclipse.draw2d.MouseMotionListener;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.ActionDescriptor;
import org.eclipse.ui.internal.PluginAction;
import org.eclipse.ui.internal.registry.IWorkbenchRegistryConstants;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelUfdSbaMessStelleNS;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelUfdSensorFBZ;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelUfdSensorHK;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelUfdSensorSW;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.PickerVertical;
import de.bsvrz.buv.plugin.anlagendarstellung.StreckenStationModel;
import de.bsvrz.buv.plugin.anlagendarstellung.StreckenStationModelA14;
import de.bsvrz.buv.plugin.anlagendarstellung.VerhaltenDoTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnschlussStelleFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeElementFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeQuerschnittFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeQuerschnittLinksFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeQuerschnittRechtsFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeQuerschnittRechtsFigureA14;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.DetektorLveFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.StreckenstationAnzeigeElementFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.StreckenstationFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.TabellenMqFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.TabellenMqZeilenFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.UmfeldDatenBandFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.fonts.VisuAnzeigeKapsel;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeElementTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.model.SituationTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.model.StatusTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.model.ZusatzElementTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.DefsPreference;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DaVHelper;
import de.bsvrz.buv.plugin.anlagendarstellung.util.ExtensionPointHelper;
import de.bsvrz.buv.plugin.anlagendarstellung.views.ZustandsDarstellungIstView;
import de.bsvrz.buv.plugin.darstellung.editparts.AnsichtEditPart;
import de.bsvrz.buv.plugin.darstellung.editparts.DarstellungEditPart;
import de.bsvrz.buv.plugin.darstellung.editparts.DarstellungsSpalteEditPart;
import de.bsvrz.buv.plugin.darstellung.editparts.StilisierteDarstellungEditPart;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.darstellung.model.DarstellungPackage;
import de.bsvrz.buv.plugin.darstellung.model.DarstellungsSpalte;
import de.bsvrz.buv.plugin.darstellung.model.Ebene;
import de.bsvrz.buv.plugin.darstellung.model.StilisierteDarstellung;
import de.bsvrz.buv.plugin.dobj.editparts.DoModelEditPart;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.plugin.dobj.model.DoTyp;
import de.bsvrz.buv.plugin.dobj.model.DobjPackage;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.ILegendeBaustein;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Abstrakte Basis-Implementierung fr einen {@link DoModelEditPart}, der diesen um die Verbindungslinien-Funktionalitt
 * erweitert. Es wird ein Model-Objekt vom Typ {@link DoModelMitVerbLinie} verwendet.
 * 
 * @author Pittner
 * 
 */
public abstract class DoMitVerbLinieEditPart extends DoModelEditPart<DoModelMitVerbLinie, IFigure> implements
    LayerConstants, ITaktListener
{

  private final static SimpleDateFormat _sdf1 = new SimpleDateFormat("HH:mm");

  private final static SimpleDateFormat _sdf2 = new SimpleDateFormat("dd.MM.yyyy HH:mm");

  private Map<DarstellungsSpalte, AdapterImpl> _mapSpaltenAdapters;
  
  /**
   * Pfad zum Basis-Image
   */
  private String _pathBaseImage = null;

  @Override
  protected void geheOnline()
  {
    super.geheOnline();
  }

  @Override
  protected void geheOffline()
  {
    super.geheOffline();
  }

  @Override
  public void activate()
  {
    super.activate();

    _mapSpaltenAdapters = new HashMap<DarstellungsSpalte, AdapterImpl>();

    DoModelMitVerbLinie target = getModel().getTarget();
    if (target != null)
    {
      anmeldenAufDarstellungsSpalten(getModel(), target);
      EStructuralFeature feature = AnlagendarstellungPackage.Literals.DO_MODEL_MIT_VERB_LINIE__TARGET;
      final SetCommand cmd = new SetCommand(getModel(), feature, target);
      getCommandStack().execute(cmd);
    }

    EList<DoModelMitVerbLinie> targetOf = getModel().getTargetOf();
    for (DoModelMitVerbLinie model2 : targetOf)
    {
      anmeldenAufDarstellungsSpalten(model2, getModel());
      PolylineConnection linie = holeVerbindungsLinie(model2, getModel());
      if (linie != null)
      {
        EStructuralFeature feature = AnlagendarstellungPackage.Literals.DO_MODEL_MIT_VERB_LINIE__TARGET;
        final SetCommand cmd = new SetCommand(model2, feature, getModel());
        getCommandStack().execute(cmd);
      }
    }
  }

  /**
   * Hilfmethode, bestimmt ob die Spalte eines {@link DoModelMitVerbLinie} sichtbar ist
   * 
   * @param doModelMitVerbLinie
   * @return true, wenn Spalte sichtbar ist
   */
  private boolean isSpalteVonDoModelMitVerbLinieVisible(DoModelMitVerbLinie doModelMitVerbLinie)
  {
    Ebene ebene = (Ebene)getModel().eContainer();
    Darstellung darstellung = ebene.getDarstellung();
    EList<Ebene> ebenen = darstellung.getEbenen();

    // Hier wird bestimmt ob die Spalten sichtbar sind zu welcher die DO zugeordnet sind
    for (Ebene e : ebenen)
    {
      Darstellung tmp = e.getDarstellung();
      if (tmp instanceof StilisierteDarstellung)
      {
        StilisierteDarstellung stilisierteDarstellung = (StilisierteDarstellung)tmp;
        EList<DarstellungsSpalte> spalten = stilisierteDarstellung.getSpalten();
        for (DarstellungsSpalte spalte : spalten)
        {
          EList<DoModel> doObjekte = spalte.getDoObjekte();
          for (DoModel doModel : doObjekte)
          {
            if (doModel instanceof DoModelMitVerbLinie)
            {
              if (doModelMitVerbLinie.equals(doModel))
              {
                return spalte.isVisible();
              }
            }
          }
        }
      }
    }
    return false;
  }

  /**
   * Fhrt die Anmeldung durch und hrt darauf ob die Spalte des DO sichtbar oder unsichtbar ist.
   * 
   * @param quelle
   * @param ziel
   */
  private void anmeldenAufDarstellungsSpalten(final DoModelMitVerbLinie quelle, final DoModelMitVerbLinie ziel)
  {
    EditPart contents = getViewer().getContents();
    if (contents instanceof StilisierteDarstellungEditPart)
    {
      StilisierteDarstellungEditPart epart = (StilisierteDarstellungEditPart)contents;
      final StilisierteDarstellung darstellung = epart.getModel();
      EList<DarstellungsSpalte> spalten = darstellung.getSpalten();
      for (DarstellungsSpalte spalte : spalten)
      {
        EList<DoModel> doObjekte = spalte.getDoObjekte();
        for (DoModel doModel : doObjekte)
        {
          if (doModel instanceof DoModelMitVerbLinie)
          {
            DoModelMitVerbLinie doModelMitVerbLinie = (DoModelMitVerbLinie)doModel;
            if (doModelMitVerbLinie.equals(quelle) || doModelMitVerbLinie.equals(ziel))
            {

              AdapterImpl adapter = new AdapterImpl() {
                @Override
                public void notifyChanged(Notification msg)
                {
                  final Object notifier = msg.getNotifier();
                  final int type = msg.getEventType();

                  if (notifier instanceof DarstellungsSpalte)
                  {
                    final int featureID = msg.getFeatureID(DarstellungsSpalte.class);
                    switch (type)
                    {
                      case Notification.SET:
                        switch (featureID)
                        {
                          case DarstellungPackage.DARSTELLUNGS_SPALTE__VISIBLE:
                            EStructuralFeature feature = AnlagendarstellungPackage.Literals.DO_MODEL_MIT_VERB_LINIE__TARGET;
                            final SetCommand cmd = new SetCommand(getModel(), feature, ziel);
                            getCommandStack().execute(cmd);
                            break;
                          default:
                        }
                    }
                  }
                }
              };

              if (!_mapSpaltenAdapters.containsKey(spalte))
              {
                spalte.eAdapters().add(adapter);
                _mapSpaltenAdapters.put(spalte, adapter);
              }
            }
          }
        }
      }
    }
  }

  /**
   * 
   */
  private void abmeldenVonDarstellungsSpalten()
  {
    EditPart contents = getViewer().getContents();
    if (contents instanceof StilisierteDarstellungEditPart)
    {
      StilisierteDarstellungEditPart epart = (StilisierteDarstellungEditPart)contents;
      final StilisierteDarstellung darstellung = epart.getModel();
      EList<DarstellungsSpalte> spalten = darstellung.getSpalten();
      for (final DarstellungsSpalte spalte : spalten)
      {
        if (_mapSpaltenAdapters.containsKey(spalte))
        {
          spalte.eAdapters().remove(_mapSpaltenAdapters.get(spalte));
        }
      }
    }
  }

  @Override
  public void deactivate()
  {
    // Hier werden die Linien von bzw. zum Objekt geloescht wenn dieses im Editor gelscht wird
    DoModelMitVerbLinie target = getModel().getTarget();
    if (target != null)
    {
      PolylineConnection linie = holeVerbindungsLinie(getModel(), target);
      if (linie != null)
      {
        getLayer(CONNECTION_LAYER).remove(linie);
      }
    }
    EList<DoModelMitVerbLinie> targetOf = getModel().getTargetOf();
    for (DoModelMitVerbLinie target2 : targetOf)
    {
      PolylineConnection linie = holeVerbindungsLinie(target2, getModel());
      if (linie != null)
      {
        getLayer(CONNECTION_LAYER).remove(linie);
      }
    }
    // Abmeldung der Listener fr die Spalten
    abmeldenVonDarstellungsSpalten();

    super.deactivate();
  }

  /*
   * (non-Javadoc)
   * 
   * @see de.bsvrz.buv.plugin.dobj.editparts.DoModelEditPart#notifyChanged(org. eclipse.emf.common.notify.Notification)
   */
  @Override
  public void notifyChanged(final Notification notification)
  {
    super.notifyChanged(notification);

    if (AnlagendarstellungPackage.Literals.DO_MODEL_MIT_VERB_LINIE__TARGET.equals(notification.getFeature()))
    {
      DoModelMitVerbLinie targetOld = (DoModelMitVerbLinie)notification.getOldValue();
      DoModelMitVerbLinie target = (DoModelMitVerbLinie)notification.getNewValue();

      if (target != null)
      {
        // Die Linie, welche vom Objekt ausgeht wird behandelt
        processLine(getModel(), target);
      }
      else
      {
        if (targetOld != null)
        {
          // Linie wird explizit gelscht
          PolylineConnection linie = holeVerbindungsLinie(getModel(), targetOld);
          if (linie != null)
          {
            getLayer(CONNECTION_LAYER).remove(linie);
          }
        }
      }

      EList<DoModelMitVerbLinie> targetOf = new BasicEList<DoModelMitVerbLinie>(getModel().getTargetOf());
      for (DoModelMitVerbLinie doModel : targetOf)
      {
        // Die Linien, welche zum Objekt fuehrt werden behandelt
        processLine(doModel, getModel());
      }

    }
    else if (AnlagendarstellungPackage.Literals.DO_MODEL_MIT_VERB_LINIE__TARGET_OF.equals(notification.getFeature()))
    {
      // Hier wird signalisiert wenn nderungen in der Zielzuordunung gemacht werden,
      // d.h. es kommem Quellen hinzu oder es werden Quellen entfernt
    }
    else if (DobjPackage.Literals.DO_MODEL__SYSTEM_OBJECT.equals(notification.getFeature()))
    {
      SystemObject so = (SystemObject)notification.getNewValue();
    }
    else if (AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__BLINKEN
        .equals(notification.getFeature()))
    {
      /*
       * Auf das Blink-Flag im Model reagieren
       */
      Object newValue = notification.getNewValue();
      Object oldValue = notification.getOldValue();

      if ((newValue != null) && (newValue instanceof Integer))
      {
        /*
         * 1=Blinken, 0=nicht Blinken, -1=Undefiniert(Wird auf nicht Blinken
         * umgesetzt)
         */

        boolean blinken = false;
        Integer iBlinken = (Integer) newValue;

        if (iBlinken.intValue() == ADefs.BLINKEN_EIN)
        {
          blinken = true;
        }

        // if (getModel().getSystemObject().getPid().equals(
        // "nw.sba.anzeigeQuerschnitt.uz.koeln.aq_1.41.0101s.anzeige1"))
        // {
        // System.err.println("###>>> Blinken NEU=" + newValue + " ALT="
        // + oldValue + " fr " + getModel().getSystemObject());
        // }

        if ((!newValue.equals(oldValue))
            || (blinken && !_angemeldetAmTaktgeber))
        {
          /*
           * An- bzw. Abmelden auf den Taktgeber, je nach dem ob geblinkt werden
           * soll oder nicht.
           */
          if (blinken)
          {
            anmeldenTaktgeber();
          } else
          {
            abmeldenTaktgeber();
          }
        }

      } else if ((newValue == null) && _angemeldetAmTaktgeber)
      {
        /*
         * Abmelden vom Taktgeber wenn der neue Wert NULL ist und wir noch am
         * Taktgeber anemeldet sind.
         */
        abmeldenTaktgeber();
      }

    }
    else
    {

    }
  }

  /**
   * Behandelt die Linie ausgehend von einem Quell- zu einem Zielobjekt. Es werden das Zeichnen der Linie sowi
   * 
   * @param source
   * @param target
   */
  private void processLine(DoModelMitVerbLinie source, DoModelMitVerbLinie target)
  {
    boolean isSpalteQuelleSichtbar = isSpalteVonDoModelMitVerbLinieVisible(source);
    boolean isSpalteZielSichtbar = isSpalteVonDoModelMitVerbLinieVisible(target);

    PolylineConnection linie = holeVerbindungsLinie(source, target);

    // Nur wenn beide Spalten sichtbar sind, wird die Linie gezeichnet bzw. sichtbar gemacht
    if (isSpalteQuelleSichtbar && isSpalteZielSichtbar)
    {
      if (linie == null)
      {
        // Hier wird gezeichnet
        createTargetConnection(source, target);
      }
      else
      {
        linie.setVisible(true);
      }
    }
    else
    {
      if (linie != null)
      {
        linie.setVisible(false);
      }
    }
  }

  /**
   * Erzeugt bzw. entfernt die Verbindung zum Ziel wenn dieses ausgewhlt wurde oder durch den Benutzer wieder entfernt
   * wird. Zusaetzlich wird die Verbing ein- bzw. wieder ausgeblendet wenn die Spalten sichtbar oder unsichtbar gemacht
   * werden.
   * 
   * @param quelle
   * @param ziel
   */
  // protected void targetChange(DoModelMitVerbLinie quelle, DoModelMitVerbLinie ziel)
  // {
  // // Ebene ebene = (Ebene)getModel().eContainer();
  // // Darstellung darstellung = ebene.getDarstellung();
  // // EList<Ebene> ebenen = darstellung.getEbenen();
  // //
  // //
  // // boolean isSpalteZielSichtbar = false;
  // // boolean isSpalteQuelleSichtbar = false;
  // //
  // //
  // // //Hier wird bestimmt ob die Spalten sichtbar sind zu welcher die DO zugeordnet sind
  // // for (Ebene e : ebenen)
  // // {
  // // StilisierteDarstellung d = (StilisierteDarstellung)e.getDarstellung();
  // // EList<DarstellungsSpalte> spalten = d.getSpalten();
  // //
  // // for (DarstellungsSpalte sp : spalten)
  // // {
  // // EList<DoModel> doObjekte = sp.getDoObjekte();
  // // for (DoModel doModel : doObjekte)
  // // {
  // // if (doModel.equals(ziel))
  // // {
  // // if (sp.isVisible())
  // // {
  // // isSpalteZielSichtbar = true;
  // // }
  // //// break;
  // // }
  // // if (doModel.equals(quelle))
  // // {
  // // if (sp.isVisible())
  // // {
  // // isSpalteQuelleSichtbar = true;
  // // }
  // //// break;
  // // }
  // // }
  // // }
  // // }
  //        
  // // EditPart contents = getViewer().getContents();
  // // if (contents instanceof StilisierteDarstellungEditPart)
  // // {
  // // StilisierteDarstellungEditPart epart = (StilisierteDarstellungEditPart)contents;
  // // final StilisierteDarstellung darstellung = epart.getModel();
  // // EList<DarstellungsSpalte> spalten = darstellung.getSpalten();
  // //
  // // for (DarstellungsSpalte sp : spalten)
  // // {
  // // EList<DoModel> doObjekte = sp.getDoObjekte();
  // // for (DoModel doModel : doObjekte)
  // // {
  // // if (doModel.equals(ziel))
  // // {
  // // if (sp.isVisible())
  // // {
  // // isSpalteZielSichtbar = true;
  // // }
  // // break;
  // // }
  // // else if (doModel.equals(quelle))
  // // {
  // // if (sp.isVisible())
  // // {
  // // isSpalteQuelleSichtbar = true;
  // // }
  // // break;
  // // }
  // // }
  // // }
  // // }
  //    
  //    
  // boolean isSpalteQuelleSichtbar = isSpalteVonDoVisible(quelle);
  // boolean isSpalteZielSichtbar = isSpalteVonDoVisible(ziel);
  //    
  // PolylineConnection linie = holeVerbindungsLinie(quelle, ziel);
  //    
  // // Nur wenn beide Spalten sichtbar sind, wird die Linie gezeichnet bzw. sichtbar gemacht
  // System.out.println("DoMitVerbLinieEditPart.targetChange(): " + isSpalteQuelleSichtbar + "" + isSpalteZielSichtbar +
  // "" + linie);
  //    
  // if (isSpalteQuelleSichtbar && isSpalteZielSichtbar)
  // {
  // if (ziel != null){
  // if (linie == null){
  // // Hier wird gezeichnet
  // createTargetConnection(getModel(), ziel);
  // }
  // else{
  // linie.setVisible(true);
  // }
  // }else{
  // System.out.println("Ziel ist null!!!");
  // // getLayer(CONNECTION_LAYER).remove(linie);
  // }
  // }
  // else
  // {
  // if (linie != null){
  // linie.setVisible(false);
  // }else{
  // // Hier werden alle Verbindungen behandelt die sich als Quelle beim Ziel eingetragen haben
  // EList<DoModelMitVerbLinie> targetOf = new BasicEList<DoModelMitVerbLinie>(getModel().getTargetOf());
  // for (DoModelMitVerbLinie doModel : targetOf)
  // {
  // targetChange(doModel, getModel());
  // }
  // }
  // }
  // }

  /**
   * Hilfsmethode, gibt eine Verbindunglinie zurck, welche zwischen zwei {@link DoModelMitVerbLinie} existiert
   * 
   * @param quelle
   *          das Quellobjekt
   * @param ziel
   *          das Zielobjekt
   * @return die Verbindunglinie oder null falls keine Linie existiert
   */
  private PolylineConnection holeVerbindungsLinie(DoModelMitVerbLinie quelle, DoModelMitVerbLinie ziel)
  {
    List children = getLayer(CONNECTION_LAYER).getChildren();
    PolylineConnection linie = null;

    // Hier wird die Linie bestimmt welche zum Quell- bzw. Ziel-DO gehoert.
    for (Object object : children)
    {
      if (object instanceof PolylineConnection)
      {
        PolylineConnection temp = (PolylineConnection)object;
        temp.setForegroundColor(ColorConstants.lightGray);
        IFigure owner = temp.getSourceAnchor().getOwner();
        IFigure target = temp.getTargetAnchor().getOwner();
        if (owner != null && target != null && quelle != null && ziel != null)
        {
          if (owner.equals(((DoMitVerbLinieEditPart)getViewer().getEditPartRegistry().get(quelle)).getFigure())
              && target.equals(((DoMitVerbLinieEditPart)getViewer().getEditPartRegistry().get(ziel)).getFigure()))
          {
            linie = temp;
          }
        }
      }
    }
    return linie;
  }

  /**
   * Erzeugt eine Verbindungslinie vom Quell- zum Zielobjekt
   * 
   * @param quelle
   *          die Quelle
   * @param ziel
   *          das Ziel
   */
  public void createTargetConnection(DoModelMitVerbLinie quelle, DoModelMitVerbLinie ziel)
  {
    // Keine konkrete Typisierung, da die Darstellung auch andere Typen enthalten kann
    final Map<EObject, EditPart> editPartRegistry = getViewer().getEditPartRegistry();

    IFigure figQuelle = null;
    IFigure figZiel = null;

    // Holen des EditParts aus der Registry, um die Quell-Figure des DO zu erhalten
    if (editPartRegistry.containsKey(quelle))
    {
      EditPart editPart = editPartRegistry.get(quelle);
      if (editPart instanceof DoMitVerbLinieEditPart)
      {
        DoMitVerbLinieEditPart part = (DoMitVerbLinieEditPart)editPart;
        figQuelle = part.getFigure();
      }
    }
    // Holen des EditParts aus der Registry, um die Ziel-Figure des DO zu erhalten
    if (editPartRegistry.containsKey(ziel))
    {
      EditPart editPart = editPartRegistry.get(ziel);
      if (editPart instanceof DoMitVerbLinieEditPart)
      {
        DoMitVerbLinieEditPart part = (DoMitVerbLinieEditPart)editPart;
        figZiel = part.getFigure();
      }
    }

    // Hinzufuegen der Information zum Zielobjekt, zu welchen Quellobjekten es gehoert
    if (!ziel.getTargetOf().contains(getModel()))
      ziel.getTargetOf().add(getModel());

    // Erzeugen und zeichnen der Verbindungs-Linie
    if (figZiel != null && figQuelle != null)
    {
      PolylineConnection linie = new PolylineConnection();
      linie.setForegroundColor(ColorConstants.lightGray);
      linie.setSourceAnchor(new ChopboxAnchor(figQuelle));
      linie.setTargetAnchor(new ChopboxAnchor(figZiel));
      getLayer(CONNECTION_LAYER).add(linie);
    }
    else
    {
      System.err.println("figs sind null: " + figQuelle + " : " + figZiel);
    }
  }

  /**
   * Gibt den Command Stack zurck.
   * 
   * @return der Command Stack, niemals <code>null</code>.
   */
  protected CommandStack getCommandStack()
  {
    CommandStack stack = getViewer().getEditDomain().getCommandStack();
    Assert.isNotNull(stack, "Command Stack nicht verfgbar.");
    return stack;
  }

  /**
   * Liest die Aktionen unter Bercksichtigung des aktuellen WorkbenchWindow ein.<br>
   * Die Aktionen werden so erzeugt, dass die Actions am SelectionProvider des aktuellen EclipseWindows hngen. <br>
   * Evtl. muss noch das aktuelle Workbenchwindow beim speichern der Aktionen bercksichtigt werden. Dies ist fr ein
   * Window aber NICHT ntig.
   */
  @SuppressWarnings("restriction")
  public void leseActions(String id)
  {
    IExtensionRegistry reg = Platform.getExtensionRegistry();
    IConfigurationElement[] aConfigurationElementsFor = reg.getConfigurationElementsFor("org.eclipse.ui.actionSets");
    IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    for (IConfigurationElement confEle : aConfigurationElementsFor)
    {
      String contributor = confEle.getContributor().getName();
      if (Activator.PLUGIN_ID.equals(contributor))
      {
        IConfigurationElement[] iceAction = confEle.getChildren("action");
        for (IConfigurationElement element : iceAction)
        {
          String att = element.getAttribute("id");
          if (att.equals(id))
          {
            boolean pullDownStyle = false;
            String style = element.getAttribute(IWorkbenchRegistryConstants.ATT_STYLE);
            if (style != null)
            {
              pullDownStyle = style.equals(ActionDescriptor.STYLE_PULLDOWN);
            }
            else
            {
              String pulldown = element.getAttribute(ActionDescriptor.STYLE_PULLDOWN);
              pullDownStyle = pulldown != null && pulldown.equals("true");
            }

            ActionDescriptor desc = null;
            if (pullDownStyle)
            {
              desc = new ActionDescriptor(element, ActionDescriptor.T_WORKBENCH_PULLDOWN, window);
            }
            else
            {
              desc = new ActionDescriptor(element, ActionDescriptor.T_WORKBENCH, window);
            }
            final PluginAction action = desc.getAction();
            action.run();
          }
        }

      }
    }
  }

  /**
   * Fgt einer {@link IFigure} die Tool-Tip Funktionalitaet fr die Darstellung hinzu
   * 
   * @param result
   *          die Figure
   */
  protected void addToolTipListener(IFigure result)
  {
    result.addMouseMotionListener(new MouseMotionListener() {

      @Override
      public void mouseMoved(MouseEvent me)
      {
      }

      @Override
      public void mouseHover(MouseEvent me)
      {
        aktionToolTip(me);
      }

      @Override
      public void mouseExited(MouseEvent me)
      {
      }

      @Override
      public void mouseEntered(MouseEvent me)
      {
      }

      @Override
      public void mouseDragged(MouseEvent me)
      {
      }
    });
  }

  @Override
  public void zoomChanged(double zoom)
  {
    super.zoomChanged(zoom);
    getFigure().repaint();
  }

  protected void dataUpdateTimeAnzeigeQuerschnittModelTypAbstract(final AnzeigeQuerschnittFigure f, long l)
  {
    final AnzeigeQuerschnittLinksFigure links = (AnzeigeQuerschnittLinksFigure)f.getLinks();

    final String zeit = _sdf1.format(new Date(l));
    final String zeit2 = _sdf2.format(new Date(l));

    PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {

      @Override
      public void run()
      {
        links.getLabelZeit().setToolTip(new Label(zeit2));
        links.setZeit(zeit);
        f.repaint();
      }
    });
  }

  protected void dataUpdateAnzeigeQuerschnittModelTypAbstract(ResultData result,
      AnzeigeQuerschnittModelTypAbstract model)
  {

    // System.out.println("dataUpdateAnzeigeQuerschnittModelTypAbstract(): " + result.getObject().getName());

    AnzeigeQuerschnittFigure aqf = (AnzeigeQuerschnittFigure)getFigure();
    AttributeGroup atg = result.getDataDescription().getAttributeGroup();
    if (atg.getPid().equals("atg.anzeigeQuerschnittEigenschaftIst"))
    {
      int kontext = model.getKontext();
      ZusatzElementTyp status = ermittleProgrammStatusAusRueckmeldung(result);
      if (kontext == 0)
      {
        // if (!(figure instanceof AnzeigeQuerschnittTypA14EditPart))
        {
          ResultData result2 = DaVHelper.holeSchaltBildAnzeigeDatenSatz(kontext);
          ZusatzElementTyp statusNeu = ermittleProgrammStatusAusSswSbaSchaltbild(result2, model);

          if (statusNeu != null)
          {
            status = umsetzenProgrammZustand(status, statusNeu);
            updateDarstellungAusRueckmeldung(result, model, aqf, status);
          }

          long l = 0;

          EList<AnzeigeModelTypAbstract> komponenten = model.getKomponenten();
          for (AnzeigeModelTypAbstract anzeige : komponenten)
          {
            // Object obj = anzeige.getDataUpdate();
            Object obj = anzeige.eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE);

            if (obj instanceof ResultData)
            {
              ResultData resultAnz = (ResultData)obj;
              if (resultAnz != null && resultAnz.hasData())
              {
                if (resultAnz.getDataDescription().getAttributeGroup().getPid().equals("atg.anzeigeEigenschaftIst"))
                {
                  if (resultAnz.getDataTime() > l)
                    l = resultAnz.getDataTime();
                }
              }

            }
            else if (obj instanceof AnzeigeInhaltTyp)
            {
              AnzeigeInhaltTyp inhaltAnz = (AnzeigeInhaltTyp)obj;
              if (inhaltAnz != null)
              {
                if (inhaltAnz.getZeit() > l)
                  l = inhaltAnz.getZeit();
              }
            }
            // else if (obj instanceof VisuAnzeigeKapsel){
            // VisuAnzeigeKapsel kapselAnz = (VisuAnzeigeKapsel)obj;
            // if (kapselAnz != null)
            // {
            // if (kapselAnz.getTime() > l)
            // l = kapselAnz.getTime();
            // }
            // }

          }
          if (l > 0)
          {
            final String zeit = _sdf1.format(new Date(l));
            dataUpdateTimeAnzeigeQuerschnittModelTypAbstract(aqf, l);
          }
        }
      }
    }
    else if (atg.getPid().equals("atg.sswSbaSchaltbildAnzeige"))
    {
      int kontext = model.getKontext();

      ZusatzElementTyp status = ermittleProgrammStatusAusSswSbaSchaltbild(result, model);

      if (kontext < 2)
      {
        // if (!(figure instanceof AnzeigeQuerschnittTypA14EditPart))
        {
          ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

          SystemObject sbaAq = DaVHelper.holeSystemObjectSswSbaAqVonAq(connection, model.getSystemObject());
          if (sbaAq != null)
          {
            AttributeGroup atgNeu = connection.getDataModel().getAttributeGroup("atg.sswSbaHandeingriffBetriebsart");
            Aspect asp = connection.getDataModel().getAspect("asp.sswSbaHandschaltungVorgabe");
            ResultData resultNeu = connection.getData(sbaAq, new DataDescription(atgNeu, asp), 1000);
            ZusatzElementTyp statusNeu = ermittleProgrammStatusAusHandeingriff(resultNeu);
            status = umsetzenProgrammZustand(statusNeu, status);
          }

          SystemObject aq = model.getSystemObject();

          if (aq != null)
          {
            IWorkbench wb = PlatformUI.getWorkbench();
            if (wb != null)
            {
              IWorkbenchWindow dw = wb.getActiveWorkbenchWindow();
              if (dw != null)
              {
                IWorkbenchPage page = dw.getActivePage();
                if (page != null)
                {
                  IViewReference[] refs = page.getViewReferences();
                  for (IViewReference ref : refs)
                  {
                    String id = ref.getId();
                    if (id.equals(ZustandsDarstellungIstView.ID))
                    {
                      ZustandsDarstellungIstView view = (ZustandsDarstellungIstView)ref.getView(false);
                      if (view != null)
                      {
                        GraphicalViewer viewer = view.getGraphicalViewer();
                        if (viewer != null)
                        {
                          EditPart contents = viewer.getContents();
                          if (contents instanceof AnsichtEditPart)
                          {
                            AnsichtEditPart aep = (AnsichtEditPart)contents;
                            AnzeigeQuerschnittModelTypAbstract modelNeu = DaVHelper
                                .holeAnzeigeQuerschnittModelTypAbstractVonSystemObject(aep, aq, 0);
                            Map reg = viewer.getEditPartRegistry();
                            if (reg != null && modelNeu != null)
                            {
                              Object editPart = reg.get(modelNeu);
                              if (editPart instanceof DoMitVerbLinieEditPart)
                              {
                                DoMitVerbLinieEditPart domEditPart = (DoMitVerbLinieEditPart)editPart;
                                IFigure figNeu = domEditPart.getFigure();
                                if (figNeu instanceof AnzeigeQuerschnittFigure)
                                {
                                  AnzeigeQuerschnittFigure aqfNeu = (AnzeigeQuerschnittFigure)figNeu;
                                  AttributeGroup atgNeu = connection.getDataModel().getAttributeGroup(
                                      "atg.anzeigeQuerschnittEigenschaftIst");
                                  Aspect asp = connection.getDataModel().getAspect("asp.aqZustand");
                                  ResultData resultNeu = connection.getData(aq, new DataDescription(atgNeu, asp), 1000);
                                  ZusatzElementTyp statusNeu = ermittleProgrammStatusAusRueckmeldung(resultNeu);
                                  if (statusNeu.equals(ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM)
                                      || statusNeu.equals(ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM_BLIND))
                                  {
                                    statusNeu = umsetzenProgrammZustand(status, statusNeu);

                                    IFigure figure = aqfNeu.getRechts();
                                    if (figure instanceof AnzeigeQuerschnittRechtsFigure)
                                    {
                                      AnzeigeQuerschnittRechtsFigure rechts = (AnzeigeQuerschnittRechtsFigure)figure;
                                      ZusatzElementTyp statusAlt = rechts.getZustandProgramm();

                                      if (!statusAlt.equals(statusNeu))
                                      {
                                        updateDarstellungAusRueckmeldung(resultNeu, modelNeu, aqfNeu, statusNeu);
                                      }
                                    }
                                  }
                                }
                              }
                            }
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
      updateDarstellungAusSchaltbild(result, model, aqf, status);
    }
    else if (atg.getPid().equals("atg.sswSbaHandeingriffBetriebsart"))
    {
      int kontext = model.getKontext();

      if (kontext == 1)
      {
        ZusatzElementTyp status = ermittleProgrammStatusAusHandeingriff(result);

        if (getModel().getSystemObject() != null)
        {

          ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

          AttributeGroup atgNeu = connection.getDataModel().getAttributeGroup("atg.sswSbaSchaltbildAnzeige");
          Aspect asp = connection.getDataModel().getAspect("asp.sswSbaBefehle");

          IPreferenceStore store = Activator.getDefault().getPreferenceStore();
          String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
          if (pidSba != null && pidSba.length() > 0)
          {
            SystemObject object = connection.getDataModel().getObject(pidSba);
            if (object != null)
            {
              ResultData resultNeu = connection.getData(object, new DataDescription(atgNeu, asp), 1000);
              ZusatzElementTyp statusNeu = ermittleProgrammStatusAusSswSbaSchaltbild(resultNeu, model);
              status = umsetzenProgrammZustand(status, statusNeu);
            }
          }
        }

        if (!status.equals(ZusatzElementTyp.PROGRAMM_UNDEFINIERT))
        {
          updateDarstellungAusHandeingriff(result, model, aqf, status);
        }

      }
    }
    else if (atg.getPid().equals("atg.sswSbaHelligkeitProzentErgebnis"))
    {
      int kontext = model.getKontext();
      if (kontext == 1)
      {
        Color color = ermittleFarbeAusHelligkeitProzent(result);
        updateDarstellungAusHelligkeitProzent(result, model, aqf, color);
      }

    }
  }

  private ZusatzElementTyp umsetzenProgrammZustand(ZusatzElementTyp status, ZusatzElementTyp statusNeu)
  {
    if (status.equals(ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM))
    {
      if (statusNeu.equals(ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM))
        status = ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM;
      if (statusNeu.equals(ZusatzElementTyp.PROGRAMM_SBA_HANDPROGRAMM))
        status = ZusatzElementTyp.PROGRAMM_SBA_HANDPROGRAMM;
      else if (statusNeu.equals(ZusatzElementTyp.PROGRAMM_SBA_SONDERPROGRAMM))
        status = ZusatzElementTyp.PROGRAMM_SBA_SONDERPROGRAMM;
      // System.out.println("Vorgabe des Programmstatus durch Befehl: " + status.getType());
    }
    else if (status.equals(ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM_BLIND))
    {
      if (statusNeu.equals(ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM))
        status = ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM_BLIND;
      else if (statusNeu.equals(ZusatzElementTyp.PROGRAMM_SBA_HANDPROGRAMM))
        status = ZusatzElementTyp.PROGRAMM_SBA_HANDPROGRAMM_BLIND;
      else if (statusNeu.equals(ZusatzElementTyp.PROGRAMM_SBA_SONDERPROGRAMM))
        status = ZusatzElementTyp.PROGRAMM_SBA_SONDERPROGRAMM_BLIND;
      // System.out.println("Vorgabe des Programmstatus durch Befehl: " + status.getType());
    }
    else
    {
      // System.out.println("Vorgabe des Programmstatus durch Rckmeldung: " + status.getType());
    }
    return status;
  }

  private String umsetzungBetriebsZustand(final String s)
  {
    String s0 = "Betriebszustand ";

    if (s.contains("AQExterneSteuerung") || s.contains("Handprogramm"))
    {
      s0 += "Handprogramm";
    }
    else if (s.contains("AQVorortbetrieb"))
    {
      s0 += "Vorortbetrieb";
    }
    else if (s.contains("Sonderprogramm"))
    {
      s0 += "Sonderprorgramm";
    }
    else if (s.contains("AQNormalbetrieb"))
    {
      s0 += "Automatikprogramm";
    }
    else if (s.contains("AQAutarkbetrieb"))
    {
      s0 += "Autarkbetrieb";
    }
    else if (s.contains("AQBlindbetrieb"))
    {
      s0 += "Automatikprogramm";
    }
    else if (s.contains("AQTestbetrieb"))
    {
      s0 += "Testbetrieb";
    }
    else if (s.contains("AQTunnelbetrieb"))
    {
      s0 += "Tunnelbetrieb";
    }
    else if (s.contains("AQNotbetrieb"))
    {
      s0 += "Notbetrieb";
    }

    if (s.contains("Blind"))
      s0 += " im Blindbetrieb";

    return s0;
  }

  protected void statusUpdateAnzeigeModelTypAbstract(ResultData result, IFigure figure)
  {
    if (result != null)
    {
      Data data = result.getData();
      if (data != null)
      {
        AttributeGroup atg = result.getDataDescription().getAttributeGroup();
        if (atg.getPid().equals("atg.anzeigeEigenschaftIst"))
        {
          boolean b = data.getItem("Status").asTextValue().getValueText().equals("OK");
          if (b)
          {
            setFigureBorder(figure, new LineBorder(ColorConstants.green, 2));
          }
          else
            setFigureBorder(figure, new LineBorder(ColorConstants.red, 2));
        }
      }
      else
      {
        setFigureBorder(figure, new LineBorder(ColorConstants.red, 2));
      }
    }
    else
    {
      setFigureBorder(figure, new LineBorder(ColorConstants.red, 2));
    }
  }

  /**
   * Setzt den {@link Border} einer {@link IFigure}, sofern diese {@link IFigure} dies erlaubt.
   * 
   * @param figure
   *          {@link IFigure} deren {@link Border} zu setzen ist.
   * @param border
   *          Die zu setzende {@link Border}.
   */
  private void setFigureBorder(IFigure figure, Border border)
  {
    if ((figure != null) && (border != null))
    {
      boolean borderChangeable = true;

      if (figure instanceof AnzeigeQuerschnittLinksFigure) // Querschnittstatus
      {
        AnzeigeQuerschnittLinksFigure linksFigure = (AnzeigeQuerschnittLinksFigure)figure;
        borderChangeable = linksFigure.isBorderChangeable();
        if (borderChangeable) // Querschnittsstatus nderbar
        {
          linksFigure.setBorder(border);
        }
      }
      else // alle anderen Objekte
      {
        figure.setBorder(border);
      }
    }
  }


  protected void updateAnzeigeInhalt(int kontext, Object newValue, Object oldValue, AnzeigeElementTyp anzeigeTyp)
  {

//    System.out.println(getModel().getSystemObject().getPid() + " -> DoMitVerbLinieEditPart.updateAnzeigeInhalt(): "  + kontext + " -> " + newValue);
    
    Object typNeu = null;
    Object typAlt = null;
    
    boolean blinkenAlt = false;
    boolean blinken = false;

    if (newValue instanceof ResultData)
    {
      ResultData result = (ResultData)newValue;
      typNeu = DaVHelper.holeAnzeigeInhaltAusDaVData(result);

      DoModelMitVerbLinie model = getModel();
      
      blinken = DaVHelper.holeIsBlinkenInhaltAusDaVData(result, getModel().getSystemObject());
      

      if (true)
      {
        if ((model != null) && (model instanceof AnzeigeModelTypAbstract))
        {
          AnzeigeModelTypAbstract anzModel = (AnzeigeModelTypAbstract) model;
          
          blinkenAlt = anzModel.getBlinken() == 1 ? true : false;
          
          if (blinken)
          {
            anzModel.setBlinken(ADefs.BLINKEN_EIN);
          } else
          {
            anzModel.setBlinken(ADefs.BLINKEN_AUS);
          }

        }
      }
      
      
      if (oldValue instanceof ResultData)
      {
        ResultData resultAlt = (ResultData)oldValue;
        typAlt = DaVHelper.holeAnzeigeInhaltAusDaVData(resultAlt);
      }
      else if (oldValue instanceof VisuAnzeigeKapsel)
      {
        VisuAnzeigeKapsel resultAlt = (VisuAnzeigeKapsel)oldValue;
        typAlt = resultAlt;
      }
      else
      {
        // System.out.println("values1: " + oldValue);
      }
    }
    else if (newValue instanceof AnzeigeInhaltTyp)
    {
      if (kontext != 0)
      {
        typNeu = (AnzeigeInhaltTyp)newValue;                
        
        if (kontext == 1 || kontext == 3){
          IPreferenceStore store = Activator.getDefault().getPreferenceStore();
          String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
          if (pidSba != null && pidSba.length() > 0)
          {
            ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
            SystemObject object = connection.getDataModel().getObject(pidSba);
            if (object != null)
            {
              if (connection != null)
              {
                
                String strAsp = "asp.sswSbaBefehle";
                if (kontext == 3)
                  strAsp = "asp.sswSbaSonderprogrammSimu";
                
                AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaSchaltbildAnzeige");
                Aspect asp = connection.getDataModel().getAspect(strAsp);
                ResultData result2 = connection.getData(object, new DataDescription(atg, asp), 1000);
                blinken = DaVHelper.holeIsBlinkenInhaltAusDaVData(result2, getModel().getSystemObject());
                
                DoModelMitVerbLinie model = getModel();
                if ((model != null) && (model instanceof AnzeigeModelTypAbstract))
                {
                  AnzeigeModelTypAbstract anzModel = (AnzeigeModelTypAbstract)model;
                  
                  blinkenAlt = anzModel.getBlinken() == 1 ? true : false;
                  
                  if (blinken)
                  {
                    anzModel.setBlinken(ADefs.BLINKEN_EIN);
                  }
                  else
                  {
                    anzModel.setBlinken(ADefs.BLINKEN_AUS);
                  }
                }
              }
            }
          }          
        }
        
        if (oldValue instanceof AnzeigeInhaltTyp)
        {
          typAlt = (AnzeigeInhaltTyp)oldValue;
        }
        else if (oldValue instanceof VisuAnzeigeKapsel)
        {
          VisuAnzeigeKapsel resultAlt = (VisuAnzeigeKapsel)oldValue;
          typAlt = resultAlt;
        }
        else
        {
          // System.out.println("values2: " + oldValue);
        }

      }
    }
    else if (newValue instanceof VisuAnzeigeKapsel)
    {
      if (kontext != 0)
      {
        typNeu = (VisuAnzeigeKapsel)newValue;
        if (oldValue instanceof AnzeigeInhaltTyp)
        {
          typAlt = (AnzeigeInhaltTyp)oldValue;
        }
        else if (oldValue instanceof ResultData)
        {
          ResultData resultAlt = (ResultData)oldValue;
          typAlt = DaVHelper.holeAnzeigeInhaltAusDaVData(resultAlt);
        }
        else
        {
          // System.out.println("values2: " + oldValue);
        }

      }
    }
    else
    {
      // System.err.println("newValue -> else1: " + newValue);
    }

    boolean neuDatenAnzeigeInhalt = false;
    if (typNeu != null && typAlt != null && !typNeu.equals(typAlt))
      neuDatenAnzeigeInhalt = true;
    else if (typNeu != null || typAlt == null )
    {
       neuDatenAnzeigeInhalt = true;

    }
    else if (blinkenAlt != blinken)
    {
//      System.out.println(getModel().getSystemObject().getName() + " -----------> neue Daten " +
//          neuDatenAnzeigeInhalt);
      neuDatenAnzeigeInhalt = true;
      
    }
    else
    {
//      System.err.println("valuesEnd: " + getModel().getSystemObject().getPid() + " : " + typNeu + " : " + typAlt + " : " + blinken + " : " + blinkenAlt);
    }

//    System.out.println(getModel().getSystemObject().getName() + " -----------> neue Daten " +
//        neuDatenAnzeigeInhalt + " " + blinken + " " + blinkenAlt);
    // System.out.println(typNeu + "+++" + neuDatenAnzeigeInhalt + " -> " + typAlt);

    if (typNeu != null && neuDatenAnzeigeInhalt)
    {
      // Update AnzeigeInhalt
      // if (kontext != 0)
      {
        if (typNeu instanceof AnzeigeInhaltTyp)
        {
          AnzeigeInhaltTyp ait = (AnzeigeInhaltTyp)typNeu;
          inhaltUpdateAnzeigeModelTypAbstract(anzeigeTyp, ait);
          // if (kontext != 0)
          {
            zeitUpdateAnzeigeQuerschnittModelTypAbstract(ait);
          }
        }
      }
      // Update Status Anzeige
      if (newValue instanceof ResultData)
      {
        ResultData result = (ResultData)newValue;
        statusUpdateAnzeigeModelTypAbstract(result, getFigure());
      }
      // Update Zeit AnzeigeQuerschnitt
    }
    else
    {
      // System.err.println("neueDaten -> else: " + newValue + " : " + oldValue);
    }
  }

  private void zeitUpdateAnzeigeQuerschnittModelTypAbstract(AnzeigeInhaltTyp typNeu)
  {
    EditPart parent = getParent();
    DoMitVerbLinieEditPart part = (DoMitVerbLinieEditPart)parent;
    if (parent instanceof DoMitVerbLinieEditPart)
    {
      DoModelMitVerbLinie model = part.getModel();
      if (model instanceof AnzeigeQuerschnittModelTypAbstract)
      {
        AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)model;
        EList<AnzeigeModelTypAbstract> komponenten = aq.getKomponenten();
        long l = 0l;
        for (AnzeigeModelTypAbstract anzeige : komponenten)
        {
          Object obj = anzeige.eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE);
          if (obj instanceof ResultData)
          {
            ResultData resultAnz = (ResultData)obj;
            if (resultAnz != null && resultAnz.hasData())
            {
              if (resultAnz.getDataDescription().getAttributeGroup().getPid().equals("atg.anzeigeEigenschaftIst"))
              {
                if (resultAnz.getDataTime() > l)
                  l = resultAnz.getDataTime();
              }
            }

          }
          else if (obj instanceof AnzeigeInhaltTyp)
          {
            AnzeigeInhaltTyp inhaltAnz = (AnzeigeInhaltTyp)obj;
            if (inhaltAnz != null && inhaltAnz.hasTime())
            {
              if (inhaltAnz.getZeit() > l)
                l = inhaltAnz.getZeit();
            }
          }
        }

        // System.out.println("zeitUpdateAnzeigeQuerschnittModelTypAbstract(): " +
        // getModel().getSystemObject().getName() + " : " + typNeu.getType() + " -> " + l);
        if (l > 0)
        {

          IFigure fig = part.getFigure();
          if (fig instanceof AnzeigeQuerschnittFigure)
          {
            AnzeigeQuerschnittFigure aqf = (AnzeigeQuerschnittFigure)fig;
            final String zeit = _sdf1.format(new Date(l));
            dataUpdateTimeAnzeigeQuerschnittModelTypAbstract(aqf, l);
          }
        }
      }

      // IFigure fig = part.getFigure();
      // if (fig instanceof AnzeigeQuerschnittFigure)
      // {
      // AnzeigeQuerschnittFigure aqf = (AnzeigeQuerschnittFigure)fig;
      // final AnzeigeQuerschnittLinksFigure links = (AnzeigeQuerschnittLinksFigure)aqf.getLinks();
      //        
      // if (typNeu.hasTime())
      // {
      // long l = typNeu.getZeit();
      // final String zeit = _sdf1.format(new Date(l));
      // final String zeit2 = _sdf2.format(new Date(l));
      //          
      // PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
      //            
      // @Override
      // public void run()
      // {
      // links.getLabelZeit().setToolTip(new Label(zeit2));
      // links.setZeit(zeit);
      // }
      // });
      // }
      // }
    }
  }

  protected void dataUpdateAnzeigeModelTypAbstract(final AnzeigeElementTyp anzeigeElementTyp, ResultData result)
  {
    // System.out.println("DoMitVerbLinieEditPart.dataUpdateAnzeigeModelTypAbstract(): " + anzeigeElementTyp.getType());

    final AnzeigeElementFigure figure = (AnzeigeElementFigure)getFigure();
    AttributeGroup atg = result.getDataDescription().getAttributeGroup();

    if (atg.getPid().equals("atg.anzeigeEigenschaftIst")
    // || atg.getPid().equals("atg.anzeigeEigenschaftSoll")
    )
    {
      Data data = result.getData();
      if (data != null)
      {

        SystemObject so = data.getItem("Eigenschaft").getItem("AnzeigeInhalt").asReferenceValue().getSystemObject();
        ClientDavInterface dav = DaVVerbindung.getInstanz().getDaVVerbindung();
        AttributeGroup atg2 = dav.getDataModel().getAttributeGroup("atg.wvzInhalt");

        if (so != null)
        {
          Data data2 = so.getConfigurationData(atg2);
          final String inhalt = data2.getItem("Bildinhalt").asTextValue().getValueText();

          AnzeigeInhaltTyp typ = AnzeigeInhaltTyp.setBildInhalt(inhalt, result.getDataTime());

          AnzeigeModelTypAbstract model = (AnzeigeModelTypAbstract)getModel();
          if (model.getSystemObject().equals(result.getObject()))
          {
            inhaltUpdateAnzeigeModelTypAbstract(anzeigeElementTyp, typ);
          }
        }
        else
        {
          inhaltUpdateAnzeigeModelTypAbstract(anzeigeElementTyp, AnzeigeInhaltTyp.GESTOERT_BI);
        }

        if (atg.getPid().equals("atg.anzeigeEigenschaftIst"))
        {
          String status = data.getItem("Status").asTextValue().getValueText();
          if (status.equals("OK"))
          {
            setFigureBorder(figure, new LineBorder(ColorConstants.green, 2));
          }
          else
          {
            setFigureBorder(figure, new LineBorder(ColorConstants.red, 2));
          }
        }
      }
      else
      {
        inhaltUpdateAnzeigeModelTypAbstract(anzeigeElementTyp, AnzeigeInhaltTyp.GESTOERT_BI);
      }
    }
  }

  protected void inhaltUpdateAnzeigeModelTypAbstract(final AnzeigeElementTyp typ, final AnzeigeInhaltTyp inhalt)
  {

    final AnzeigeElementFigure figure = (AnzeigeElementFigure)getFigure();
    Boolean isFrNord = null;
    EditPart parentEditPart = getParent();
    if (parentEditPart != null)
    {
      Object parentModel = parentEditPart.getModel();

      if (parentModel instanceof AnzeigeQuerschnittModelTypAbstract)
      {
        AnzeigeQuerschnittModelTypAbstract aqModel = (AnzeigeQuerschnittModelTypAbstract)parentModel;
        isFrNord = aqModel.getFahrtrichtung() == 0 ? true : false;
      }
    }

    if (isFrNord == null)
    {
      /*
       * Versuchen aus sich selbst heraus die Fahrtrichtung zu bestimmen
       */
      if ((getModel() != null) && (getModel() instanceof AnzeigeModelTypAbstract))
      {
        AnzeigeModelTypAbstract anzModel = (AnzeigeModelTypAbstract)getModel();
        isFrNord = anzModel.getFahrtrichtung() == 0 ? true : false;
      }
    }

    if (isFrNord == null)
    {
      isFrNord = true;
    }

    final boolean fIsFrNord = isFrNord;
    String imagepath = AnzeigeElementFigure.createImagePath(typ, inhalt, fIsFrNord);
    
    /*
     * Gesetztes Image merken
     */
    _pathBaseImage = imagepath;    
    
    final Image image = ImageFactory.getImage(imagepath);
        
    
    PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
      @Override
      public void run()
      {         
        if (image != null && !image.isDisposed())
        {
          figure.setImage(image);
        }
        else
        {
          System.out.println("Disposed-> inhaltUpdate: " + typ.getType() + " : " + getModel().getSystemObject()
              + " -> " + inhalt.getType());
        }
      }
    });
  }

  protected void dataUpdateStreckenStationModel(final AnzeigeInhaltTyp status)
  {
    final StreckenStationModel model = (StreckenStationModel)getModel();
    final StreckenstationFigure figure = ((StreckenstationFigure)getFigure());
    final StreckenstationAnzeigeElementFigure inhalt = (StreckenstationAnzeigeElementFigure)figure.getLinks();

    PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
      @Override
      public void run()
      {

        AnzeigeElementTyp typ = AnzeigeElementTyp.STRECKEN_STATION;

        DoModelMitVerbLinie model = getModel();

        if (model instanceof StreckenStationModelA14)
        {
          typ = AnzeigeElementTyp.STRECKEN_STATION_A14;
        }

        String path = AnzeigeElementFigure.createImagePath(typ, status);
        Image image = ImageFactory.getImage(path);
        inhalt.getSst().setImage(image);
      }
    });
  }

  protected void dataUpdateUmfeldDaten(ResultData result)
  {
    SystemObject so = result.getObject();
    if (so != null)
    {
      if (so.isOfType("typ.sswSbaUmfeldDatenMessStelle"))
      {
        AttributeGroup atg = result.getDataDescription().getAttributeGroup();
        if (atg.getPid().equals("atg.sswSbaUfdmsNsseStufe"))
        {
          Data data = result.getData();
          if (data != null)
          {
            if (getModel().getSystemObject().equals(result.getObject()))
            {
              String stufe = data.getItem("NsseStufe").asTextValue().getValueText();

              ZusatzElementTyp inhalt = null;

              if (stufe.equals("trocken"))
              {
                inhalt = ZusatzElementTyp.NIEDERSCHLAG_TROCKEN;
              }
              else if (stufe.equals("nass1"))
              {
                inhalt = ZusatzElementTyp.NIEDERSCHLAG_NASS_1;
              }
              else if (stufe.equals("nass2"))
              {
                inhalt = ZusatzElementTyp.NIEDERSCHLAG_NASS_2;
              }
              else if (stufe.equals("nass3"))
              {
                inhalt = ZusatzElementTyp.NIEDERSCHLAG_NASS_3;
              }
              else if (stufe.equals("nass4"))
              {
                inhalt = ZusatzElementTyp.NIEDERSCHLAG_NASS_4;
              }
              else if (stufe.equals("Platzregen"))
              {
                inhalt = ZusatzElementTyp.NIEDERSCHLAG_NASS_5;
              }
              else if (stufe.equals("darf nicht ermittelt werden"))
              {
                inhalt = ZusatzElementTyp.NIEDERSCHLAG_NICHT_ERMITTELBAR;
              }
              else
              {
                inhalt = ZusatzElementTyp.NIEDERSCHLAG_GESTOERT;
              }

              final ZusatzElementTyp finhalt = inhalt;

              PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
                @Override
                public void run()
                {
                  IFigure figure = getFigure();
                  if (figure instanceof UmfeldDatenBandFigure)
                  {
                    UmfeldDatenBandFigure fig = (UmfeldDatenBandFigure)figure;
                    fig.setImage(finhalt);
                  }
                }
              });
            }
          }
          else
          {
            if (getModel().getSystemObject().equals(result.getObject()))
            {

              PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
                @Override
                public void run()
                {
                  UmfeldDatenBandFigure fig = (UmfeldDatenBandFigure)getFigure();
                  fig.setImage(ZusatzElementTyp.NIEDERSCHLAG_GESTOERT);
                }
              });
            }
          }
        }
      }
      else if (so.isOfType("typ.ufdsHelligkeit"))
      {
        AttributeGroup atg = result.getDataDescription().getAttributeGroup();
        if (atg.getPid().equals("atg.ufdsStufeHelligkeit"))
        {
          Data data = result.getData();
          if (data != null)
          {
            if (getModel().getSystemObject().equals(result.getObject()))
            {
              String stufe = data.getItem("Stufe").asTextValue().getValueText();
              ZusatzElementTyp inhalt = null;
              try
              {
                Integer i = new Integer(stufe);
                switch (i)
                {
                  case 0:
                    inhalt = ZusatzElementTyp.HELLIGKEIT_STUFE_1;
                    break;
                  case 1:
                    inhalt = ZusatzElementTyp.HELLIGKEIT_STUFE_2;
                    break;
                  case 2:
                    inhalt = ZusatzElementTyp.HELLIGKEIT_STUFE_3;
                    break;
                  case 3:
                    inhalt = ZusatzElementTyp.HELLIGKEIT_STUFE_4;
                    break;
                  case 4:
                    inhalt = ZusatzElementTyp.HELLIGKEIT_STUFE_5;
                    break;
                  case 5:
                    inhalt = ZusatzElementTyp.HELLIGKEIT_STUFE_6;
                    break;
                  case 6:
                    inhalt = ZusatzElementTyp.HELLIGKEIT_STUFE_7;
                    break;
                  case 7:
                    inhalt = ZusatzElementTyp.HELLIGKEIT_STUFE_8;
                    break;
                  case 8:
                    inhalt = ZusatzElementTyp.HELLIGKEIT_STUFE_9;
                    break;
                  case 9:
                    inhalt = ZusatzElementTyp.HELLIGKEIT_STUFE_10;
                    break;

                  default:
                    inhalt = ZusatzElementTyp.HELLIGKEIT_GESTOERT;
                    break;
                }
              }
              catch (NumberFormatException e)
              {
                inhalt = ZusatzElementTyp.HELLIGKEIT_GESTOERT;
              }

              final ZusatzElementTyp finhalt = inhalt;

              PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
                @Override
                public void run()
                {
                  UmfeldDatenBandFigure fig = (UmfeldDatenBandFigure)getFigure();
                  fig.setImage(finhalt);
                }
              });
            }
          }
          else
          {
            if (getModel().getSystemObject().equals(result.getObject()))
            {

              PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
                @Override
                public void run()
                {
                  UmfeldDatenBandFigure fig = (UmfeldDatenBandFigure)getFigure();
                  fig.setImage(ZusatzElementTyp.HELLIGKEIT_GESTOERT);
                }
              });
            }
          }
        }
      }
      else if (so.isOfType("typ.ufdsSichtWeite"))
      {
        AttributeGroup atg = result.getDataDescription().getAttributeGroup();
        if (atg.getPid().equals("atg.ufdsStufeSichtWeite"))
        {
          Data data = result.getData();
          if (data != null)
          {
            if (getModel().getSystemObject().equals(result.getObject()))
            {
              String stufe = data.getItem("Stufe").asTextValue().getValueText();

              ZusatzElementTyp inhalt = null;
              try
              {
                Integer i = new Integer(stufe);
                switch (i)
                {
                  case 0:
                    inhalt = ZusatzElementTyp.SICHTWEITE_STUFE_1;
                    break;
                  case 1:
                    inhalt = ZusatzElementTyp.SICHTWEITE_STUFE_2;
                    break;
                  case 2:
                    inhalt = ZusatzElementTyp.SICHTWEITE_STUFE_3;
                    break;
                  case 3:
                    inhalt = ZusatzElementTyp.SICHTWEITE_STUFE_4;
                    break;
                  case 4:
                    inhalt = ZusatzElementTyp.SICHTWEITE_STUFE_5;
                    break;
                  case 5:
                    inhalt = ZusatzElementTyp.SICHTWEITE_STUFE_6;
                    break;

                  default:
                    inhalt = ZusatzElementTyp.SICHTWEITE_GESTOERT;
                    break;
                }
              }
              catch (NumberFormatException e)
              {
                inhalt = ZusatzElementTyp.SICHTWEITE_GESTOERT;
              }

              final ZusatzElementTyp finhalt = inhalt;

              PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
                @Override
                public void run()
                {
                  UmfeldDatenBandFigure fig = (UmfeldDatenBandFigure)getFigure();
                  fig.setImage(finhalt);
                }
              });
            }
          }
          else
          {
            if (getModel().getSystemObject().equals(result.getObject()))
            {

              PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
                @Override
                public void run()
                {
                  UmfeldDatenBandFigure fig = (UmfeldDatenBandFigure)getFigure();
                  fig.setImage(ZusatzElementTyp.SICHTWEITE_GESTOERT);
                }
              });
            }
          }
        }
      }
    }
  }

  // protected void dataUpdateFahrStreifen(ResultData result)
  // {
  // final IFigure inhalt = getFigure();
  //
  // AttributeGroup atg = result.getDataDescription().getAttributeGroup();
  //
  // Data data = result.getData();
  // if (atg.getPid().equals("atg.strfallZustand"))
  // {
  // if (data != null)
  // {
  //
  // String s = data.getItem("Situation").asTextValue().getValueText();
  // _situation = SituationTyp.setTyp(s);
  // }
  // else
  // {
  // _situation = SituationTyp.setTyp(SituationTyp.KEINE_AUSSAGE.getType());
  // }
  // situationChange(inhalt);
  // }
  // else if ("atg.passivierungFs".equals(atg.getPid()))
  // {
  // if (data != null)
  // {
  // String value = data.getItem("passiviert").asTextValue().getValueText();
  // _passiviert = value.equalsIgnoreCase("Ja");
  // }
  // else
  // {
  // _passiviert = false;
  // }
  // situationChange(inhalt);
  // }
  // }
  //  
  // private void situationChange(final IFigure inhalt)
  // {
  // PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
  // @Override
  // public void run()
  // {
  // if (_situation.getType().equals(SituationTyp.STOERUNG.getType()))
  // {
  // inhalt.setBackgroundColor(ColorConstants.white);
  // inhalt.setBorder(new LineBorder(ColorConstants.white, 1));
  // }
  // else if (_situation.getType().equals(SituationTyp.KEINE_AUSSAGE.getType()))
  // {
  // inhalt.setBackgroundColor(ColorConstants.gray);
  // inhalt.setBorder(new LineBorder(ColorConstants.gray, 1));
  // }
  // else if (_situation.getType().equals(SituationTyp.VERKEHR_FREI.getType()))
  // {
  // inhalt.setBackgroundColor(ColorConstants.lightGreen);
  // inhalt.setBorder(new LineBorder(ColorConstants.lightGreen, 1));
  // }
  // else if (_situation.getType().equals(SituationTyp.VERKEHR_LEBHAFT.getType()))
  // {
  // inhalt.setBackgroundColor(ColorConstants.green);
  // inhalt.setBorder(new LineBorder(ColorConstants.green, 1));
  // }
  // else if (_situation.getType().equals(SituationTyp.VERKEHR_DICHT.getType()))
  // {
  // inhalt.setBackgroundColor(ColorConstants.yellow);
  // inhalt.setBorder(new LineBorder(ColorConstants.yellow, 1));
  // }
  // else if (_situation.getType().equals(SituationTyp.VERKEHR_STOCKEND.getType()))
  // {
  // inhalt.setBackgroundColor(ColorConstants.red);
  // inhalt.setBorder(new LineBorder(ColorConstants.red, 1));
  // }
  // else if (_situation.getType().equals(SituationTyp.VERKEHR_ZAEHFLIESSEND.getType()))
  // {
  // inhalt.setBackgroundColor(ColorConstants.orange);
  // inhalt.setBorder(new LineBorder(ColorConstants.orange, 1));
  // }
  // else if (_situation.getType().equals(SituationTyp.STAU.getType()))
  // {
  // inhalt.setBackgroundColor(ColorConstants.red);
  // inhalt.setBorder(new LineBorder(ColorConstants.red, 1));
  // }
  // }
  // });
  // }
  protected void dataUpdateAnschlussStelle(ResultData result)
  {
    final AnschlussStelleFigure poly = (AnschlussStelleFigure)getFigure();

    AttributeGroup atg = result.getDataDescription().getAttributeGroup();

    if (atg.getPid().equals("atg.strfallZustand"))
    {
      Data data = result.getData();
      if (data != null)
      {
        String s = data.getItem("Situation").asTextValue().getValueText();
        final SituationTyp situation = SituationTyp.setTyp(s);

        PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
          @Override
          public void run()
          {
            if (situation.getType().equals(SituationTyp.STOERUNG.getType()))
            {
              poly.setFillColor(ColorConstants.white);
              poly.setBorder(poly.new AnschlussStelleFigureBorder(ColorConstants.white));
            }
            else if (situation.getType().equals(SituationTyp.KEINE_AUSSAGE.getType()))
            {
              poly.setFillColor(ColorConstants.gray);
              poly.setBorder(poly.new AnschlussStelleFigureBorder(ColorConstants.gray));
            }
            else if (situation.getType().equals(SituationTyp.VERKEHR_FREI.getType()))
            {
              poly.setFillColor(ColorConstants.lightGreen);
              poly.setBorder(poly.new AnschlussStelleFigureBorder(ColorConstants.lightGreen));
            }
            else if (situation.getType().equals(SituationTyp.VERKEHR_LEBHAFT.getType()))
            {
              poly.setFillColor(ColorConstants.green);
              poly.setBorder(poly.new AnschlussStelleFigureBorder(ColorConstants.green));
            }
            else if (situation.getType().equals(SituationTyp.VERKEHR_DICHT.getType()))
            {
              poly.setFillColor(ColorConstants.yellow);
              poly.setBorder(poly.new AnschlussStelleFigureBorder(ColorConstants.yellow));
            }
            else if (situation.getType().equals(SituationTyp.VERKEHR_STOCKEND.getType()))
            {
              poly.setFillColor(ColorConstants.yellow);
              poly.setBorder(poly.new AnschlussStelleFigureBorder(ColorConstants.yellow));
            }
            else if (situation.getType().equals(SituationTyp.VERKEHR_ZAEHFLIESSEND.getType()))
            {
              poly.setFillColor(ColorConstants.orange);
              poly.setBorder(poly.new AnschlussStelleFigureBorder(ColorConstants.orange));
            }
            else if (situation.getType().equals(SituationTyp.STAU.getType()))
            {
              poly.setFillColor(ColorConstants.red);
              poly.setBorder(poly.new AnschlussStelleFigureBorder(ColorConstants.red));
            }
          }
        });

      }
      else
      {
        PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
          @Override
          public void run()
          {
            poly.setFillColor(ColorConstants.gray);
            poly.setBorder(poly.new AnschlussStelleFigureBorder(ColorConstants.gray));
          }
        });
      }
    }
  }

  protected void dataUpdateDetektorLVE(ResultData result)
  {

    final DetektorLveFigure figure = (DetektorLveFigure)getFigure();
    AttributeGroup atg = result.getDataDescription().getAttributeGroup();

    if (atg.getPid().equals("atg.tlsGloDeFehler"))
    {
      Data data = result.getData();
      if (data != null)
      {

        String s = data.getItem("DEFehlerStatus").asTextValue().getValueText();
        StatusTyp status = StatusTyp.setTyp(s);

        // Wenn Passiviert, dann ist der DeFehlerStatus egal!
        if (data.getItem("DEKanalStatus").asTextValue().getValueText().equals("passiv"))
          status = StatusTyp.PASSIVIERT;

        if (getModel().getSystemObject().equals(result.getObject()))
        {
          // if (getModel().getSystemObject().equals(result.getObject()))
          {
            {

              final StatusTyp fstatus = status;

              PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
                @Override
                public void run()
                {
                  if (fstatus.equals(StatusTyp.PASSIVIERT))
                  {
                    figure.getRf().setBackgroundColor(ColorConstants.orange);
                  }
                  else if (fstatus.equals(StatusTyp.OK_DE))
                  {
                    figure.getRf().setBackgroundColor(ColorConstants.white);
                  }
                  else if (fstatus.equals(StatusTyp.STOER_SM))
                  {
                    figure.getRf().setBackgroundColor(ColorConstants.red);
                  }
                  else if (fstatus.equals(StatusTyp.STOER_EAK))
                  {
                    figure.getRf().setBackgroundColor(ColorConstants.red);
                  }
                  else
                  {
                    figure.getRf().setBackgroundColor(ColorConstants.gray);
                  }
                }
              });
            }
          }
        }
      }
      else
      {
        PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
          @Override
          public void run()
          {
            figure.getRf().setBackgroundColor(ColorConstants.red);
          }
        });
      }
    }

  }

  protected void dataUpdateTabelleMessQuerschnitt(ResultData result)
  {
    final IFigure figure = getFigure();

    AttributeGroup atg = result.getDataDescription().getAttributeGroup();
    SystemObject so = result.getObject();

    if (atg.getPid().equals("atg.verkehrsDatenKurzZeitMq"))
    {
      Data data = result.getData();
      if (data != null)
      {
        if (getModel().getSystemObject().equals(result.getObject()))
        {
          final String qKfz = data.getItem("QKfz").getItem("Wert").asTextValue().getValueText();
          final String qPkw = data.getItem("QPkw").getItem("Wert").asTextValue().getValueText();
          final String qLkw = data.getItem("QLkw").getItem("Wert").asTextValue().getValueText();
          final String vKfz = data.getItem("VKfz").getItem("Wert").asTextValue().getValueText();
          final String vPkw = data.getItem("VPkw").getItem("Wert").asTextValue().getValueText();
          final String vLkw = data.getItem("VLkw").getItem("Wert").asTextValue().getValueText();

          PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
            @Override
            public void run()
            {
              List children = figure.getChildren();
              List<TabellenMqZeilenFigure> list1 = ((TabellenMqFigure)children.get(1)).getListCompartments();
              Label l1 = (Label)((TabellenMqZeilenFigure)list1.get(list1.size() - 1)).getChildren().get(0);
              l1.setText(qKfz);
              List<TabellenMqZeilenFigure> list2 = ((TabellenMqFigure)children.get(2)).getListCompartments();
              Label l2 = (Label)((TabellenMqZeilenFigure)list2.get(list2.size() - 1)).getChildren().get(0);
              l2.setText(qPkw);
              List<TabellenMqZeilenFigure> list3 = ((TabellenMqFigure)children.get(3)).getListCompartments();
              Label l3 = (Label)((TabellenMqZeilenFigure)list3.get(list3.size() - 1)).getChildren().get(0);
              l3.setText(qLkw);
              List<TabellenMqZeilenFigure> list4 = ((TabellenMqFigure)children.get(4)).getListCompartments();
              Label l4 = (Label)((TabellenMqZeilenFigure)list4.get(list4.size() - 1)).getChildren().get(0);
              l4.setText(vKfz);
              List<TabellenMqZeilenFigure> list5 = ((TabellenMqFigure)children.get(5)).getListCompartments();
              Label l5 = (Label)((TabellenMqZeilenFigure)list5.get(list5.size() - 1)).getChildren().get(0);
              l5.setText(vPkw);
              List<TabellenMqZeilenFigure> list6 = ((TabellenMqFigure)children.get(6)).getListCompartments();
              Label l6 = (Label)((TabellenMqZeilenFigure)list6.get(list6.size() - 1)).getChildren().get(0);
              l6.setText(vLkw);
            }
          });
        }
      }
      else
      {
        if (getModel().getSystemObject().equals(result.getObject()))
        {
          final String s = "keine Quelle / keine Daten";

          PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
            @Override
            public void run()
            {
              List children = figure.getChildren();
              List<TabellenMqZeilenFigure> list1 = ((TabellenMqFigure)children.get(1)).getListCompartments();
              Label l1 = (Label)((TabellenMqZeilenFigure)list1.get(list1.size() - 1)).getChildren().get(0);
              l1.setText(s);
              List<TabellenMqZeilenFigure> list2 = ((TabellenMqFigure)children.get(2)).getListCompartments();
              Label l2 = (Label)((TabellenMqZeilenFigure)list2.get(list2.size() - 1)).getChildren().get(0);
              l2.setText(s);
              List<TabellenMqZeilenFigure> list3 = ((TabellenMqFigure)children.get(3)).getListCompartments();
              Label l3 = (Label)((TabellenMqZeilenFigure)list3.get(list3.size() - 1)).getChildren().get(0);
              l3.setText(s);
              List<TabellenMqZeilenFigure> list4 = ((TabellenMqFigure)children.get(4)).getListCompartments();
              Label l4 = (Label)((TabellenMqZeilenFigure)list4.get(list4.size() - 1)).getChildren().get(0);
              l4.setText(s);
              List<TabellenMqZeilenFigure> list5 = ((TabellenMqFigure)children.get(5)).getListCompartments();
              Label l5 = (Label)((TabellenMqZeilenFigure)list5.get(list5.size() - 1)).getChildren().get(0);
              l5.setText(s);
              List<TabellenMqZeilenFigure> list6 = ((TabellenMqFigure)children.get(6)).getListCompartments();
              Label l6 = (Label)((TabellenMqZeilenFigure)list6.get(list6.size() - 1)).getChildren().get(0);
              l6.setText(s);
            }
          });
        }
      }
    }
    else if (atg.getPid().equals("atg.verkehrsDatenKurzZeitFs"))
    {
      ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
      AttributeGroup atg2 = connection.getDataModel().getAttributeGroup("atg.fahrStreifen");
      Data cd = so.getConfigurationData(atg2);

      DoModel model = getModel();
      SystemObject mq = model.getSystemObject();
      ConfigurationObject co = (ConfigurationObject)mq;
      NonMutableSet set = co.getNonMutableSet("FahrStreifen");
      List<SystemObject> fss = set.getElements();
      for (SystemObject fs : fss)
      {
        if (fs.equals(so))
        {
          final int pos = cd.getItem("Lage").asUnscaledValue().intValue();

          Data data = result.getData();

          if (data != null)
          {
            final String qKfz = data.getItem("qKfz").getItem("Wert").asTextValue().getValueText();
            final String qPkw = data.getItem("qPkw").getItem("Wert").asTextValue().getValueText();
            final String qLkw = data.getItem("qLkw").getItem("Wert").asTextValue().getValueText();
            final String vKfz = data.getItem("vKfz").getItem("Wert").asTextValue().getValueText();
            final String vPkw = data.getItem("vPkw").getItem("Wert").asTextValue().getValueText();
            final String vLkw = data.getItem("vLkw").getItem("Wert").asTextValue().getValueText();

            PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
              @Override
              public void run()
              {

                List children = figure.getChildren();
                List<TabellenMqZeilenFigure> list1 = ((TabellenMqFigure)children.get(1)).getListCompartments();
                if (pos < list1.size())
                {
                  Label l1 = (Label)((TabellenMqZeilenFigure)list1.get(pos)).getChildren().get(0);
                  l1.setText(qKfz);
                }
                List<TabellenMqZeilenFigure> list2 = ((TabellenMqFigure)children.get(2)).getListCompartments();
                if (pos < list2.size())
                {
                  Label l2 = (Label)((TabellenMqZeilenFigure)list2.get(pos)).getChildren().get(0);
                  l2.setText(qPkw);
                }
                List<TabellenMqZeilenFigure> list3 = ((TabellenMqFigure)children.get(3)).getListCompartments();
                if (pos < list3.size())
                {
                  Label l3 = (Label)((TabellenMqZeilenFigure)list3.get(pos)).getChildren().get(0);
                  l3.setText(qLkw);
                }
                List<TabellenMqZeilenFigure> list4 = ((TabellenMqFigure)children.get(4)).getListCompartments();
                if (pos < list4.size())
                {
                  Label l4 = (Label)((TabellenMqZeilenFigure)list4.get(pos)).getChildren().get(0);
                  l4.setText(vKfz);
                }
                List<TabellenMqZeilenFigure> list5 = ((TabellenMqFigure)children.get(5)).getListCompartments();
                if (pos < list5.size())
                {
                  Label l5 = (Label)((TabellenMqZeilenFigure)list5.get(pos)).getChildren().get(0);
                  l5.setText(vPkw);
                }
                List<TabellenMqZeilenFigure> list6 = ((TabellenMqFigure)children.get(6)).getListCompartments();
                if (pos < list6.size())
                {
                  Label l6 = (Label)((TabellenMqZeilenFigure)list6.get(pos)).getChildren().get(0);
                  l6.setText(vLkw);
                }
              }
            });

          }
          else
          {
            final String s = "keine Quelle / keine Daten";

            PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
              @Override
              public void run()
              {
                List children = figure.getChildren();
                List<TabellenMqZeilenFigure> list1 = ((TabellenMqFigure)children.get(1)).getListCompartments();
                if (pos < list1.size())
                {
                  Label l1 = (Label)((TabellenMqZeilenFigure)list1.get(pos)).getChildren().get(0);
                  l1.setText(s);
                }
                List<TabellenMqZeilenFigure> list2 = ((TabellenMqFigure)children.get(2)).getListCompartments();
                if (pos < list2.size())
                {
                  Label l2 = (Label)((TabellenMqZeilenFigure)list2.get(pos)).getChildren().get(0);
                  l2.setText(s);
                }
                List<TabellenMqZeilenFigure> list3 = ((TabellenMqFigure)children.get(3)).getListCompartments();
                if (pos < list3.size())
                {
                  Label l3 = (Label)((TabellenMqZeilenFigure)list3.get(pos)).getChildren().get(0);
                  l3.setText(s);
                }
                List<TabellenMqZeilenFigure> list4 = ((TabellenMqFigure)children.get(4)).getListCompartments();
                if (pos < list4.size())
                {
                  Label l4 = (Label)((TabellenMqZeilenFigure)list4.get(pos)).getChildren().get(0);
                  l4.setText(s);
                }
                List<TabellenMqZeilenFigure> list5 = ((TabellenMqFigure)children.get(5)).getListCompartments();
                if (pos < list5.size())
                {
                  Label l5 = (Label)((TabellenMqZeilenFigure)list5.get(pos)).getChildren().get(0);
                  l5.setText(s);
                }
                List<TabellenMqZeilenFigure> list6 = ((TabellenMqFigure)children.get(6)).getListCompartments();
                if (pos < list6.size())
                {
                  Label l6 = (Label)((TabellenMqZeilenFigure)list6.get(pos)).getChildren().get(0);
                  l6.setText(s);
                }
              }
            });
          }
        }
      }
    }
  }

  private void aktionToolTip(MouseEvent me)
  {
    if (getModel() != null)
    {
      DoTyp doTyp = getModel().getDoTyp();

      if (doTyp instanceof VerhaltenDoTyp)
      {
        VerhaltenDoTyp coDoTyp = (VerhaltenDoTyp)doTyp;

        int kontext = -1;

        if (getModel() instanceof AnzeigeQuerschnittModelTypAbstract)
        {
          AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)getModel();
          kontext = aq.getKontext();
        }
        else if (getModel() instanceof AnzeigeModelTypAbstract)
        {

          EditPart parent = getParent();

          if (parent instanceof DoMitVerbLinieEditPart)
          {
            DoMitVerbLinieEditPart ep = (DoMitVerbLinieEditPart)parent;
            DoModelMitVerbLinie model = ep.getModel();
            if (model instanceof AnzeigeQuerschnittModelTypAbstract)
            {
              AnzeigeQuerschnittModelTypAbstract aqModel = (AnzeigeQuerschnittModelTypAbstract)model;
              kontext = aqModel.getKontext();
            }
          }
        }
        else if (getModel() instanceof StreckenStationModel)
        {
          StreckenStationModel sm = (StreckenStationModel)getModel();
          kontext = sm.getKontext();
        }
        else
        {

        }

        if (getModel() instanceof AnzeigeModelUfdSensorFBZ)
        {

          AnzeigeModelUfdSensorFBZ sm = (AnzeigeModelUfdSensorFBZ)getModel();
          kontext = sm.getKontext();
        }
        else if (getModel() instanceof AnzeigeModelUfdSensorHK)
        {
          AnzeigeModelUfdSensorHK sm = (AnzeigeModelUfdSensorHK)getModel();
          kontext = sm.getKontext();
        }
        else if (getModel() instanceof AnzeigeModelUfdSensorSW)
        {
          AnzeigeModelUfdSensorSW sm = (AnzeigeModelUfdSensorSW)getModel();
          kontext = sm.getKontext();
        }
        else if (getModel() instanceof AnzeigeModelUfdSbaMessStelleNS)
        {
          AnzeigeModelUfdSbaMessStelleNS sm = (AnzeigeModelUfdSbaMessStelleNS)getModel();
          kontext = sm.getKontext();
        }
        else if (getModel() instanceof PickerVertical)
        {
          PickerVertical pv = (PickerVertical)getModel();
          kontext = 0;
        }

        String id = null;

        switch (kontext)
        {
          case 0:
            id = coDoTyp.getToolTipIdIstZustand();
            break;
          case 1:
            id = coDoTyp.getToolTipIdSollZustand();
            break;
          case 2:
            id = coDoTyp.getToolTipIdHandSchaltung();
            break;
          case 3:
            id = coDoTyp.getToolTipIdSimulation();
            break;
          case 4:
            id = coDoTyp.getToolTipIdTrichter();
            break;

          default:
            break;
        }
        if (id != null)
        {
          ExtensionPointHelper.leseActionsUndFuehreAusMitEvent(id, getModel(), me);
        }
      }
    }
  }

  protected ZusatzElementTyp ermittleProgrammStatusAusSswSbaSchaltbild(final ResultData result,
      final AnzeigeQuerschnittModelTypAbstract model)
  {
    return DaVHelper.ermittleProgrammStatusAq((GraphicalViewer)getViewer(), model, result);
  }

  protected void updateDarstellungAusSchaltbild(final ResultData result,
      final AnzeigeQuerschnittModelTypAbstract model, final AnzeigeQuerschnittFigure figure,
      final ZusatzElementTyp betriebsart)
  {
    final AnzeigeQuerschnittLinksFigure links = (AnzeigeQuerschnittLinksFigure)figure.getLinks();
    final AnzeigeQuerschnittRechtsFigure rechts = (AnzeigeQuerschnittRechtsFigure)figure.getRechts();

    if (result != null)
    {
      Data data = result.getData();
      if (data != null)
      {
        PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {

          @Override
          public void run()
          {
            int kontext = model.getKontext();
            // Nur im Ist-Zustand den ProgrammStatus des AQ updaten
            // if (kontext != 0)
            {
              String toolTip = "";

              if (betriebsart != null)
              {
                toolTip = umsetzungBetriebsZustand(betriebsart.getType());
              }

              if (rechts instanceof AnzeigeQuerschnittRechtsFigureA14)
              {
                AnzeigeQuerschnittRechtsFigureA14 rechtsA14 = (AnzeigeQuerschnittRechtsFigureA14)rechts;

                String path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
                    ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
                String path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                    ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
                if (betriebsart.equals(ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM))
                {
                  path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
                      ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM);
                  path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                      ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
                }
                else if (betriebsart.equals(ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM_BLIND))
                {
                  path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
                      ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM);
                  path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                      ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM_BLIND);
                }
                else if (betriebsart.equals(ZusatzElementTyp.PROGRAMM_SBA_SONDERPROGRAMM))
                {
                  path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
                      ZusatzElementTyp.PROGRAMM_SBA_SONDERPROGRAMM);
                  path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                      ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
                }
                else if (betriebsart.equals(ZusatzElementTyp.PROGRAMM_SBA_SONDERPROGRAMM_BLIND))
                {
                  path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
                      ZusatzElementTyp.PROGRAMM_SBA_SONDERPROGRAMM);
                  path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                      ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM_BLIND);
                }
                else if (betriebsart.equals(ZusatzElementTyp.PROGRAMM_SBA_HANDPROGRAMM))
                {
                  path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
                      ZusatzElementTyp.PROGRAMM_SBA_HANDPROGRAMM);
                  path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                      ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
                }
                else if (betriebsart.equals(ZusatzElementTyp.PROGRAMM_SBA_HANDPROGRAMM_BLIND))
                {
                  path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
                      ZusatzElementTyp.PROGRAMM_SBA_HANDPROGRAMM);
                  path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                      ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM_BLIND);
                }
                else
                {
                  path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM, betriebsart);
                  path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                      ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
                }

                Image image = ImageFactory.getImage(path);
                Image image2 = ImageFactory.getImage(path2);

                if (rechtsA14.getLabelProgramm() != null)
                {
                  rechtsA14.getLabelProgramm().setIcon(image);
                  rechtsA14.getLabelProgramm().setToolTip(new Label(toolTip));
                  rechtsA14.setZustandProgramm(betriebsart);
                }
                if (rechtsA14.getLabelProgrammBlind() != null)
                {
                  rechtsA14.getLabelProgrammBlind().setIcon(image2);
                  rechtsA14.getLabelProgrammBlind().setToolTip(new Label(toolTip));
                  rechtsA14.setZustandProgramm(betriebsart);
                }
              }
              else
              {
                String path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM, betriebsart);
                Image image = ImageFactory.getImage(path);
                if (rechts.getLabelProgramm() != null)
                {
                  rechts.getLabelProgramm().setIcon(image);
                  rechts.getLabelProgramm().setToolTip(new Label(toolTip));
                  rechts.setZustandProgramm(betriebsart);
                }
              }
            }
          }
        });
      }
    }
    else
    {
      PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {

        @Override
        public void run()
        {
          int kontext = model.getKontext();
          if (kontext == 0)
          {
            setFigureBorder(links, new LineBorder(ColorConstants.red, 2));
          }
        }
      });
      PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {

        @Override
        public void run()
        {
          String path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
              ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
          Image image = ImageFactory.getImage(path);
          if (rechts.getLabelProgramm() != null)
          {
            rechts.getLabelProgramm().setIcon(image);
            rechts.setZustandProgramm(ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
          }
        }
      });
    }
  }

  protected void updateDarstellungAusHandeingriff(final ResultData result,
      final AnzeigeQuerschnittModelTypAbstract model, final AnzeigeQuerschnittFigure figure,
      final ZusatzElementTyp betriebsart)
  {
    final AnzeigeQuerschnittLinksFigure links = (AnzeigeQuerschnittLinksFigure)figure.getLinks();
    final AnzeigeQuerschnittRechtsFigure rechts = (AnzeigeQuerschnittRechtsFigure)figure.getRechts();

    Data data = result.getData();
    if (data != null)
    {
      PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {

        @Override
        public void run()
        {
          int kontext = model.getKontext();
          // Nur im Ist-Zustand den ProgrammStatus des AQ updaten
          // if (kontext != 0)
          {
            String toolTip = "";

            if (betriebsart != null)
            {
              toolTip = umsetzungBetriebsZustand(betriebsart.getType());
            }

            if (rechts instanceof AnzeigeQuerschnittRechtsFigureA14)
            {
              String path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
                  ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
              String path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                  ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
              if (betriebsart.equals(ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM))
              {
                path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
                    ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM);
                path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                    ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
              }
              else if (betriebsart.equals(ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM_BLIND))
              {
                path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
                    ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM);
                path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                    ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM_BLIND);
              }
              else if (betriebsart.equals(ZusatzElementTyp.PROGRAMM_SBA_SONDERPROGRAMM))
              {
                path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
                    ZusatzElementTyp.PROGRAMM_SBA_SONDERPROGRAMM);
                path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                    ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
              }
              else if (betriebsart.equals(ZusatzElementTyp.PROGRAMM_SBA_SONDERPROGRAMM_BLIND))
              {
                path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
                    ZusatzElementTyp.PROGRAMM_SBA_SONDERPROGRAMM);
                path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                    ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM_BLIND);
              }
              else if (betriebsart.equals(ZusatzElementTyp.PROGRAMM_SBA_HANDPROGRAMM))
              {
                path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
                    ZusatzElementTyp.PROGRAMM_SBA_HANDPROGRAMM);
                path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                    ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
              }
              else if (betriebsart.equals(ZusatzElementTyp.PROGRAMM_SBA_HANDPROGRAMM_BLIND))
              {
                path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
                    ZusatzElementTyp.PROGRAMM_SBA_HANDPROGRAMM);
                path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                    ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM_BLIND);
              }
              else
              {
                path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM, betriebsart);
                path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                    ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
              }

              AnzeigeQuerschnittRechtsFigureA14 rechtsA14 = (AnzeigeQuerschnittRechtsFigureA14)rechts;
              Image image = ImageFactory.getImage(path);
              Image image2 = ImageFactory.getImage(path2);
              if (rechtsA14.getLabelProgramm() != null)
              {
                rechtsA14.getLabelProgramm().setIcon(image);
                rechtsA14.getLabelProgramm().setToolTip(new Label(toolTip));
              }
              if (rechtsA14.getLabelProgrammBlind() != null)
              {
                rechtsA14.getLabelProgrammBlind().setIcon(image2);
                rechtsA14.getLabelProgrammBlind().setToolTip(new Label(toolTip));
              }

            }
            else
            {
              String path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM, betriebsart);
              Image image = ImageFactory.getImage(path);
              if (rechts.getLabelProgramm() != null)
              {
                rechts.getLabelProgramm().setIcon(image);
                rechts.getLabelProgramm().setToolTip(new Label(toolTip));
              }
            }
          }
        }
      });
    }
    else
    {
      PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {

        @Override
        public void run()
        {
          int kontext = model.getKontext();
          if (kontext == 0)
          {
            setFigureBorder(links, new LineBorder(ColorConstants.red, 2));
          }
        }
      });
      PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {

        @Override
        public void run()
        {
          String path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
              ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
          Image image = ImageFactory.getImage(path);
          if (rechts.getLabelProgramm() != null)
          {
            rechts.getLabelProgramm().setIcon(image);
            rechts.setZustandProgramm(betriebsart);
          }
        }
      });
    }
  }

  protected void updateDarstellungAusHelligkeitProzent(final ResultData result,
      final AnzeigeQuerschnittModelTypAbstract model, final AnzeigeQuerschnittFigure figure, final Color farbe)
  {
    final AnzeigeQuerschnittLinksFigure links = (AnzeigeQuerschnittLinksFigure)figure.getLinks();
    final AnzeigeQuerschnittRechtsFigure rechts = (AnzeigeQuerschnittRechtsFigure)figure.getRechts();

    Data data = result.getData();
    if (data != null)
    {
      String sHell = data.getItem("Steuerung").getItem("Helligkeit").asTextValue().getValueText();
      String sStatus = data.getItem("Steuerung").getItem("Status").asTextValue().getValueText();

      String sToolTip = data.getItem("Steuerung").getItem("Helligkeit").asTextValue().getText() + " " + sStatus;

      try
      {
        Integer i = new Integer(sHell);
        if (sStatus.equals("Automatik"))
        {

        }
        else if (sStatus.equals("Handschaltung"))
        {

        }

      }
      catch (NumberFormatException e)
      {
        // TODO Auto-generated catch block
        if (sHell.equals("nicht ermittelbar"))
        {
          sHell = "lok";
          sToolTip = "lokale Steuerung";
        }
      }

      String slabelTmp = sHell;
      slabelTmp = slabelTmp.trim();

      if (slabelTmp.length() < 3)
      {
        slabelTmp += "%";
      }
      if (slabelTmp.contains(" "))
      {
        slabelTmp = slabelTmp.replace(" ", "");
      }
      final String slabel = slabelTmp;
      final String stooltip = "Helligkeit " + sToolTip;

      final int kontext = model.getKontext();
      // Nur im Soll-Zustand den ProgrammStatus des AQ updaten
      if (kontext == 1)
      {
        PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {

          @Override
          public void run()
          {
            if (rechts instanceof AnzeigeQuerschnittRechtsFigureA14)
            {
              AnzeigeQuerschnittRechtsFigureA14 rechtsA14 = (AnzeigeQuerschnittRechtsFigureA14)rechts;
              {
                if (rechtsA14.getLabelHelligkeit() != null)
                {
                  rechtsA14.getLabelHelligkeit().setText(slabel);
                  rechtsA14.getLabelHelligkeit().setToolTip(new Label(stooltip));

                  Color foreground = ColorConstants.black;
                  if (farbe.equals(ColorConstants.blue))
                    foreground = ColorConstants.white;
                  rechtsA14.getLabelHelligkeit().setForegroundColor(foreground);
                  rechtsA14.getLabelHelligkeit().setBackgroundColor(farbe);
                }
              }
            }
            else
            {
              if (rechts.getLabelHelligkeit() != null)
              {
                rechts.getLabelHelligkeit().setToolTip(new Label(stooltip));
                rechts.getLabelHelligkeit().setText(slabel);
                Color foreground = ColorConstants.black;
                if (farbe.equals(ColorConstants.blue))
                  foreground = ColorConstants.white;
                rechts.getLabelHelligkeit().setForegroundColor(foreground);
                rechts.getLabelHelligkeit().setBackgroundColor(farbe);
              }
            }
          }
        });
      }
    }
    else
    {
      PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {

        @Override
        public void run()
        {
          int kontext = model.getKontext();
          // if (kontext == 1)
          // links.setBorder(new LineBorder(ColorConstants.red, 2));
        }
      });
    }
  }

  protected ZusatzElementTyp ermittleProgrammStatusAusRueckmeldung(final ResultData result1)
  {
    // System.out.println("DoMitVerbLinieEditPart.ermittleProgrammStatusAusRueckmeldung()");

    ZusatzElementTyp betriebsart = ZusatzElementTyp.PROGRAMM_UNDEFINIERT;

    Data data1 = result1.getData();
    if (data1 != null)
    {
      final String s = data1.getItem("Eigenschaft").getItem("Betriebszustand").asTextValue().getValueText();
      betriebsart = ZusatzElementTyp.setTyp(s);

    }
    return betriebsart;
  }

  protected ZusatzElementTyp ermittleProgrammStatusAusHandeingriff(final ResultData result)
  {
    ZusatzElementTyp betriebsart = ZusatzElementTyp.PROGRAMM_UNDEFINIERT;

    Data data = result.getData();
    if (data != null)
    {
      final String s = data.getItem("Betriebsart").asTextValue().getValueText();

      if (s.equals("Normalbetrieb") || s.equals("AQNormalbetrieb"))
      {
        betriebsart = ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM;
      }
      else if (s.equals("Blindbetrieb") || s.equals("AQBlindbetrieb"))
      {
        betriebsart = ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM_BLIND;
      }
      else if (s.equals("Autarkbetrieb") || s.equals("AQAutarkbetrieb"))
      {
        betriebsart = ZusatzElementTyp.PROGRAMM_SBA_AUTARKBETRIEB;
      }

    }
    return betriebsart;
  }

  protected Color ermittleFarbeAusHelligkeitProzent(final ResultData result)
  {
    Color color = ColorConstants.black;

    Data data = result.getData();
    if (data != null)
    {
      final String sHell = data.getItem("Steuerung").getItem("Helligkeit").asTextValue().getValueText();
      final String sStatus = data.getItem("Steuerung").getItem("Status").asTextValue().getValueText();

      try
      {
        Integer i = new Integer(sHell);
        if (sStatus.equals("Automatik"))
        {
          color = ColorConstants.yellow;
        }
        else if (sStatus.equals("Handschaltung"))
        {
          color = ColorConstants.blue;
        }

      }
      catch (NumberFormatException e)
      {
        // TODO Auto-generated catch block
        if (sHell.equals("nicht ermittelbar"))
        {
          color = ColorConstants.white;
        }
      }

    }
    return color;
  }

  protected void updateDarstellungAusRueckmeldung(final ResultData result,
      final AnzeigeQuerschnittModelTypAbstract model, final AnzeigeQuerschnittFigure figure,
      final ZusatzElementTyp betriebsart)
  {
    final AnzeigeQuerschnittLinksFigure links = (AnzeigeQuerschnittLinksFigure)figure.getLinks();
    final AnzeigeQuerschnittRechtsFigure rechts = (AnzeigeQuerschnittRechtsFigure)figure.getRechts();

    Data data = result.getData();
    if (data != null)
    {
      final String status = data.getItem("Status").asTextValue().getValueText();

      String slabelTmp = data.getItem("Eigenschaft").getItem("Helligkeit").asTextValue().getValueText();
      slabelTmp = slabelTmp.trim();

      if (slabelTmp.length() < 3)
      {
        slabelTmp += "%";
      }
      if (slabelTmp.contains(" "))
      {
        slabelTmp = slabelTmp.replace(" ", "");
      }
      final String slabel = slabelTmp;
      final String stooltip = "Helligkeit " + data.getItem("Eigenschaft").getItem("Helligkeit").asTextValue().getText();

      final int kontext = model.getKontext();

      if (kontext < 2)
      {

        final String toolTip = umsetzungBetriebsZustand(betriebsart.getType());

        PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {

          @Override
          public void run()
          {
            int kontext = model.getKontext();
            if (kontext == 0)
            {
              if (status.equals("OK"))
              {
                setFigureBorder(links, new LineBorder(ColorConstants.green, 2));
              }
              else if (status.equals("Teilstrung"))
              {
                setFigureBorder(links, new LineBorder(ColorConstants.orange, 2));
              }
              else
              {
                setFigureBorder(links, new LineBorder(ColorConstants.red, 2));
              }
              figure.repaint();
            }
          }
        });

        PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {

          @Override
          public void run()
          {
            if (rechts instanceof AnzeigeQuerschnittRechtsFigureA14)
            {
              String path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
                  ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
              String path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                  ZusatzElementTyp.PROGRAMM_UNDEFINIERT);

              if (betriebsart.equals(ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM))
              {
                path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
                    ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM);
                path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                    ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
              }
              else if (betriebsart.equals(ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM_BLIND))
              {
                path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
                    ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM);
                path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                    ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM_BLIND);
              }
              else if (betriebsart.equals(ZusatzElementTyp.PROGRAMM_SBA_SONDERPROGRAMM))
              {
                path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
                    ZusatzElementTyp.PROGRAMM_SBA_SONDERPROGRAMM);
                path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                    ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
              }
              else if (betriebsart.equals(ZusatzElementTyp.PROGRAMM_SBA_SONDERPROGRAMM_BLIND))
              {
                path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
                    ZusatzElementTyp.PROGRAMM_SBA_SONDERPROGRAMM);
                path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                    ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM_BLIND);
              }
              else if (betriebsart.equals(ZusatzElementTyp.PROGRAMM_SBA_HANDPROGRAMM))
              {
                path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
                    ZusatzElementTyp.PROGRAMM_SBA_HANDPROGRAMM);
                path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                    ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
              }
              else if (betriebsart.equals(ZusatzElementTyp.PROGRAMM_SBA_HANDPROGRAMM_BLIND))
              {
                path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
                    ZusatzElementTyp.PROGRAMM_SBA_HANDPROGRAMM);
                path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                    ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM_BLIND);
              }
              else
              {
                path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM, betriebsart);
                path2 = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM_A14,
                    ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
              }

              AnzeigeQuerschnittRechtsFigureA14 rechtsA14 = (AnzeigeQuerschnittRechtsFigureA14)rechts;
              Image image = ImageFactory.getImage(path);
              Image image2 = ImageFactory.getImage(path2);

              if (rechtsA14.getLabelHelligkeit() != null)
              {
                rechtsA14.getLabelHelligkeit().setForegroundColor(ColorConstants.black);
                rechtsA14.getLabelHelligkeit().setText(slabel);
                rechtsA14.getLabelHelligkeit().setToolTip(new Label(stooltip));
              }

              if (kontext == 0)
              {
                if (rechtsA14.getLabelProgramm() != null)
                {
                  rechtsA14.getLabelProgramm().setIcon(image);
                  rechtsA14.getLabelProgramm().setToolTip(new Label(toolTip));
                  rechtsA14.setZustandProgramm(betriebsart);
                }
                if (rechtsA14.getLabelProgrammBlind() != null)
                {
                  rechtsA14.getLabelProgrammBlind().setIcon(image2);
                  rechtsA14.getLabelProgrammBlind().setToolTip(new Label(toolTip));
                  rechtsA14.setZustandProgramm(betriebsart);
                }
              }
            }
            else
            {

              String path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM, betriebsart);
              Image image = ImageFactory.getImage(path);
              if (rechts.getLabelHelligkeit() != null)
              {
                rechts.getLabelHelligkeit().setForegroundColor(ColorConstants.black);
                rechts.getLabelHelligkeit().setText(slabel);
                rechts.getLabelHelligkeit().setToolTip(new Label(stooltip));
                rechts.setZustandProgramm(betriebsart);
              }
              if (kontext == 0)
              {
                if (rechts.getLabelProgramm() != null)
                {
                  rechts.getLabelProgramm().setIcon(image);
                  rechts.getLabelProgramm().setToolTip(new Label(toolTip));
                  rechts.setZustandProgramm(betriebsart);
                }
              }
            }
            figure.repaint();
          }
        });
      }
    }
    else
    {
      PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {

        @Override
        public void run()
        {
          int kontext = model.getKontext();
          if (kontext == 0)
          {
            /*
             * Hier wurde eine Sonderprfung eingelegt: Nur wenn das Flag isBorderChangeable in der Figure gesetzt ist,
             * ndern wir die Randfarbe.
             */
            setFigureBorder(links, new LineBorder(ColorConstants.red, 2));
          }
        }
      });
      PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {

        @Override
        public void run()
        {
          String path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.PROGRAMM,
              ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
          Image image = ImageFactory.getImage(path);
          if (rechts.getLabelProgramm() != null)
          {
            rechts.getLabelProgramm().setIcon(image);
            rechts.setZustandProgramm(ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
          }
          if (rechts.getLabelHelligkeit() != null)
          {
            rechts.getLabelHelligkeit().setText("---");
          }
        }
      });
    }
  } 
  
  
  /**
   * Mit diesem Flag kann geprft werden, ob wir am {@link Taktgeber} angemeldet
   * sind oder nicht.
   */
  private boolean _angemeldetAmTaktgeber = false;

  /**
   * Fhrt die Anmeldung am Taktgeber durch, sofern wir dort noch nicht
   * angemeldet sind.
   */
  private void anmeldenTaktgeber()
  {
    synchronized (this)
    {

      if (!_angemeldetAmTaktgeber)
      {
        Taktgeber.getInstance().addTaktListener(this);
        _angemeldetAmTaktgeber = true;

        // System.out.println("-> Am Taktgeber angemeldet fr "
        // + getModel().getSystemObject());
      }
    }
  }

  /**
   * Fhrt die Abmeldung vom Taktgeber durch, sofern wir dort noch nicht
   * angemeldet sind.
   */
  public void abmeldenTaktgeber()
  {
    synchronized (this)
    {

      if (_angemeldetAmTaktgeber)
      {
        Taktgeber.getInstance().removeTaktListener(this);
        _angemeldetAmTaktgeber = false;

        // System.out.println("-> Vom Taktgeber ABGEMELDET fr "
        // + getModel().getSystemObject());
        
        
        final AnzeigeElementFigure figure = (AnzeigeElementFigure)getFigure();
        
        final Image image = ImageFactory.getImage(_pathBaseImage);
                
        PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
          @Override
          public void run()
          {         
            if (image != null && !image.isDisposed())
            {
              figure.setImage(image);
            }            
          }
        });        
      }
    }
  }

  @Override
  public void takt(final boolean taktflanke)
  {
    String imgPath = _pathBaseImage;

    if ((imgPath != null) && (imgPath.length() > 0))
    {
      if (taktflanke)
      {
        imgPath = imgPath + ";blinken=true";
      }

      final AnzeigeElementFigure figure = (AnzeigeElementFigure) getFigure();
      final String fImgPath = imgPath;
      final Image fImage = ImageFactory.getImage(fImgPath);

      PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable()
      {
        @Override
        public void run()
        {
          if (fImage != null)
          {
            if (!fImage.isDisposed())
            {
              figure.setImage(fImage);
              figure.repaint();
            } else
            {
              System.err
                  .println("DoMitVerbiLinieEditPart::takt > Image is disposed fr Pfad => "
                      + fImgPath);
            }
          } else
          {
            System.err
                .println("DoMitVerbiLinieEditPart::takt > Image ist null fr Pfad => "
                    + fImgPath);
          }
        }
      });
    } else
    {
      // System.err.println("=> Kein Imagepath definiert fr "
      // + getModel().getSystemObject());
    }
  }
  
}
