/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.section;

import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.DetektorModelTypAbstract;
import de.bsvrz.buv.plugin.dobj.properties.AbstractSection;

/**
 * Sektion zur Darstellung eines DetektorLve in der Properties View.
 * 
 * @author Pittner
 * @param <T>
 */
public class DetetktorAbstractExtraSection extends AbstractSection<DetektorModelTypAbstract>
{

  private Button _btn;

  private CCombo kontext;

  @Override
  public void createControls(final Composite parent, final TabbedPropertySheetPage aTabbedPropertySheetPage)
  {
    super.createControls(parent, aTabbedPropertySheetPage);

    final TabbedPropertySheetWidgetFactory factory = getWidgetFactory();
    final Composite container = factory.createFlatFormComposite(parent);

    FormData data;

    _btn = factory.createButton(container, "", SWT.CHECK);
    data = new FormData();
    data.left = new FormAttachment(0, STANDARD_LABEL_WIDTH);
    data.right = new FormAttachment(100, 0);
    data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
    _btn.setLayoutData(data);

    final CLabel label = factory.createCLabel(container, "Gerunded: ");
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.right = new FormAttachment(_btn, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(_btn, 0, SWT.CENTER);
    label.setLayoutData(data);

    _btn.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        // TODO Auto-generated method stub
        EStructuralFeature feature = AnlagendarstellungPackage.Literals.DETEKTOR_MODEL_TYP_ABSTRACT__GERUNDED;
        boolean sel = _btn.getSelection();

        getElement().setGerunded(sel);

        final SetCommand cmd = new SetCommand(getElement(), feature, sel);
        getCommandStack().execute(cmd);

        _btn.setSelection(sel);
      }

    });

    kontext = factory.createCCombo(container);
    kontext.add("Zustandsdarstellung", 0);
    kontext.add("Automatikprogramm", 1);
    kontext.add("Handschaltung", 2);
    kontext.add("Sonderprogramm", 3);
    data = new FormData();
    data.left = new FormAttachment(0, STANDARD_LABEL_WIDTH);
    data.right = new FormAttachment(100, 0);
    data.top = new FormAttachment(_btn, ITabbedPropertyConstants.VSPACE);
    kontext.setLayoutData(data);

    final CLabel label3 = factory.createCLabel(container, "Kontext:");
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.right = new FormAttachment(kontext, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(kontext, 0, SWT.CENTER);
    label3.setLayoutData(data);

    kontext.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        // TODO Auto-generated method stub
        if (!kontext.getSelection().equals(getElement().getKontext()))
        {
          // XXX: Der Text wurde in der PropertyPage gendert, also
          // versenden wir ein neues Command an das Modell-Objekt
          EStructuralFeature feature = AnlagendarstellungPackage.Literals.AQ_MODEL_BEZEICHNUNG__KONTEXT;

          int i = kontext.getSelectionIndex();
          getElement().setKontext(i);

          final SetCommand cmd = new SetCommand(getElement(), feature, i);
          getCommandStack().execute(cmd);
        }

      }

    });
  }

  @Override
  public void refresh()
  {
    super.refresh();

    boolean bool = getElement().isGerunded();
    _btn.setSelection(bool);

    int i = getElement().getKontext();
    kontext.select(i);
  }
}
