/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.einstellungen;

import java.util.HashMap;

import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.jface.preference.IPreferenceStore;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.DefsPreference;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;

/**
 * Die Klasse kapselt die Verwaltungsmethoden fr die Listener-Funktionalitt des IDaVDataUpdateListener. Diese Klasse
 * implemntiert das Singleton-Pattern und stellt dadurch sicher dass der Listener nur einmal Instanziiert wird.
 * 
 * 
 * @author Pittner
 * 
 */
public class SbaAnsichtProvider
{

  /**
   * Die interne Map fr die Ansicht aus den Einstellungen
   */
  private HashMap<String, Ansicht> map = new HashMap<String, Ansicht>();
  
  /**
   * Singleton-Instanz
   */
  private static SbaAnsichtProvider _instanz = null;

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static SbaAnsichtProvider getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new SbaAnsichtProvider();
    }
    return _instanz;
  }
    
  public Ansicht getAnsichtKopie(String bezeichnungAnsicht){    
    if (bezeichnungAnsicht == null)
      throw new IllegalArgumentException("Name der Ansicht ist null!");
    
    Ansicht re = null;
    if (!map.containsKey(bezeichnungAnsicht)){
      IPreferenceStore store = Activator.getDefault().getPreferenceStore();
      String name = store.getString(bezeichnungAnsicht);
      if (name != null && name.length() > 0){        
        Ansicht ansicht = SbaEinstellungen.INSTANCE.getAnsicht(name);
        if (ansicht != null)
          map.put(bezeichnungAnsicht, ansicht);              
      }
    }

    if (map.get(bezeichnungAnsicht) != null)
      re = (Ansicht)EcoreUtil.copy(map.get(bezeichnungAnsicht));
    
    return re; 
  }
}
