/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.figures;

import org.eclipse.draw2d.AbstractBorder;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.swt.graphics.Color;

import de.bsvrz.buv.plugin.anlagendarstellung.model.AnschlussStelleTyp;

/**
 * Figure, welche eine Anschlussstelle reprsentiert.               
 * 
 * @author Pittner
 * 
 */
public class AnschlussStelleFigure extends ColoredPolygon
{

  private PointList _pl;

  public PointList getPointList()
  {
    return _pl;
  }

  public void setPointList(PointList pl)
  {
    _pl = pl;
  }

  public AnschlussStelleFigure(String name, Point pos, Dimension size, int strength)
  {
    super(ColorConstants.gray);

    setOpaque(false);
    setBorder(new AnschlussStelleFigureBorder(ColorConstants.gray));

    createPointList(AnschlussStelleTyp.ZUFAHRT_LINKS, pos.x, pos.y, size.width, size.height, strength);

  }

  public void createPointList(AnschlussStelleTyp typ, Point pos, Dimension size, int strength)
  {
    createPointList(typ, pos.x, pos.y, size.width, size.height, strength);
  }

  public void createPointList(AnschlussStelleTyp typ, int x, int y, int width, int height, int strentgh)
  {

    int[] pts = null;

    if (typ != null)
    {
      if (typ.equals(AnschlussStelleTyp.ZUFAHRT_LINKS))
      {
        pts = new int[] { width, 0, width, height, 0, height, 0, height - strentgh, width - strentgh,
            height - strentgh, width - strentgh, strentgh, width, 0 };
      }
      else if (typ.equals(AnschlussStelleTyp.ZUFAHRT_RECHTS))
      {
        pts = new int[] { 0, 0, strentgh, strentgh, strentgh, height - strentgh, width, height - strentgh, width,
            height, 0, height, 0, 0 };
      }
      else if (typ.equals(AnschlussStelleTyp.ABFAHRT_LINKS))
      {
        pts = new int[] { 0, 0, width, 0, width, height, width - strentgh, height - strentgh, width - strentgh,
            strentgh, 0, strentgh, 0, 0 };
      }
      else if (typ.equals(AnschlussStelleTyp.ABFAHRT_RECHTS))
      {
        pts = new int[] { 0, height, strentgh, height - strentgh, strentgh, strentgh, width, strentgh, width, 0, 0, 0 };
      }
      else if (typ.equals(AnschlussStelleTyp.UNDEFINIERT))
      {
      }
    }

    _pl = null;
    if (pts != null)
    {
      _pl = new PointList(pts.length);
      for (int i = 0; i < pts.length - 1; i += 2)
      {
        int ptx = pts[i];
        int pty = pts[i + 1];
        _pl.addPoint(x + ptx, y + pty);
      }
    }

    if (_pl != null)
      setPoints(_pl);
  }

  public void createPointList(PointList pointList, Point pos)
  {

    if (pointList != null)
    {
      _pl = new PointList(pointList.size());
      for (int i = 0; i < pointList.size() - 1; i += 2)
      {
        int ptx = pointList.getPoint(i).x;
        int pty = pointList.getPoint(i).y;
        _pl.addPoint(pos.x + ptx, pos.y + pty);
      }
    }

    if (_pl != null)
      setPoints(_pl);
  }

  public class AnschlussStelleFigureBorder extends AbstractBorder
  {

    private Color color;

    private int width;

    public Color getColor()
    {
      return color;
    }

    public void setColor(Color color)
    {
      this.color = color;
    }

    public int getWidth()
    {
      return width;
    }

    public void setWidth(int width)
    {
      this.width = width;
    }

    public AnschlussStelleFigureBorder(Color c, int w)
    {
      color = c;
      width = w;

    }

    public AnschlussStelleFigureBorder(Color c)
    {
      color = c;
      width = 1;
    }

    public Insets getInsets(IFigure figure)
    {
      return new Insets(0, 0, 0, 0);
    }

    public void paint(IFigure figure, Graphics graphics, Insets insets)
    {

      graphics.setForegroundColor(color);
      graphics.setLineWidth(width);
      if (_pl != null)
        graphics.drawPolygon(_pl);      
    }

  }
}
