/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.figures;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.Polyline;
import org.eclipse.draw2d.geometry.Geometry;
import org.eclipse.swt.graphics.Color;

/**
 * Figure zum  Zeichnen farbiger Pologone.               
 * 
 * @author Pittner
 * 
 */
public class ColoredPolygon extends Polyline
{
  private Color _color = ColorConstants.gray;

  public ColoredPolygon()
  {
  }

  public ColoredPolygon(Color color)
  {
    super();
    _color = color;
  }

  /**
   * Returns whether the point (x,y) is contained inside this polygon.
   * 
   * @param x
   *          the X coordinate
   * @param y
   *          the Y coordinate
   * @return whether the point (x,y) is contained in this polygon
   */
  public boolean containsPoint(int x, int y)
  {
    if (!getBounds().contains(x, y))
      return false;
    return shapeContainsPoint(x, y) || childrenContainsPoint(x, y);
  }

  protected boolean shapeContainsPoint(int x, int y)
  {    
    return Geometry.polygonContainsPoint(getPoints(), x, y);
  }

  /**
   * Fill the Polygon with the background color set by <i>g</i>.
   * 
   * @param g
   *          the Graphics object
   * @since 2.0
   */
  protected void fillShape(Graphics g)
  {    
    g.setForegroundColor(_color);
    g.setBackgroundColor(_color);
    g.fillPolygon(getPoints());
  }

  /**
   * Draw the outline of the Polygon.
   * 
   * @param g
   *          the Graphics object
   * @since 2.0
   */
  protected void outlineShape(Graphics g)
  {    
    g.drawPolygon(getPoints());
    fillShape(g);
  }

  @Override
  public void setBackgroundColor(Color bg)
  {    
    super.setBackgroundColor(bg);
  }

  public void setFillColor(Color color)
  {
    _color = color;
    repaint();
  }

  public Color getFillColor()
  {
    return _color;
  }
}
