/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.figures;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.swt.graphics.Image;

import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.model.ZusatzElementTyp;

/**
 * Figure, welche einen Ufd-Sensor als Band reprsentiert.               
 * 
 * @author Pittner
 * 
 */
public class UmfeldDatenBandFigure extends Figure
{

  private String _pid = null;

  private ZusatzElementTyp _typ = null;

  private ZusatzElementTyp _inhalt = null;

  public ZusatzElementTyp getTyp()
  {
    return _typ;
  }

  public void setTyp(ZusatzElementTyp typ)
  {
    _typ = typ;
  }

  public ZusatzElementTyp getInhalt()
  {
    return _inhalt;
  }

  private Label _imageLabel = null;

  public UmfeldDatenBandFigure(String pid, ZusatzElementTyp typ, ZusatzElementTyp inhalt)
  {

    _pid = pid;
    _typ = typ;
    _inhalt = inhalt;

    setOpaque(false);
    setLayoutManager(new ToolbarLayout());

    _imageLabel = new Label();

    setImage(inhalt);

    add(_imageLabel);

  }

  public void setImage(ZusatzElementTyp inhalt)
  {
    String path = null;
    if (_typ.equals(ZusatzElementTyp.NIEDERSCHLAG))
    {
      if (inhalt.equals(ZusatzElementTyp.NIEDERSCHLAG_TROCKEN))
      {
        path = "icons/nass_0.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.NIEDERSCHLAG_NASS_1))
      {
        path = "icons/nass_1.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.NIEDERSCHLAG_NASS_2))
      {
        path = "icons/nass_2.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.NIEDERSCHLAG_NASS_3))
      {
        path = "icons/nass_3.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.NIEDERSCHLAG_NASS_4))
      {
        path = "icons/nass_4.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.NIEDERSCHLAG_NASS_5))
      {
        path = "icons/nass_5.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.NIEDERSCHLAG_NICHT_ERMITTELBAR))
      {
        path = "icons/nass_ermittelbar.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.NIEDERSCHLAG_GESTOERT))
      {
        path = "icons/nass_stoer.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.NIEDERSCHLAG_UNDEFINIERT))
      {
        path = "icons/nass_stoer.png";
      }
      else
      {
        path = "icons/nass_stoer.png";
      }
    }
    else if (_typ.equals(ZusatzElementTyp.SICHTWEITE))
    {
      if (inhalt.equals(ZusatzElementTyp.SICHTWEITE_STUFE_1))
      {
        path = "icons/sicht_0.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.SICHTWEITE_STUFE_2))
      {
        path = "icons/sicht_1.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.SICHTWEITE_STUFE_3))
      {
        path = "icons/sicht_2.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.SICHTWEITE_STUFE_4))
      {
        path = "icons/sicht_3.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.SICHTWEITE_STUFE_5))
      {
        path = "icons/sicht_4.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.SICHTWEITE_STUFE_6))
      {
        path = "icons/sicht_5.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.SICHTWEITE_GESTOERT))
      {
        path = "icons/sicht_stoer.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.SICHTWEITE_UNDEFINIERT))
      {
        path = "icons/sicht_stoer.png";
      }
      else
      {
        path = "icons/sicht_stoer.png";
      }
    }
    else if (_typ.equals(ZusatzElementTyp.HELLIGKEIT))
    {
      if (inhalt.equals(ZusatzElementTyp.HELLIGKEIT_STUFE_1))
      {
        path = "icons/helligkeit_1.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.HELLIGKEIT_STUFE_2))
      {
        path = "icons/helligkeit_2.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.HELLIGKEIT_STUFE_3))
      {
        path = "icons/helligkeit_3.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.HELLIGKEIT_STUFE_4))
      {
        path = "icons/helligkeit_4.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.HELLIGKEIT_STUFE_5))
      {
        path = "icons/helligkeit_5.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.HELLIGKEIT_STUFE_6))
      {
        path = "icons/helligkeit_6.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.HELLIGKEIT_STUFE_7))
      {
        path = "icons/helligkeit_7.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.HELLIGKEIT_STUFE_8))
      {
        path = "icons/helligkeit_8.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.HELLIGKEIT_STUFE_9))
      {
        path = "icons/helligkeit_9.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.HELLIGKEIT_STUFE_10))
      {
        path = "icons/helligkeit_10.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.HELLIGKEIT_GESTOERT))
      {
        path = "icons/helligkeit_stoer.png";
      }
      else if (inhalt.equals(ZusatzElementTyp.HELLIGKEIT_UNDEFINIERT))
      {
        path = "icons/helligkeit_stoer.png";
      }
      else
      {
        path = "icons/helligkeit_stoer.png";
      }
    }
    if (path != null)
    {
      Image image1 = ImageFactory.getImage(path);
      _imageLabel.setIcon(image1);
    }

  }

  public Label getImageLabel()
  {
    return _imageLabel;
  }

  public void setImageLabel(Label imageLabel)
  {
    _imageLabel = imageLabel;
  }

  public String getPid()
  {
    return _pid;
  }

  public void setInhalt(ZusatzElementTyp inhalt)
  {
    _inhalt = inhalt;
  }

  public void setPid(String pid)
  {
    _pid = pid;
  }

}
