/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.menu;

import org.eclipse.jface.action.IAction;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.action.ActionSonderFrNO;
import de.bsvrz.buv.plugin.anlagendarstellung.action.ActionSonderFrSW;
import de.bsvrz.buv.plugin.anlagendarstellung.action.ActionZustandIst;
import de.bsvrz.buv.plugin.anlagendarstellung.action.ActionZustandSoll;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.DefsPreference;
import de.bsvrz.buv.rw.basislib.menu.generatoren.IMenueLadeListener;
import de.bsvrz.buv.rw.basislib.menu.generatoren.MenueLadeInformation;
import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorAktion;

/**
 * Zentrale Verwaltungsklasse zum Anpassen der Rahmenwerk-Menueleiste
 * 
 * @author Pittner
 * 
 */
public class MenuModifier implements IMenueLadeListener
{

  /**
   * Singleton-Instanz
   */
  private static MenuModifier _instanz = null;

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static MenuModifier getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new MenuModifier();
    }
    return _instanz;
  }

  /**
   * Konstruktor.
   * 
   */
  private MenuModifier()
  {
    // System.out.println("MenuModifier.MenuModifier()");
  }

  public void anmeldenAufMenuLadeListener()
  {
    MenueLadeInformation.getInstance().addListener(this);
  }

  public void abmeldenAufMenuLadeListener()
  {
    MenueLadeInformation.getInstance().removeListener(this);
  }

  @Override
  public void menueLeisteGeladen()
  {
    aktionModifiziereMenueLeiste();
  }

  @Override
  public void symbolLeisteGeladen()
  {
    // tue nichts
  }

  /**
   * Modifiziert die Rahmenwerk-Menleiste fr die Menpunkte IST- und Soll-Zustand sowie die Menpunkte fr die beiden
   * Sonderprogramm-Fahrtrichtungen
   */
  private void aktionModifiziereMenueLeiste()
  {
    final String menuTextZustandIst = Activator.getDefault().getPreferenceStore()
        .getString(DefsPreference.P_MENUE_ZUSTAND_IST);
    if (menuTextZustandIst != null && menuTextZustandIst.length() > 0)
    {
      DeskriptorAktion deskriptorZustandIst = ElementRegistrierung.getInstanz()
          .getDeskriptorAktion(ActionZustandIst.ID);
      if (deskriptorZustandIst != null)
      {
        final IAction action = deskriptorZustandIst.getAktion();
        if (action != null)
        {
          action.setText(menuTextZustandIst);
        }
      }
    }

    final String menuTextZustandSoll = Activator.getDefault().getPreferenceStore()
        .getString(DefsPreference.P_MENUE_ZUSTAND_SOLL);
    if (menuTextZustandSoll != null && menuTextZustandSoll.length() > 0)
    {
      DeskriptorAktion deskriptorZustandSoll = ElementRegistrierung.getInstanz().getDeskriptorAktion(
          ActionZustandSoll.ID);
      if (deskriptorZustandSoll != null)
      {
        final IAction action = deskriptorZustandSoll.getAktion();
        if (action != null)
        {
          action.setText(menuTextZustandSoll);
        }
      }
    }

    final String menuTextSonderProgrammNo = Activator.getDefault().getPreferenceStore()
        .getString(DefsPreference.P_MENUE_SONDER_PROGRAMM_NO);
    if (menuTextSonderProgrammNo != null && menuTextSonderProgrammNo.length() > 0)
    {
      DeskriptorAktion deskriptorSonderProgrammNo = ElementRegistrierung.getInstanz().getDeskriptorAktion(
          ActionSonderFrNO.ID);
      if (deskriptorSonderProgrammNo != null)
      {
        final IAction action = deskriptorSonderProgrammNo.getAktion();
        if (action != null)
        {
          action.setText(menuTextSonderProgrammNo);
        }
      }
    }

    final String menuTextSonderProgrammSw = Activator.getDefault().getPreferenceStore()
        .getString(DefsPreference.P_MENUE_SONDER_PROGRAMM_SW);
    if (menuTextSonderProgrammSw != null && menuTextSonderProgrammSw.length() > 0)
    {
      DeskriptorAktion deskriptorSonderProgrammSw = ElementRegistrierung.getInstanz().getDeskriptorAktion(
          ActionSonderFrSW.ID);
      if (deskriptorSonderProgrammSw != null)
      {
        final IAction action = deskriptorSonderProgrammSw.getAktion();
        if (action != null)
        {
          action.setText(menuTextSonderProgrammSw);
        }
      }
    }
  }
}
