/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.model;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * Model Object of a AnzeigeQuerschnitt, wird aus Kompatibilittsgrunden bentigt
 * 
 * @deprecated  
 * @author Pittner
 * 
 */
public class AnzeigeQuerschnitt extends PropertyAwareObject implements Comparable<AnzeigeQuerschnitt>
{

  private IFigure _figure;

  private String _pid;

  private String _name;

  private String _uhrzeit;

  private Long _kilometrierung;

  private List _anzeigeElemente = new ArrayList();

  private List _zusatzElemente = new ArrayList();

  private List _zusatzElemente2 = new ArrayList();

  private Rectangle _bounds;

  private List _sourceConnections = new ArrayList();

  private List _targetConnections = new ArrayList();

  private StatusTyp _status;

  private AnzeigeQuerschnittTyp _typ;

  private FahrstreifenTyp _fsTyp;

  private boolean _typHandEingriff = false;

  private boolean _handEingriff = false;

  private boolean _handInitial = false;

  private int _x = 0;

  private int _y = 0;

  private boolean _aktualisieren = true;

  private String _viewId;

  public AnzeigeQuerschnitt()
  {
    super();
  }

  public AnzeigeQuerschnitt(AnzeigeQuerschnittTyp typ, FahrstreifenTyp fsTyp, String pid, String name, String zeit,
      List anzeigen, List zusatz, int x, int y, String viewId)
  {

    this._typ = typ;
    this._fsTyp = fsTyp;
    this._pid = pid;
    this._name = name;
    this._uhrzeit = zeit;
    this._anzeigeElemente = anzeigen;
    this._zusatzElemente = zusatz;
    this._x = x;
    this._y = y;
    this._viewId = viewId;

    _status = StatusTyp.UNDEFINERT;
  }

  public AnzeigeQuerschnitt(AnzeigeQuerschnittTyp typ, FahrstreifenTyp fsTyp, String pid, String name, String zeit,
      List anzeigen, List zusatz, int x, int y, boolean aktualisieren, String viewId)
  {

    this._typ = typ;
    this._fsTyp = fsTyp;
    this._pid = pid;
    this._name = name;
    this._uhrzeit = zeit;
    this._anzeigeElemente = anzeigen;
    this._zusatzElemente = zusatz;
    this._x = x;
    this._y = y;
    this._viewId = viewId;

    _status = StatusTyp.UNDEFINERT;
    _aktualisieren = aktualisieren;

  }

  public AnzeigeQuerschnitt(AnzeigeQuerschnittTyp typ, FahrstreifenTyp fsTyp, String pid, List anzeigen, List zusatz,
      List zusatz2, int x, int y, String viewId)
  {

    this._typ = typ;
    this._fsTyp = fsTyp;
    this._pid = pid;
    this._anzeigeElemente = anzeigen;
    this._zusatzElemente = zusatz;
    this._zusatzElemente2 = zusatz2;
    this._x = x;
    this._y = y;
    this._viewId = viewId;

    _status = StatusTyp.UNDEFINERT;
  }

  public AnzeigeQuerschnitt(AnzeigeQuerschnittTyp typ, FahrstreifenTyp fsTyp, String pid, List anzeigen, List zusatz,
      List zusatz2, int x, int y, boolean aktualisieren, String viewId)
  {

    this._typ = typ;
    this._fsTyp = fsTyp;
    this._pid = pid;
    this._anzeigeElemente = anzeigen;
    this._zusatzElemente = zusatz;
    this._zusatzElemente2 = zusatz2;
    this._x = x;
    this._y = y;
    this._viewId = viewId;

    _status = StatusTyp.UNDEFINERT;
  
    _aktualisieren = aktualisieren;
  }

  public AnzeigeQuerschnitt(AnzeigeQuerschnittTyp typ, FahrstreifenTyp fsTyp, String pid, String name, String zeit,
      List anzeigen, List zusatz, long km, int x, int y, boolean aktualisieren, String viewId)
  {

    new AnzeigeQuerschnitt(typ, fsTyp, pid, name, zeit, anzeigen, zusatz, x, y, aktualisieren, viewId);
    this._kilometrierung = km;
  }

  public AnzeigeQuerschnitt(AnzeigeQuerschnittTyp typ, FahrstreifenTyp fsTyp, String pid, List anzeigen, List zusatz,
      List zusatz2, long km, int x, int y, boolean aktualisieren, String viewId)
  {

    new AnzeigeQuerschnitt(typ, fsTyp, pid, anzeigen, zusatz, zusatz2, x, y, aktualisieren, viewId);
    this._kilometrierung = km;
  }
   

  /**
   * Returns the persistent position and size of the node. The edit part handling this node has the task to translate it
   * into a constraint that enables the draw2D system to place the figure of the node. Therefore, the coordinate system
   * of the rectangle is known only to the edit part (but in fact, in this example it is always the local coordinate
   * system of the figure of the node).
   */
  public Rectangle getConstraint()
  {
    return (Rectangle)getProperty("constraint");
  }

  /**
   * Sets the persistent position and size of the node.
   * 
   * @param constraint
   *          The data that enables the edit part handling this node to create a constraint that enables the draw2D
   *          system to place the figure of the node.
   */
  public void setConstraint(Rectangle constraint)
  {
    setProperty("constraint", constraint);
  }

  /**
   * Sets name without firing off any event notifications
   * 
   * @param name
   *          The name to set.
   */
  public void setPid(String name)
  {
    String old = this._pid;
    this._pid = name;
  }

  /**
   * Sets bounds without firing off any event notifications
   * 
   * @param bounds
   *          The bounds to set.
   */
  public void setBounds(Rectangle bounds)
  {
    this._bounds = bounds;
  }

  /**
   * Adds relationship where the current object is the foreign key table in a relationship
   * 
   * @param table
   *          the primary key relationship
   */
  public void addTargetConnections(LinkElement table)
  {
    _targetConnections.add(table);
    // firePropertyChange(OUTPUT, null, table);
  }

  /**
   * Adds relationship where the current object is the primary key table in a relationship
   * 
   * @param table
   *          the foreign key relationship
   */
  public void addSourceConnections(LinkElement table)
  {
    _sourceConnections.add(table);
    // firePropertyChange(INPUT, null, table);
  }

  /**
   * Removes relationship where the current object is the foreign key table in a relationship
   * 
   * @param table
   *          the primary key relationship
   */
  public void removeSourceConnections(LinkElement table)
  {
    _sourceConnections.remove(table);
    // firePropertyChange(OUTPUT, table, null);
  }

  /**
   * Removes relationship where the current object is the primary key table in a relationship
   * 
   * @param table
   *          the foreign key relationship
   */
  public void removeTargetConnections(LinkElement table)
  {
    _targetConnections.remove(table);
    // firePropertyChange(INPUT, table, null);
  }

  public void addAnzeigeElement(AnzeigeElement ae)
  {
    if (_anzeigeElemente.contains(ae))
    {
      throw new IllegalArgumentException("AnzeigeElement already present");
    }
    _anzeigeElemente.add(ae);
  }

  public void addAnzeigeElement(AnzeigeElement ae, int index)
  {
    if (_anzeigeElemente.contains(ae))
    {
      throw new IllegalArgumentException("AnzeigeElement already present");
    }
    _anzeigeElemente.add(index, ae);
  }

  public void removeAnzeigeElement(AnzeigeElement ae)
  {
    _anzeigeElemente.remove(ae);
    // firePropertyChange(CHILD, column, null);
  }

  public void switchAnzeigeElement(AnzeigeElement ae, int index)
  {
    _anzeigeElemente.remove(ae);
    _anzeigeElemente.add(index, ae);
  }

  public void addZusatzElement(ZusatzElement ze)
  {
    if (_anzeigeElemente.contains(ze))
    {
      throw new IllegalArgumentException("AnzeigeElement already present");
    }
    _anzeigeElemente.add(ze);
  }

  public void addZusatzElement(ZusatzElement ze, int index)
  {
    if (_anzeigeElemente.contains(ze))
    {
      throw new IllegalArgumentException("AnzeigeElement already present");
    }
    _anzeigeElemente.add(index, ze);
  }

  public void removeZusatzElement(ZusatzElement ze)
  {
    _anzeigeElemente.remove(ze);
    // firePropertyChange(CHILD, column, null);
  }

  public void switchZusatzElement(ZusatzElement ze, int index)
  {
    _anzeigeElemente.remove(ze);
    _anzeigeElemente.add(index, ze);
  }

  /**
   * If modified, sets name and fires off event notification
   * 
   * @param _pid
   *          The name to set.
   */
  public void statusChange(String status)
  {

    AnzeigeQuerschnitt old = this;

    StatusTyp neu = _status.setTyp(status);

    if (neu != null)
    {
      firePropertyChange(STATUS, old, neu);
      // _status = neu;
    }
    else
      System.err.println(_pid + " : Status konnte nicht gesetzt werden: " + status);
  }

  /**
   * If modified, sets name and fires off event notification
   * 
   * @param pid
   *          The name to set.
   */
  public void timeChange(String zeit)
  {

    AnzeigeQuerschnitt old = this;

    firePropertyChange(ZEIT, old, zeit);
    _uhrzeit = zeit;
  }

  /**
   * If modified, sets bounds and fires off event notification
   * 
   * @param bounds
   *          The bounds to set.
   */
  public void modifyBounds(Rectangle bounds)
  {
    Rectangle oldBounds = this._bounds;
    if (!bounds.equals(oldBounds))
    {
      this._bounds = bounds;
      // firePropertyChange(BOUNDS, null, bounds);
    }
  }

  public String getPid()
  {
    return _pid;
  }

  public List getAnzeigeElemente()
  {

    // System.out.println("Groesse Aes: " + _anzeigeElemente.size());

    return _anzeigeElemente;
  }

  public List getZusatzElemente()
  {
    return _zusatzElemente;
  }

  /**
   * @return Returns the foreignKeyRelationships.
   */
  public List getTargetConnections()
  {
    return _targetConnections;
  }

  /**
   * @return Returns the primaryKeyRelationships.
   */
  public List getSourceConnections()
  {
    return _sourceConnections;
  }

  /**
   * @return Returns the bounds.
   */
  public Rectangle getBounds()
  {
    return _bounds;
  }

  public String toString()
  {
    return _pid;
  }

  public StatusTyp getStatus()
  {
    return _status;
  }

  public void setStatus(StatusTyp status)
  {
    _status = status;
  }

  public int getX()
  {
    return _x;
  }

  public int getY()
  {
    return _y;
  }

  public String getName()
  {
    return _name;
  }

  public void setName(String name)
  {
    _name = name;
  }

  public String getUhrzeit()
  {
    return _uhrzeit;
  }

  public void setUhrzeit(String uhrzeit)
  {
    _uhrzeit = uhrzeit;
  }

  public List getZusatzElemente2()
  {
    return _zusatzElemente2;
  }

  public void setZusatzElemente2(List zusatzElemente2)
  {
    _zusatzElemente2 = zusatzElemente2;
  }

  public AnzeigeQuerschnittTyp getTyp()
  {
    return _typ;
  }

  public void setTyp(AnzeigeQuerschnittTyp typ)
  {
    _typ = typ;
  }

  public FahrstreifenTyp getFsTyp()
  {
    return _fsTyp;
  }

  public void setFsTyp(FahrstreifenTyp fsTyp)
  {
    _fsTyp = fsTyp;
  }

  public boolean isTypHandEingriff()
  {
    return _typHandEingriff;
  }

  public void setTypHandEingriff(boolean typHandEingriff)
  {
    _typHandEingriff = typHandEingriff;
  }

  public boolean isHandEingriff()
  {
    return _handEingriff;
  }

  public void setHandEingriff(boolean handEingriff)
  {
    this._handEingriff = handEingriff;
  }

  public boolean isHandInitial()
  {
    return _handInitial;
  }

  public void setHandInitial(boolean handInitial)
  {
    // System.out.println("Handeingriff initialisiert zu AQ " +_name+" mit: " + handInitial);
    _handInitial = handInitial;
    setHandEingriff(handInitial);
  }

  public boolean getAktualisieren()
  {
    return _aktualisieren;
  }

  public String getViewId()
  {
    return _viewId;
  }

  public Long getKilometrierung()
  {
    return _kilometrierung;
  }

  public void setKilometrierung(Long kilometrierung)
  {
    _kilometrierung = kilometrierung;
  }

  @Override
  public int compareTo(AnzeigeQuerschnitt o)
  {
    if (getKilometrierung() < o.getKilometrierung())
      return -1;
    else if (getKilometrierung() == o.getKilometrierung())
      return 0;
    else
      return 1;

  }

}
