/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.model;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import java.util.HashMap;

/**
 * Provides base class support for model objects to participate in event handling framework, 
 * wird aus Kompatibilittsgrunden bentigt
 * 
 * @deprecated
 * @author Pittner
 * 
 */
public abstract class PropertyAwareObject extends Object implements Serializable
{

  // a property container
  private HashMap properties = new HashMap();

  public static final String CONSTRAINT = "CONSTRAINT";

  public static final String STATUS = "STATUS";

  public static final String ZEIT = "ZEIT";

  public static final String INHALT = "INHALT";

  public static final String ZOOM = "ZOOM";

  public static final String SITUATION = "SITUATION";

  public static final String AUSWAHL = "AUSWAHL";

  public static final String BENDPOINTS = "BENDPOINTS";

  public static final String SOURCE_ANCHOR_OFFSET = "SOURCE_ANCHOR_OFFSET";

  public static final String TARGET_ANCHOR_OFFSET = "TARGET_ANCHOR_OFFSET";

  protected transient PropertyChangeSupport listeners = new PropertyChangeSupport(this);

  protected PropertyAwareObject()
  {
  }

  public void addPropertyChangeListener(PropertyChangeListener l)
  {
    listeners.addPropertyChangeListener(l);
  }

  public void removePropertyChangeListener(PropertyChangeListener l)
  {
    listeners.removePropertyChangeListener(l);
  }

  protected void firePropertyChange(String prop, Object old, Object newValue)
  {
    listeners.firePropertyChange(prop, old, newValue);
  }

  /**
   * Returns a property of the model element.
   * 
   * @param propertyName
   *          the name of the property
   */
  public Object getProperty(String propertyName)
  {
    return properties.get(propertyName);
  }

  /**
   * Sets a property of the model element.
   * 
   * @param propertyName
   *          the name of the property
   * @param propertyValue
   *          The value of the property
   */
  public void setProperty(String propertyName, Object propertyValue)
  {
    Object oldValue = properties.put(propertyName, propertyValue);
    firePropertyChange(propertyName, oldValue, propertyValue);
  }

  public HashMap getProperties()
  {
    return properties;
  }

}
