/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.anlagendarstellung.programmtyp;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IViewSite;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;

/**
 * Darstellung des Fensters zur Parameternderung der Parameter Ablauf-TSF (Stufe)
 * @author Pittner
 */
/**
 * @author Pittner
 *
 */
public class CompositeParameterBereiche extends Composite
{
	/**
	 * Schalter fr Debugausgaben
	 */
  private static final boolean _isDebug = true;
  
  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe TSF-Ablauf
   */
  private ResultData _resultData;

  /**
   * Hauptcomposite
   */
  private Composite _composite;


  /**
   * Composite mit den Reitern fr die Stufen
   */
  private CTabFolder _tabFolder = null;

  /**
   * Liste mit den unterschiedlichen Parametergruppen
   */
	private ArrayList<String> _list;

	/**
	 * Composites mit den Darstellungen
	 */
	private CompositeParameterDefinitionen[] _compositeParameterDarstellung;


  /**
	 * Anzahl der zu versorgenden Fahrstreifen
	 */
	private int _anzahlStufen;

	
	/**
	 * Groups
	 */
	private Group groupStufen;

  /**
   * Anzahl der Stufen
   */
  private int _indexPrg;

  
  private Group groupHandschaltung;

  private Button radioButtonHandSchaltung;

  private Button radioButtonPrio;

  private Combo _comboPrio;

  private ComboViewer _cvComboPrio;

  private Label labelQuelle;

  private Text textBezeichnung;

  private Button radioButtonSpurSperrung;
  
  /**
   * Propertychangelistener
   */
  private PropertyChangeSupport _pcs = new PropertyChangeSupport(this);
  
  public final static String PROP_BEZEICHNUG_TYP = "BEZEICHNUNG_TYP";
  public final static String PROP_SPUR_SPERRUNG = "SPUR_SPERRUNG";
 
  
  
//  private final static String[] STR_TAB_NAME = new String[]{"Kernbereich","Nachbereich","Vorbereich1","Vorbereich2"};
  private final static String[] STR_TAB_NAME = new String[]{"Vorbereich2","Vorbereich1","Kernbereich","Nachbereich",};

  private boolean _isSpurSperrung;
	
	/**
	 * Konstruktor der Klasse
	 * @param parent Aufrufendes Composite
	 * @param style Stil
	 * @param resultData Ergebnis Stufen 
	 */
  public CompositeParameterBereiche(IViewSite viewSite, Composite parent, int style, ResultData resultData, int index)
  {
    super(parent, style);   
//    this._viewSite = viewSite;
    this._resultData = resultData;
    this._indexPrg = index;
    initialize();
  }
  
  
  
  /**
   * Methode fgt einen Listener hinzu
   * @param l Listener
   * @param funktion Funktion auf die der Listener reagiert
   */
  public void addPropertyChangeListener(PropertyChangeListener l)
  {
    _pcs.addPropertyChangeListener(l);
  }

  /**
   * Methode lscht einen Listener
   * @param l Listener
   * @param funktion Funktion auf die der Listener reagiert
   */
  public void removePropertyChangeListener(PropertyChangeListener l)
  {
    _pcs.removePropertyChangeListener(l);
  }

  /**
   * Methode benachrichtigt die Listener ber die nderung einer Funktion
   * @param funktion Funktion auf die der Listener reagiert
   */
  public void benachrichtigeListener(String prop, Object old, Object newValue)
  {
    _pcs.firePropertyChange(prop, old, newValue);
  }
  
  
  
  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    
//    System.out.println("CompositeParameterStufen.initialize()");
    
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

    _list = new ArrayList<String>();

    createInputs();
    createGroupBereiche();
    
    createCompositeTab();
    
    Data data = _resultData.getData();
    
    Array array = data.getItem("ProgrammTypen").asArray();
    
    _isSpurSperrung = false;
    
    if (_indexPrg < array.getLength())
      _isSpurSperrung = array.getItem(_indexPrg).getItem("Spursperrung").asTextValue().getValueText().equals("Ja");
   
    erzeugeBenoetigteReiter(4);
        
    updateData (); 
        
    benachrichtigeListener(PROP_SPUR_SPERRUNG, null, _isSpurSperrung);
    
  }
  
  /**
   * This method initializes groupFilterDefinitionen178
   * 
   */
  private void createInputs()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.verticalAlignment = GridData.CENTER;
    GridData gridData21 = new GridData();
    gridData21.horizontalAlignment = GridData.BEGINNING;
    gridData21.grabExcessHorizontalSpace = true;
    gridData21.verticalAlignment = GridData.CENTER;
    GridData gridData18 = new GridData();
    gridData18.horizontalAlignment = GridData.FILL;
    gridData18.grabExcessHorizontalSpace = true;
    gridData18.verticalAlignment = GridData.FILL;
    GridData gridData19 = new GridData();
    gridData19.horizontalAlignment = GridData.FILL;
    gridData19.grabExcessHorizontalSpace = true;
    gridData19.horizontalSpan = 2;
    GridData gridData20 = new GridData();
    gridData20.horizontalAlignment = GridData.FILL;
    gridData20.grabExcessHorizontalSpace = true;
    GridLayout gridLayout13 = new GridLayout();
    gridLayout13.numColumns = 2;
        
    labelQuelle = new Label(_composite, SWT.NONE);
    labelQuelle.setText("Bezeichnung");
    textBezeichnung = new Text(_composite, SWT.BORDER);
    textBezeichnung.setLayoutData(gridData1);
    textBezeichnung.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {        
       benachrichtigeListener(PROP_BEZEICHNUG_TYP, null, textBezeichnung.getText());         
      }
    });
    
    groupHandschaltung = new Group(_composite, SWT.NONE);
    groupHandschaltung.setText("Handschaltung");
    groupHandschaltung.setLayoutData(gridData18);
    groupHandschaltung.setLayout(gridLayout13);
    
    radioButtonHandSchaltung = new Button(groupHandschaltung, SWT.CHECK);
    radioButtonHandSchaltung.setLayoutData(gridData19);
    radioButtonHandSchaltung.setText("Aktiv");

    radioButtonPrio = new Button(groupHandschaltung, SWT.CHECK);
    radioButtonPrio.setText("Prioritt");
    radioButtonPrio.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        Button btn = (Button)e.getSource();
        boolean sel = btn.getSelection();
        _comboPrio.setEnabled(sel);
      }
    });
    radioButtonPrio.setSelection(false);

    _comboPrio = new Combo(groupHandschaltung, SWT.BORDER | SWT.NONE);
    _comboPrio.setLayoutData(gridData21);
    _comboPrio.setEnabled(false);
    _cvComboPrio = new ComboViewer(_comboPrio);
    _cvComboPrio.add("undefiniert");
    _cvComboPrio.add("0");
    _cvComboPrio.add("60000");

    _cvComboPrio.setSelection(new StructuredSelection("undefiniert"));
    
    radioButtonSpurSperrung = new Button(_composite, SWT.CHECK);
    radioButtonSpurSperrung.setLayoutData(gridData20);
    radioButtonSpurSperrung.setText("Spursperrung");
    radioButtonSpurSperrung.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        Button button = (Button)e.getSource();
        _isSpurSperrung = button.getSelection();
               
        benachrichtigeListener(PROP_SPUR_SPERRUNG, null, _isSpurSperrung);
      }
      
    });
  }

  /**
   * Methode bestimmt die Anzahl der Stufen aus dem Ergebnis des DaV
   * @return Anzahl Stufen
   */
  private int bestimmeAnzahlStufen ()
  {
    int anzahl = 4;
    if (_resultData != null && _resultData.hasData())
    {
    	if (_isSpurSperrung){
    	  anzahl = 2;
    	}
    }
    return anzahl;
  }
  
  /**
   * Updatefunktion zum Aktualisieren der Daten
   */
  private void updateData ()
  {
    if (_resultData != null){
    	updataDataAlgorithmus ();
    	updateDataProgramme();
    }
  }

  /**
   * Aktualisiert die ausgewaehlten Programme
   */
  private void updateDataProgramme()
  {
    if (_resultData.hasData()){
      Data data = _resultData.getData();
      
      Array array = data.getArray("ProgrammTypen");      
      if (array.getLength() > _indexPrg)
      {
        String strPrg = array.getItem(_indexPrg).getItem("Bezeichnung").asTextValue().getValueText();   
        textBezeichnung.setText(strPrg);
        
        boolean hand = array.getItem(_indexPrg).getItem("Handschaltung").getItem("Handprogramm").asTextValue().getValueText().equals("Ja");   
        String prio = array.getItem(_indexPrg).getItem("Handschaltung").getItem("Prioritt").asTextValue().getValueText();
        boolean sperrung = array.getItem(_indexPrg).getItem("Spursperrung").asTextValue().getValueText().equals("Ja");   
        
        radioButtonHandSchaltung.setSelection(hand);
        _comboPrio.setText(prio);
        if (hand){
          radioButtonPrio.setSelection(true);
          _comboPrio.setEnabled(true);
        }
        radioButtonSpurSperrung.setSelection(sperrung);
        
        
//        String[] items = _combo.getItems();
//        int index = 0;
//        for (int i = 0; i < items.length; i++)
//        {
//          String s = items[i];
//          if (s.equals(strPrg)){
//            _combo.setText(s);
//            break;
//          }        
//        }
      }            
    }
  }

  /**
   * Methode legt die bentigte Anzahl der Reiter an
   * @param anzahl Anzahl bentigter Reiter (= Anzahl Stufen)
   */
  private void erzeugeBenoetigteReiter (int anzahl)
  {
  	if (anzahl == _anzahlStufen)
  		return;
  	
  	/*
  	 * Beim ersten Anlegen der Tabelle wird Reiter 0 markiert, bei nderung der Anzahl der
  	 * Reiter ber den Spinner wird der letzten Reiter markiert.
  	 */
  	int indexAnwahl = 0;
  	if (_compositeParameterDarstellung != null)
  		indexAnwahl = anzahl - 1;

//		if (_spinnerAnzahlStufen != null)
//			_spinnerAnzahlStufen.setSelection(anzahl);
		
		/*
		 * Namen der Reiter anpassen
		 */
		_list.clear();
    for (int i=0; i<anzahl; ++i){     
      _list.add (STR_TAB_NAME[i]);
//      _list.add ("Stufe " + (i+1));
    }
    
    /*
     * Bisherige Composite temp. speichern
     */
		Vector<CompositeParameterDefinitionen> tmp = new Vector<CompositeParameterDefinitionen>();
		if (_compositeParameterDarstellung != null)
			for (CompositeParameterDefinitionen cpa : _compositeParameterDarstellung)
				tmp.add(cpa);

		/*
		 * Nicht bentigte Reiter entfernen
		 */
	  CTabItem[] items = _tabFolder.getItems();
	  for (int i=items.length - 1; i >= anzahl; i--)
		  items [i].dispose();

		/*
		 * Feld mit den Composites erweitern bzw. reduzieren
		 */
  	_compositeParameterDarstellung = new CompositeParameterDefinitionen [_list.size()];
  	for (int i=0; i<tmp.size() && i<anzahl; ++i)
  		_compositeParameterDarstellung [i] = tmp.get(i);
		
		/*
		 * Reiter hinzufgen
		 */
		if (anzahl > _anzahlStufen)
		{
		  for (int i=_anzahlStufen; i<anzahl; ++i)
		  {
		    if (radioButtonSpurSperrung.getSelection() && i > 1){

		    }else{
		      erzeugeTabItem(_list.get(i));
		      updataDataAlgorithmus (i);		      
		    }
		  }
		}
    
	  _anzahlStufen = anzahl;

	  if (indexAnwahl >= 0)
  	  _tabFolder.setSelection( indexAnwahl );

    _tabFolder.redraw();
  }

  /**
   * Updatefunktion fr die Stufen
   */
  private void updataDataAlgorithmus()
  {
//    int indexStufe = _tabFolder.getSelectionIndex();
    
    for (int index = 0; index < _tabFolder.getItems().length; ++index){
//      if (indexStufe == index)
        updataDataAlgorithmus(index);
    }
  }

  /**
   * Updatefunktion fr die Stufen
   */
  private void updataDataAlgorithmus (int indexStufe)
  {
    Data data = null;
    if (_resultData != null && _resultData.hasData())
    {
      data = _resultData.getData();

			CTabItem[] items = _tabFolder.getItems();
	    CTabItem item = items [indexStufe];
	    
	    if (item != null )
	    {
        Composite c = (Composite) item.getControl();
        Control[] children = c.getChildren();
        CompositeParameterDefinitionen compPar = null;
        
        if ( children.length == 0 )
        {
          compPar = new CompositeParameterDefinitionen(c, SWT.NONE, _resultData, _indexPrg, indexStufe);
          _pcs.addPropertyChangeListener(compPar);
          
          _compositeParameterDarstellung [indexStufe] = compPar;
          compPar.layout( true );
          c.layout( true );         
        }
        else
        {
          compPar = (CompositeParameterDefinitionen) children[0]; 
          compPar.layout( true );
          c.layout( true );
        }
      }
    }
  }

  /**
   * Methode erteugt die Darstellung der Auswahl der Messquerschnitte
   * 
   */
  private void createGroupBereiche()
  {
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;

    groupStufen = new Group(_composite, SWT.NONE);
    groupStufen.setText("Bereiche");
    groupStufen.setLayout(new GridLayout(1, false));
    groupStufen.setLayoutData(gridData2);
  }

  /**
   * Methode zum Anlegen des Composite mit den Reitern
   */
  private void createCompositeTab()
  {
    GridData gridDataGroup = new GridData();
    gridDataGroup.horizontalAlignment = GridData.FILL;
    gridDataGroup.grabExcessHorizontalSpace = true;
    gridDataGroup.grabExcessVerticalSpace = true;
    gridDataGroup.verticalAlignment = GridData.FILL;

  	GridData gridDataTabFolder = new GridData();
    gridDataTabFolder.horizontalAlignment = GridData.FILL;
    gridDataTabFolder.grabExcessHorizontalSpace = true;
    gridDataTabFolder.grabExcessVerticalSpace = true;
    gridDataTabFolder.verticalAlignment = GridData.FILL;
    gridDataTabFolder.heightHint = 300;

    _tabFolder = new CTabFolder(groupStufen, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridDataTabFolder);
    _tabFolder.addSelectionListener(new SelectionListener() {

      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktualisiereViewer();
      }
    });
  }

  /**
   * Reiter fr die Liste der Objekte erstellen
   * 
   * @param list
   */
  public void erzeugeTabItems( List<String> list )
  {
  	for (int i=0; i<list.size(); ++i)
  		erzeugeTabItem(list.get(i));

  	_tabFolder.setSelection( 0 );
    _tabFolder.redraw();
    _tabFolder.layout(true);
  }

  private void erzeugeTabItem (String name)
  { 	
    String so = name;
    
    CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE );

    String tabName = so;
    tabItem.setText( tabName );
    tabItem.setData( tabName, so );

    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;

    Composite c1 = new Composite( _tabFolder, SWT.NONE );
    c1.setLayout( new GridLayout() );
    c1.setLayoutData( gridData2 );

    tabItem.setControl( c1 );
  }

  /**
   * Methode liefert die einzelnen Composite fr die Stufen zurck
   * @return Composites
   */
  public CompositeParameterDefinitionen[] getCompositeParameterStauBeleg ()
  {
  	return _compositeParameterDarstellung;
  }
  
  /**
   * @return die aktuelle Stufe
   */
  public int getIndexStufe()
  {
    return _indexPrg;
  }

  /**
   * Aktualisiert die Viewer der Darstellung
   */
  public void aktualisiereViewer()
  {    
    CTabItem[] items = _tabFolder.getItems();
    for (int i = 0; i < items.length; i++)
    {
      CTabItem itemSel = items[i];
      Composite cSel = (Composite)itemSel.getControl();
      if (((Composite)cSel).getChildren().length > 0)
      {
        CompositeParameterDefinitionen compParSel = (CompositeParameterDefinitionen)cSel.getChildren()[0];
        compParSel.getTableViewerAktuell().getTable().layout(true);
        compParSel.getTableViewerAktuell().refresh();
        
        CTabFolder tabFolder = compParSel.getTabFolder();
                
        cSel.redraw();
        cSel.layout(true);        
        
        tabFolder.setSelection(0);
        tabFolder.redraw();
        tabFolder.layout(true);
                
//        compParSel.setIndexStufe(i);
      }
    }
  }
  
  /**
   * @return
   */
  public CompositeParameterDefinitionen[] getCompositeParameterDarstellung()
  {
    return _compositeParameterDarstellung;
  }
  
  public ResultData getResultData()
  {
    return _resultData;
  }

  public void setResultData(ResultData resultData)
  {
    _resultData = resultData;
    CTabItem[] items = _tabFolder.getItems();
    for (int i = 0; i < items.length; i++)
    {
      CTabItem item = items[i];
      Composite cSel = (Composite)item.getControl();
      CompositeParameterDefinitionen tabComp = (CompositeParameterDefinitionen)cSel.getChildren()[0];
      tabComp.setAktuelleDaten(resultData.getData());          
    }
  }

  public CTabFolder getTabFolder()
  {
    return _tabFolder;
  }

  public void setTabFolder(CTabFolder tabFolder)
  {
    _tabFolder = tabFolder;
  }
  
  
  @Override
  public void dispose()
  {
    super.dispose();
    CTabItem[] items = _tabFolder.getItems();
    for (int i = 0; i < items.length; i++)
    {
      CTabItem item = items[i];
      if (!item.isDisposed()){
        Composite cSel = (Composite)item.getControl();
        CompositeParameterDefinitionen tabComp = (CompositeParameterDefinitionen)cSel.getChildren()[0];
        _pcs.removePropertyChangeListener(tabComp);
        if (!tabComp.isDisposed())
          tabComp.dispose();
        tabComp = null;
        item.dispose();
      }
      item = null;
    }
    
    
  }



  public Group getGroupHandschaltung()
  {
    return groupHandschaltung;
  }



  public void setGroupHandschaltung(Group groupHandschaltung)
  {
    this.groupHandschaltung = groupHandschaltung;
  }



  public Button getRadioButtonHandSchaltung()
  {
    return radioButtonHandSchaltung;
  }



  public void setRadioButtonHandSchaltung(Button radioButtonHandSchaltung)
  {
    this.radioButtonHandSchaltung = radioButtonHandSchaltung;
  }



  public Button getRadioButtonPrio()
  {
    return radioButtonPrio;
  }



  public void setRadioButtonPrio(Button radioButtonPrio)
  {
    this.radioButtonPrio = radioButtonPrio;
  }



  public Text getTextBezeichnung()
  {
    return textBezeichnung;
  }



  public void setTextBezeichnung(Text textBezeichnung)
  {
    this.textBezeichnung = textBezeichnung;
  }



  public Button getRadioButtonSpurSperrung()
  {
    return radioButtonSpurSperrung;
  }



  public void setRadioButtonSpurSperrung(Button radioButtonSpurSperrung)
  {
    this.radioButtonSpurSperrung = radioButtonSpurSperrung;
  }



  public Combo getComboPrio()
  {
    return _comboPrio;
  }



  public void setComboPrio(Combo comboPrio)
  {
    _comboPrio = comboPrio;
  }
  
}
