/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.anlagendarstellung.programmtyp;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.Perspective;
import org.eclipse.ui.internal.WorkbenchPage;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Nach Auswahl des Menpunkts "KnotenBeeinflussungsAnlage/KnotenbeeinflussungParameter" in der Rahmenapplikation muss sich der Dialog zur Anzeige der
 * KnotenBeeinflussungsAnlagen ffnen.
 * 
 */
public class SonderProgrammTypAuswahlViewParameter extends ViewPart //implements IDaVDataUpdateListener
{

  public static String ID = "de.bsvrz.buv.plugin.anlagendarstellung.views.SonderProgrammTypAuswahlViewParameter";

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  private Composite _parent;
  
  private Composite compositeTop;

  private Group _gruppeUebersicht;

  private Table _tabelleKmdPar;

  private TableViewer _tableViewerKmdPar;

  private ISelectionChangedListener _selectionChangedListener;

  private Composite compositeBottom;

  private Button buttonOk;

  private Button buttonAbbrechen;

  private ClientDavInterface _connection;

  private Button buttonHilfe;

  @Override
  public void createPartControl(Composite parent)
  {
    _parent = parent;

    //Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp( _parent, Activator.PLUGIN_ID + ".auswahlParameter" );
    
    parent.setLayout(new GridLayout());
    createCompositeTop(parent);
    createCompositeBottom(parent);
    _connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();

    SystemObjectType type = _connection.getDataModel().getType("typ.sswSbaStreckenBeeinflussungsAnlage");
    List<SystemObject> kbas = type.getElements();

    List<SystemObject> list = new ArrayList<SystemObject>();

    list.addAll(kbas);

    Map<String, SystemObject> treeMap = new TreeMap<String, SystemObject>();
    for (SystemObject so : list)
    {
      treeMap.put(so.getName(), so);
    }

    List<SystemObject> sortList = new ArrayList<SystemObject>();
    sortList.addAll(treeMap.values());

    _tableViewerKmdPar.setInput(sortList);

  }

  @Override
  public void setFocus()
  {
    //Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();

  }

  /**
   * Erzeugt gruppeUebersicht
   * 
   */
  private void createCompositeTop(Composite parent)
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    compositeTop = new Composite(parent, SWT.NONE);
    compositeTop.setLayout(new GridLayout());
    compositeTop.setLayoutData(gridData);
    GridData gridDataTabelle = new GridData();
    gridDataTabelle.horizontalAlignment = GridData.FILL;
    gridDataTabelle.grabExcessHorizontalSpace = true;
    gridDataTabelle.grabExcessVerticalSpace = true;
    gridDataTabelle.verticalAlignment = GridData.FILL;
    GridData gridDataGruppeUebersicht = new GridData();
    gridDataGruppeUebersicht.horizontalAlignment = GridData.FILL;
    gridDataGruppeUebersicht.grabExcessHorizontalSpace = true;
    gridDataGruppeUebersicht.grabExcessVerticalSpace = true;
    gridDataGruppeUebersicht.verticalAlignment = GridData.FILL;
    _gruppeUebersicht = new Group(compositeTop, SWT.NONE);
    _gruppeUebersicht.setLayout(new GridLayout());
    _gruppeUebersicht.setLayoutData(gridDataGruppeUebersicht);
    _gruppeUebersicht.setText("Auswahl StreckenBeeinflussungsAnlage:");
    _tabelleKmdPar = new Table(_gruppeUebersicht, SWT.BORDER | SWT.MULTI | SWT.FULL_SELECTION);
    _tabelleKmdPar.setHeaderVisible(true);
    _tabelleKmdPar.setLayoutData(gridDataTabelle);
    _tabelleKmdPar.setLinesVisible(true);
    _tableViewerKmdPar = new TableViewer(_tabelleKmdPar);
    // _tableViewerKmdPar = new CheckboxTableViewer(_tabelleKmdPar);

    TableColumn tcTyp = new TableColumn(_tabelleKmdPar, SWT.LEFT);
    tcTyp.setText("Typ:");
    tcTyp.setWidth(100);

    TableColumn tcKey = new TableColumn(_tabelleKmdPar, SWT.LEFT);
    tcKey.setText("Name:");
    tcKey.setWidth(150);

    TableColumn tcValue = new TableColumn(_tabelleKmdPar, SWT.LEFT);
    tcValue.setText("Pid:");
    tcValue.setWidth(340);

    _tableViewerKmdPar.setContentProvider(new ContentProvider());
    _tableViewerKmdPar.setLabelProvider(new LabelProvider());

    _selectionChangedListener = new ISelectionChangedListener() {
      private LinkedList<SystemObject> _systemObjekteUz;

      public void selectionChanged(SelectionChangedEvent event)
      {
        IStructuredSelection selection = (IStructuredSelection)event.getSelection();
        Object firstElement = selection.getFirstElement();

        if (firstElement instanceof SystemObject)
        {

        }
      }

    };
    _tableViewerKmdPar.addSelectionChangedListener(_selectionChangedListener);
    _tableViewerKmdPar.addDoubleClickListener(new IDoubleClickListener() {

      @Override
      public void doubleClick(DoubleClickEvent event)
      {
        // TODO Auto-generated method stub
        createViewFromSelection();       
      }


    });
  }
  
  
  private void createViewFromSelection()
  {
    IStructuredSelection selection = (IStructuredSelection)_tableViewerKmdPar.getSelection();
    
    Object firstElement = selection.getFirstElement();
    if (firstElement instanceof SystemObject)
    {      
      StringBuffer sb = new StringBuffer();
      
      Iterator it = selection.iterator();
      
      ArrayList<SystemObject> list = new ArrayList<SystemObject>();
      
      while(it.hasNext()){
        SystemObject so = (SystemObject)it.next();
        list.add(so);
      }      
      
      for (int i = 0; i < list.size(); i++)
      {
        SystemObject so = list.get(i);
        if (i<list.size()-1)
          sb.append(so.getPid() + ";");
        else
          sb.append(so.getPid());        
      }
       
      String sbaPid = sb.toString();
      
      IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
      IWorkbenchPage page = dw.getActivePage();

      try
      {                    
        page.showView( SonderProgrammTypViewParameter.ID, sbaPid, IWorkbenchPage.VIEW_CREATE );        
        IViewReference ref = page.findViewReference( SonderProgrammTypViewParameter.ID, sbaPid );
        Perspective ap = ((WorkbenchPage) page).getActivePerspective();
        if (ap != null)
        {
          if (ref != null)
          {
            ap.getPresentation().detachPart( ref );
            
            IWorkbenchPart wbPart = ref.getPart( false );
            Shell shell = wbPart.getSite().getShell();
            shell.setSize( 720, 700 );
            IViewPart vPart = ref.getView( true );              
          }
        }
        else
        {
          System.out.println("Perspective ist null!");
        }
      }
      catch (PartInitException e)
      {
        e.printStackTrace();        
      }
        
        
    }
  }
  
  private void createCompositeBottom(Composite parent)
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    compositeBottom = new Composite(parent, SWT.NONE);
    compositeBottom.setLayoutData(gridData3);
    compositeBottom.setLayout(rowLayout);
    buttonOk = new Button(compositeBottom, SWT.NONE);
    buttonOk.setText("Ok");
    buttonOk.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
         createViewFromSelection();       
      }
    });

    buttonAbbrechen = new Button(compositeBottom, SWT.NONE);
    buttonAbbrechen.setText("Abbrechen");
    buttonAbbrechen.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        // TODO Auto-generated method stub
        super.mouseDown(e);
        aktionAbbrechen();
      }

    });
    
    buttonHilfe = new Button(compositeBottom, SWT.NONE);
    buttonHilfe.setText("Hilfe");
    buttonHilfe.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        // TODO Auto-generated method stub
        super.mouseDown(e);
        aktionHilfe();
      }

    });
  }

  protected void aktionAbbrechen()
  {
//    System.err.println("---> TsfAblaufAuswahlViewParameter.aktionAbbrechen()");
    dispose();
  }

  /**
   * Zeigt die Hilfe zum Dialog an.
   * 
   */
  private void aktionHilfe()
  {
    String contextId = Activator.PLUGIN_ID + ".auswahlParameter";
    _debug.fine("ContextID: " + contextId);
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  private class ContentProvider implements IStructuredContentProvider
  {
    @Override
    public Object[] getElements(Object inputElement)
    {
      java.util.List list = (java.util.List)inputElement;

      Object[] re = new Object[list.size()];

      for (int i = 0; i < list.size(); i++)
      {
        re[i] = list.get(i);
      }
      return re;
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }

  }

  private class LabelProvider implements ITableLabelProvider
  {

    @Override
    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    @Override
    public String getColumnText(Object element, int columnIndex)
    {

      // String[] ss = (String[])element;

      String returnString = null;

      switch (columnIndex)
      {
        case 0:
          SystemObject so = (SystemObject)element;

          SystemObjectType type = so.getType();

          if (type.getPid().equals("typ.sswSbaStreckenBeeinflussungsAnlage"))
          {
            returnString = "SswSba";
          }
          else if (type.getPid().equals("typ.knotenBeeinflussungsAnlage"))
          {
            returnString = "KBA";
          }
          else if (type.getPid().equals("typ.knotenBeeinflussungsAnlageZufluss"))
          {
            returnString = "Zufluss";
          }
          else
            returnString = "nicht bestimmbar";

          // returnString = ss[0];
          break;
        case 1:
          returnString = ((SystemObject)element).getName();
          // returnString = ss[0];
          break;

        case 2:
          returnString = ((SystemObject)element).getPid();
          // returnString = ss[1];
          break;

        default:
          returnString = "default";
      }

      return returnString;

    }

    @Override
    public void addListener(ILabelProviderListener listener)
    {
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    @Override
    public void removeListener(ILabelProviderListener listener)
    {
    }

  }
  
   @Override
  public void dispose()
  {     
    super.dispose();
    IWorkbench wb = PlatformUI.getWorkbench();
    if (wb != null){
      IWorkbenchWindow dw = wb.getActiveWorkbenchWindow();
      if (dw != null)
      {
        IWorkbenchPage page = dw.getActivePage();
        if (page != null){
          page.hideView(this);
        }
      }      
    }
  }
}
