/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.util;

import java.util.concurrent.ConcurrentHashMap;

import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.SituationsWrapper;

/**
 * Zentrale Klasse. Dient zum Speichern der Instanzen der {@link SituationsWrapper}
 * 
 * @author Pittner
 * 
 */
public class SituationsWrapperProvider
{


  /**
   * Singleton-Instanz
   */
  private static SituationsWrapperProvider _instanz = null;
  
  
  private ConcurrentHashMap<String, SituationsWrapper> _mapSituationsWrapper = null;

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static SituationsWrapperProvider getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new SituationsWrapperProvider();
    }
    return _instanz;
  }

  /**
   * Konstruktor, erzeugt die Map.
   * 
   * @param connection
   *          Verbindung zum DaV.
   */
  private SituationsWrapperProvider()
  {
      _mapSituationsWrapper = new ConcurrentHashMap<String, SituationsWrapper>();
  }
  
  /**
   * @param pid
   * @param wrapper
   */
  public void addSituationsWrapper(String pid, SituationsWrapper wrapper){    
//    if (!_mapSituationsWrapper.containsKey(pid))
      _mapSituationsWrapper.put(pid, wrapper);    
  }
  
  /**
   * @param pid
   * @param wrapper
   * @return
   */
  public SituationsWrapper getSituationsWrapper(String pid){    
    if (_mapSituationsWrapper.containsKey(pid))
      return _mapSituationsWrapper.get(pid);
    return null;
  }
  
  /**
   * @param pid
   * @param wrapper
   * @return
   */
  public boolean removeSituationsWrapper(String pid){    
    if (_mapSituationsWrapper.containsKey(pid)){
      _mapSituationsWrapper.remove(pid);
      return true;
    }
    return false;
    
  }
  
  
}
