/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.views;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.Assert;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.CheckboxCellEditor;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableFontProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.TreeViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateEvent;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.IDaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.DefsPreference;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.Programm;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.SonderProgramm;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.Wanderbaustelle;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.SonderProgramm.EnumSonderProgrammAktion;
import de.bsvrz.buv.rw.basislib.login.UrlasserDialog;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Sicht zum Auswhlen von Sonderprogrammen.
 * 
 * @author Pittner
 * 
 */
public class AuswahlWanderBaustelleView extends ViewPart implements IDaVDataUpdateListener
{

  public static String ID = "de.bsvrz.buv.plugin.anlagendarstellung.auswahlWanderBaustelleView";

  /**
   * Composites
   */
  private Composite _parent;

  private Composite compositeTop;

  /**
   * Die Instanz der Sonderprogramm-Sicht
   */
  private ZustandsDarstellungSonderProgramm _contentsView;

  /**
   * Maps, zum Speichern der Selektion bzw. der Expansion im TreeTable
   */
  private Map<String, Object> _mapSelection = new HashMap<String, Object>();

  private Map<String, Object[]> _mapExpandiert = new HashMap<String, Object[]>();
  
  /**
   * Die Liste mit den DaV-Sonderprogramm-Daten
   */
//  private Map<String, ResultData> _sonderProgrammMap = new HashMap<String, ResultData>(); 

  /**
   * Die TreeViewer
   */
  private TreeViewer _tvFilter;

  /**
   * String-Definitionen
   */
  private final String _alle = "Alle Sonderprogramm-Typen";

  private final String _defaultText = "Suchkriterium eingeben";

  /**
   * Die Combo zum Auswahl der SonderprogrammTypen
   */
  private Combo _comboObjTypFilter;

  /**
   * Die Group
   */
  private Group _grpAuswahl;

  /**
   * Das Textfeld zur Eingabe des Filters
   */
  private Text _textNamen;

//  private CTabFolder _folder;

//  private Composite compositeTop1;

//  private Composite compositeTop2;

  private TreeViewer _tvFilter2;

//  private Composite compositeTop3;

  private Combo _comboObjTypFilter2;

  private Text _textNamen2;

  private String _secondaryId;

  private Composite _composite3;

  private Composite _compositeButton3;

  private Button _buttonOk;

  private Button _buttonUbernehmen;

  private Button _buttonAbbrechen;

  private Button _buttonHilfe;

  private Wanderbaustelle aktuelleWbs;

  private ClientDavInterface _connection;
  
  private static final Image CHECKED = Activator.getImageDescriptor("icons/checkbox_yes.png").createImage();
  private static final Image UNCHECKED = Activator.getImageDescriptor("icons/checkbox_no.png").createImage();
  

  /**
   * Das Format der nderungszeit
   */
  private final static SimpleDateFormat _sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");

  
  @Override
  public void init(IViewSite site) throws PartInitException
  {
    // TODO Auto-generated method stub
    super.init(site);
    _secondaryId = site.getSecondaryId();
  }
  
  @Override
  public void createPartControl(Composite parent)
  {
//    System.out.println("AuswahlWanderBaustelleView.createPartControl()");
    
    _parent = parent;
    
    _connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();
    
    // Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp( _parent, Activator.PLUGIN_ID + ".auswahlWanderBaustelle" );

    parent.setLayout(new GridLayout());

    createCompositeTop(parent);   
//    erzeugeToolbar();
    
    DaVDataUpdateListener.getInstanz().addDaVDataUpdateListener(this);  
    
    wanderBaustelleInListeAnzeigen();
    
  }

  @Override
  public void setFocus()
  {
    // Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();

  }

  /**
   * Erzeugt das CompositeTop
   * 
   * @param parent
   */
  private void createCompositeTop(Composite parent)
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    compositeTop = new Composite(parent, SWT.NONE);
    compositeTop.setLayout(new GridLayout());
    compositeTop.setLayoutData(gridData);

//    compositeTop1 = new Composite(compositeTop, SWT.NONE);
//    compositeTop1.setLayout(new GridLayout());
//    compositeTop1.setLayoutData(gridData);
    
    createGroupTree2(compositeTop);    
    createCompositeButton();
    
  }

  /**
   * Schliesst die View
   */
  protected void aktionAbbrechen()
  {
    // TODO Auto-generated method stub
    if (!isDirty()){
      PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().hideView(this);              
    }else{
      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.YES | SWT.NO);
      mb.setText("Fehler Schliessen");
      mb.setMessage("Es wurden nderungen vorgenommen: " + "\nSollen diese verworfen werden?");
      int open = mb.open();
      if (open == SWT.YES){
        PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().hideView(this);              
      }     
      return;
    }
    
  }

  /**
   * Zeigt die Hilfe zum Dialog an.
   * 
   */
  private void aktionHilfe()
  {
     String contextId = Activator.PLUGIN_ID + ".auswahlWanderBaustelle";
     PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

//  /**
//   * Setzt den Inhalt der Tabelle mit der aktuellen Sonderprogramm-View
//   * 
//   * @param contents
//   *          die View der Sonderprogramme
//   */
//  public void setContent(ZustandsDarstellungSonderProgramm contents, boolean neueDaten)
//  {
//     setContent(contents, neueDaten, null);
//  }
//  
//  /**
//   * Setzt den Inhalt der Tabelle mit der aktuellen Sonderprogramm-View
//   * 
//   * @param contents
//   *          die View der Sonderprogramme
//   */
//  public void setContent(ZustandsDarstellungSonderProgramm contents, boolean neueDaten, String pid)
//  {
//    if (contents != null)
//    {
//      String secIdNeu = contents.getViewSite().getSecondaryId();
//      if (_contentsView != null)
//      {
//        String secId = _contentsView.getViewSite().getSecondaryId();
//        if (secId.equals(secIdNeu))
//        {
//          if (neueDaten)
//          {
//            wanderBaustelleInListeAnzeigen(contents);
//            expansionWiederHerstellen(secId);
//            selektionWiederHerstellen(secId, pid);
//            setGroupAuswahlText();
//          }
//        }
//        else
//        {
//          if (!neueDaten)
//          {
//            wanderBaustelleInListeAnzeigen(contents);
//            expansionWiederHerstellen(secIdNeu);
//            selektionWiederHerstellen(secIdNeu, pid);
//            setGroupAuswahlText();
//          }
//        }
//      }
//      else
//      {
//        wanderBaustelleInListeAnzeigen(contents);
//      }
//    }
//    else
//    {
//      this._contentsView = contents;
//      _comboObjTypFilter.removeAll();
//      _tvFilter.getTree().removeAll();
//      _tvFilter.setInput(null);
//      _comboObjTypFilter2.removeAll();
//      _tvFilter2.getTree().removeAll();
//      _tvFilter2.setInput(null);
//    }
//  }
  
//  /**
//   * Hilfsmethode, holt die entsprechende ProgrammTyp-Bezeichnung aus den DaV-Daten (atg.sonderProgramm) eines Sonderprogrammes
//   * 
//   * @param secId
//   *            die secondary-Id der View
//   * @param result
//   *            die DaV-Daten
//   * @return
//   */
//  private Programmart holeProgrammArtVonResultData(String secId, ResultData result){
//    List<Programmart> listeProgramme = getListeProgramme();
//    for (Programmart programmart : listeProgramme)
//    {
//      if (result.hasData()){
//        Data data = result.getData();        
//        String text = data.getItem("typBezeichnung").asTextValue().getText();
//        if (programmart.bezeichnung.equals(text)){
//          return programmart;
//        }
//      }
//    }
//    
//    return null;
//    
//  }

  /**
   * Setzt den Namen der Sicht Sonderprogramme der Gruppe Auswahl
   */
  private void setGroupAuswahlText()
  {
    if (_contentsView != null)
    {
      String partName = _contentsView.getPartName();
      if (_grpAuswahl != null && !_grpAuswahl.isDisposed() && partName != null)
      {
        _grpAuswahl.setText("Auswahl " + partName);
      }
    }
  }

  /**
   * Hilfsmethode, holt die entsprechende ProgrammTyp-Bezeichnung aus den DaV-Daten (atg.sonderProgramm) eines Sonderprogrammes
   * 
   * @param secId
   *            die secondary-Id der View
   * @param result
   *            die DaV-Daten
   * @return
   */
  private Wanderbaustelle holeWanderBaustelleVonResultData(String secId, ResultData result){
    List<Wanderbaustelle> listeProgramme = getListeWanderbaustellen2();
    for (Wanderbaustelle programmart : listeProgramme)
    {
      if (result.hasData()){
        Data data = result.getData();        
        String text = data.getItem("typBezeichnung").asTextValue().getText();
        if (programmart.bezeichnung.equals(text)){
          return programmart;
        }
      }
    }
    
    return null;
    
  }
  
  
  /**
   * Wiederherstellen der Selektion der Tabelle anhand der SecondaryViewId
   * 
   * @param secId
   */
  private void selektionWiederHerstellen(String secId, String pid)
  {
    if (_mapSelection.containsKey(secId))
    {
      
      Object obj = _mapSelection.get(secId);
      
      if (pid != null){
        if (_contentsView.getSonderProgrammMap().containsKey(pid))
        {
          ResultData result = _contentsView.getSonderProgrammMap().get(pid);
          
          Wanderbaustelle parent = holeWanderBaustelleVonResultData(secId, result);
          if (parent != null)
            obj = new Programm(parent, result);
        }
      }       
            
      List<Wanderbaustelle> input = (List<Wanderbaustelle>)_tvFilter2.getInput();
      if (input != null)
      {
        for (Wanderbaustelle programmart : input)
        {
          if (obj instanceof Wanderbaustelle)
          {
            Wanderbaustelle art = (Wanderbaustelle)obj;

            if (art.bezeichnung.equals(programmart.bezeichnung))
            {
              _tvFilter2.setSelection(new StructuredSelection(programmart), true);
              _tvFilter2.refresh();
              return;
            }

          }
          else if (obj instanceof Programm)
          {
            Programm prg = (Programm)obj;
            Collection<Programm> programme = programmart.getProgramme();
            for (Programm programm : programme)
            {
              String pid1 = programm.result.getObject().getPid();
              String pid2 = prg.result.getObject().getPid();
              
              if (pid1.equals(pid2))
              {
                _tvFilter2.setSelection(new StructuredSelection(programm), true);
                _tvFilter2.refresh();
                return;
              }
            }
          }
        }
      }
    }
  }

//  /**
//   * Wiederherstellen der Expansion der Tabelle anhand der SecondaryViewId
//   * 
//   * @param secId
//   */
//  private void expansionWiederHerstellen(String secId)
//  {
//    if (_mapExpandiert.containsKey(secId))
//    {
//      Object[] objects = _mapExpandiert.get(secId);
//
//      ArrayList<Object> listNeu = new ArrayList<Object>();
//      for (Object object : objects)
//      {
//        if (object instanceof Programmart)
//        {
//          Programmart art = (Programmart)object;
//          List<Programmart> input = (List<Programmart>)_tvFilter.getInput();
//          if (input != null)
//          {
//            for (Programmart programmart : input)
//            {
//              String pid1 = programmart.toString();
//              String pid2 = art.toString();
//
//              if (pid1.equals(pid2))
//              {
//                listNeu.add(programmart);
//              }
//            }
//          }
//        }
//      }
//      if (_tvFilter != null && !_tvFilter.getTree().isDisposed())
//      {
//        _tvFilter.setExpandedElements(listNeu.toArray());
//        _tvFilter.refresh();
//      }
//    }
//  }

  @Override
  public void dispose()
  {
    // TODO Auto-generated method stub
    super.dispose();
    
    DaVDataUpdateListener.getInstanz().removeDaVDataUpdateListener(this);

    IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    IWorkbenchPage page = dw.getActivePage();
    if (page != null)
    {
      IViewReference[] refs = page.getViewReferences();
      if (refs != null)
      {
        for (IViewReference ref : refs)
        {
          IViewPart view = ref.getView(true);
          if (view instanceof ZustandsDarstellungSonderProgramm)
          {
            ((ZustandsDarstellungSonderProgramm)view).aktualisiereSichtKeineSelektion();
          }
        }
      }
    }
  }

  /**
   * Anzeigenn der Sonderprogramme in der Tabelle
   * 
   * @param contents
   *          die Instanz der SonderprogrammView
   */
  public void wanderBaustelleInListeAnzeigen()
  {
//    System.out.println("AuswahlWanderBaustelleView.wanderBaustelleInListeAnzeigen()");   
    if (_tvFilter2 != null && !_tvFilter2.getTree().isDisposed())
    {
//      _tvFilter.getTree().removeAll();
      
      List<Wanderbaustelle> input = getListeWanderbaustellen2();
      List<Wanderbaustelle> input2 = new ArrayList<Wanderbaustelle>();
      for (Wanderbaustelle wbs : input)
      {
        if (wbs.bezeichnung.equals(_secondaryId))
        {
          input2.add(wbs);
          aktuelleWbs = wbs;
          break;
        }
      }
      
      _tvFilter2.setInput(input2);
       if (input2.size() > 0)
         _tvFilter2.setSelection(new StructuredSelection(input2.get(0)));
      _tvFilter2.getTree().setRedraw(false);
      _tvFilter2.expandAll();
      _tvFilter2.getTree().setRedraw(true);
      
    }
//    fuelleCombo();
  }
  
//  /**
//   * Anzeigenn der Sonderprogramme in der Tabelle
//   * 
//   * @param contents
//   *          die Instanz der SonderprogrammView
//   */
//  public void wanderBaustelleInListeAnzeigen2()
//  {
//    if (_tvFilter2 != null && !_tvFilter2.getTree().isDisposed())
//    {
//      List<Wanderbaustelle> input2 = new ArrayList<Wanderbaustelle>();
//      
////      input2.add(new Wanderbaustelle(_secondaryId, _sonderProgrammMap.values()));            
//      _tvFilter2.setInput(input2);
//    }
//  }
  
//  /**
//   * Liefert die Liste der Sonderprogramme der aktuellen Instanz der SonderprogrammView
//   * 
//   * @return
//   */
//  public List<Wanderbaustelle> getListeWanderbaustellen()
//  {       
//    if (_contentsView != null)
//    {
//      Map<String, List<ResultData>> wanderBaustellen = _contentsView.getWanderBaustellen();
//      List<Wanderbaustelle> input = new ArrayList<Wanderbaustelle>();
//      for (Map.Entry<String, List<ResultData>> me : wanderBaustellen.entrySet())
//      {
//        input.add(new Wanderbaustelle(me.getKey(), me.getValue()));
//      }
//      return input;      
//    }
//    return null;
//  }
  
  /**
   * Liefert die Liste der Sonderprogramme der aktuellen Instanz der SonderprogrammView
   * 
   * @return
   */
  public List<Wanderbaustelle> getListeWanderbaustellen2()
  {   
    if (_connection != null)
    {
      AttributeGroup atg1 = _connection.getDataModel().getAttributeGroup("atg.sswSbaWanderbaustelle");
      AttributeGroup atg2 = _connection.getDataModel().getAttributeGroup("atg.sswSbaSonderprogramm");
      Aspect asp = _connection.getDataModel().getAspect("asp.parameterSoll");
      
      SystemObjectType type = _connection.getDataModel().getType("typ.sswSbaStreckenBeeinflussungsAnlageRichtung");
      List<SystemObject> elements = type.getElements();
      
      List<Wanderbaustelle> ret = new ArrayList<Wanderbaustelle>();
      for (SystemObject systemObject : elements)
      {
        Data cd = systemObject.getConfigurationData(systemObject.getDataModel().getAttributeGroup("atg.sswSbaStreckenBeeinflussungsAnlageRichtung"));
        String strFr = cd.getItem("Fahrtrichtung").asTextValue().getValueText();
        
//        if (strFr.equals(_contentsView.getViewSite().getSecondaryId()))
        {                   
          ResultData rd = _connection.getData(systemObject, new DataDescription(atg1, asp), 1000);
          if (rd != null && rd.hasData()){
            Data data = rd.getData();
            Array array = data.getItem("Wanderbaustelle").asArray();            
            for (int i = 0; i < array.getLength(); i++)
            {
              String name = array.getItem(i).getItem("Name").asTextValue().getValueText();
              
              if (name.equals(_secondaryId)){
                SystemObject[] sps = array.getItem(i).getItem("Sonderprogramme").asReferenceArray().getSystemObjectArray();
                
                
                List<ResultData> list = new ArrayList<ResultData>();
                for (SystemObject sp : sps)
                {         
                  if (sp.isValid()){                  
//                  if (result == null)
                    ResultData result = _connection.getData(sp, new DataDescription(atg2, asp), 1000);
                    if (result != null && result.hasData()){
                      Data data2 = result.getData();
                      boolean equals = data2.getItem("Zustand").asTextValue().getValueText().equals("Aktiv");                     
                      list.add(result);                                                                             
                    }
                  }else{
//                    System.out.println("Invalid: " + sp);
                  }
                }              
                ret.add(new Wanderbaustelle(name, list));                              
              }              
            }
          }
        }
      }
      return ret;      
    }
    return null;
  }
  

//  /**
//   * Liefert die Liste der Sonderprogramme der aktuellen Instanz der SonderprogrammView
//   * 
//   * @return
//   */
//  public List<Programmart> getListeProgramme()
//  {
//    SortedMap<String, Collection<ResultData>> typenMap = new TreeMap<String, Collection<ResultData>>();
//
//    if (_contentsView != null)
//    {
//      Map<String, ResultData> sonderProgrammMap = _contentsView.getSonderProgrammMap();
//
//      Collection<ResultData> values = sonderProgrammMap.values();
//      
//      for (ResultData resultData : values)
//      {
//        Data dta = resultData.getData();
//        if (dta != null)
//        {
//          String name = dta.getItem("typBezeichnung").asTextValue().getText();
//          if (name != null && name.length() > 0)
//          {
//            Collection<ResultData> list = null;
//            if (!typenMap.containsKey(name))
//            {
//              list = new ArrayList<ResultData>();
//            }
//            else
//            {
//              list = typenMap.get(name);
//            }
//            list.add(resultData);
//            typenMap.put(name, list);
//          }
//        }
//      }
//    }
//
//    List<Programmart> input = new ArrayList<Programmart>();
//    for (Map.Entry<String, Collection<ResultData>> me : typenMap.entrySet())
//    {
//      input.add(new Programmart(me.getKey(), me.getValue()));
//    }
//    return input;
//  }

  /**
   * Gibt den Command Stack zurck.
   * 
   * @return der Command Stack, niemals <code>null</code>.
   */
  protected CommandStack getCommandStack()
  {
    CommandStack stack = null;
    if (_contentsView != null)
    {
      GraphicalViewer viewer = (GraphicalViewer)_contentsView.getAdapter(GraphicalViewer.class);
      stack = viewer.getEditDomain().getCommandStack();
      Assert.isNotNull(stack, "Command Stack nicht verfgbar.");
    }
    return stack;
  }

//  /**
//   * Erzeugt die Gruppe Auswahl mit dem TreeViewer
//   * 
//   * @param parent
//   */
//  private void createGroupTree(Composite parent)
//  {
//    GridData gd = new GridData();
//    gd.horizontalAlignment = GridData.FILL;
//    gd.grabExcessHorizontalSpace = true;
//    gd.verticalAlignment = GridData.FILL;
//    gd.grabExcessVerticalSpace = true;
//
//    erzeugeFilterTypen(parent);
//    erzeugeFilterNamen(parent);
//
//    Tree filterTree = new Tree(parent, SWT.FULL_SELECTION | SWT.BORDER);
//    filterTree.setHeaderVisible(true);
//    filterTree.setLinesVisible(true);
//    filterTree.setLayoutData(gd);
//    _tvFilter = new TreeViewer(filterTree);
//
//    TreeColumn column0 = new TreeColumn(filterTree, SWT.LEFT);
//    column0.setText("Status");
//    column0.setWidth(80);
//    
//    TreeColumn column1 = new TreeColumn(filterTree, SWT.LEFT);
//    column1.setText("Programm-Typ/-Name");
//    column1.setWidth(230);
//
//    TreeColumn column2 = new TreeColumn(filterTree, SWT.LEFT);
//    column2.setText("nderungszeitpunkt");
//    column2.setWidth(130);
//
//    TreeColumn column3 = new TreeColumn(filterTree, SWT.LEFT);
//    column3.setText("Stand");
//    column3.setWidth(100);
//
//    TreeColumn column4 = new TreeColumn(filterTree, SWT.LEFT);
//    column4.setText("Simulation");
//    column4.setWidth(100);
//
//    TreeColumn column5 = new TreeColumn(filterTree, SWT.LEFT);
//    column5.setText("Benutzer");
//    column5.setWidth(100);
//
//    TreeColumn column6 = new TreeColumn(filterTree, SWT.LEFT);
//    column6.setText("Ursache");
//    column6.setWidth(100);
//
//    TreeColumn column7 = new TreeColumn(filterTree, SWT.LEFT);
//    column7.setText("Veranlasser");
//    column7.setWidth(100);
//
//    _tvFilter.setContentProvider(new ProgrammContentProvider());
//    _tvFilter.setLabelProvider(new ProgrammLabelProvider());
//
//    _tvFilter.addSelectionChangedListener(new ISelectionChangedListener() {
//      public void selectionChanged(SelectionChangedEvent event)
//      {
//        aktionSelektion(event);
//      }
//    });
//
//    _tvFilter.addTreeListener(new ITreeViewerListener() {
//
//      @Override
//      public void treeExpanded(TreeExpansionEvent event)
//      {
//        Object element = event.getElement();
//        if (element instanceof Programmart)
//        {
//          sichereExpandierteObjekte(element, true);
//        }
//
//      }
//
//      @Override
//      public void treeCollapsed(TreeExpansionEvent event)
//      {
//        Object element = event.getElement();
//        if (element instanceof Programmart)
//        {
//          sichereExpandierteObjekte(element, false);
//        }
//      }
//    });
//
//    TreeColumn[] columns = filterTree.getColumns();
//    for (final TreeColumn tc : columns)
//    {
//      tc.addSelectionListener(new SelectionAdapter() {
//        @Override
//        public void widgetSelected(SelectionEvent e)
//        {
//          Object source = e.getSource();
//          if (source instanceof TreeColumn)
//          {
//            TreeColumn column = (TreeColumn)tc;
//
//            String text = column.getText();
//            if (column.getData() != null)
//            {
//              Boolean b = (Boolean)column.getData();
//              if (b)
//                aktionSortierung(SWT.UP, text);
//              else
//                aktionSortierung(SWT.DOWN, text);
//
//              column.setData(!(Boolean)column.getData());
//            }
//            else
//            {
//              column.setData(true);
//            }
//          }
//        }
//      });
//    }
//  }
  
  /**
   * Erzeugt die Gruppe Auswahl mit dem TreeViewer
   * 
   * @param parent
   */
  private void createGroupTree2(Composite parent)
  {    
    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.FILL;
    gd.grabExcessHorizontalSpace = true;
    gd.verticalAlignment = GridData.FILL;
    gd.grabExcessVerticalSpace = true;
    
    Tree filterTree = new Tree(parent, SWT.FULL_SELECTION | SWT.BORDER);
    filterTree.setHeaderVisible(true);
    filterTree.setLinesVisible(true);
    filterTree.setLayoutData(gd);
    _tvFilter2 = new TreeViewer(filterTree);
    
    
    TreeColumn column0 = new TreeColumn(filterTree, SWT.LEFT);
    column0.setText("Status");
    column0.setWidth(80);
    
    TreeViewerColumn tvc0 = new TreeViewerColumn(_tvFilter2, column0);    
    tvc0.setEditingSupport(new TableColumnEditingSupport(_tvFilter2));  
    tvc0.setLabelProvider(new ColumnLabelProvider(){
      
      @Override
      public String getText(Object element)
      {        
        return null;
      }
      
      @Override
      public Image getImage(Object element)
      {
        if (element instanceof Programm)
        {
          Programm prg = (Programm)element;
          if (prg.isAktiv()){
            return CHECKED;
          }                    
          return UNCHECKED;          
        }        
        return null;
      }
    });
    
    
    TreeColumn column1 = new TreeColumn(filterTree, SWT.LEFT);
    column1.setText("Wanderbaustelle");
    column1.setWidth(230);
    
    TreeViewerColumn tvc1 = new TreeViewerColumn(_tvFilter2, column1);    
//    tvc1.setEditingSupport(new TableColumnEditingSupport());  
    tvc1.setLabelProvider(new ColumnLabelProvider(){
      
      @Override
      public String getText(Object element)
      {
        return element.toString();
      } 
            
      @Override
      public Font getFont(Object element)
      {
        FontData fontData = Display.getCurrent().getSystemFont().getFontData()[0];
        String name = fontData.getName();
        if (element instanceof Wanderbaustelle)
        {
          return JFaceResources.getFontRegistry().getBold(name);
        }else{
          return JFaceResources.getFontRegistry().get(name);          
        }
      }      
    });
    
    
    TreeColumn column00 = new TreeColumn(filterTree, SWT.LEFT);
    column00.setText("Stand (Soll)");
    column00.setWidth(80);
    TreeViewerColumn tvc00 = new TreeViewerColumn(_tvFilter2, column00);    
//  tvc1.setEditingSupport(new TableColumnEditingSupport());  
    tvc00.setLabelProvider(new ColumnLabelProvider(){
    
    @Override
    public String getText(Object element)
    {
      if (element instanceof Programm)
      {
        Programm prg = (Programm)element;
        String std = prg.getStandSoll();
        return std;
      }
      return null;
    }          
  });
    
    TreeColumn column000 = new TreeColumn(filterTree, SWT.LEFT);
    column000.setText("Stand (Ist)");
    column000.setWidth(80);
    TreeViewerColumn tvc000 = new TreeViewerColumn(_tvFilter2, column000);    
//  tvc1.setEditingSupport(new TableColumnEditingSupport());  
    tvc000.setLabelProvider(new ColumnLabelProvider(){
      
      @Override
      public String getText(Object element)
      {
        if (element instanceof Programm)
        {
          Programm prg = (Programm)element;
          String std = prg.getStand();
          return std;
        }
        return null;
      }          
    });
    
    
    TreeColumn column01 = new TreeColumn(filterTree, SWT.LEFT);
    column01.setText("Simulation");
    column01.setWidth(80);
    
    TreeViewerColumn tvc01 = new TreeViewerColumn(_tvFilter2, column01);    
//    tvc00.setEditingSupport(new TableColumnEditingSupport());  
    tvc01.setLabelProvider(new ColumnLabelProvider(){
      
      @Override
      public String getText(Object element)
      {
        // TODO Auto-generated method stub
        
        if (element instanceof Programm)
        {
          Programm prg = (Programm)element;
          String sim = prg.getSimulation();
          return sim;
        }
        
        return null;
      }      
      
    });
       
    _tvFilter2.setContentProvider(new ProgrammContentProvider2());    
//    _tvFilter2.setLabelProvider(new ProgrammLabelProvider2());    
    _tvFilter2.addSelectionChangedListener(new ISelectionChangedListener() {
      public void selectionChanged(SelectionChangedEvent event)
      {
//        System.out
//            .println("{...}.selectionChanged(): " + event); 
        aktionSelektion(event);
        
      }
    });
    
//    _tvFilter2.addTreeListener(new ITreeViewerListener() {
//      
//      @Override
//      public void treeExpanded(TreeExpansionEvent event)
//      {
//        Object element = event.getElement();
//        if (element instanceof Wanderbaustelle)
//        {
//          sichereExpandierteObjekte(element, true);
//        }
//        
//      }
//      
//      @Override
//      public void treeCollapsed(TreeExpansionEvent event)
//      {
//        Object element = event.getElement();
//        if (element instanceof Wanderbaustelle)
//        {
//          sichereExpandierteObjekte(element, false);
//        }
//      }
//      
//    });
    
    TreeColumn[] columns = filterTree.getColumns();
    for (final TreeColumn tc : columns)
    {
//      tc.addSelectionListener(new SelectionAdapter() {
//        @Override
//        public void widgetSelected(SelectionEvent e)
//        {
//          Object source = e.getSource();
//          if (source instanceof TreeColumn)
//          {
//            TreeColumn column = (TreeColumn)tc;
//            
//            String text = column.getText();
//            if (column.getData() != null)
//            {
//              Boolean b = (Boolean)column.getData();
//              if (b)
//                aktionSortierung(SWT.UP, text);
//              else
//                aktionSortierung(SWT.DOWN, text);
//              
//              column.setData(!(Boolean)column.getData());
//            }
//            else
//            {
//              column.setData(true);
//            }
//          }
//        }
//      });
    }
    
//    _tvFilter2.getTree().addMouseListener(new MouseAdapter() {
//    
//      
//      @Override
//      public void mouseUp(MouseEvent e)
//      {
//        // TODO Auto-generated method stub
//        
//      }
//      
//      @Override
//      public void mouseDown(MouseEvent e)
//      {
//        // TODO Auto-generated method stub
//        if (e.button == 3){
//          MenuManager menuMgr = new MenuManager();
//          
//          ISelection selection = _tvFilter2.getSelection();
//          if (selection instanceof StructuredSelection)
//          {
//            StructuredSelection ss = (StructuredSelection)selection;
//            Object firstElement = ss.getFirstElement();
//            if (firstElement instanceof Wanderbaustelle){
//              
//              Menu menu = menuMgr.createContextMenu(_tvFilter2.getControl());
//              _tvFilter2.getControl().setMenu(menu);
//              getSite().registerContextMenu(menuMgr, _tvFilter2);
//              
//              menuMgr.add(new Action("Wanderbaustelle editieren") {
//                public void run() {
//                  
//                  IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
//                  IWorkbenchPage page = dw.getActivePage();
//                  try
//                  {
//                    page.showView(TabelleMqView.ID, "", IWorkbenchPage.VIEW_ACTIVATE);
//                    
//                    IViewReference ref = page.findViewReference(TabelleMqView.ID, "");
//                    ((WorkbenchPage)page).getActivePerspective().getPresentation().detachPart(ref);
//                    IWorkbenchPart viewPart = ref.getPart(false);
//                    
//                    Shell shell = viewPart.getSite().getShell();
//                    
//                    Display display = PlatformUI.getWorkbench().getDisplay();
//                    Point p = display.getCursorLocation();
//                    
//                    shell.setLocation(p.x - 10, p.y - 10);
//                    shell.setSize(520, 315);
//                  }
//                  catch (PartInitException e) {
//                    e.printStackTrace();
//                  }                  
//                };
//              });              
//            }
//          }
//        }
//      }
//      
//      @Override
//      public void mouseDoubleClick(MouseEvent e)
//      {
//        // TODO Auto-generated method stub
//        
//      }
//    });
    
     
  }
  
private class TableColumnEditingSupport extends EditingSupport {
    
    private CheckboxCellEditor _cellEditor;

    private TreeViewer _viewer;
    
    public TableColumnEditingSupport(TreeViewer viewer) {
      super(_tvFilter2);   
      _cellEditor = new CheckboxCellEditor((Composite)_tvFilter2.getTree(), SWT.NONE);
      _viewer = viewer;
    }

    @Override
    protected CellEditor getCellEditor(Object element) {                      
      return _cellEditor;
    }

    @Override
    protected boolean canEdit(Object element) {          
        return true;  
    }

    @Override
    protected Object getValue(Object element) {       
      if (element instanceof Programm)
      {                       
        Programm prg = (Programm)element;
        return prg.isAktiv();
      }
      
      return null;
    }

    @Override
    protected void setValue(Object element, Object value) {      
      if (value instanceof Boolean && element instanceof Programm)
      {        
        Programm prg = (Programm)element;      
        Boolean aktiv = (Boolean)value;
        
        String stand = prg.getStand();
        
//        System.out.println("Aktiv: " + aktiv + " -> Stand: " + stand);        
        
        String standSoll = " - ";
        
        if (aktiv){
          if (!stand.equals("Aktiv")){
              standSoll = "Aktiv";             
          }
        }else{
          if (stand.equals("Aktiv")){
            standSoll = "Erstellt";  
          }
        }
        
        prg.setAktiv(aktiv);
        prg.setStandSoll(standSoll);
        _viewer.update(element, null);
      }
      
    }
  } 
  

//  /**
//   * Erzeugt die Combo zur Auswahl der Sonderprogramm-Typen
//   * 
//   * @param parent
//   */
//  private void erzeugeFilterTypen(Composite parent)
//  {
//    GridData gdTableItem = new GridData();
//    gdTableItem.horizontalAlignment = GridData.FILL;
//    gdTableItem.grabExcessHorizontalSpace = true;
//    gdTableItem.verticalAlignment = GridData.CENTER;
//
//    _comboObjTypFilter = new Combo(parent, SWT.BORDER);
//    _comboObjTypFilter.setLayoutData(gdTableItem);
//
//    _comboObjTypFilter.addSelectionListener(new SelectionAdapter() {
//      @Override
//      public void widgetSelected(SelectionEvent e)
//      {
//        bestimmeGefilterteSonderProgramme();
//      }
//    });
//  }
  
//  /**
//   * Erzeugt die Combo zur Auswahl der Sonderprogramm-Typen
//   * 
//   * @param parent
//   */
//  private void erzeugeFilterTypen2(Composite parent)
//  {
//    GridData gdTableItem = new GridData();
//    gdTableItem.horizontalAlignment = GridData.FILL;
//    gdTableItem.grabExcessHorizontalSpace = true;
//    gdTableItem.verticalAlignment = GridData.CENTER;
//    
//    _comboObjTypFilter2 = new Combo(parent, SWT.BORDER);
//    _comboObjTypFilter2.setLayoutData(gdTableItem);
//    
//    _comboObjTypFilter2.addSelectionListener(new SelectionAdapter() {
//      @Override
//      public void widgetSelected(SelectionEvent e)
//      {
//        bestimmeGefilterteSonderProgramme();
//      }
//    });
//  }

//  /**
//   * Erzeugt das Textfeld zur Filtern der Sonderprogramme
//   * 
//   * @param parent
//   */
//  private void erzeugeFilterNamen(Composite parent)
//  {
//    GridData gdTableItem = new GridData();
//    gdTableItem.horizontalAlignment = GridData.FILL;
//    gdTableItem.grabExcessHorizontalSpace = true;
//    gdTableItem.verticalAlignment = GridData.CENTER;
//
//    _textNamen = new Text(parent, SWT.BORDER);
//    _textNamen.setLayoutData(gdTableItem);
//    _textNamen.setText(_defaultText);
//    _textNamen.addFocusListener(new FocusListener() {
//      @Override
//      public void focusLost(FocusEvent e)
//      {
//        if (_textNamen.getText().isEmpty())
//        {
//          _textNamen.setText(_defaultText);
//        }
//      }
//
//      @Override
//      public void focusGained(FocusEvent e)
//      {
//        if (_textNamen.getText().equals(_defaultText))
//        {
//          _textNamen.setText("");
//        }
//
//      }
//    });
//
//    _textNamen.addModifyListener(new ModifyListener() {
//      @Override
//      public void modifyText(ModifyEvent e)
//      {
//        String text = _textNamen.getText();
//        if (text.length() > 0 && !text.equals(_defaultText))
//          aktionFilter(text);
//        else
//        {
//          bestimmeGefilterteSonderProgramme();
//          aktionFilter(".*");
//        }
//      }
//    });
//  }
  
//  /**
//   * Erzeugt das Textfeld zur Filtern der Sonderprogramme
//   * 
//   * @param parent
//   */
//  private void erzeugeFilterNamen2(Composite parent)
//  {
//    GridData gdTableItem = new GridData();
//    gdTableItem.horizontalAlignment = GridData.FILL;
//    gdTableItem.grabExcessHorizontalSpace = true;
//    gdTableItem.verticalAlignment = GridData.CENTER;
//    
//    _textNamen2 = new Text(parent, SWT.BORDER);
//    _textNamen2.setLayoutData(gdTableItem);
//    _textNamen2.setText(_defaultText);
//    _textNamen2.addFocusListener(new FocusListener() {
//      @Override
//      public void focusLost(FocusEvent e)
//      {
//        if (_textNamen2.getText().isEmpty())
//        {
//          _textNamen2.setText(_defaultText);
//        }
//      }
//      
//      @Override
//      public void focusGained(FocusEvent e)
//      {
//        if (_textNamen2.getText().equals(_defaultText))
//        {
//          _textNamen2.setText("");
//        }
//        
//      }
//    });
//    
//    _textNamen2.addModifyListener(new ModifyListener() {
//      @Override
//      public void modifyText(ModifyEvent e)
//      {
//        String text = _textNamen2.getText();
//        if (text.length() > 0 && !text.equals(_defaultText))
//          aktionFilter(text);
//        else
//        {
//          bestimmeGefilterteSonderProgramme();
//          aktionFilter(".*");
//        }
//      }
//    });
//  }

//  /**
//   * Setzt den Filter des Baums auf den angegebenen Ausdruck - wobei aus Performance-Grnden die contains-Methode mit
//   * verwendet wird
//   * 
//   * @param ausdruck
//   *          Ausdruck
//   */
//  private void aktionFilter(final String ausdruck)
//  {
//    // ViewFilter setzen
//    final ViewerFilter textFilter = new ViewerFilter() {
//
//      @Override
//      public boolean select(Viewer viewer, Object parentElement, Object element)
//      {
//        // String als Matcher fr den Vergleich setzen und Ergebnis abfragen
//        if (element instanceof Programm)
//        {
//          Programm prg = (Programm)element;
//          if (prg.toString().contains(ausdruck))
//            return true;
//          else if (prg.toString().matches(ausdruck))
//            return true;
//
//        }
//        else if (element instanceof Programmart)
//        {
//          Collection<Programm> programme = ((Programmart)element).getProgramme();
//          for (Programm programm : programme)
//          {
//            if (select(viewer, element, programm))
//              return true;
//          }
//        }
//        return false;
//      }
//    };
//
//    _tvFilter.getTree().setRedraw(false);
//    // alten Filter lschen
//    if (_tvFilter.getFilters().length > 0)
//      _tvFilter.removeFilter(_tvFilter.getFilters()[0]);
//
//    // neuen Filter einfgen und Ebenen entsprechend expandieren
//    if (ausdruck.length() > 0)
//    {
//      _tvFilter.addFilter(textFilter);
//      List<Programmart> input = (List<Programmart>)_tvFilter.getInput();
//      if (input != null)
//      {
//        for (Programmart programmart : input)
//        {
//          if (_mapExpandiert.containsKey(getViewSite().getSecondaryId()))
//          {
//            Object[] objects = _mapExpandiert.get(getViewSite().getSecondaryId());
//            for (Object object : objects)
//            {
//              if (object instanceof Programmart)
//              {
//                Programmart art = (Programmart)object;
//                if (programmart.toString().equals(art.toString()))
//                {
//                  _tvFilter.expandToLevel(programmart, 1);
//                }
//              }
//            }
//          }
//        }
//      }
//    }
//    _tvFilter.getTree().setRedraw(true);
//  }

//  /**
//   * Setzt die Sortierung des Baums entsprechend auf- oder abwrts unter Bercksichtigung des gegebenen Comparators
//   * 
//   * @param up
//   *          Richtung der Sortierung (SWT.UP/SWT.DOWN)
//   */
//  private void aktionSortierung(int up, final String column)
//  {
//    if (up == SWT.UP)
//    {
//      _tvFilter.setSorter(new ViewerSorter() {
//        public int compare(Viewer viewer, Object e1, Object e2)
//        {
//          int ret = 0;
//
//          if (e1 instanceof Programmart)
//          {
//            ret = -1;
//          }
//          else if (e1 instanceof Programm)
//          {
//            Programm prg1 = ((Programm)e1);
//            Programm prg2 = ((Programm)e2);
//            ret = compareProgramm(column, prg1, prg2);
//          }
//          return ret;
//        }
//      });
//    }
//    else
//    {
//      _tvFilter.setSorter(new ViewerSorter() {
//        public int compare(Viewer viewer, Object e1, Object e2)
//        {
//          int ret = 0;
//
//          if (e1 instanceof Programmart)
//          {
//            ret = -1;
//          }
//          else if (e1 instanceof Programm)
//          {
//            Programm prg1 = ((Programm)e1);
//            Programm prg2 = ((Programm)e2);
//            ret = compareProgramm(column, prg2, prg1);
//          }
//          return ret;
//        }
//      });
//    }
//  }

//  /**
//   * Fllt die Liste der Sonderprogramme.
//   */
//  public void bestimmeGefilterteSonderProgramme()
//  {
//    List<Programmart> arten = getListeProgramme();
//    List<Programmart> input = new ArrayList<Programmart>();
//    input.addAll(arten);
//
//    int index = _comboObjTypFilter2.getSelectionIndex();
//    String nameTyp = _comboObjTypFilter2.getItem(index);
//
//    List<Programmart> gefilterteProgrammArten = new ArrayList<Programmart>();
//
//    /*
//     * Nach Typen filtern
//     */
//    if (nameTyp.equals(_alle))
//    {
//      for (Programmart art : input)
//        gefilterteProgrammArten.add(art);
//    }
//    else
//    {
//      for (Programmart art : input)
//      {
//        if (art.toString().equals(nameTyp))
//          gefilterteProgrammArten.add(art);
//      }
//    }
//
//    /*
//     * Nach Namen filtern
//     */
//    List<Programmart> gefilterteDatensaetzeNamen = new ArrayList<Programmart>();
//
//    String filterName = _textNamen.getText();
//    if (filterName.isEmpty() || filterName.equals(_defaultText))
//    {
//      for (Programmart art : gefilterteProgrammArten)
//      {
//        gefilterteDatensaetzeNamen.add(art);
//      }
//    }
//    else
//    {
//      for (Programmart art : gefilterteProgrammArten)
//      {
//        List<Programm> gefilterteProgrammeNamen = new ArrayList<Programm>();
//        Collection<Programm> programme = art.getProgramme();
//        for (Programm prg : programme)
//        {
//          try
//          {
//            if (prg.toString().contains(filterName))
//              gefilterteProgrammeNamen.add(prg);
//            else if (prg.toString().matches(filterName))
//              gefilterteProgrammeNamen.add(prg);
//          }
//          catch (Exception e)
//          {
//            e.printStackTrace();
//          }
//        }
//
//        art.setProgramme(gefilterteProgrammeNamen);
//
//        if (art.getProgramme().size() > 0)
//        {
//          gefilterteDatensaetzeNamen.add(art);
//        }
//
//      }
//    }
//    _tvFilter2.setInput(gefilterteDatensaetzeNamen);
//  }

  /**
   * Alle Anzeigen auf beliebigen Inhalt setzten
   */
  private void schaltbildZuruecksetzen()
  {
    if (_contentsView != null)
    {
      _contentsView.aktualisiereSichtKeineSelektion();
    }
  }

  /**
   * Sichert die Expandierten Objekte in einer Map
   * 
   * @param element
   * 
   * @param flag
   */
  private void sichereExpandierteObjekte(Object element, boolean flag)
  {
    Object[] objs = _tvFilter.getExpandedElements();

    List<Object> list = new ArrayList<Object>();

    for (Object object : objs)
    {
      list.add(object);
    }

    boolean inBearbeitung = false;

    if (flag)
    {
      if (!list.contains(element))
      {
        inBearbeitung = true;
        list.add(element);
      }
    }
    else
    {
      if (list.contains(element) && !inBearbeitung)
      {
        list.remove(element);
      }
    }

    _mapExpandiert.put(_contentsView.getViewSite().getSecondaryId(), list.toArray());
  }

//  /**
//   * Der Content-Provider
//   * 
//   * @author Pittner
//   * 
//   */
//  private class ProgrammContentProvider implements ITreeContentProvider
//  {
//
//    public Object[] getChildren(Object parentElement)
//    {
//
//      if (parentElement instanceof List)
//        return ((List<?>)parentElement).toArray();
//
//      if (parentElement instanceof Programmart)
//        return ((Programmart)parentElement).getProgramme().toArray();
//
//      return new Object[0];
//    }
//
//    public Object getParent(Object element)
//    {
//
//      if (element instanceof Programm){                
//        return ((Programm)element).getParent();
//      }
//
//      return null;
//    }
//
//    public boolean hasChildren(Object element)
//    {
//      if (element instanceof List)
//        return ((List<?>)element).size() > 0;
//      if (element instanceof Programmart)
//        return ((Programmart)element).getProgramme().size() > 0;
//
//      return false;
//    }
//
//    public Object[] getElements(Object input)
//    {
//      // input ist das, was in setInput(..) gesetzt wurde.
//      return getChildren(input);
//    }
//
//    public void dispose()
//    {
//    }
//
//    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
//    {
//    }
//  }
  /**
   * Der Content-Provider
   * 
   * @author Pittner
   * 
   */
  private class ProgrammContentProvider2 implements ITreeContentProvider
  {
    
    public Object[] getChildren(Object parentElement)
    {
      
      if (parentElement instanceof List)
        return ((List<?>)parentElement).toArray();
      
      if (parentElement instanceof Wanderbaustelle)
        return ((Wanderbaustelle)parentElement).getProgramme().toArray();
      
      return new Object[0];
    }
    
    public Object getParent(Object element)
    {
      
      if (element instanceof Programm){                
        return ((Programm)element).getParent();
      }
      
      return null;
    }
    
    public boolean hasChildren(Object element)
    {
      if (element instanceof List)
        return ((List<?>)element).size() > 0;
        if (element instanceof Wanderbaustelle)
          return ((Wanderbaustelle)element).getProgramme().size() > 0;
          
          return false;
    }
    
    public Object[] getElements(Object input)
    {
      // input ist das, was in setInput(..) gesetzt wurde.
      return getChildren(input);
    }
    
    public void dispose()
    {
    }
    
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }
  }

//  /**
//   * Der Label-Provider des Trees
//   * 
//   * @author Pittner
//   * 
//   */
//  private class ProgrammLabelProvider implements ITableLabelProvider, ITableFontProvider
//  , IColorProvider
//  {
//
//    public Image getColumnImage(Object element, int columnIndex)
//    {
//      Image re = null;
//      
//      switch (columnIndex)
//      {
//        case 0:
//          if (element instanceof Programm)
//          {
//            String simu = ((Programm)element).getSimulation();
//            if (simu.contains("Ja")){
//              re = ImageFactory.getImage("icons/sp_simu.png");                            
//            }
//            String stand = ((Programm)element).getStand();
//            if (stand.equals("Aktiv")){
//              re = ImageFactory.getImage("icons/sp_aktiv.png");              
//            }else if (stand.equals("Pause")){
//              re = ImageFactory.getImage("icons/sp_pause.png");                            
//            }
//          }
//          break;
//        default:
//      }
//      
//      return re;
//    }
//
//    public String getColumnText(Object element, int columnIndex)
//    {
//
//      switch (columnIndex)
//      {
//        case 1:
//          return element.toString();
//        case 2:
//          if (element instanceof Programm)
//            return ((Programm)element).getZeitpunkt();
//        case 3:
//          if (element instanceof Programm)
//            return ((Programm)element).getStand();
//        case 4:
//          if (element instanceof Programm)
//            return ((Programm)element).getSimulation();
//        case 5:
//          if (element instanceof Programm)
//            return ((Programm)element).getBenutzer();
//        case 6:
//          if (element instanceof Programm)
//            return ((Programm)element).getUrsache();
//        case 7:
//          if (element instanceof Programm)
//            return ((Programm)element).getVeranlasser();
//      }
//      return null;
//    }
//
//    public void addListener(ILabelProviderListener listener)
//    {
//    }
//
//    public void dispose()
//    {
//    }
//
//    public boolean isLabelProperty(Object element, String property)
//    {
//      return false;
//    }
//
//    public void removeListener(ILabelProviderListener listener)
//    {
//    }
//
//    @Override
//    public Font getFont(Object element, int columnIndex)
//    {
//      switch (columnIndex)
//      {
//        case 1:
//          if (element instanceof Programmart)
//          {
//            FontData fontData = Display.getCurrent().getSystemFont().getFontData()[0];
//            String name = fontData.getName();
//            return JFaceResources.getFontRegistry().getBold(name);
//          }
//          if (element instanceof Programm)
//          {
//            String stand = ((Programm)element).getStand();
//            if (stand.equals("Aktiv"))
//            {
//              FontData fontData = Display.getCurrent().getSystemFont().getFontData()[0];
//              String name = fontData.getName();
//              return JFaceResources.getFontRegistry().getItalic(name);
//            }
//          }
//
//      }
//      return null;
//    }
//  }
  
  /**
   * Der Label-Provider des Trees
   * 
   * @author Pittner
   * 
   */
  private class ProgrammLabelProvider2 implements ITableLabelProvider
  , ITableFontProvider
//  , IColorProvider
  {
    
    public Image getColumnImage(Object element, int columnIndex)
    {
      Image re = null;      
//      System.out.println("getColumnImage(): " + columnIndex + " -> " + element);
      
      switch (columnIndex)
      {
        
        case 0:
          if (element instanceof Programm)
          {                       
            
            String simu = ((Programm)element).getSimulation();
            if (simu.contains("Ja")){
              re = ImageFactory.getImage("icons/sp_simu.png");                            
            }
            String stand = ((Programm)element).getStand();
            if (stand.equals("Aktiv")){
              re = ImageFactory.getImage("icons/sp_aktiv.png");              
            }else if (stand.equals("Pause")){
              re = ImageFactory.getImage("icons/sp_pause.png");                            
            }
          }
          break;         
        default:
      }
      
      return re;
    }
    
    public String getColumnText(Object element, int columnIndex)
    {
      
      switch (columnIndex)
      {
        case 1:
          return element.toString();
        case 2:
          if (element instanceof Programm)
            return ((Programm)element).getStand();
        case 3:
          if (element instanceof Programm)
            return ((Programm)element).getSimulation();
          
      }
      return null;
    }
    
    public void addListener(ILabelProviderListener listener)
    {
    }
    
    public void dispose()
    {
    }
    
    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }
    
    public void removeListener(ILabelProviderListener listener)
    {
    }
    
    @Override
    public Font getFont(Object element, int columnIndex)
    {
      switch (columnIndex)
      {
        case 1:
          if (element instanceof Wanderbaustelle)
          {
            FontData fontData = Display.getCurrent().getSystemFont().getFontData()[0];
            String name = fontData.getName();
            return JFaceResources.getFontRegistry().getBold(name);
          }
          if (element instanceof Programm)
          {
            String stand = ((Programm)element).getStand();
            if (stand.equals("Aktiv"))
            {
              FontData fontData = Display.getCurrent().getSystemFont().getFontData()[0];
              String name = fontData.getName();
              return JFaceResources.getFontRegistry().getItalic(name);
            }
          }
          
      }
      return null;
    }    
  }

//  /**
//   * Liefert die Toolbar mit den Kontroll-Aktionen zur SonderprogrammAuswahl
//   */
//  private void erzeugeToolbar()
//  {
//    // selektierte Baumknoten verstecken
//    IAction actionCollapse = new Action("Collapse") {
//      public void run()
//      {
//        actionCollapseActualSelection();
//      };
//    };
//    actionCollapse.setToolTipText("Kindknoten verstecken");
//    actionCollapse.setImageDescriptor(Activator.getImageDescriptor("icons/tree_collapse.png"));
//
//    // selektierte Baumknoten expandieren
//    IAction actionExpand = new Action("Expand") {
//      public void run()
//      {
//        actionExpandActualSelection();
//      };
//    };
//    actionExpand.setToolTipText("Kindknoten anzeigen");
//    actionExpand.setImageDescriptor(Activator.getImageDescriptor("icons/tree_expand.png"));
//
//    // alle Kindknoten verstecken
//    IAction actionCollapseAll = new Action("CollapseAll") {
//      public void run()
//      {
//        _tvFilter.getTree().setRedraw(false);
//        _tvFilter.collapseAll();
//        _tvFilter.getTree().setRedraw(true);
//      };
//    };
//    actionCollapseAll.setToolTipText("alle Kindknoten verstecken");
//    actionCollapseAll.setImageDescriptor(Activator.getImageDescriptor("icons/tree_collapseAll.png"));
//
//    // alle Kindknoten anzeigen
//    IAction actionExpandAll = new Action("ExpandAll") {
//      public void run()
//      {
//        // Informiert den Baum mit dem Neuzeichnen bis zur Fertigstellung zu
//        // warten
//        _tvFilter.getTree().setRedraw(false);
//        _tvFilter.expandAll();
//        _tvFilter.getTree().setRedraw(true);
//      };
//    };
//    actionExpandAll.setToolTipText("alle Kindknoten anzeigen");
//    actionExpandAll.setImageDescriptor(Activator.getImageDescriptor("icons/tree_expandAll.png"));
//
//    // ToolBar erzeugen
//    IToolBarManager toolBarManager = getViewSite().getActionBars().getToolBarManager();
//    toolBarManager.add(actionCollapse);
//    toolBarManager.add(actionExpand);
//    toolBarManager.add(actionCollapseAll);
//    toolBarManager.add(actionExpandAll);
//
//  }

//  /**
//   * Kollabiert die aktuelle Selektion im Baum
//   */
//  private void actionCollapseActualSelection()
//  {
//    if (_tvFilter != null)
//    {
//      LinkedList<Object> expandedElem = new LinkedList<Object>();
//
//      ISelection selection = _tvFilter.getSelection();
//      if (selection instanceof StructuredSelection)
//      {
//        StructuredSelection ss = (StructuredSelection)selection;
//
//        Iterator itExpElem = ss.iterator();
//        while (itExpElem.hasNext())
//        {
//          Object obj = itExpElem.next();
//
//          if (obj instanceof Programmart)
//          {
//            Programmart elem = (Programmart)obj;
//            if (elem.getProgramme().size() > 0)
//            {
//              if (!expandedElem.contains(elem))
//              {
//                expandedElem.add(elem);
//              }
//            }
//          }
//          else if (obj instanceof Programm)
//          {
//            Programm elem = (Programm)obj;
//            Object parent = elem.getParent();
//            if (!expandedElem.contains(parent))
//            {
//              expandedElem.add(parent);
//            }
//          }
//        }
//      }
//
//      if (expandedElem.size() > 0)
//      {
//        _tvFilter.getTree().setRedraw(false);
//        Iterator<Object> it = expandedElem.iterator();
//        while (it.hasNext())
//        {
//          Object elem = it.next();
//          _tvFilter.collapseToLevel(elem, AbstractTreeViewer.ALL_LEVELS);
//        }
//        _tvFilter.getTree().setRedraw(true);
//      }
//    }
//  }

//  /**
//   * Expandiert die aktuelle Selektion im Baum
//   */
//  private void actionExpandActualSelection()
//  {
//    if (_tvFilter != null)
//    {
//      LinkedList<Object> collapsedNodes = new LinkedList<Object>();
//
//      ISelection selection = _tvFilter.getSelection();
//      if (selection instanceof StructuredSelection)
//      {
//        StructuredSelection ss = (StructuredSelection)selection;
//
//        @SuppressWarnings("unchecked")
//        Iterator itColBest = ss.iterator();
//        while (itColBest.hasNext())
//        {
//          Object obj = itColBest.next();
//
//          if (obj instanceof Programmart)
//          {
//            Programmart elem = (Programmart)obj;
//            if (elem.getProgramme().size() > 0)
//            {
//              collapsedNodes.add(elem);
//            }
//          }
//        }
//      }
//
//      if (collapsedNodes.size() > 0)
//      {
//        _tvFilter.getTree().setRedraw(false);
//        Iterator<Object> it = collapsedNodes.iterator();
//        while (it.hasNext())
//        {
//          Object colNode = it.next();
//          _tvFilter.expandToLevel(colNode, AbstractTreeViewer.ALL_LEVELS);
//        }
//        _tvFilter.getTree().setRedraw(true);
//      }
//    }
//  }

//  /**
//   * Fllt die Combo mit den SonderprogrammTypen
//   */
//  private void fuelleCombo()
//  {
//
//    List<Programmart> listeProgramme = getListeProgramme();
//
//    List<String> sortList = new ArrayList<String>();
//
//    for (Programmart art : listeProgramme)
//    {
//      sortList.add(art.toString());
//    }
//
//    Collections.sort(sortList);
//    sortList.add(0, _alle);
//
//    if (_comboObjTypFilter != null && !_comboObjTypFilter.isDisposed())
//    {
//      _comboObjTypFilter.removeAll();
//      for (String s : sortList)
//      {
//        _comboObjTypFilter.add(s);
//      }
//      _comboObjTypFilter.select(0);
//    }
//  }

  /**
   * Vergleicht die Einzelnen Programmbestandteile fuer die Sortierung
   * 
   * @param column
   *          der Text der Spalte
   * @param prg1
   *          ein Sonderprogramm
   * @param prg2
   *          ein Sonderprogramm
   * 
   * @return Ergebnis fuer die Sortierung
   * 
   */
  private int compareProgramm(final String column, Programm prg1, Programm prg2)
  {
    int ret = 0;

    if (column.equals("Programm-Typ/-Name"))
    {
      ret = prg2.toString().compareTo(prg1.toString());
    }
    else if (column.equals("nderungszeitpunkt"))
    {
      try
      {
        Long l1 = _sdf.parse(prg1.getZeitpunkt()).getTime();
        Long l2 = _sdf.parse(prg2.getZeitpunkt()).getTime();
        ret = l1.compareTo(l2);
      }
      catch (ParseException e)

      {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    }
    else if (column.equals("Stand"))
    {
      ret = prg1.getStand().compareTo(prg2.getStand());
    }
    else if (column.equals("Simulation"))
    {
      ret = prg1.getSimulation().compareTo(prg2.getSimulation());
    }
    else if (column.equals("Benutzer"))
    {
      ret = prg1.getBenutzer().compareTo(prg2.getBenutzer());
    }
    else if (column.equals("Ursache"))
    {
      ret = prg1.getUrsache().compareTo(prg2.getUrsache());
    }
    else if (column.equals("Veranlasser"))
    {
      ret = prg1.getVeranlasser().compareTo(prg2.getVeranlasser());
    }
    else
    {
      System.err.println("else: " + column);
    }
    return ret;
  }

  public ZustandsDarstellungSonderProgramm getContentsView()
  {
    return _contentsView;
  }
  
  public void setContentsView(ZustandsDarstellungSonderProgramm contents)
  {
    _contentsView = contents;     
  }

  public TreeViewer getTvFilter()
  {
    return _tvFilter;
  }

  public void setTvFilter(TreeViewer tvFilter)
  {
    _tvFilter = tvFilter;
  }

  private void aktionSelektion(SelectionChangedEvent event)
  {
    boolean reset = false;

    IStructuredSelection selection = (IStructuredSelection)event.getSelection();
    if (!selection.isEmpty())
    {
      Object firstElement = selection.getFirstElement();
      if (firstElement instanceof Programm)
      {
        Programm prg = (Programm)firstElement;

        ResultData rd = prg.getResult();

        if (_contentsView != null)
        {
          SonderProgramm ds = _contentsView.fuelleDatenStruktur(_connection, rd);
          if (ds != null)
          {
            _contentsView.aktualisiereSicht(ds);
          }
          _mapSelection.put(_contentsView.getViewSite().getSecondaryId(), prg);
        }
      }
//      else if (firstElement instanceof Programmart)
//      {
//        Programmart art = (Programmart)firstElement;
//        if (_contentsView != null)
//        {
//        _mapSelection.put(_contentsView.getViewSite().getSecondaryId(), art);
//        reset = true;
//        }
//      }
      else if (firstElement instanceof Wanderbaustelle)
      {
        Wanderbaustelle wbs = (Wanderbaustelle)firstElement;
        if (_contentsView != null)
        {
          _mapSelection.put(_contentsView.getViewSite().getSecondaryId(), wbs);
//          reset = true;          
        }
      }
      else
      {
        reset = true;
      }
    }
    else
    {
      // Schaltbild zuruecksetzen
      reset = true;
    }

    if (reset)
      schaltbildZuruecksetzen();
  }

  @Override
  public void daVDataUpdate(DaVDataUpdateEvent e)
  {
//    System.out.println("AuswahlWanderBaustelleView.daVDataUpdate()");
    
    // TODO Auto-generated method stub
    ResultData result = e.getResult();
    if (result != null){
      AttributeGroup atg = result.getDataDescription().getAttributeGroup();
      if (atg.getPid().equals("atg.sswSbaSonderprogramm"))
      {
        Data data = result.getData();
        if (data != null)
        {
          String fr = data.getItem("Fahrtrichtung").asTextValue().getText();
          String frRef = "";
          
          List<Wanderbaustelle> liste = getListeWanderbaustellen2();
          for (Wanderbaustelle wbs : liste)
          {
            if (wbs.bezeichnung.equals(_secondaryId)){
              Collection<Programm> programme = wbs.getProgramme();
              for (Programm programm : programme)
              {
                if (programm.result.getObject().equals(result.getObject())){
//                  System.out.println("1");            
                  wanderBaustelleInListeAnzeigen();
                  if (_contentsView != null)
                    selektionWiederHerstellen(_contentsView.getViewSite().getSecondaryId(), result.getObject().getPid());
                }
              }
            }
          }          
        }
      }
      else if (atg.getPid().equals("atg.sswSbaManahmeVorschlagAntwort"))
      {     
        Data data = result.getData();

        if (data != null)
        {
          SystemObject so = data.getItem("Sonderprogramm").asReferenceValue().getSystemObject();
          String pid = null;
          if (so != null)
          {
            pid = so.getPid();
          }
          
          List<Wanderbaustelle> liste = getListeWanderbaustellen2();
          for (Wanderbaustelle wbs : liste)
          {
            if (wbs.bezeichnung.equals(_secondaryId)){
              Collection<Programm> programme = wbs.getProgramme();
              for (Programm programm : programme)
              {
                if (programm.result.getObject().equals(so)){
                  String status = data.getItem("Status").asTextValue().getText();
//                  final String fehlerArt = data.getItem("Fehler").getItem("Art").asTextValue().getText();
//                  String fehlerBeschreibung = data.getItem("Fehler").getItem("Beschreibung").asTextValue().getText();
                  if (status.equals("gelscht") && pid != null)
                  {
//                    System.out.println("2");
                    wanderBaustelleInListeAnzeigen();
                    if (_contentsView != null)
                      selektionWiederHerstellen(_contentsView.getViewSite().getSecondaryId(), result.getObject().getPid());
                  }
                }
              }
            }
          }
        }
      }      
      else if (atg.getPid().equals("atg.sswSbaWanderbaustelle"))
      {
        SystemObject so = result.getObject();
        atg = so.getDataModel().getAttributeGroup("atg.sswSbaStreckenBeeinflussungsAnlageRichtung");
        Data cd = so.getConfigurationData(atg);
//        String fr = cd.getItem("Fahrtrichtung").asTextValue().getValueText();
        if (result.hasData()){
          Data data = result.getData();
          Array array = data.getItem("Wanderbaustelle").asArray();
          
//          System.out.println("WBS: " + array.getLength());
          
          for (int i = 0; i < array.getLength(); i++)
          {
            String name = array.getItem(i).getItem("Name").asTextValue().getValueText();
            
            if (name.equals(_secondaryId))
            {
//              SystemObject[] sps = array.getItem(i).getItem("Sonderprogramme").asReferenceArray().getSystemObjectArray();                                        
//              List<ResultData> list = new ArrayList<ResultData>();              
              if (_contentsView != null)
              {                  
//                System.out.println("3");
                wanderBaustelleInListeAnzeigen();
              }                
            }              
          }          
        }
      }
    }    
  }
  
  /**
   * This method initializes _composite3
   * 
   */
  private void createCompositeButton()
  {

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    _composite3 = new Composite(compositeTop, SWT.NONE);
    _composite3.setLayout(new GridLayout());
    _composite3.setLayoutData(gridData);

    createCompositeButton3();
  }

  /**
   * This method initializes _compositeButton3
   * 
   */
  private void createCompositeButton3()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.justify = false;
    rowLayout.wrap = true;
    rowLayout.pack = false;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.END;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.CENTER;
    _compositeButton3 = new Composite(_composite3, SWT.NONE);
    _compositeButton3.setLayout(rowLayout);
    _compositeButton3.setLayoutData(gridData);

    _buttonOk = new Button(_compositeButton3, SWT.NONE);
    _buttonOk.setText("Ok");
    _buttonOk.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionSchalten();
      }
    });

    _buttonUbernehmen = new Button(_compositeButton3, SWT.NONE);
    _buttonUbernehmen.setText("bernehmen");
    _buttonUbernehmen.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionUebernehmen();
      }
    });

    _buttonAbbrechen = new Button(_compositeButton3, SWT.NONE);
    _buttonAbbrechen.setText("Abbrechen");
    _buttonAbbrechen.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionAbbrechen();
      }
    });

    _buttonHilfe = new Button(_compositeButton3, SWT.NONE);
    _buttonHilfe.setText("Hilfe");
    _buttonHilfe.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
//        aktionHilfe();
      }
    });
  }

//  @Override
//  public void doSave(IProgressMonitor monitor)
//  {
//    // TODO Auto-generated method stub
//    
//  }
//
//  @Override
//  public void doSaveAs()
//  {
//    // TODO Auto-generated method stub
//    
//  }
//
//  @Override
  public boolean isDirty()
  {
    if (aktuelleWbs != null){
      Collection<Programm> programme = aktuelleWbs.getProgramme();
      for (Programm prg : programme)
      {
        if (!prg.getStandSoll().equals(" - ")){
          return true;
        }
      }
    }
    
    return false;
  }
//
//  @Override
//  public boolean isSaveAsAllowed()
//  {
//    // TODO Auto-generated method stub
//    return false;
//  }
//
//  @Override
//  public boolean isSaveOnCloseNeeded()
//  {
//    // TODO Auto-generated method stub
//    return true;
//  }
//
//  @Override
//  public int promptToSaveOnClose()
//  {
//    // TODO Auto-generated method stub
//    return ISaveablePart2.CANCEL;
//  }
  
  /**
   * Daten an DaV senden, danach schliesen
   */
  protected void aktionSchalten()
  {
    if (versendeDaten())
    {
      PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().hideView(this);      
    }
    else
    {
      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
      mb.setText("Fehler Urlasser");
      mb.setMessage("Objekt konnte nicht versendet werden: " + "\nEventuell kein Programm ausgewhlt");
      mb.open();
      return;
    }

  }

  /**
   * Daten an DaV senden
   */
  protected void aktionUebernehmen()
  {
    if (versendeDaten())
    {
      // keine weiteren Aktionen
    }
    else
    {
      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
      mb.setText("Fehler Urlasser");
      mb.setMessage("Objekt konnte nicht versendet werden: " + "\nEventuell keinen Programm ausgewhlt");
      mb.open();
      return;
    }

  }

  /**
   * Methode bernimmt das Versenden der Daten
   * 
   * @return true wenn erfolgreich
   */
  private boolean versendeDaten()
  {
    if (aktuelleWbs != null)
    {
      AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.sswSbaManahmeVorschlagAnforderung");
      Aspect asp = _connection.getDataModel().getAspect("asp.sswSbaAnforderung");
      
      SystemObject object = null;
      IPreferenceStore store = Activator.getDefault().getPreferenceStore();
      String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
      if (pidSba != null && pidSba.length() > 0)
      {
        object = _connection.getDataModel().getObject(pidSba);
      }
      
      List<ResultData> list = new ArrayList<ResultData>();
      
      Collection<Programm> programme = aktuelleWbs.getProgramme();
      
//      System.out.println("Prgs: " + programme.size());
      
      for (Programm prg : programme)
      {
        if (!prg.getStandSoll().equals(" - "))
        {
          ResultData result = prg.result;
          if (result != null && result.hasData())
          {          
            
           
            
            SonderProgramm sonderprogramm = ZustandsDarstellungSonderProgramm.fuelleDatenStruktur(_connection, result);
            
//            System.out.println("SP-Array: " + sonderprogramm.getSchaltbilder()[0].getAnzeigen().length);
            
            sonderprogramm.setGewaehlteSonderProgrammAktion(EnumSonderProgrammAktion.ndern);
            
            Boolean[] betriebsArt = sonderprogramm.getBetriebsArt();
            
            if (!prg.getStandSoll().equals(" - ")){
              if (betriebsArt != null && betriebsArt.length > 1){
                betriebsArt[1] = prg.isAktiv();              
                sonderprogramm.setBetriebsArt(betriebsArt);
              }else{           
              }
            }
            
            Data data = _connection.createData(atg);
//            System.out.println("Prg: " + prg.result.getObject() + " -> " + data);                        
            ZustandsDarstellungSonderProgramm.schreibeData(_connection, sonderprogramm, false, data);  
            
            int length = sonderprogramm.getSperrungen().length;
            
            if (_contentsView != null ){
              if (_contentsView.getSonderProgrammDatensatz() != null){
                SonderProgramm sonderProgrammausdarstellung = _contentsView.getSonderProgrammDatensatz();
                String pid = sonderProgrammausdarstellung.getPid();
                if (pid.equals(sonderprogramm.getPid())){
                  
//                  System.out.println("---> gefunden: " + pid);
                  
                  int stufen = _contentsView.getCompositeParameterStufen().bestimmeAnzahlStufen();
                  if (length != stufen)
                    length = stufen;     
                  _contentsView.holeSchaltbildAusDarstellung(_connection, sonderprogramm);
                }
              }
            }
            
            ZustandsDarstellungSonderProgramm.schreibeDataStreckenAbschnitt(sonderprogramm, data, length);            
            ZustandsDarstellungSonderProgramm.schreibeDataSchaltbild(sonderprogramm, data);
            
            if (object != null)
            {
              ResultData resultData = new ResultData(object, new DataDescription(atg, asp), new Date().getTime(), data);
              list.add(resultData);                 
            }                           
          }
        }
      }
      UrlasserDialog urlasserDialog = new UrlasserDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell());
      Boolean schlieenOk = (urlasserDialog.anzeigen(list)).getEingabeOk();
      return schlieenOk;
    }
    return false;
  } 
}
