/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.views;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateEvent;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.IDaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.DefsPreference;
import de.bsvrz.buv.plugin.anlagendarstellung.semi.MassnahmenAnfrageGuiData;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.SonderProgramm;
import de.bsvrz.buv.rw.basislib.login.UrlasserDialog;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * View mit dem eine Schaltung bzw. explizite NICHT-Schaltung der UrsachenEnheiten ausgelst werden kann. Es wird
 * hierzu eine MassnahmenAntwortGUI versendet.<br>
 * <br>
 * Der Button Schalten bzw. Nicht Schalten darf nur dann aktiv sein, wenn ein oder mehrere Tabelleintraege
 * mit der Checkbox ausgewhlt wurden. ber das Kontext-Men kann jeder EIntrag einzeln geschaltet werden. 
 * 
 * @author Pittner
 * 
 */
public class SemiAutomatikUrsachenEinheit extends ViewPart implements IDaVDataUpdateListener
{

  public static String ID = "de.bsvrz.buv.plugin.anlagendarstellung.views.semiAutomatikUrsachenEinheit";

  
  /**
   * Composites
   */
  private Composite _parent;
  private Composite _compTable;
  private Composite _compCtrl;

  /**
   * Group
   */
  private Group _gruppeTabelleAnzeigen;

  
  /**
   * TableViewer
   */
  private CheckboxTableViewer _tvAnzeigen;

  /**
   * Menue-Manager
   */
  private MenuManager _menuMgr;

  /**
   * Actions
   */
  private Action _actionSchalten;
  private Action _actionNichtSchalten;


  /**
   * Buttons
   */
  private Button _bSchalten;
  private Button _bNichtSchalten;
  
  
  /**
   * Liste der zu sendenen Antworten
   * 
   */
  private List<MassnahmenAnfrageGuiData> _listSend = new ArrayList<MassnahmenAnfrageGuiData>();
  
  /**
   * Liste der gltigen Aspekte
   */
  private List<String> _listAspekte = new ArrayList<String>();
  
  private boolean _aspekteAngemeldet = false;
  
  /**
   * Map mit denm Anfragen
   */
  private Map<String, MassnahmenAnfrageGuiData> _map = new HashMap<String, MassnahmenAnfrageGuiData>();

 
  @Override
  public void createPartControl(Composite parent)
  {
    _parent = parent;
    
    // Fr die F1 Taste
//    PlatformUI.getWorkbench().getHelpSystem().setHelp(_parent, Activator.PLUGIN_ID + ".semiAutomatikUrsachenEinheit");

    _parent.setLayout(new GridLayout());
  
    initGui(_parent);
    
    createContextMenu();
    createActions();
    
    
    DaVDataUpdateListener.getInstanz().addDaVDataUpdateListener(this);    
    DaVKommunikationDobj.getInstanz().anmeldenAufDatenSemiAspekt();    
  }

  @Override
  public void setFocus()
  {
    // Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();
  }
 
  @Override
  public void dispose()
  {
    super.dispose();
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection != null)
    {
      DaVKommunikationDobj.getInstanz().abmeldenAufDatenSemiAspekt();
      if (_listAspekte != null && _listAspekte.size() > 0)
        DaVKommunikationDobj.getInstanz().abmeldenAufDatenSemi(_listAspekte);    
      DaVDataUpdateListener.getInstanz().removeDaVDataUpdateListener(this);
    }
    IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
    if (page != null)
    {
      page.hideView(this);
    }
  }

  @Override
  public void daVDataUpdate(DaVDataUpdateEvent e)
  {
    ResultData rd = e.getResult();
    if (rd != null && rd.hasData()){
      processDaVData(rd);
    }    
  }
  
  
  /**
   * Verarbeitet die DaV-Daten
   * @param rd
   */   
  private void processDaVData(ResultData rd)
  {
    DataDescription dd = rd.getDataDescription();
    Data data = rd.getData();
    AttributeGroup atg = dd.getAttributeGroup();
    Aspect asp = dd.getAspect();
    if (atg.getPid().equals("atg.sswSbaZuBearbeitendeAspekte")){
      Array array = data.getItem("Zuordnungen").asArray();
      
      _listAspekte.clear();
      
      for (int i = 0; i < array.getLength(); i++)
      {
        Data item = array.getItem(i);
        Array array3 = item.getItem("AspekteEingang").asArray();
        for (int j = 0; j < array3.getLength(); j++)
        {
          Data item3 = array3.getItem(j);
          SystemObject so = item3.asReferenceValue().getSystemObject();
          if (so != null){
            if (!_listAspekte.contains(so.getPid()))
              _listAspekte.add(so.getPid());
          }
        }
      }        
      if (_listAspekte.size() > 0 && !_aspekteAngemeldet){
        DaVKommunikationDobj.getInstanz().anmeldenAufDatenSemi(_listAspekte);
        _aspekteAngemeldet = true;
      }
    }
    else if (atg.getPid().equals("atg.sswSbaMassnahmenAnfrageGUI")){
      SystemObject ursachenEinheit = data.getItem("Ursacheneinheit").asReferenceValue().getSystemObject();        
      boolean spontan = data.getItem("spontan").asTextValue().getValueText().equals("Ja");
      boolean loeschen = data.getItem("lschen").asTextValue().getValueText().equals("Ja");
      
      long nummer = data.getItem("BenutzerInfo").getItem("nummer").asUnscaledValue().longValue();
      String absenderAlgorithmus = data.getItem("BenutzerInfo").getItem("absenderAlgorithmus").asTextValue().getValueText();
      SystemObject absenderApplikation = data.getItem("BenutzerInfo").getItem("absenderApplikation").asReferenceValue().getSystemObject();
      SystemObject empfaenger = data.getItem("BenutzerInfo").getItem("empfnger").asReferenceValue().getSystemObject();
      int alternative = data.getItem("Alternative").asUnscaledValue().intValue();
      Array array = data.getItem("EinzelMassnahmeAlternative").asArray();
      
      List<SonderProgramm> listeSchaltbilder = new ArrayList<SonderProgramm>();
      for (int i = 0; i < array.getLength(); i++)
      {
        Data item = array.getItem(i);
        SonderProgramm sonderProgramm = new SonderProgramm();
        ZustandsDarstellungSonderProgramm.schreibeDataSchaltbild(sonderProgramm, item);
        listeSchaltbilder.add(sonderProgramm);
      }
      
      SonderProgramm[] schaltBilder = listeSchaltbilder.toArray(new SonderProgramm[listeSchaltbilder.size()]);
      
      // Interne Datenstruktur wird erzeugt
      MassnahmenAnfrageGuiData daten = new MassnahmenAnfrageGuiData(ursachenEinheit, 
          spontan, 
          loeschen, 
          nummer, 
          absenderAlgorithmus, 
          absenderApplikation, 
          empfaenger, 
          alternative, 
          asp, 
          schaltBilder);
      
      boolean entfernt = false;
      if (!_map.containsKey(ursachenEinheit.getPid())){

      }else{
        entfernt = true;
        _map.remove(ursachenEinheit.getPid());
      }
      
      boolean geloescht = false;
      if (schaltBilder.length > 0)          
        _map.put(ursachenEinheit.getPid(), daten);
      else{
        geloescht = true;
      }
      
      if (_map != null){
        setTableInput(_map);
      }
    }
  }

    
  /**
   * Initialisiert die Darstellung
   * @param parent
   */
  public void initGui(Composite parent)
  {    
    getCompositeTable(parent);
    getCompButton(parent);

    /*
     * Layout
     */
    parent.setLayout(new GridLayout(1, false));

    GridData gdCompInfoText = new GridData();
    gdCompInfoText.grabExcessHorizontalSpace = true;
    gdCompInfoText.grabExcessVerticalSpace = true;
    gdCompInfoText.horizontalAlignment = GridData.FILL;
    gdCompInfoText.verticalAlignment = GridData.FILL;
    getCompositeTable(parent).setLayoutData(gdCompInfoText);

    GridData gdCompCtrl = new GridData();
    gdCompCtrl.grabExcessHorizontalSpace = true;
    gdCompCtrl.horizontalAlignment = GridData.CENTER;
    getCompButton(parent).setLayoutData(gdCompCtrl);
  }

  /**
   * Initialisiert die Tabelle mit dem entsprechenden Composite
   * @param parent
   * @return
   */
  private Composite getCompositeTable(Composite parent)
  {
    if (_compTable == null)
    {
      _compTable = new Composite(parent, SWT.BORDER);

      GridData gridData = new GridData();
      gridData.horizontalAlignment = GridData.FILL;
      gridData.grabExcessVerticalSpace = true;
      gridData.grabExcessHorizontalSpace = true;
      gridData.verticalAlignment = GridData.FILL;
      _compTable.setLayout(new GridLayout());
      _compTable.setLayoutData(gridData);
      GridData gridDataTabelle = new GridData();
      gridDataTabelle.horizontalAlignment = GridData.FILL;
      gridDataTabelle.grabExcessHorizontalSpace = true;
      gridDataTabelle.grabExcessVerticalSpace = true;
      gridDataTabelle.verticalAlignment = GridData.FILL;

      GridData gridDataGruppeAnzeigequerschnitte = new GridData();
      gridDataGruppeAnzeigequerschnitte.horizontalAlignment = GridData.FILL;
      gridDataGruppeAnzeigequerschnitte.grabExcessHorizontalSpace = true;
      gridDataGruppeAnzeigequerschnitte.grabExcessVerticalSpace = true;
      gridDataGruppeAnzeigequerschnitte.verticalAlignment = GridData.FILL;
      _gruppeTabelleAnzeigen = new Group(_compTable, SWT.NONE);
      _gruppeTabelleAnzeigen.setLayout(new GridLayout());
      _gruppeTabelleAnzeigen.setLayoutData(gridDataGruppeAnzeigequerschnitte);
      _gruppeTabelleAnzeigen.setText("UrsachenEinheiten");
      
      
      Table table = new Table(_gruppeTabelleAnzeigen, SWT.FULL_SELECTION | SWT.CHECK);      
      _tvAnzeigen = new CheckboxTableViewer (table);
    
      TableColumn column1 = new TableColumn(table, SWT.LEFT);
      column1.setWidth(200);
      column1.setText("Name");

      TableColumn column2 = new TableColumn(table, SWT.LEFT);
      column2.setWidth(80);
      column2.setText("Applikation");

      TableColumn column3 = new TableColumn(table, SWT.LEFT);
      column3.setWidth(80);
      column3.setText("Algorithmus");

      TableColumn column4 = new TableColumn(table, SWT.LEFT);
      column4.setWidth(90);
      column4.setText("Alternative");

      _tvAnzeigen.setContentProvider(new ArrayContentProvider());
      _tvAnzeigen.setLabelProvider(new LabelProvider());
      _tvAnzeigen.getTable().setLayoutData(gridDataTabelle);
      _tvAnzeigen.getTable().setLinesVisible(true);
      _tvAnzeigen.getTable().setHeaderVisible(true);
      
      _tvAnzeigen.addCheckStateListener(new ICheckStateListener() {
        
        @Override
        public void checkStateChanged(CheckStateChangedEvent event)
        {         
          if (event != null){
            Object element = event.getElement();
            if (element instanceof MassnahmenAnfrageGuiData)
            {
              MassnahmenAnfrageGuiData data = (MassnahmenAnfrageGuiData)element;
              if (event.getChecked()){
                if (!_listSend.contains(data))
                  _listSend.add(data);               
              }else{
                if (_listSend.contains(data))
                  _listSend.remove(data);
              }               
            }            
          }
          
          if (_listSend.size() > 0){
            _bSchalten.setEnabled(true);
            _bNichtSchalten.setEnabled(true);
          }
          else{
            _bSchalten.setEnabled(false);
            _bNichtSchalten.setEnabled(false);
          }
        }
      });
    }
    return _compTable;
  }
  
  
  /**
   * Erzeugt das Kontext-Men
   */
  private void createContextMenu()
  {
    _menuMgr = new MenuManager("#PopupMenu");
    _menuMgr.setRemoveAllWhenShown(true);
    _menuMgr.addMenuListener(new IMenuListener() {
      public void menuAboutToShow(IMenuManager manager)
      {
        fillContextMenu(manager);
      }
    });

    Menu menu = _menuMgr.createContextMenu(_tvAnzeigen.getControl());
    _tvAnzeigen.getControl().setMenu(menu);
  }

  private void fillContextMenu(IMenuManager manager)
  {
    manager.add(_actionSchalten);
    manager.add(_actionNichtSchalten);
  }
  
  /**
   * Erzeugt die Actions
   */
  private void createActions()
  {
    if (_actionSchalten == null)
    {
      _actionSchalten = new Action() {
        public void run()
        {
          ISelection selection = _tvAnzeigen.getSelection();
          if (selection instanceof StructuredSelection)
          {
            StructuredSelection ss = (StructuredSelection)selection;
            Object object = ss.getFirstElement();
            if (object instanceof MassnahmenAnfrageGuiData)
            {
              Iterator it = ss.iterator();
              while (it.hasNext())
              {
                MassnahmenAnfrageGuiData data = (MassnahmenAnfrageGuiData)it.next();
                if (!_listSend.contains(data))
                  _listSend.add(data);               
                
              }               
              aktionSchalten(true);                          
            }            
          }          
        }
      };
      _actionSchalten.setId(ID + ".actionSchalten");
      
      String ss = "UrsachenEinheit schalten";
      
      _actionSchalten.setText(ss);
      _actionSchalten.setToolTipText(ss);
    }
    if (_actionNichtSchalten == null)
    {
      _actionNichtSchalten = new Action() {
        public void run()
        {
          aktionSchalten(false);          
        }
      };
      _actionNichtSchalten.setId(ID + ".actionNichtSchalten");
      
      String ss = "UrsachenEinheit nicht schalten";
      
      _actionNichtSchalten.setText(ss);
      _actionNichtSchalten.setToolTipText(ss);
    }
     
    _menuMgr.add(_actionSchalten);
    _menuMgr.add(_actionNichtSchalten);    
  }
   
  /**
   * Getter, liefert die Map mit den Anfragen
   * @return
   */
  public Map<String, MassnahmenAnfrageGuiData> getMap()
  {
    return _map;
  }

  /**
   * Initialisiert die Buttons mit dem entsprechenden Composite
   * @param parent
   * @return
   */
  public Composite getCompButton(Composite parent)
  {
    if (_compCtrl == null)
    {
      _compCtrl = new Composite(parent, SWT.NONE);

      _bSchalten = new Button(_compCtrl, SWT.PUSH);
      _bSchalten.setText("Schalten");
      _bSchalten.setEnabled(false);
      _bSchalten.addSelectionListener(new SelectionAdapter()
      {
        @Override
        public void widgetSelected(SelectionEvent e)
        {
          aktionSchalten(true);
        }
      });

      _bNichtSchalten = new Button(_compCtrl, SWT.PUSH);
      _bNichtSchalten.setText("Nicht schalten"); 
      _bNichtSchalten.setEnabled(false);
      _bNichtSchalten.addSelectionListener(new SelectionAdapter()
      {
        @Override
        public void widgetSelected(SelectionEvent e)
        {
          aktionSchalten(false);
        }
      });

      Button bSchliessen = new Button(_compCtrl, SWT.PUSH);
      bSchliessen.setText("Schlieen");
      bSchliessen.addSelectionListener(new SelectionAdapter()
      {
        @Override
        public void widgetSelected(SelectionEvent e)
        {
           dispose();
        }
      });

      /*
       * Layout
       */
      _compCtrl.setLayout(new RowLayout());
    }

    return _compCtrl;
  }

  /**
   * Fhrt das Versenden der Schaltung durch. Die Daten werden erzeugt und dem {@link UrlasserDialog} bergeben.
   * @param schalten
   *          Flag, das anzeigt ob Schalten oder Nicht-Schalten ausgefhrt werden soll.
   */
  private void aktionSchalten(boolean schalten)
  {
    if (_listSend.size() > 0)
    {
      ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
      if (connection != null)
      {
        IPreferenceStore store = Activator.getDefault().getPreferenceStore();
        String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
        if (pidSba != null && pidSba.length() > 0)
        {
          SystemObject object = connection.getDataModel().getObject(pidSba);
          if (object != null)
          {
            AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaMassnahmenAntwortGUI");
            Data data = connection.createData(atg);

            List<ResultData> listResultData = new ArrayList<ResultData>();

            for (MassnahmenAnfrageGuiData tmp : _listSend)
            {
              data.getItem("BenutzerInfo").getItem("nummer").asUnscaledValue().set(tmp.getNummer());
              data.getItem("BenutzerInfo").getItem("absenderAlgorithmus").asTextValue().setText(
                  tmp.getAbsenderAlgorithmus());
              data.getItem("BenutzerInfo").getItem("absenderApplikation").asReferenceValue().setSystemObject(
                  tmp.getAbsenderApplikation());
              
              String strSenden = "Ja";
              if (!schalten)
                strSenden = "Nein";
              
              data.getItem("MassnahmeVersenden").asTextValue().setText(strSenden);

              ResultData rd = new ResultData(object, new DataDescription(atg, tmp.getAspekt()), new Date().getTime(),
                  data);
              listResultData.add(rd);
            }

            UrlasserDialog urlasserDialog = new UrlasserDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell());
            Boolean schlieenOk = (urlasserDialog.anzeigen(listResultData)).getEingabeOk();
          }
        }
      }
    }
  }

  /**
   * Label-Provider, erwartet als Input Instanzen vom Typ {@link MassnahmenAnfrageGuiData}
   * @author Pittner
   *
   */
  private static class LabelProvider implements ITableLabelProvider
  {
    @Override
    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    @Override
    public String getColumnText(Object element, int columnIndex)
    {
      String result = null;
      MassnahmenAnfrageGuiData modelObject = (MassnahmenAnfrageGuiData)element;

      switch (columnIndex)
      {
        case 0:
          String name = modelObject.getUrsachenEinheit().getName();
          result = name != null ? name : "";
          break;
        case 1:
          String applikation = modelObject.getAbsenderApplikation().getName();
          result = applikation != null ? applikation : "";
          break;
        case 2:
          String algorithmus = modelObject.getAbsenderAlgorithmus();
          result = algorithmus != null ? algorithmus : "";
          break;
        case 3:
          String alternative = "" + modelObject.getAlternative();
          
          // Umsetzung in eine verstndliche Ausgabe
          if (alternative.equals("-1")){
            alternative = "keine Vorgabe";
          }
          
          result = alternative != null ? alternative : "";
          break;
        default:
          result = "";
          break;
      }

      return result;
    }

    @Override
    public void addListener(ILabelProviderListener listener)
    {
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    @Override
    public void removeListener(ILabelProviderListener listener)
    {
    }
  }

  /**
   * Aktualisiert die Tabelle mit der Liste der Anfragen
   * @param map
   */
  public void setTableInput(Map<String, MassnahmenAnfrageGuiData> map)
  {
    if (_tvAnzeigen != null)
    {
      _tvAnzeigen.setInput(map.values());
      _tvAnzeigen.refresh();
    }
  }
  

}
