/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.editors;

import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IPersistableElement;

import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.EngstellenWrapper;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Situation;

/**
 * Der abstrakte EditorInput fr Engstellen.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: EngstellenEditorInput.java,v 1.2 2014/11/14 07:29:40 Pittner Exp $
 */
public abstract class EngstellenEditorInput implements IEditorInput {

	/** Der Engstellen-Wrapper. */
	private final EngstellenWrapper wrapper;

	/**
	 * Der Konstruktor.
	 * 
	 * @param wrapper
	 *            der Wrapper
	 */
	public EngstellenEditorInput(final EngstellenWrapper wrapper) {
		this.wrapper = wrapper;
	}

	/**
	 * Liefert das zu der Engstelle gehrende Wrapper-Objekt zurck. Handelt es
	 * sich um einen EditorInput einer neu anzulegenden Engstelle, so ist dieses
	 * Feld <code>null</code>.
	 * 
	 * @return das zu der Engstelle gehrende Wrapper-Objekt, kann
	 *         <code>null</code> sein
	 */
	public EngstellenWrapper getWrapper() {
		return wrapper;
	}

	@Override
	public boolean exists() {
		return false;
	}

	@Override
	public String getName() {
		return wrapper.getName();
	}

	@Override
	public IPersistableElement getPersistable() {
		return null;
	}

	@Override
	public String getToolTipText() {
		return getName();
	}

	@Override
	public Object getAdapter(@SuppressWarnings("rawtypes") final Class adapter) {
		return null;
	}

	/**
	 * Liefert den Namen des Typs zurck.
	 * 
	 * @return den Namen des Typs
	 */
	public abstract String getTyp();

	@Override
	public boolean equals(final Object obj) {
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof EngstellenEditorInput)) {
			return false;
		}
		final EngstellenEditorInput other = (EngstellenEditorInput) obj;
		if (other.wrapper == null) {
			return false;
		}
		final Situation otherSituation = other.wrapper.getSituation();
		if (otherSituation == null) {
			return false;
		}
		return otherSituation.equals(wrapper.getSituation());
	}

	@Override
	public int hashCode() {
		if (wrapper != null) {
			return wrapper.hashCode();
		}
		return super.hashCode();
	}
}
