/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.parameter.tsf;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.widgets.Display;

import de.bsvrz.buv.plugin.parameter.tsf.daVDataUpdateListener.DaVDataUpdateListener;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ClientApplication;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.MutableSetChangeListener;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageCauser;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;
import de.bsvrz.sys.funclib.operatingMessage.MessageState;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;

/**
 * Dient der Kommunikation mit dem DaV bzgl. der UrsachenEinheitVerwaltung.
 * Implementiert die Interface {@link ClientReceiverInterface, MutableSetChangeListener}.
 * 
 * @author Krause
 * 
 */
public class DaVKommunikation implements ClientReceiverInterface, MutableSetChangeListener
{

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Das Verbindungsobjekt
   */
  private ClientDavInterface _connection = null;

  /**
   * Singleton-Instanz
   */
  private static DaVKommunikation _instanz = null;
  
  /**
   * SynchronisationsObjekt fr den Singleton
   */
  private static Object _syncObject = new Object();


  /**
   * Konfigurationsobjekt
   */  
  private ConfigurationObject _konfigObj = null;
  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static DaVKommunikation getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new DaVKommunikation();
    }
//    if (_instanz == null) { 
//      synchronized(_syncObject) { 
//          if (_instanz == null) { 
//              _instanz = new DaVKommunikation();
//          }
//      }
//    }
    return _instanz;
  }

  /**
   * Konstruktor, setzt die Verbindung zum DaV.
   * 
   * @param connection
   *          Verbindung zum DaV.
   */
  private DaVKommunikation()
  {

    this._connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    /*
     * Hier muss auf die DaVVerbindung mit den entsprechenden Listenern gehrt werden.
     */
    IDaVVerbindungZuhoerer zuhoerer = new IDaVVerbindungZuhoerer() {
      public void verbindungAbgebaut(DaVVerbindungEreignis e)
      {
        _connection = null;
      }

      public void verbindungAufgebaut(DaVVerbindungEreignis e)
      {
        // Anmelden, wenn die Verbindung aufgebaut wurde
        _connection = e.getDaVVerbindung();
      }

    };

    if (_connection != null)
    {
      // Anmelden, wenn die Verbindung vorhanden ist.
      // Ansonsten wird dies dem Listener berlassen.
    }
    else
      _debug.error("Plugin Automatikprogramme/Verwaltung : Connection ist null!");

  }

  /**
   * Setzt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public void setzeDaVVerbindung(ClientDavInterface connnection)
  {
    this._connection = connnection;
  }

  /**
   * Holt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public ClientDavInterface holeDaVVerbindung()
  {
    return _connection;
  }


  /**
   * Methode die das Objekt objId beim Datenverteiler abmeldet zum Lesen der Attibutgruppe atgPid unter dem Aspekt
   * aspPid.
   * 
   * @param objId
   *          Id des Objekts
   * @param atgPid
   *          Attributgruppe die abgemeldet werden soll
   * @param aspPid
   *          Aspekt der abgemeldet werden soll
   */
  private void abmeldeLeseDaV(Long objId, String atgPid, String aspPid)
  {
    if (objId == null || atgPid == null || aspPid == null)
      throw new IllegalArgumentException();

    SystemObject obj = _connection.getDataModel().getObject(objId);
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, obj, dd);
  }

  /**
   * Methode die die Objekte objs beim Datenverteiler abmeldet zum Lesen der Attibutgruppe atgPid unter dem Aspekt
   * aspPid.
   * 
   * @param objs
   *          Liste der Ids der Objekte
   * @param atgPid
   *          Attributgruppe die abgemeldet werden soll
   * @param aspPid
   *          Aspekt der abgemeldet werden soll
   */
  private void abmeldeLeseDaV(List objs, String atgPid, String aspPid)
  {
    if (objs == null || atgPid == null || aspPid == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, objs, dd);
  }

  /**
   * Methode die die Objekte aus der Liste der UrsachenEinheitEintraege lscht und den ChangeListener zum
   * Konfigurationsobjekt beim Datenverteiler abmeldet.
   * 
   * @param konfigObj
   *          zugehriges KonfigurationsObjekt
   */
  private void abmeldeLeseDaV(ConfigurationObject konfigObj)
  {
    if (konfigObj == null)
      throw new IllegalArgumentException();

    MutableSet ms = konfigObj.getMutableSet("UrsachenEinheiten");

    if (ms != null)
    {
      ms.removeChangeListener( this );
      _debug.fine("Listener Menge UrsachenEinheiten abgemeldet.");
    }
    else
    {
      _debug.error("DAVKommunikation/abmeldeLeseDaV: Menge UrsachenEinheiten ist null");
      ClientApplication app = _connection.getLocalApplicationObject();
      
      SystemObject user = (SystemObject)_connection.getLocalUser();
      MessageCauser causer = new MessageCauser(user, "Fehler: Abmelden auf Menge", app.getName());
      MessageSender instance = MessageSender.getInstance();
      instance.init(_connection, "", app.getName());
      instance.setApplicationLabel(app.getName());

      Long id = app.getId();        
      
      instance.sendMessage(id.toString(), 
          MessageType.SYSTEM_DOMAIN, 
          "", 
          MessageGrade.ERROR,
          (SystemObject)_connection.getLocalConfigurationAuthority(), 
          MessageState.NEW_MESSAGE, 
          causer,
          "Menge UrsachenEinheiten ist null!");    
    }

  }
  
  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum Lesen der Attibutgruppe atgPid unter dem
   * Aspekt aspPid.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Aspekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeLeseDaV(String objPid, String atgPid, String aspPid, ReceiverRole role)
  {
    if (objPid == null || atgPid == null || aspPid == null || role == null)
      throw new IllegalArgumentException();

    SystemObject obj = _connection.getDataModel().getObject(objPid);
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, obj, dd, ReceiveOptions.normal(), role);

  }
  
//  /**
//   * Methode die die Objekte mit der PIDs objs in eine Liste eintrgt und den ChangeListener zum 
//   * Konfigurationsobjekte beim Datenverteiler anmeldet. 
//   * Alternativ zum Lesen der Attibutgruppe atgPid unter dem Aspekt aspPid.
//   * 
//   * @param objs
//   *          Liste der Pids der Objekte
//   * @param atgPid
//   *          Attributgruppe die angemeldet werden soll
//   * @param aspPid
//   *          Aspekt der angemeldet werden soll
//   * @param role
//   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
//   * @param konfigObj
//   *          zugehriges KonfigurationsObjekt
//   */
//  private void anmeldeLeseDaV(List<SystemObject> objs, String atgPid, String aspPid, ReceiverRole role, ConfigurationObject konfigObj)
//  {
//    
//    if (objs == null || ( (atgPid == null || aspPid == null || role == null ) && konfigObj == null ) )
//      throw new IllegalArgumentException();
//    
//    if ( konfigObj == null )
//    {
//      //nur notwendig, wenn auf Parameter zugegriffen wird, die aktualisiert werden mssen
//      AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
//      Aspect asp = _connection.getDataModel().getAspect(aspPid);
//      DataDescription dd = new DataDescription(atg, asp);
//  
//      _connection.subscribeReceiver(this, objs, dd, ReceiveOptions.delta(), role);
//  
//      konfigObj = (ConfigurationObject)_connection.getLocalConfigurationAuthority();
//    }
//   }
  
  /**
   * Methode die die Liste der Objekte objs beim Datenverteiler anmeldet 
   * zum Lesen der Attibutgruppe atgPid unter dem Aspekt aspPid.
   * 
   * @param objs
   *          Liste der Pids der Objekte
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeLeseDaV(List<SystemObject> objs, String atgPid, String aspPid, ReceiverRole role)
  {

    if (objs == null || atgPid == null || aspPid == null || role == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, objs, dd, ReceiveOptions.normal(), role);

  }

  /**
   * Verarbeitet smtliche Updates
   */
  public void update(ResultData[] results)
  {
	for (int i = 0; i < results.length; i++)
    {
      final ResultData result = results[i];
      
      Display.getDefault().asyncExec(new Runnable() {
        public void run()
        {
          if (result != null)
            DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
          else
            System.err.println("DAVKommunikation:update: Ein Result ist leer!");
        }
      });
    }
  }

  @Override
  public void update( MutableSet set, SystemObject[] addedObjects, SystemObject[] removedObjects )
  {

    _debug.fine( "update Menge: " + set.getName() );

    if ( set.getName().equals( "UrsachenEinheiten" ) )
    {
      if ( addedObjects != null )
      {
        for (SystemObject so : addedObjects)
        {
          
        }
      }
      if ( removedObjects != null )
      {
        for (SystemObject so : addedObjects)
        {
          
        } 
      }
    }
  }

//  /**
//   * Fhrt eine Neuanmeldung der UrsachenEinheiten zum 1.KonfigurationsObjekt durch
//   */
//  public void anmeldenNeu()
//  {
//
//    if ( _connection != null )
//    {
//      DataModel dataModel = _connection.getDataModel();
//      
//      ConfigurationObject konfigObjekt = getFirstKonfigObjekt( dataModel );
//      
//      anmeldenNeu( konfigObjekt );
//    }
//
//  }
//  
//  /**
//   * Fhrt eine Neuanmeldung der UrsachenEinheiten zum angegebenen KonfigurationsObjekt durch
//   * 
//   * @param obj
//   *          KonfigurationsObjekt aus dem StreckenBeeinflussungsAnlageBereich
//   */
//  public void anmeldenNeu(ConfigurationObject obj)
//  {
//    _konfigObj = obj;
//    
//    if ( _konfigObj != null )
//    {
//     
//      System.out.println("_konfigObj" + _konfigObj);
//      
//      ObjectSet set = _konfigObj.getObjectSet( "UrsachenEinheiten" );
//
//      if (set != null){
//        List<SystemObject> elements = set.getElements();
//        
//        List<SystemObject> listTmp = new ArrayList<SystemObject>();
//        for (SystemObject systemObject : elements)
//        {
//          if ( systemObject.isValid() )
//          {
//            listTmp.add( systemObject );
//            
//          }
//        }
//        
//        if ( listTmp != null && listTmp.size() > 0 )
//          anmeldeLeseDaV( listTmp, "atg.sbaUrsachenEinheitAutomatikprogramm", "asp.eigenschaften", ReceiverRole
//              .receiver(), _konfigObj );        
//      }
//    }
//
//  }

  /**
   * Fhrt eine Abmeldung der UrsachenEinheiten zum angegebenen KonfigurationsObjekt durch
   * 
   * @param obj
   *          KonfigurationsObjekt aus dem StreckenBeeinflussungsAnlageBereich
   */
  public void abmelden(ConfigurationObject obj)
  {
    _konfigObj = obj;
    
    if ( _konfigObj != null )
    {
     
      ObjectSet set = _konfigObj.getObjectSet( "UrsachenEinheiten" );

      List<SystemObject> elements = set.getElements();

      List<SystemObject> listTmp = new ArrayList<SystemObject>();
      for (SystemObject systemObject : elements)
      {
        if ( systemObject.isValid() )
        {
          listTmp.add( systemObject );

        }
      }

      if ( listTmp != null && listTmp.size() > 0 )
        abmeldeLeseDaV( _konfigObj );
    }

  }
  
  /**
   * Bestimmt den Default-Konfigurationsbereich des DaV
   * 
   * @param connection
   * @return Konfigurationsbereich
   */
  private ConfigurationArea bestimmeDefaultBereich(ClientDavInterface connection)
  {
    Data data = connection.getDataModel().getConfigurationAuthority().getConfigurationData(
        connection.getDataModel().getAttributeGroup("atg.konfigurationsVerantwortlicherEigenschaften"));

    if (data != null)
    {
      TextArray array = data.getTextArray("defaultBereich");

      if (array.getLength() == 1)
      {
        return (ConfigurationArea)connection.getDataModel().getObject(array.getText(0));
      }
    }

    return null;
  }
  
  /**
   * Liefert den Default-Konfigurationsbereich des DaV
   * 
   * @param dataModel DatenModell
   * @return Konfigurationsbereich
   */
  private ConfigurationArea getDefaultBereich( DataModel dataModel )
  {
    return dataModel.getConfigurationArea( "kb.default.aoe.nw.uz.koeln" );
  }
  
  /**
   * Liefert das 1.KonfigurationsObjekt zur Menge der UrsachenEinheiten des DaV
   * 
   * @param dataModel DatenModell
   * @return 1.KonfigurationsObjekt
   */
  private ConfigurationObject getFirstKonfigObjekt( DataModel dataModel )
  {
    List<SystemObject> listStrecken = dataModel.getType( "typ.streckenBeeinflussungsAnlageBereich" ).getElements();
   
    for (SystemObject systemObject : listStrecken)
    {
      return (ConfigurationObject)systemObject;
    }
    return null;
  }
  
  /**
   * Liefert alle KonfigurationsObjekte zur Menge der UrsachenEinheiten des DaV
   * 
   * @param dataModel DatenModell
   * @return alle KonfigurationsObjekte
   */
  public List<SystemObject> getAllKonfigObjekte()
  {
    DataModel dataModel = _connection.getDataModel();
       
    return dataModel.getType( "typ.streckenBeeinflussungsAnlage" ).getElements();
    
//    return dataModel.getType( "typ.streckenBeeinflussungsAnlageBereich" ).getElements();
//
//    List<ConfigurationObject> listKonfigObj= new ArrayList<ConfigurationObject>();
//    for (SystemObject systemObject : listStrecken)
//    {
//      ConfigurationObject konfigObj = (ConfigurationObject)systemObject;
//      listKonfigObj.add( konfigObj );
//    }
//    return listKonfigObj;
  }
  
  /**
   * Prft, ob das SystemObjekt zur Pid vorhanden ist
   * 
   * @param pid
   *          Pid des Objektes
   * @return  Liefert <code>true</code>, wenn das SystemObjekt zur Pid vorhanden ist
   *          und <code>false</code>, wenn nicht.
   */
  public boolean isObjektVorhanden( String pid )
  {

    SystemObject object = _connection.getDataModel().getObject( pid );

    if ( object != null )
      return true;

    return false;
  }
  
  /**
   * Liefert die Pid zur Parameter-Attributgruppe zu der bergebenen Pid
   * 
   * @param pid
   *          Pid des Objektes
   * @return  Pid der Parameter-Attributgruppe
   */
  public String getParameterAttributgruppe( String pid )
  {

    DataModel dataModel = _connection.getDataModel();
    SystemObject object = dataModel.getObject( pid );

    if ( object != null )
    {
      SystemObjectType type = object.getType();
      if ( type != null )
      {
        List<AttributeGroup> attributeGroups = type.getAttributeGroups();
        for (AttributeGroup attGroup : attributeGroups)
        {
          if( attGroup.getAspects().contains( dataModel.getAspect( "asp.parameterIst" ) ) )
            return attGroup.getPid();        
        }
      }
    } 

    return null;
  }
  
  /**
   * Methode liest fr das Objekt mit der PID objPid die Attibutgruppe atgPid 
   * unter dem Aspekt aspPid direkt vom DaV und ruft sofort die Auswertemethode auf
   * (nicht ber den Thread) 
   * @param objPid Pid des Objekts 
   * @param atgPid Attributgruppe die angemeldet werden soll
   * @param aspPid Apekt der angemeldet werden soll
   */
  public ResultData leseDatenDirektVonDaV (String objPid, String atgPid, String aspPid)
  {
    ClientDavInterface verb = _connection;

    AttributeGroup atg = verb.getDataModel().getAttributeGroup(atgPid);

    Aspect asp = verb.getDataModel().getAspect(aspPid);

    DataDescription dd = new DataDescription(atg, asp);

    SystemObject obj = verb.getDataModel().getObject(objPid); 

    try
    {
      ResultData data = verb.getData( obj, dd, 10000 );
      
      ResultData[] results = new ResultData [] {data};
      
      if (results.length == 1)
        return results [0];
    }
    catch (Exception e) 
    {
      e.printStackTrace();
    }
    
    return null;
  }
  
  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet 
   * zum Lesen der Attibutgruppe atgPid unter dem Aspekt aspPid
   * und der Rolle des normalen Empfngers.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   */
  public void anmeldeDaten(String objPid, String atgPid, String aspPid)
  {
    anmeldeLeseDaV(objPid, atgPid, aspPid, ReceiverRole.receiver());

  }

  /**
   * Methode die die Liste der Objekte objs beim Datenverteiler anmeldet 
   * zum Lesen der Attibutgruppe atgPid unter dem Aspekt aspPid
   * und der Rolle des normalen Empfngers.
   * 
   * @param objs
   *          Liste der Pids der Objekte
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   */
  public void anmeldeDaten(List<SystemObject> objs, String atgPid, String aspPid)
  {
    anmeldeLeseDaV(objs, atgPid, aspPid, ReceiverRole.receiver());
    
  }

}
