/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.parameter.tsf.common;


import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.WorkbenchPage;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.parameter.tsf.Activator;
import de.bsvrz.buv.plugin.parameter.tsf.DaVKommunikation;
import de.bsvrz.buv.plugin.parameter.tsf.ablauf.TsfAblaufViewParameter;
import de.bsvrz.buv.plugin.parameter.tsf.mqZuordnung.MqZuweisungViewParameter;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Nach Auswahl des Menpunkts "Automatikprogramme/Parameter/Harmonisierung" in der Rahmenapplikation 
 * ffnet sich das Fenster zur Auswahl der Objekte zur Harmonisierung.
 * 
 * @author Krause
 *
 */
public class TsfAbschnittAuswahlViewParameter extends ViewPart //implements IDaVDataUpdateListener
{

  public static String ID = "de.bsvrz.buv.plugin.parameter.tsf.common.viewTsfAbschnittAuswahlParameter";

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();
  
  /**
   * Objekttypen des DAV, deren Objekte angezeigt werden sollen
   */
  private static final String[] _objekttypen = {
    "typ.sswSbaTsfAbschnitt" 
  };
 
  private Composite _parent;
  
  private Composite _compositeTop;
  
  private Composite _compositeRight;

  private Group _gruppeUebersicht;

  private Table _tabelleObjekte;

  private TableViewer _tableViewerObjekte;

  private ISelectionChangedListener _selectionChangedListener;

  private Composite _compositeBottom;

//  private Button _buttonOk;
  private Button _buttonTsfAblauf;
//  private Button _buttonMqZuordnung;

  private Button _buttonAbbrechen;

  private Button _buttonHilfe;
  
  private ClientDavInterface _connection;

  @Override
  public void createPartControl(Composite parent)
  {   
    _parent = parent;

    //Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp( _parent, Activator.PLUGIN_ID + ".tsfBereichParameterAuswahl" );

    parent.setLayout(new GridLayout());
    createCompositeTop(parent);
    createCompositeRight();
    createCompositeBottom(parent);
    _connection = DaVKommunikation.getInstanz().holeDaVVerbindung();

    List<SystemObject> list = new ArrayList<SystemObject>();
    
    for (int i = 0; i < _objekttypen.length; i++)
    {
      SystemObjectType type = _connection.getDataModel().getType( _objekttypen[i] );
      List<SystemObject> objs = type.getElements();

      list.addAll(objs);
    }
    
    Map<String, SystemObject> treeMap = new TreeMap<String, SystemObject>();
    for (SystemObject so : list)
    {
      treeMap.put(so.getName(), so);
    }

    List<SystemObject> sortList = new ArrayList<SystemObject>();
    sortList.addAll(treeMap.values());

    _tableViewerObjekte.setInput(sortList);

  }

  @Override
  public void setFocus()
  {
    //Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();

  }

  /**
   * Erzeugt die eigentliche AuswahlView zur bersicht der Objekte zur Gruppe
   * 
   */
  private void createCompositeTop(Composite parent)
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 2;
    _compositeTop = new Composite(parent, SWT.NONE);
    _compositeTop.setLayout(gridLayout);
    _compositeTop.setLayoutData(gridData);
    GridData gridDataTabelle = new GridData();
    gridDataTabelle.horizontalAlignment = GridData.FILL;
    gridDataTabelle.grabExcessHorizontalSpace = true;
    gridDataTabelle.grabExcessVerticalSpace = true;
    gridDataTabelle.verticalAlignment = GridData.FILL;
    GridData gridDataGruppeUebersicht = new GridData();
    gridDataGruppeUebersicht.horizontalAlignment = GridData.FILL;
    gridDataGruppeUebersicht.grabExcessHorizontalSpace = true;
    gridDataGruppeUebersicht.grabExcessVerticalSpace = true;
    gridDataGruppeUebersicht.verticalAlignment = GridData.FILL;
    
    _gruppeUebersicht = new Group(_compositeTop, SWT.NONE);
    _gruppeUebersicht.setLayout(new GridLayout());
    _gruppeUebersicht.setLayoutData(gridDataGruppeUebersicht);
    _gruppeUebersicht.setText("TSF-Abschnitte:");
    
    _tabelleObjekte = new Table(_gruppeUebersicht, SWT.BORDER | SWT.MULTI | SWT.FULL_SELECTION);
    _tabelleObjekte.setHeaderVisible(true);
    _tabelleObjekte.setLayoutData(gridDataTabelle);
    _tabelleObjekte.setLinesVisible(true);
    _tableViewerObjekte = new TableViewer(_tabelleObjekte);

    TableColumn tcKey = new TableColumn(_tabelleObjekte, SWT.LEFT);
    tcKey.setText("Name:");
    tcKey.setWidth(200);

    TableColumn tcValue = new TableColumn(_tabelleObjekte, SWT.LEFT);
    tcValue.setText("Pid:");
    tcValue.setWidth(320);

    _tableViewerObjekte.setContentProvider(new ContentProvider());
    _tableViewerObjekte.setLabelProvider(new LabelProvider());

    _selectionChangedListener = new ISelectionChangedListener() {

      public void selectionChanged(SelectionChangedEvent event)
      {
        IStructuredSelection selection = (IStructuredSelection)event.getSelection();
        
        if (selection == null || selection.isEmpty()) {
        	_buttonTsfAblauf.setEnabled(false);
//        	_buttonMqZuordnung.setEnabled(false);
        }
        else {
        	_buttonTsfAblauf.setEnabled(true);
//        	_buttonMqZuordnung.setEnabled(true);
        }
        
        Object firstElement = selection.getFirstElement();

        if (firstElement instanceof SystemObject)
        {

        }
      }

    };
    _tableViewerObjekte.addSelectionChangedListener(_selectionChangedListener);
//    _tableViewerObjekte.addDoubleClickListener(new IDoubleClickListener() {
//
//      @Override
//      public void doubleClick(DoubleClickEvent event)
//      {
//        createViewFromSelection();       
//      }
//
//
//    });
  }
  
  /**
   * Aufruf der ViewParameter fr die selektierten Objekte 
   * 
   */
  private void createViewFromSelection(String viewId, int width, int height)
  {
    IStructuredSelection selection = (IStructuredSelection)_tableViewerObjekte.getSelection();
    
    Object firstElement = selection.getFirstElement();
    if (firstElement instanceof SystemObject)
    {      
        Object[] array = selection.toArray();
        List<SystemObject> listObj = new ArrayList<SystemObject>();
        for (Object object : array)
        {
          SystemObject so = (SystemObject)object;
          listObj.add(so);
        }
        
        IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
        IWorkbenchPage page = dw.getActivePage();
        try
        {
          if (listObj.size() > 0){
            
            StringBuffer sb = new StringBuffer();
            for (int i = 0; i < listObj.size(); i++)
            {
              SystemObject so = listObj.get(i);
              if ( i < listObj.size()-1 )
                sb.append(so.getPid() + ";");
              else
                sb.append(so.getPid());
              
            }
            
            page.showView(viewId, sb.toString(), IWorkbenchPage.VIEW_VISIBLE);
            IViewReference ref = page.findViewReference(viewId, sb.toString()); 
            if (ref != null){
              System.out.println("createViewFromSelection(): " + sb.toString());
              ((WorkbenchPage)page).getActivePerspective().getPresentation().detachPart(ref);
              IWorkbenchPart wbPart = ref.getPart(true);
              Shell shell = wbPart.getSite().getShell();
              shell.setSize(width, height);              
            }                        
          }
          
        }
        catch (PartInitException e1)
        {
          // TODO Auto-generated catch block
          e1.printStackTrace();
        }
        
        
    }
  }

  private void createCompositeBottom(Composite parent)
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    _compositeBottom = new Composite(parent, SWT.NONE);
    _compositeBottom.setLayoutData(gridData3);
    _compositeBottom.setLayout(rowLayout);
//    _buttonOk = new Button(_compositeBottom, SWT.NONE);
//    _buttonOk.setText("Ok");
//    _buttonOk.addMouseListener(new MouseAdapter() {
//
//      @Override
//      public void mouseDown(MouseEvent e)
//      {
//         createViewFromSelection();       
//      }
//   
//
//    });

    _buttonAbbrechen = new Button(_compositeBottom, SWT.NONE);
    _buttonAbbrechen.setText("Abbrechen");
    _buttonAbbrechen.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionAbbrechen();
      }

    });
    
    _buttonHilfe = new Button(_compositeBottom, SWT.NONE);
    _buttonHilfe.setText("Hilfe");
    _buttonHilfe.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionHilfe();
      }

    });
  }

  protected void aktionAbbrechen()
  {
    PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().hideView(this);
  }

  /**
   * Zeigt die Hilfe zum Dialog an.
   * 
   */
  protected void aktionHilfe()
  {
    //String contextId = Activator.PLUGIN_ID + ".automatikprogrammeParameterAuswahl";
    String contextId = Activator.PLUGIN_ID + ".tsfBereichParameterAuswahl";
    _debug.fine("ContextID: " + contextId);
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  private class ContentProvider implements IStructuredContentProvider
  {
    @Override
    public Object[] getElements(Object inputElement)
    {
      java.util.List list = (java.util.List)inputElement;

      Object[] re = new Object[list.size()];

      for (int i = 0; i < list.size(); i++)
      {
        re[i] = list.get(i);
      }
      return re;
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }

  }

  private class LabelProvider implements ITableLabelProvider
  {

    @Override
    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    @Override
    public String getColumnText(Object element, int columnIndex)
    {
      String returnString = null;

      switch (columnIndex)
      {
        case 0:
          returnString = ((SystemObject)element).getName();
           break;

        case 1:
          returnString = ((SystemObject)element).getPid();
          break;

        default:
          returnString = "default";
      }

      return returnString;

    }

    @Override
    public void addListener(ILabelProviderListener listener)
    {
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    @Override
    public void removeListener(ILabelProviderListener listener)
    {
    }
  }
  
  /**
   * This method initializes _compositeRight
   * 
   */
  private void createCompositeRight()
  {
    RowLayout rowLayout2 = new RowLayout();
    rowLayout2.type = org.eclipse.swt.SWT.VERTICAL;
    rowLayout2.wrap = false;
    rowLayout2.pack = false;
    GridData gridData6 = new GridData();
    gridData6.horizontalAlignment = GridData.CENTER;
    gridData6.grabExcessVerticalSpace = true;
    gridData6.verticalAlignment = GridData.FILL;
    
    gridData6.verticalIndent = 3;
    
    _compositeRight = new Composite(_compositeTop, SWT.NONE);
    _compositeRight.setLayoutData(gridData6);
    _compositeRight.setLayout(rowLayout2);
    
    _buttonTsfAblauf = new Button(_compositeRight, SWT.NONE);
    _buttonTsfAblauf.setText("Parameter Ablauf TSF");
    _buttonTsfAblauf.addMouseListener(new MouseAdapter() {
		
		@Override
		public void mouseDown(MouseEvent e)
		{
			if (_tabelleObjekte.getSelectionCount() > 0) {
				createViewFromSelection(TsfAblaufViewParameter.ID, 1180, 900);				
			}
			else {
				MessageBox mb1 = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
			      mb1.setText( "Fehler" );
			      mb1.setMessage( "Es ist kein TSF-Abschnitt selektiert!" );
			      mb1.open();	
			}
		}
	
	  });
    _buttonTsfAblauf.setEnabled(false);
	    
//    _buttonMqZuordnung = new Button(_compositeRight, SWT.NONE);
//    _buttonMqZuordnung.setText("Parameter MQ-Zuordnung");
//    _buttonMqZuordnung.addMouseListener(new MouseAdapter() {
//		
//		@Override
//		public void mouseDown(MouseEvent e)
//		{
//			if (_tabelleObjekte.getSelectionCount() > 0) {
//				   createViewFromSelection(MqZuweisungViewParameter.ID, 500, 280);				
//			}
//			else {
//				MessageBox mb1 = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
//			      mb1.setText( "Fehler" );
//			      mb1.setMessage( "Es ist kein TSF-Abschnitt selektiert!" );
//			      mb1.open();	
//			}
//		}
//	
//	  });
//    _buttonMqZuordnung.setEnabled(false);
   
  }

}
