/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.pua.a8.projektInfo;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bsvrz.buv.plugin.pua.basis.expose.IPIGanglinienKonvertierung;
import de.bsvrz.buv.plugin.pua.basis.expose.IProjektInfo;

public class ProjektInfoAqBefehle implements IPIGanglinienKonvertierung
{
  protected int SPALTE_BLINKER_LINKS = 0;
  protected int SPALTE_TSF_LINKS = 1;
  protected int SPALTE_BAQ_LINKS = 1;
  protected int SPALTE_ATYP_4UEFS = 2;
  protected int SPALTE_BTYP_4UEFS_3UEFS = 3;
  protected int SPALTE_CTYP_4UEFS_3UEFS = 4;
  protected int SPALTE_ATYP_3UEFS = 5;
  protected int SPALTE_BTYP_3UEFS_2UEFS = 6;
  protected int SPALTE_CTYP_3UEFS_2UEFS = 7;
  protected int SPALTE_ATYP_2UEFS = 8;
  protected int SPALTE_BTYP_2UEFS_1UEFS = 9;
  protected int SPALTE_CTYP_2UEFS_1UEFS = 10;
  protected int SPALTE_BLINKER_WAQ = 10;
  protected int SPALTE_ATYP_1UEFS = 11;
  protected int SPALTE_WAQ = 11;
  protected int SPALTE_BTYP_1UEFS_HFS = 12;
  protected int SPALTE_CTYP_1UEFS_HFS = 13;
  protected int SPALTE_ATYP_HFS = 14;
  protected int SPALTE_ATYP_1UEFS_EIN = 15;
  protected int SPALTE_BTYP_1UEFS_HFS_EIN = 16;
  protected int SPALTE_CTYP_1UEFS_HFS_EIN = 17;
  protected int SPALTE_DTYP = 18;
  protected int SPALTE_ATYP_HFS_EIN = 19;
  protected int SPALTE_ATYP_TSF = 19;
  protected int SPALTE_TSF_RECHTS = 20;
  protected int SPALTE_BAQ_RECHTS = 20;
  protected int SPALTE_BLINKER_RECHTS = 21;
  protected int SPALTE_DZEILE = 22;
  
	protected List<String> _ausgabeTabelleSpalten = new ArrayList<String>() 
	{
		{
//      add("A-Typ (4FS)");
//      add("B-Typ (4FS/3FS)");
//      add("C-Typ (4FS/3FS)");
//      
//      add("A-Typ (3FS)");
//      add("B-Typ (3FS/2FS)");
//      add("C-Typ (3FS/2FS)");
//      
//      add("A-Typ (2FS)");
//      add("B-Typ (2FS/1FS)");
//      add("C-Typ (2FS/1FS)");
//      
//      add("A-Typ (1FS)");
//      add("B-Typ (1FS/HFS)");
//      add("C-Typ (1FS/HFS)");
//      
//      add("A-Typ (HFS)");
      
      // SBA A8 Gesamt

      add("Blinker links"); // 0
      add("TSF/BAQ links"); // 1

      add("A-Typ (4FS)"); // 2
      add("B-Typ (4FS/3FS)"); // 3
      add("C-Typ (4FS/3FS)"); // 4
      
      add("A-Typ (3FS)"); // 5
      add("B-Typ (3FS/2FS)"); // 6
      add("C-Typ (3FS/2FS)"); // 7
      
      add("A-Typ (2FS)"); // 8
      add("B-Typ (2FS/1FS)"); // 9
      add("C-Typ (2FS/1FS) / Bl. WAQ"); // 10
      
      add("A-Typ (1FS) / WAQ"); // 11
      add("B-Typ (1FS/HFS)"); // 12
      add("C-Typ (1FS/HFS)"); //13
      
      add("A-Typ (HFS)"); //14

      add("A-Typ (1FS Ein)"); // 15
      add("B-Typ (1FS/HFS Ein)"); // 16
      add("C-Typ (1FS/HFS Ein)"); //17

      add("d-Typ (TSF)"); //18

      add("A-Typ (TSF / HFS Ein)"); //19

      add("TSF/BAQ rechts"); // 20
      add("Blinker rechts"); // 21

      add("D-Zeile"); // 22
		}
	};
	
	protected Map<String, Integer> _spaltenIndexRelativ = new HashMap<String, Integer>() 
	{
		{

		  // KA01
      put("anzeige.A8.6101.HFB.W.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6101.HFB.W.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6101.HFB.W.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6101.HFB.W.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6101.HFB.W.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6101.HFB.W.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6101.HFB.W.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6101.HFB.W.anzeige8", SPALTE_DZEILE);

      // KA02
      put("anzeige.A8.6103.HFB.W.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6103.HFB.W.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6103.HFB.W.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6103.HFB.W.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6103.HFB.W.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6103.HFB.W.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6103.HFB.W.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);

      // KA03
      put("anzeige.A8.6104.HFB.W.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6104.HFB.W.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6104.HFB.W.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6104.HFB.W.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6104.HFB.W.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6104.HFB.W.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6104.HFB.W.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6104.HFB.W.anzeige8", SPALTE_DZEILE);

      // KA04
      put("anzeige.A8.6106.HFB.W.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6106.HFB.W.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6106.HFB.W.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6106.HFB.W.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6106.HFB.W.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6106.HFB.W.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6106.HFB.W.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);

      // KA05
      put("anzeige.A8.6107.HFB.W.anzeige1", SPALTE_ATYP_HFS_EIN);
      put("anzeige.A8.6107.HFB.W.anzeige2", SPALTE_ATYP_HFS);
      put("anzeige.A8.6107.HFB.W.anzeige3", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6107.HFB.W.anzeige4", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6107.HFB.W.anzeige5", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6107.HFB.W.anzeige6", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6107.HFB.W.anzeige7", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6107.HFB.W.anzeige8", SPALTE_CTYP_2UEFS_1UEFS);

      // KA06
      put("anzeige.A8.6108.HFB.W.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6108.HFB.W.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6108.HFB.W.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6108.HFB.W.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6108.HFB.W.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6108.HFB.W.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6108.HFB.W.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);

      // KA07
      put("anzeige.A8.6109.HFB.W.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6109.HFB.W.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6109.HFB.W.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6109.HFB.W.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6109.HFB.W.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6109.HFB.W.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6109.HFB.W.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6109.HFB.W.anzeige8", SPALTE_DZEILE);

      // KA08
      put("anzeige.A8.6110.HFB.W.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6110.HFB.W.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6110.HFB.W.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6110.HFB.W.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6110.HFB.W.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6110.HFB.W.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6110.HFB.W.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);

      // KA09
      put("anzeige.A8.6111.HFB.W.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6111.HFB.W.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6111.HFB.W.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6111.HFB.W.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6111.HFB.W.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6111.HFB.W.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6111.HFB.W.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);

      // KA10
      put("anzeige.A8.6112.HFB.W.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6112.HFB.W.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6112.HFB.W.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6112.HFB.W.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6112.HFB.W.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6112.HFB.W.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6112.HFB.W.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);

      // KA11     
      put("anzeige.A8.6113.HFB.W.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6113.HFB.W.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6113.HFB.W.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6113.HFB.W.anzeige4", SPALTE_ATYP_3UEFS);
      put("anzeige.A8.6113.HFB.W.anzeige5", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6113.HFB.W.anzeige6", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6113.HFB.W.anzeige7", SPALTE_BTYP_3UEFS_2UEFS);
      put("anzeige.A8.6113.HFB.W.anzeige8", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6113.HFB.W.anzeige9", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6113.HFB.W.anzeige10", SPALTE_CTYP_3UEFS_2UEFS);
      put("anzeige.A8.6113.HFB.W.anzeige11", SPALTE_DZEILE);

      // KA12     
      put("anzeige.A8.6114.HFB.W.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6114.HFB.W.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6114.HFB.W.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6114.HFB.W.anzeige4", SPALTE_ATYP_3UEFS);
      put("anzeige.A8.6114.HFB.W.anzeige5", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6114.HFB.W.anzeige6", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6114.HFB.W.anzeige7", SPALTE_BTYP_3UEFS_2UEFS);
      put("anzeige.A8.6114.HFB.W.anzeige8", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6114.HFB.W.anzeige9", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6114.HFB.W.anzeige10", SPALTE_CTYP_3UEFS_2UEFS);
      put("anzeige.A8.6114.HFB.W.anzeige11", SPALTE_DZEILE);

      // KA13
      put("anzeige.A8.6115.HFB.W.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6115.HFB.W.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6115.HFB.W.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6115.HFB.W.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6115.HFB.W.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6115.HFB.W.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6115.HFB.W.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);

      // KA13E
      put("anzeige.A8.6115.EIN.W.anzeige1", SPALTE_ATYP_HFS);

      // KA14
      put("anzeige.A8.6116.HFB.W.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6116.HFB.W.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6116.HFB.W.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6116.HFB.W.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6116.HFB.W.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6116.HFB.W.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6116.HFB.W.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);

      // KA15
      put("anzeige.A8.6117.HFB.W.anzeige1", SPALTE_ATYP_HFS_EIN);
      put("anzeige.A8.6117.HFB.W.anzeige2", SPALTE_ATYP_1UEFS_EIN);
      put("anzeige.A8.6117.HFB.W.anzeige3", SPALTE_ATYP_HFS);
      put("anzeige.A8.6117.HFB.W.anzeige4", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6117.HFB.W.anzeige5", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6117.HFB.W.anzeige6", SPALTE_BTYP_1UEFS_HFS_EIN);
      put("anzeige.A8.6117.HFB.W.anzeige7", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6117.HFB.W.anzeige8", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6117.HFB.W.anzeige9", SPALTE_CTYP_1UEFS_HFS_EIN);
      put("anzeige.A8.6117.HFB.W.anzeige10", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6117.HFB.W.anzeige11", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6117.HFB.W.anzeige12", SPALTE_DZEILE);

      // KA16     
      put("anzeige.A8.6118.HFB.W.anzeige1", SPALTE_BLINKER_RECHTS);
      put("anzeige.A8.6118.HFB.W.anzeige2", SPALTE_BLINKER_LINKS);
      put("anzeige.A8.6118.HFB.W.anzeige3", SPALTE_BLINKER_WAQ);
      put("anzeige.A8.6118.HFB.W.anzeige4", SPALTE_TSF_RECHTS);
      put("anzeige.A8.6118.HFB.W.anzeige5", SPALTE_TSF_LINKS);
      put("anzeige.A8.6118.HFB.W.anzeige6", SPALTE_WAQ);

      // KA17     
      put("anzeige.A8.6122.HFB.W.anzeige1", SPALTE_ATYP_TSF);
      put("anzeige.A8.6122.HFB.W.anzeige2", SPALTE_ATYP_HFS);
      put("anzeige.A8.6122.HFB.W.anzeige3", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6122.HFB.W.anzeige4", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6122.HFB.W.anzeige5", SPALTE_BTYP_1UEFS_HFS_EIN);
      put("anzeige.A8.6122.HFB.W.anzeige6", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6122.HFB.W.anzeige7", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6122.HFB.W.anzeige8", SPALTE_CTYP_1UEFS_HFS_EIN);
      put("anzeige.A8.6122.HFB.W.anzeige9", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6122.HFB.W.anzeige10", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6122.HFB.W.anzeige11", SPALTE_DTYP);
      put("anzeige.A8.6122.HFB.W.anzeige12", SPALTE_TSF_RECHTS);
      put("anzeige.A8.6122.HFB.W.anzeige13", SPALTE_TSF_LINKS);
      
      // KA18     
      put("anzeige.A8.6125.HFB.W.anzeige1", SPALTE_ATYP_TSF);
      put("anzeige.A8.6125.HFB.W.anzeige2", SPALTE_ATYP_HFS);
      put("anzeige.A8.6125.HFB.W.anzeige3", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6125.HFB.W.anzeige4", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6125.HFB.W.anzeige5", SPALTE_BTYP_1UEFS_HFS_EIN);
      put("anzeige.A8.6125.HFB.W.anzeige6", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6125.HFB.W.anzeige7", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6125.HFB.W.anzeige8", SPALTE_CTYP_1UEFS_HFS_EIN);
      put("anzeige.A8.6125.HFB.W.anzeige9", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6125.HFB.W.anzeige10", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6125.HFB.W.anzeige11", SPALTE_DTYP);
      put("anzeige.A8.6125.HFB.W.anzeige12", SPALTE_TSF_RECHTS);
      put("anzeige.A8.6125.HFB.W.anzeige13", SPALTE_TSF_LINKS);
      
      // KA19     
      put("anzeige.A8.6126.HFB.W.anzeige1", SPALTE_BLINKER_RECHTS);
      put("anzeige.A8.6126.HFB.W.anzeige2", SPALTE_BLINKER_LINKS);
      put("anzeige.A8.6126.HFB.W.anzeige3", SPALTE_TSF_RECHTS);
      put("anzeige.A8.6126.HFB.W.anzeige4", SPALTE_TSF_LINKS);
      
      // KA20     
      put("anzeige.A8.6127.HFB.W.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6127.HFB.W.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6127.HFB.W.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6127.HFB.W.anzeige4", SPALTE_ATYP_3UEFS);
      put("anzeige.A8.6127.HFB.W.anzeige5", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6127.HFB.W.anzeige6", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6127.HFB.W.anzeige7", SPALTE_BTYP_3UEFS_2UEFS);
      put("anzeige.A8.6127.HFB.W.anzeige8", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6127.HFB.W.anzeige9", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6127.HFB.W.anzeige10", SPALTE_CTYP_3UEFS_2UEFS);
      put("anzeige.A8.6127.HFB.W.anzeige11", SPALTE_DZEILE);

      // KA21
      put("anzeige.A8.6128.HFB.W.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6128.HFB.W.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6128.HFB.W.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6128.HFB.W.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6128.HFB.W.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6128.HFB.W.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6128.HFB.W.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);

      // KA21E
      put("anzeige.A8.6128.EIN.W.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6128.EIN.W.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6128.EIN.W.anzeige3", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6128.EIN.W.anzeige4", SPALTE_CTYP_1UEFS_HFS);
      
      // KA22
      put("anzeige.A8.6130.HFB.W.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6130.HFB.W.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6130.HFB.W.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6130.HFB.W.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6130.HFB.W.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6130.HFB.W.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6130.HFB.W.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6130.HFB.W.anzeige8", SPALTE_DZEILE);

      // KA23
      put("anzeige.A8.6131.HFB.W.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6131.HFB.W.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6131.HFB.W.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6131.HFB.W.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6131.HFB.W.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6131.HFB.W.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6131.HFB.W.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);
      
      // KA24
      put("anzeige.A8.6132.HFB.W.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6132.HFB.W.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6132.HFB.W.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6132.HFB.W.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6132.HFB.W.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6132.HFB.W.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6132.HFB.W.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);
      
      // KA25
      put("anzeige.A8.6133.HFB.W.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6133.HFB.W.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6133.HFB.W.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6133.HFB.W.anzeige4", SPALTE_ATYP_3UEFS);
      put("anzeige.A8.6133.HFB.W.anzeige5", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6133.HFB.W.anzeige6", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6133.HFB.W.anzeige7", SPALTE_BTYP_3UEFS_2UEFS);
      put("anzeige.A8.6133.HFB.W.anzeige8", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6133.HFB.W.anzeige9", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6133.HFB.W.anzeige10", SPALTE_CTYP_3UEFS_2UEFS);
      put("anzeige.A8.6133.HFB.W.anzeige11", SPALTE_DZEILE);
      
      // M01
      put("anzeige.A8.6053.HFB.O.anzeige1", SPALTE_ATYP_HFS_EIN);
      put("anzeige.A8.6053.HFB.O.anzeige2", SPALTE_ATYP_HFS);
      put("anzeige.A8.6053.HFB.O.anzeige3", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6053.HFB.O.anzeige4", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6053.HFB.O.anzeige5", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6053.HFB.O.anzeige6", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6053.HFB.O.anzeige7", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6053.HFB.O.anzeige8", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6053.HFB.O.anzeige9", SPALTE_DZEILE);
      
      // M02
      put("anzeige.A8.6052.HFB.O.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6052.HFB.O.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6052.HFB.O.anzeige3", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6052.HFB.O.anzeige4", SPALTE_CTYP_1UEFS_HFS);
      
      // M03     
      put("anzeige.A8.6051.HFB.O.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6051.HFB.O.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6051.HFB.O.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6051.HFB.O.anzeige4", SPALTE_ATYP_4UEFS);
      put("anzeige.A8.6051.HFB.O.anzeige5", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6051.HFB.O.anzeige6", SPALTE_BTYP_3UEFS_2UEFS);
      put("anzeige.A8.6051.HFB.O.anzeige7", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6051.HFB.O.anzeige8", SPALTE_CTYP_3UEFS_2UEFS);

      // M04     
      put("anzeige.A8.6050.HFB.O.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6050.HFB.O.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6050.HFB.O.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6050.HFB.O.anzeige4", SPALTE_ATYP_3UEFS);
      put("anzeige.A8.6050.HFB.O.anzeige5", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6050.HFB.O.anzeige6", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6050.HFB.O.anzeige7", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6050.HFB.O.anzeige8", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6050.HFB.O.anzeige9", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6050.HFB.O.anzeige10", SPALTE_CTYP_3UEFS_2UEFS);
      put("anzeige.A8.6050.HFB.O.anzeige11", SPALTE_DZEILE);

      // M05     
      put("anzeige.A8.6049.HFB.O.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6049.HFB.O.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6049.HFB.O.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6049.HFB.O.anzeige4", SPALTE_ATYP_3UEFS);
      put("anzeige.A8.6049.HFB.O.anzeige5", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6049.HFB.O.anzeige6", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6049.HFB.O.anzeige7", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6049.HFB.O.anzeige8", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6049.HFB.O.anzeige9", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6049.HFB.O.anzeige10", SPALTE_CTYP_3UEFS_2UEFS);

      // M06     
      put("anzeige.A8.6048.HFB.O.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6048.HFB.O.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6048.HFB.O.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6048.HFB.O.anzeige4", SPALTE_ATYP_3UEFS);
      put("anzeige.A8.6048.HFB.O.anzeige5", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6048.HFB.O.anzeige6", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6048.HFB.O.anzeige7", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6048.HFB.O.anzeige8", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6048.HFB.O.anzeige9", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6048.HFB.O.anzeige10", SPALTE_CTYP_3UEFS_2UEFS);
      put("anzeige.A8.6048.HFB.O.anzeige11", SPALTE_DZEILE);

      // M06E     
      put("anzeige.A8.6048.EIN.O.anzeige1", SPALTE_ATYP_HFS);

      // M07     
      put("anzeige.A8.6047.HFB.O.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6047.HFB.O.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6047.HFB.O.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6047.HFB.O.anzeige4", SPALTE_ATYP_3UEFS);
      put("anzeige.A8.6047.HFB.O.anzeige5", SPALTE_ATYP_4UEFS);
      put("anzeige.A8.6047.HFB.O.anzeige6", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6047.HFB.O.anzeige7", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6047.HFB.O.anzeige8", SPALTE_BTYP_3UEFS_2UEFS);
      put("anzeige.A8.6047.HFB.O.anzeige9", SPALTE_BTYP_4UEFS_3UEFS);
      put("anzeige.A8.6047.HFB.O.anzeige10", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6047.HFB.O.anzeige11", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6047.HFB.O.anzeige12", SPALTE_CTYP_3UEFS_2UEFS);
      put("anzeige.A8.6047.HFB.O.anzeige13", SPALTE_CTYP_4UEFS_3UEFS);
      put("anzeige.A8.6047.HFB.O.anzeige14", SPALTE_DZEILE);

      // M08     
      put("anzeige.A8.6046.HFB.O.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6046.HFB.O.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6046.HFB.O.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6046.HFB.O.anzeige4", SPALTE_ATYP_3UEFS);
      put("anzeige.A8.6046.HFB.O.anzeige5", SPALTE_ATYP_4UEFS);
      put("anzeige.A8.6046.HFB.O.anzeige6", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6046.HFB.O.anzeige7", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6046.HFB.O.anzeige8", SPALTE_BTYP_3UEFS_2UEFS);
      put("anzeige.A8.6046.HFB.O.anzeige9", SPALTE_BTYP_4UEFS_3UEFS);
      put("anzeige.A8.6046.HFB.O.anzeige10", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6046.HFB.O.anzeige11", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6046.HFB.O.anzeige12", SPALTE_CTYP_3UEFS_2UEFS);
      put("anzeige.A8.6046.HFB.O.anzeige13", SPALTE_CTYP_4UEFS_3UEFS);
      put("anzeige.A8.6046.HFB.O.anzeige14", SPALTE_DZEILE);

      // M09     
      put("anzeige.A8.6044.HFB.O.anzeige1", SPALTE_BLINKER_RECHTS);
      put("anzeige.A8.6044.HFB.O.anzeige2", SPALTE_BLINKER_LINKS);
      put("anzeige.A8.6044.HFB.O.anzeige3", SPALTE_BLINKER_WAQ);
      put("anzeige.A8.6044.HFB.O.anzeige4", SPALTE_TSF_RECHTS);
      put("anzeige.A8.6044.HFB.O.anzeige5", SPALTE_TSF_LINKS);
      put("anzeige.A8.6044.HFB.O.anzeige6", SPALTE_WAQ);
      
      // M10     
      put("anzeige.A8.6041.HFB.O.anzeige1", SPALTE_ATYP_TSF);
      put("anzeige.A8.6041.HFB.O.anzeige2", SPALTE_ATYP_HFS);
      put("anzeige.A8.6041.HFB.O.anzeige3", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6041.HFB.O.anzeige4", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6041.HFB.O.anzeige5", SPALTE_BTYP_1UEFS_HFS_EIN);
      put("anzeige.A8.6041.HFB.O.anzeige6", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6041.HFB.O.anzeige7", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6041.HFB.O.anzeige8", SPALTE_CTYP_1UEFS_HFS_EIN);
      put("anzeige.A8.6041.HFB.O.anzeige9", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6041.HFB.O.anzeige10", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6041.HFB.O.anzeige11", SPALTE_DTYP);
      put("anzeige.A8.6041.HFB.O.anzeige12", SPALTE_TSF_RECHTS);
      put("anzeige.A8.6041.HFB.O.anzeige13", SPALTE_TSF_LINKS);
      
      // M11     
      put("anzeige.A8.6039.HFB.O.anzeige1", SPALTE_BLINKER_WAQ);
      put("anzeige.A8.6039.HFB.O.anzeige2", SPALTE_WAQ);
      
      // M12     
      put("anzeige.A8.6037.HFB.O.anzeige1", SPALTE_ATYP_TSF);
      put("anzeige.A8.6037.HFB.O.anzeige2", SPALTE_ATYP_HFS);
      put("anzeige.A8.6037.HFB.O.anzeige3", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6037.HFB.O.anzeige4", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6037.HFB.O.anzeige5", SPALTE_BTYP_1UEFS_HFS_EIN);
      put("anzeige.A8.6037.HFB.O.anzeige6", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6037.HFB.O.anzeige7", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6037.HFB.O.anzeige8", SPALTE_CTYP_1UEFS_HFS_EIN);
      put("anzeige.A8.6037.HFB.O.anzeige9", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6037.HFB.O.anzeige10", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6037.HFB.O.anzeige11", SPALTE_DTYP);
      put("anzeige.A8.6037.HFB.O.anzeige12", SPALTE_TSF_RECHTS);
      put("anzeige.A8.6037.HFB.O.anzeige13", SPALTE_TSF_LINKS);
      put("anzeige.A8.6037.HFB.O.anzeige14", SPALTE_BLINKER_RECHTS);
      put("anzeige.A8.6037.HFB.O.anzeige15", SPALTE_BLINKER_LINKS);
      
      // M13     
      put("anzeige.A8.6033.HFB.O.anzeige1", SPALTE_ATYP_TSF);
      put("anzeige.A8.6033.HFB.O.anzeige2", SPALTE_ATYP_HFS);
      put("anzeige.A8.6033.HFB.O.anzeige3", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6033.HFB.O.anzeige4", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6033.HFB.O.anzeige5", SPALTE_BTYP_1UEFS_HFS_EIN);
      put("anzeige.A8.6033.HFB.O.anzeige6", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6033.HFB.O.anzeige7", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6033.HFB.O.anzeige8", SPALTE_CTYP_1UEFS_HFS_EIN);
      put("anzeige.A8.6033.HFB.O.anzeige9", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6033.HFB.O.anzeige10", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6033.HFB.O.anzeige11", SPALTE_DTYP);
      put("anzeige.A8.6033.HFB.O.anzeige12", SPALTE_TSF_RECHTS);
      put("anzeige.A8.6033.HFB.O.anzeige13", SPALTE_TSF_LINKS);
      put("anzeige.A8.6033.HFB.O.anzeige14", SPALTE_DZEILE);
      
      // M14     
      put("anzeige.A8.6032.HFB.O.anzeige1", SPALTE_BLINKER_WAQ);
      put("anzeige.A8.6032.HFB.O.anzeige2", SPALTE_WAQ);
      
      // M15     
      put("anzeige.A8.6030.HFB.O.anzeige1", SPALTE_BLINKER_RECHTS);
      put("anzeige.A8.6030.HFB.O.anzeige2", SPALTE_BLINKER_LINKS);
      put("anzeige.A8.6030.HFB.O.anzeige3", SPALTE_TSF_RECHTS);
      put("anzeige.A8.6030.HFB.O.anzeige4", SPALTE_TSF_LINKS);
      
      // M16    
      put("anzeige.A8.6029.HFB.O.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6029.HFB.O.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6029.HFB.O.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6029.HFB.O.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6029.HFB.O.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6029.HFB.O.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6029.HFB.O.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);
      
      // M17    
      put("anzeige.A8.6028.HFB.O.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6028.HFB.O.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6028.HFB.O.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6028.HFB.O.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6028.HFB.O.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6028.HFB.O.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6028.HFB.O.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);
      
      // M18    
      put("anzeige.A8.6026.HFB.O.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6026.HFB.O.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6026.HFB.O.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6026.HFB.O.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6026.HFB.O.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6026.HFB.O.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6026.HFB.O.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);
      
      // M19    
      put("anzeige.A8.6021.HFB.O.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6021.HFB.O.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6021.HFB.O.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6021.HFB.O.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6021.HFB.O.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6021.HFB.O.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6021.HFB.O.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6021.HFB.O.anzeige8", SPALTE_DZEILE);
      
      // M20    
      put("anzeige.A8.6017.HFB.O.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6017.HFB.O.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6017.HFB.O.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6017.HFB.O.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6017.HFB.O.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6017.HFB.O.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6017.HFB.O.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);
      
      // M21    
      put("anzeige.A8.6012.HFB.O.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6012.HFB.O.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6012.HFB.O.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6012.HFB.O.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6012.HFB.O.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6012.HFB.O.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6012.HFB.O.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);
      
      // M22    
      put("anzeige.A8.6006.HFB.O.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6006.HFB.O.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6006.HFB.O.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6006.HFB.O.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6006.HFB.O.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6006.HFB.O.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6006.HFB.O.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);
      
      // M23    
      put("anzeige.A8.6005.HFB.O.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6005.HFB.O.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6005.HFB.O.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6005.HFB.O.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6005.HFB.O.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6005.HFB.O.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6005.HFB.O.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6005.HFB.O.anzeige8", SPALTE_DZEILE);
      
      // M23.BAQ    
      put("anzeige.A8.6005.BAQ.O.anzeige1", SPALTE_BAQ_RECHTS);
      put("anzeige.A8.6005.BAQ.O.anzeige2", SPALTE_BAQ_LINKS);

      // M24    
      put("anzeige.A8.6003.HFB.O.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6003.HFB.O.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6003.HFB.O.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6003.HFB.O.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6003.HFB.O.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6003.HFB.O.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6003.HFB.O.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);
      
      // M25    
      put("anzeige.A8.6002.HFB.O.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6002.HFB.O.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6002.HFB.O.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6002.HFB.O.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6002.HFB.O.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6002.HFB.O.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6002.HFB.O.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);
      
      // M26    
      put("anzeige.A8.6001.HFB.O.anzeige1", SPALTE_ATYP_HFS);
      put("anzeige.A8.6001.HFB.O.anzeige2", SPALTE_ATYP_1UEFS);
      put("anzeige.A8.6001.HFB.O.anzeige3", SPALTE_ATYP_2UEFS);
      put("anzeige.A8.6001.HFB.O.anzeige4", SPALTE_BTYP_1UEFS_HFS);
      put("anzeige.A8.6001.HFB.O.anzeige5", SPALTE_BTYP_2UEFS_1UEFS);
      put("anzeige.A8.6001.HFB.O.anzeige6", SPALTE_CTYP_1UEFS_HFS);
      put("anzeige.A8.6001.HFB.O.anzeige7", SPALTE_CTYP_2UEFS_1UEFS);
      put("anzeige.A8.6001.HFB.O.anzeige8", SPALTE_DZEILE);
		}
	 };
	 
	 protected Map<String, String> _bildInhalt = new HashMap<String, String>() {
		 {
			  put("Beibehalten", "Beibehalten");
		      put("Beliebig", "Beliebig");
		      put("DontCare", "don't care");
		      put("Dunkel", "Aus");
		      put("TLS.500Meter", "500 m");
		      put("TLS.1000Meter", "1000 m");
		      put("TLS.1500Meter", "1500 m");
		      put("TLS.2000Meter", "2000 m");
		      put("TLS.2500Meter", "2500 m");
		      put("TLS.7Komma5Tonnen", "7,5 t");
		      put("TLS-AS.Gltte", "Gltte");
		      put("TLS-AS.lspur", "lspur");
		      put("TLS.Auf500Meter", "Auf 500 m");
		      put("TLS.Auf1000Meter", "Auf 1000 m");
		      put("TLS.Auf1500Meter", "Auf 1500 m");
		      put("TLS.Auf2000Meter", "Auf 2000 m");
		      put("TLS.Auf2500Meter", "Auf 2500 m");
		      put("TLS.Baustelle", "Baustelle");
		      put("TLS.EndeAllerStreckenverbote", "Ende alles");
		      put("TLS.EndeUeberholverbotAllgemein", "Ende V");
		      put("TLS.EndeUeberholverbotLkw", "Ende V Lkw");
		      put("TLS.EndeZulaessigeHoechst100", "Ende 100 km/h");
		      put("TLS.EndeZulaessigeHoechst120", "Ende 120 km/h");
		      put("TLS.EndeZulaessigeHoechst40", "Ende 40 km/h");
		      put("TLS.EndeZulaessigeHoechst50", "Ende 50 km/h");
		      put("TLS.EndeZulaessigeHoechst60", "Ende 60 km/h");
		      put("TLS.EndeZulaessigeHoechst80", "Ende 80 km/h");
		      put("TLS.EndeZulaessigeHoechst130", "Ende 130 km/h");
		      put("TLS.FahrstreifenGesperrt", "Gesperrt");
		      put("TLS.FahrstreifenHalten", "Halten");
		      put("TLS.FahrstreifenWechselnNachLinks", "Links");
		      put("TLS.FahrstreifenWechselnNachRechts", "Rechts");
		      put("TLS.Gefahrenstelle", "Gefahr");
		      put("TLS.GelbesBlinklicht", "Blinken");
		      put("TLS.Glaettegefahr", "Gltte");
		      put("TLS.Naesse", "Nsse");
		      put("TLS.Nebel", "Nebel");
		      put("TLS.Schleudergefahr", "Schleuder");
		      put("TLS.Sicht", "Sicht");
		      put("TLS.Smog", "Smog");
		      put("TLS.Stau", "Stau");
		      put("TLS.Staugefahr", "Staugefahr");
		      put("TLS.UeberholverbotAllgemein", "V");
		      put("TLS.UeberholverbotLkw", "V Lkw");
		      put("TLS.Unfall", "Unfall");
		      put("TLS.ZulaessigeHoechst100", "100 km/h");
		      put("TLS.ZulaessigeHoechst120", "120 km/h");
		      put("TLS.ZulaessigeHoechst40", "40 km/h");
		      put("TLS.ZulaessigeHoechst50", "50 km/h");
		      put("TLS.ZulaessigeHoechst60", "60 km/h");
		      put("TLS.ZulaessigeHoechst80", "80 km/h");
		      put("TLS.ZulaessigeHoechst130", "130 km/h");
          put("TLS.3000Meter", "3000 m");
          put("TLS.1Kilometer", "1 km");
          put("TLS.1Komma5Kilometer", "1,5 km");
          put("TLS.2Kilometer", "2 km");
          put("TLS.2Komma5Kilometer", "2,5 km");
          put("TLS.3Kilometer", "3 km");
          put("TLS.Tiere", "Tiere");
          put("TLS.Kontrolle", "Kontrolle");
          put("TLS.Oelspur", "lspur");
          
          // Neu 24.04.2013
          put("TLS.DunkelStau", "Dunkel im Stau");
          
          put("TLS.BlockabfertigungLkw", "Blockabfertigung Lkw");
          put("TLS.BlockabfertigungLkwStop", "Blockabfertigung Lkw Stop");
                   
          put("TLS.Neutral", "neutral");
          put("TLS.Unbeklebt", "neutral");
                  
          put("TLS.SeitenstreifenBefahren", "Seitenstreifen befahren");
	      put("TLS.SeitenstreifenNichtBefahren", "Seitenstreifen nicht befahren");
	      put("TLS.SeitenstreifenRaeumen", "Seitenstreifen rumen");
                   
          put("TLS.PrismaSeite1", "Prisma Seite 1");
          put("TLS.PrismaSeite2", "Prisma Seite 2");
          put("TLS.PrismaSeite3", "Prisma Seite 3");
          put("TLS.PrismaSeite4", "Prisma Seite 4");	

		 }	 
	 };
	
	public int getAnzahlAttribute() {
		return _ausgabeTabelleSpalten.size();
	}
	 
	public List<String> getAusgabeTabelleSpalten() {
		return Collections.unmodifiableList(_ausgabeTabelleSpalten);
	}
	
	public int getSpaltenIndexRelativ(String pid) {
		Integer result = _spaltenIndexRelativ.get(pid);
		return result != null ? result : -1;
	}
	
	public String getBildInhalt(String bildInhalt) {
		return _bildInhalt.get(bildInhalt);
	}
	
	protected final Map<String, Integer> _bildInhaltWerte = new HashMap<String, Integer>() 
  {
    {
      // Allgemein
      put("Dunkel", 160);
      
      // A-Zeichen 
      put("TLS.EndeZulaessigeHoechst130", 140);
      put("TLS.EndeZulaessigeHoechst120", 140);
      put("TLS.EndeZulaessigeHoechst100", 140);
      put("TLS.EndeZulaessigeHoechst80", 140);
      put("TLS.EndeZulaessigeHoechst60", 140);
      put("TLS.EndeZulaessigeHoechst50", 140);
      put("TLS.EndeZulaessigeHoechst40", 140);
      
      put("TLS.ZulaessigeHoechst130", 130);
      put("TLS.ZulaessigeHoechst120", 120);
      put("TLS.ZulaessigeHoechst100", 100);
      put("TLS.ZulaessigeHoechst80", 80);
      put("TLS.ZulaessigeHoechst60", 60);
      put("TLS.ZulaessigeHoechst50", 50);
      put("TLS.ZulaessigeHoechst40", 40);
             
      put("TLS.DunkelStau", 20);
      
      put("TLS.FahrstreifenWechselnNachRechts", 10);
      put("TLS.FahrstreifenWechselnNachLinks", 10);
      put("TLS.FahrstreifenGesperrt", 5);
      
      // B-Zeichen
      put("TLS.Glaettegefahr", 180);
      put("TLS-AS.Gltte", 180);
      
      put("TLS.EndeAllerStreckenverbote", 130);
      put("TLS.EndeUeberholverbotAllgemein", 130);
      put("TLS.EndeUeberholverbotLkw", 130);
      
      put("TLS.UeberholverbotAllgemein", 110);
      put("TLS.UeberholverbotLkw", 90);
      put("TLS.Baustelle", 70);
      put("TLS.Gefahrenstelle", 50);
      put("TLS.Schleudergefahr", 30);
      put("TLS.Stau", 15);
    }  
  };
   
  public boolean isAttributKonvertierbar(String attribut) {
    return attribut != null && (attribut.startsWith("A-Typ") || attribut.startsWith("B-Typ")); 
  }
  
  public String getWertKonvertiert(String attribut, String value, String objekt) {
    
      String key = null;
      
      for (Map.Entry<String, String> entry: _bildInhalt.entrySet()) {
        if (entry.getValue().equals(value)) {
          key = entry.getKey();
          break;
        }
      }
      
      if (key != null) {
        Integer result = _bildInhaltWerte.get(key);
        if (result == null) {
          result = 200;
        }
        return String.valueOf(result);
      }
      else {
        return null;  
      }
  }

  @Override
  public String getAttributKonvertiert(String attribut)
  {
    String result = attribut;
    if (isAttributKonvertierbar(attribut)) {
      int kla = attribut.indexOf("[");
      int klz = attribut.indexOf("]"); 
      if (kla < 0 || klz < kla)
      {
        result = attribut.trim() + " [km/h]";
      }
    }
    return result ;
  }
  
  @Override
  public long getMilliesTreppenfunktionErgaenzen()
  {
    return 1000;
  }
  
  @Override
  public boolean anfangAuffuellen() {
    return true;
  }
  
  @Override
  public boolean endeAuffuellen() {
    return true;
  }
  
  protected final List<String> _zuIgnorierendeWerte = new ArrayList<String>() {
    {
      add("");
    }
  };
  
  @Override
  public Collection<String> getZuIgnorierendeWerte() {
    return Collections.unmodifiableList(_zuIgnorierendeWerte);
  }

}
