/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.actions;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IContributor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.osgi.framework.Bundle;

import de.bsvrz.buv.plugin.pua.basis.Activator;
import de.bsvrz.buv.plugin.pua.basis.DaVKommunikation;
import de.bsvrz.buv.plugin.pua.basis.pua.ProtokolleUndAuswertungen;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.pua.prot.util.ProtocolRequestResult;
import de.bsvrz.sys.funclib.losb.datk.AtlBase;

public class ActionSkripteErstellen  implements IWorkbenchWindowActionDelegate {
	  
	@Override
	  public void run(IAction action) {
		  ClientDavInterface connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
		  if (connection != null) {
			  
			 MessageBox mb = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK | SWT.CANCEL);
		     mb.setText( "PUA-Skripte erstellen" );
		     mb.setMessage( "Es werden alle im PUA-Plugin enthaltenen Skripte, die fr die Eigenschaft \"Erstellen\"" +
		    		 " mit \"true\" markiert sind, auf dem Server erstellt.\nVorhandene Skripte werden berschrieben");
		     if (mb.open() == SWT.CANCEL) {
		    	 return;
		     }
			  
	  	     ProtokolleUndAuswertungen pua = new ProtokolleUndAuswertungen(connection);
		
			 IConfigurationElement[] configurationElementsFor = 
				 Platform.getExtensionRegistry().getConfigurationElementsFor("de.bsvrz.buv.plugin.pua.basis.skriptInfos");
			  
			 int gesamt = 0;
			 int fehler = 0;
			 
			  for (IConfigurationElement ce : configurationElementsFor) {
				  
				  if("skriptInfos".equals(ce.getName())) {
					  
					  for (IConfigurationElement ch : ce.getChildren()) {
						  
						  if("skriptInfo".equals(ch.getName())) {
							  
							  if ("true".equals(ch.getAttribute("Erstellen"))) {
								  IContributor plugin = ch.getContributor();
								  String skriptDatei = ch.getAttribute("Skriptdatei");
								  InputStream inputStream = null;
								  String urlText = "platform:/plugin/" + plugin.getName() +
								  (skriptDatei.startsWith("/") ? "" : "/") + skriptDatei;
								  
								  gesamt++;
								  
								  try {
									  URL url = new URL(urlText);
									  inputStream = url.openConnection().getInputStream();
									  BufferedReader source = new BufferedReader(new InputStreamReader(inputStream));
									  StringBuffer buffer = new StringBuffer();
									  String line;
									    
									  while ((line = source.readLine()) != null) {
									      buffer.append(line + "\n");
									  }
									  
									  pua.erstelleSkript(ch.getAttribute("Pid"), ch.getAttribute("Name"),
										  new AtlBase(ch.getAttribute("Name"), ch.getAttribute("Beschreibung"),
											  ch.getAttribute("Autor"), ch.getAttribute("Status"), 
									    	  ch.getAttribute("Datum")), buffer.toString());
									    
									  ProtocolRequestResult result = pua.getRequestResult();
					     		  	  if (result.isRequestSuccessful()) {
					     		  	      String msg = "Erfolgreich erstellt: " + skriptDatei;
					     			  	  Activator.getDefault().getLog().log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
					     			  	  System.err.println(msg);
  				     		  		  }
					     		  	  else {
					     		  		  fehler++;
					     		  		  Status status = new Status(Status.ERROR, Activator.PLUGIN_ID, result.getErrorMsg()); 
					     		  		  String msg = "Fehler bei Erstellung: " + skriptDatei;
					     			  	  Activator.getDefault().getLog().log(new MultiStatus(Activator.PLUGIN_ID, Status.INFO, 
					     			  	      new IStatus[] { status }, msg, null));
					     			  	  System.err.println(msg);
					     			  	  System.err.println(result.getErrorMsg());
					     		  	  }
								  } 
								  catch (MalformedURLException e) {
									  fehler++;
									  String msg = "Ungltige URL \"" + urlText + "\"";
							  	      Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, msg, e));
							  	      System.err.println(msg);
							  	      e.printStackTrace(System.err);
								  }
								  catch (IOException e) {
									  fehler++;
									  String msg = "I/O-Fehler beim Lesen von \"" + skriptDatei + "\" als Bundle-Eintrag";
							  	      Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, msg, e));
							  	      System.err.println(msg);
							  	      e.printStackTrace(System.err);
								  }
								  catch (RuntimeException e) {
									  fehler++;
									  String msg = "Fehler beim Bearbeiten der URL \"" + urlText + "\"";
							  	      Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, msg, e));
							  	      System.err.println(msg);
							  	      e.printStackTrace(System.err);
								  }
								  finally {
									  if (inputStream != null) {
										  try {
											inputStream.close();
										} catch (IOException e) {
											// TODO Auto-generated catch block
											e.printStackTrace();
										}
									  }
								  }
							  }
	 					  }
					  }
				  }
	    	 }
			 Activator.getDefault().getLog().log(new Status(Status.INFO, Activator.PLUGIN_ID, 
					  (gesamt - fehler) + "/" + gesamt + " PUA-Skripte erstellt"));
		 }	
		 else {
			 String msg =  "Es knnen keine Skripte erstellt werden, da keine Verbindung zum Datenverteiler besteht";
			 Activator.getDefault().getLog().log(new Status(Status.WARNING, Activator.PLUGIN_ID, msg));
			 System.err.println(msg);
		 }
	  }	  


	  @Override
	  public void selectionChanged(IAction action, ISelection selection)
	  {
	  }

	  @Override
	  public void dispose()
	  {
	  }

	  @Override
	  public void init(IWorkbenchWindow window)
	  {
	  }

}
