/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.dav;

import java.util.List;

import de.bsvrz.buv.rw.basislib.kalender.KalenderZeitFilterDatenStrukturZeitGruppe;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKindCombination;
import de.bsvrz.dav.daf.main.archive.ArchiveOrder;
import de.bsvrz.dav.daf.main.archive.ArchiveQueryPriority;
import de.bsvrz.dav.daf.main.archive.ArchiveRequestOption;
import de.bsvrz.dav.daf.main.archive.TimingType;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse kapselt eine Archivanfrage und liefert die Zugriffsmethoden auf die Archiv-Datenstruktur
 * 
 * @author Pittner
 * 
 */
public class AnfrageDaten
{
  private List<SystemObject> _objekteAnzeige;

  private List<SystemObject> _objekte;

  private List<KalenderZeitFilterDatenStrukturZeitGruppe> _zeitgruppen;

  private DataDescription _dataDescription;

  private ArchiveQueryPriority _archiveQueryPriority;

  private TimingType _timingType;

  private boolean _startRelative;

  private ArchiveDataKindCombination _archiveDataKindCombination;

  private ArchiveOrder _archiveOrder;

  private ArchiveRequestOption _archiveRequestOption;

  private ProtokollierungsAnfrageTyp[] _anfrageTypen;

  private String _beschreibung;

  /**
   * Konstruktor
   * 
   * @param objekte
   * @param objekteAnzeige
   * @param anfrageTypen
   * @param zeitgruppen
   * @param dataDescription
   * @param archiveQueryPriority
   * @param timingType
   * @param startRelative
   * @param archiveDataKindCombination
   * @param archiveOrder
   * @param archiveRequestOption
   */
  public AnfrageDaten(List<SystemObject> objekte, List<SystemObject> objekteAnzeige,
      ProtokollierungsAnfrageTyp[] anfrageTypen, List<KalenderZeitFilterDatenStrukturZeitGruppe> zeitgruppen,
      DataDescription dataDescription, ArchiveQueryPriority archiveQueryPriority, TimingType timingType,
      boolean startRelative, ArchiveDataKindCombination archiveDataKindCombination, ArchiveOrder archiveOrder,
      ArchiveRequestOption archiveRequestOption)
  {
    if (objekte == null || zeitgruppen == null || dataDescription == null || archiveQueryPriority == null
        || timingType == null || archiveDataKindCombination == null || archiveOrder == null
        || archiveRequestOption == null)
      throw new IllegalArgumentException("bergabeparameter ist null");

    _objekte = objekte;
    _objekteAnzeige = objekteAnzeige;
    _anfrageTypen = anfrageTypen;
    _zeitgruppen = zeitgruppen;
    _dataDescription = dataDescription;
    _archiveQueryPriority = archiveQueryPriority;
    _timingType = timingType;
    _startRelative = startRelative;
    _archiveDataKindCombination = archiveDataKindCombination;
    _archiveOrder = archiveOrder;
    _archiveRequestOption = archiveRequestOption;

  }

  public List<SystemObject> getObjekte()
  {
    return _objekte;
  }

  public void setObjekte(List<SystemObject> objekte)
  {
    _objekte = objekte;
  }

  public List<SystemObject> getObjekteAnzeige()
  {
    return _objekteAnzeige;
  }

  public void set_objekteAnzeige(List<SystemObject> objekteAnzeige)
  {
    _objekteAnzeige = objekteAnzeige;
  }

  public List<KalenderZeitFilterDatenStrukturZeitGruppe> getZeitgruppen()
  {
    return _zeitgruppen;
  }

  public void setZeitgruppen(List<KalenderZeitFilterDatenStrukturZeitGruppe> zeitgruppen)
  {
    _zeitgruppen = zeitgruppen;
  }

  public ArchiveQueryPriority getArchiveQueryPriority()
  {
    return _archiveQueryPriority;
  }

  public void setArchiveQueryPriority(ArchiveQueryPriority archiveQueryPriority)
  {
    _archiveQueryPriority = archiveQueryPriority;
  }

  public TimingType getTimingType()
  {
    return _timingType;
  }

  public void setTimingType(TimingType timingType)
  {
    _timingType = timingType;
  }

  public boolean getStartRelative()
  {
    return _startRelative;
  }

  public void setStartRelative(boolean startRelative)
  {
    this._startRelative = startRelative;
  }

  public ArchiveDataKindCombination getArchiveDataKindCombination()
  {
    return _archiveDataKindCombination;
  }

  public void setArchiveDataKindCombination(ArchiveDataKindCombination archiveDataKindCombination)
  {
    _archiveDataKindCombination = archiveDataKindCombination;
  }

  public ArchiveOrder getArchiveOrder()
  {
    return _archiveOrder;
  }

  public void setArchiveOrder(ArchiveOrder archiveOrder)
  {
    _archiveOrder = archiveOrder;
  }

  public ArchiveRequestOption getArchiveRequestOption()
  {
    return _archiveRequestOption;
  }

  public void setArchiveRequestOption(ArchiveRequestOption archiveRequestOption)
  {
    _archiveRequestOption = archiveRequestOption;
  }

  public DataDescription getDataDescription()
  {
    // TODO Auto-generated method stub
    return _dataDescription;
  }

  public ProtokollierungsAnfrageTyp[] getAnfrageTypen()
  {
    return _anfrageTypen;
  }

  public void setAnfrageTypen(ProtokollierungsAnfrageTyp[] anfrageTypen)
  {
    _anfrageTypen = anfrageTypen;
  }

  /**
   * Methode liefert datenArt zurck
   * 
   * @return datenArt
   */
  public String getBeschreinung()
  {
    return _beschreibung;
  }

  /**
   * Methode setzt die Variable datenArt
   * 
   * @param beschreibung
   *          zu bernehmender Wert fr beschreibung
   */
  public void setBeschreibung(String beschreibung)
  {
    _beschreibung = beschreibung;
  }

}
