/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.expose;

import java.util.List;
import java.util.Map;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Schnittstelle fr das Providing der Objekte fr Pua-Protokolle.<br>
 * Die gelieferten Daten beziehen sich auf ein Pua-Skript bzw. auf einen Typ von Pua-Skripten.
 * 
 * @author Sans
 *
 */
public interface IObjektProvider {
	
	/**
	 * Liefert eine Sammlung der Objekte, fr die eine Pua-Anfrage gestellt wird.
	 * 	 * 
	 * @param systemObjectType Der Objekttyp - Pid des <code>SystemObjectType> - auf den sich 
	 * die Anfrage bezieht
	 * @param ausfilternDE255 Flag, das angibt, ob Objekte vom Typ typ.de und DE-Kanal 255
	 * ausgefiltert werden
	 * param umsetzungsModulFilterText wenn hier ein nichtleerer Text angegeben ist, werden nur 
	 * Objekte vom Typ typ.de zurckgegeben, deren Wert des Attributes "Umsetungsmodul" den Text 
	 * enthlt
	 * @return die Objekte, fr die die Pua-Anfrage gestellt wird
	 */
	public List<SystemObject> getObjekte(String pidSystemObjectType, boolean ausfilternDE255,
				String umsetzungsModulFilterText);
	
	public SystemObject getHauptObjekt(List<SystemObject> objekte);
	
	/**
	 * Bestimmt aus den bergebenen Objekten die Objekte, die einer Pua-Anfrage als Pseudo-Objekte
	 * bergeben werden.<br>
	 * Dies knnen - mssen aber nicht - diesselben Objekte sein, die in <code>objekte</code> 
	 * bergeben werden.
	 * 
	 * @param objekte die Objekte, fr die die Pua-Anfrage gestellt werden soll
	 * @return die Pseudo-Objekte, fr die Pua-Anfrage: Es wird jeweils unter dem Schlssel eines Pseudoobjekt-
	 * Alias ein Feld von Objekten zurckgegeben 
	 */
	public Map<String, String[]> getPseudoObjekte(List<SystemObject> objekte);
	
	public boolean isSpeichernSelektierteObjekte();

}
