/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.expose;

import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import de.bsvrz.buv.plugin.pua.basis.pua.daten.ProtokollInformationen;
import de.bsvrz.buv.plugin.pua.basis.views.ProtokollAusgabeSicht;
import de.bsvrz.buv.plugin.pua.basis.views.Tools;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.pua.prot.client.dataobject.ProtocolResultData;
import de.bsvrz.pua.prot.data.ProtocolData;

public class NachbereitungAqRueckmeldung extends NachbereitungAqBefehle implements INachbereitung {
	
	@Override
	public List<String[]> getZellenInhalte(List<SystemObject> objekte, int[] objektZuordnung, 
			ProtocolResultData prd, IProjektInfo projektInfo, ProtokollInformationen protokollInformationen) {
		if (projektInfo == null) {
			throw new IllegalArgumentException("ProjektInfo darf nicht null sein");
		}
		List<String[]> result = new ArrayList<String[]>();
		String zeitStempel = SDF.format(new Date(prd.getTimeStamp()));
		int anzahlAttribute = projektInfo.getAnzahlAttribute();
		Map<String, String[]> zellenwerteMap = new LinkedHashMap<String, String[]>();
		
//		System.out.println("OBJEKTE: " + objekte.size() + "  PRD: " + prd.values.length);
		
		for (int i = 0; i < objekte.size(); i++) {
			String[] zellenwerte;
			SystemObject anfrageObj = objekte.get(i); // Anzeige oder AQ
			SystemObject anzeigeObj = null; // immer AQ
			if (anfrageObj.isOfType("typ.anzeige")) {
				anzeigeObj = getAnzeigeQuerschnitt(anfrageObj);
			}
			else if (anfrageObj.isOfType("typ.anzeigeQuerschnitt")) {
				anzeigeObj = anfrageObj;
			}
			
       	  	if (anzeigeObj != null) {
       	  		zellenwerte = zellenwerteMap.get(anzeigeObj.getPid());
       	  		if (zellenwerte == null) {
       	  			zellenwerte = new String[ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL + anzahlAttribute];
 					zellenwerteMap.put(anzeigeObj.getPid(), zellenwerte);
       	  		}
       	  		// Datum + Objektbezeichner
       	  		zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_DATUMZEIT_INDEX] = zeitStempel;
       	  		zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_OBJEKTBEZEICHNER_INDEX] = anzeigeObj.getNameOrPidOrId();
       	  	}
       	  	else {
       		  // AQ zur Anzeige nicht gefunden
       		  continue;  
       	  	}
	            
            // Attributwerte
       	  	if (i < prd.values.length) {
            	
            	// Prfe ob Daten vorhanden sind
            	if (prd.values[i].getValue() != null && !prd.values[i].getValue().hasValue()) {
            		continue;
            	}
            	
            	boolean hinzufuegen = false;
            	DataModel dataModel = anfrageObj.getDataModel();
				ProtocolData protocolData = prd.getData(dataModel, i);
				
////					System.out.println("NAME: " + protocolData.getName() + " ISLIST: " + protocolData.isList() + " TYP: " + anfrageObj.getType().getPid());
//				
//				// Bearbeite Attributgruppe atg.anzeigeEigenschaftIst
//				if (protocolData.getName().equals(dataModel.getAttributeGroup("atg.anzeigeEigenschaftIst").getName()) &&
//						protocolData.isList()) {
//					int index = projektInfo.getSpaltenIndexRelativ(anfrageObj.getPid());
////					System.out.println("INDEX: "  + index);
//            		if (index >= 0) {
//            			String text = "";
//            			Data eigenschaft = protocolData.getItem("Eigenschaft");
//            			SystemObject anzeigeInhalt = eigenschaft.getReferenceValue("AnzeigeInhalt").getSystemObject();
////            			System.out.println("Bearbeite Attribut \"AnzeigeInhalt\" fr Objekt " + anfrageObj.getPid() + " am Index " + i);
//            			
//            			if (anzeigeInhalt != null)
//            			{
//            				AttributeGroup atg = dataModel.getAttributeGroup("atg.wvzInhalt");
//            				Data data = anzeigeInhalt.getConfigurationData(atg);
//            				if (data != null) {
//            					String bildInhalt = data.getTextValue("Bildinhalt").getValueText();
//            					if (bildInhalt != null && ! bildInhalt.isEmpty()) {
//            						text = projektInfo.getBildInhalt(bildInhalt);
//            					}
//            					else {
//            						text = bildInhalt;
//            					}
//            				}	
//            			}
//            			zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL + index] = text;
//            		}
//	            	hinzufuegen = hinzufuegen || !result.contains(zellenwerte);
//				}	
//				// Bearbeite Attributgruppe atg.anzeigeQuerschnittEigenschaftIst
//				if (protocolData.getName().equals(dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftIst").getName()) &&
//						protocolData.isList()) {
//					Data eigenschaft = protocolData.getItem("Eigenschaft");
//        			String betriebszustand = eigenschaft.getTextValue("Betriebszustand").getText();
//        			// Betriebszustand kommt in die Spalte unmittelbar nach "Zeitstempel"
//        			zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL] = betriebszustand;
////        			System.out.println("Betriebszustand ist \"" + betriebszustand + "\" fr Objekt " + anfrageObj.getPid() + " am Index " + i);
//					hinzufuegen = hinzufuegen || !result.contains(zellenwerte);
//				}
				
				
				
				// Bearbeite Attribut "AnzeigeInhalt" der Attributgruppe atg.anzeigeEigenschaftIst
				System.out.println("protocolData.getName(): '"  + protocolData.getName() + "'");
				if (protocolData.getName().equals("AnzeigeInhalt")) {
					int index = projektInfo.getSpaltenIndexRelativ(anfrageObj.getPid());
//					System.out.println("INDEX: "  + index);
            		if (index >= 0) {
            			String text = "";
            			SystemObject anzeigeInhalt = null;
            			try {
            				anzeigeInhalt = protocolData.asReferenceValue().getSystemObject();
//                			System.out.println("Bearbeite Attribut \"AnzeigeInhalt\" fr Objekt " + anfrageObj.getPid() + " am Index " + i);	
            			}
            			catch (ArithmeticException e) {
//            				System.err.println(e.getMessage());
            				text = Tools.toText(protocolData);
            			}
            			
            			if (anzeigeInhalt != null)
            			{
            				AttributeGroup atg = dataModel.getAttributeGroup("atg.wvzInhalt");
            				Data data = anzeigeInhalt.getConfigurationData(atg);
            				if (data != null) {
            					String bildInhalt = data.getTextValue("Bildinhalt").getValueText();
            					if (bildInhalt != null && ! bildInhalt.isEmpty()) {
            						text = projektInfo.getBildInhalt(bildInhalt);
            					}
            					else {
            						text = bildInhalt;
            					}
            				}	
            			}
            			else 
            			{
            				text = projektInfo.getBildInhalt("ReferenzUndefiniert");
            				
            				if (text == null)
            				{
            					text = "<unbekannt>";
            				}
            			}
            		
            			zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL + index] = text;
            		}
	            	hinzufuegen = hinzufuegen || !result.contains(zellenwerte);
				}	
				// Bearbeite Attribut "Betriebszustand" der Attributgruppe atg.anzeigeQuerschnittEigenschaftIst
				if (protocolData.getName().equals("Betriebszustand")) {
        			String betriebszustand = protocolData.asTextValue().getText();
        			// Betriebszustand kommt in die Spalte unmittelbar nach "Zeitstempel"
        			zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL] = betriebszustand;
//        			System.out.println("Betriebszustand ist \"" + betriebszustand + "\" fr Objekt " + anfrageObj.getPid() + " am Index " + i);
					hinzufuegen = hinzufuegen || !result.contains(zellenwerte);
				}
				
				
				
				if (hinzufuegen) {
		    		result.add(zellenwerte);
		        }	
            }
		}
		
		return result;
	}

}
