/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.extension;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.Platform;

import de.bsvrz.buv.plugin.pua.basis.expose.INachbereitung;
import de.bsvrz.buv.plugin.pua.basis.expose.IObjektProvider;
import de.bsvrz.buv.plugin.pua.basis.expose.IProjektInfo;
import de.bsvrz.buv.plugin.pua.basis.expose.ObjektProviderDefault;
import de.bsvrz.buv.plugin.pua.basis.pua.daten.ProtokollInformationen;

/**
 * Singleton-Klasse zum Verwalten der Skriptinfos
 * 
 * @author Sans
 *
 */

public class ExtensionVerwaltung {
	
	private Map<String, SkriptInfo> _skriptInfos = new HashMap<String, SkriptInfo>();
	
	private Map<String, String> _aktionsTexte = new HashMap<String, String>();
	
	private static ExtensionVerwaltung _instanz = null;
	
	private SkriptInfo erzeugeSkriptInfo(IConfigurationElement ce) {
		  String pid = ce.getAttribute("Pid");
		  String name = ce.getAttribute("Name");
		  String objektTyp = ce.getAttribute("Objekttyp");
		  String aktionsId = ce.getAttribute("Aktions-Id");
		  String beschreibung = ce.getAttribute("Beschreibung");
		  String autor = ce.getAttribute("Autor");
		  String status = ce.getAttribute("Status");
		  String datum = ce.getAttribute("Datum");
		  String skriptDatei = ce.getAttribute("Skriptdatei");
		  boolean erstellen = "true".equals(ce.getAttribute("Erstellen"));
		  
		  boolean aktuell = !"false".equals(ce.getAttribute("Aktuell"));
		  boolean nachgeliefert = "true".equals(ce.getAttribute("Nachgeliefert"));
		  boolean nachgefordertAktuell = "true".equals(ce.getAttribute("Nachgefordert-Aktuell"));
		  boolean nachgefordertNachgeliefert = "true".equals(ce.getAttribute("Nachgefordert-Nachgeliefert"));
		  
		  boolean aenderungsProtokoll = !"false".equals(ce.getAttribute("nderungsprotokoll"));
		  boolean ganglinien = !"false".equals(ce.getAttribute("Ganglinien"));
		  boolean keinNoChange = "true".equals(ce.getAttribute("KeinNoChange"));
		  boolean ausfilternDE255 = "true".equals(ce.getAttribute("Ausfiltern-DE255"));
		  String umsetungsModulFilterText = ce.getAttribute("Umsetzungsmodul-Filtertext");
		  
		  IObjektProvider objektProvider = null;
		  
		  try {
			  objektProvider = (IObjektProvider)ce.createExecutableExtension("Objektprovider-Klasse");
		  }
		  catch (CoreException e ) {
			 // wenn Objektprovider-Klasse nicht gefunden bzw. nicht versorgt,
			 // wird der Default Provider verwendet
			  objektProvider = new ObjektProviderDefault();
		  }
		  
//		  IProjektInfo projektInfo = null;
		  
//		  if (pid.equals("pua.skript.st.uz.a14.atgSswSbaSchaltbildAnzeige")) {
//			  projektInfo = null;
//		  }
		  
//		  try {
//			  projektInfo = (IProjektInfo)ce.createExecutableExtension("Projektinfo-Klasse");
//		  }
//		  catch (CoreException e ) {
//			  // tue nichts
//		  }
		  
//		  INachbereitung nachbereitung = null;
//		  try {
//			  nachbereitung = (INachbereitung)ce.createExecutableExtension("Nachbereitung-Klasse");
//		  }
//		  catch (CoreException e ) {
//			  // tue nichts
//		  }
		  
		  List<Bindung> bindungen = new ArrayList<Bindung>();
		  
		  for (IConfigurationElement ch1 : ce.getChildren()) {
			  if("bindungen".equals(ch1.getName())) {
				  for (IConfigurationElement ch2 : ch1.getChildren()) {
					  if("bindung".equals(ch2.getName())) {
						  String bezeichner = ch2.getAttribute("Bezeichner");
						  if (bezeichner != null) {
							  Bindung bindung = new Bindung(bezeichner);
							  for (IConfigurationElement ch3 : ch2.getChildren()) {
								  if("bindungsAttribute".equals(ch3.getName())) {
									  String bindungsName = ch3.getAttribute("Name");
									  if (bindungsName != null) {
										  String bindungsWert = ch3.getAttribute("Wert");
										  if (bindungsWert != null) {
											  bindung.addBindungsAttribute(bindungsName, bindungsWert);
										  }	  
									  }
								  }
							  }
							  if (bindung.getAnzahlBindungsAttribute() > 0) {
								  bindungen.add(bindung);												  
							  }
						  }
					  }
				  }
			  }
		  }
		  
		  return new SkriptInfo(pid, name, aktionsId, objektTyp, beschreibung, autor, 
				  status, datum, skriptDatei, erstellen, aktuell, nachgeliefert, 
				  nachgefordertAktuell, nachgefordertNachgeliefert, aenderungsProtokoll, 
				  ganglinien, keinNoChange, ausfilternDE255, umsetungsModulFilterText, 
				  objektProvider, bindungen);
	}
	
	private ExtensionVerwaltung() {
		
		IConfigurationElement[] configurationElementsFor = Platform.getExtensionRegistry().getConfigurationElementsFor(
			"de.bsvrz.buv.plugin.pua.basis.skriptInfos");
		  
		for (IConfigurationElement ce : configurationElementsFor) {
			  
			if("skriptInfos".equals(ce.getName())) {
				  
				for (IConfigurationElement ch : ce.getChildren()) {
					  
					if("skriptInfo".equals(ch.getName())) {
						  
						SkriptInfo skriptInfo = erzeugeSkriptInfo(ch);
						_skriptInfos.put(skriptInfo.getAktionsId(), skriptInfo);
					}	  
				}
			}
		}
		
		configurationElementsFor = Platform.getExtensionRegistry().getConfigurationElementsFor(
		"org.eclipse.ui.actionSets");
		for (IConfigurationElement ce : configurationElementsFor) {
			if("actionSet".equals(ce.getName())) {
				  
				for (IConfigurationElement ch : ce.getChildren()) {
					  
					if("action".equals(ch.getName())) {
						  
						String id = ch.getAttribute("id");
						if (id != null) {
							String label = ch.getAttribute("label");
							if (label != null) {
								_aktionsTexte.put(id, label);
							}
						}
					}	  
				}
			}	
		}
	}
	
	public static ExtensionVerwaltung getInstanz() {
		if (_instanz == null) {
			_instanz = new ExtensionVerwaltung();
		}
		return _instanz;
	}
	
	/**
	 * 
	 * @param skriptPid
	 * @return SkriptInfo-Instanz zur bergebenene Aktions-Id
	 * @throws RuntimeException, wenn keine Erweiterung zur Aktions-Id deklariert ist
	 */
	public SkriptInfo getSkriptInfo(String aktionsId) throws RuntimeException {
		SkriptInfo result =  _skriptInfos.get(aktionsId);
		if (result == null) {
			throw new RuntimeException("Es wurde keine Erweiterung \"skriptInfo\" mit Aktions-Id \"" + 
					aktionsId + "\" gefunden");
		}
		return result;
	}
	
	public String getAktionsText(String aktionsId) {
		return _aktionsTexte.get(aktionsId);
	}
	
	public SkriptInfo getSkriptInfo(ProtokollInformationen protokollInformationen) {
		String informationsText = protokollInformationen.getInformationsText();
    	int index = informationsText.indexOf('[');
    	if (index == 0) {
    		index = informationsText.indexOf(']');
    		if (index > 0) {
    			String verborgenerText = informationsText.substring(1, index);
    			String[] zerlegung = verborgenerText.split(":");
    			if (zerlegung.length > 0)
    			{
    				String aktionsId = zerlegung[0];
    				return ExtensionVerwaltung.getInstanz().getSkriptInfo(aktionsId);
    			}
    		}
    	}
    	return null;
	}

}
