/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.font;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;

public class DaVHelper
{
  
  /**
   * Umsetzung der Ascii Codes der Textkonserve als String 
   */
  private  static Map<Integer,String> mapTextKonserve = new HashMap<Integer, String>(){
    {
      // Textkonserve "Mharbeiten"
      put(40,  "M"); 
      put(41,  "har"); 
      put(91,  "bei"); 
      put(63,  "ten");      
      // Textkonserve "Falschfahrer"
      put(34,  "Fal"); 
      put(167, "sch"); 
      put(36,  "fah"); 
      put(37,  "rer"); 
      // Zeichen mit "OberUndUnterstrich"
      put(194,  "0");
      put(185,  "1");
      put(186,  "2");
      put(187,  "3");
      put(188,  "4");
      put(189,  "5");
      put(190,  "6");
      put(191,  "7");
      put(192,  "8");
      put(193,  "9");      
      put(195,  "a");
      put(196,  "b");
      put(197,  "c");
      put(198,  "d");
      // Zeichen mit "Anschlustelle"
      put(123,  "(");
      put(125,  ")");
      // Zeichen mit "BABAuf" 1-3Stellig      
      put(199,  "[");
      put(91,   "[");
      put(60,   "[");
      // Zeichen mit "BABZu" 1-3Stellig            
      put(200,  "]");
      put(93,   "]");
      put(62,   "]");
      // Zeichen "PfeilAuf"            
      put(94,   "^");
      
    }                          
  };
  
  public static Map<Integer, String> getMapTextKonserve()
  {
    return mapTextKonserve;
  }

  /**
   * @param result
   * @return
   */
  public static VisuAnzeigeKapsel holeDynamischerWechselTextAusData(Data item)
  {
    VisuAnzeigeKapsel re = null;

    SystemObject font = item.getItem("Font").asReferenceValue().getSystemObject();
    
    String strAusrichtung = item.getItem("TextAusrichtung").asTextValue().getValueText();   
    
    SystemObject[] textDefinition = item.getItem("Textdefinition").asReferenceArray()
    .getSystemObjectArray();
    
    List<String> list = new ArrayList<String>();
    for (SystemObject so : textDefinition)
    {
      list.add(so.getPid());
    }
    
    if (font != null && strAusrichtung != null && textDefinition != null){
      
      ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
      
      int abstand = 0;
      if (connection != null){
        AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.font");
        Data cd = font.getConfigurationData(atg);
        if (cd != null){
          double dv = cd.getItem("ZeichenAbstandStandard").asScaledValue().doubleValue();
          
          if (dv > 0.0)
            abstand = (int)(1/dv);                
        }
      }
      
      re = new VisuAnzeigeKapsel(list.toArray(new String[list.size()]), font.getPid(), strAusrichtung, abstand);
    }           
        
    return re;
  } 

}
