/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.font;

import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.awt.image.DirectColorModel;
import java.awt.image.IndexColorModel;
import java.awt.image.WritableRaster;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.PaletteData;
import org.eclipse.swt.graphics.RGB;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

public class VisuAnzeigeFonts
{
  private static VisuAnzeigeFonts _instanz;


  /**
   * Fonts
   */
  HashMap<String, HashMap<String, VisuAnzeigeFontsZeichen>> m_fonts = new HashMap<String, HashMap<String, VisuAnzeigeFontsZeichen>>();

  HashMap<String, HashMap<String, HashMap<String, Integer>>> m_ligaturenFuerFonts = new HashMap<String, HashMap<String, HashMap<String, Integer>>>();

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static VisuAnzeigeFonts getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new VisuAnzeigeFonts();
    }
    return _instanz;
  }
  
  
  private VisuAnzeigeFonts()
  {    
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    DataModel dataModel = connection.getDataModel();

    HashMap<String, VisuAnzeigeFontsZeichen> m_font = null;
    VisuAnzeigeFontsZeichen zeichen = null;
    
    SystemObjectType typBuvWechseltextFont = dataModel.getType("typ.buvWechseltextFont");
    
    AttributeGroup atgBuvWechseltextFont = dataModel.getAttributeGroup("atg.buvWechseltextFont");
    AttributeGroup atgBuvWechseltextZeichen = dataModel.getAttributeGroup("atg.buvWechseltextZeichen");
    
    if (typBuvWechseltextFont != null)
    {
      List<SystemObject> fonts = typBuvWechseltextFont.getElements();
      
      for (SystemObject systemObject : fonts)
      {
        Data dataBuvWechseltextFont = systemObject.getConfigurationData(atgBuvWechseltextFont);
        
        String pidFont = null;
        
        if (dataBuvWechseltextFont != null)
        {
          pidFont = dataBuvWechseltextFont.getReferenceValue("Font").getSystemObjectPid();
        }

        m_font = new HashMap<String, VisuAnzeigeFontsZeichen>();

        ConfigurationObject font = (ConfigurationObject)systemObject;
        
        NonMutableSet mengeZeichen = font.getNonMutableSet("Zeichen");
        
        if (mengeZeichen != null)
        {
          List<SystemObject> elementeZeichen = mengeZeichen.getElements();
          
          for (SystemObject zeichen1 : elementeZeichen)
          {
            
            Data data = zeichen1.getConfigurationData(atgBuvWechseltextZeichen);
            
            ReferenceValue zeichenReferenz = data.getReferenceValue("Zeichen");
            int asciiCode = (int)data.getScaledValue("AsciiCode").longValue();
            int uniCode = (int)data.getScaledValue("UniCode").longValue();
            int bitmapHoehe = (int)data.getScaledValue("BitmapHhe").longValue();
            int bitmapBreite = (int)data.getScaledValue("BitmapBreite").longValue();
            String[] bitmap = data.getTextArray("BitmapZeilen").getTextArray();
            
            zeichen = new VisuAnzeigeFontsZeichen();
            zeichen.setZeichenname(zeichenReferenz.getSystemObjectPid());
            zeichen.setAsciiCode(asciiCode);
            zeichen.setUniCode(uniCode);
            zeichen.setBitmap(bitmap);
            zeichen.setHoehe(bitmapHoehe);
            zeichen.setBreite(bitmapBreite);
            zeichen.erstelleBitmap();            
            m_font.put(zeichen.getZeichenname(), zeichen);
          }
        }
        
        m_fonts.put(pidFont, m_font);
        
        // Ligaturen auslesen
        
        if (connection.getDataModel().getObject(pidFont) != null)
        {
          ligaturenAuslesen(connection, pidFont);
        }
      }
    }        
  }

  /**
   * @param connection
   * @param pidFont
   */
  private void ligaturenAuslesen(ClientDavInterface connection, String pidFont)
  {
    HashMap<String, HashMap<String, Integer>> ligaturen = new HashMap<String, HashMap<String, Integer>>();

    ConfigurationObject obj = (ConfigurationObject)connection.getDataModel().getObject(pidFont);

    AttributeGroup atgFont = connection.getDataModel().getAttributeGroup("atg.font");

    Data dataFont = obj.getConfigurationData(atgFont);

    int zeichenHoehe = dataFont.getScaledValue("SchrifthhePixel").intValue();

    NonMutableSet set = obj.getNonMutableSet("Ligaturen");

    List<?> list = set.getElements();

    for (int i = 0; i < list.size(); i++)
    {
      ConfigurationObject o1 = (ConfigurationObject)list.get(i);

      AttributeGroup atgLigaturen = connection.getDataModel().getAttributeGroup("atg.ligaturen");
      Data dataLigaturen = o1.getConfigurationData(atgLigaturen);

      double zeichenAbstand = dataLigaturen.getScaledArray("Zeichenabstand").getValue(0).doubleValue();

      ReferenceArray array = dataLigaturen.getReferenceArray("Zeichen");

      HashMap<String, Integer> hm1 = null;

      for (int j = 0; j < array.getLength(); j++)
      {
        ConfigurationObject o2 = (ConfigurationObject)array.getSystemObject(j);

        if (j == 0)
        {
          hm1 = (HashMap<String, Integer>)ligaturen.get(o2.getPid());

          if (hm1 == null)
          {
            hm1 = new HashMap<String, Integer>();

            ligaturen.put(o2.getPid(), hm1);
          }
        }

        if (j == 1)
        {
          Integer i1 = hm1.get(o2.getPid());

          if (i1 == null)
          {
            i1 = new Integer((int)(zeichenAbstand * zeichenHoehe + 0.5));

            hm1.put(o2.getPid(), i1);
          }
        }
      }
    }

    m_ligaturenFuerFonts.put(pidFont, ligaturen);
  }

  /**
   * Holen eines Fontzeichens aus einem Font ber die Pid des Zeichen.<br>
   * 
   * @param pidFont
   *          Pid des Font.
   * @param pidZeichen
   *          Pid des Zeichen.
   * @return Fontzeichen wenn gefunden, sonst <code>null</code>.
   */
  public VisuAnzeigeFontsZeichen getFontZeichenUeberPid(String pidFont, String pidZeichen)
  {
    HashMap<String, VisuAnzeigeFontsZeichen> font;

    font = m_fonts.get(pidFont);

    if (font != null) // Font gefunden
    {
      return (VisuAnzeigeFontsZeichen)font.get(pidZeichen);
    }
    else
    // Font nicht gefunden
    {
      return null;
    }
  }

  /**
   * Holen eines Fontzeichens aus einem Font ber den ASCII-Code des Zeichen.<br>
   * 
   * @param pidFont
   *          Pid des Font.
   * @param asciiCode
   *          ASCII-Code des Zeichen.
   * @return Fontzeichen wenn gefunden, sonst <code>null</code>.
   */
  public VisuAnzeigeFontsZeichen getFontZeichenUeberAsciiCode(String pidFont, int asciiCode)
  {
    HashMap<String, VisuAnzeigeFontsZeichen> font;

    font = m_fonts.get(pidFont);

    if (font != null) // Font gefunden
    {
      Set<String> set = font.keySet();
      Iterator<String> iterator = set.iterator();
      String pidZeichen;
      VisuAnzeigeFontsZeichen zeichen;

      while (iterator.hasNext())
      {
        pidZeichen = iterator.next();

        zeichen = font.get(pidZeichen);

        if (zeichen.getAsciiCode() == asciiCode)
        {
          return zeichen;
        }
      }
    }

    return null;
  }

  /**
   * Holen eines Fontzeichens aus einem Font ber den UNI-Code des Zeichen.<br>
   * 
   * @param pidFont
   *          Pid des Font.
   * @param uniCode
   *          UNI-Code des Zeichen.
   * @return Fontzeichen wenn gefunden, sonst <code>null</code>.
   */
  public VisuAnzeigeFontsZeichen getFontZeichenUeberUniCode(String pidFont, int uniCode)
  {
    HashMap<String, VisuAnzeigeFontsZeichen> font;

    font = m_fonts.get(pidFont);

    if (font != null) // Font gefunden
    {
      Set<String> set = font.keySet();
      Iterator<String> iterator = set.iterator();
      String pidZeichen;
      VisuAnzeigeFontsZeichen zeichen;

      while (iterator.hasNext())
      {
        pidZeichen = iterator.next();

        zeichen = font.get(pidZeichen);

        if (zeichen.getUniCode() == uniCode)
        {
          return zeichen;
        }
      }
    }

    return null;
  }

  /**
   * Holen des Abstandes zwischen zwei aufeinanderfolgenden Zeichen.<br>
   * 
   * @param pidFont
   *          Pid des Font.
   * @param pidErstesZeichen
   *          Pid des ersten Zeichen.
   * @param pidZweitesZeichen
   *          Pid des zweiten Zeichen.
   * @return Abstand zwischen den Zeichen.
   */
  public int getZeichenAbstand(String pidFont, String pidErstesZeichen, String pidZweitesZeichen, final int abstandFix)
  {
    HashMap<String, HashMap<String, Integer>> ligaturen;

    int abstand = abstandFix;

    ligaturen = m_ligaturenFuerFonts.get(pidFont);

    if (ligaturen == null) // keine Ligaturen fr Font gefunden
    {
      return abstand;
    }

    if (ligaturen.get(pidErstesZeichen) != null)
    {
      if (ligaturen.get(pidErstesZeichen).get(pidZweitesZeichen) != null)
      {
        abstand = ((Integer)ligaturen.get(pidErstesZeichen).get(pidZweitesZeichen)).intValue();
      }
    }

    return abstand;
  }

  /**
   * Erstellt aus den bergebenen Zeicheninformationen eine Bitmap <br>
   * 
   * @param pidZeichen
   *          Die Pids der Zeichen
   * @param ausrichtung
   *          Ausrichtung der Zeichen
   * @param pidFont
   *          Pid des Font
   * @param pixelBreite
   *          Pixelbreite
   * @param pixelHoehe
   *          Pixelhhe
   * @return die Bitmap als Image
   */
  public BufferedImage erstelleBitmap(String[] pidZeichen, String ausrichtung, String pidFont, int pixelBreite,
      int pixelHoehe, final int abstandFix)
  {
    BufferedImage bitmap = new BufferedImage(pixelBreite, pixelHoehe, BufferedImage.TYPE_BYTE_BINARY);
    int start = 0;
    int abstand = abstandFix;

    int laengeText = 0;

    for (int i = 0; i < pidZeichen.length; i++)
    {
      VisuAnzeigeFontsZeichen zeichen = getFontZeichenUeberPid(pidFont, pidZeichen[i]);

      if (i < pidZeichen.length - 1)
      {
        abstand = getZeichenAbstand(pidFont, pidZeichen[i], pidZeichen[i + 1], abstandFix);
      }
//      else
//      {
//        abstand = 0;
//      }

      if (zeichen != null)
      {
        laengeText += zeichen.getBreite();
      }

      laengeText += abstand;
    }

    if (ausrichtung.equals("Rechtsbndig"))
    {
      start = pixelBreite - laengeText;
    }

    if (ausrichtung.equals("Zentriert"))
    {
      start = (pixelBreite - laengeText) / 2;
    }

    Graphics2D grafik = bitmap.createGraphics();

    for (int i = 0; i < pidZeichen.length; i++)
    {
      VisuAnzeigeFontsZeichen zeichen = getFontZeichenUeberPid(pidFont, pidZeichen[i]);
      BufferedImage bm = null;

      if (i < pidZeichen.length - 1)
      {
        abstand = getZeichenAbstand(pidFont, pidZeichen[i], pidZeichen[i + 1], abstandFix);         
      }

      if (zeichen != null)
      {
        bm = zeichen.getBufferedImage();         
      }

      if (bm != null)
      {
        grafik.drawImage(bm, null, start, 0);

        start += zeichen.getBreite();
        start += abstand;
      }
    }

    grafik.dispose();

    return bitmap;
  } // erstelleBitmap

  public ImageData convertBufferedImageToSWT(BufferedImage bufferedImage)
  {
    if (bufferedImage.getColorModel() instanceof DirectColorModel)
    {
      DirectColorModel colorModel = (DirectColorModel)bufferedImage.getColorModel();
      PaletteData palette = new PaletteData(colorModel.getRedMask(), colorModel.getGreenMask(), colorModel
          .getBlueMask());
      ImageData data = new ImageData(bufferedImage.getWidth(), bufferedImage.getHeight(), colorModel.getPixelSize(),
          palette);
      WritableRaster raster = bufferedImage.getRaster();
      int[] pixelArray = new int[3];
      for (int y = 0; y < data.height; y++)
      {
        for (int x = 0; x < data.width; x++)
        {
          raster.getPixel(x, y, pixelArray);
          int pixel = palette.getPixel(new RGB(pixelArray[0], pixelArray[1], pixelArray[2]));
          data.setPixel(x, y, pixel);
        }
      }
      return data;
    }
    else if (bufferedImage.getColorModel() instanceof IndexColorModel)
    {
      IndexColorModel colorModel = (IndexColorModel)bufferedImage.getColorModel();
      int size = colorModel.getMapSize();
      byte[] reds = new byte[size];
      byte[] greens = new byte[size];
      byte[] blues = new byte[size];
      colorModel.getReds(reds);
      colorModel.getGreens(greens);
      colorModel.getBlues(blues);
      RGB[] rgbs = new RGB[size];
      for (int i = 0; i < rgbs.length; i++)
      {
        rgbs[i] = new RGB(reds[i] & 0xFF, greens[i] & 0xFF, blues[i] & 0xFF);
      }
      PaletteData palette = new PaletteData(rgbs);
      ImageData data = new ImageData(bufferedImage.getWidth(), bufferedImage.getHeight(), colorModel.getPixelSize(),
          palette);
      data.transparentPixel = colorModel.getTransparentPixel();
      WritableRaster raster = bufferedImage.getRaster();
      int[] pixelArray = new int[1];
      for (int y = 0; y < data.height; y++)
      {
        for (int x = 0; x < data.width; x++)
        {
          raster.getPixel(x, y, pixelArray);
          data.setPixel(x, y, pixelArray[0]);
        }
      }
      return data;
    }
    return null;
  }

  public Image erstelleBitmapSWT(String[] pidZeichen, String ausrichtung, String pidFont, int pixelBreite,
      int pixelHoehe, final int abstandFix)
  {    
    Image swtImage = null;
    if (pidZeichen != null){
      BufferedImage image = erstelleBitmap(pidZeichen, ausrichtung, pidFont, pixelBreite, pixelHoehe, abstandFix);
      ImageData data = convertBufferedImageToSWT(image);       
      swtImage = new Image(null, data);      
    }

    return swtImage;
  }

  public int berechneTextLaenge(String[] pidZeichen, String pidFont, int abstand)
  {
//    int abstand = 4;

    int laengeText = 0;

    for (int i = 0; i < pidZeichen.length; i++)
    {
      VisuAnzeigeFontsZeichen zeichen = getFontZeichenUeberPid(pidFont, pidZeichen[i]);

      if (i < pidZeichen.length - 1)
      {
        abstand = getZeichenAbstand(pidFont, pidZeichen[i], pidZeichen[i + 1], abstand);
      }
      else
      {
        abstand = 0;
      }

      if (zeichen != null)
      {
        laengeText += zeichen.getBreite();
      }

      laengeText += abstand;
    } // for i

    return laengeText;
  } // berechneTextLaenge

}
