/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.views;

import java.io.File;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import net.sf.paperclips.Print;
import net.sf.paperclips.PrintJob;

import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.pua.basis.ATools;
import de.bsvrz.buv.plugin.pua.basis.Activator;
import de.bsvrz.buv.plugin.pua.basis.DaVKommunikation;
import de.bsvrz.buv.plugin.pua.basis.ExportCSV;
import de.bsvrz.buv.plugin.pua.basis.ExportXml;
import de.bsvrz.buv.plugin.pua.basis.ProtokollInfo;
import de.bsvrz.buv.plugin.pua.basis.archivDataUpdateListener.ArchiveDataUpdateEvent;
import de.bsvrz.buv.plugin.pua.basis.archivDataUpdateListener.IArchiveDataUpdateListener;
import de.bsvrz.buv.plugin.pua.basis.dav.AnfrageDaten;
import de.bsvrz.buv.plugin.pua.basis.dav.ArchiveRequest;
import de.bsvrz.buv.plugin.pua.basis.dav.ArchiveRequestStarter;
import de.bsvrz.buv.plugin.pua.basis.pua.ProtokolleUndAuswertungen;
import de.bsvrz.buv.plugin.pua.basis.pua.daten.ProtokollInformationen;
import de.bsvrz.buv.rw.basislib.kalender.KalenderZeitFilterDatenStrukturZeitGruppe;
import de.bsvrz.buv.rw.basislib.seiteneinstellungen.druck.vorschau.DruckvorschauDialog;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.archive.ArchiveData;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKindCombination;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.losb.util.Tuple;

/**
 * View zur Anzeige der Ergebnisse der Anforderung Stoerungen
 * 
 * @author ObertM
 * 
 */
public class ArchivErgebnisAuswertungSicht extends ViewPart implements IArchiveDataUpdateListener
{

  private LinkedList<String[]> _exportDaten;

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Die ID der View
   */
  public static final String ID = "de.bsvrz.buv.plugin.pua.viewErgebnisAuswertungRueckmeldungen";

  /**
   * Merker
   */
  private boolean _isDisposed = false;

  /**
   * TableViewer
   */
  private static TableViewer _tv = null;

  /**
   * Gibt die Sortierreihenfolge der Spalten an.
   */
  private static int[] _tvSpaltenSortierung = null;

  private Composite top = null;

  private Composite compositeTop = null;

  private Composite composite = null;

  private Composite composite3 = null;

  private Button buttonSchliessen = null;

  private Button buttonHilfe = null;

  private Label labelZeit = null;

  private Label labelStatus = null;

  // private List<WzgStellZustandDaten> _meldungsList = null;

  private boolean _debugAusgabenEmpfang = false;

  private TableSort _ts;

  private Label labelObjekte;

  private Button buttonExport;

  private AnfrageDaten _anfrageDaten;

  private Button buttonDrucken;

  private ClientDavInterface _connection = null;

  private SimpleDateFormat _sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");

  /**
   * Fr die Archivanfrage.
   */
  private ArchiveRequest _archiveRequest = null;

  private static final String HELP_ID = Activator.PLUGIN_ID + ".schaltungen";

  private Map<String, Integer> _mapAnzeigeSpalte = new HashMap<String, Integer>() {
    {
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_07.anzeige1", 0);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_07.anzeige3", 1);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_07.anzeige2", 2);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_07.anzeige4", 3);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_07.anzeige6", 4);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_07.anzeige5", 5);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_07.anzeige7", 6);

      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_08.anzeige1", 0);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_08.anzeige3", 1);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_08.anzeige2", 2);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_08.anzeige4", 3);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_08.anzeige6", 4);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_08.anzeige5", 5);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_08.anzeige7", 6);

      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_09.anzeige1", 0);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_09.anzeige3", 1);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_09.anzeige2", 2);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_09.anzeige4", 3);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_09.anzeige6", 4);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_09.anzeige5", 5);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_09.anzeige7", 6);

      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_1.anzeige1", 0);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_1.anzeige3", 1);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_1.anzeige2", 2);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_1.anzeige4", 3);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_1.anzeige6", 4);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_1.anzeige5", 5);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_1.anzeige7", 6);

      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_2.anzeige1", 0);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_2.anzeige3", 1);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_2.anzeige2", 2);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_2.anzeige4", 3);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_2.anzeige6", 4);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_2.anzeige5", 5);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq1_2.anzeige7", 6);

      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq2_1.anzeige1", 0);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq2_1.anzeige3", 1);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq2_1.anzeige2", 2);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq2_1.anzeige4", 3);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq2_1.anzeige6", 4);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq2_1.anzeige5", 5);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq2_1.anzeige7", 6);

      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq2_2.anzeige1", 1);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq2_2.anzeige2", 2);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq2_2.anzeige3", 7);

      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq3_1.anzeige1", 0);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq3_1.anzeige3", 1);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq3_1.anzeige2", 2);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq3_1.anzeige4", 3);

      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq3_2.anzeige1", 0);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq3_2.anzeige3", 1);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq3_2.anzeige2", 2);
      put("nw.sba.anzeigeQuerschnitt.uz.koeln.aq3_2.anzeige4", 3);

    }
  };

  private Map<String, String> _mapBildinhaltText = new HashMap<String, String>() {
    {
      put("Dunkel", "Aus");
      put("SBA.Dunkel", "Aus");
      put("TLS.1000Meter", "1000 m");
      put("TLS.2000Meter", "2000 m");
      put("TLS.7Komma5Tonnen", "7,5 t");
      put("TLS.Baustelle", "Baustelle");
      put("TLS.EndeAllerStreckenverbote", "Ende alles");
      put("TLS.EndeUeberholverbotAllgemein", "Ende V");
      put("TLS.EndeUeberholverbotLkw", "Ende V Lkw");
      put("TLS.FahrstreifenGesperrt", "Gesperrt");
      put("TLS.FahrstreifenHalten", "Halten");
      put("TLS.FahrstreifenWechselnNachLinks", "Links");
      put("TLS.FahrstreifenWechselnNachRechts", "Rechts");
      put("TLS.Gefahrenstelle", "Gefahr");
      put("TLS.GelbesBlinklicht", "Blinken");
      put("TLS.Glaettegefahr", "Gltte");
      put("TLS.Nebel", "Nebel");
      put("TLS.Schleudergefahr", "Schleuder");
      put("TLS.Stau", "Stau");
      put("TLS.UeberholverbotAllgemein", "V");
      put("TLS.UeberholverbotLkw", "V Lkw");
      put("TLS.Unfall", "Unfall");
      put("TLS.ZulaessigeHoechst100", "100 km/h");
      put("TLS.ZulaessigeHoechst120", "120 km/h");
      put("TLS.ZulaessigeHoechst40", "40 km/h");
      put("TLS.ZulaessigeHoechst60", "60 km/h");
      put("TLS.ZulaessigeHoechst80", "80 km/h");
    }
  };

  @Override
  public void createPartControl(Composite parent)
  {
    _connection = DaVKommunikation.getInstanz().holeDaVVerbindung();

    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;
    top = new Composite(parent, SWT.NONE);

    createCompositeTop();
    top.setLayout(gridLayout);

    PlatformUI.getWorkbench().getHelpSystem().setHelp(top, HELP_ID);

    createCompositeTable();

    createCompositeBottom();

    String secondaryId = getViewSite().getSecondaryId();

    if (secondaryId != null)
      setPartName("Auswertung " + secondaryId);

    // ArchiveDataUpdateListener.getInstanz().addArchiveDataUpdateListener(this);
  }

  @Override
  public void setFocus()
  {
    if (top != null)
      top.setFocus();
  }

  /**
   * ffnet die Hilfe zum Dialog {@link MeldungsanzeigeDialog}.
   * 
   */
  private void aktionHilfe()
  {
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(HELP_ID);
  }

  /**
   * Schliet den Dialog {@link MeldungsanzeigeDialog}.
   * 
   * Betriebsmeldungen werden auch nach dem Schlieen des Dialogs gespeichert, so dass beim nchsten ffnen des Fensters
   * alle bisher empfangenen Betriebsmeldungen (bis zur maximalen Anzahl) dargestellt werden.
   * 
   */
  private void aktionSchliessen()
  {
    _debug.fine("Schliessen");
    getViewSite().getPage().hideView(ArchivErgebnisAuswertungSicht.this);
  }

  /**
   * This method initializes compositeTop
   * 
   */
  private void createCompositeTop()
  {
    GridData gridData11 = new GridData();
    gridData11.horizontalAlignment = GridData.BEGINNING;
    gridData11.grabExcessHorizontalSpace = false;
    gridData11.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout1 = new GridLayout();
    gridLayout1.numColumns = 1;
    GridData gridData = new GridData();
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.CENTER;
    gridData.horizontalAlignment = GridData.FILL;
    compositeTop = new Composite(top, SWT.NONE);
    compositeTop.setLayoutData(gridData);
    compositeTop.setLayout(gridLayout1);

    labelObjekte = new Label(compositeTop, SWT.NONE);
    labelObjekte.setText("Objekte: ");
    labelObjekte.setLayoutData(gridData11);

    labelZeit = new Label(compositeTop, SWT.NONE);
    labelZeit.setText("Zeitbereich: ");

    labelStatus = new Label(compositeTop, SWT.NONE);
    labelStatus.setText("Status: ");
  }

  /**
   * This method initializes composite
   * 
   */
  private void createCompositeTable()
  {
    GridData gridData1 = new GridData();
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.verticalAlignment = GridData.FILL;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.horizontalAlignment = GridData.FILL;
    composite = new Composite(top, SWT.NONE);
    composite.setLayout(new FillLayout());
    composite.setLayoutData(gridData1);

    erzeugeTabelle();
  }

  /**
   * This method initializes composite3
   * 
   */
  private void createCompositeBottom()
  {
    RowLayout rowLayout1 = new RowLayout();
    rowLayout1.pack = false;
    rowLayout1.justify = false;
    rowLayout1.fill = false;
    rowLayout1.wrap = false;
    GridData gridData2 = new GridData();
    gridData2.grabExcessHorizontalSpace = false;
    gridData2.verticalAlignment = GridData.CENTER;
    gridData2.horizontalAlignment = GridData.END;

    composite3 = new Composite(top, SWT.NONE);
    composite3.setLayoutData(gridData2);
    composite3.setLayout(rowLayout1);

    buttonDrucken = new Button(composite3, SWT.NONE);
    buttonDrucken.setText("Drucken");
    buttonDrucken.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
      public void mouseDown(org.eclipse.swt.events.MouseEvent e)
      {        
        Print print = Tools.createPrint(_tv.getTable(), "Auswertung AQs");
        PrintJob printJob = new PrintJob(getPartName(), print);
        DruckvorschauDialog dvd = new DruckvorschauDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell());
        dvd.oeffne(printJob);
      }
    });

    buttonExport = new Button(composite3, SWT.NONE);
    buttonExport.setText("Export");
    buttonExport.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
      public void mouseDown(org.eclipse.swt.events.MouseEvent e)
      {
        aktionExport();
      }

    });

    buttonSchliessen = new Button(composite3, SWT.NONE);
    buttonSchliessen.setText("Schlieen");
    buttonSchliessen.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
      public void mouseDown(org.eclipse.swt.events.MouseEvent e)
      {
        aktionSchliessen();
      }
    });

    buttonHilfe = new Button(composite3, SWT.NONE);
    buttonHilfe.setText("Hilfe");
    buttonHilfe.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
      public void mouseDown(org.eclipse.swt.events.MouseEvent e)
      {
        aktionHilfe();
      }
    });
  }

private String getZuordnung(int anzeigeIndex) {
	  String result = null;
	  
	  if (_connection != null) {
		  DataModel dataModel = _connection.getDataModel();
		  AttributeGroup atg = dataModel.getAttributeGroup("atg.sbaAnzeige");
		  
		  for (Map.Entry<String, Integer> entry: _mapAnzeigeSpalte.entrySet()) {
			  Integer index = entry.getValue();
			  
			  if (index != null && index.intValue() == anzeigeIndex)  {
				  String pid = entry.getKey();
				  SystemObject anzeige = dataModel.getObject(pid);
				  if (anzeige != null) {
					  
					  for (SystemObject sbaAnzeige: dataModel.getType("typ.sbaAnzeige").getElements())
					  {
						  Data data = sbaAnzeige.getConfigurationData(atg);
						  if (data != null) {
							  if (anzeige == data.getReferenceValue("Anzeige").getSystemObject()) {
								  result = data.getTextValue("Zuordnung").getText();
								  break;  
							  }
						  }
					  }
					  if (result != null && !result.isEmpty()) {
						  break;
					  }
				  }
			  }
		  }
	  }
	  
	  return result;
  }

  private void erzeugeTabelle()
  {
    _tv = new TableViewer(composite, SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
    _ts = new TableSort(_tv);

    Table _tabelle = _tv.getTable();

    TableViewerColumn tvc0 = new TableViewerColumn(_tv, SWT.LEFT);
    tvc0.setLabelProvider(new ToolTipCellLabelProvider());
    TableColumn tc0 = tvc0.getColumn();
    tc0.setText("Zeit");
    tc0.setWidth(150);
    tc0.addListener(SWT.Selection, new Listener() {
      public void handleEvent(Event event)
      {
        _ts.aendereSortierung();
        _ts.sortiere(0, TableSort.SORT.TIME);
      }
    });

    TableViewerColumn tvc1 = new TableViewerColumn(_tv, SWT.LEFT);
    tvc1.setLabelProvider(new ToolTipCellLabelProvider());
    TableColumn tc1 = tvc1.getColumn();
    tc1.setText("Objekt");
    tc1.setWidth(150);
    tc1.addListener(SWT.Selection, new Listener() {
      public void handleEvent(Event event)
      {
        _ts.aendereSortierung();
        _ts.sortiere(1, TableSort.SORT.STRING);
      }
    });

    TableViewerColumn tvc2 = new TableViewerColumn(_tv, SWT.LEFT);
    tvc2.setLabelProvider(new ToolTipCellLabelProvider());
    TableColumn tc2 = tvc2.getColumn();
    tc2.setText("A-Typ HFS");
    tc2.setWidth(150);

    TableViewerColumn tvc3 = new TableViewerColumn(_tv, SWT.LEFT);
    tvc3.setLabelProvider(new ToolTipCellLabelProvider());
    TableColumn tc3 = tvc3.getColumn();
    tc3.setText("B-Typ HFS/1FS");
    tc3.setWidth(150);

    TableViewerColumn tvc4 = new TableViewerColumn(_tv, SWT.LEFT);
    tvc4.setLabelProvider(new ToolTipCellLabelProvider());
    TableColumn tc4 = tvc4.getColumn();
    tc4.setText("C-Typ HFS/1FS");
    tc4.setWidth(150);

    TableViewerColumn tvc5 = new TableViewerColumn(_tv, SWT.LEFT);
    tvc5.setLabelProvider(new ToolTipCellLabelProvider());
    TableColumn tc5 = tvc5.getColumn();
    tc5.setText("A-Typ 1FS");
    tc5.setWidth(150);

    TableViewerColumn tvc6 = new TableViewerColumn(_tv, SWT.LEFT);
    tvc6.setLabelProvider(new ToolTipCellLabelProvider());
    TableColumn tc6 = tvc6.getColumn();
    tc6.setText("B-Typ 1FS/2FS");
    tc6.setWidth(150);

    TableViewerColumn tvc7 = new TableViewerColumn(_tv, SWT.LEFT);
    tvc7.setLabelProvider(new ToolTipCellLabelProvider());
    TableColumn tc7 = tvc7.getColumn();
    tc7.setText("C-Typ 1FS/2FS");
    tc7.setWidth(150);

    TableViewerColumn tvc8 = new TableViewerColumn(_tv, SWT.LEFT);
    tvc8.setLabelProvider(new ToolTipCellLabelProvider());
    TableColumn tc8 = tvc8.getColumn();
    tc8.setText("A-Typ 2FS");
    tc8.setWidth(150);

    TableViewerColumn tvc9 = new TableViewerColumn(_tv, SWT.LEFT);
    tvc9.setLabelProvider(new ToolTipCellLabelProvider());
    TableColumn tc9 = tvc9.getColumn();
    tc9.setText("Blinker HFS/1FS");
    tc9.setWidth(150);

    _tv.getTable().setHeaderVisible(true);
    _tv.getTable().setLinesVisible(true);
  }

  @Override
  public void dispose()
  {
    _isDisposed = true;

    if (_archiveRequest != null)
    {
      _archiveRequest.removeArchiveDataUpdateListener(this);
      _archiveRequest.stopRequestThread();
    }

    super.dispose();
  }

  @Override
  public void archiveDataUpdate(ArchiveDataUpdateEvent e)
  {
    if (_isDisposed)
    {
      System.out.println(ArchivErgebnisAuswertungSicht.ID + " ist disposed!!!");
      return;
    }

    ArchiveData result = e.getResult();

    if (result != null)
    {
      Data data = result.getData();
      String pid = result.getObject().getPid();

      if (data != null)
      {
        AttributeGroup atg = result.getDataDescription().getAttributeGroup();

        if (atg.getPid().equals("atg.anzeigeEigenschaftIst"))
        {
          SystemObject so = data.getItem("Eigenschaft").getItem("AnzeigeInhalt").asReferenceValue().getSystemObject();

          String inhalt = "";
          String text = "";
          if (so != null)
          {
            ConfigurationObject co = (ConfigurationObject)so;
            AttributeGroup atg2 = _connection.getDataModel().getAttributeGroup("atg.wvzInhalt");

            Data data2 = co.getConfigurationData(atg2);
            inhalt = data2.getItem("Bildinhalt").asTextValue().getValueText();
            if (inhalt != null && inhalt.length() > 0)
            {
              text = _mapBildinhaltText.get(inhalt);

              if (text != null && text.length() > 0)
              {
                // System.out.println("gefunden: ");
              }
              else
                System.out.println("nicht gefunden: " + inhalt);

            }
            else
              text = inhalt;
          }

          SystemObjectType type = _connection.getDataModel().getType("typ.anzeigeQuerschnitt");

          List<SystemObject> aqs = type.getElements();

          for (SystemObject aq : aqs)
          {
            ConfigurationObject co = (ConfigurationObject)aq;
            NonMutableSet set = co.getNonMutableSet("Anzeigen");
            List<SystemObject> azs = set.getElements();
            for (SystemObject az : azs)
            {
              if (az.equals(result.getObject()))
              {
                TableItem[] items = _tv.getTable().getItems();

                boolean gefunden = false;

                for (TableItem ti : items)
                {
                  String t = ti.getText(0);
                  Long l1;
                  Long l2;
                  try
                  {
                    l1 = _sdf.parse(t).getTime();
                    String text2 = _sdf.format(new Date(result.getArchiveTime()));
                    l2 = _sdf.parse(text2).getTime();

                    if (l1.equals(l2) && aq.getName().equals(ti.getText(1)))
                    {
                      ti.setText(0, _sdf.format(new Date(result.getArchiveTime())));
                      ti.setText(1, aq.getName());

                      Integer i = _mapAnzeigeSpalte.get(pid);

                      if (i != null)
                      {
                        ti.setData(new Integer(2 + i).toString(), inhalt);
                        ti.setText(2 + i, text);
                      }

                      gefunden = true;
                    }
                  }
                  catch (ParseException e1)
                  {
                    // TODO Auto-generated catch block
                    e1.printStackTrace();
                  }
                }

                if (!gefunden)
                {
                  TableItem ti = new TableItem(_tv.getTable(), SWT.NONE);
                  ti.setText(0, _sdf.format(new Date(result.getArchiveTime())));
                  ti.setText(1, aq.getName());
                  Integer i = _mapAnzeigeSpalte.get(pid);
                  if (i != null)
                  {
                    ti.setData(new Integer(2 + i).toString(), inhalt);
                    ti.setText(2 + i, text);
                  }
                }
              }
            }
          }
        }
        else if (atg.getPid().equals("atg.anzeigeEigenschaftSoll"))
        {
          SystemObject so = data.getItem("Eigenschaft").getItem("AnzeigeInhalt").asReferenceValue().getSystemObject();

          String inhalt = "";
          String text = "";
          if (so != null)
          {
            ConfigurationObject co = (ConfigurationObject)so;
            AttributeGroup atg2 = _connection.getDataModel().getAttributeGroup("atg.wvzInhalt");

            Data data2 = co.getConfigurationData(atg2);
            inhalt = data2.getItem("Bildinhalt").asTextValue().getValueText();
            if (inhalt != null && inhalt.length() > 0)
            {
              text = _mapBildinhaltText.get(inhalt);

              if (text != null && text.length() > 0)
              {
                // System.out.println("gefunden: ");
              }
              else
                System.out.println("nicht gefunden: " + inhalt);

            }
            else
              text = inhalt;
          }

          SystemObjectType type = _connection.getDataModel().getType("typ.anzeigeQuerschnitt");

          List<SystemObject> aqs = type.getElements();

          for (SystemObject aq : aqs)
          {
            ConfigurationObject co = (ConfigurationObject)aq;
            NonMutableSet set = co.getNonMutableSet("Anzeigen");
            List<SystemObject> azs = set.getElements();
            for (SystemObject az : azs)
            {
              if (az.equals(result.getObject()))
              {
                TableItem[] items = _tv.getTable().getItems();

                boolean gefunden = false;

                for (TableItem ti : items)
                {
                  String t = ti.getText(0);
                  Long l1;
                  Long l2;
                  try
                  {
                    l1 = _sdf.parse(t).getTime();
                    String text2 = _sdf.format(new Date(result.getArchiveTime()));
                    l2 = _sdf.parse(text2).getTime();

                    if (l1.equals(l2) && aq.getName().equals(ti.getText(1)))
                    {
                      ti.setText(0, _sdf.format(new Date(result.getArchiveTime())));
                      ti.setText(1, aq.getName());

                      Integer i = _mapAnzeigeSpalte.get(pid);

                      if (i != null)
                      {
                        ti.setData(new Integer(2 + i).toString(), inhalt);
                        ti.setText(2 + i, text);
                      }

                      gefunden = true;
                    }
                  }
                  catch (ParseException e1)
                  {
                    // TODO Auto-generated catch block
                    e1.printStackTrace();
                  }
                }

                if (!gefunden)
                {
                  TableItem ti = new TableItem(_tv.getTable(), SWT.NONE);
                  ti.setText(0, _sdf.format(new Date(result.getArchiveTime())));
                  ti.setText(1, aq.getName());
                  Integer i = _mapAnzeigeSpalte.get(pid);
                  if (i != null)
                  {
                    ti.setData(new Integer(2 + i).toString(), inhalt);
                    ti.setText(2 + i, text);
                  }
                }
              }
            }
          }
        }else if (atg.getPid().equals("atg.sbaUeSchaltbild"))
        {
          SystemObject aq = result.getObject();
    
          Array array = data.getItem("Anzeigen").asArray();
          for (int j = 0; j < array.getLength(); j++)
          {
            Data item = array.getItem(j);       
            pid = item.getItem("Anzeige").asReferenceValue().getSystemObjectPid();
            SystemObject so = item.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue().getSystemObject();
            String inhalt = "";
            String text = "";
            if (so != null)
            {
              ConfigurationObject co = (ConfigurationObject)so;
              AttributeGroup atg2 = _connection.getDataModel().getAttributeGroup("atg.wvzInhalt");

              Data data2 = co.getConfigurationData(atg2);
              inhalt = data2.getItem("Bildinhalt").asTextValue().getValueText();
              if (inhalt != null && inhalt.length() > 0)
              {
                text = _mapBildinhaltText.get(inhalt);

                if (text != null && text.length() > 0)
                {
                  // System.out.println("gefunden: ");
                }
                else
                  System.out.println("nicht gefunden: " + inhalt);

              }
              else
                text = inhalt;

              // if (so.equals(result.getObject()))
              {
                TableItem[] items = _tv.getTable().getItems();

                boolean gefunden = false;

                for (TableItem ti : items)
                {
                  String t = ti.getText(0);
                  Long l1;
                  Long l2;
                  try
                  {
                    l1 = _sdf.parse(t).getTime();
                    String text2 = _sdf.format(new Date(result.getArchiveTime()));
                    l2 = _sdf.parse(text2).getTime();

                    if (l1.equals(l2) && aq.getName().equals(ti.getText(1)))
                    {
                      ti.setText(0, _sdf.format(new Date(result.getArchiveTime())));
                      ti.setText(1, aq.getName());

                      Integer index = _mapAnzeigeSpalte.get(pid);

                      if (index != null)
                      {
                        ti.setData(new Integer(2 + index).toString(), inhalt);
                        ti.setText(2 + index, text);
                      }

                      gefunden = true;
                    }
                  }
                  catch (ParseException e1)
                  {
                    // TODO Auto-generated catch block
                    e1.printStackTrace();
                  }
                }

                if (!gefunden)
                {
                  TableItem ti = new TableItem(_tv.getTable(), SWT.NONE);
                  ti.setText(0, _sdf.format(new Date(result.getArchiveTime())));
                  ti.setText(1, aq.getName());
                  Integer index = _mapAnzeigeSpalte.get(pid);
                  if (index != null)
                  {
                    ti.setData(new Integer(2 + index).toString(), inhalt);
                    ti.setText(2 + index, text);
                  }
                }
              }
            }
          }
        }
      }
    }
    else
    {
      if (_tv != null)
      {
        _ts.sortiere(0, TableSort.SORT.TIME);
        ATools.spaltenBreiteOptimieren(_tv);
      }

      String s3 = "Status: Ausgabe abgeschlossen";
      final String s = s3;

      labelStatus.setText(s);

      compositeTop.layout(true);
      top.layout(true);

    }
  }

  public void setAnfrageDaten(List<AnfrageDaten> anfrageDaten)
  {
    if (anfrageDaten != null)
    {
      _archiveRequest = new ArchiveRequest();
      _archiveRequest.addArchiveDataUpdateListener(this);
      // _archiveRequest.startRequest(anfrageDaten);
      new ArchiveRequestStarter(_archiveRequest, anfrageDaten);
    }

    if (anfrageDaten.size() > 0)
      setAnfrageDaten(anfrageDaten.get(0));
  }

  public void setAnfrageDaten(AnfrageDaten anfrageDaten)
  {
    _anfrageDaten = anfrageDaten;

    // Angewhlte Objekte ausgeben

    List<SystemObject> objekte = anfrageDaten.getObjekteAnzeige();

    String s1 = "Objekte: ";

    int anzahl = 0;
    if (objekte.size() > 0)
    {
      for (SystemObject so : objekte)
      {
        if (anzahl == 0)
          s1 += so.getName();
        else
          s1 += ", " + so.getName();

        anzahl++;
      }
    }

    String s21 = s1;
    if (s1.length() > 60){
      s21 = s1.substring(0, 59) + " ...";
    }
       
    
//    final String fs1 = s1;
    labelObjekte.setText(s21);
    labelObjekte.setToolTipText(s1);

    // Zeitbereich

    List<KalenderZeitFilterDatenStrukturZeitGruppe> zeitgruppen = anfrageDaten.getZeitgruppen();

    SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
    Date d1 = new Date();
    Date d2 = new Date();

    String s2 = "";

    if (zeitgruppen.size() > 0)
    {
      for (KalenderZeitFilterDatenStrukturZeitGruppe zg : zeitgruppen)
      {
        d1.setTime(zg.getStartzeitpunkt());
        d2.setTime(zg.getEndzeitpunkt());
        s2 += "Zeitbereich: " + sdf.format(d1) + " - " + sdf.format(d2) + "\n";
      }
    }
    else
    {
      s2 = "Zeitbereich: ";
    }

    final String fs2 = s2;
    labelZeit.setText(fs2);

    // Status der Ausgabe

    String s3 = "Status: Ausgabe in Bearbeitung";

    final String fs3 = s3;
    labelStatus.setText(fs3);

    compositeTop.layout(true);
    top.layout(true);
  }

  /**
   * Generiert eine grafische Fehlermeldung und zeigt diese an.
   * 
   * @param titel
   *          Titel der Fehlermeldung
   * @param meldung
   *          Mitteilung der Fehlermeldung
   */
  private void generiereMeldungFehler(String titel, String meldung)
  {
    MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), SWT.OK | SWT.ICON_ERROR);
    mb.setText(titel);
    mb.setMessage(meldung);
    mb.open();
  }

//  /**
//   * This function would print the document witout the print preview.
//   * 
//   * @param doc
//   */
//  public void print(PDocument doc)
//  {
//    PrintDialog dialog = new PrintDialog(composite3.getShell(), SWT.BORDER);
//    PrinterData data = dialog.open();
//    if (data == null)
//      return;
//
//    if (data.printToFile)
//    {
//      FileDialog d = new FileDialog(composite3.getShell(), SWT.SAVE);
//      d.setFilterNames(new String[] { "All Files (*.*)" }); //$NON-NLS-1$
//      d.setFilterExtensions(new String[] { "*.*" }); // Windows wild cards
//      // //$NON-NLS-1$
//      d.setFilterPath("c:\\"); // Windows path //$NON-NLS-1$
//      d.setFileName(""); //$NON-NLS-1$
//      d.open();
//      data.fileName = d.getFilterPath() + "\\" + d.getFileName(); //$NON-NLS-1$
//    }
//
//    if (data.printToFile)
//    {
//      data.fileName = "print.out";
//    }
//
//    Printer printer = new Printer(data);
//    GC gc = new GC(printer);
//    PBox.setParameters(gc, printer, printer.getDPI(), (int)(doc.getScale() * 100));
//
//    if (printer.startJob(doc.getTitle()))
//    {
//      if (data.scope == PrinterData.ALL_PAGES)
//      {
//        data.startPage = 1;
//        data.endPage = doc.getNumOfPages();
//      }
//      else if (data.scope == PrinterData.SELECTION)
//      {
//        // data.startPage = page;
//        // data.endPage = page;
//      }
//      else if (data.scope == PrinterData.PAGE_RANGE)
//      {
//        if (data.startPage > doc.getNumOfPages())
//          data.startPage = doc.getNumOfPages();
//        if (data.endPage > doc.getNumOfPages())
//          data.endPage = doc.getNumOfPages();
//      }
//      printer.startPage();
//      doc.layout();
//      doc.getNumOfPages();
//      for (int i = 0; i < doc.getNumOfPages(); i++)
//      {
//        if (i != 0)
//          printer.startPage();
//        doc.draw(i + 1);
//        printer.endPage();
//
//      }
//
//      printer.endJob();
//
//    }
//
//    gc.dispose();
//    printer.dispose();
//  }

  // /**
  // * ffnet den Dialog zur Auswahl der Werte fr das Balkendiagramm und stellt
  // * das Balkendiagramm dar.
  // */
  // @SuppressWarnings("unused")
  // private void aktionBalkenDiagramm()
  // {
  // SpaltenDefinition defObjekt = new SpaltenDefinition("Objekt",
  // "Objekt zur Zuordnung der Werte.", _tv.getTable().getColumns()[1]);
  // SpaltenDefinition defWert = new SpaltenDefinition("Werte",
  // "Spalte mit dem einem Objekt zuordnenbaren Werten.");
  // LinkedList<SpaltenDefinition> lSpaltenDefinitionen = new LinkedList<SpaltenDefinition>();
  // lSpaltenDefinitionen.add(defObjekt);
  // lSpaltenDefinitionen.add(defWert);
  //
  // DialogSpaltenWaehler dlg = new DialogSpaltenWaehler(Display.getCurrent()
  // .getActiveShell(), "Spaltenauswahl fr Balkendiagramm", _tv.getTable(),
  // lSpaltenDefinitionen);
  // TableColumn[] columns = _tv.getTable().getColumns();
  //
  // if (dlg.open() == Window.OK)
  // {
  //
  // int idxSpalteObjekt = -1;
  // int idxSpalteVal = -1;
  // int i = 0;
  // for (TableColumn tableColumn : columns)
  // {
  // if (tableColumn == defObjekt.getTableColumn())
  // {
  // idxSpalteObjekt = i;
  // }
  //
  // if (tableColumn == dlg
  // .getSelectedTableColumnForSpaltenDefinition(defWert))
  // {
  // idxSpalteVal = i;
  // }
  //
  // i++;
  //
  // if ((idxSpalteObjekt > -1) && (idxSpalteVal > -1))
  // {
  // break;
  // }
  // }
  //
  // if ((idxSpalteObjekt > -1) && (idxSpalteVal > -1))
  // {
  // TreeMap<String, TreeMap<String, Double>> chartInput = ChartTools
  // .erzeugeChartInputBalkenDiagramm(_tv.getTable(), idxSpalteObjekt,
  // idxSpalteVal);
  //
  // String columnText = columns[idxSpalteVal].getText();
  //
  // ViewBarChart viewBarChart = OpenViewBarChart.openView();
  //
  // viewBarChart.erzeugeBarChart(chartInput, "Auswertung Stellzustand",
  // columnText, "Anzahl Werte (" + defObjekt.getTableColumn().getText()
  // + ")", "Objekte ("
  // + dlg.getSelectedTableColumnForSpaltenDefinition(defWert)
  // .getText() + ")");
  // }
  //
  // } else
  // {
  // if (false)
  // {
  // TreeMap<String, TreeMap<String, Double>> chartInput = ChartTools
  // .erzeugeChartInputBalkenDiagramm(_tv.getTable(), 1, 2);
  // String columnText = columns[2].getText();
  // ViewBarChart viewBarChart = OpenViewBarChart.openView();
  // viewBarChart.erzeugeBarChart(chartInput, "Auswertung Stellzustand",
  // columnText, "Anzahl Schaltungen", "Objekte");
  // }
  // }
  //
  // }

  /**
   * Methode ermglicht den Export der Daten
   */
  private void aktionExport()
  {
    Shell shell = PlatformUI.getWorkbench().getDisplay().getActiveShell();

    FileDialog fld = new FileDialog(shell, SWT.SAVE);
    fld.setText("CSV/XML Datei exportieren");
    fld.setFilterNames(new String[] { "CSV Datei", "XML Datei" });
    fld.setFilterExtensions(new String[] { "*.csv", "*.xml" });
//    String extensions[] = new String[1];
//    extensions[0] = "*.csv";
//    dia.setFilterExtensions(extensions);

    
    ProtokollInformationen info = new ProtokollInformationen();
    String pid = _anfrageDaten.getObjekte().get(0).getPid();
    Hashtable<String, String[]> ht = new Hashtable<String, String[]>();
    ht.put(pid, new String[]{pid});
    info.setPseudoObjekte(ht);
    info.setObjekte(new HashSet<SystemObject>(_anfrageDaten.getObjekte()));
    
    List<KalenderZeitFilterDatenStrukturZeitGruppe> gruppen = _anfrageDaten.getZeitgruppen();    
    List<Tuple<Long, Long>> intervalle = new ArrayList<Tuple<Long, Long>>();
    for (KalenderZeitFilterDatenStrukturZeitGruppe gruppe : gruppen)
    {
      intervalle.add(new Tuple<Long, Long>(gruppe.getStartzeitpunkt(), gruppe.getEndzeitpunkt()));
    }    
    info.setIntervalle(intervalle);
    
    
    ArchiveDataKindCombination archiveDataKindCombination = _anfrageDaten.getArchiveDataKindCombination();
    
    
    Set<ArchiveDataKind> archivDatenArt = info.getArchivDatenArt();
    
    if (archivDatenArt == null)
      archivDatenArt = new HashSet<ArchiveDataKind>();    
        
    if (archiveDataKindCombination.isOnline()){
      archivDatenArt.add(ArchiveDataKind.ONLINE);            
    }
    if (archiveDataKindCombination.isOnlineDelayed()){
      archivDatenArt.add(ArchiveDataKind.ONLINE_DELAYED);                  
    }
    if (archiveDataKindCombination.isRequested()){      
      archivDatenArt.add(ArchiveDataKind.REQUESTED);            
    }
    if (archiveDataKindCombination.isRequestedDelayed()){
      archivDatenArt.add(ArchiveDataKind.REQUESTED_DELAYED);                  
    }
    
    info.setArchivDatenArt(archivDatenArt);   
    
    ProtokollInfo protokollInfo = new ProtokollInfo(info, new ProtokolleUndAuswertungen(_connection));        
    protokollInfo.setScriptName("Auswertung AQs");
    protokollInfo.setProtokollId(12345);
    
//    List<SystemObject> objs = _anfrageDaten.getObjekte();
//            
//    for (SystemObject so : objs)
//    {
//      protokollInfo.getObjekte().add(so);          
//    }
//    
//    List<KalenderZeitFilterDatenStrukturZeitGruppe> zeitgruppen = _anfrageDaten.getZeitgruppen();
//    for (KalenderZeitFilterDatenStrukturZeitGruppe gruppe : zeitgruppen)
//    {
//      protokollInfo.getZeitbereiche().add(gruppe.getStartzeitpunktAlsString() + "-" + gruppe.getEndzeitpunktAlsString());
//    }
    
    
    String filename = fld.open();
    if (filename != null && filename.length() > 0)
    {
      File file = new File(filename);
      
      Collection<SystemObject> objekte = Collections.emptyList();
      if (_anfrageDaten != null) {
    	  objekte = _anfrageDaten.getObjekteAnzeige();
      }
      
      if (file.getAbsoluteFile().toString().endsWith(".csv"))
      {       
        ExportCSV exportCSV = new ExportCSV(_tv.getTable());
        exportCSV.speichereCsv(file, protokollInfo, objekte);
      }

      if (file.getAbsoluteFile().toString().endsWith(".xml"))
      {
        ExportXml exportXml = new ExportXml(_tv.getTable());
        exportXml.speichereXml(file, protokollInfo, objekte);
      }
    }
//    if (filename != null)
//    {
//      if (!filename.contains("."))
//        filename = filename + ".csv";
//
//      if (filename.endsWith(".csv"))
//      {
//        try
//        {
//          Table t = _tv.getTable();
//          TableColumn[] c2 = t.getColumns();
//
//          TableItem[] items = _tv.getTable().getItems();
//
//          LinkedList<String[]> exportDaten = new LinkedList<String[]>();
//
//          for (TableItem ti : items)
//          {
//            int cnt = _tv.getTable().getColumnCount();
//            String[] ss = new String[cnt];
//            for (int i = 0; i < cnt; i++)
//            {
//              ss[i] = ti.getText(i);
//            }
//            exportDaten.add(ss);
//          }
//
//          ExportCSV2 ecsv = new ExportCSV2(exportDaten);
//          ecsv.setTableColumns(c2);
//          ecsv.speichereCsv(filename, _anfrageDaten);
//        }
//        catch (Exception e)
//        {
//          e.printStackTrace();
//        }
//      }
//      else
//      {
//        generiereMeldungFehler("Export", "Datentyp wird nicht untersttzt !");
//      }
//    }
  }

  public class ToolTipCellLabelProvider extends CellLabelProvider
  {

    public String getToolTipText(Object element)
    {

      Table table = _tv.getTable();

      Point location = PlatformUI.getWorkbench().getDisplay().getCursorLocation();

      String s = "";

      location = _tv.getControl().toControl(location);
      TableItem item = table.getItem(location);

      if (item != null)
      {
        // String text1 = "Typ       : " + item.getText(0);
        // String text2 = "MessStelle: ";
        // String text3 = "Wert      : ";
        // String text4 = "Stufe     : ";

        // boolean gefunden = false;
        for (int i = 0; i < table.getColumnCount(); i++)
        {
          Rectangle rect = item.getBounds(i);
          if (rect.contains(location))
          {
            String text = (String)item.getData(new Integer(i).toString());

            s += text;

            // text2 += table.getColumn(i).getText();
            //
            // UmfeldDatenTabelleDaten daten = (UmfeldDatenTabelleDaten)item.getData(new Integer(i).toString());
            //
            // if (daten != null)
            // {
            // text3 += daten.getWert();
            // if (!daten.getWert().equals(UmfeldDatenTabelleDaten.UNDEFINERT)){
            // String z1 = _sdf.format(new Date(daten.getWertZeit()));
            // text3 += " (" + z1 + ")";
            // }
            //              
            // text4 += daten.getStufe();
            // if (!daten.getStufe().equals(UmfeldDatenTabelleDaten.UNDEFINERT)){
            // String z2 = _sdf.format(new Date(daten.getStufeZeit()));
            // text4 += " (" + z2 + ")";
            // }
            // }
            // break;
          }
        }

        // if (text1 != null)
        // s += text1 + "\n";
        // if (text2 != null)
        // s += text2 + "\n";
        // if (text3 != null)
        // s += text3 + "\n";
        // if (text4 != null)
        // s += text4;

      }

      return s;
    }

    @Override
    public boolean useNativeToolTip(Object object)
    {
      // TODO Auto-generated method stub
      return false;
    }

    public void update(ViewerCell cell)
    {
      // cell.setText(cell.getElement().toString());
    }
  }

}
