/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.views;

import java.awt.Dimension;
import java.awt.event.WindowFocusListener;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TimeZone;

import javax.swing.JComponent;
import javax.swing.JRootPane;
import javax.swing.SwingUtilities;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IPartListener2;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPartReference;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.forms.events.ExpansionAdapter;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.widgets.Form;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.part.ViewPart;

import swingintegration.EmbeddedSwingComposite;

import com.swarco.sts.chart.control.TAxis;
import com.swarco.sts.chart.control.TChart;
import com.swarco.sts.chart.control.TSeries;
import com.swarco.sts.chart.control.event.TActionEvent;
import com.swarco.sts.chart.control.event.TActionListener;
import com.swarco.sts.chart.control.format.defaults.TDecimalFormat;
import com.swarco.sts.chart.control.format.interfaces.TFormat;
import com.swarco.sts.chart.control.label.defaults.TRenderableLabel;
import com.swarco.sts.chart.control.label.interfaces.TAxisLabelSize;
import com.swarco.sts.chart.control.label.interfaces.TRenderableComponent;
import com.swarco.sts.chart.control.legend.interfaces.TSeriesLegendAgent;
import com.swarco.sts.chart.control.series.defaults.TSeriesLinesRenderSettings;
import com.swarco.sts.chart.control.series.interfaces.TSeriesGroup;
import com.swarco.sts.chart.control.tooltip.interfaces.TToolTipFormat;
import com.swarco.sts.chart.impl.STSChartHelper;
import com.swarco.sts.chart.impl.STSDualSeriesLegendAgent;

import de.bsvrz.buv.plugin.pua.basis.Activator;
import de.bsvrz.buv.plugin.pua.basis.ImageFactory;
import de.bsvrz.buv.plugin.pua.basis.actions.ChartAktionenZiel;
import de.bsvrz.buv.plugin.pua.basis.actions.ExportAktion;
import de.bsvrz.buv.plugin.pua.basis.actions.GitterlinienAktion;

/**
 * View-Klasse, welche die Sicht zur Anzeige der Ganglinien darstellt
 * 
 * Da JFreeChart unter SWT nicht korrekt funktioniert, wurde die Funktionalitt in Swing realisiert.
 * 
 * @author Pittner
 */
public class GanglinieSicht extends ViewPart implements ChartAktionenZiel
{
  public static final String ID = "de.bsvrz.buv.plugin.pua.views.ganglinieSicht";
  
  public final static SimpleDateFormat SDF = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
  
  public static final int MAX_Y_UNITS_COUNT = 4;

  public static final int MAX_GANGLINIEN = 40;
  
  public static long INSTANCE_COUNT = 0;

  public static int mehrmal = 0;

  private Composite _parent;

  // private JFreeChart chart;

  private FormToolkit toolkit;

  /**
   * Composites
   */
  // private Composite sectionClient;

  private Composite sectionClient2;

  private Composite sectionComposite;

  private Composite sectionComposite2;

  /**
   * Tables
   */
  private Table table;

  private Table table2;

  /**
   * Values
   */
  protected Double value2;

  protected Double value3;

  protected Double value4;

  /**
   * Diagrammoptionen Buttons und Texte
   */
  private Text[] textVon = new Text[4];

  private Text[] textBis = new Text[4];

  private Button[] buttonAnwenden = new Button[4];

  private Button[] buttonAuto = new Button[4];

  private List<String> legendList = new ArrayList<String>(); // @jve:decl-index=0:

  /**
   * Secondary View ID
   */
  private String _secondaryId;
  
  private String _skriptName;

  /**
   * Creates a chart.
   * 
   * @param dataset
   *          a dataset.
   * 
   * @return A chart.
   */

  private ImageFactory imageFactory = ImageFactory.getInstanz();

  private ChartComposite chartComposite = null;

  private ExportAktion exportAktion = null;

  private GitterlinienAktion gitterlinienAktion = null;

  private Button gitterLinksAussen = null;

  private Button gitterLinks = null;

  private Button gitterRechts = null;

  private Button gitterRechtsAussen = null;

  private class PaintActionListener implements TActionListener
  {
    public void actionPerformed(TActionEvent actionEvent)
    {
      if (actionEvent.isAction(TChart.CHART_POST_PAINT))
      {
        final boolean auLinksVisible = getChart().getAxis(TAxis.AC_Y0).isVisible();
        final boolean linksVisible = getChart().getAxis(TAxis.AC_Y1).isVisible();
        final boolean rechtsVisible = getChart().getAxis(TAxis.AC_Y2).isVisible();
        final boolean auRechtsVisible = getChart().getAxis(TAxis.AC_Y3).isVisible();

        Display.getDefault().asyncExec(new Runnable() {
          public void run()
          {
            if (!gitterLinksAussen.isDisposed() && !gitterLinks.isDisposed() && !gitterRechtsAussen.isDisposed()
                && !gitterRechts.isDisposed())
            {
              if (gitterLinksAussen.isEnabled() != auLinksVisible)
              {
                if (gitterLinksAussen.isEnabled())
                {
                  if (gitterLinksAussen.getSelection())
                  {
                    gitterlinienAktion.doGitterlinienAktion(GitterlinienAktion.GA_AUS);
                  }
                  gitterLinksAussen.setSelection(false);
                  gitterLinksAussen.setEnabled(false);
                }
                else
                {
                  gitterLinksAussen.setSelection(false);
                  gitterLinksAussen.setEnabled(true);
                }
              }

              if (gitterLinks.isEnabled() != linksVisible)
              {
                if (gitterLinks.isEnabled())
                {
                  if (gitterLinks.getSelection())
                  {
                    gitterlinienAktion.doGitterlinienAktion(GitterlinienAktion.GA_AUS);
                  }
                  gitterLinks.setSelection(false);
                  gitterLinks.setEnabled(false);
                }
                else
                {
                  gitterLinks.setSelection(false);
                  gitterLinks.setEnabled(true);
                }
              }

              if (gitterRechts.isEnabled() != rechtsVisible)
              {
                if (gitterRechts.isEnabled())
                {
                  if (gitterRechts.getSelection())
                  {
                    gitterlinienAktion.doGitterlinienAktion(GitterlinienAktion.GA_AUS);
                  }
                  gitterRechts.setSelection(false);
                  gitterRechts.setEnabled(false);
                }
                else
                {
                  gitterRechts.setSelection(false);
                  gitterRechts.setEnabled(true);
                }
              }

              if (gitterRechtsAussen.isEnabled() != auRechtsVisible)
              {
                if (gitterRechtsAussen.isEnabled())
                {
                  if (gitterRechtsAussen.getSelection())
                  {
                    gitterlinienAktion.doGitterlinienAktion(GitterlinienAktion.GA_AUS);
                  }
                  gitterRechtsAussen.setSelection(false);
                  gitterRechtsAussen.setEnabled(false);
                }
                else
                {
                  gitterRechtsAussen.setSelection(false);
                  gitterRechtsAussen.setEnabled(true);
                }
              }
            }
          }
        });
      }
    }
  }

  public void setGitterlinienAktion(final int type)
  {
    Display.getDefault().asyncExec(new Runnable() {
      public void run()
      {
        if (type == GitterlinienAktion.GA_AU_LINKS && gitterLinksAussen.isEnabled())
        {
          gitterLinksAussen.setSelection(true);
          gitterLinks.setSelection(false);
          gitterRechts.setSelection(false);
          gitterRechtsAussen.setSelection(false);
        }
        else if (type == GitterlinienAktion.GA_LINKS && gitterLinks.isEnabled())
        {
          gitterLinksAussen.setSelection(false);
          gitterLinks.setSelection(true);
          gitterRechts.setSelection(false);
          gitterRechtsAussen.setSelection(false);
        }
        else if (type == GitterlinienAktion.GA_RECHTS && gitterRechts.isEnabled())
        {
          gitterLinksAussen.setSelection(false);
          gitterLinks.setSelection(false);
          gitterRechts.setSelection(true);
          gitterRechtsAussen.setSelection(false);
        }
        else if (type == GitterlinienAktion.GA_AU_RECHTS && gitterRechtsAussen.isEnabled())
        {
          gitterLinksAussen.setSelection(false);
          gitterLinks.setSelection(false);
          gitterRechts.setSelection(false);
          gitterRechtsAussen.setSelection(true);
        }
        else if (type == GitterlinienAktion.GA_AUS)
        {
          gitterLinksAussen.setSelection(false);
          gitterLinks.setSelection(false);
          gitterRechts.setSelection(false);
          gitterRechtsAussen.setSelection(false);
        }
      }
    });
  }
  
  private final static String STR_SIMPLE_DATE_FORMAT = "HH:mm";
  private final static String STR_SIMPLE_DATE_FORMAT_MAX = "88:88";
  private final static SimpleDateFormat SIMPLE_DATE_FORMAT = new SimpleDateFormat(STR_SIMPLE_DATE_FORMAT);
  
  static
  {
    GanglinieSicht.SIMPLE_DATE_FORMAT.setTimeZone(TimeZone.getTimeZone("UTC"));
  }
  
  private TChart createChart(Map<String, Map<String, SortedMap<Long, String>>> daten)
  {
    TChart ret = null;

    try
    {
      for (int i = 0; i < buttonAnwenden.length; i++)
      {
        setSektionEinstellungen(i, false, 0, 0);
      }

      STSChartHelper chartHelper = new STSChartHelper();
      
      if (_skriptName != null && !_skriptName.isEmpty()) {
    	  chartHelper.setTitle("Ganglinie" + " " + _skriptName);
      }
      else {
    	  chartHelper.setTitle("Ganglinie");  
      }
      
      TSeriesGroup linesGroup = chartHelper.getChart().addSeriesGroup(TSeries.SC_LINES);

      for (Map.Entry<String, Map<String, SortedMap<Long, String>>> attributEntry : daten.entrySet())
      {
        String attribut = attributEntry.getKey();
        Map<String, SortedMap<Long, String>> objektEntry = attributEntry.getValue();
        
        for (Map.Entry<String, SortedMap<Long, String>> zeitstempelWertEntry : objektEntry.entrySet())
        {
          String objekt = zeitstempelWertEntry.getKey();
          String quantitySymbol = "<NO_VAR>";
          String unitSymbol = "<NO_UNIT>";
          if (attribut.contains("[") && attribut.contains("]"))
          {
            quantitySymbol = attribut.substring(0, attribut.indexOf("[")).trim();
            unitSymbol = attribut.substring(attribut.indexOf("[") + 1, attribut.indexOf("]"));
          }
          else
          {
            quantitySymbol = attribut;
            unitSymbol = "";
          }
//          System.out.println("q: " + quantitySymbol + " u: " + unitSymbol);
          
          linesGroup.addSeries(chartHelper.createSeries(objekt, quantitySymbol, unitSymbol));
        }
      }
      chartHelper.finalizeLegend();

      TDecimalFormat labelFormat = new TDecimalFormat(3, 2);
      for (int s = 0; s < linesGroup.getSeriesCount(); s++)
      {
        linesGroup.getSeries(s).setLabelFormat(labelFormat);
        TSeriesLinesRenderSettings renderSettings = (TSeriesLinesRenderSettings)linesGroup.getSeries(s).getSeriesRenderSettings(TSeries.SC_LINES);
        linesGroup.getSeries(s).setSeriesRenderSettings(TSeries.SC_LINES, new TSeriesLinesRenderSettings(renderSettings.getLineSettings(), renderSettings.getOutlineSettings(), renderSettings.getFillSettings(), null, renderSettings.getBreakRenderer()));
        linesGroup.getSeries(s).setBreakValue(Integer.MAX_VALUE);
      }

      chartHelper.getChart().getAxis(TAxis.AC_X).setHeader("");
      chartHelper.getChart().getAxis(TAxis.AC_X).setType(TAxis.DT_TIME);

      int points = 0;
      int seriesIndex = 0;
      double dwert = Double.NaN;

      for (Map.Entry<String, Map<String, SortedMap<Long, String>>> attributEntry : daten.entrySet())
      {
        boolean isTimeFormat = false;
        Map<String, SortedMap<Long, String>> objektEntry = attributEntry.getValue();
        for (Map.Entry<String, SortedMap<Long, String>> zeitstempelWertEntry : objektEntry.entrySet())
        {
          int maxVorkommastellen = 3;
          int maxNachkommastellen = 0;
          TSeries series = linesGroup.getSeries(seriesIndex++);
          Map<Long, String> zeitstempelWertListe = zeitstempelWertEntry.getValue();
          for (Map.Entry<Long, String> zeitstempelWertPaar : zeitstempelWertListe.entrySet())
          {
            long zeitstempel = zeitstempelWertPaar.getKey().longValue();
            String wert = zeitstempelWertPaar.getValue();
            try
            {
              String s = null;
              if (wert.contains(","))
              {
                s = wert.replace(",", ".");
              }
              else
              {
                s = wert;
              }
              dwert = Double.parseDouble(s);

              if (dwert != series.getBreakValue())
              {
                int pIndex = s.indexOf(".");
                if (pIndex == -1)
                {
                  maxVorkommastellen = Math.max(maxVorkommastellen, s.length());
                }
                else
                {
                  maxVorkommastellen = Math.max(maxVorkommastellen, pIndex);
                  maxNachkommastellen = Math.max(maxNachkommastellen, s.length() - pIndex - 1);
                }
              }
            }
            catch (NumberFormatException e)
            {
              boolean parsed = false;
              try
              {
                  Date date = null;
                  synchronized(GanglinieSicht.SIMPLE_DATE_FORMAT)
                  {
                  date = GanglinieSicht.SIMPLE_DATE_FORMAT.parse(wert);
                  }
                dwert = date.getTime();
                parsed = true;
                isTimeFormat = true;
              }
              catch (ParseException pe)
              {
              }

              if (!parsed)
              {
                dwert = series.getBreakValue();
              }
            }

            series.addPoint(zeitstempel / 1000, dwert);
            points++;
          }

          // TODO Hier diese Zeile bitte herauskommentieren, wenn die Zahl an der Achse die genaue
          // Nachkommastellenanzahl haben soll, sonst werden immer mindestens 2 Nachkommastellen ausgegeben
//          maxNachkommastellen = Math.max(2, maxNachkommastellen);
          if (isTimeFormat)
          {
            series.setLabelFormat(new CSimpleDateFormat(GanglinieSicht.SIMPLE_DATE_FORMAT, GanglinieSicht.STR_SIMPLE_DATE_FORMAT_MAX));
            series.setToolTipFormat(new CToolTipFormat(GanglinieSicht.SIMPLE_DATE_FORMAT));
            TSeriesLegendAgent[] legendList = series.getLegendList();
            if(legendList != null)
            {
              for(int i = 0; i < legendList.length; i++)
              {
                    if(legendList[i] instanceof STSDualSeriesLegendAgent)
                    {
                          ((STSDualSeriesLegendAgent)legendList[i]).setFormat(GanglinieSicht.SIMPLE_DATE_FORMAT, GanglinieSicht.STR_SIMPLE_DATE_FORMAT_MAX);
                    }
              }
            }
          }
          else
          {
            series.setLabelFormat(new TDecimalFormat(maxVorkommastellen, maxNachkommastellen));
          }
        }
      }

      chartHelper.getChart().updateData(false);

      ret = chartHelper.getChart();

      AxisRangeListener axisRangeListener = new AxisRangeListener();
      for (int i = 0; i < TAxis.AC_YA.length; i++)
      {
        ret.getAxis(TAxis.AC_YA[i]).addActionListener(axisRangeListener);
      }
    }
    catch (Throwable t)
    {
      t.printStackTrace();
    }

    return (ret);
  }
  
  private class CToolTipFormat implements TToolTipFormat
  {
      private SimpleDateFormat tDateTimeFormat = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
    private GregorianCalendar calendar = null;
    private SimpleDateFormat simpleDateFormat = null;
    
      private CToolTipFormat(SimpleDateFormat format)
      {
      calendar = new GregorianCalendar(format.getTimeZone());
        simpleDateFormat = format;
      }
        
      public String format(TSeries series, double xValue, double yValue, Object meta)
      {
      String ret = null;

      calendar.setTimeInMillis((long)yValue);
      Date dateTime = calendar.getTime();
      String yFormatted = null;
      synchronized(simpleDateFormat)
      {
        yFormatted = simpleDateFormat.format(dateTime);
      }
      
      if(series.getChart().getAxis(TAxis.AC_X).getType() == TAxis.DT_TIME)
      {
      calendar.setTimeInMillis((long)xValue * (long)1000);
        dateTime = calendar.getTime();
        ret = tDateTimeFormat.format(dateTime);
      }
      else
      {
        ret = "" + yFormatted;
      }
      ret = ret + "   " + series.getCaption() + ": " + series.getQuantitySymbol() + " = " + yFormatted + " " + series.getUnitSymbol();
      
      return (ret);
      }

      public String formatLegend(TSeries series, double xValue, double yValue, Object meta)
      {
        return (format(series, xValue, yValue, meta));
      }
  }
  
  private class CSimpleDateFormat implements TFormat
  {
    protected TRenderableLabel label = new TRenderableLabel();
    protected GregorianCalendar calendar = null;
    protected SimpleDateFormat simpleDateFormat = null;
    protected String maxFormat = null;
    
    private CSimpleDateFormat(SimpleDateFormat format, String maxFormat)
    {
      calendar = new GregorianCalendar(format.getTimeZone());
      simpleDateFormat = format;
      simpleDateFormat.setTimeZone(TimeZone.getTimeZone("UTC"));
      this.maxFormat = maxFormat;
    }
    
    public TRenderableComponent format(TAxis axis, TFormat[] otherFormatList, double xValue, double yValue, int labelCount, int labelPosition)
    {
      int rotation = axis.getLabelRotation() % 360;
      double value = Double.NaN;
      if(axis.isAxis(TAxis.AC_X))
      {
        value = xValue;
      }
      else
      {
        value = yValue;
      }
      
      calendar.setTimeInMillis((long)value);
      Date dateTime = calendar.getTime();

      String simpleDate = null;
      synchronized(simpleDateFormat)
      {
        simpleDate = simpleDateFormat.format(dateTime);
      }
      label.setFont(axis.getLabelFont());
      label.setText(simpleDate);
      label.setRotation(rotation);
      
      return (label);
    }

    public TAxisLabelSize formatMax(TAxis axis)
    {
      int width = 0;
      int height = 0;
      int rotatedWidth = 0;
      int rotatedHeight = 0;

      label.setFont(axis.getLabelFont());
      label.setText(maxFormat);
      label.setRotation(0);
      Dimension prefSize = label.getPreferredSize();
      width = prefSize.width;
      height = prefSize.height;
      
      label.setRotation(axis.getLabelRotation());
      prefSize = label.getPreferredSize();
      rotatedWidth = prefSize.width;
      rotatedHeight = prefSize.height;
      
      return (new TAxisLabelSize(width, height, axis.getLabelRotation(), rotatedWidth, rotatedHeight, axis.isAxis(TAxis.AC_X)));
    }
  }


  @Override
  public TChart getChart()
  {
    TChart ret = null;

    ChartComposite tmp = chartComposite;
    if (tmp != null)
    {
      ret = tmp.getChart();
    }

    return (ret);
  }

  private class AxisRangeListener implements TActionListener
  {
    public void actionPerformed(TActionEvent actionEvent)
    {
      if (actionEvent.isAction(TAxis.AXIS_CLIP))
      {
        double[] clip = (double[])actionEvent.getValue();

        int axisIndex = -1;
        TAxis axis = (TAxis)actionEvent.getSource();
        if (axis.isAxis(TAxis.AC_Y0))
        {
          axisIndex = 0;
        }
        else if (axis.isAxis(TAxis.AC_Y1))
        {
          axisIndex = 1;
        }
        else if (axis.isAxis(TAxis.AC_Y2))
        {
          axisIndex = 2;
        }
        else if (axis.isAxis(TAxis.AC_Y3))
        {
          axisIndex = 3;
        }

        if (axisIndex != -1)
        {
          if (Double.isNaN(clip[0]) || Double.isNaN(clip[1]))
          {
            setSektionEinstellungen(axisIndex, false, 0, 0);
          }
          else
          {
            int vonWert = (int)Math.floor(clip[0]);
            int bisWert = (int)Math.ceil(clip[1]);
            setSektionEinstellungen(axisIndex, true, vonWert, bisWert);
          }
        }
      }
    }
  }


  public void createPartControl(Composite parent)
  {
    try
    {
      _parent = parent;

      IViewSite site = getViewSite();
      if (site != null)
      {
        String partName = getPartName();
        _secondaryId = site.getSecondaryId();
        int cnt = new Integer(_secondaryId);
        cnt += 1;
       
        partName += " - " + cnt;

        setPartName(partName);
      }

      // Fr die F1 Taste
      PlatformUI.getWorkbench().getHelpSystem().setHelp(_parent, Activator.PLUGIN_ID + ".anzeigeGanglinie");

      GridData gridData8 = new GridData();
      gridData8.grabExcessHorizontalSpace = true;
      gridData8.verticalAlignment = GridData.FILL;
      gridData8.grabExcessVerticalSpace = false;
      gridData8.horizontalAlignment = GridData.FILL;
      GridData gridData7 = new GridData();
      gridData7.grabExcessHorizontalSpace = true;
      gridData7.verticalAlignment = GridData.FILL;
      gridData7.grabExcessVerticalSpace = false;
      gridData7.horizontalAlignment = GridData.FILL;
      GridData gridData6 = new GridData();
      gridData6.grabExcessHorizontalSpace = true;
      gridData6.verticalAlignment = GridData.CENTER;
      gridData6.grabExcessVerticalSpace = false;
      gridData6.horizontalAlignment = GridData.FILL;
      GridData gridData5 = new GridData();
      gridData5.grabExcessHorizontalSpace = true;
      gridData5.verticalAlignment = GridData.CENTER;
      gridData5.grabExcessVerticalSpace = false;
      gridData5.horizontalAlignment = GridData.FILL;
      GridData gridData4 = new GridData();
      gridData4.horizontalAlignment = GridData.FILL;
      gridData4.grabExcessHorizontalSpace = true;
      gridData4.grabExcessVerticalSpace = false;
      gridData4.verticalAlignment = GridData.CENTER;
      GridData gridData3 = new GridData(GridData.HORIZONTAL_ALIGN_FILL, GridData.VERTICAL_ALIGN_FILL, true, true);
      gridData3.grabExcessVerticalSpace = false;
      gridData3.horizontalAlignment = GridData.FILL;
      gridData3.verticalAlignment = GridData.CENTER;
      gridData3.grabExcessHorizontalSpace = true;
      GridData gridData2 = new GridData();
      gridData2.grabExcessHorizontalSpace = false;
      gridData2.verticalAlignment = GridData.CENTER;
      gridData2.horizontalAlignment = GridData.BEGINNING;
      GridData gridData1 = new GridData();
      gridData1.horizontalAlignment = GridData.FILL;
      gridData1.grabExcessHorizontalSpace = true;
      gridData1.grabExcessVerticalSpace = true;
      gridData1.verticalAlignment = GridData.FILL;
      GridData gridData = new GridData();
      gridData.horizontalAlignment = GridData.FILL;
      gridData.grabExcessHorizontalSpace = true;
      gridData.grabExcessVerticalSpace = false;
      gridData.verticalAlignment = GridData.FILL;

      GridLayout compositeLayout = new GridLayout();
      compositeLayout.numColumns = 1;
      compositeLayout.makeColumnsEqualWidth = false;
      compositeLayout.verticalSpacing = 8;
      Composite composite = new Composite(_parent, SWT.NONE);
      composite.setLayout(compositeLayout);
      composite.setLayoutData(gridData2);
      composite.setBackground(new org.eclipse.swt.graphics.Color(null, 255, 255, 255));

      toolkit = new FormToolkit(composite.getDisplay());
      final Form form = toolkit.createForm(composite);
      form.setLayoutData(gridData4);
      form.getBody().setLayout(new GridLayout());

      /*
       * Section section = toolkit.createSection(form.getBody(), Section.DESCRIPTION | Section.TITLE_BAR |
       * Section.TWISTIE | Section.COMPACT); section.addExpansionListener(new ExpansionAdapter() { public void
       * expansionStateChanged(ExpansionEvent e) { // System.out.println("expanded1: " + e.getState()); } });
       * section.setText("Datenauswahl"); section.setLayoutData(gridData5); //
       * section.setDescription("This is the description that goes below the title"); sectionClient =
       * toolkit.createComposite(section); sectionClient.setLayout(new GridLayout());
       * sectionClient.setLayoutData(gridData7);
       */

      Section section2 = toolkit.createSection(form.getBody(), Section.DESCRIPTION | Section.TITLE_BAR
          | Section.TWISTIE | Section.COMPACT);
      section2.addExpansionListener(new ExpansionAdapter() {
        public void expansionStateChanged(ExpansionEvent e)
        {
          // System.out.println("expanded2: " + e.getState());
        }
      });
      section2.setText("Diagrammoptionen");
      section2.setLayoutData(gridData6);
      sectionClient2 = toolkit.createComposite(section2);
      sectionClient2.setLayout(new GridLayout());
      sectionClient2.setLayoutData(gridData8);

      // section.setClient(sectionClient);
      section2.setClient(sectionClient2);

      // createSectionComposite();
      createSectionComposite2();

      /*
       * ToolBar toolBar = new ToolBar(composite.getShell(), SWT.FLAT | SWT.WRAP | SWT.RIGHT); ToolItem itemPush = new
       * ToolItem(toolBar, SWT.PUSH); itemPush.setText("PUSH item"); Image icon =
       * Activator.getImageDescriptor("icons/grid_common.gif").createImage(); itemPush.setImage(icon);
       */

      // registriereActions();

      Composite composite2 = new Composite(composite, SWT.NONE);
      composite2.setLayout(new FillLayout());
      composite2.setLayoutData(gridData1);

      chartComposite = new ChartComposite(composite2, SWT.EMBEDDED, this);

      GridData diagrammGruppeData = new GridData();
      diagrammGruppeData.horizontalAlignment = GridData.END;
      diagrammGruppeData.grabExcessHorizontalSpace = false;
      diagrammGruppeData.grabExcessVerticalSpace = false;
      diagrammGruppeData.verticalAlignment = GridData.END;
      GridLayout diagrammGruppeLayout = new GridLayout();
      diagrammGruppeLayout.numColumns = 3;
      diagrammGruppeLayout.makeColumnsEqualWidth = false;
      diagrammGruppeLayout.marginHeight = 4;
      diagrammGruppeLayout.marginWidth = 4;
      diagrammGruppeLayout.verticalSpacing = 0;
      diagrammGruppeLayout.horizontalSpacing = 8;
      Composite diagrammGruppe = new Composite(composite, SWT.NONE);
      diagrammGruppe.setLayoutData(diagrammGruppeData);
      diagrammGruppe.setLayout(diagrammGruppeLayout);
      diagrammGruppe.setBackground(composite.getBackground());

      GridData diagrammSteuerungData = new GridData();
      diagrammSteuerungData.horizontalAlignment = GridData.END;
      diagrammSteuerungData.grabExcessHorizontalSpace = false;
      diagrammSteuerungData.grabExcessVerticalSpace = false;
      diagrammSteuerungData.verticalAlignment = GridData.END;

      GridData exportGruppeData = new GridData();
      exportGruppeData.horizontalAlignment = GridData.END;
      exportGruppeData.grabExcessHorizontalSpace = false;
      exportGruppeData.grabExcessVerticalSpace = false;
      exportGruppeData.verticalAlignment = GridData.END;
      GridLayout exportGruppeLayout = new GridLayout();
      exportGruppeLayout.numColumns = 3;
      exportGruppeLayout.makeColumnsEqualWidth = false;
      exportGruppeLayout.marginHeight = 0;
      exportGruppeLayout.marginWidth = 0;
      exportGruppeLayout.verticalSpacing = 0;
      exportGruppeLayout.horizontalSpacing = 3;
      Composite exportGruppe = new Composite(diagrammGruppe, SWT.NONE);
      exportGruppe.setLayoutData(exportGruppeData);
      exportGruppe.setLayout(exportGruppeLayout);
      exportGruppe.setBackground(composite.getBackground());

      Button drucken = toolkit.createButton(exportGruppe, "", SWT.PUSH);
      Button exportPdfHochformat = toolkit.createButton(exportGruppe, "", SWT.PUSH);
      Button exportPdfQuerformat = toolkit.createButton(exportGruppe, "", SWT.PUSH);

      drucken.setImage(imageFactory.getImage("PRINTVIEW_TSK"));
      drucken.setLayoutData(diagrammSteuerungData);
      drucken.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e)
        {
          exportAktion.doExportAktion(ExportAktion.EX_PRINT_VIEW);
        }
      });

      exportPdfHochformat.setImage(imageFactory.getImage("EXPORT_A4_PT"));
      exportPdfHochformat.setLayoutData(diagrammSteuerungData);
      exportPdfHochformat.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e)
        {
          exportAktion.doExportAktion(ExportAktion.EX_PDF_A4_HOCHFORMAT);
        }
      });

      exportPdfQuerformat.setImage(imageFactory.getImage("EXPORT_A4_LS"));
      exportPdfQuerformat.setLayoutData(diagrammSteuerungData);
      exportPdfQuerformat.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e)
        {
          exportAktion.doExportAktion(ExportAktion.EX_PDF_A4_QUERFORMAT);
        }
      });

      GridData diagrammSteuerungSeparatorData = new GridData();
      diagrammSteuerungSeparatorData.heightHint = (int)(imageFactory.getImage("PRINTVIEW_TSK").getImageData().height * 1.5);
      diagrammSteuerungSeparatorData.verticalAlignment = GridData.CENTER;
      Label diagrammSteuerungSeparator = new Label(diagrammGruppe, SWT.SEPARATOR | SWT.VERTICAL);
      diagrammSteuerungSeparator.setBackground(composite.getBackground());
      diagrammSteuerungSeparator.setLayoutData(diagrammSteuerungSeparatorData);

      GridData gitterlinienGruppeData = new GridData();
      gitterlinienGruppeData.horizontalAlignment = GridData.END;
      gitterlinienGruppeData.grabExcessHorizontalSpace = false;
      gitterlinienGruppeData.grabExcessVerticalSpace = false;
      gitterlinienGruppeData.verticalAlignment = GridData.END;
      GridLayout gitterlinienGruppeLayout = new GridLayout();
      gitterlinienGruppeLayout.numColumns = 4;
      gitterlinienGruppeLayout.makeColumnsEqualWidth = false;
      gitterlinienGruppeLayout.marginHeight = 0;
      gitterlinienGruppeLayout.marginWidth = 0;
      gitterlinienGruppeLayout.verticalSpacing = 0;
      gitterlinienGruppeLayout.horizontalSpacing = 3;
      Composite gitterlinienGruppe = new Composite(diagrammGruppe, SWT.NONE);
      gitterlinienGruppe.setLayoutData(gitterlinienGruppeData);
      gitterlinienGruppe.setLayout(gitterlinienGruppeLayout);
      gitterlinienGruppe.setBackground(composite.getBackground());

      gitterLinksAussen = toolkit.createButton(gitterlinienGruppe, "", SWT.TOGGLE);
      gitterLinksAussen.setEnabled(false);
      gitterLinks = toolkit.createButton(gitterlinienGruppe, "", SWT.TOGGLE);
      gitterLinks.setEnabled(false);
      gitterRechts = toolkit.createButton(gitterlinienGruppe, "", SWT.TOGGLE);
      gitterRechts.setEnabled(false);
      gitterRechtsAussen = toolkit.createButton(gitterlinienGruppe, "", SWT.TOGGLE);
      gitterRechtsAussen.setEnabled(false);

      gitterLinksAussen.setImage(imageFactory.getImage("GRID_OSLEFT"));
      gitterLinksAussen.setLayoutData(diagrammSteuerungData);
      gitterLinksAussen.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e)
        {
          gitterLinks.setSelection(false);
          gitterRechts.setSelection(false);
          gitterRechtsAussen.setSelection(false);

          if (gitterlinienAktion != null)
          {
            if (gitterLinksAussen.getSelection())
            {
              gitterlinienAktion.doGitterlinienAktion(GitterlinienAktion.GA_AU_LINKS);
            }
            else
            {
              gitterlinienAktion.doGitterlinienAktion(GitterlinienAktion.GA_AUS);
            }
          }
        }
      });

      gitterLinks.setImage(imageFactory.getImage("GRID_LEFT"));
      gitterLinks.setLayoutData(diagrammSteuerungData);
      gitterLinks.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e)
        {
          gitterLinksAussen.setSelection(false);
          gitterRechts.setSelection(false);
          gitterRechtsAussen.setSelection(false);

          if (gitterlinienAktion != null)
          {
            if (gitterLinks.getSelection())
            {
              gitterlinienAktion.doGitterlinienAktion(GitterlinienAktion.GA_LINKS);
            }
            else
            {
              gitterlinienAktion.doGitterlinienAktion(GitterlinienAktion.GA_AUS);
            }
          }
        }
      });

      gitterRechts.setImage(imageFactory.getImage("GRID_RIGHT"));
      gitterRechts.setLayoutData(diagrammSteuerungData);
      gitterRechts.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e)
        {
          gitterLinksAussen.setSelection(false);
          gitterLinks.setSelection(false);
          gitterRechtsAussen.setSelection(false);

          if (gitterlinienAktion != null)
          {
            if (gitterRechts.getSelection())
            {
              gitterlinienAktion.doGitterlinienAktion(GitterlinienAktion.GA_RECHTS);
            }
            else
            {
              gitterlinienAktion.doGitterlinienAktion(GitterlinienAktion.GA_AUS);
            }
          }
        }
      });

      gitterRechtsAussen.setImage(imageFactory.getImage("GRID_OSRIGHT"));
      gitterRechtsAussen.setLayoutData(diagrammSteuerungData);
      gitterRechtsAussen.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e)
        {
          gitterLinksAussen.setSelection(false);
          gitterLinks.setSelection(false);
          gitterRechts.setSelection(false);

          if (gitterlinienAktion != null)
          {
            if (gitterlinienAktion != null)
            {
              if (gitterRechtsAussen.getSelection())
              {
                gitterlinienAktion.doGitterlinienAktion(GitterlinienAktion.GA_AU_RECHTS);
              }
              else
              {
                gitterlinienAktion.doGitterlinienAktion(GitterlinienAktion.GA_AUS);
              }
            }
          }
        }
      });
    }
    catch (Throwable t)
    {
      t.printStackTrace();
    }
  }

  public void setFocus()
  {
    // Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();
  }

  /**
   * Registriert die Actions zum Aufruf der Outline-View
   */
  private void registriereActions()
  {
    IMenuManager menuManager = this.getViewSite().getActionBars().getMenuManager();
    IToolBarManager toolBarManager = this.getViewSite().getActionBars().getToolBarManager();

    Action action = new Action() {
      public void run()
      {
        // TODO RAHO
        // new DruckThread(panel);
      }

    };
    action.setText("Print-out");

    ImageDescriptor id = Activator.getImageDescriptor("icons/editor.gif");

    action.setImageDescriptor(id);
    menuManager.add(action);
    toolBarManager.add(action);
  }

  public void erzeugeChart(Map<String, Map<String, SortedMap<Long, String>>> daten, String skriptName)
  { 
	  if (skriptName != null) {
    	_skriptName = skriptName;
    	String partName = "Ganglinie";
    	if (!skriptName.isEmpty()) {
    		partName += " " + skriptName;
    	}
    	IViewSite site = getViewSite();
        if (site != null)
        {
        	int cnt = new Integer(_secondaryId);
            cnt += 1;
            partName += " - " + cnt;
        }    
        setPartName(partName);
    }
    chartComposite.create(daten);
    exportAktion = new ExportAktion(this);
    gitterlinienAktion = new GitterlinienAktion(this);
    _parent.layout(true);
  }

  @Override
  public void dispose()
  {
    // TODO Auto-generated method stub
    chartComposite.destroy();
    super.dispose();
  }

  private void testeWert(Event event)
  {
    boolean valid = true;

    String wert = event.text;
    wert.replace(',', '.');
    try
    {
      Double.parseDouble(wert);
    }
    catch (Exception e)
    {
      valid = false;
    }

    if (!valid)
    {
      if (wert.length() == 0)
      {
        valid = true;
      }
      else if (wert.length() == 1 && wert.charAt(0) == '-')
      {
        valid = true;
      }
      else if (wert.endsWith("."))
      {
        wert = wert + "0";
        try
        {
          Double.parseDouble(wert);
          valid = true;
        }
        catch (Exception e)
        {
        }
      }
    }

    if (!valid)
    {
      event.doit = false;
    }
  }

  private void createSectionComposite2()
  {
    RowLayout rowLayout1 = new RowLayout(SWT.VERTICAL);
    rowLayout1.pack = false;
    rowLayout1.fill = true;
    rowLayout1.wrap = false;
    RowLayout rowLayout = new RowLayout(SWT.VERTICAL);
    rowLayout.fill = true;
    rowLayout.wrap = false;
    rowLayout.pack = false;
    GridData gridData25 = new GridData();
    gridData25.horizontalAlignment = GridData.FILL;
    gridData25.grabExcessHorizontalSpace = true;
    gridData25.verticalAlignment = GridData.CENTER;
    GridData gridData24 = new GridData();
    gridData24.horizontalAlignment = GridData.FILL;
    gridData24.grabExcessHorizontalSpace = true;
    gridData24.verticalAlignment = GridData.CENTER;
    GridData gridData23 = new GridData();
    gridData23.horizontalAlignment = GridData.FILL;
    gridData23.grabExcessHorizontalSpace = true;
    gridData23.verticalAlignment = GridData.CENTER;
    GridData gridData22 = new GridData();
    gridData22.horizontalAlignment = GridData.FILL;
    gridData22.grabExcessHorizontalSpace = true;
    gridData22.verticalAlignment = GridData.CENTER;
    GridData gridData21 = new GridData();
    gridData21.horizontalAlignment = GridData.CENTER;
    gridData21.verticalAlignment = GridData.BEGINNING;
    GridData gridData20 = new GridData();
    gridData20.horizontalAlignment = GridData.CENTER;
    gridData20.verticalAlignment = GridData.BEGINNING;
    GridData gridData19 = new GridData();
    gridData19.horizontalAlignment = GridData.FILL;
    gridData19.grabExcessHorizontalSpace = true;
    gridData19.verticalAlignment = GridData.CENTER;
    GridData gridData18 = new GridData();
    gridData18.horizontalAlignment = GridData.FILL;
    gridData18.grabExcessHorizontalSpace = true;
    gridData18.verticalAlignment = GridData.CENTER;
    GridData gridData17 = new GridData();
    gridData17.horizontalAlignment = GridData.CENTER;
    gridData17.verticalAlignment = GridData.BEGINNING;
    GridData gridData16 = new GridData();
    gridData16.horizontalAlignment = GridData.CENTER;
    gridData16.verticalAlignment = GridData.BEGINNING;
    GridData gridData15 = new GridData();
    gridData15.horizontalAlignment = GridData.FILL;
    gridData15.grabExcessHorizontalSpace = true;
    gridData15.verticalAlignment = GridData.CENTER;
    GridData gridData14 = new GridData();
    gridData14.horizontalAlignment = GridData.FILL;
    gridData14.grabExcessHorizontalSpace = true;
    gridData14.verticalAlignment = GridData.CENTER;
    GridData gridData13 = new GridData();
    gridData13.horizontalAlignment = GridData.FILL;
    gridData13.grabExcessHorizontalSpace = true;
    gridData13.verticalAlignment = GridData.CENTER;
    GridData gridData12 = new GridData();
    gridData12.horizontalAlignment = GridData.FILL;
    gridData12.grabExcessHorizontalSpace = true;
    gridData12.verticalAlignment = GridData.CENTER;
    GridData gridData11 = new GridData();
    gridData11.horizontalAlignment = GridData.FILL;
    gridData11.grabExcessHorizontalSpace = true;
    gridData11.verticalAlignment = GridData.CENTER;
    GridData gridData10 = new GridData();
    gridData10.horizontalAlignment = GridData.FILL;
    gridData10.grabExcessHorizontalSpace = true;
    gridData10.verticalAlignment = GridData.CENTER;
    GridData gridData7 = new GridData();
    gridData7.grabExcessHorizontalSpace = true;
    gridData7.verticalAlignment = GridData.CENTER;
    gridData7.grabExcessVerticalSpace = false;
    gridData7.horizontalAlignment = GridData.FILL;

    sectionComposite2 = toolkit.createComposite(sectionClient2, SWT.NONE);
    sectionComposite2.setLayout(new GridLayout(4, true));
    sectionComposite2.setLayoutData(gridData7);

    // y-achse (links aussen)
    GridData gridData30 = new GridData();
    gridData30.grabExcessHorizontalSpace = true;
    gridData30.verticalAlignment = GridData.CENTER;
    gridData30.grabExcessVerticalSpace = false;
    gridData30.horizontalAlignment = GridData.FILL;

    Composite yLinksAussen = toolkit.createComposite(sectionComposite2, SWT.NONE);
    yLinksAussen.setLayout(new GridLayout(2, false));
    yLinksAussen.setLayoutData(gridData30);

    toolkit.createLabel(yLinksAussen, "Y-Achse (links aussen)");
    toolkit.createLabel(yLinksAussen, "");

    Composite comp1 = toolkit.createComposite(yLinksAussen, SWT.NULL);
    comp1.setLayout(new GridLayout(2, false));
    comp1.setLayoutData(gridData10);

    Composite buttonComposite = toolkit.createComposite(yLinksAussen, SWT.NONE);
    buttonComposite.setLayoutData(gridData16);
    buttonComposite.setLayout(rowLayout);
    buttonAnwenden[0] = toolkit.createButton(buttonComposite, "Anwenden", SWT.NONE);
    buttonAnwenden[0].addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e)
      {
        final double vonWert = Double.parseDouble(textVon[0].getText());
        final double bisWert = Double.parseDouble(textBis[0].getText());
        SwingUtilities.invokeLater(new Runnable() {
          public void run()
          {
            getChart().getAxis(TAxis.AC_Y0).setClip(vonWert, bisWert, false);
            getChart().repaint(false);
          }
        });
      }
    });

    buttonAuto[0] = toolkit.createButton(buttonComposite, "Auto", SWT.NONE);
    buttonAuto[0].addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e)
      {
        SwingUtilities.invokeLater(new Runnable() {
          public void run()
          {
            getChart().getAxis(TAxis.AC_Y0).resetClip();
            getChart().repaint(false);
          }
        });
      }
    });

    toolkit.createLabel(comp1, "Von");
    textVon[0] = new Text(comp1, SWT.BORDER);
    textVon[0].setText("0");
    textVon[0].setLayoutData(gridData12);
    textVon[0].addListener(SWT.Verify, new Listener() {
      public void handleEvent(Event event)
      {
        testeWert(event);
      }
    });

    toolkit.createLabel(comp1, "Bis");
    textBis[0] = new Text(comp1, SWT.BORDER);
    textBis[0].setLayoutData(gridData13);
    textBis[0].addListener(SWT.Verify, new Listener() {
      public void handleEvent(Event event)
      {
        testeWert(event);
      }
    });

    // y-achse (links)
    GridData gridData31 = new GridData();
    gridData31.grabExcessHorizontalSpace = true;
    gridData31.verticalAlignment = GridData.CENTER;
    gridData31.grabExcessVerticalSpace = false;
    gridData31.horizontalAlignment = GridData.FILL;

    Composite yLinks = toolkit.createComposite(sectionComposite2, SWT.NONE);
    yLinks.setLayout(new GridLayout(2, false));
    yLinks.setLayoutData(gridData31);

    toolkit.createLabel(yLinks, "Y-Achse (links)");
    toolkit.createLabel(yLinks, "");

    Composite comp2 = toolkit.createComposite(yLinks, SWT.NULL);
    comp2.setLayout(new GridLayout(2, false));
    comp2.setLayoutData(gridData11);

    Composite buttonComposite2 = toolkit.createComposite(yLinks, SWT.NONE);
    buttonComposite2.setLayoutData(gridData17);
    buttonComposite2.setLayout(rowLayout1);
    buttonAnwenden[1] = toolkit.createButton(buttonComposite2, "Anwenden", SWT.NONE);
    buttonAnwenden[1].addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e)
      {
        final double vonWert = Double.parseDouble(textVon[1].getText());
        final double bisWert = Double.parseDouble(textBis[1].getText());
        SwingUtilities.invokeLater(new Runnable() {
          public void run()
          {
            getChart().getAxis(TAxis.AC_Y1).setClip(vonWert, bisWert, false);
            getChart().repaint(false);
          }
        });
      }
    });

    buttonAuto[1] = toolkit.createButton(buttonComposite2, "Auto", SWT.NONE);
    buttonAuto[1].addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e)
      {
        SwingUtilities.invokeLater(new Runnable() {
          public void run()
          {
            getChart().getAxis(TAxis.AC_Y1).resetClip();
            getChart().repaint(false);
          }
        });
      }
    });

    toolkit.createLabel(comp2, "Von");
    textVon[1] = new Text(comp2, SWT.BORDER);
    textVon[1].setText("0");
    textVon[1].setLayoutData(gridData14);
    textVon[1].addListener(SWT.Verify, new Listener() {
      public void handleEvent(Event event)
      {
        testeWert(event);
      }
    });

    toolkit.createLabel(comp2, "Bis");
    textBis[1] = new Text(comp2, SWT.BORDER);
    textBis[1].setLayoutData(gridData15);
    textBis[1].addListener(SWT.Verify, new Listener() {
      public void handleEvent(Event event)
      {
        testeWert(event);
      }
    });

    // y-achse (rechts)
    GridData gridData32 = new GridData();
    gridData32.grabExcessHorizontalSpace = true;
    gridData32.verticalAlignment = GridData.CENTER;
    gridData32.grabExcessVerticalSpace = false;
    gridData32.horizontalAlignment = GridData.FILL;

    Composite yRechts = toolkit.createComposite(sectionComposite2, SWT.NONE);
    yRechts.setLayout(new GridLayout(2, false));
    yRechts.setLayoutData(gridData32);

    toolkit.createLabel(yRechts, "Y-Achse (rechts)");
    toolkit.createLabel(yRechts, "");

    Composite comp3 = toolkit.createComposite(yRechts, SWT.NULL);
    comp3.setLayout(new GridLayout(2, false));
    comp3.setLayoutData(gridData18);

    Composite buttonComposite3 = toolkit.createComposite(yRechts, SWT.NONE);
    buttonComposite3.setLayoutData(gridData20);
    buttonComposite3.setLayout(rowLayout);
    buttonAnwenden[2] = toolkit.createButton(buttonComposite3, "Anwenden", SWT.NONE);
    buttonAnwenden[2].addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e)
      {
        final double vonWert = Double.parseDouble(textVon[2].getText());
        final double bisWert = Double.parseDouble(textBis[2].getText());
        SwingUtilities.invokeLater(new Runnable() {
          public void run()
          {
            getChart().getAxis(TAxis.AC_Y2).setClip(vonWert, bisWert, false);
            getChart().repaint(false);
          }
        });
      }
    });

    buttonAuto[2] = toolkit.createButton(buttonComposite3, "Auto", SWT.NONE);
    buttonAuto[2].addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e)
      {
        SwingUtilities.invokeLater(new Runnable() {
          public void run()
          {
            getChart().getAxis(TAxis.AC_Y2).resetClip();
            getChart().repaint(false);
          }
        });
      }
    });

    toolkit.createLabel(comp3, "Von");
    textVon[2] = new Text(comp3, SWT.BORDER);
    textVon[2].setText("0");
    textVon[2].setLayoutData(gridData22);
    textVon[2].addListener(SWT.Verify, new Listener() {
      public void handleEvent(Event event)
      {
        testeWert(event);
      }
    });

    toolkit.createLabel(comp3, "Bis");
    textBis[2] = new Text(comp3, SWT.BORDER);
    textBis[2].setLayoutData(gridData23);
    textBis[2].addListener(SWT.Verify, new Listener() {
      public void handleEvent(Event event)
      {
        testeWert(event);
      }
    });

    // y-achse (rechts aussen)
    GridData gridData33 = new GridData();
    gridData33.grabExcessHorizontalSpace = true;
    gridData33.verticalAlignment = GridData.CENTER;
    gridData33.grabExcessVerticalSpace = false;
    gridData33.horizontalAlignment = GridData.FILL;

    Composite yRechtsAussen = toolkit.createComposite(sectionComposite2, SWT.NONE);
    yRechtsAussen.setLayout(new GridLayout(2, false));
    yRechtsAussen.setLayoutData(gridData33);

    toolkit.createLabel(yRechtsAussen, "Y-Achse (rechts aussen)");
    toolkit.createLabel(yRechtsAussen, "");

    Composite comp4 = toolkit.createComposite(yRechtsAussen, SWT.NULL);
    comp4.setLayout(new GridLayout(2, false));
    comp4.setLayoutData(gridData19);

    Composite buttonComposite4 = toolkit.createComposite(yRechtsAussen, SWT.NONE);
    buttonComposite4.setLayoutData(gridData21);
    buttonComposite4.setLayout(rowLayout);
    buttonAnwenden[3] = toolkit.createButton(buttonComposite4, "Anwenden", SWT.NONE);
    buttonAnwenden[3].addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e)
      {
        final double vonWert = Double.parseDouble(textVon[3].getText());
        final double bisWert = Double.parseDouble(textBis[3].getText());
        SwingUtilities.invokeLater(new Runnable() {
          public void run()
          {
            getChart().getAxis(TAxis.AC_Y3).setClip(vonWert, bisWert, false);
            getChart().repaint(false);
          }
        });
      }
    });

    buttonAuto[3] = toolkit.createButton(buttonComposite4, "Auto", SWT.NONE);
    buttonAuto[3].addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e)
      {
        SwingUtilities.invokeLater(new Runnable() {
          public void run()
          {
            getChart().getAxis(TAxis.AC_Y3).resetClip();
            getChart().repaint(false);
          }
        });
      }
    });

    toolkit.createLabel(comp4, "Von");
    textVon[3] = new Text(comp4, SWT.BORDER);
    textVon[3].setText("0");
    textVon[3].setLayoutData(gridData24);
    textVon[3].addListener(SWT.Verify, new Listener() {
      public void handleEvent(Event event)
      {
        testeWert(event);
      }
    });

    toolkit.createLabel(comp4, "Bis");
    textBis[3] = new Text(comp4, SWT.BORDER);
    textBis[3].setLayoutData(gridData25);
    textBis[3].addListener(SWT.Verify, new Listener() {
      public void handleEvent(Event event)
      {
        testeWert(event);
      }
    });
  }

  private void setSektionEinstellungen(final int sektionIndex, final boolean eingeschaltet, final int vonWert,
      final int bisWert)
  {
    Display.getDefault().asyncExec(new Runnable() {
      public void run()
      {
        _setSektionEinstellungen(sektionIndex, eingeschaltet, vonWert, bisWert);
      }
    });
  }

  private void _setSektionEinstellungen(int sektionIndex, boolean eingeschaltet, int vonWert, int bisWert)
  {
    if (sektionIndex >= 0 && sektionIndex < buttonAnwenden.length)
    {
      if (!buttonAnwenden[sektionIndex].isDisposed() && !buttonAuto[sektionIndex].isDisposed())
      {
        buttonAnwenden[sektionIndex].setEnabled(eingeschaltet);
        buttonAuto[sektionIndex].setEnabled(eingeschaltet);
      }

      if (!textVon[sektionIndex].isDisposed() && !textBis[sektionIndex].isDisposed())
      {
        textVon[sektionIndex].setEnabled(eingeschaltet);
        if (eingeschaltet)
        {
          textVon[sektionIndex].setText("" + vonWert);
        }
        else
        {
          textVon[sektionIndex].setText("0");
        }
        textBis[sektionIndex].setEnabled(eingeschaltet);
        if (eingeschaltet)
        {
          textBis[sektionIndex].setText("" + bisWert);
        }
        else
        {
          textBis[sektionIndex].setText("");
        }
      }
    }
  }

  class ChartComposite extends EmbeddedSwingComposite implements WindowFocusListener, IPartListener2
  {
    private ViewPart viewPart = null;

    private JRootPane rootPane = null;

    private TChart tChart = null;

    private PaintActionListener paintActionListener = null;

    private Object destroySync = new Object();

    public ChartComposite(Composite parent, int style, ViewPart viewPart)
    {
      super(parent, style);
      this.viewPart = viewPart;
      populate();
      getFrame().addWindowFocusListener(this);
      // System.out.println("composite constructed: " + viewPart.getPartName());
    }

    protected JComponent createSwingComponent()
    {
      rootPane = new JRootPane();

      return (rootPane);
    }

    public void create(Map<String, Map<String, SortedMap<Long, String>>> daten)
    {
      String info = "unknown view part";
      synchronized (destroySync)
      {
        if (viewPart != null)
        {
          tChart = createChart(daten);
          viewPart.getViewSite().getPage().addPartListener(this);
          paintActionListener = new PaintActionListener();
          tChart.addPaintListener(paintActionListener);
          rootPane.setContentPane(tChart.getChartComponent());
          info = viewPart.getPartName();
        }
      }
      // System.out.println("composite created: " + info);
    }

    public void destroy()
    {
      String info = "unknown view part";
      synchronized (destroySync)
      {
        if (viewPart != null)
        {
          info = viewPart.getPartName();
          // getFrame().removeWindowFocusListener(this);
          viewPart = null;
          tChart.removePaintListener(paintActionListener);
          tChart.destroy();
          rootPane = null;
          tChart = null;
          paintActionListener = null;
        }
      }
      // System.out.println("composite destroyed: " + info);
    }

    public TChart getChart()
    {
      return (tChart);
    }

    public void windowGainedFocus(java.awt.event.WindowEvent arg0)
    {
      synchronized (destroySync)
      {
        if (!isDisposed() && viewPart != null)
        {
          final String fInfo = viewPart.getPartName();
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {  if (!isDisposed()) {
                forceFocus();
              }
             
              // System.out.println("window gained focus forced: " + fInfo);
            }
          });
        }
      }
    }

    public void windowLostFocus(java.awt.event.WindowEvent arg0)
    {
    }

    public void partActivated(IWorkbenchPartReference partRef)
    {
      synchronized (destroySync)
      {
        if (viewPart != null && partRef.getPart(false) == viewPart)
        {
          if (!isDisposed()) {
            forceFocus();  
          }
          // System.out.println("part activated: " + viewPart.getPartName());
        }
      }
    }

    public void partBroughtToTop(IWorkbenchPartReference partRef)
    {
    }

    public void partClosed(IWorkbenchPartReference partRef)
    {
    }

    public void partDeactivated(IWorkbenchPartReference partRef)
    {
    }

    public void partHidden(IWorkbenchPartReference partRef)
    {
    }

    public void partInputChanged(IWorkbenchPartReference partRef)
    {
    }

    public void partOpened(IWorkbenchPartReference partRef)
    {
    }

    public void partVisible(IWorkbenchPartReference partRef)
    {
    }
  }
}

// @jve:decl-index=0:visual-constraint="17,21"
