/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.util.EContentAdapter;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.ICheckStateProvider;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.Activateable;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.ActivationState;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.AnzeigeQuerschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilPackage;

/**
 * {@link ICheckStateProvider} fr die Baumansicht des Streckenprofils.
 * 
 * @author BitCtrl Systems GmbH, Schnepel / Hoesel
 * @version $Id: StreckenprofilTreeCheckStateProvider.java,v 1.3 2014/04/17 15:51:18 Pittner Exp $
 */
public class StreckenprofilTreeCheckStateProvider implements
		ICheckStateProvider, ICheckStateListener {

	/**
	 * Konstruktor des {@link StreckenprofilTreeCheckStateProvider}s.
	 * 
	 * @param checkable
	 *            der {@link CheckboxTreeViewer}, dessen Elemente checkable
	 *            sind.
	 * @param model
	 *            das Streckenprofil, das betrachtet wird.
	 */
	public StreckenprofilTreeCheckStateProvider(
			final CheckboxTreeViewer checkable, final Modell model) {

		model.getEMFModell().eAdapters().add(new EContentAdapter() {

			/*
			 * (non-Javadoc)
			 * 
			 * @see
			 * org.eclipse.emf.ecore.util.EContentAdapter#notifyChanged(org.
			 * eclipse.emf.common.notify.Notification)
			 */
			@Override
			public void notifyChanged(final Notification notification) {
				super.notifyChanged(notification);
				
//				System.out
//            .println("notifyChanged(): " + notification);
				
				final Object feature = notification.getFeature();
				if (StreckenprofilPackage.Literals.ACTIVATEABLE__STATE
						.equals(feature) 
						//|| StreckenprofilPackage.Literals.ACTIVATEABLE__INTERNAL_STATE.equals(feature)
						) 
				{
					if (notification.getNotifier() instanceof Activateable) {
						final Activateable activateable = (Activateable) notification
								.getNotifier();
						final ActivationState state = activateable.getState();
						
//						if (notification
//                .getNotifier() instanceof AnzeigeQuerschnitt)
//            {
//              AnzeigeQuerschnitt aq = (AnzeigeQuerschnitt)activateable;
//              System.out
//                  .println("---> AQ: " + aq.getModellObjekt().getSystemObject() + " -> " + state.toString() + " --- " + checkable);
//            }
						
						switch (state) {
						case ACTIVE:
							checkable.setGrayed(activateable, false);
							checkable.setChecked(activateable, true);
							break;
						case SEMI_ACTIVE:
							checkable.setGrayed(activateable, true);
							checkable.setChecked(activateable, true);
							break;
						case INACTIVE:
							checkable.setGrayed(activateable, false);
							checkable.setChecked(activateable, false);
							break;
						default:
							throw new IllegalStateException(state.toString());
						}
					}else{
//					  System.out.println("else notifier: " + notification.getNotifier().getClass().getSimpleName());
					}
				}else {				 
//				  System.out.println("else feature: " + feature);				  
				}
			}

		});
		checkable.addCheckStateListener(this);
	}

	@Override
	public boolean isChecked(final Object element) {
		final ActivationState state = ((Activateable) element).getState();
		return ActivationState.ACTIVE.equals(state)
				|| ActivationState.SEMI_ACTIVE.equals(state);
	}

	@Override
	public boolean isGrayed(final Object element) {
		final ActivationState state = ((Activateable) element).getState();
		return ActivationState.SEMI_ACTIVE.equals(state);
	}

	@Override
	public void checkStateChanged(final CheckStateChangedEvent event) {
		((Activateable) event.getElement())
				.setState(event.getChecked() ? ActivationState.ACTIVE
						: ActivationState.INACTIVE);
	}
}
