/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.actions;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IViewActionDelegate;
import org.eclipse.ui.IViewPart;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.StreckenprofilEinstellungen;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilEditorModel;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler.EinstellungsArt;

/**
 * Aktion zum Lschen eines Streckenprofils, das im Streckenprofil Navigator
 * ausgewhlt wurde.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: StreckenprofilLoeschenAction.java,v 1.3 2014/04/17 15:51:11 Pittner Exp $
 */
public class StreckenprofilLoeschenAction implements IViewActionDelegate {

	/**
	 * Map, in der alle {@link StreckenprofilEditorModel}e und die jeweilige
	 * {@link EinstellungsArt} gespeichert wird.
	 */
	private final Map<StreckenprofilEditorModel, EinstellungsArt> modellListe = new HashMap<StreckenprofilEditorModel, EinstellungsArt>();

	@Override
	public void init(final IViewPart view) {
	}

	@Override
	public void run(final IAction action) {

		if (MessageDialog.openQuestion(Display.getDefault().getActiveShell(),
				"Streckenprofil lschen ...", "Mchten Sie das Streckenprofil "
						+ modellListe.keySet().iterator().next()
								.getStreckenZug().getName()
						+ " wirklich lschen?")) {
			for (final StreckenprofilEditorModel m : modellListe.keySet()) {
				StreckenprofilEinstellungen.getInstance()
						.setModellEinstellungen(modellListe.get(m),
								m.getStreckenZug().getName(), null);
			}
			modellListe.clear();
		}

	}

	@Override
	public void selectionChanged(final IAction action,
			final ISelection selection) {
		synchronized (modellListe) {
			modellListe.clear();
			if (selection instanceof TreeSelection) {
				final TreeSelection treeS = (TreeSelection) selection;
				for (final Object obj : treeS.toArray()) {
					final TreePath[] treePath = treeS.getPathsFor(obj);
					if (treePath.length == 1) {
						final Object parentObj = treePath[0].getFirstSegment();
						if (parentObj instanceof EinstellungsArt
								&& obj instanceof StreckenprofilEditorModel) {
							modellListe.put((StreckenprofilEditorModel) obj,
									(EinstellungsArt) parentObj);
						}
					}
				}
			}
		}
	}
}
