/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.streckenprofil.sts.a8.daten;


import java.awt.Color;
import java.awt.Font;
import java.util.Arrays;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.StreckenprofilEditor;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.type.StreckenProfilElementTyp;
 
/**
 * Modelliert den Administrationsdatensatz, der den Aufbau des Streckenprofils beschreibt
 * @author Pittner
 *
 */
public class StreckenProfilKonfigurationData implements IStreckenProfilDaten, Cloneable {

	private static final long serialVersionUID = -6713427173153117154L;

	public enum BezeichnungVerkehrsDatenQuellen{		
		MQ("MQ"), 
		HFS("HFS"), 
		UEFS1("1FS") , UEFS2("2FS") , UEFS3("3FS") , 
		UEFS4("4FS"), UEFS5("5FS"), UEFS6("6FS");
				
		public final String _string;				
		
		private BezeichnungVerkehrsDatenQuellen(String string){
			_string = string;
		}
		
		@Override
		public String toString() {			
			return _string;
		}
		
		public static Object get(int index) {
			return values()[index];			
		}
		
	}
	
	public enum BezeichnungVerkehrsDatenTypen{		
		QKFZ("QKfz", "Q", "Fz/h", new Double(65535.0), new Long(1), new Long(8), new Long(0)), 
		QPKW("QPkw", "Q", "Fz/h", new Double(65535.0), new Long(1), new Long(8), new Long(0)), 
		QLKW("QLkw", "Q", "Fz/h", new Double(65535.0), new Long(1), new Long(8), new Long(0)), 
		VKFZ("VKfz", "V", "km/h", new Double(65535.0), new Long(1), new Long(3), new Long(0)), 
		VPKW("VPkw", "V", "km/h", new Double(65535.0), new Long(1), new Long(3), new Long(0)), 
		VLKW("VLkw", "V", "km/h", new Double(65535.0), new Long(1), new Long(3), new Long(0)),
		ALKW("ALkw", "A", "%", new Double(65535.0), new Long(1), new Long(3), new Long(0)),
		KKFZ("KKfz", "K", "Fz/km", new Double(65535.0), new Long(1), new Long(3), new Long(0)), 
		KPKW("KPkw", "K", "Fz/km", new Double(65535.0), new Long(1), new Long(3), new Long(0)), 
		KLKW("KLkw", "K", "Fz/km", new Double(65535.0), new Long(1), new Long(3), new Long(0)),
		KB("KB", "KB", "PkwE/km", new Double(65535.0), new Long(1), new Long(8), new Long(0)), 
		QB("QB", "QB", "PkwE/h", new Double(65535.0), new Long(1), new Long(8), new Long(0));
		
	  public final String _string;
    public final String _formelzeichen;
    public final String _einheitenzeichen;
    public final Double _datenlueckenWert;
    public final Long _minimalWertGanzzahlLaenge;
    public final Long _maximalWertGanzzahllaenge;
    public final Long _anzahlNachkommastellen;    

    private BezeichnungVerkehrsDatenTypen(String string, 
                                          String formelzeichen, 
                                          String einheitenzeichen, 
                                          Double datenlueckenWert,
                                          Long minimalWertGanzzahlLaenge, 
                                          Long maximalWertGanzzahllaenge,
                                          Long anzahlNachkommastellen)

    {
      _string = string;
      _formelzeichen = formelzeichen;
      _einheitenzeichen = einheitenzeichen;
      _datenlueckenWert = datenlueckenWert;
      _minimalWertGanzzahlLaenge = minimalWertGanzzahlLaenge;
      _maximalWertGanzzahllaenge = maximalWertGanzzahllaenge;
      _anzahlNachkommastellen = anzahlNachkommastellen;   
    }
		
		@Override
		public String toString() {			
			return _string;
		}
		
		public static BezeichnungVerkehrsDatenTypen get(int index) {
			return values()[index];			
		}
	}
	
  public enum BezeichnungUmfeldDatenMessWerte{    
    FBT("FahrBahnOberFlchenTemperatur", "t", "C", new Double(65535.0), new Long(1), new Long(3), new Long(1)), 
    GT("GefrierTemperatur", "t", "C", new Double(65535.0), new Long(1), new Long(3), new Long(1)), 
    HK("Helligkeit", "Ev", "Lx", new Double(65535.0), new Long(1), new Long(5), new Long(0)), 
//    LD("LuftDruck", "p", "hPa", new Double(65535.0), new Long(3), new Long(4), new Long(0)), 
    LT("LuftTemperatur", "t", "C", new Double(65535.0), new Long(1), new Long(3), new Long(1)), 
    NI("NiederschlagsIntensitt", "R", "mm/h", new Double(65535.0), new Long(1), new Long(3), new Long(1)),
//    NSM("NiederschlagsMenge", "n", "mm", new Double(65535.0), new Long(1), new Long(4), new Long(1)),
    RLF("RelativeLuftFeuchte", "rF", "%", new Double(65535.0), new Long(2), new Long(3), new Long(0)), 
    RS("RestSalz", "L", "%", new Double(65535.0), new Long(1), new Long(3), new Long(0)), 
//    SH("SchneeHhe", "h", "cm", new Double(65535.0), new Long(1), new Long(3), new Long(0)),
    SW("SichtWeite", "s", "m", new Double(65535.0), new Long(2), new Long(4), new Long(0)),
    TPT("TaupunktTemperatur", "t", "C", new Double(65535.0), new Long(1), new Long(3), new Long(1)),
    TT1("TemperaturInTiefe1", "t", "C", new Double(65535.0), new Long(1), new Long(3), new Long(1)),
//    TT2("TemperaturInTiefe2", "t", "C", new Double(65535.0), new Long(1), new Long(3), new Long(1)),
    TT3("TemperaturInTiefe3", "t", "C", new Double(65535.0), new Long(1), new Long(3), new Long(1)),
    WFD("WasserFilmDicke", "h", "mm", new Double(65535.0), new Long(1), new Long(2), new Long(2));
//    WGM("WindGeschwindigkeitMittelWert", "v", "m/s", new Double(65535.0), new Long(1), new Long(2), new Long(1)),
//    WGS("WindGeschwindigkeitSpitzenWert", "v", "m/s", new Double(65535.0), new Long(1), new Long(2), new Long(1)),
//    WR("WindRichtung", "t", "", new Double(65535.0), new Long(1), new Long(3), new Long(0));

    public final String _string;
    public final String _formelzeichen;
    public final String _einheitenzeichen;
    public final Double _datenlueckenWert;
    public final Long _minimalWertGanzzahlLaenge;
    public final Long _maximalWertGanzzahllaenge;
    public final Long _anzahlNachkommastellen;    

    private BezeichnungUmfeldDatenMessWerte(String string, 
                                            String formelzeichen, 
                                            String einheitenzeichen, 
                                            Double datenlueckenWert,
                                            Long minimalWertGanzzahlLaenge, 
                                            Long maximalWertGanzzahllaenge,
                                            Long anzahlNachkommastellen)

    {
      _string = string;
      _formelzeichen = formelzeichen;
      _einheitenzeichen = einheitenzeichen;
      _datenlueckenWert = datenlueckenWert;
      _minimalWertGanzzahlLaenge = minimalWertGanzzahlLaenge;
      _maximalWertGanzzahllaenge = maximalWertGanzzahllaenge;
      _anzahlNachkommastellen = anzahlNachkommastellen;    
    }
    
    @Override
    public String toString() {      
      return _string;
    }
    
    public static BezeichnungUmfeldDatenMessWerte get(int index) {
      return values()[index];     
    }
  }

  public enum BezeichnungFahrtRichtung{    
//    NO("NO"),
//    SW("SW"),
    NO("NO"),
    SW("SW"),
    MD("Magdeburg"), 
    DD("Dresden"); 
        
    public final String _string;        
    
    private BezeichnungFahrtRichtung(String string){
      _string = string;
    }
    
    @Override
    public String toString() {      
      return _string;
    }
    
    public static Object get(int index) {
      return values()[index];     
    }
  }

  // TODO quers: #1 Neuer Enum, ob im Anzeigestreifen Texte oder Bilder angezeigt werden sollen. 
  public enum BildTyp{
    BILD("bild"),
    TEXT("text");
    
    public final String _typ;
    
    private BildTyp(String typ)
    {
      _typ = typ;
    }

    @Override
    public String toString()
    { 
      return _typ;
    }

    public static int numberOfEntities()
    {
      return values().length;
    }
    
    public static BildTyp get(int index)
    {
      return values()[index];     
    }
  }
  
  public static class TextKonfiguration
  {
    private String _text = null;
    private Color _color = null;
    private Font _font = null;
    private String _fontname = null;
    private int _fontstyle = 0;
    private double _fontsize = Double.NaN;
    private double _rotation = Double.NaN;
    private String[] _tooltip = null;
    
    public TextKonfiguration(String text, Color color, String fontname, int fontstyle, double fontsize, double rotation, String[] tooltip)
    {
      _text = text;
      _color = color;
      _font = new Font(fontname, fontstyle, (int)Math.round(fontsize));
      _fontname = fontname;
      _fontstyle = fontstyle;
      _fontsize = fontsize;
      _rotation = rotation;
      _tooltip = tooltip;
    }

    public String getText()
    {
      return _text;
    }
    
    public Color getColor()
    {
      return _color;
    }
    
    public Font getFont()
    {
      return _font;
    }
    
    public double getRotation()
    {
      return _rotation;
    }
    
    public String[] getTooltip()
    {
      return _tooltip;
    }
  
    @Override
    public String toString()
    {
      return _text + "::" + (_color == null ? "null" : _color.getRGB()) + "::" + _fontname + "::" + _fontstyle + "::" + _fontsize + "::" + _rotation;  
    }
  }
  
  // TODO quers: #2 Neue Klasse, die alle Informationen zu einem Bildzeichentypen haelt. Also welcher Elementtyp,
  //                ob oberhalb oder unterhalb des Charts anzuzeigen, ob sich die einzelnen dargestellten Werte
  //                des Anzeigestreifens ueberlappen duerfen und ob Bilder oder Texte darzustellen sind.
  public static class BildzeichenTypKonfiguration{
    private StreckenProfilElementTyp _elementTyp = null;
    private boolean _obenAnordnen = true;
    private boolean _inhalteUeberlappen = false;
    private boolean _dynamischeHoehe = false;
    private boolean _ausdehnungAuswerten = false;
    private BildTyp _bildTyp = null;
    private boolean _zeitstempelAktualisierer = true;
    
    public BildzeichenTypKonfiguration(StreckenProfilElementTyp elementTyp, boolean obenAnordnen, boolean inhalteUeberlappen, boolean dynamischeHoehe, boolean ausdehungAuswerten, BildTyp bildTyp, boolean zeitstempelAktualisierer)
    {
      _elementTyp = elementTyp;
      _obenAnordnen = obenAnordnen;
      _inhalteUeberlappen = inhalteUeberlappen;
      _dynamischeHoehe = dynamischeHoehe;
      _ausdehnungAuswerten = ausdehungAuswerten;
      _bildTyp = bildTyp;
      _zeitstempelAktualisierer = zeitstempelAktualisierer;
    }

    public StreckenProfilElementTyp getElementTyp() {
      return _elementTyp;
    }
    
    public boolean isObenAnordnen() {
      return _obenAnordnen;
    }
    
    public boolean isInhalteUeberlappen() {
      return _inhalteUeberlappen;
    }
    
    public boolean isDynamischeHoehe() {
      return _dynamischeHoehe;
    }
    
    public boolean isAusdehnungAuswerten() {
      return _ausdehnungAuswerten;
    }
    
    public BildTyp getBildTyp() {
      return _bildTyp;
    }
    
    public boolean isZeitstempelAktualisierer()
    {
      return _zeitstempelAktualisierer;
    }
  }
  
  public enum BezeichnungBildzeichenTypen{
    // TODO staBes: #2 Hier wurden die 2 neuen Bildzeichentypen eingefuegt.
    WZG_A_STATISCH(new BildzeichenTypKonfiguration(StreckenProfilElementTyp.WZG_A_STATISCH, true, true, false, true, BildTyp.BILD, false)),
    WZG_A_STATISCH_UV(new BildzeichenTypKonfiguration(StreckenProfilElementTyp.WZG_A_STATISCH_UV, true, true, false, true, BildTyp.BILD, false)),  
    WZG_A(new BildzeichenTypKonfiguration(StreckenProfilElementTyp.WZG_A, true, true, false, false, BildTyp.BILD, true)), 
    WZG_B(new BildzeichenTypKonfiguration(StreckenProfilElementTyp.WZG_B, true, true, false, false, BildTyp.BILD, true)), 
    WZG_C(new BildzeichenTypKonfiguration(StreckenProfilElementTyp.WZG_C, true, true, false, false, BildTyp.BILD, true)),
    // UFD_HELLIGKEIT(new BildzeichenTypKonfiguration(StreckenProfilElementTyp.UFD_HELLIGKEIT, true, false, false, false, BildTyp.BILD, true)), 
    UFD_SICHTWEITE(new BildzeichenTypKonfiguration(StreckenProfilElementTyp.UFD_SICHTWEITE, true, false, false, false, BildTyp.BILD, true)), 
    UFD_NIEDERSCHLAG(new BildzeichenTypKonfiguration(StreckenProfilElementTyp.UFD_NIEDERSCHLAG, true, false, false, false, BildTyp.BILD, true)),
    // TODO quers: #3 Ein neuer Anzeigestreifen fr die AQ-, MQ- und UQ-Texte
    STANDORT_QUERSCHNITT(new BildzeichenTypKonfiguration(StreckenProfilElementTyp.STANDORT_QUERSCHNITT, false, true, true, true, BildTyp.TEXT, false));
       
    public final BildzeichenTypKonfiguration _bildzeichenTypKonfiguration;
    
    private BezeichnungBildzeichenTypen(BildzeichenTypKonfiguration bildzeichenTypKonfiguration){
      _bildzeichenTypKonfiguration = bildzeichenTypKonfiguration;
    }

    @Override
    public String toString() { 
      return _bildzeichenTypKonfiguration.getElementTyp().getType();
    }

    public static int numberOfEntities()
    {
      return values().length;
    }
    
    public static BezeichnungBildzeichenTypen get(int index) {
      return values()[index];     
    }

    public BildzeichenTypKonfiguration getBildzeichenTypKonfiguration()
    {
      return _bildzeichenTypKonfiguration;
    }
  }
	
  public enum BezeichnungGeschwindigkeitBeschraenkung{    
    GSB_HFS("GSB_HFS"), 
    GSB_UEFS1("GSB_1FS") , 
    GSB_UEFS2("GSB_2FS") , 
    GSB_UEFS3("GSB_3FS"), 
    GSB_UEFS4("GSB_4FS"), 
    GSB_UEFS5("GSB_5FS"), 
    GSB_UEFS6("GSB_6FS"),
    GSB_STNDS("STDNS"),
    GSB_STATISCH("GSB_STATISCH")
    ;
        
    public final String _string;        
    public final String _formelzeichen;
    public final String _einheitenzeichen;
    public final Double _datenlueckenWert;
    public final Boolean _gemeinsameLegende;
    
    private BezeichnungGeschwindigkeitBeschraenkung(String string){
      _string = string;
      _formelzeichen = "V";
      _einheitenzeichen = "km/h";
      _datenlueckenWert = new Double(65535.0);
      _gemeinsameLegende = new Boolean(true);
    }
    
    @Override
    public String toString() {      
      return _string;
    }
    
    public static BezeichnungGeschwindigkeitBeschraenkung get(int index) {
      return values()[index];     
    }
    
    public static String getBezeichnungGeschwindigkeitBeschraenkungEnum()
    {
      return ("GSB");
    }
    
  }
  
  public enum BezeichnungPrognoseTypen{      
    FLINK_GEGLAETTET("FlinkGeglaettet"),
    NORMAL_GEGLAETTET("NormalGeglaettet"), 
    TRAEGE_GEGLAETTET("TraegeGeglaettet"),
    FLINK_EXTRA_POLATION("FlinkExtraPolation"), 
    NORMAL_EXTRA_POLATION("NormalExtraPolation"),
    TRAEGE_EXTRA_POLATION("TraegeExtraPolation"),
    ANALYSE("Analyse");
        
    public final String _string;        
    
    private BezeichnungPrognoseTypen(String string){
      _string = string;
    }
    
    @Override
    public String toString() {      
      return _string;
    }
    
    public static Object get(int index) {
      return values()[index];     
    }
    
  }
  
  private String bezeichnung;
  private long startKm;
	private long endeKm;
	private boolean verkehrsdatenQuellen[];	
	private boolean verkehrsdatenTypen[];
	private boolean bildzeichenTypen[];
	private boolean umfeldDatenMesswerte[];	
	private boolean geschwindigkeitBeschraenkung[];
//	private BezeichnungFahrtRichtung fahrtRichtung;
	private BezeichnungPrognoseTypen prognoseTypen;
  private String editor;
 
  /**
	 * @param startKm
	 * @param endeKm
	 * @param verkehrsdatenQuellen
	 * @param verkehrsdatenTypen
	 * @param anzeigeTypen
	 * @param umfeldDatenMesswerte
	 * @param umfeldDatenZustaende
	 */
	public StreckenProfilKonfigurationData(
	    String bezeichnung,			
//	    BezeichnungFahrtRichtung fahrtRichtung,
	    String editor,
			long startKm,			
			long endeKm,
			boolean verkehrsdatenQuellen[],			
			boolean verkehrsdatenTypen[],
			boolean bildzeichenTypen[],
			boolean umfeldDatenMesswerte[],
			boolean geschwindigkeitBeschraenkung[]
	) {
	  this.bezeichnung = bezeichnung;
	  this.editor = editor;
//	  this.fahrtRichtung = fahrtRichtung;
		this.startKm = startKm;
		this.endeKm = endeKm;
		this.verkehrsdatenQuellen = verkehrsdatenQuellen;
		this.verkehrsdatenTypen = verkehrsdatenTypen;
		this.bildzeichenTypen = bildzeichenTypen;
		this.umfeldDatenMesswerte = umfeldDatenMesswerte;
		this.geschwindigkeitBeschraenkung = geschwindigkeitBeschraenkung;
		this.prognoseTypen = BezeichnungPrognoseTypen.ANALYSE;
	}

	/** @see java.lang.Object#equals(java.lang.Object) */
	@Override
    public boolean equals(Object object) {
		if(object == null) {
			return false;
		}
		else if(object instanceof StreckenProfilKonfigurationData) {
			StreckenProfilKonfigurationData other = (StreckenProfilKonfigurationData)object;

			return this.bezeichnung.equals(other.bezeichnung) 
			&& this.startKm == other.startKm && this.endeKm == other.endeKm
			&& this.editor == other.editor
			&& Arrays.equals(this.verkehrsdatenQuellen, other.verkehrsdatenQuellen)
			&& Arrays.equals(this.verkehrsdatenTypen, other.verkehrsdatenTypen)
			&& Arrays.equals(this.bildzeichenTypen, other.bildzeichenTypen)
      && Arrays.equals(this.geschwindigkeitBeschraenkung, other.geschwindigkeitBeschraenkung)
			&& Arrays.equals(this.umfeldDatenMesswerte, other.umfeldDatenMesswerte);
		}
		else {
			return false;
		}
	}

	/** @see java.lang.Object#toString() */
	@Override
    public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("StreckenprofilKonfiguration: " + bezeichnung + "\n");
//		sb.append("Fahrtrichtung: " + fahrtRichtung + "\n");
		sb.append("Editor: " + editor + "\n");
		sb.append("Betriebskilometer: " + startKm + " bis " + endeKm + "\n");
		sb.append("VerkehrsdatenQuellen:\t");
		for (boolean b : verkehrsdatenQuellen) {
			sb.append(" " + b);			
		}
		sb.append("\n");
		sb.append("VerkehrsdatenTypen:\t");
		for (boolean b : verkehrsdatenTypen) {
			sb.append(" " + b);			
		}
		sb.append("\n");
		sb.append("BildzeichenTypen:\t");
		for (boolean b : bildzeichenTypen) {
			sb.append(" " + b);			
		}
		sb.append("\n");
		sb.append("UmfeldDatenMesswerte:\t");
		for (boolean b : umfeldDatenMesswerte) {
			sb.append(" " + b);			
		}
		sb.append("\n");
		sb.append("GeschwindigkeitsBeschraenkung:\t");
		for (boolean b : geschwindigkeitBeschraenkung) {
		  sb.append(" " + b);			
		}
		return sb.toString();
	}

	/** @see de.bsvrz.pua.prot.client.dataobject.ProtocolResult#getType() */
	public int getType() {
		return IStreckenProfilDaten.KONFIGURATION_DATA;
	}
	
	public String getBezeichnung()
  {
    return bezeichnung;
  }
	
//	// hor9tr added get here
//	public BezeichnungFahrtRichtung getFahrtrichtung()
//	{
//	  return (fahrtRichtung);
//	}
	
  public String getEditor()
  {
    return editor;
  }
	
	public long getStartKm() {
		return startKm;
	}

	public long getEndeKm() {
		return endeKm;
	}

	public boolean[] getVerkehrsdatenQuellen() {
		return verkehrsdatenQuellen;
	}

	public boolean[] getVerkehrsdatenTypen() {
		return verkehrsdatenTypen;
	}

	public boolean[] getBildzeichenTypen() {
		return bildzeichenTypen;
	}

	public boolean[] getUmfeldDatenMesswerte() {
		return umfeldDatenMesswerte;
	}

	public boolean[] getGeschwindigkeitBeschraenkung()
  {
    return geschwindigkeitBeschraenkung;
  }
	
	@Override
	public Object clone()
	{
	  return new StreckenProfilKonfigurationData(
	      bezeichnung,
//	      fahrtRichtung,
	      editor,
	      startKm,
	      endeKm,
	      verkehrsdatenQuellen.clone(),
	      verkehrsdatenTypen.clone(),
	      bildzeichenTypen.clone(),
	      umfeldDatenMesswerte.clone(),
	      geschwindigkeitBeschraenkung.clone()    	  
	  );
	}
	
  public BezeichnungPrognoseTypen getPrognoseTypen()
  {
    return prognoseTypen;
  }

  public void setPrognoseTypen(BezeichnungPrognoseTypen prognoseTypen)
  {
    this.prognoseTypen = prognoseTypen;
  }
 }
