/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.export;

import java.util.ArrayList;
import java.util.List;

import net.sf.paperclips.DefaultGridLook;
import net.sf.paperclips.GridColumn;
import net.sf.paperclips.GridPrint;
import net.sf.paperclips.LayerPrint;
import net.sf.paperclips.LineBorder;
import net.sf.paperclips.Print;
import net.sf.paperclips.TextStyle;

import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.nebula.widgets.grid.Grid;
import org.eclipse.nebula.widgets.grid.GridCellRenderer;
import org.eclipse.nebula.widgets.grid.GridColumnGroup;
import org.eclipse.nebula.widgets.grid.GridItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.RGB;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.export.MatrixGrid.ColumnAdvisor;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.export.MatrixGrid.MatrixGridColumnRenderer;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.export.MatrixGrid.MatrixGridFactoryHelper;

/**
 * Ermoeglicht das Drucken von Grid-Objekten.
 * 
 * @param <R>
 *            "Row" - der Typ der Zeilenobjekte
 * @param <C>
 *            "Column" - der Typ der Spaltenobjekte
 * @param <D>
 *            "Data" - der Typ der Datenobjekte
 * 
 * 
 * @author BitCtrl Systems GmbH, Enrico Schnepel
 * @version $Id: MatrixGridPrint.java,v 1.3 2014/04/17 15:51:15 Pittner Exp $
 */
public class MatrixGridPrint<R, C, D> extends LayerPrint {

	/**
	 * Erzeugt ein neues Objekt zum Drucken von {@link MatrixGrid} Objekten.
	 * 
	 * @param grid
	 *            das {@link MatrixGrid}, das gedruckt werden soll.
	 */
	@SuppressWarnings("unchecked")
	public MatrixGridPrint(final MatrixGrid<R, C, D> grid) {
		final MatrixGridFactoryHelper<R, C, D> factory = grid.getFactory();
		final ColumnAdvisor<C> columnAdvisor = factory.getColumnAdvisor();
		final GridPrint printTable = new GridPrint();
		final DefaultGridLook look = new DefaultGridLook();
		final RGB headerBackground = new RGB(200, 200, 200);
		look.setHeaderBackground(headerBackground);

		TextStyle textStyle = new TextStyle();
		final TextStyle headerStyle = textStyle.font("Arial", 8, SWT.BOLD);
		textStyle = textStyle.font("Arial", 8, SWT.NORMAL);

		look.setCellBorder(new LineBorder(new RGB(0, 0, 0)));
		printTable.setLook(look);

		final Grid gridWidget = grid.getGrid();
		final int[] columnOrder = gridWidget.getColumnOrder();
		final org.eclipse.nebula.widgets.grid.GridColumn[] columns = grid
				.getGrid().getColumns();

		final boolean rowHeaderVisible = gridWidget.isRowHeaderVisible();
		if (rowHeaderVisible) {
			printTable.addColumn(new GridColumn(SWT.LEFT,
					GridColumn.DEFAULT_SIZE, 1));
		}

		final List<Integer> group = new ArrayList<Integer>();
		int j = rowHeaderVisible ? 1 : 0;
		for (final int columnIndex : columnOrder) {
			final org.eclipse.nebula.widgets.grid.GridColumn column = columns[columnIndex];
			final C c = (C) column.getData(MatrixGrid.COLUMN_KEY);
			columnAdvisor.setColumnObject(c);
			final GridColumn gridColumn = new GridColumn(columnAdvisor
					.getJustification(), GridColumn.DEFAULT_SIZE, 1);
			printTable.addColumn(gridColumn);

			final GridCellRenderer cellRenderer = column.getCellRenderer();
			if (cellRenderer instanceof MatrixGridColumnRenderer) {
				group.add(j++);
			}
		}
		if (group.size() > 0) {
			final int[] group2 = new int[group.size()];
			for (int i = 0; i < group.size(); i++) {
				group2[i] = group.get(i);
			}
			if (rowHeaderVisible) {
				printTable.setColumnGroups(new int[][] { new int[] { 0 },
						group2 });
			} else {
				printTable.setColumnGroups(new int[][] { group2 });
			}
		}
		if (rowHeaderVisible) {
			printTable.addHeader(headerStyle.create(""));
		}
		GridColumnGroup oldColumnGroup = null;
		for (final int columnIndex : columnOrder) {
			final org.eclipse.nebula.widgets.grid.GridColumn column = columns[columnIndex];
			final GridColumnGroup columnGroup = column.getColumnGroup();
			if ((columnGroup != oldColumnGroup) && (null != columnGroup)) {
				printTable.addHeader(headerStyle.create(columnGroup.getText()),
						columnGroup.getColumns().length);
			} else if (null == columnGroup) {
				printTable.addHeader(headerStyle.create(""));
			}
			oldColumnGroup = columnGroup;
		}

		if (rowHeaderVisible) {
			printTable.addHeader(headerStyle.create(""));
		}

		for (final int columnIndex : columnOrder) {
			final C c = (C) columns[columnIndex].getData(MatrixGrid.COLUMN_KEY);
			columnAdvisor.setColumnObject(c);
			printTable.addHeader(headerStyle.create(columnAdvisor
					.getColumnText()));
		}

		final ITableLabelProvider matrixLabelProvider = (ITableLabelProvider) grid
				.getLabelProvider();
		for (final GridItem row : gridWidget.getItems()) {
			final R rData = (R) row.getData();
			if (rowHeaderVisible) {
				printTable.add(textStyle.create(row.getHeaderText()));
			}
			for (final int columnIndex : columnOrder) {
				final org.eclipse.nebula.widgets.grid.GridColumn column = columns[columnIndex];
				final C c = (C) column.getData(MatrixGrid.COLUMN_KEY);
				columnAdvisor.setColumnObject(c);
				final GridCellRenderer cellRenderer = column.getCellRenderer();
				if (cellRenderer instanceof MatrixGridColumnRenderer) {
					printTable.add(SWT.FILL, SWT.FILL,
							new GridCellRendererPrint(cellRenderer, row));
				} else {
					printTable.add(textStyle.create(matrixLabelProvider
							.getColumnText(rData, columnIndex)));
				}
			}
		}

		super.add(printTable);
	}

	@Override
	public void add(final Print print) {
		throw new UnsupportedOperationException();
	}

	@Override
	public void add(final Print print, final int align) {
		throw new UnsupportedOperationException();
	}
}
