/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.export;

import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.swt.graphics.Image;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Activator;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.StreckenprofilUtil;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.export.MatrixGrid.ColumnAdvisor;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.export.MatrixGrid.MatrixGridFactoryHelper;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.export.MatrixGrid.MatrixLabelProvider;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.LinienEigenschaften;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.TypDatenQuellen;
import de.bsvrz.sys.funclib.bitctrl.modell.OnlineDatum;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.MessQuerschnittAllgemein;

/**
 * Helper Klasse, die einen {@link org.eclipse.jface.viewers.LabelProvider} und
 * einen {@link ColumnAdvisor} fr unser {@link MatrixGrid} erstellt.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: StreckenprofilMatrixGridFactoryHelper.java,v 1.3 2014/04/17 15:51:15 Pittner Exp $
 */
public class StreckenprofilMatrixGridFactoryHelper extends
		MatrixGridFactoryHelper<SystemObjekt, LinienEigenschaften, OnlineDatum> {

	@Override
	public ITableLabelProvider createTableLabelProvider(
			final MatrixGrid<SystemObjekt, LinienEigenschaften, OnlineDatum> grid) {
		return new MatrixLabelProvider<SystemObjekt, LinienEigenschaften, OnlineDatum>(
				grid) {

			@Override
			protected Image getCellImage(final SystemObjekt row,
					final LinienEigenschaften column, final OnlineDatum data) {
				return null;
			}

			@Override
			protected String getCellText(final SystemObjekt row,
					final LinienEigenschaften column, final OnlineDatum data) {
				Double result = null;

				if (row instanceof MessQuerschnittAllgemein) {

					if (column.getTypDatenQuelle().equals(TypDatenQuellen.MQ)) {
						result = StreckenprofilUtil.getMQValue(column,
								((MessQuerschnittAllgemein) row), data);
					} else {

						result = StreckenprofilUtil.getFSValue(column,
								((MessQuerschnittAllgemein) row), data);
					}
				} else {
					Activator.getDefault().getLogger().warning(
							"Unbekannter Objekt-Typ " + row.getTyp());
				}

				if (result == null) {
					return "-";
				} else if (result.intValue() == -1) {
					return "nicht ermittelbar";
				} else if (result.intValue() == -2) {
					return "fehlerhaft";
				} else if (result.intValue() == -3) {
					return "nicht ermittelbar/fehlerhaft";
				} else {
					return result.toString();
				}
			}

		};
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.plugin.streckenprofil.export.MatrixGrid.MatrixGridFactoryHelper
	 * #getColumnAdvisor()
	 */
	@Override
	public ColumnAdvisor<LinienEigenschaften> getColumnAdvisor() {
		return new StreckenprofilMatrixColumnAdvisor();
	}

}
