/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.streckenprofil.sts.a8.histdata;

import java.util.Collection;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.Assert;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Activator;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKindCombination;
import de.bsvrz.dav.daf.main.archive.ArchiveDataQueryResult;
import de.bsvrz.dav.daf.main.archive.ArchiveDataSpecification;
import de.bsvrz.dav.daf.main.archive.ArchiveDataStream;
import de.bsvrz.dav.daf.main.archive.ArchiveOrder;
import de.bsvrz.dav.daf.main.archive.ArchiveQueryPriority;
import de.bsvrz.dav.daf.main.archive.ArchiveRequestManager;
import de.bsvrz.dav.daf.main.archive.ArchiveRequestOption;
import de.bsvrz.dav.daf.main.archive.ArchiveTimeSpecification;
import de.bsvrz.dav.daf.main.archive.TimingType;
import de.bsvrz.dav.daf.main.config.SystemObject;

public class ArchivAnfrage implements IArchivAnfrage{
  
  private final static int RETRIES = 5;
  
  private enum STATE { OK, TIMEOUT, ERROR };
  
  public ArchivAnfrage(ClientDavInterface connection, Date start, Date ende, 
      IDatenZusammenstellung datenZusammenstellung) throws 
      InterruptedException {
    this(start, ende, datenZusammenstellung, null);
  }
  
  public ArchivAnfrage(Date start, Date ende, IDatenZusammenstellung datenZusammenstellung, 
      SystemObject archiveKonfigObj) 
    throws InterruptedException {
    
    Assert.isNotNull(start, "Startzeit darf nicht null sein");
    Assert.isNotNull(ende, "Endezeit darf nicht null sein");
    Assert.isTrue(start.before(ende), "Startzeit muss kleiner als Endezeit sein");
    Assert.isNotNull(datenZusammenstellung, "Datenzusammenstellung darf nicht null sein");
    Assert.isTrue(datenZusammenstellung.getDaten() != null && !datenZusammenstellung.getDaten().isEmpty(),
        "Es mssen Daten in der Datenzusammenstellung vorhanden sein");
    
    ClientDavInterface connection = Activator.getDefault().getObjektFactory().getDav();
    final ArchiveRequestManager archive = archiveKonfigObj != null ? 
        connection.getArchive(archiveKonfigObj) : connection.getArchive();
    int anzahl = 0;
    while(!archive.isArchiveAvailable()) {
      anzahl++;
      if (anzahl > RETRIES) {  
//          throw new IllegalStateException("Abbruch der Archivanfrage, da das Archiv NICHT bereit ist");
        throw new IllegalStateException("Abbruch der Archivanfrage, da das Archiv " + (archiveKonfigObj != null ? 
            archiveKonfigObj.getNameOrPidOrId() + " " : "") + "NICHT bereit ist");
        }
        System.out.println("Warte auf das Archivsystem: Versuch " + anzahl + " von " + RETRIES);
        Thread.sleep(1000);
  
      if (anzahl > 0) {
        System.out.println("Archiv ist bereit");
      }
    }  
  
    if (archive.isArchiveAvailable()) {
      
      final ArchiveQueryPriority archiveQueryPriority = ArchiveQueryPriority.LOW;
      TimingType timingType = TimingType.ARCHIVE_TIME;
      boolean startRelative = false;
      long intervalStart = start.getTime();
      long intervalEnd = ende.getTime();

      ArchiveDataKindCombination archiveDataKindCombination = 
        new ArchiveDataKindCombination(ArchiveDataKind.ONLINE);
      ArchiveOrder archiveOrder = ArchiveOrder.BY_DATA_TIME;
      ArchiveRequestOption archiveRequestOption = ArchiveRequestOption.NORMAL;
      ArchiveTimeSpecification archiveTimeSpecification = new ArchiveTimeSpecification(timingType, startRelative, intervalStart, intervalEnd);
      final List<ArchiveDataSpecification> archiveDataSpecifications = new LinkedList<ArchiveDataSpecification>();
      
      Map<DataDescription, Collection<SystemObject>> daten = datenZusammenstellung.getDaten(); 
      for (DataDescription dd: daten.keySet()) {
        Collection<SystemObject> objekte = daten.get(dd);
        for (SystemObject so: objekte) {
          archiveDataSpecifications.add(new ArchiveDataSpecification(
              archiveTimeSpecification, archiveDataKindCombination,
              archiveOrder, archiveRequestOption, dd, so));  
        }
      }
      
      // Anfrage der Archivdaten          
//      ArchiveDataQueryResult queryResult = archive.request(archiveQueryPriority, archiveDataSpecifications);
//      
//      if (queryResult.isRequestSuccessful())
//      {
//        ergebnis = queryResult.getStreams();
////        System.out.println("Archivanfrage konnte erfolgreich bearbeitet werden.");
//      }
//      else {
//        throw new RuntimeException(queryResult.getErrorMessage());
//      }
      
      final ArchiveDataQueryResult queryResult = archive.request(archiveQueryPriority, archiveDataSpecifications);
      state = STATE.TIMEOUT;
      
      PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable()
      {
        public void run() {
          try
          {
            synchronized(ArchivAnfrage.this) {
              if (queryResult.isRequestSuccessful())
              {
                  state = STATE.OK;
                  ArchivAnfrage.this.notify();  
              }
              else {
                state = STATE.ERROR;
              }
            }
          }  
          catch (InterruptedException e)
          {
            // TODO Auto-generated catch block
            e.printStackTrace();
          }
        }
      });
      
      synchronized(this) {
        wait(60000);
        switch (state)
        {
          case OK:
            ergebnis = queryResult.getStreams();
//          System.out.println("Archivanfrage konnte erfolgreich bearbeitet werden.");
            break;
          case TIMEOUT:
            throw new RuntimeException("Das Archiv " + (archiveKonfigObj != null ? 
                archiveKonfigObj.getNameOrPidOrId() + " " : "") + "hat nicht geantwortet");
          case ERROR:
            throw new RuntimeException(queryResult.getErrorMessage());
          default:
            throw new IllegalStateException("Illegal State");
        }

      }
      
    }
    
  }
  
  private STATE state;
  
  private ArchiveDataStream[] ergebnis;
  
  public ArchiveDataStream[] getErgebnis() {
    return ergebnis;
  }

  public void close() {
  
    for (int i = 0; i < ergebnis.length; i++) {
      ergebnis[i].abort();
    }
  }
  
}
