/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.adapter;

import org.eclipse.emf.common.util.EList;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.ActivationState;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.AlternativeStreckenAbschnitteContainer;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.RealerStreckenAbschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenAbschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilPackage;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.StrassenSegment;

/**
 * Adapter zur Bestimmung des aktiven {@link StreckenAbschnitt}s eines
 * {@link AlternativeStreckenAbschnitteContainer}s.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: ASACAktiverSAAdapter.java,v 1.3 2014/04/17 15:51:19 Pittner Exp $
 */
public final class ASACAktiverSAAdapter extends DerivedAttributeAdapter {

	/**
	 * Erzeugt einen neuen Adapter fr die Ermittlung des aktiven
	 * {@link StreckenAbschnitt}s.
	 * 
	 * @param source
	 *            der {@link AlternativeStreckenAbschnitteContainer}, dessen
	 *            aktiver {@link StreckenAbschnitt} bestimmt werden soll.
	 */
	public ASACAktiverSAAdapter(
			final AlternativeStreckenAbschnitteContainer<? extends RealerStreckenAbschnitt<? extends StrassenSegment>> source) {
		super(
				source,
				StreckenprofilPackage.Literals.ALTERNATIVE_STRECKEN_ABSCHNITTE_CONTAINER__AKTIVER_STRECKEN_ABSCHNITT,
				StreckenprofilPackage.Literals.ALTERNATIVE_STRECKEN_ABSCHNITTE_CONTAINER__INTERNAL_AKTIVER_STRECKEN_ABSCHNITT);
		new SetDerivedAttributeAdapter(
				source,
				StreckenprofilPackage.Literals.ALTERNATIVE_STRECKEN_ABSCHNITTE_CONTAINER__INTERNAL_AKTIVER_STRECKEN_ABSCHNITT,
				StreckenprofilPackage.Literals.ACTIVATEABLE_CONTAINER__ACTIVATEABLES,
				StreckenprofilPackage.Literals.ACTIVATEABLE__STATE) {

			@Override
			protected Object compute() {

				if (source.getAktiverStreckenAbschnitt() != null) {
					// bei Streckenknoten wird nur die Lnge der
					// Hauptfahrbeziehung benutzt.
					return source.getAktiverStreckenAbschnitt();
				}

				final EList<? extends StreckenAbschnitt> alternativen = source
						.getStreckenEintraege();
				for (final StreckenAbschnitt abschnitt : alternativen) {
					if (ActivationState.ACTIVE.equals(abschnitt.getState())) {
						return abschnitt;
					}
				}
				return null;
			}
		};
	}
}
