/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.adapter;

import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.Strecke;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenAbschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenKnoten;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenZugEintrag;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilEditorModel;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilPackage;

/**
 * Berechnet die Positionen und Wege innerhalb des Streckenprofils.
 * 
 * @author BitCtrl Systems GmbH, Schnepel / Hoesel
 * @version $Id: WegeAdapter.java,v 1.3 2014/04/17 15:51:19 Pittner Exp $
 */
public final class WegeAdapter extends AbstractDependantAttributeAdapter {

	/**
	 * Erezeugt einen neuen {@link WegeAdapter} fr einen
	 * {@link StreckenZugEintrag}.
	 * 
	 * @param source
	 *            der {@link StreckenZugEintrag}, dessen "Wege" berechnet werden
	 *            sollen.
	 */
	public WegeAdapter(final StreckenZugEintrag source) {
		super(source);
		if (!(source instanceof Strecke)) {
			addNavigatedDependency(
					StreckenprofilPackage.Literals.ACTIVATEABLE_CONTAINER__ACTIVATEABLES,
					StreckenprofilPackage.Literals.STRECKEN_ZUG_EINTRAG__LAENGE);
		}
		if (!(source instanceof StreckenKnoten)) {
			addNavigatedDependency(
					StreckenprofilPackage.Literals.STRECKEN_KNOTEN__VON_STRECKE,
					StreckenprofilPackage.Literals.STRECKEN_ZUG_EINTRAG__LAENGE);
			addNavigatedDependency(
					StreckenprofilPackage.Literals.STRECKEN_KNOTEN__VON_STRECKE,
					StreckenprofilPackage.Literals.STRECKEN_ZUG_EINTRAG__WEG_BISHER);
		}
		if (!(source instanceof StreckenAbschnitt)) {
			addNavigatedDependency(
					StreckenprofilPackage.Literals.STRECKEN_ABSCHNITT__VON_KNOTEN,
					StreckenprofilPackage.Literals.STRECKEN_ZUG_EINTRAG__LAENGE);
			addNavigatedDependency(
					StreckenprofilPackage.Literals.STRECKEN_ABSCHNITT__VON_KNOTEN,
					StreckenprofilPackage.Literals.STRECKEN_ZUG_EINTRAG__WEG_BISHER);
		}
	}

	@Override
	protected void notifyNavigationTargetChange(final Notification notification) {
		calculate(getSource());
	}

	@Override
	protected void notifyDependantChange(final Notification notification) {
		calculate(getSource());
	}

	/**
	 * Startet die Berechnung der Entferungen innerhalb des bergebenen
	 * {@link EObject}s.
	 * 
	 * @param eObject
	 *            das {@link EObject}, dessen Position innerhalb des
	 *            Streckenprofils bestimmt werden soll.
	 */
	private void calculate(final EObject eObject) {
		if (null == eObject) {
			return;
		}
		if (eObject instanceof StreckenprofilEditorModel) {
			// Das ist eigentlich nich so doll: Wenn man hier keine 50 ms
			// wartet, dann gibt es eine ArrayIndexOutOfBoundException im
			// WegeCalculatorJob :(
			final StreckenprofilEditorModel model = (StreckenprofilEditorModel) eObject;
			final Job wegeCalculator = model.getWegeCalculator();
			if ((wegeCalculator != null)
					&& (wegeCalculator.getState() == Job.NONE)) {
				wegeCalculator.schedule(150);
			}
		} else {
			calculate(eObject.eContainer());
		}
	}
}
