package de.bsvrz.buv.plugin.ursacheneinheit.defaultParameter;

import java.io.IOException;

import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.ursacheneinheit.Activator;
import de.bsvrz.buv.plugin.ursacheneinheit.verwaltung.VersorgungsArt;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.sys.funclib.dataSerializer.NoSuchVersionException;

public class DefaultParameterHandler
{
  Button _button;
  IDefaultParameterVersorgung _versorgung;
  
  public DefaultParameterHandler(Composite parent, IDefaultParameterVersorgung versorgung) {
    
    switch (versorgung.getVersorgungsArt()) {
        case EinReiterProUrsachenEinheit:
        case EinReiterFuerAlleUrsachenEinheiten:
          _versorgung = versorgung;  
          break;
        default:
          throw new IllegalStateException("Nicht untersttzte Versorgungsart");
    }

    _button = new Button( parent, SWT.NONE );
    _button.setText("Defaultparam.");
    
    if (versorgung.getVersorgungsArt() == VersorgungsArt.EinReiterFuerAlleUrsachenEinheiten) {
      _button.setToolTipText("Setzen aller Parameterwerte der ausgewhlten Objekte"
          + " (nicht: Zuordnungen von MQ/UFD) auf die Werte der"
          + " entsprechenden Defaultparameter");  
    }
    else {
      _button.setToolTipText("Setzen aller Parameterwerte des aktuellen Objekts"
          + " (nicht: Zuordnungen von MQ/UFD) auf die Werte der"
          + " entsprechenden Defaultparameter");
    }
   
    _button.addSelectionListener(new SelectionListener());
  }
  
  private void buttonPressed() {
    System.out.println("_versorgung.getVersorgungsArt():" + _versorgung.getVersorgungsArt());
    System.out.println("_versorgung.getRootObjektPid():" + _versorgung.getRootObjektPid());
    System.out.println("_versorgung.getParameterAtgPid():" + _versorgung.getParameterAtgPid());
  
    Data data;
    Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
    
    try {
       data = DefaultParameterTools.getData(_versorgung.getRootObjektPid(), _versorgung.getParameterAtgPid());  
    }
    catch (NoSuchVersionException e) {
      String msg = "Versions-Fehler beim Deserialisieren von Defaultparametern fr " + _versorgung.getRootObjektPid();
      Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, msg, e));
      MessageBox msgBox = new MessageBox(shell, SWT.ICON_ERROR);
      msgBox.setText("Fehler");
      msgBox.setMessage(msg + ":\n" + e.getMessage());
      msgBox.open();
      return;
    }
    catch (IOException e) {
      String msg = "I/O-Fehler beim Lesen von Defaultparametern fr " + _versorgung.getRootObjektPid();
      Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, msg, e));
      MessageBox msgBox = new MessageBox(shell, SWT.ICON_ERROR);
      msgBox.setText("Fehler");
      msgBox.setMessage(msg + ":\n" + e.getMessage());
      msgBox.open();
      return;
    }
      
    if (data != null) {
      MessageBox msgBox = new MessageBox(shell, SWT.ICON_QUESTION | SWT.OK | SWT.CANCEL);
      msgBox.setText("Defaultparameter setzen");
      if (_versorgung.getVersorgungsArt() == VersorgungsArt.EinReiterFuerAlleUrsachenEinheiten) {
        msgBox.setMessage("Sollen die Parameterwerte der ausgewhlten Objekte (nicht: Zuordnungen von MQ/UFD) auf die Werte der entsprechenden Defaultparameter gesetzt werden?");  
      }
      else {
        msgBox.setMessage("Sollen die Parameterwerte des aktuellen Objekts (nicht: Zuordnungen von MQ/UFD) auf die Werte der entsprechenden Defaultparameter gesetzt werden?");  
      }
      
      if (msgBox.open() == SWT.OK) {
        _versorgung.setData(data);  
      }
    }
    else {
      MessageBox msgBox = new MessageBox(shell, SWT.ICON_WARNING);
      msgBox.setText("Defaultparameter nicht gefunden");
      msgBox.setMessage("Defaultparameter nicht gefunden fr " +  _versorgung.getRootObjektPid());
      msgBox.open();
    }
    
    System.out.println("Deserialisierte Daten:\n" + data);
  }
  
  private class SelectionListener extends SelectionAdapter {
    
    public void widgetSelected(SelectionEvent e) {
      buttonPressed();
    }
    
  }
  
}
  
