/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.einzelmassnahmen;

import java.util.Arrays;

import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.Schaltbild;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.Sperrung;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Liefert die Datenstruktur eines SonderProgrammes. <br>
 * Implementiert das Interface {@link Cloneable}.
 * 
 * @author Pittner
 * 
 */
public class SonderProgramm implements Cloneable
{

  @Override
  public String toString()
  {
    return "SonderProgramm [_auswahlBisKm=" + _auswahlBisKm + ", _auswahlVonKm=" + _auswahlVonKm + ", _betriebsArt="
        + Arrays.toString(_betriebsArt) + ", _bezeichnung=" + _bezeichnung + ", _datenZeit=" + _datenZeit
        + ", _ereignisse=" + Arrays.toString(_ereignisse) + ", _gewaehlteSonderProgrammAktion="
        + _gewaehlteSonderProgrammAktion + ", _gewaehlteSonderProgrammRichtung=" + _gewaehlteSonderProgrammRichtung
        + ", _gewaehlterSonderProgrammZustand=" + _gewaehlterSonderProgrammZustand + ", _handSchaltung="
        + _handSchaltung + ", _monatsAuswahlBereiche=" + _monatsAuswahlBereiche + ", _monatsAuswahlMonate="
        + Arrays.toString(_monatsAuswahlMonate) + ", _pid=" + _pid + ", _prioritaetHandschaltung="
        + _prioritaetHandschaltung + ", _querLaengsabgleich=" + _querLaengsabgleich + ", _schaltOptionen="
        + Arrays.toString(_schaltOptionen) + ", _schaltbilder=" + Arrays.toString(_schaltbilder) + ", _sperrungen="
        + Arrays.toString(_sperrungen) + ", _spurSperrung=" + _spurSperrung + ", _stundenAuswahlBereiche="
        + _stundenAuswahlBereiche + ", _stundenAuswahlStunden=" + Arrays.toString(_stundenAuswahlStunden)
        + ", _systemKalenderEintraege=" + Arrays.toString(_systemKalenderEintraege) + ", _typBezeichnung="
        + _typBezeichnung + ", _wochenTagAuswahlBereiche=" + _wochenTagAuswahlBereiche + ", _wochenTagAuswahlTage="
        + Arrays.toString(_wochenTagAuswahlTage) + ", _zeitBereichBis=" + Arrays.toString(_zeitBereichBis)
        + ", _zeitBereichVon=" + Arrays.toString(_zeitBereichVon) + ", _zusatzPruefung="
        + Arrays.toString(_zusatzPruefung) + "]";
  }

  /**
   * Bezeichnung des SonderProgramms
   */
  private String _pid = null;

  /**
   * Bezeichnung des SonderProgramms
   */
  private String _bezeichnung = null;

  /**
   * Das SystemObjekt des Grundprogramms
   */
  private String _typBezeichnung = null;

  /**
   * Mgliche SonderProgrammAktionen
   * 
   * @author Pittner
   */
  public enum EnumSonderProgrammAktion {
    Bilden, ndern, Lschen, Parametrieren, undefiniert
  };

  /**
   * Gewhlte SonderProgrammAktion
   */
  private EnumSonderProgrammAktion _gewaehlteSonderProgrammAktion = null;

  /**
   * Mgliche SonderProgrammZustaende
   * 
   * @author Pittner
   */
  public enum EnumSonderProgrammZustand {
    Aktiv, SIM, Fehler, Pause, Wartend, abgelaufen, abgebrochen, inaktiv, undefiniert
  };

  /**
   * Gewhlter SonderProgrammzustand
   */
  private EnumSonderProgrammZustand _gewaehlterSonderProgrammZustand = null;

  /**
   * Mgliche SonderProgrammRichtung
   * 
   * @author Pittner
   */
  public enum EnumSonderProgrammRichtung {
    NO, SW
  };

  /**
   * Gewhlte SonderProgrammRichtung
   */
  private EnumSonderProgrammRichtung _gewaehlteSonderProgrammRichtung = null;

  /**
   * Die Auswahl in Autobahn-Km
   */
  private Double _auswahlVonKm = null;

  /**
   * Die Auswahl in Autobahn-Km
   */

  private Double _auswahlBisKm = null;
  
  /**
   * Die Auswahl in Autobahn-Km
   */
  private Double _auswahlVonKm2 = null;
  
  /**
   * Die Auswahl in Autobahn-Km
   */
  
  private Double _auswahlBisKm2 = null;

  /**
   * Handschaltung
   */
  private Boolean _handSchaltung = null;

  /**
   * Sperrung
   */
  private Sperrung[] _sperrungen = null;

  /**
   * Prioritaet
   */
  private String _prioritaetHandschaltung = null;

  /**
   * QuerLaengsabgleich
   */
  private Boolean _querLaengsabgleich = null;

  /**
   * Zusatzpruefung
   */
  private Boolean[] _zusatzPruefung = new Boolean[3];

  /**
   * Schaltoption
   */
  private Boolean[] _schaltOptionen = new Boolean[2];

  /**
   * Betriebsart
   */
  private Boolean[] _betriebsArt = new Boolean[2];

  /**
   * ZeitBereich Von
   */
  private Object[] _zeitBereichVon = new Object[3];

  /**
   * ZeitBereich Bis
   */
  private Object[] _zeitBereichBis = new Object[3];

  /**
   * Stundenauswahl Bereiche
   */
  private String _stundenAuswahlBereiche = null;

  /**
   * Stundenauswahl Stunden
   */
  private Boolean[] _stundenAuswahlStunden = new Boolean[24];

  /**
   * Wochentagauswahl Bereiche
   */
  private String _wochenTagAuswahlBereiche = null;

  /**
   * Wochentagaauswahl Stunden
   */

  private Boolean[] _wochenTagAuswahlTage = new Boolean[7];

  /**
   * Monatsauswahl Bereiche
   */
  private String _monatsAuswahlBereiche = null;

  /**
   * Monatsauswahl Monate
   */
  private Boolean[] _monatsAuswahlMonate = new Boolean[12];

  /**
   * Schaltbild
   */
  private Schaltbild[] _schaltbilder = null;

  /**
   * Schaltbild
   */
  private Long _datenZeit = null;

  /**
   * Ereignisse
   */
  private SystemObject[] _ereignisse = null;

  /**
   * Ereignisse
   */
  private SystemObject[] _systemKalenderEintraege = null;

  public Schaltbild[] getSchaltbilder()
  {
    return _schaltbilder;
  }

  public void setSchaltbilder(Schaltbild[] schaltbilder)
  {
    _schaltbilder = schaltbilder;
  }

  /**
   * Handschaltung
   */
  private Boolean _spurSperrung;

  public SonderProgramm()
  {
  }
  
  public SonderProgramm(SonderProgramm sonderProgrammDatensatz)
  {
    _pid = sonderProgrammDatensatz._pid;   
    _bezeichnung = sonderProgrammDatensatz._bezeichnung;   
    _typBezeichnung =  sonderProgrammDatensatz._typBezeichnung;   
    _gewaehlteSonderProgrammAktion = sonderProgrammDatensatz._gewaehlteSonderProgrammAktion;
    _gewaehlterSonderProgrammZustand = sonderProgrammDatensatz._gewaehlterSonderProgrammZustand;
    _gewaehlteSonderProgrammRichtung = sonderProgrammDatensatz._gewaehlteSonderProgrammRichtung;
    _auswahlVonKm = sonderProgrammDatensatz._auswahlVonKm;
    _auswahlBisKm = sonderProgrammDatensatz._auswahlBisKm;
    _handSchaltung = sonderProgrammDatensatz._handSchaltung;
    _sperrungen = sonderProgrammDatensatz._sperrungen;
    _prioritaetHandschaltung = sonderProgrammDatensatz._prioritaetHandschaltung;      
    _querLaengsabgleich = sonderProgrammDatensatz._querLaengsabgleich;
    _zusatzPruefung = sonderProgrammDatensatz._zusatzPruefung;
    _schaltOptionen = sonderProgrammDatensatz._schaltOptionen;
    _betriebsArt = sonderProgrammDatensatz._betriebsArt;
    _zeitBereichVon = sonderProgrammDatensatz._zeitBereichVon;
    _zeitBereichBis = sonderProgrammDatensatz._zeitBereichBis;
    _stundenAuswahlBereiche = sonderProgrammDatensatz._stundenAuswahlBereiche;
    _stundenAuswahlStunden = sonderProgrammDatensatz._stundenAuswahlStunden;
    _wochenTagAuswahlBereiche = sonderProgrammDatensatz._wochenTagAuswahlBereiche;
    _wochenTagAuswahlTage = sonderProgrammDatensatz._wochenTagAuswahlTage;
    _monatsAuswahlBereiche = sonderProgrammDatensatz._monatsAuswahlBereiche;
    _monatsAuswahlMonate = sonderProgrammDatensatz._monatsAuswahlMonate;
    _schaltbilder = sonderProgrammDatensatz._schaltbilder;
    _schaltbilder = sonderProgrammDatensatz._schaltbilder;
    _datenZeit = sonderProgrammDatensatz._datenZeit;
    _ereignisse = sonderProgrammDatensatz._ereignisse;
    _systemKalenderEintraege = sonderProgrammDatensatz._systemKalenderEintraege;
    _spurSperrung = sonderProgrammDatensatz._spurSperrung;
  }
  
  @Override
  protected Object clone() throws CloneNotSupportedException
  {
    SonderProgramm sonderProgrammDatensatz = new SonderProgramm();
    sonderProgrammDatensatz._pid = _pid;   
    sonderProgrammDatensatz._bezeichnung = _bezeichnung;   
    sonderProgrammDatensatz._typBezeichnung =  _typBezeichnung;   
    sonderProgrammDatensatz._gewaehlteSonderProgrammAktion = _gewaehlteSonderProgrammAktion;
    sonderProgrammDatensatz._gewaehlterSonderProgrammZustand = _gewaehlterSonderProgrammZustand;
    sonderProgrammDatensatz._gewaehlteSonderProgrammRichtung = _gewaehlteSonderProgrammRichtung;
    sonderProgrammDatensatz._auswahlVonKm = _auswahlVonKm;
    sonderProgrammDatensatz._auswahlBisKm = _auswahlBisKm;
    sonderProgrammDatensatz._handSchaltung = _handSchaltung;
    sonderProgrammDatensatz._sperrungen = _sperrungen;
    sonderProgrammDatensatz._prioritaetHandschaltung = _prioritaetHandschaltung;      
    sonderProgrammDatensatz._querLaengsabgleich = _querLaengsabgleich;
    sonderProgrammDatensatz._zusatzPruefung = _zusatzPruefung;
    sonderProgrammDatensatz._schaltOptionen = _schaltOptionen;
    sonderProgrammDatensatz._betriebsArt = _betriebsArt;
    sonderProgrammDatensatz._zeitBereichVon = _zeitBereichVon;
    sonderProgrammDatensatz._zeitBereichBis = _zeitBereichBis;
    sonderProgrammDatensatz._stundenAuswahlBereiche = _stundenAuswahlBereiche;
    sonderProgrammDatensatz._stundenAuswahlStunden = _stundenAuswahlStunden;
    sonderProgrammDatensatz._wochenTagAuswahlBereiche = _wochenTagAuswahlBereiche;
    sonderProgrammDatensatz._wochenTagAuswahlTage = _wochenTagAuswahlTage;
    sonderProgrammDatensatz._monatsAuswahlBereiche = _monatsAuswahlBereiche;
    sonderProgrammDatensatz._monatsAuswahlMonate = _monatsAuswahlMonate;
    sonderProgrammDatensatz._schaltbilder = _schaltbilder;
    sonderProgrammDatensatz._schaltbilder = _schaltbilder;
    sonderProgrammDatensatz._datenZeit = _datenZeit;
    sonderProgrammDatensatz._ereignisse = _ereignisse;
    sonderProgrammDatensatz._systemKalenderEintraege = _systemKalenderEintraege;
    sonderProgrammDatensatz._spurSperrung = _spurSperrung;
    
    return sonderProgrammDatensatz;
  }
  
  
  public SonderProgramm kopie()
  {
    // TODO Auto-generated method stub
    try
    {
      return (SonderProgramm)clone();
    }
    catch (CloneNotSupportedException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    return null;
  }
  

  /**
   * Liefert den Namen der Engstelle.
   * 
   * @return Name der Engstelle.
   */
  public String getBezeichnung()
  {
    return _bezeichnung;
  }

  /**
   * Setzt den Namen der Engstelle.
   * 
   * @param _name
   *          Name der Engstelle.
   */
  public void setBezeichnung(String _name)
  {
    this._bezeichnung = _name;
  }

  /**
   * Liefert den gewhlten Engstellentyp.
   * 
   * @return Gewhlter Engstellentyp.
   */
  public EnumSonderProgrammAktion getGewaehlterEngstellenTyp()
  {
    return _gewaehlteSonderProgrammAktion;
  }

  /**
   * Setzt den EngstellenTyp.
   * 
   * @param engstellenTyp
   *          EngstellenTyp.
   */
  public void setGewaehlterEngstellenTyp(EnumSonderProgrammAktion engstellenTyp)
  {
    _gewaehlteSonderProgrammAktion = engstellenTyp;
  }

  public String getTypBezeichnung()
  {
    return _typBezeichnung;
  }

  public void setTypBezeichnung(String typBezeichnung)
  {
    _typBezeichnung = typBezeichnung;
  }

  public EnumSonderProgrammAktion getGewaehlteSonderProgrammAktion()
  {
    return _gewaehlteSonderProgrammAktion;
  }

  public void setGewaehlteSonderProgrammAktion(EnumSonderProgrammAktion gewaehlteSonderProgrammAktion)
  {
    _gewaehlteSonderProgrammAktion = gewaehlteSonderProgrammAktion;
  }

  public EnumSonderProgrammZustand getGewaehlterSonderProgrammZustand()
  {
    return _gewaehlterSonderProgrammZustand;
  }

  public void setGewaehlterSonderProgrammZustand(EnumSonderProgrammZustand gewaehlterSonderProgrammZustand)
  {
    _gewaehlterSonderProgrammZustand = gewaehlterSonderProgrammZustand;
  }

  public EnumSonderProgrammRichtung getGewaehlteSonderProgrammRichtung()
  {
    return _gewaehlteSonderProgrammRichtung;
  }

  public void setGewaehlteSonderProgrammRichtung(EnumSonderProgrammRichtung gewaehlteSonderProgrammRichtung)
  {
    _gewaehlteSonderProgrammRichtung = gewaehlteSonderProgrammRichtung;
  }

  public static String umsetzungGeschwindigkeitsbeschrnkung(AnzeigeInhaltTyp inhalt)
  {

    String returnString = null;

    if (inhalt.equals(AnzeigeInhaltTyp.UNDEFINIERT))
    {
      returnString = "undefiniert";
    }
    else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_120))
    {
      returnString = "120 km/h";
    }
    else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_100))
    {
      returnString = "100 km/h";
    }
    else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_80))
    {
      returnString = "80 km/h";
    }
    else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_60))
    {
      returnString = "60 km/h";
    }
    else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_40))
    {
      returnString = "40 km/h";
    }
    else
    {

    }

    return returnString;
  }

  public static String umsetzungUeberholverbot(AnzeigeInhaltTyp inhalt)
  {

    String returnString = null;

    if (inhalt.equals(AnzeigeInhaltTyp.UNDEFINIERT))
    {
      returnString = "undefiniert";
    }
    else if (inhalt.equals(AnzeigeInhaltTyp.TLS_UEBERHOLVERBOT_ALLGEMEIN))
    {
      returnString = "allgemeines berholverbot";
    }
    else if (inhalt.equals(AnzeigeInhaltTyp.TLS_UEBERHOLVERBOT_LKW))
    {
      returnString = "Lkw-berholverbot";
    }
    else
    {

    }

    return returnString;
  }

  public Boolean[] getSchaltOptionen()
  {
    return _schaltOptionen;
  }

  public void setSchaltOptionen(Boolean[] schaltOptionen)
  {
    _schaltOptionen = schaltOptionen;
  }

  public Boolean[] getBetriebsArt()
  {
    return _betriebsArt;
  }

  public void setBetriebsArt(Boolean[] betriebsArt)
  {
    _betriebsArt = betriebsArt;
  }

  public Object[] getZeitBereichVon()
  {
    return _zeitBereichVon;
  }

  public void setZeitBereichVon(Object[] zeitBereichVon)
  {
    _zeitBereichVon = zeitBereichVon;
  }

  public Object[] getZeitBereichBis()
  {
    return _zeitBereichBis;
  }

  public void setZeitBereichBis(Object[] zeitBereichBis)
  {
    _zeitBereichBis = zeitBereichBis;
  }

  public String getStundenAuswahlBereiche()
  {
    return _stundenAuswahlBereiche;
  }

  public void setStundenAuswahlBereiche(String stundenAuswahlBereiche)
  {
    _stundenAuswahlBereiche = stundenAuswahlBereiche;
  }

  public Boolean[] getStundenAuswahlStunden()
  {
    return _stundenAuswahlStunden;
  }

  public void setStundenAuswahlStunden(Boolean[] stundenAuswahlStunden)
  {
    _stundenAuswahlStunden = stundenAuswahlStunden;
  }

  public String getWochenTagAuswahlBereiche()
  {
    return _wochenTagAuswahlBereiche;
  }

  public void setWochenTagAuswahlBereiche(String wochenTagAuswahlBereiche)
  {
    _wochenTagAuswahlBereiche = wochenTagAuswahlBereiche;
  }

  public Boolean[] getWochenTagAuswahlTage()
  {
    return _wochenTagAuswahlTage;
  }

  public void setWochenTagAuswahlTage(Boolean[] wochenTagAuswahlTage)
  {
    _wochenTagAuswahlTage = wochenTagAuswahlTage;
  }

  public String getMonatsAuswahlBereiche()
  {
    return _monatsAuswahlBereiche;
  }

  public void setMonatsAuswahlBereiche(String monatsAuswahlBereiche)
  {
    _monatsAuswahlBereiche = monatsAuswahlBereiche;
  }

  public Boolean[] getMonatsAuswahlMonate()
  {
    return _monatsAuswahlMonate;
  }

  public void setMonatsAuswahlMonate(Boolean[] monatsAuswahlMonate)
  {
    _monatsAuswahlMonate = monatsAuswahlMonate;
  }

  public Double getAuswahlVonKm()
  {
    return _auswahlVonKm;
  }

  public void setAuswahlVonKm(Double auswahlVonKm)
  {
    _auswahlVonKm = auswahlVonKm;
  }

  public Double getAuswahlBisKm()
  {
    return _auswahlBisKm;
  }

  public void setAuswahlBisKm(Double auswahlBisKm)
  {
    _auswahlBisKm = auswahlBisKm;
  }
  
  public Double getAuswahlVonKm2()
  {
    return _auswahlVonKm2;
  }
  
  public void setAuswahlVonKm2(Double auswahlVonKm)
  {
    _auswahlVonKm2 = auswahlVonKm;
  }
  
  public Double getAuswahlBisKm2()
  {
    return _auswahlBisKm2;
  }
  
  public void setAuswahlBisKm2(Double auswahlBisKm)
  {
    _auswahlBisKm2 = auswahlBisKm;
  }

  public Long getDatenZeit()
  {
    return _datenZeit;
  }

  public void setDatenZeit(Long datenZeit)
  {
    _datenZeit = datenZeit;
  }

  public Boolean getQuerLaengsabgleich()
  {
    return _querLaengsabgleich;
  }

  public void setQuerLaengsabgleich(Boolean querLaengsabgleich)
  {
    _querLaengsabgleich = querLaengsabgleich;
  }

  public Boolean[] getZusatzPruefung()
  {
    return _zusatzPruefung;
  }

  public void setZusatzPruefung(Boolean[] zusatzPruefung)
  {
    _zusatzPruefung = zusatzPruefung;
  }

  public Boolean getHandSchaltung()
  {
    return _handSchaltung;
  }

  public void setHandSchaltung(Boolean handSchaltung)
  {
    _handSchaltung = handSchaltung;
  }

  public String getPrioritaetHandschaltung()
  {
    return _prioritaetHandschaltung;
  }

  public void setPrioritaetHandschaltung(String prioritaetHandschaltung)
  {
    _prioritaetHandschaltung = prioritaetHandschaltung;
  }

  public Boolean isSpurSperrung()
  {
    return _spurSperrung;
  }

  public void setSpurSperrung(Boolean spurSperrung)
  {
    _spurSperrung = spurSperrung;
  }

  public String getPid()
  {
    return _pid;
  }

  public void setPid(String pid)
  {
    _pid = pid;
  }

  public Sperrung[] getSperrungen()
  {
    return _sperrungen;
  }

  public void setSperrungen(Sperrung[] sperrungen)
  {    
    _sperrungen = sperrungen;
  }

  public Boolean getSpurSperrung()
  {
    return _spurSperrung;
  }

  public SystemObject[] getEreignisse()
  {
    return _ereignisse;
  }

  public void setEreignisse(SystemObject[] ereignisse)
  {
    _ereignisse = ereignisse;
  }

  public SystemObject[] getSystemKalenderEintraege()
  {
    return _systemKalenderEintraege;
  }

  public void setSystemKalenderEintraege(SystemObject[] systemKalenderEintraege)
  {
    _systemKalenderEintraege = systemKalenderEintraege;
  }

}
