/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.seitenwind;

import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

/**
 * Klasse erzeugt ein Composite mit denen es mglich ist ein Objekt zuzuordnen
 * @author ObertM
 */
public class AuswahlObjekt
{
	/**
	 * Parent Composite
	 */
  private Composite _parent;

  /**
   * Textuelle Beschreibung der Objekte
   */
	private String _kennung;

	/**
	 * ComboBox zur Auswahl eines Objekts
	 */
	private Combo _combo;

	/**
	 * Konstruktor der Klasse
	 * @param parent parent Composite
	 * @param kennung Kennung die bei der Auswahl eingeblendet wird (z.B. Messquerschnitte)
	 */
  public AuswahlObjekt (Composite parent, String kennung)
  {
  	_parent = parent;
  	
  	_kennung = kennung;
  	
  	erzeugeEingabeElemente();
  }
	
	/**
	 * Methode erzeugt die Eingabeelemente
	 */
  private void erzeugeEingabeElemente()
  {
    GridData gridDataGroup = new GridData();
    gridDataGroup.horizontalAlignment = GridData.FILL;
    gridDataGroup.grabExcessHorizontalSpace = true;

    Group groupObjekte = new Group(_parent, SWT.NONE);
    groupObjekte.setText(_kennung);
    groupObjekte.setLayout(new GridLayout(1, false));
    groupObjekte.setLayoutData(gridDataGroup);
    
    GridData gridDataComboBox = new GridData();
    gridDataComboBox.horizontalAlignment = GridData.FILL;
    gridDataComboBox.grabExcessHorizontalSpace = true;
    gridDataComboBox.verticalAlignment = GridData.CENTER;
    
    _combo = new Combo( groupObjekte, SWT.BORDER );
    _combo.setLayoutData( gridDataComboBox );
  }

  /**
   * Methode definiert die mglichen Objekte
   * @param moeglicheObjekte mgliche Objekte
   */
  public void setMoeglicheObjekte (List<String> moeglicheObjekte)
  {
	  String[] t = new String [moeglicheObjekte.size()];
	  for (int i=0; i<moeglicheObjekte.size(); ++i)
	  	t[i] = moeglicheObjekte.get(i);
	  	
	  _combo.setItems(t);
	  _combo.select(0);
  }

  /**
   * Methode setzt das angewhlte Objekt
   * @param moeglicheObjekte Liste mit den mglichen objekte
   * @param angewaehltesObjekt angewhltes Objekt
   */
  public void setAngewaehltesObjekt (List<String> moeglicheObjekte, String angewaehltesObjekt)
  {
  	setMoeglicheObjekte(moeglicheObjekte);
  	
    for (int k = 0; k < _combo.getItemCount(); k++)
    {
      if (angewaehltesObjekt.equals(_combo.getItem( k )))
        _combo.select(k);      
    }
  }

  /**
   * Methode liefert das angewhlter Objekt
   * @return angewhltes Objekt
   */
  public String getAusgewaehltesObjekt ()
  {
    return _combo.getItem( _combo.getSelectionIndex() );
  }
}
