/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.umfelddaten;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import de.bsvrz.buv.plugin.ursacheneinheit.tools.AuswahlObjekte;
import de.bsvrz.buv.plugin.ursacheneinheit.tools.Tools;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;

/**
 * Darstellung des Fensters zur Parameternderung mit den Umfelddaten
 * @author ObertM
 */
public class CompositeParameterGesamt extends Composite
{
  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe Umfelddatenmesstellen
   */
  private ResultData _resultDataUFD;

  /**
   * Liste mit den mglichen Umfelddatenmessstellen
   */
  private List<String> _listeMoeglicherUFD;

  /**
   * Hauptcomposite
   */
  private Composite _composite;

	/**
	 * Merker ob die Versorgung der Messquerschnitte angeboten wird oder nicht
	 */
	private boolean _auswahlMQ;

	/**
	 * Instanz zur Auswahl der Umfelddatenmessstellen
	 */
	private AuswahlObjekte _auswahlObjekteUFD;

	/**
	 * Konstruktor der Klasse
	 * @param parent Aufrufendes Composite
	 * @param style Stil
	 * @param resultDataUFD Ergebnis Umfelddatenmesstellen 
	 * @param listeMoeglicherUFD Liste mit den mglichen Umfelddatenmessstellen
	 * @param auswahlMQ Merker ob die Versorgung der Messquerschnitte angeboten wird oder nicht
	 */
  CompositeParameterGesamt(Composite parent, int style, ResultData resultDataUFD, List<String> listeMoeglicherUFD, boolean auswahlMQ)
  {
    super(parent, style);
    
    this._resultDataUFD = resultDataUFD;
    
    this._auswahlMQ          = auswahlMQ;
    
    this._listeMoeglicherUFD = listeMoeglicherUFD;
    
    initialize();
  }
  
  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

    if (_auswahlMQ)
    	_auswahlObjekteUFD = new AuswahlObjekte(_composite, "Umfelddatenmessstellen");
    
    updateData ();
  }

  /**
   * Updatefunktion zur bernahme von neuen Daten aus dem DaV
	 * @param resultDataMq Ergebnis Messquerschnitte
	 * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
   * @param resultData 
   */
  public synchronized void update (ResultData resultDataUFD)
  {
    this._resultDataUFD = resultDataUFD;

    updateData ();
  }
  
  /**
   * Updatefunktion zum Aktualisieren der Daten
   */
  private void updateData ()
  {
    if (_resultDataUFD != null)
    {
    	if (_auswahlMQ)
        updateDataUFD();
    }
  }
  
  /**
   * Updatefunktion ber die Umfelddaten
   */
	private void updateDataUFD()
  {
		List<String> angewaehlteObjekte = new ArrayList<String>();
		
//		String tagNachtSensor = Tools._undefiniert;
		
    if ( _resultDataUFD != null)
    {
      if ( _resultDataUFD.hasData() )
      {
      	Data data = _resultDataUFD.getData();
      	
        Array arrayUFD = data.getArray( "UmfeldDatenMessStelle" );
        for (int k = 0; k < arrayUFD.getLength(); k++)
        {
          String text = "";
          ReferenceValue rV = arrayUFD.getReferenceValue( k );
          if ( rV != null )
            text = Tools.getName(rV.getSystemObject());
          
          angewaehlteObjekte.add(text);
        }
        
//        SystemObject so = data.getReferenceValue("UmfeldDatenSensorTagNacht").getSystemObject();
//        if (so != null)
//        	tagNachtSensor = Tools.getName(so);
      }
    }
    
    _auswahlObjekteUFD.setAngewaehlteObjekte(_listeMoeglicherUFD, angewaehlteObjekte);
    
//    _auswahlTNS.setAngewaehltesObjekt(_listeMoeglicherTNS, tagNachtSensor);
  }

  /**
   * Methode liefert die Liste der angewhlten Umfelddatenmessstellen
   * @return Liste der angewhlten Messquerschnitte
   */
  public org.eclipse.swt.widgets.List getZugeordneteUFD()
  {
    return _auswahlObjekteUFD.getZugeordneteObjekte();
  }

  /**
   * Dispose
   */
  @Override
  public void dispose ()
  {
  	super.dispose();
  }

}
