/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.umfelddaten;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.ursacheneinheit.Activator;
import de.bsvrz.buv.plugin.ursacheneinheit.tools.Tools;
import de.bsvrz.buv.plugin.ursacheneinheit.tools.UebersichtAngewaehlteUrsachenEinheiten;
import de.bsvrz.buv.plugin.ursacheneinheit.verwaltung.VersorgungsArt;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.koelnloevenich.dif.basislib.urlasserdialog.UrlasserDialog;

/**
 * View zum Versorgen der Umfelddaten
 * @author ObertM
 */
public class ViewParameterUmfeldDaten extends ViewPart
{
	/**
	 * eindeutige ID der View
	 */
  public static final String ID = "de.bsvrz.buv.plugin.ursacheneinheit.parameter.umfelddaten"; 

  /**
   * Schalter fr Debug Ausgaben
   */
  private static final boolean _isDebug = false;

  /**
   * Attributgruppe des DAV, die bercksichtigt wird
   */
  public static final String _pidAtgUFD = "atg.sswSbaUeUfdmsZuordnung";

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();
  
  /**
   * Aufrufendes Compoiste
   */
  private Composite _parent;
  
  /**
   * Compoiste fr die Reiter
   */
  private CTabFolder _tabFolder = null;

  /**
   * Hauptcomposite
   */
  private Composite _composite = null;

  /**
   * Composite mit den Buttons
   */
  private Composite _compositeBottom;

  /**
   * Button "OK" 
   */
  private Button _buttonOk;

  /**
   * Button "bernehmen"
   */
  private Button _buttonUebernehemen;

  /**
   * Button "Abbrechen"
   */
  private Button _buttonAbbrechen;

  /**
   * Button Hilfe
   */
  private Button _buttonHilfe;

  /**
   * Button "Fr alle bernehmen"
   */
  private Button _buttonAlle;

  /**
   * Memento
   */
  private IMemento _memento = null;

  /**
   * Second View ID
   */
  private String _secondViewId;

  /**
   * Verbindung zum DaV
   */
  private ClientDavInterface _connection;

  /**
   * Ergebnisdaten des DaV fr die Umfelddatenmessstellen in einer Hashmap. Als Key dient die Pid
   * des Systemobjekts, als Value das Ergebnis aus dem DaV
   */
  private HashMap<String,ResultData> _resultDataUFD = new HashMap<String, ResultData>();

  /**
   * Liste mit den mglichen Umfeldddatenmessstellen
   */
  private List<SystemObject> _listeMoeglicherUFD = null;

  /**
   * Merker ob die Versorgung der Zuordnung der Messquerschnitte angeboten wird
   */
	private boolean _auswahlMQ;

	/**
	 * Art der Datenversorgung
	 */
	private VersorgungsArt _art;

	/**
	 * bersicht angewhlter Ursacheneinheiten
	 */
	private UebersichtAngewaehlteUrsachenEinheiten _uebersicht;

  /**
   * Liste mit den zu versorgenden Ursacheneinheiten;
   */
  List<SystemObject> _ursachenEinheiten = new ArrayList<SystemObject>();


  @Override
  public void createPartControl( Composite parent )
  {
    if ( _isDebug )
      System.out.println( "--> ViewParameter: createPartControl" );

    _parent = parent;

   	_auswahlMQ = true; 
    
    //Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp( _parent, Activator.PLUGIN_ID + ".parameterUmfeldDaten" );

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;
    _composite = new Composite( parent, SWT.NONE );
    _composite.setLayout( gridLayout );
    _composite.setLayoutData( gridData );

    /*
     * DaV Verbindung holen
     */
    _connection = DaVKommunikation.getInstanz().holeDaVVerbindung();

    /*
     * SeconviewID auswerten
     */

    List<SystemObject> list = new ArrayList<SystemObject>();
    
    _art = Tools.auswerteSeconViewID(_secondViewId, _ursachenEinheiten, list);

    /* 
     * Darstellung aufbauen 
     */
    
    createCompositeTab();
    createCompositeBottom();

    restoreState();

    /*
     *  Versorgte ursacheneinheiten einblenden
     */
    
    if (_art == VersorgungsArt.EinReiterFuerAlleUrsachenEinheiten)
    {
    	List<String> l = new ArrayList<String>();
    	for (SystemObject so : _ursachenEinheiten)
    		l.add(so.getName());
    	
    	if (_uebersicht != null)
    	  _uebersicht.setMoeglicheObjekte(l);
    }

    /*
     *  Reiter erstellen
     */

    erzeugeTabItems( list );
    
    leseDatenVonDav(list);

    /*
     *  Button "Alle versorgen" setzen
     */
    
    _buttonAlle.setEnabled((_ursachenEinheiten.size() > 1) && (_art == VersorgungsArt.EinReiterProUrsachenEinheit));
  }

  @Override
  public void setFocus()
  {
    //Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();

  }

  /**
   * Methode liest die bentigten Parameter direkt vom DaV
   * @param liste Liste mit Systemobjekten die versorgt werden sollen
   */
  private void leseDatenVonDav (List<SystemObject> liste)
  {
  	for (int i=0; i<liste.size(); ++i)
  	{
  		String pidObj = liste.get(i).getPid();
  		
  		ResultData result = null;
  		
  		/*
  		 * Parameter Messquerschnitte
  		 */
  		
  		if (_auswahlMQ)
  		{
  		  result = DaVKommunikation.getInstanz().leseDatenDirektVonDaV(pidObj, _pidAtgUFD, "asp.parameterSoll");
  		  _resultDataUFD.put (pidObj, result);

  		}

  		verarbeiteDavDaten(pidObj);
		}
  }

  /**
   * This method initializes compositeTab (Bereich ber der ButtonLeiste)
   * 
   */
  private void createCompositeTab()
  {
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;

    if (_art == VersorgungsArt.EinReiterFuerAlleUrsachenEinheiten)
      _uebersicht = new UebersichtAngewaehlteUrsachenEinheiten(_composite, "Ursacheneinheiten");

    _tabFolder = new CTabFolder(_composite, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridData3);
    _tabFolder.addSelectionListener(new SelectionListener() {

      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      @Override
      public void widgetSelected(SelectionEvent e)
      {
      }

    });
  }

  /**
   * Reiter fr die Liste der Objekte erstellen
   * 
   * @param list
   */
  public void erzeugeTabItems( List<SystemObject> list )
  {
    for (SystemObject so : list)
    {
      CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE );

      String tabName = Tools.getName(so);
      tabItem.setText( tabName );
      tabItem.setData( tabName, so.getPid() );

      GridData gridData2 = new GridData();
      gridData2.horizontalAlignment = GridData.FILL;
      gridData2.grabExcessHorizontalSpace = true;
      gridData2.grabExcessVerticalSpace = true;
      gridData2.verticalAlignment = GridData.FILL;

      Composite c1 = new Composite( _tabFolder, SWT.NONE );
      c1.setLayout( new GridLayout() );
      c1.setLayoutData( gridData2 );

      tabItem.setControl( c1 );

    }
    _tabFolder.setSelection( 0 );
  }

  /**
   * Reiter fr Untergruppen des SystemObjekts erstellen
   * 
   * @param so
   *          SystemObjekt
   * @param ug
   *          Name der Untergruppe
   * @param pos
   *          Position des Reiters
   */
  public CTabItem erzeugeTabItems( SystemObject so, String ug, int pos )
  {
    CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE, pos );

    String tabName = Tools.getName(so);
    tabItem.setText( tabName );
    tabItem.setData( tabName, so.getPid() );

    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;

    Composite c1 = new Composite( _tabFolder, SWT.NONE );
    c1.setLayout( new GridLayout() );
    c1.setLayoutData( gridData2 );

    tabItem.setControl( c1 );

    return tabItem;
  }

  /**
   * This method initializes compositeBottom (ButtonLeiste)
   * 
   */
  private void createCompositeBottom()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    _compositeBottom = new Composite( _composite, SWT.NONE );
    _compositeBottom.setLayoutData( gridData3 );
    _compositeBottom.setLayout( rowLayout );

    _buttonAlle = new Button( _compositeBottom, SWT.NONE );
    _buttonAlle.setText( "Alle setzen" );
    _buttonAlle.setToolTipText( "Einstellungen bei allen ausgewhlten gleichartigen AQs setzen" );
    _buttonAlle.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
     }
    } );
    _buttonAlle.setEnabled(!_auswahlMQ);

    _buttonOk = new Button( _compositeBottom, SWT.NONE );
    _buttonOk.setText( "Ok" );
    _buttonOk.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionSchalten();
      }
    } );

    _buttonUebernehemen = new Button( _compositeBottom, SWT.NONE );
    _buttonUebernehemen.setText( "bernehmen" );
    _buttonUebernehemen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionUebernehmen();
      }
    } );

    _buttonAbbrechen = new Button( _compositeBottom, SWT.NONE );
    _buttonAbbrechen.setText( "Abbrechen" );
    _buttonAbbrechen.addMouseListener( new MouseAdapter()
    {

      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        dispose();
      }

    } );
    _buttonHilfe = new Button( _compositeBottom, SWT.NONE );
    _buttonHilfe.setText( "Hilfe" );
    _buttonHilfe.addMouseListener( new MouseAdapter()
    {

      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionHilfe();
      }

    } );
  }

  protected void aktionHilfe()
  {
	String contextId = Activator.PLUGIN_ID + ".parameterUmfeldDaten";  
    _debug.fine("ContextID: " + contextId);
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  protected void aktionSchalten()
  {
    if ( schaltenUndParametrieren() )
    {
      dispose();
    }
    else
    {
      MessageBox mb = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
      mb.setText( "Fehler Urlasser" );
      mb.setMessage( "Objekt konnte nicht parametriert werden: " + "" );
      mb.open();
      return;
    }

  }

  protected void aktionUebernehmen()
  {
    if ( schaltenUndParametrieren() )
    {
      // keine weiteren Aktionen
    }
    else
    {
      MessageBox mb = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
      mb.setText( "Fehler Urlasser" );
      mb.setMessage( "Objekt konnte nicht parametriert werden: " + "" );
      mb.open();
      return;
    }

  }

  @Override
  public void init( IViewSite site, IMemento memento ) throws PartInitException
  {
    _memento = memento;
    _secondViewId = site.getSecondaryId();
    super.init( site, memento );
  }

  private void restoreState()
  {

    if ( _memento == null )
    {
      return;
    }

    _secondViewId = _memento.getString( "pidMemento" );

  }

  @Override
  public void saveState( IMemento memento )
  {
    memento.putString( "pidMemento", _secondViewId );
    super.saveState( memento );
  }
  
  /**
   * Schalten und Parametrieren im DAV durchfhren
   * 
   * @return
   */
  private Boolean schaltenUndParametrieren()
  {
    List<ResultData> list = new ArrayList<ResultData>();
    
    if (_auswahlMQ)
    {
  	  list.addAll(erzeugeResultDataUFD());
    }

    UrlasserDialog urlasserDialog = new UrlasserDialog( Display.getCurrent().getActiveShell() );
    Boolean schlieenOk = (urlasserDialog.anzeigen( list )).getEingabeOk();

    return schlieenOk;
  }

  /**
   * Methode erzeugt die Resultdaten fr den DaV fr alle Reiter fr die Umfelddatenmessstellen
   * @return Resultdaten fr den DaV
   */
  private List<ResultData> erzeugeResultDataUFD ()
  {
    List<ResultData> list = new ArrayList<ResultData>();
    
    DataModel dataModel = _connection.getDataModel();

    Aspect asp = dataModel.getAspect( "asp.parameterVorgabe" );
    AttributeGroup atg = dataModel.getAttributeGroup( _pidAtgUFD );
    
    CTabItem[] items = _tabFolder.getItems();

    /*
     * Achtung: nur ein Reiter in Verwendung
     */
    for (CTabItem ti : items)
    {
      Data data1 = _connection.createData( atg );
      Array array1 = data1.getItem( "UmfeldDatenMessStelle" ).asArray();
      
      Composite c = (Composite) ti.getControl();
      CompositeParameterGesamt compPar = (CompositeParameterGesamt) c.getChildren()[0];
      
      int anzahl = compPar.getZugeordneteUFD().getItems().length;
      array1.setLength( anzahl );
      
      /*
       * Angewhlte UFDs auslesen
       */
      for (int i=0; i<anzahl; ++i)
      {
      	String nameUFD = compPar.getZugeordneteUFD().getItem(i);
        for (SystemObject soUFD : _listeMoeglicherUFD)
        {
          if ( Tools.getName(soUFD).equals( nameUFD  ) )
          {
            array1.getReferenceValue( i ).setSystemObject( soUFD );
            break;
          }
        }
      }
      
      for (SystemObject so : _ursachenEinheiten)
      {
      	/*
      	 * Da der Tag/Nacht Sensor nicht ber diesen Dialog versorgbar ist, aber ebenfalls in dieser
      	 * Attributgruppe gespeichert wird, wird vor dem Speichern der neuen Daten der bisherige Wert
      	 * fr den Tag/Nacht Sensor ausgelesen
      	 */
        
      	ResultData bisher = DaVKommunikation.getInstanz().leseDatenDirektVonDaV(so.getPid(), atg.getPid(), "asp.parameterSoll");
      	SystemObject soTNS = bestimmeTagNachtSensor(bisher);
      	
      	data1.getReferenceValue("UmfeldDatenSensorTagNacht").setSystemObject(soTNS);
      	
        ResultData rd1 = new ResultData( so, new DataDescription( atg, asp ), new Date().getTime(), data1 );
        list.add( rd1 );
      }
    }

    return list;
  }

  /**
   * Updatefunktion ber die Umfelddaten
   */
	private SystemObject bestimmeTagNachtSensor (ResultData _resultDataUFD)
  {
    SystemObject so = null;
    
    if ( _resultDataUFD != null)
    {
      if ( _resultDataUFD.hasData() )
      {
      	Data data = _resultDataUFD.getData();
        so = data.getReferenceValue("UmfeldDatenSensorTagNacht").getSystemObject();
      }
    }

    return so;
  }

  /**
   * Methode zum Verarbeiten der Daten des DaV
   * @param pidObj Pid des Systemobjekts
   */
  private void verarbeiteDavDaten (String pidObj)
  {
    if ( _isDebug )
      System.out.println( "--> ViewParameter: VerarbeiteDavDaten" );
    
    /*
     *  einmalig alle bentigten Objekte ermitteln
     */
    
    if ( _listeMoeglicherUFD == null )
    	_listeMoeglicherUFD = Tools.getSystemObjekte("typ.sswSbaUmfeldDatenMessStelle");

    /*
     *  alle Reiter zur Pid ermitteln
     */
    CTabItem[] items = _tabFolder.getItems();
    List<CTabItem> itemPidList = new ArrayList<CTabItem>();
    for (CTabItem ti : items)
    {
      String pid = (String) ti.getData( ti.getText() );

      if ( pid == pidObj )
      {
        itemPidList.add( ti );
      }
    }

    if ( itemPidList != null )
    {
      CTabItem item = null;
  
      for (CTabItem ti : itemPidList)
      {
        item = ti;

        Composite c = (Composite) item.getControl();
        Control[] children = c.getChildren();
        CompositeParameterGesamt compPar = null;

        if ( children.length == 0 )
        {
          compPar = new CompositeParameterGesamt( c, SWT.None, 
          		_resultDataUFD.get(pidObj), 
          		Tools.bestimmeObjektNamen(_listeMoeglicherUFD), 
          		_auswahlMQ);
          compPar.layout( true );
          c.layout( true );

        }
        else
        {
          compPar = (CompositeParameterGesamt) children[0];
          compPar.update( _resultDataUFD.get(pidObj) );
          
          compPar.layout( true );
          c.layout( true );
        }
      }
    }
  }

  @Override
  public void dispose()
  {
    disposeTabItems();
    
    super.dispose();
    
    IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
    if ( page != null )
    {
      page.hideView( this );
    }
  }
  
  private void disposeTabItems ()
  {
  	try
  	{
	    CTabItem[] items = _tabFolder.getItems();
	  	
	    for (CTabItem item : items)
	    {
	    	if (item.isDisposed())
	    		continue;
	    	
	      Composite c = (Composite) item.getControl();
	      if (c.isDisposed())
	      	continue;
	      
	      Control[] children = c.getChildren();
	
	      if ( children.length != 0 )
	      {
	        CompositeParameterGesamt compPar = (CompositeParameterGesamt) children[0];
	        if (!compPar.isDisposed())
	          compPar.dispose();
	      }
	    }
  	}
  	catch (Exception e) 
  	{
  		System.err.println("=== abgefangen ===");
			e.printStackTrace();
		}
  }
}
