/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.verwaltung;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bsvrz.buv.plugin.ursacheneinheit.tools.SimpleSenderMitWartenAufParametrierung;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.losb.exceptions.SenderException;

/**
 * Klasse bietet Methoden zum Kopieren der Parameter der Ursacheneinheiten
 * @author ObertM
 */
public class UrsachenEinheitenKopieren
{
	/**
	 * Originalursacheneinheit
	 */
  private EintragUrsachenEinheit _original;
  
  /**
   * Kopie der Ursacheneinheit
   */
  private EintragUrsachenEinheit _kopie;

	private ClientDavInterface _connection;
  
	public void kopiereParameter(EintragUrsachenEinheit original,
                               EintragUrsachenEinheit kopie)
  {
		_connection = DaVKommunikation.getInstanz().getConnection();
		
		_original = original;
		
		_kopie = kopie;
		
		if (!original.getObjTyp().getPid().equals(kopie.getObjTyp().getPid()))
		{
			System.err.println(original.getName() + " kann nicht nach " + kopie.getName() + " kopiert werden !");
			return;
		}
		
		/*
		 * Kopierfunktionen fr die unterschiedlichen Typen der Ursacheneinheiten
		 */

		List<String> pidAtg = null;
		
		pidAtg = bestimmeParametrierendeAttributGruppen (original.getObjTyp().getPid());
		

//		if (original.getObjTyp().getPid().equals("typ.sswSbaUeHarmoBelastungUnruhe120"))
//		{
//			pidAtg = new String []
//	 	  {
//				"atg.sswSbaUrsachenEinheit",
//	 			"atg.sswSbaUeMqZuordnung", 
//	 			"atg.sswSbaUeHarmoBelastungUnruhe120"
//	 	  };
//		}
//		else if (original.getObjTyp().getPid().equals("typ.sswSbaUeHarmoBelastung100") ||
//				     original.getObjTyp().getPid().equals("typ.sswSbaUeHarmoBelastung80")  ||
//				     original.getObjTyp().getPid().equals("typ.sswSbaUeHarmoBelastung60"))
//		{
//			pidAtg = new String []
//	 	  {
//				"atg.sswSbaUrsachenEinheit",
//	 			"atg.sswSbaUeMqZuordnung", 
//	 			"atg.sswSbaUeHarmoBelastung"
//	 	  };
//		}
//		else if (original.getObjTyp().getPid().equals("typ.sswSbaUeHarmoGeschwDiff120") ||
//				     original.getObjTyp().getPid().equals("typ.sswSbaUeHarmoGeschwDiff100") ||
//				     original.getObjTyp().getPid().equals("typ.sswSbaUeHarmoGeschwDiff80"))
//		{
//			pidAtg = new String []
//		  {
//				"atg.sswSbaUrsachenEinheit",
//				"atg.sswSbaUeMqZuordnung", 
//				"atg.sswSbaUeHarmoGeschwDiff"
//		  };
//		}
//		else if (original.getObjTyp().getPid().equals("typ.sswSbaUeStau"))
//		{
//			pidAtg = new String []
// 		  {
//				"atg.sswSbaUrsachenEinheit",
//		  	"atg.sswSbaUeMqZuordnung",
//		  	"atg.sswSbaUeStauBeleg",
//		  	"atg.sswSbaUeStauGeschw",
//		  	"atg.sswSbaUeStauStrfall",
//		  	"atg.sswSbaUeStaukriterium",
//		 };
//   }
//	 else if (original.getObjTyp().getPid().equals("typ.sswSbaUeErweiterterStaualgorithmus"))
//	 {
//		  pidAtg = new String []
// 		  {
//				"atg.sswSbaUrsachenEinheit",
//		  	"atg.sswSbaUeErweiterterStaualgorithmus",
//		 };
//   }
//	 else if (original.getObjTyp().getPid().equals("typ.sswSbaUeLkwberholverbotBelastung"))
//	 {
//		  pidAtg = new String []
// 		  {
//				"atg.sswSbaUrsachenEinheit",
//		  	"atg.sswSbaUeMqZuordnung",
//		  	"atg.sswSbaUeLkwberholverbotBelastung",
//		 };
//   }
//	 else if (original.getObjTyp().getPid().equals("typ.sswSbaUeLkwberholverbotNsse"))
//	 {
//		  pidAtg = new String []
// 		  {
//				"atg.sswSbaUrsachenEinheit",
//		  	"atg.sswSbaUeMqZuordnung",
//		  	"atg.sswSbaUeUfdmsZuordnung",
//		  	"atg.sswSbaUeLkwberholverbotNsse",
//		 };
//   }
//	 else if (original.getObjTyp().getPid().equals("typ.sswSbaUeLkwberholverbotSicht"))
//	 {
//		  pidAtg = new String []
// 		  {
//				"atg.sswSbaUrsachenEinheit",
//		  	"atg.sswSbaUeMqZuordnung",
//		  	"atg.sswSbaUeUfdsZuordnung",
//		  	"atg.sswSbaUeLkwberholverbotSicht",
//		 };
//   }
//	 else if (original.getObjTyp().getPid().equals("typ.sswSbaUeNssewarnung"))
//	 {
//		  pidAtg = new String []
// 		  {
//				"atg.sswSbaUrsachenEinheit",
//		  	"atg.sswSbaUeMqZuordnung",
//		  	"atg.sswSbaUeUfdmsZuordnung",
//		  	"atg.sswSbaUeNssewarnung",
//		 };
//   }
//	 else if (original.getObjTyp().getPid().equals("typ.sswSbaUeGlttewarnung"))
//	 {
//		  pidAtg = new String []
// 		  {
//				"atg.sswSbaUrsachenEinheit",
//		  	"atg.sswSbaUeGlttewarnung",
//		 };
//   }
//	 else if (original.getObjTyp().getPid().equals("typ.sswSbaUeNebelwarnung"))
//	 {
//		  pidAtg = new String []
// 		  {
//				"atg.sswSbaUrsachenEinheit",
//		  	"atg.sswSbaUeUfdmsZuordnung",
//		  	"atg.sswSbaUeNebelwarnung",
//		 };
//   }
//	 else if (original.getObjTyp().getPid().equals("typ.sswSbaUeSeitenwind"))
//	 {
//		  pidAtg = new String []
// 		  {
//				"atg.sswSbaUrsachenEinheit",
//		  	"atg.sswSbaUeSeitenwind",
//		 };
//   }
//	 else if (original.getObjTyp().getPid().equals("typ.sswSbaUeLangsameFahrzeuge"))
//	 {
//		  pidAtg = new String []
// 		  {
//				"atg.sswSbaUrsachenEinheit",
//		  	"atg.sswSbaUeMqZuordnung",
//		  	"atg.sswSbaUeLangsameFahrzeuge",
//		 };
//   }
//	 else if (original.getObjTyp().getPid().equals("typ.sswSbaUeAbstandswarnung1"))
//	 {
//		  pidAtg = new String []
// 		  {
//				"atg.sswSbaUrsachenEinheit",
//		  	"atg.sswSbaUeMqZuordnung",
//		  	"atg.sswSbaUeAbstandswarnung1",
//		 };
//   }
//	 else if (original.getObjTyp().getPid().equals("typ.sswSbaUeAbstandswarnung2"))
//	 {
//		  pidAtg = new String []
// 		  {
//				"atg.sswSbaUrsachenEinheit",
//		  	"atg.sswSbaUeMqZuordnung",
//		  	"atg.sswSbaUeAbstandswarnung2",
//		 };
//   }
//	 else if (original.getObjTyp().getPid().equals("typ.sswSbaUeTemposplit"))
//	 {
//		  pidAtg = new String []
// 		  {
//				"atg.sswSbaUrsachenEinheit",
//		  	"atg.sswSbaUeTemposplit",
//		 };
//   }
//	 else if (original.getObjTyp().getPid().equals("typ.sswSbaUeFahrstreifenZuweisung"))
//	 {
//		  pidAtg = new String []
// 		  {
//				"atg.sswSbaUrsachenEinheit",
//		  	"atg.sswSbaUeFahrstreifenZuweisung",
//		 };
//   }
//	 else if (original.getObjTyp().getPid().equals("typ.sswSbaUeExterneTrigger"))
//	 {
//		  pidAtg = new String []
// 		  {
//				"atg.sswSbaUrsachenEinheit",
//		  	"atg.sswSbaUeExterneTrigger",
//		 };
//   }
		
		if (pidAtg != null)
			kopiere (pidAtg);
  }

	/**
	 * Methode bestimmt die zu kopierenden Attributgruppen. Dabei werden aus dem Datenmodell alle
	 * parametrierenden Attributgruppen des Typs der Ursacheneinheit bestimmt.
	 * @param pidTypUrsachenEinheit Pid des Typs der Ursacheneinheit
	 * @return Feld mit den Pids der zu kopierenden Attributgruppen
	 */
	private List<String> bestimmeParametrierendeAttributGruppen(String pidTypUrsachenEinheit)
  {
		ClientDavInterface connection = DaVKommunikation.getInstanz().getConnection();

		List<String> pids = new ArrayList<String>();
		
		List<AttributeGroup> attributeGroups = connection.getDataModel().getType(pidTypUrsachenEinheit).getAttributeGroups();
		for (AttributeGroup atg : attributeGroups)
		{
			Collection<Aspect> aspekte = atg.getAspects();
			for (Aspect a : aspekte)
				if (a.getPid().equals("asp.parameterVorgabe"))
					pids.add(atg.getPid());
		}
		
		return pids;
  }

	/**
	 * Methode zum Kopieren der Attributgruppen
	 * @param pidAtg Pid der zu kopierenden Attributgruppen
	 */
	private void kopiere(List<String> pidAtg)
  {
		for (String pid : pidAtg)
		{
    	System.out.println("Kopieren von " + pid + _original.getName() + " " + _original.getPid());
    	
	    ResultData result = DaVKommunikation.getInstanz().leseDatenDirektVonDaV (_original.getPid(), pid, "asp.parameterSoll");
	    if (result.hasData())
	    {
	    	Data data = result.getData();
	    	
	      SystemObject sysObj = _connection.getDataModel().getObject(_kopie.getPid());

	      Aspect asp = _connection.getDataModel().getAspect ("asp.parameterVorgabe");
				AttributeGroup atg = _connection.getDataModel().getAttributeGroup(pid);
				DataDescription dd = new DataDescription(atg, asp);

				try
        {
	        SimpleSenderMitWartenAufParametrierung.sendWait(_connection, sysObj, dd, data, 60000);
        } 
				catch (SenderException e)
        {
	        e.printStackTrace();
        }
	    	
	    }
	    else
	    	System.err.println("Keine Daten fr " + pid);
		}
  }
}
