package de.bsvrz.sba.plugin.actions.parameter.verwaltung;

import java.util.LinkedList;

import de.bsvrz.sba.plugin.actions.parameter.IParameterAction;

/**
 * Java-Darstellung des Erweiterungspunktes
 * <code>de.bsvrz.sba.plugin.action.parameterAction</code>.
 * 
 * @author HofmannJ
 * 
 */
public class ParameterActionRegistryElement
{
	/**
	 * Speichert die {@link IParameterAction}
	 */
	private IParameterAction _parameterAction = null;

	/**
	 * Speichert die ID der User, welchem it der {@link IParameterAction} arbeiten
	 * knnen.
	 */
	private LinkedList<String> _userIds = null;

	/**
	 * Speichert den Contributor des Erweiterungspunktes.
	 */
	private String _contributor = null;

	/**
	 * Konstruktor
	 * 
	 * @param parameterAction
	 *          {@link IParameterAction}
	 * @param userIds
	 *          IDs der User, welche dieser {@link IParameterAction} verwenden
	 *          knnen.
	 * @param contributor
	 *          Contributor des Erweiterungspunktes.
	 */
	public ParameterActionRegistryElement(IParameterAction parameterAction,
			LinkedList<String> userIds, String contributor)
	{
		this._parameterAction = parameterAction;
		this._userIds = userIds;
		this._contributor = contributor;
	}

	/**
	 * Parameterloser Konstruktor
	 */
	public ParameterActionRegistryElement()
	{
	}

	/**
	 * Setzt die {@link IParameterAction}.
	 * 
	 * @param parameterAction
	 *          Zu setzende {@link IParameterAction}
	 */
	protected void setParameterAction(IParameterAction parameterAction)
	{
		this._parameterAction = parameterAction;
	}

	/**
	 * Setzt die Liste der UserIds.
	 * 
	 * @param userIds
	 *          Zu setzende Liste der UserIds.
	 */
	protected void setUserIds(LinkedList<String> userIds)
	{
		this._userIds = userIds;
	}

	/**
	 * Setzt den Contributor des Erweiterungspunktes.
	 * 
	 * @param contributor
	 *          Contributor des Erweiterungspunktes
	 */
	protected void setContributor(String contributor)
	{
		this._contributor = contributor;
	}

	/**
	 * Liefert die {@link IParameterAction}.
	 * 
	 * @return {@link IParameterAction}
	 */
	public IParameterAction getParameterAction()
	{
		return _parameterAction;
	}

	/**
	 * Liefert die Liste der UserIds, welche mit dieser {@link IParameterAction}
	 * arbeiten sollen.
	 * 
	 * @return Liste der UserIds
	 */
	public LinkedList<String> getUserIds()
	{
		return _userIds;
	}

	/**
	 * Liefert den Contributor des Erweiterungspunktes.
	 * 
	 * @return Contributor des Erweiterungspunktes
	 */
	public String getContributor()
	{
		return _contributor;
	}

}
