/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.dynobj.configurationAreas.dialogs;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.rw.dav.datenbasis.GtmDatenBasis;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.sba.plugin.dynobj.Activator;
import de.bsvrz.sba.plugin.dynobj.configurationAreas.ConfigurationAreaModelRecord;
import de.bsvrz.sba.plugin.dynobj.configurationAreas.compsites.CompositeAssociationObjectTypeConfigurationArea;
import de.bsvrz.sba.plugin.sonstiges.shell.ShellPosition;

/**
 * Mit Hilfe dieses {@link Dialog}s kann eine Zuordnung zwischen Objekttyp und
 * Konfigurationsbereich festgelegt werden.
 * 
 * @author HofmannJ
 * 
 */
public class DialogAssociationObjTypConfAreaEditor extends Dialog
{
	/**
	 * Verbindung zum DaV
	 */
	private ClientDavInterface _connection = null;

	/**
	 * Datenbasis des DaV
	 */
	private GtmDatenBasis _gtmDatenBasis = null;

	/**
	 * Shell zur Darstellung es Dialogs
	 */
	private Shell _shell = null;

	/**
	 * bergebene Zuordnung von Objekttyp zu Konfigurationsbereich
	 */
	private ConfigurationAreaModelRecord _record = null;

	/**
	 * Mit Hilfe des Dialogs getroffene Zuordnung von Objekttyp zu
	 * Konfigurationsbereich.
	 */
	private ConfigurationAreaModelRecord _result = null;

	/**
	 * Composite zur Darstellung der Zuordnung von Objekttyp/Konfigurationsbereich
	 */
	private CompositeAssociationObjectTypeConfigurationArea _compAssObjTypeConfArea;

	/**
	 * Konstruktor
	 * 
	 * @param parent
	 *          Eltern-Shell, auf deren Basis die Darstellung erfolgen soll.
	 * @param style
	 *          SWT-Stil
	 * @param connection
	 *          Verbindung zum DaV
	 * @param gtmDatenBasis
	 *          Datenbasis des DaV
	 * @param record
	 *          Eine zu bearbeitende Zuordnung von
	 *          Objekttyp/Konfigurationsbereich. Wenn <code>null</code>, dann
	 *          wird eine leere Zuordnung erzeugt.
	 */
	public DialogAssociationObjTypConfAreaEditor(Shell parent, int style,
			ClientDavInterface connection, GtmDatenBasis gtmDatenBasis,
			ConfigurationAreaModelRecord record)
	{
		super(parent, style);
		this._connection = connection;
		this._gtmDatenBasis = gtmDatenBasis;
		this._record = record;
	}

	/**
	 * Konstruktor
	 * 
	 * @param parent
	 *          Eltern-Shell, auf deren Basis die Darstellung erfolgen soll.
	 * @param connection
	 *          Verbindung zum DaV
	 * @param gtmDatenBasis
	 *          Datenbasis des DaV
	 * @param record
	 *          Eine zu bearbeitende Zuordnung von
	 *          Objekttyp/Konfigurationsbereich. Wenn <code>null</code>, dann
	 *          wird eine leere Zuordnung erzeugt.
	 */
	public DialogAssociationObjTypConfAreaEditor(Shell parent,
			ClientDavInterface connection, GtmDatenBasis gtmDatenBasis,
			ConfigurationAreaModelRecord record)
	{
		this(parent, 0, connection, gtmDatenBasis, record);
	}

	/**
	 * ffnet den Dialog.
	 * 
	 * @param posAtMouse
	 *          Wenn <code>true</code>, dann wird die linke obere Ecke des
	 *          Dialogs beim Mauszeiger positioniert, ansonsten in der Mitte des
	 *          Workbench-Fensters.
	 * @return Liefert die getroffene Zuordnung als
	 *         {@link ConfigurationAreaModelRecord} zurck oder <code>null</code>,
	 *         wenn keine Zuordnung getroffen wurde.
	 */
	public ConfigurationAreaModelRecord open(boolean posAtMouse)
	{
		Shell parent = getParent();
		_shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL
				| SWT.RESIZE);

		_shell.setText("Zuordnung Objekttyp -> Konfigurationsbereich");
		_shell.setSize(800, 550);

		_shell.setImage(Activator.getImageDescriptor("icons/zuordnung_16x16.png")
				.createImage());

		/*
		 * Positionieren
		 */
		if (posAtMouse)
		{
			ShellPosition.positionShellAtMouseCursor(_shell);
		}
		else
		{
			_shell.setBounds(ShellPosition.getBoundsChildCenteredOnParent(PlatformUI
					.getWorkbench().getActiveWorkbenchWindow().getShell().getBounds(),
					_shell.getBounds()));
		}

		_compAssObjTypeConfArea = new CompositeAssociationObjectTypeConfigurationArea(
				_shell, SWT.NONE, _connection, _gtmDatenBasis);
		Composite compDown = new Composite(_shell, SWT.NONE);

		Button bOk = new Button(compDown, SWT.PUSH);
		bOk.setText("Ok");
		bOk.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				actionOk();
			}
		});

		Button bCancel = new Button(compDown, SWT.PUSH);
		bCancel.setText("Abbrechen");
		bCancel.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				actionCancel();
			}
		});

		Button bHelp = new Button(compDown, SWT.PUSH);
		bHelp.setText("Hilfe");
		bHelp.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				actionHelp();
			}
		});

		/*
		 * Vorselektion
		 */
		if (_record != null)
		{
			// _compAssObjTypeConfArea.setse
			_compAssObjTypeConfArea.setPreselectionRecord(_record);
		}

		/*
		 * Layout
		 */
		GridLayout gl = new GridLayout(1, false);
		_shell.setLayout(gl);

		RowLayout rlDown = new RowLayout();
		rlDown.pack = false;
		compDown.setLayout(rlDown);

		GridData gdUp = new GridData();
		gdUp.grabExcessHorizontalSpace = true;
		gdUp.horizontalAlignment = GridData.FILL;
		gdUp.grabExcessVerticalSpace = true;
		gdUp.verticalAlignment = GridData.FILL;
		_compAssObjTypeConfArea.setLayoutData(gdUp);

		GridData gdDown = new GridData();
		gdDown.grabExcessHorizontalSpace = true;
		gdDown.horizontalAlignment = GridData.END;
		compDown.setLayoutData(gdDown);

		/*
		 * ffnen des Dialogs
		 */
		_shell.open();
		Display display = parent.getDisplay();
		while (!_shell.isDisposed())
		{
			if (!display.readAndDispatch())
				display.sleep();
		}

		return _result;
	}

	/**
	 * Aktion hinter dem OK-Button
	 */
	private void actionOk()
	{
		String objectTypePid = _compAssObjTypeConfArea
				.getFirstSelectedObjectTypePid();
		String confAreaPid = _compAssObjTypeConfArea
				.getFirstSelectedConfigurationAreaPid();

		if ((objectTypePid != null) && (confAreaPid != null))
		{
			_result = new ConfigurationAreaModelRecord(objectTypePid, confAreaPid);
		}

		_shell.dispose();
	}

	/**
	 * Aktion hinter dem Abbrechen-Button
	 */
	private void actionCancel()
	{
		_shell.dispose();
	}

	private void actionHelp()
	{
		String hilfeKontextId = Activator.PLUGIN_ID + ".dynobj";
		PlatformUI.getWorkbench().getHelpSystem().displayHelp(hilfeKontextId);
	}

}
