/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.daten;

/**
 * Klasse definiert ber ein Enum die Richtung in die eine Regelklasse arbeiten soll 
 * @author Dambach Werke GmbH
 * @author Matthias Obert
 */
public class Richtung
{
  /**
   * Arbeitet die Regel in Vorgnger- oder Nachfolgerrichtung .
   * @author Dambach Werke GmbH
   * @author Matthias Obert
   */
  public enum ArbeitsRichtungVN
  {
    /**
     * Regelklasse arbeitet in Nachfolgerrichtung
     */
    NACHFOLGER ("Nachfolger", 0),
    
    /**
     * Regelklasse arbeitet in Vorgngerrichtung
     */
    VORGAENGER ("Vorgnger", 1),
    
    ;
    
    /**
     * Kodierung im DaV
     */
    private int m_kodierung;
    
    /**
     * Bezeichnung
     */
    private String m_name;
    
    private ArbeitsRichtungVN (String name, int kodierung)
    {
    	m_name = name;
    	
      m_kodierung = kodierung;
    }

    /**
     * Methode liefert die Variable m_kodierung zurck
     * @return Variable m_kodierung
     */
    public int getKodierung()
    {
      return m_kodierung;
    }
    
    /**
     * Methode liefert die Richtung zu einer bergebenen Kodierung
     * @param kodierung Kodierung
     * @return Richtung
     */
    public ArbeitsRichtungVN getRichtung (int kodierung)
    {
      for (ArbeitsRichtungVN r : ArbeitsRichtungVN.values())
      {
        if (r.getKodierung() == kodierung)
          return r;
      }
      
      return null;
    }

    /**
     * Methode liefert die Richtung zu einem bestimmten Namen
     * @param nameRichtung Name
     * @return Richtung
     */
		public static ArbeitsRichtungVN getRichtung(String nameRichtung)
    {
			for (ArbeitsRichtungVN r : ArbeitsRichtungVN.values())
				if (r.getName().equals(nameRichtung))
					return r;
			
	    return null;
    }

		/**
     * Methode liefert name zurck
     * @return name
     */
    public String getName()
    {
    	return m_name;
    }

		/**
     * Methode setzt die Variable name
     * @param name zu bernehmender Wert fr name
     */
    public void setName(String name)
    {
    	m_name = name;
    }

		/**
     * Methode setzt die Variable kodierung
     * @param kodierung zu bernehmender Wert fr kodierung
     */
    public void setKodierung(int kodierung)
    {
    	m_kodierung = kodierung;
    }
    
    /**
     * Methode liefert die Namen aller Richtungen
     * @return Liste mit Namen
     */
    public static String [] getNamen ()
    {
      String[] namen = new String [values().length];
      
      int i=0;
      for (ArbeitsRichtungVN avn : values())
      	namen [i++] = avn.getName();
      
      return namen;
    }
  }
  
  /**
   * Arbeitet die Regel in Quer- oder Lngsrichtung .
   * @author Dambach Werke GmbH
   * @author Matthias Obert
   */
  public enum ArbeitsRichtungQL
  {
    /**
     * Regelklasse arbeitet in Querrichtung
     */
    QUERRICHTUNG ("Querrichtung", 0),
    
    /**
     * Regelklasse arbeitet in Vorgngerrichtung
     */
    LAENGSRICHTUNG ("Lngsrichtung", 1),
    
    ;
    
    /**
     * Kodierung im DaV
     */
    private int m_kodierung;
    
    /**
     * Bezeichnung
     */
    private String m_name;
    
    private ArbeitsRichtungQL (String name, int kodierung)
    {
    	m_name = name;
    	
      m_kodierung = kodierung;
    }

    /**
     * Methode liefert die Variable m_kodierung zurck
     * @return Variable m_kodierung
     */
    public int getKodierung()
    {
      return m_kodierung;
    }
    
    /**
     * Methode liefert die Richtung zu einer bergebenen Kodierung
     * @param kodierung Kodierung
     * @return Richtung
     */
    public ArbeitsRichtungQL getRichtung (int kodierung)
    {
      for (ArbeitsRichtungQL r : ArbeitsRichtungQL.values())
      {
        if (r.getKodierung() == kodierung)
          return r;
      }
      
      return null;
    }

    /**
     * Methode liefert die Richtung zu einem bestimmten Namen
     * @param nameRichtung Name
     * @return Richtung
     */
		public static ArbeitsRichtungQL getRichtung(String nameRichtung)
    {
			for (ArbeitsRichtungQL r : ArbeitsRichtungQL.values())
				if (r.getName().equals(nameRichtung))
					return r;
			
	    return null;
    }

		/**
     * Methode liefert name zurck
     * @return name
     */
    public String getName()
    {
    	return m_name;
    }

		/**
     * Methode setzt die Variable name
     * @param name zu bernehmender Wert fr name
     */
    public void setName(String name)
    {
    	m_name = name;
    }

		/**
     * Methode setzt die Variable kodierung
     * @param kodierung zu bernehmender Wert fr kodierung
     */
    public void setKodierung(int kodierung)
    {
    	m_kodierung = kodierung;
    }
  }

}
