package de.bsvrz.sba.plugin.parameter.steuerungskern.dav;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.ProjektSpezifischeEinstellungen;

/**
 * Klasse realisiert die Kommunikation mit dem DaV. ber diese Klasse werden alle Parameter lesend angemeldet
 * und zu den entsprechenden Instanzen verteilt.
 * @author MO
 */
public class DaVKommunikationSBA implements ClientReceiverInterface, Runnable
{
  /**
   * Verbindung DaV
   */
  private ClientDavInterface _connection = null;

  /**
   * Debug Anmelden am DaV
   */
  private boolean _debugAnmelde = false;

  /**
   * Debug WWA
   */
  private boolean _debugAusgaben = false;

  /**
   * Vector mit empfangenen Daten
   */
  private Vector<ResultData> m_resultInput = new Vector<ResultData>();

  /**
   * Hasmap mit den empfangenen Daten
   */
  private HashMap<String, ResultData> _empfangeneDaten = new HashMap<String, ResultData>();
  
  /**
   * Pid der SBA
   */
  public static String _pidSBA;

  /**
   * Singleton-Instanz
   */
  private static DaVKommunikationSBA _instanz = null;

  /**
   * Verwaltung der Listener
   */
  private PropertyChangeSupport m_pcs = new PropertyChangeSupport (this);

	private int m_counter = 0;

  /**
   * Liefert die Singletoninstanz.
   * @return Instanz dieser Klasse.
   */
  public static DaVKommunikationSBA getInstanz()
  {
    if (_instanz == null)
    {
    	synchronized (DaVKommunikationSBA.class)
      {
    		if (_instanz == null)
          _instanz = new DaVKommunikationSBA();
      }
    }
    return _instanz;
  }

  /**
   * Privater  Konstruktor, setzt die Verbindung zum DaV.
   * 
   * @param connection
   *          Verbindung zum DaV.
   */
  private DaVKommunikationSBA()
  {
    this._connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    
    _pidSBA = ProjektSpezifischeEinstellungen.getInstance().getPidSba();

    /*
     * Hier muss auf die DaVVerbindung mit den entsprechenden Listenern gehrt werden.
     */
    IDaVVerbindungZuhoerer zuhoerer = new IDaVVerbindungZuhoerer() 
    {
      public void verbindungAbgebaut(DaVVerbindungEreignis e)
      {
        _connection = null;
      }

      public void verbindungAufgebaut(DaVVerbindungEreignis e)
      {
        _connection = e.getDaVVerbindung();
      }
    };

    DaVVerbindung.getInstanz().hinzufuegenZuhoerer(zuhoerer);
    
    // Auswertung der Daten des DaV als  Thread starten   
    
    Thread thread = new Thread(this);
    thread.setName (this.getClass().getName());
    
    thread.start();
  }

  public ResultData addDatenArt (String pidObj, String pidAtg, String pidAsp, ReceiverRole role)
  {
//  	System.out.println("DaVKommunikationSBA.addDatenArt() " + pidAtg);
  	
  	ResultData result = leseDatenDirektVonDaV(pidObj, pidAtg, pidAsp);
  	
  	synchronized (_empfangeneDaten)
    {
  		if (result != null)
	      _empfangeneDaten.put(pidAtg, result);
    }
  	
    anmeldeStringsLeseDaV(pidObj, pidAtg, pidAsp, role);
    
    return result;
  }
  
  /**
   * Methode fgt einen Listener der Klasse hinzu
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void addPropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.addPropertyChangeListener( pcl );    
  }
  
  /**
   * Methode entfernt einen Listener aus der Klasse
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void removePropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.removePropertyChangeListener( pcl );    
  }

  /**
   * Methode benachrichtigt die Listener
   */
  private void benachrichtigeListener (String kennung)
  {
//  	System.out.println("DaVKommunikationSBA.benachrichtigeListener() " + kennung);
  	
    m_pcs.firePropertyChange( kennung, null, null );
  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum Lesen der Attibutgruppe atgPid unter dem
   * Aspekt aspPid.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeStringsLeseDaV(String obj, String atgPid, String aspPid, ReceiverRole role)
  {

    if (obj == null || atgPid == null || aspPid == null || role == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    List<SystemObject> sos = new ArrayList<SystemObject>();

    SystemObject so = _connection.getDataModel().getObject(obj);

    if (so != null)
    {
      if (_debugAnmelde)
        System.out.println("Anmelde DaV: " + so.getPid() + " " + atgPid + " " + aspPid);
      
      sos.add(so);
    }
    else
    {
      System.out.println(obj + " : nicht in der Konfiguration vorhanden");
    }

    _connection.subscribeReceiver(this, sos, dd, ReceiveOptions.normal(), role);

  }

  /**
   * Update Methode des ClientReceiverInterface
   */
  public void update( ResultData[] results )
  {
    try 
    {
      for (int resultIndex = 0; resultIndex < results.length; ++resultIndex) 
      {
        ResultData result = results[resultIndex];
        
        synchronized (m_resultInput)
        {
          m_resultInput.add(result);
          
          m_counter ++;
          m_resultInput.notifyAll();
        }
      }   
    } 
    catch (Exception e) 
    {
      e.printStackTrace();
    }
  }
    
  /**
   * Run Methode des Threads zum Auswerten der vom DaV empfangenen Daten in einem eigenen Thread
   *(Entkoppeln von der update-Methode).
   */
  public void run()
  {
    while (true)
    {
      boolean flagDaten = false;
        
      synchronized (m_resultInput)
      {
      	int tmp = 0;
      		
        // Keine Daten in Queue vorhanden
        
        if (m_resultInput.size() < 1)
        {
          // in Wait Zustand gehen
          
          try
          {
          	if (m_counter == 0)
              m_resultInput.wait();
          	
          	tmp = m_counter;

            flagDaten = true;
          }
          catch (InterruptedException e)
          {
//            m_debug.error("" + e);
          }
          
          m_counter -= tmp;
        }
        else
        {
          flagDaten = true;
        }
      }
        
      // Daten verarbeiten
      
      if (flagDaten)
      {
        verarbeiten();
      }
    }
  }

  /**
   * Methode zum eigentlichen Verarbeiten der empfangenen Daten des DaV. 
   */
  protected void verarbeiten() 
  {
    ResultData[] results = null;
    
    int anzahl;

    while (true)
    {
      anzahl = 0;

      // Daten aus Eingangsqueue holen      
      
      synchronized (m_resultInput)
      {
        if (m_resultInput.size() > 0)
        {
          anzahl = m_resultInput.size();
          
          results = new ResultData[anzahl];
          
          for (int i = 0; i < anzahl; i++)
            results[i] = m_resultInput.remove(0);
        }
      }
      
      if (anzahl == 0)
        break;
          
      // Daten aus Eingangsqueue bearbeiten      
      
      for (int i = 0; i < anzahl; i++)
      {
        final ResultData result = results[i];
        
        verteileDaten(result);
      }
    }
  }
  
  /**
   * Methode verteilt die Daten an die einzelne Instanzen die die Parameter bentigen
   * @param result zu verteilende Daten des DaV
   */
  private void verteileDaten (ResultData result)
  {
    final AttributeGroup atg = result.getDataDescription().getAttributeGroup();

    synchronized (_empfangeneDaten)
    {
      _empfangeneDaten.put(atg.getPid(), result);
    }
    benachrichtigeListener(atg.getPid());
  }
  
  public ResultData getData (String kennung)
  {
  	synchronized (_empfangeneDaten)
    {
    	if (_empfangeneDaten.containsKey(kennung))
    		return _empfangeneDaten.get(kennung);
    }
  	
  	return null;
  }
  
  /**
   * Methode liest fr das Objekt mit der PID objPid die Attibutgruppe atgPid 
   * unter dem Aspekt aspPid direkt vom DaV und ruft sofort die Auswertemethode auf
   * (nicht ber den Thread) 
   * @param objPid Pid des Objekts 
   * @param atgPid Attributgruppe die angemeldet werden soll
   * @param aspPid Apekt der angemeldet werden soll
   */
  public ResultData leseDatenDirektVonDaV (String objPid, String atgPid, String aspPid)
  {
  	System.out.println("DaVKommunikationSBA.leseDatenDirektVonDaV() " + objPid + " " +
  			atgPid + " " + aspPid);
  	
  	if (_connection == null)
  		_connection = DaVVerbindung.getInstanz().getDaVVerbindung();
  	
    ClientDavInterface verb = _connection;

    AttributeGroup atg = verb.getDataModel().getAttributeGroup(atgPid);

    Aspect asp = verb.getDataModel().getAspect(aspPid);

    DataDescription dd = new DataDescription(atg, asp);

    SystemObject obj = verb.getDataModel().getObject(objPid); 

    try
    {
      ResultData data = verb.getData( obj, dd, 10000 );
      
      ResultData[] results = new ResultData [] {data};
      
      if (results.length == 1)
      	return results [0];
    }
    catch (Exception e) 
    {
      e.printStackTrace();
    }
    
    return null;
  }
}
