package de.bsvrz.sba.plugin.parameter.steuerungskern.konfig;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse speichert die betigten Informationen eines Objekts von Typ 'typ.sbaAnzeigeQuerschnitt'
 * @author ObertM
 *
 */
public class SBAAnzeigeQuerschnitt
{
	/**
	 * Systemobjekt im DaV
	 */
  SystemObject _systemObjekt;

  /**
   * AnzeigeQuerschnitt auf den der SBA Anzeigequerschnitt verweist
   */
  SystemObject _anzeigeQuerschnitt;
  
  /**
   * Verbindung zum DaV
   */
	private ClientDavInterface _connection;

	/**
	 * Richtung der SBA Anzeige
	 */
	private String _richtung;
  
	/**
	 * Konstruktor der Klasse
	 * @param so Systemobjekt im DaV
	 */
  public SBAAnzeigeQuerschnitt (SystemObject so)
  {
  	_connection = DaVVerbindung.getInstanz().getDaVVerbindung();
  	
  	_systemObjekt = so;
  	
  	bestimmeKonfigurierendeEigenschaften();
  }

  /**
   * Methode bestimmt die konfigurierenden Eigenschaften des SbaAnzeigeQuerschnitts
   */
  private void bestimmeKonfigurierendeEigenschaften ()
  {
    // Datenmodell bestimmen
    
    DataModel dataModel = _connection.getDataModel();
    
    // Konfiguration auslesen
    
    AttributeGroup atg  = dataModel.getAttributeGroup("atg.sswSbaAnzeigeQuerschnitt");
    
    if (atg != null)
    {
      Data confData = _systemObjekt.getConfigurationData(atg);
    
      if (confData != null)
      {
        _anzeigeQuerschnitt = confData.getReferenceValue("AnzeigeQuerschnitt").getSystemObject();
      }
    }
    
    atg  = dataModel.getAttributeGroup("atg.sswSbaBetriebsKilometer");
    
    if (atg != null)
    {
      Data confData = _systemObjekt.getConfigurationData(atg);
    
      if (confData != null)
      {
      	_richtung = confData.getTextValue("Richtung").getText();
      }
    }
  }

	/**
   * Methode liefert systemObjekt zurck
   * @return systemObjekt
   */
  public SystemObject getSystemObjekt()
  {
  	return _systemObjekt;
  }

	/**
   * Methode setzt die Variable systemObjekt
   * @param systemObjekt zu bernehmender Wert fr systemObjekt
   */
  public void setSystemObjekt(SystemObject systemObjekt)
  {
  	_systemObjekt = systemObjekt;
  }

	/**
   * Methode liefert anzeigeQuerschnitt zurck
   * @return anzeigeQuerschnitt
   */
  public SystemObject getAnzeigeQuerschnitt()
  {
  	return _anzeigeQuerschnitt;
  }

	/**
   * Methode liefert richtung zurck
   * @return richtung
   */
  public String getRichtung()
  {
  	return _richtung;
  }
}
