/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.konfig;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Vector;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.DaVKommunikationSBA;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.SendeParameter;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.aufhebungen.aufhebung.VerwaltungParaAufhebung;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausfallandereszeichen.VerwaltungParaAusfallAnderesZeichen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausfallgleicheszeichen.VerwaltungParaAusfallGleichesZeichen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausschaltregel.VerwaltungParaAusschaltRegel;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.lueckenfuellen.VerwaltungParaLueckenFuellen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.monotonie.VerwaltungParaMonotonie;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.trichterung.VerwaltungParaTrichterung;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.VerwaltungParaUniversal;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.verdopplung.VerwaltungParaVerdopplung;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.zusammenfassen.VerwaltungParaZusammenfassen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.notwendigeschaltungen.absolutnotwendigeschaltung.VerwaltungParaAbsolutNotwendigeSchaltung;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.notwendigeschaltungen.notwendigeschaltung.VerwaltungParaNotwendigeSchaltung;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.prio.DefaultWerte;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemeineVerriegelungsMatrix.VerwaltungParaAllgemeineVerriegelungsMatrix;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.spezielleVerriegelungsMatrix.VerwaltungParaSpezielleVerriegelungsMatrix;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verschiebungen.verschieben.VerwaltungParaVerschieben;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.ProjektSpezifischeEinstellungen;

/**
 * Klasse beinhaltet die Parameter mit den Schaltgrnden die in den einzelnen
 * Regeln verwendet werden knnen.
 * 
 * @author ObertM
 * 
 */
public class VerwaltungMoeglicheSchaltgruende
{
	/**
	 * Einzige Instanz der Klasse
	 */
	private static VerwaltungMoeglicheSchaltgruende _instance;

	/**
	 * Verwaltung der Listener
	 */
	private PropertyChangeSupport m_pcs = new PropertyChangeSupport(this);

	/**
	 * Pid der angeforderten Attributgruppe
	 */
	public static String _pidAtg = "atg.sswSbaMoeglicheSchaltgruende";

	/**
	 * Pid der SBA
	 */
	private String _pidSBA;

	/**
	 * Vektor mit den mglichen Schaltgrnden
	 */
	private Vector<SystemObject> _schaltgruende = new Vector<SystemObject>();

	/**
	 * Privater Konstruktor der Klasse
	 */
	private VerwaltungMoeglicheSchaltgruende()
	{
		_pidSBA = ProjektSpezifischeEinstellungen.getPidSba();

		/*
		 * Daten lesen ber DaVKommunikation
		 */
		final DaVKommunikationSBA kom = DaVKommunikationSBA.getInstanz();
		kom.addDatenArt(_pidSBA, _pidAtg, "asp.parameterSoll", ReceiverRole
		    .receiver());

		kom.addPropertyChangeListener(new PropertyChangeListener()
		{
			@Override
			public void propertyChange(PropertyChangeEvent evt)
			{
				if (evt.getPropertyName().equals(_pidAtg))
				{
					ResultData resultData = kom.getData(_pidAtg);
					if (resultData != null)
						auswerteAttributgruppe(resultData);
				}
			}
		});

		ResultData resultData = kom.getData(_pidAtg);
		if (resultData != null)
			auswerteAttributgruppe(resultData);

		pruefeAufDefaultWerte();
	}

	/**
	 * Methode prft ob der Parameter im DaV vorhanden und versorgt ist. Wenn
	 * nicht werden, sofern vorhanden, Defaultdaten fr den Parametersatz erzeugt
	 * und in den DaV geschrieben.
	 */
	private void pruefeAufDefaultWerte()
	{
		synchronized (_schaltgruende)
    {
			if (_schaltgruende.size() == 0)
			{
				System.err.println("Parameter " + _pidAtg + " fr " + _pidSBA
				    + " nicht definiert. Es wird ein" + "Defaultdatensatz erzeugt !");
	
				_schaltgruende
				    .add(getSystemObject(VerwaltungParaTrichterung._pidDefaultAspekt));
				_schaltgruende
				    .add(getSystemObject(VerwaltungParaVerdopplung._pidDefaultAspekt));
				_schaltgruende
				    .add(getSystemObject(VerwaltungParaLueckenFuellen._pidDefaultAspekt));
				_schaltgruende
				    .add(getSystemObject(VerwaltungParaMonotonie._pidDefaultAspekt));
				_schaltgruende
				    .add(getSystemObject(VerwaltungParaZusammenfassen._pidDefaultAspekt));
				_schaltgruende
				    .add(getSystemObject(VerwaltungParaUniversal._pidDefaultAspekt));
				_schaltgruende
				    .add(getSystemObject(VerwaltungParaAusfallGleichesZeichen._pidDefaultAspekt));
				_schaltgruende
				    .add(getSystemObject(VerwaltungParaAusfallAnderesZeichen._pidDefaultAspekt));
				_schaltgruende
				    .add(getSystemObject(VerwaltungParaAusschaltRegel._pidDefaultAspekt));
				_schaltgruende
				    .add(getSystemObject(VerwaltungParaAbsolutNotwendigeSchaltung._pidDefaultAspekt));
				_schaltgruende
				    .add(getSystemObject(VerwaltungParaNotwendigeSchaltung._pidDefaultAspekt));
				_schaltgruende
				    .add(getSystemObject(VerwaltungParaVerschieben._pidDefaultAspekt));
				_schaltgruende
				    .add(getSystemObject(VerwaltungParaAufhebung._pidDefaultAspekt));
				_schaltgruende
				    .add(getSystemObject(VerwaltungParaAllgemeineVerriegelungsMatrix._pidDefaultAspekt));
//				_schaltgruende
//				    .add(getSystemObject(VerwaltungParaSpezielleVerriegelungsMatrix._pidDefaultAspekt));
	
				_schaltgruende.add(getSystemObject(DefaultWerte._pidHand));
				_schaltgruende.add(getSystemObject(DefaultWerte._pidNaesse));
				_schaltgruende.add(getSystemObject(DefaultWerte._pidStau));
			}
	
   		speichereInDaV();
		}
	}

	private SystemObject getSystemObject(String pid)
	{
		SystemObject so = Konfig.getInstance().getSystemObject(pid);

		System.out
		    .println("VerwaltungParaMoeglicheSchaltgruende.getSystemObject() "
		        + pid);
		if (so == null)
			System.out
			    .println("VerwaltungParaMoeglicheSchaltgruende.getSystemObject() ---> null");

		return Konfig.getInstance().getSystemObject(pid);
	}

	/**
	 * Methode zum Speichern der Parameter im DaV
	 */
	private void speichereInDaV()
	{
		Data data = getData();

		if (data != null)
		{
			SendeParameter sende = new SendeParameter(_pidAtg);
			sende.sendeParameter(getData());
		}
	}

	/**
	 * Methode liefert die Singleton Instanz der Klasse
	 * 
	 * @return Singleton Instanz der Klasse
	 */
	public static VerwaltungMoeglicheSchaltgruende getInstance()
	{
		if (_instance == null)
		{
			synchronized (VerwaltungMoeglicheSchaltgruende.class)
			{
				if (_instance == null)
				{
					_instance = new VerwaltungMoeglicheSchaltgruende();
				}
			}
		}

		return _instance;
	}

	/**
	 * Methode fgt einen Listener der Klasse hinzu
	 * 
	 * @param pcl
	 *          Listern der Klasse PropertyChangeListener
	 */
	public void addPropertyChangeListener(PropertyChangeListener pcl)
	{
		m_pcs.addPropertyChangeListener(pcl);
	}

	/**
	 * Methode entfernt einen Listener aus der Klasse
	 * 
	 * @param pcl
	 *          Listern der Klasse PropertyChangeListener
	 */
	public void removePropertyChangeListener(PropertyChangeListener pcl)
	{
		m_pcs.removePropertyChangeListener(pcl);
	}

	/**
	 * Methode benachrichtigt die Listener
	 */
	private void benachrichtigeListener()
	{
		m_pcs.firePropertyChange("Aktualisierung", null, null);
	}

	/**
	 * Methode zum Auswerten der empfangenen Attributgruppen
	 * 
	 * @param results
	 *          empfangenen Attributgruppen
	 */
	public void auswerteAttributgruppen(ResultData[] results)
	{
		int anzahl = results.length;

		// Daten aus Eingangsqueue bearbeiten

		for (int i = 0; i < anzahl; i++)
		{
			ResultData dat = results[i];

			if (dat.getDataDescription().getAttributeGroup().getPid().equals(_pidAtg))
			{
				auswerteAttributgruppe(dat);

				benachrichtigeListener();
			}
		}
	}

	/**
	 * Methode zum Auswerten der Attributgruppe 'atg.sswSbaZuordnungWVZInhalte'
	 * 
	 * @param dat
	 *          empfangender Datensatz
	 */
	private void auswerteAttributgruppe(ResultData result)
	{
		if (result.hasData())
		{
			Data data = result.getData();

			ReferenceArray a = data.getReferenceArray("Schaltgruende");

			synchronized (_schaltgruende)
      {
				_schaltgruende.clear();
				
				for (int i = 0; i < a.getLength(); ++i)
				{
					SystemObject so = a.getSystemObject(i);
					
					if ((so != null) && !_schaltgruende.contains(so))
						_schaltgruende.add(so);
				}
      }
		}
	}

	/**
	 * Methode zum Speichern der Parameter "Abhngigkeiten" im DaV
	 * 
	 * @param abhaengigkeiten
	 *          Abhngigkeiten
	 */
	public Data erzeugeAttributgruppe()
	{
		ClientDavInterface connection = DaVVerbindung.getInstanz()
		    .getDaVVerbindung();
		if (connection == null)
			return null;

		// Attributgruppe zusammensetzen und versenden

		AttributeGroup atg = (AttributeGroup) connection.getDataModel()
		    .getAttributeGroup(_pidAtg);
		Data data = connection.createData(atg);

		synchronized (_schaltgruende)
    {
			ReferenceArray a = data.getReferenceArray("Schaltgruende");
			a.setLength(_schaltgruende.size());
			
			for (int i = 0; i < _schaltgruende.size(); ++i)
				a.getReferenceValue(i).setSystemObject(_schaltgruende.get(i));
    }

		return data;
	}

	/**
	 * Methode liefert die Parameter als Attributgruppe fr den DaV
	 * 
	 * @return Attributgruppe
	 */
	public Data getData()
	{
		return erzeugeAttributgruppe();
	}

	/**
	 * Methode liefert schaltgruende zurck
	 * 
	 * @return schaltgruende
	 */
	public Vector<SystemObject> getSchaltgruende()
	{
		Vector<SystemObject> v = new Vector<SystemObject>();
		synchronized (_schaltgruende)
    {
			v.addAll(_schaltgruende);
    }
		
		return v;
	}

	/**
	 * Methode setzt die Variable schaltgruende
	 * 
	 * @param schaltgruende
	 *          zu bernehmender Wert fr schaltgruende
	 */
	public void setSchaltgruende(Vector<SystemObject> schaltgruende)
	{
		synchronized (_schaltgruende)
    {
			_schaltgruende.clear();
			for (SystemObject so : schaltgruende)
				if (!_schaltgruende.contains(so))
			    _schaltgruende.add (so);
    }
	}
}
