/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.konfig;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.HashMap;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.DaVKommunikationSBA;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.SendeParameter;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemeineVerriegelungsMatrix.InhaltMitTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.DefaultEinstellungen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.IProjektEinstellungen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.ProjektSpezifischeEinstellungen;

/**
 * Klasse beinhaltet die Parameter ber die speziellen WVZ-Inhalte 'dunkel',
 * 'beliebig', 'beibehalten' und 'don't care' definiert werden.
 * 
 * @author ObertM
 * 
 */
public class VerwaltungSpezialWVZInhalte
{
	/**
	 * Einzige Instanz der Klasse
	 */
	private static VerwaltungSpezialWVZInhalte _instance;

	/**
	 * Hashmap mit den Spezial WVZ-Inhalten
	 */
	private HashMap<ESpezialWVZInhalt, SystemObject> _inhalte = new HashMap<ESpezialWVZInhalt, SystemObject>();

	/**
	 * Verwaltung der Listener
	 */
	private PropertyChangeSupport m_pcs = new PropertyChangeSupport(this);

	/**
	 * Pid der angeforderten Attributgruppe
	 */
	public static String _pidAtg = "atg.sswSbaSpezialWVZInhalte";

	/**
	 * Pid der SBA
	 */
	private String _pidSBA;

	/**
	 * Privater Konstruktor der Klasse
	 */
	private VerwaltungSpezialWVZInhalte()
	{
		_pidSBA = ProjektSpezifischeEinstellungen.getPidSba();

		/*
		 * Daten lesen ber DaVKommunikation
		 */
		final DaVKommunikationSBA kom = DaVKommunikationSBA.getInstanz();
		kom.addDatenArt(_pidSBA, _pidAtg, "asp.parameterSoll", ReceiverRole
		    .receiver());

		kom.addPropertyChangeListener(new PropertyChangeListener()
		{
			@Override
			public void propertyChange(PropertyChangeEvent evt)
			{
				if (evt.getPropertyName().equals(_pidAtg))
				{
					ResultData resultData = kom.getData(_pidAtg);
					if (resultData != null)
						auswerteAttributgruppe(resultData);
				}
			}
		});

		ResultData resultData = kom.getData(_pidAtg);
		if (resultData != null)
			auswerteAttributgruppe(resultData);

		pruefeAufDefaultWerte();
	}

	/**
	 * Methode prft ob der Parameter im DaV vorhanden und versorgt ist. Wenn
	 * nicht werden, sofern vorhanden, Defaultdaten fr den Parametersatz erzeugt
	 * und in den DaV geschrieben.
	 */
	private void pruefeAufDefaultWerte()
	{
		if ((getDunkel() == null) && (getBeliebig() == null)
		    && (getBeibehalten() == null) && (getDontCare() == null))
		{
			System.err.println("Parameter " + _pidAtg + " fr " + _pidSBA
			    + " nicht definiert. Es wird ein" + "Defaultdatensatz erzeugt !");

			IProjektEinstellungen ipe = DefaultEinstellungen
			    .getProjektEinstellungen();
			if (ipe != null)
			{
				SystemObject so = Konfig.getInstance().getSystemObject(ipe.getPidDunkel());
				setDunkel(so);
				
				so = Konfig.getInstance().getSystemObject(
				    ipe.getPidBeibehalten());
				setBeibehalten(so);
				
				so = Konfig.getInstance().getSystemObject(ipe.getPidBeliebig());
				setBeliebig(so);
				
				so = Konfig.getInstance().getSystemObject(ipe.getPidDontCare());
				setDontCare(so);

				speichereInDaV();
			}
		}
	}

	/**
	 * Methode zum Speichern der Parameter im DaV
	 */
	private void speichereInDaV()
	{
		Data data = getData();

		if (data != null)
		{
			SendeParameter sende = new SendeParameter(_pidAtg);
			sende.sendeParameter(getData());
		}
	}

	/**
	 * Methode liefert die Singleton Instanz der Klasse
	 * 
	 * @return Singleton Instanz der Klasse
	 */
	public static VerwaltungSpezialWVZInhalte getInstance()
	{
		if (_instance == null)
		{
			synchronized (VerwaltungSpezialWVZInhalte.class)
			{
				if (_instance == null)
				{
					_instance = new VerwaltungSpezialWVZInhalte();
				}
			}
		}

		return _instance;
	}

	/**
	 * Methode fgt einen Listener der Klasse hinzu
	 * 
	 * @param pcl
	 *          Listern der Klasse PropertyChangeListener
	 */
	public void addPropertyChangeListener(PropertyChangeListener pcl)
	{
		m_pcs.addPropertyChangeListener(pcl);
	}

	/**
	 * Methode entfernt einen Listener aus der Klasse
	 * 
	 * @param pcl
	 *          Listern der Klasse PropertyChangeListener
	 */
	public void removePropertyChangeListener(PropertyChangeListener pcl)
	{
		m_pcs.removePropertyChangeListener(pcl);
	}

	/**
	 * Methode benachrichtigt die Listener
	 */
	private void benachrichtigeListener()
	{
		m_pcs.firePropertyChange("Aktualisierung", null, null);
	}

	/**
	 * Methode zum Auswerten der empfangenen Attributgruppen
	 * 
	 * @param results
	 *          empfangenen Attributgruppen
	 */
	public void auswerteAttributgruppen(ResultData[] results)
	{
		int anzahl = results.length;

		// Daten aus Eingangsqueue bearbeiten

		for (int i = 0; i < anzahl; i++)
		{
			ResultData dat = results[i];

			if (dat.getDataDescription().getAttributeGroup().getPid().equals(_pidAtg))
			{
				auswerteAttributgruppe(dat);

				benachrichtigeListener();
			}
		}
	}

	/**
	 * Methode zum Auswerten der Attributgruppe 'atg.sswSbaZuordnungWVZInhalte'
	 * 
	 * @param dat
	 *          empfangender Datensatz
	 */
	private void auswerteAttributgruppe(ResultData result)
	{
		if (result.hasData())
		{
			SystemObject so = result.getData().getReferenceValue("Dunkel").getSystemObject();
			_inhalte.put(ESpezialWVZInhalt.Dunkel, so);

			so = result.getData().getReferenceValue("Beliebig")
			    .getSystemObject();
			_inhalte.put(ESpezialWVZInhalt.Beliebig, so);

			so = result.getData().getReferenceValue("Beibehalten")
			    .getSystemObject();
			_inhalte.put(ESpezialWVZInhalt.Beibehalten, so);

			so = result.getData().getReferenceValue("DontCare")
			    .getSystemObject();
			_inhalte.put(ESpezialWVZInhalt.DontCare, so);

		}
	}

	/**
	 * Methode zum Speichern der Parameter "Abhngigkeiten" im DaV
	 * 
	 * @param abhaengigkeiten
	 *          Abhngigkeiten
	 */
	public Data erzeugeAttributgruppe()
	{
		ClientDavInterface connection = DaVVerbindung.getInstanz()
		    .getDaVVerbindung();
		if (connection == null)
			return null;

		// Attributgruppe zusammensetzen und versenden

		AttributeGroup atg = (AttributeGroup) connection.getDataModel()
		    .getAttributeGroup(_pidAtg);
		Data data = connection.createData(atg);

		data.getReferenceValue("Dunkel").setSystemObject(getDunkel());

		data.getReferenceValue("Beliebig").setSystemObject(getBeliebig());

		data.getReferenceValue("Beibehalten").setSystemObject(getBeibehalten());

		data.getReferenceValue("DontCare").setSystemObject(getDontCare());

		return data;
	}

	/**
	 * Methode liefert die Parameter als Attributgruppe fr den DaV
	 * 
	 * @return Attributgruppe
	 */
	public Data getData()
	{
		return erzeugeAttributgruppe();
	}

	/**
	 * Methode liefert dunkel zurck
	 * 
	 * @return dunkel
	 */
	public SystemObject getDunkel()
	{
		return _inhalte.get(ESpezialWVZInhalt.Dunkel);
	}

	/**
	 * Methode setzt die Variable dunkel
	 * 
	 * @param dunkel
	 *          zu bernehmender Wert fr dunkel
	 */
	public void setDunkel(SystemObject dunkel)
	{
		_inhalte.put(ESpezialWVZInhalt.Dunkel, dunkel);
	}

	/**
	 * Methode liefert beliebig zurck
	 * 
	 * @return beliebig
	 */
	public SystemObject getBeliebig()
	{
		return _inhalte.get(ESpezialWVZInhalt.Beliebig);
	}

	/**
	 * Methode setzt die Variable beliebig
	 * 
	 * @param beliebig
	 *          zu bernehmender Wert fr beliebig
	 */
	public void setBeliebig(SystemObject beliebig)
	{
		_inhalte.put(ESpezialWVZInhalt.Beliebig, beliebig);
	}

	/**
	 * Methode liefert beibehalten zurck
	 * 
	 * @return beibehalten
	 */
	public SystemObject getBeibehalten()
	{
		return _inhalte.get(ESpezialWVZInhalt.Beibehalten);
	}

	/**
	 * Methode setzt die Variable beibehalten
	 * 
	 * @param beibehalten
	 *          zu bernehmender Wert fr beibehalten
	 */
	public void setBeibehalten(SystemObject beibehalten)
	{
		_inhalte.put(ESpezialWVZInhalt.Beibehalten, beibehalten);
	}

	/**
	 * Methode liefert dontCare zurck
	 * 
	 * @return dontCare
	 */
	public SystemObject getDontCare()
	{
		return _inhalte.get(ESpezialWVZInhalt.DontCare);
	}

	/**
	 * Methode setzt die Variable dontCare
	 * 
	 * @param dontCare
	 *          zu bernehmender Wert fr dontCare
	 */
	public void setDontCare(SystemObject dontCare)
	{
		_inhalte.put(ESpezialWVZInhalt.DontCare, dontCare);
	}
	
	/**
	 * Methode liefert die Spezial WVZ-Inhalte
	 * @return Spezial WVZ-Inhalte
	 */
	public HashMap<ESpezialWVZInhalt, SystemObject> getSpezialWVZInhalte ()
	{
		return _inhalte;
	}
	
	public void setSpezialWVZInhalte (HashMap<ESpezialWVZInhalt, SystemObject> inhalte)
	{
		_inhalte = inhalte;
	}
}
