package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausfallandereszeichen;

/**
 * Definition der zu schaltenden Anzeigequerschnitte
 * @author Swarco Traffic Systems GmbH
 */
public enum SelektionAnzeigeQuerschnitt
{
  /**
   * Kein Anzeigequerschnitt
   */
  KEINE ("Kein", 0),
  
  /**
   * Aktueller Anzeigequerschnitt
   */
  AKTUELL ("Aktueller", 1),
  
  /**
   * Anzeigequerschnitt in Lngsrichtung
   */
  LAENGSRICHTUNG ("Vorgnger/Nachfolger", 2);
  
  private int m_kodierung;
  
	private String m_bezeichnung;
  
  /**
   * Konstruktor der Klasse
   * @param kodierung Kodierung der Enum-Werte im DaV
   */
  SelektionAnzeigeQuerschnitt (String bezeichnung, int kodierung)
  {
  	m_bezeichnung = bezeichnung;
  	
    m_kodierung = kodierung;  
  }
  
  /**
   * Methode liefert den Wert der zu einer Kodierung im DaV gehrt
   * @param kodierung Kodierung deren Wert gesucht wird
   * @return Wert, bei einer falschen Kodierung: 'keinAnzeigeQuerschnitt'
   */
  public static SelektionAnzeigeQuerschnitt getValue (int kodierung)
  {
    for (SelektionAnzeigeQuerschnitt s: SelektionAnzeigeQuerschnitt.values())
    {
      if (s.m_kodierung == kodierung)
        return s;
    }
    
    return KEINE;
  }

  /**
   * Methode liefert die Variable m_kodierung zurck
   * @return Variable m_kodierung
   */
  public int getKodierung()
  {
    return m_kodierung;
  }

	/**
   * Methode liefert bezeichnung zurck
   * @return bezeichnung
   */
  public String getBezeichnung()
  {
  	return m_bezeichnung;
  }

  /**
   * Methode liefert die Selektion, die zu einer Bezeichnung passe
   * @param bezeichnung Bezeichnung der Selektion
   * @return Selektion
   */
	public static SelektionAnzeigeQuerschnitt getSelektion(String bezeichnung)
  {
		for (SelektionAnzeigeQuerschnitt sa : values())
			if (sa.getBezeichnung().equals(bezeichnung))
				return sa;
		
	  return null;
  }
};
