package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausfallandereszeichen;

import java.util.Observable;
import java.util.Observer;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;

/**
 * Tabellen Modell fr die Parameter Ausfall anderes Zeichen
 * @author ObertM
 *
 */
public class TableModel extends Observable implements Observer
{
	/**
	 * Merker zum Lschen der Zeile
	 */
	private Composite _loeschen;
	
	/**
	 * Schaltgrund
	 */
  private Composite _schaltgrund;
  
  /**
   * Arbeitsrichtung
   */
  private Composite _richtung;

  /**
   * Parameter X (zu prfender Anzeigeinhalt)
   */
  private Composite _x;

  /**
   * Regelteil anwenden
   */
  private Composite _regelteilAnwenden;
  
  /**
   * Parameter Y (angefordeter Inhalt)
   */
  private Composite _y;
  
  /**
   * Selektion Anzeige
   */
  private Composite _selektionAnzeige;
  
  /**
   * Auswahl der zustzlich zu schaltenden Anzeigen
   * (= Anzeigetyp + Selektion Anzeigequerschnitt)
   */
  private Composite _auswahlZusaetzlich;
  
  /**
   * zustzlich angefordeter Anzeigeninhalt
   */
  private Composite _inhaltZusaetzlich;

  /**
   * Auswahl der zustzlich zu schaltenden Anzeigen an Vorgnger / Nachfolger
   * (= Anzeigetyp + Ebende)
   */
  private Composite _auswahlVerschieben;
  
  /**
   * angefordeter Anzeigeninhalt fr die Verschiebung
   */
  private Composite _inhaltVerschiebung;
  
  /**
   * Konstruktor der Klasse
   * @param loeschen Composite zum Lschen einer Regel
   * @param schaltgrund Compoiste fr den regelspezifischen Schaltgrund
   * @param richtung Composite fr die Arbeitsrichtung
   * @param x Composite fr den zu prfenden Anzeigeninhalt
   * @param regelAnwenden Composite fr Auswahlfeld "Regelteil anwenden"
   * @param y Composite fr den zu schaltenden Anzeigeinhalt (in der Regel "Aus")
   * @param selektionAnzeige Composite fr die Selektion der Anzeige
   * @param auswahlZusaetzlich Composite fr die zustzlichen Anzeigen
   * @param inhaltZusaetzlich  Composite fr die zustzlichen Anzeigeinhalte (in der Regel "X", "Y", "Z"
   * @param auswahlVerschieben Composite fr die zu Anzeigen in Vorgnger/Nachfolger Richtung
   * @param inhaltVerschieben Composite fr die Vorgnger/Nachfolger Anzeigeinhalte (in der Regel "X", "Y", "Z"
   */
  public TableModel (Composite loeschen, 
  		               Composite schaltgrund, 
  		               Composite richtung,
  		               Composite x, 
  		               Composite regelAnwenden,
  		               Composite y,
  		               Composite selektionAnzeige,
  		               Composite auswahlZusaetzlich,
  		               Composite inhaltZusaetzlich,
  		               Composite auswahlVerschieben,
  		               Composite inhaltVerschieben)
  {
  	this._loeschen = loeschen;
    this._schaltgrund = schaltgrund;
    this._richtung = richtung;
    this._x  = x;    
    this._regelteilAnwenden = regelAnwenden;
    this._y = y;
    this._selektionAnzeige = selektionAnzeige;
    this._auswahlZusaetzlich = auswahlZusaetzlich;
    this._inhaltZusaetzlich = inhaltZusaetzlich;
    this._auswahlVerschieben = auswahlVerschieben;
    this._inhaltVerschiebung = inhaltVerschieben;
  }

  /**
   * Lesen der Einstellung des Composite "Lschen"
   * @return eingestellter Wert als String
   */
  public String getSelectedLoeschen()
  {
    return getSelectedWert(_loeschen);    
  }

  /**
   * Setzen des Wertes des Composite "Lschen"
   * @param selWert zu setzender Wert
   */
  public void setSelectedLoeschen(String selWert)
  {
    setSelectedWert(_loeschen, selWert, null);
  }

  /**
   * Lesen der Einstellung des Composite "Schaltgrund"
   * @return eingestellter Wert als String
   */
  public String getSelectedSchaltgrund ()
  {
    return getSelectedWert(_schaltgrund);    
  }

  /**
   * Setzen des Wertes des Composite "Schaltgrund"
   * @param selWert zu setzender Wert
   */
  public void setSelectedSchaltgrund(String selWert)
  {
    setSelectedWert(_schaltgrund, selWert, null);
  }

  /**
   * Lesen der Einstellung des Composite "Richtung"
   * @return eingestellter Wert als String
   */
  public String getSelectedRichtung ()
  {
    return getSelectedWert(_richtung);    
  }

  /**
   * Setzen des Wertes des Composite "Richtung"
   * @param selWert zu setzender Wert
   */
  public void setSelectedRichtung(String selWert)
  {
    setSelectedWert(_richtung, selWert, null);
  }

  /**
   * Lesen der Einstellung des Composite "X"
   * @return eingestellter Wert als String
   */
  public String getSelectedX ()
  {
    return getSelectedWert(_x);    
  }
  
  /**
   * Setzen des Wertes des Composite "X"
   * @param selImage zu setzendes Image
   */
  public void setSelectedX(Image selImage)
  {
    setSelectedWert(_x, null, selImage);
  }
  
  /**
   * Lesen der Einstellung des Composite "Regelteil anwenden"
   * @return eingestellter Wert als String
   */
  public String getSelectedRegelTeilAnwenden ()
  {
    return getSelectedWert(_regelteilAnwenden);    
  }

  /**
   * Setzen des Wertes des Composite "Regelteil anwenden"
   * @param selWert zu setzender Wert
   */
  public void setSelectedRegelTeilAnwenden(String selWert)
  {
    setSelectedWert(_regelteilAnwenden, selWert, null);
  }

  /**
   * Lesen der Einstellung des Composite "Y"
   * @return eingestellter Wert als String
   */
  public String getSelectedY ()
  {
    return getSelectedWert(_y);    
  }
  
  /**
   * Setzen des Wertes des Composite "Y"
   * @param selImage zu setzendes Image
   */
  public void setSelectedY(Image selImage)
  {
    setSelectedWert(_y, null, selImage);
  }

  /**
   * Lesen der Einstellung des Composite "Selektion Anzeige"
   * @return eingestellter Wert als String
   */
  public String getSelectedSelektionAnzeige ()
  {
    return getSelectedWert(_selektionAnzeige);    
  }
  
  /**
   * Setzen des Wertes des Composite "Selektion Anzeige"
   * @param selImage zu setzendes Image
   */
  public void setSelectedSelektionAnzeige (String selWert)
  {
    setSelectedWert(_selektionAnzeige, selWert, null);
  }
  
  /**
   * Methode liest den eingestellten Wert eines Composites als String. Dabei wird nur
   * das erste Kontrollelement des Composite ausgewertet.
   * @param c Composite
   * @return eingestellter Wert als String
   */
  public String getSelectedWert (Composite c)
  {
    Control control = c.getChildren()[0];
    return getSelectedWert(control);
  }  
  
  /**
   * Lesen der Einstellung des Composite "Selektion Anzeige", Element "Auswahl WZG Typ"
   * @return eingestellter Wert als String
   */
  public String getSelectedWZGTypZusaetzlich ()
  {
    return getSelectedWert(_auswahlZusaetzlich.getChildren() [0]);    
  }
  
  /**
   * Setzen des Wertes des Composite "Auswahl zustzlich", Element "Auswahl WZG Typ"
   * @param selWert zu setzender Wert
   */
  public void setSelectedWZGTypZusaetzlich (String selWert)
  {
    setSelectedWert(_auswahlZusaetzlich.getChildren()[0], selWert, null);
  }

  /**
   * Lesen der Einstellung des Composite "Auswahl zustzlich", Element "Selektion AQ"
   * @return eingestellter Wert als String
   */
  public String getSelectedSelektionAQ ()
  {
    return getSelectedWert(_auswahlZusaetzlich.getChildren() [1]);    
  }
  
  /**
   * Setzen des Wertes des Composite "Selektion Anzeige", Element "Selektion AQ"
   * @param selWert zu setzender Wert
   */
  public void setSelectedSelektionAQ (String selWert)
  {
    setSelectedWert(_auswahlZusaetzlich.getChildren()[1], selWert, null);
  }

  /**
   * Lesen der Einstellung des Composite "Auswahl Verschieben", Element "Auswahl WZG Typ"
   * @return eingestellter Wert als String
   */
  public String getSelectedWZGTypVerschieben ()
  {
    return getSelectedWert(_auswahlVerschieben.getChildren() [0]);    
  }
  
  /**
   * Setzen des Wertes des Composite "Auswahl Verschieben", Element "Auswahl WZG Typ"
   * @param selWert zu setzender Wert
   */
  public void setSelectedWZGTypVerschieben (String selWert)
  {
    setSelectedWert(_auswahlVerschieben.getChildren()[0], selWert, null);
  }

  /**
   * Lesen der Einstellung des Composite "Auswahl Verschieben", Element "Ebene"
   * @return eingestellter Wert als String
   */
  public int getSelectetEbene()
  {
    Control control = _auswahlVerschieben.getChildren()[1];
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      int intPrio = sp.getSelection();
      return intPrio;
    }
    else
      return 0;
  }

  /**
   * Setzen des Wertes des Composite "Auswahl Verschieben", Element "Ebene"
   * @param selWert zu setzender Wert
   */
  public void setSelectedEbene(String selWert)
  {
    setSelectedWert(_auswahlVerschieben.getChildren()[1], selWert, null);
  }

  /**
   * Methode liest den eingestellten Wert eines Kontrollelements eines Composites
   * @param control Kontrollelement
   * @return eingestellter Wert als String
   */
  public String getSelectedWert(Control control)
  {
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      int spSel = sp.getSelection();
      return String.valueOf(spSel);
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      String cbSel = cb.getItem( cb.getSelectionIndex() );
      return cbSel;
    }
    else if ( control instanceof Button )
    {
      if ( (control.getStyle() & SWT.CHECK) != 0 )
      {
        Button ch = (Button)control;
        boolean chSel = ch.getSelection();
        return String.valueOf(chSel);
      }
      else
        return "";
    }
    else if ( control instanceof Label )
    {
      Label im = (Label)control;
      String imSel = (String)im.getData();
      return imSel;
    }
    else
      return "";
  }

  /**
   * Methode setzt den Wert des ersten Kontrollelements eines Composites
   * @param c Composite
   * @param selWert Wert als String
   * @param selImage Wert als Image
   */
  public void setSelectedWert(Composite c, String selWert, Image selImage)
  {
    Control control = c.getChildren()[0];
    
    setSelectedWert(control, selWert, selImage);
  }
  
  /**
   * Methode setzt den Wert eines Kontrollelements
   * @param control Kontrollelement
   * @param selWert Wert als String
   * @param selImage Wert als Image
   */
  public void setSelectedWert(Control control, String selWert, Image selImage)
  {
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      try
      {
        sp.setSelection(Integer.valueOf( selWert ));
      }
      catch (NumberFormatException e)
      {
        e.printStackTrace();
      }
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      for (int k = 0; k < cb.getItemCount(); k++)
      {
        if (selWert.equals(cb.getItem( k )))
          cb.select(k);      
      }
    }
    else if ( control instanceof Button )
    {
      if ( (control.getStyle() & SWT.CHECK) != 0 )
      {
        Button ch = (Button)control;
        ch.setSelection( Boolean.valueOf( selWert ) );
      }
    }
    else if ( control instanceof Label )
    {
        Label im = (Label)control;
        im.setData( selWert );
        im.setImage( selImage ); 
    }
  }

  @Override
  public void update(Observable o, Object arg)
  {
    fireEvent();
  }

  public void fireEvent()
  {
    setChanged();
    notifyObservers();
  }

	/**
   * Methode liefert loeschen zurck
   * @return loeschen
   */
  public Composite getLoeschen()
  {
  	return _loeschen;
  }

	/**
   * Methode liefert schaltgrund zurck
   * @return schaltgrund
   */
  public Composite getSchaltgrund()
  {
  	return _schaltgrund;
  }

	/**
   * Methode liefert richtung zurck
   * @return richtung
   */
  public Composite getRichtung()
  {
  	return _richtung;
  }

	/**
   * Methode liefert x zurck
   * @return x
   */
  public Composite getX()
  {
  	return _x;
  }

	/**
   * Methode liefert regelteilAnwenden zurck
   * @return regelteilAnwenden
   */
  public Composite getRegelteilAnwenden()
  {
  	return _regelteilAnwenden;
  }

	/**
   * Methode liefert y zurck
   * @return y
   */
  public Composite getY()
  {
  	return _y;
  }

	/**
   * Methode liefert selektionAnzeige zurck
   * @return selektionAnzeige
   */
  public Composite getSelektionAnzeige()
  {
  	return _selektionAnzeige;
  }

	/**
   * Methode liefert auswahlZusaetzlich zurck
   * @return auswahlZusaetzlich
   */
  public Composite getAuswahlZusaetzlich()
  {
  	return _auswahlZusaetzlich;
  }

	/**
   * Methode liefert inhaltZusaetzlich zurck
   * @return inhaltZusaetzlich
   */
  public Composite getInhaltZusaetzlich()
  {
  	return _inhaltZusaetzlich;
  }

	/**
   * Methode liefert auswahlVerschieben zurck
   * @return auswahlVerschieben
   */
  public Composite getAuswahlVerschieben()
  {
  	return _auswahlVerschieben;
  }

	/**
   * Methode liefert inhaltVerschiebung zurck
   * @return inhaltVerschiebung
   */
  public Composite getInhaltVerschiebung()
  {
  	return _inhaltVerschiebung;
  }

  /**
   * Lesen der Einstellungen des Composite "Inhalt verschieben"
   * @return eingestellte Werte
   */
  public String[] getSelectedInhaltVerschieben ()
  {
  	Control[] controls = _inhaltVerschiebung.getChildren();
  	
  	String [] selected = new String [controls.length];
  	
  	for (int i=0; i<controls.length; ++i)
  	{
  	  selected [i] = getSelectedWert(controls [i]);
  	}
  	
    return selected;    
  }

  /**
   * Lesen der Einstellungen des Composite "Inhalt zustzlich"
   * @return eingestellte Werte
   */
  public String[] getSelectedInhaltZusaetlich ()
  {
  	Control[] controls = _inhaltZusaetzlich.getChildren();
  	
  	String [] selected = new String [controls.length];
  	
  	for (int i=0; i<controls.length; ++i)
  	{
  	  selected [i] = getSelectedWert(controls [i]);
  	}
  	
    return selected;    
  }

}
