package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausfallgleicheszeichen;

import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;

/**
 * Klasse enthlt ein Feld (Liste) mit Parametern
 * @author       Dambach Werke GmbH
 */
public class ListeMitRegeln
{
  /**
   * Liste mit den Parametern
   */
  private Vector<RegelAusfallGleichesZeichen> m_parameter = new Vector<RegelAusfallGleichesZeichen> ();

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public ListeMitRegeln ()
  {
  }
  
  /**
   * Methode liefert die Variable parameter zurck
   * @return Variable parameter
   */
  public Vector<RegelAusfallGleichesZeichen> getParameter()
  {
    Vector<RegelAusfallGleichesZeichen> v = new Vector<RegelAusfallGleichesZeichen>();
    synchronized (m_parameter)
    {
      for (RegelAusfallGleichesZeichen r : m_parameter)
        v.add( r );
    }
    return v;
  }

  /**
   * Methode zum Auswerten der Regeln der Attributliste 'atl.sswSbaAusfallGleichesZeichenProAnzeigeTyp'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data, Aspect schaltgrundAllgemein)
  {
    Array a = data.getArray( "Regeln" );
    int anzahl = a.getLength();
    
    synchronized (m_parameter)
    {
      m_parameter.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        RegelAusfallGleichesZeichen r = new RegelAusfallGleichesZeichen ();
        r.auswerteAttributgruppe( a.getItem( x ) , schaltgrundAllgemein );
        
        m_parameter.add( r );
      }
    }
  }
  
  /**
   * Methode fgt eine Regel hinzu
   * @param x Anzeigeinhalt der aktuell betrachteten Anzeige 
   * @param y Angeforderter Anzeigeinhalt
   * @param pruefeAndereZeichen Sollen die anderen Anzeigen des AQ geprft werden
   * @param schaltgrund Schaltgrund unter dem diese Regel publiziert wird
   */
  public void addRegel (WVZInhalt x, WVZInhalt y, boolean pruefeAndereZeichen, Aspect schaltgrund )
  {
    synchronized (m_parameter)
    {
      RegelAusfallGleichesZeichen r = new RegelAusfallGleichesZeichen ();
      
      r.setSchaltgrundRegel( schaltgrund );
      r.setAnzeigenInhalt( x );
      r.setAngeforderterAnzeigenInhalt( y );
      r.setPruefeAndereZeichen( pruefeAndereZeichen );
      
      m_parameter.add( r );
    }
  }

  /**
   * Methode liefert den Inahlt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    synchronized (m_parameter)
    {
      s += "Regeln:\n";
      for (RegelAusfallGleichesZeichen r : m_parameter)
        s += r.toString() + "\n";
    }
    
    return s;
  }

  /**
   * Methode setzt die Variable parameter
   * @param parameter Wert fr parameter
   */
  public void setParameter(Vector<RegelAusfallGleichesZeichen> parameter)
  {
    synchronized (m_parameter)
    {
      m_parameter.clear();
      
      for (RegelAusfallGleichesZeichen r : parameter)
         m_parameter.add(r);
    } 
  }
}
