package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausfallgleicheszeichen;

import java.util.Observable;
import java.util.Observer;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;

/**
 * Tabellen Modell fr die Parameter Trichterung
 * @author ObertM
 *
 */
public class TableModel extends Observable implements Observer
{
	/**
	 * Merker zum Lschen der Zeile
	 */
	private Composite _loeschen;
	
	/**
	 * Schaltgrund
	 */
  private Composite _schaltgrund;
  
  /**
   * Parameter X
   */
  private Composite _x;
  
  /**
   * Parameter Y
   */
  private Composite _y;
  
  /**
   * Parameter Regelteil anwenden
   */
  private Composite _anwenden;
  
  /**
   * Konstruktor der Klasse
   * @param loeschen Loeschen
   * @param schaltgrund Schaltgrund
   * @param x X
   * @param y Y
   * @param anwenden Regelteil anwenden
   */
  public TableModel(Composite loeschen, Composite schaltgrund, Composite x, Composite y, Composite anwenden)
  {
  	this._loeschen = loeschen;
    this._schaltgrund = schaltgrund;
    this._x = x;    
    this._y = y;    
    this._anwenden = anwenden;    
  }

  public String getSelectedLoeschen()
  {
    return getSelectedWert(_loeschen);    
  }

  public String getSelectedSchaltgrund ()
  {
    return getSelectedWert(_schaltgrund);    
  }

  public String getSelectedX ()
  {
    return getSelectedWert(_x);    
  }

  public String getSelectedY ()
  {
    return getSelectedWert(_y);    
  }

  public String getSelectedAnwenden ()
  {
    return getSelectedWert(_anwenden);    
  }

  public void setSelectedLoeschen(String selWert)
  {
    setSelectedWert(_loeschen, selWert, null);
  }

  public void setSelectedSchaltgrund(String selWert)
  {
    setSelectedWert(_schaltgrund, selWert, null);
  }

  public void setSelectedX(Image selImage)
  {
    setSelectedWert(_x, null, selImage);
  }

  public void setSelectedY(Image selImage)
  {
    setSelectedWert(_y, null, selImage);
  }
  
  public void setSelectedAnwenden(String selWert)
  {
    setSelectedWert(_anwenden, selWert, null);
  }


  public String getSelectedWert(Composite c)
  {
    Control control = c.getChildren()[0];
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      int spSel = sp.getSelection();
      return String.valueOf(spSel);
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      String cbSel = cb.getItem( cb.getSelectionIndex() );
      return cbSel;
    }
    else if ( control instanceof Button )
    {
      if ( (control.getStyle() & SWT.CHECK) != 0 )
      {
        Button ch = (Button)control;
        boolean chSel = ch.getSelection();
        return String.valueOf(chSel);
      }
      else
        return "";
    }
    else if ( control instanceof Label )
    {
      Label im = (Label)control;
      String imSel = (String)im.getData();
      return imSel;
    }
    else
      return "";
  }

  public void setSelectedWert(Composite c, String selWert, Image selImage)
  {
    Control control = c.getChildren()[0];
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      try
      {
        sp.setSelection(Integer.valueOf( selWert ));
      }
      catch (NumberFormatException e)
      {
        e.printStackTrace();
      }
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      for (int k = 0; k < cb.getItemCount(); k++)
      {
        if (selWert.equals(cb.getItem( k )))
          cb.select(k);      
      }
    }
    else if ( control instanceof Button )
    {
      if ( (control.getStyle() & SWT.CHECK) != 0 )
      {
        Button ch = (Button)control;
        ch.setSelection( Boolean.valueOf( selWert ) );
      }
    }
    else if ( control instanceof Label )
    {
        Label im = (Label)control;
        im.setData( selWert );
        im.setImage( selImage ); 
    }
  }

  @Override
  public void update(Observable o, Object arg)
  {
    fireEvent();
  }

  public void fireEvent()
  {
    setChanged();
    notifyObservers();
  }

	/**
   * Methode liefert schaltgrund zurck
   * @return schaltgrund
   */
  public Composite getSchaltgrund()
  {
  	return _schaltgrund;
  }

	/**
   * Methode setzt die Variable schaltgrund
   * @param schaltgrund zu bernehmender Wert fr schaltgrund
   */
  public void setSchaltgrund(Composite schaltgrund)
  {
  	_schaltgrund = schaltgrund;
  }

	/**
   * Methode liefert x zurck
   * @return x
   */
  public Composite getX()
  {
  	return _x;
  }

	/**
   * Methode setzt die Variable x
   * @param x zu bernehmender Wert fr x
   */
  public void setX(Composite x)
  {
  	_x = x;
  }

	/**
   * Methode liefert y zurck
   * @return y
   */
  public Composite getY()
  {
  	return _y;
  }

	/**
   * Methode setzt die Variable y
   * @param y zu bernehmender Wert fr y
   */
  public void setY(Composite y)
  {
  	_y = y;
  }


	/**
   * Methode liefert loeschen zurck
   * @return loeschen
   */
  public Composite getLoeschen()
  {
  	return _loeschen;
  }

	/**
   * Methode liefert anwenden zurck
   * @return anwenden
   */
  public Composite getAnwenden()
  {
  	return _anwenden;
  }

	/**
   * Methode setzt die Variable anwenden
   * @param anwenden zu bernehmender Wert fr anwenden
   */
  public void setAnwenden(Composite anwenden)
  {
  	_anwenden = anwenden;
  }

}
