package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausschaltregel;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse enthlt eine Fllregel der Regel Ausfallbehandlung
 * @author Dambach Werke GmbH
 * @author Matthias Obert
 */
public class FuellRegel
{
  /**
   * Anzeigetyp der geschaltet werden soll (in der Regel "A")
   */
  private SystemObject m_anzeigeTyp;
  
  /**
   * Angeforderter Inhalt (in Regel "Y")
   */
  private WVZInhalt m_angeforderterInhalt;

  /**
   * Prioritt (in der Regel "Z")
   */
  private int m_prioritaet;

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public FuellRegel ()
  {
    
  }

  /**
   * Konstruktor der Klasse 
   * @param anzeigeTyp Anzeigetyp der geschaltet werden soll (in der Regel "A")
   * @param angeforderterInhalt Angeforderter Inhalt (in Regel "Y")
   * @param prioritaet Prioritt (in der Regel "Z")
   */
  public FuellRegel (SystemObject anzeigeTyp, WVZInhalt angeforderterInhalt, int prioritaet)
  {
    m_anzeigeTyp          = anzeigeTyp;
    
    m_angeforderterInhalt = angeforderterInhalt;
    
    m_prioritaet          = prioritaet;
  }
  
  /**
   * Methode zum Auswerten einer Regel der Attributliste 'atl.sswSbaRegelFllRegel'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe( Data data )
  {
    m_anzeigeTyp = data.getReferenceValue( "AnzeigeTyp" ).getSystemObject();
    
    m_prioritaet = data.getUnscaledValue( "Prioritt" ).intValue();
    
    Data d = data.getItem( "AngeforderterAnzeigeInhalt" );

    m_angeforderterInhalt = new WVZInhalt ();
    m_angeforderterInhalt.auswerteAttributgruppe( d );
  }

  /**
   * Methode liefert die Variable m_anzeigeTyp zurck
   * @return Variable m_anzeigeTyp
   */
  public SystemObject getAnzeigeTyp()
  {
    return m_anzeigeTyp;
  }

  /**
   * Methode setzt die Variable m_anzeigeTyp
   * @param anzeigeTyp Wert auf den die Variable m_anzeigeTyp gesetzt werden soll
   */
  public void setAnzeigeTyp( SystemObject anzeigeTyp )
  {
    m_anzeigeTyp = anzeigeTyp;
  }

  /**
   * Methode liefert die Variable m_angeforderterInhalt zurck
   * @return Variable m_angeforderterInhalt
   */
  public WVZInhalt getAngeforderterInhalt()
  {
    return m_angeforderterInhalt;
  }

  /**
   * Methode setzt die Variable m_angeforderterInhalt
   * @param angeforderterInhalt Wert auf den die Variable m_angeforderterInhalt gesetzt werden soll
   */
  public void setAngeforderterInhalt( WVZInhalt angeforderterInhalt )
  {
    m_angeforderterInhalt = angeforderterInhalt;
  }

  /**
   * Methode liefert die Variable m_prioritaet zurck
   * @return Variable m_prioritaet
   */
  public int getPrioritaet()
  {
    return m_prioritaet;
  }

  /**
   * Methode setzt die Variable m_prioritaet
   * @param prioritaet Wert auf den die Variable m_prioritaet gesetzt werden soll
   */
  public void setPrioritaet( int prioritaet )
  {
    m_prioritaet = prioritaet;
  }
  
  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    s += Tools.getObjektName( getAnzeigeTyp() ) + ": " + Tools.getWVZInhalt( m_angeforderterInhalt ) + " Prio: " + getPrioritaet();
    return s;
  }
}
