package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.entfernungsangaben;

import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeElementFigure;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeInhaltAuswahlDialog;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeInhaltTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.VerwaltungWvzInhalte;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeElementTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Darstellung des Fensters zur Versorgung der Parameter
 */
public class CompositeParameterEntfernungsAngaben extends Composite
{
	/**
	 * Maximal zulssige Entfernung
	 */
	private int _maxEntfernung = 10000;
	
	/**
	 * Aktuelles Composite
	 */
  private Composite _composite;

  /**
   * Composite der Tabelle
   */
  private Composite _compositeTabelle;

  /**
   * Gruppe mit den Regeln
   */
  private Group _groupRegeln;

  /**
   * Tabelviewer der Tabelle
   */
  private TableViewer _tvAnzeige;

  /**
   * Aktuelle Parameter
   */
  private TreppenFunktion _aktuelleParameter;

  /**
   * Composite fr die Buttons
   */
	private Composite _compositeButtons;

	/**
	 * Button Hinzufgen
	 */
	private Button _buttonHinzufuegen;

	/**
	 * Button Lschen
	 */
	private Button _buttonLoeschen;

	/**
	 * Tabellenmodell
	 */
	private TableModel[] _tableModel;

	/**
	 * Bezeichnung der Treppenfunktion
	 */
	private String _nameTreppenFunktion;

	/**
	 * Anzeigetyp
	 */
	protected AnzeigeElementTyp _anzeigeTyp = AnzeigeElementTyp.WZG_C;
	
  /**
   * Konstruktor der Klasse
   * @param parent Parent Composite
   * @param style Stil des Fenstern
   * @param aktuell Aktuelle Parameter
   * @param nameTreppenFunktion Name der Treppenfunktion
   */
  CompositeParameterEntfernungsAngaben (Composite parent, int style, TreppenFunktion aktuell, String nameTreppenFunktion)
  {
    super (parent, style);
    
    _aktuelleParameter = erzeugeKopie(aktuell);
    
    _nameTreppenFunktion = nameTreppenFunktion;
    
    initialize();
  }

  /**
   * Methode legt eine Arbeitskopie der akutellen Daten an
   * @param aktuell aktuelle Daten
   * @return Arbeitskopie
   */
  private TreppenFunktion erzeugeKopie(TreppenFunktion aktuell)
  {
  	TreppenFunktion kopie = new TreppenFunktion();

  	if (aktuell != null)
  	{
	  	kopie.setBezeichnung(aktuell.getBezeichnung());
	  	
	  	Vector<Stufe> stufen = aktuell.getStufen();
	  	for (Stufe s : stufen)
	  		kopie.addStufe(s.getVon(), s.getBis(), s.getWvzInhalt());
  	}
  	
	  return kopie;
  }

  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    
    _composite = new Composite(this, SWT.NONE);
    _composite.setLayout(new GridLayout(1, false));
    _composite.setLayoutData(gridData1);
 
//    erzeugeKommentar();
    
    //Initialisierungen der Arrays
    
    List<TableModel[]> inputList = new ArrayList<TableModel[]>();
    
    erzeugeGruppeRegeln();
    
    Table table = _tvAnzeige.getTable();
    _tableModel = createModel1(table);
    
    if ( _tableModel != null )
      _tvAnzeige.setInput(_tableModel); 
    
    createButtons();
    
    updateData(_aktuelleParameter);
  }

  /**
   * Methode blendet einen Kommentar zu dem Parametersatz ein
   */
  private void erzeugeKommentar ()
  {
  	String kommentar = "Ausfallbehandlung: Ausschaltregel\n\n" + 
    "Wenn Anzeigeinhalt = X\n" +
    "und Inhalt X an dieser Anzeige nicht darstellbar ist\n" +
    "dann alle Vorgnger und Nachfolger in Querrichtung vom WZG Typ A nach Y mit Prioritt Z schalten.\n\n";
  	
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    
    Group groupKommentar = new Group (_composite, SWT.NONE);
    groupKommentar.setLayout(new FillLayout());
    groupKommentar.setLayoutData(gridData1);

//    groupKommentar.setText("Erklrung");
    
    Label text = new Label (groupKommentar, SWT.NONE);
    text.setText(kommentar);
  }

  /**
   * Methode bernimmt die Daten und zeigt diese in der Tabelle an
   */
  private void updateData (TreppenFunktion parameter)
  {
  	int anzahlStufen = parameter.getStufen().size();
  	
  	erzeugeTabellenModell (anzahlStufen);
  	
  	for (int j=0; j<anzahlStufen; ++j)
  	{
      TableModel tm = _tableModel [j];    	
    	Stufe aktuell = parameter.getStufen().get(j);
    	
    	/*
    	 * Lschen
    	 */
    	tm.setSelectedLoeschen("false");
    	
    	/*
    	 * Von
    	 */
    	tm.setSelectedVon ("" + aktuell.getVon());

    	/*
    	 * Bis
    	 */
    	tm.setSelectedBis ("" + aktuell.getBis());

    	/*
    	 * WVZ Inhalt
    	 */
    	
  	  Label imX = (Label) tm.getWvzInhalt().getChildren()[0];
  	  WVZInhalt x = aktuell.getWvzInhalt();
  	  
  	  Tools.setzeBildInhalte(imX, x, _anzeigeTyp);
    }
  }

  /**
   * Methode erzeugt ein Tabellenmodell einer bestimmten Gre. Dabei werden bereits vorhandene
   * Composite in der Tabelle weiter verwendet
   * @param anzahlParameter Gre des erzeugten Tabellenmodells
   */
  private void erzeugeTabellenModell (int anzahlParameter)
  {
  	/*
  	 * Hat sich die Gre gendert
  	 */
  	if (_tableModel.length != anzahlParameter)
  	{
  		/*
  		 * Tabelle erweitern
  		 */
  		if (_tableModel.length < anzahlParameter)
  		{
  		  Vector<TableModel> v = new Vector<TableModel>();
  		  
  		  for (int i=0; i<_tableModel.length; ++i)
  		  	v.add(_tableModel [i]);
  		  
  		  for (int i=_tableModel.length; i<anzahlParameter; ++i)
  		  	v.add (erzeugeZeileInTabelle(_tvAnzeige.getTable()));
  		  
  		  _tableModel = new TableModel [anzahlParameter];
  		  for (int i=0; i<v.size(); ++i)
  		  	_tableModel [i] = v.get(i);
  		}
  		/*
  		 * Tabelle reduzieren
  		 */
  		else if (_tableModel.length > anzahlParameter)
  		{
  			/*
  			 * Nicht mehr bentigte Elemente lschen
  			 */
  			
  			for (int i = _tableModel.length - 1; i > anzahlParameter - 1; i--)
  			{
  				_tableModel [i].getWvzInhalt().dispose();
  				_tableModel [i].getBis().dispose();
  				_tableModel [i].getVon().dispose();
  				
  				_tableModel [i].getLoeschen().dispose();
  			}
  				
  		  Vector<TableModel> v = new Vector<TableModel>();
  		  for (int i=0; i<anzahlParameter; ++i)
  		  	v.add(_tableModel [i]);

  		  _tableModel = new TableModel [anzahlParameter];
  		  for (int i=0; i<v.size(); ++i)
  		  	_tableModel [i] = v.get(i);
  		}
  		
  	  _tvAnzeige.setInput(_tableModel);
  	  _tvAnzeige.getTable().layout(true);
  	}
  }
  
  /**
   * Methode erzeugt die Gruppe mit den Regeln
   */
  private void erzeugeGruppeRegeln()
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.marginHeight = 5;
    fillLayout1.marginWidth = 5;
    fillLayout1.type = org.eclipse.swt.SWT.HORIZONTAL;
    
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    
    _groupRegeln = new Group(_composite, SWT.NONE);
    _groupRegeln.setText("Stufen");
    
    _groupRegeln.setLayout(fillLayout1);
    _groupRegeln.setLayoutData(gridData2);

    erzeugeTabelle();
  }

  /**
   * Methode erzeugt das Composite mit den Buttons
   */
  private void createButtons()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    
    _compositeButtons = new Composite( _composite, SWT.NONE );
    _compositeButtons.setLayoutData( gridData3 );
    _compositeButtons.setLayout( rowLayout );

    _buttonHinzufuegen = new Button( _compositeButtons, SWT.NONE );
    _buttonHinzufuegen.setText( "Stufe hinzufgen" );
    _buttonHinzufuegen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionRegelHinzufuegen();
      }
    } );

    _buttonLoeschen = new Button( _compositeButtons, SWT.NONE );
    _buttonLoeschen.setText( "Selektierte Stufen loeschen" );
    _buttonLoeschen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionRegelLoeschen();
      }

    } );
    _buttonLoeschen.setEnabled(false);

  }

  /**
   * Methode prft ob Regeln zum Lschen ausgewhlt sind
   */
  private void pruefeObLoeschenAngewaehlt ()
  {
  	boolean angewaehlt = false;
  	
		for (TableModel tm : _tableModel)
			if (tm.getSelectedLoeschen().equals("true"))
				angewaehlt = true;
		
		_buttonLoeschen.setEnabled(angewaehlt);
  }
  
  /**
   * Methode zum Lschen der Regeln
   */
	private void aktionRegelLoeschen()
  {
		Vector<Stufe> stufen = leseAktuelleEinstellungen().getStufen();
		
    Vector<Stufe> loeschen = new Vector<Stufe>();
    
    for (int i=0; i<_tableModel.length; ++i)
    	if (!_tableModel [i].getSelectedLoeschen().equals("false"))
    		loeschen.add(stufen.get(i));

    for (Stufe r : loeschen)
    {
    	stufen.remove(r);
    }
    
    _aktuelleParameter.setStufen(stufen);
    
    updateData(_aktuelleParameter);
  }

	/**
	 * Aktion Regel hinzufgen
	 */
	private void aktionRegelHinzufuegen()
  {
		Vector<Stufe> stufen = leseAktuelleEinstellungen().getStufen();
    _aktuelleParameter.setStufen(stufen);

    VerwaltungWvzInhalte verw = VerwaltungWvzInhalte.getInstance();
    
		WVZInhalt dunkel = verw.getWvzInhalt(AnzeigeElementTyp.WZG_C, AnzeigeInhaltTyp.DUNKEL.getBildInhalt());
		
		_aktuelleParameter.addStufe (0, 0, dunkel);
		
		updateData(_aktuelleParameter);
  }

	/**
	 * Methode erzeugt das Composite mit der Tabelle
	 */
  private void erzeugeTabelle ()
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.type = org.eclipse.swt.SWT.VERTICAL;
    
    _compositeTabelle = new Composite(_groupRegeln, SWT.None);
    _compositeTabelle.setLayout(fillLayout1);
    
    _tvAnzeige = new TableViewer(_compositeTabelle, SWT.HIDE_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvAnzeige.setContentProvider(new TableContentProvider());

    TableViewerColumn column0 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column0.getColumn().setWidth(70);
    column0.getColumn().setText("Lschen");
    column0.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column1 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column1.getColumn().setWidth(100);
    column1.getColumn().setText("Von");
    column1.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column2 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column2.getColumn().setWidth(100);
    column2.getColumn().setText("Bis");
    column2.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column3 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column3.getColumn().setWidth(80);
    column3.getColumn().setText("WZG Inhalt");
    column3.setLabelProvider(new MyCellLabelProvider());

    // resize the row height using a MeasureItem listener
    
    Listener listener = new Listener() {
      @Override
      public void handleEvent(Event event)
      {
        switch (event.type)
        {
          case SWT.MeasureItem:
            event.height = 40;
            break;

          default:
        }

      }
    };
    
    _tvAnzeige.getTable().addListener(SWT.MeasureItem, listener);
    _tvAnzeige.getTable().setLinesVisible(true);
    _tvAnzeige.getTable().setHeaderVisible(true);
  }

  /**
   * Klasse realsiert den TableContentProvider fr die Tabelle
   * @author ObertM
   */
  private class TableContentProvider implements IStructuredContentProvider, Observer
  {
  	/**
  	 * Tabelviewer
  	 */
    private TableViewer tableViewer;

    /**
     * Methode liefert die Elemente der Tabelle
     */
    public Object[] getElements(Object inputElement)
    {
			Object[] re = null;

			if (inputElement instanceof TableModel [])
			{
        re = (TableModel[]) inputElement;
			}
			
			return re;
    }

    /**
     * Dispose
     */
    public void dispose()
    {
    }

    /**
     * Methode wird aufgerufen wenn sich der Inhalt der Tabelle gendert hat
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
      tableViewer = (TableViewer)viewer;

      if (oldInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;

        if (tms != null)
        {
          for (TableModel tm : tms)
            tm.deleteObserver(this);
        }
      }
      
      if (newInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;
        if (tms != null)
        {
          for (TableModel tm : tms)
            tm.addObserver(this);
        }
      }
    }

    @Override
    public void update(Observable arg0, Object arg1)
    {
      if (tableViewer != null)
      {
        tableViewer.refresh();
      }
    }
  }

  /**
   * Erzeugt ein TabellenModell fr den Inhalt der Tabellen
   * @param table
   * @return TabellenModell
   */
  private TableModel[] createModel1 (Table table)
  {
    int tabLength = getAnzahlParameter();
    int elIndex   = 0;
    
    TableModel[] elements = new TableModel [tabLength];
    
    if (elements.length == 0)
    	return elements;

    for (int x=0; x<getAnzahlParameter(); ++x)
    {
	    elements[elIndex] = erzeugeZeileInTabelle(table);
	    elIndex++;
    }
    
    return elements;
  }

  /**
   * Methode erzeugt ein Tabellenmodell fr eine Zeile in der Tabelle
   * @param table Tabelle 
   * @return Tabellenmodell fr eine Zeile
   */
  private TableModel erzeugeZeileInTabelle (Table table)
  {
    /*
     * Layout festlegen
     */
  	
    GridData gd0 = new GridData();
    gd0.horizontalAlignment = GridData.CENTER;
    gd0.verticalAlignment = GridData.CENTER;
    gd0.grabExcessHorizontalSpace = true;
    gd0.grabExcessVerticalSpace = true;

    GridData gd1 = new GridData();
    gd1.horizontalAlignment = GridData.CENTER;
    gd1.verticalAlignment = GridData.CENTER;
    gd1.grabExcessHorizontalSpace = true;
    gd1.grabExcessVerticalSpace = true;

    GridData gd2 = new GridData();
    gd2.horizontalAlignment = GridData.CENTER;
    gd2.verticalAlignment = GridData.CENTER;
    gd2.grabExcessHorizontalSpace = true;
    gd2.grabExcessVerticalSpace = true;

    GridData gd3 = new GridData();
    gd3.horizontalAlignment = GridData.CENTER;
    gd3.verticalAlignment = GridData.CENTER;
    gd3.grabExcessHorizontalSpace = true;
    gd3.grabExcessVerticalSpace = true;

    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;

    /*
     * Button Lschen
     */
    Composite comp0 = new Composite( table, SWT.NONE );
    comp0.setBackground( table.getBackground() );
    comp0.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp0.setLayoutData( gd1 );
    comp0.setLayout( new GridLayout() );
    Button b = new Button( comp0, SWT.CHECK );
    
    b.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				pruefeObLoeschenAngewaehlt();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}
		});
    
    /*
     * Spinner Von
     */
    Composite comp1 = new Composite( table, SWT.NONE );
    comp1.setBackground( table.getBackground() );
    comp1.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp1.setLayoutData( gd1 );
    comp1.setLayout( new GridLayout(2, false) );
    
    Spinner spinner1 = new Spinner( comp1, SWT.BORDER );

    spinner1.setMinimum(0);
    spinner1.setMaximum(_maxEntfernung);
    spinner1.setDigits(0);
      
    String toolTip1 = "Unterer Grenzwert fr diese Stufe (Entfernungsangabe). Zulssig von  " + spinner1.getMinimum() 
                     + "m bis " + spinner1.getMaximum() + "m";
     
    comp1.setToolTipText(toolTip1);
    spinner1.setToolTipText(toolTip1);
    spinner1.setLayoutData( gdTableItem );

    
    Label l1 = new Label(comp1, SWT.NONE);
    l1.setText("m");
    
    /*
     * Spinner Bis
     */
    Composite comp2 = new Composite( table, SWT.NONE );
    comp2.setBackground( table.getBackground() );
    comp2.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp2.setLayoutData( gd2 );
    comp2.setLayout( new GridLayout(2, false) );
    
    Spinner spinner2 = new Spinner( comp2, SWT.BORDER );

    spinner2.setMinimum(0);
    spinner2.setMaximum(_maxEntfernung);
    spinner2.setDigits(0);
      
    String toolTip2 = "Oberer Grenzwert fr diese Stufe (Entfernungsangabe). Zulssig von  " + spinner2.getMinimum() 
                     + "m bis " + spinner2.getMaximum() + "m";
     
    comp2.setToolTipText(toolTip2);
    spinner2.setToolTipText(toolTip2);
    spinner2.setLayoutData( gdTableItem );

    Label l2 = new Label(comp2, SWT.NONE);
    l2.setText("m");

    /*
     * WVZ Inhalt
     */
    Composite comp3 = new Composite( table, SWT.NONE );
    comp3.setBackground( table.getBackground() );
    comp3.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp3.setLayoutData( gd3 );
    comp3.setLayout( new FillLayout() );
    
    Label lbImageX = new Label( comp3, SWT.CENTER );
    lbImageX.addMouseListener( new MouseAdapter()
    {
			@Override
      public void mouseDown( MouseEvent e )
      {
				Tools.waehleZuSchaltendesBild(e, _anzeigeTyp);
      }         
    });

    return new TableModel ( comp0, comp1, comp2, comp3);
  }
  
  /**
   * Methode liefert die Anzahl der Parameter
   * @return Anzahl der Parameter
   */
  public int getAnzahlParameter()
  {
    return _aktuelleParameter.getStufen().size();
  }
  
  /**
   * Methode liefert den Tabelviewer
   * @return
   */
  public TableViewer getTvAnzeige()
  {
    return _tvAnzeige;
  }

  /**
   * Renderer fr die Tabelle
   * @author ObertM
   */
  private class MyCellLabelProvider extends CellLabelProvider 
  {
    @Override
    public void update(ViewerCell cell)
    {
      TableItem item = (TableItem)cell.getItem();

//      if (item.getData("EDITOR") != null)
//      {
//        TableEditor editor = (TableEditor)item.getData("EDITOR");
//        editor.dispose();
//      }

      int cnt = cell.getColumnIndex();

      TableModel model = (TableModel)item.getData();
      Composite comp = null;
      switch (cnt)
      {
        case 0:
          comp = model.getLoeschen();
          break;
          
        case 1:
          comp = model.getVon();
          break;
          
        case 2:
          comp = model.getBis();
          break;
          
        case 3:
          comp = model.getWvzInhalt();
          break;
          
        default:
      }
      
      TableEditor editor = new TableEditor(item.getParent());
      editor.grabHorizontal = true;
      editor.setEditor(comp, item, cnt);

    }
  }

  /**
   * Methode liefert die eingestellten Parameter
   * @return eingestellte Parameter
   */
	public TreppenFunktion getParameter()
  {
 		aktionRegelLoeschen();
 		return leseAktuelleEinstellungen();
  }
		
	private TreppenFunktion leseAktuelleEinstellungen ()
	{
		TreppenFunktion regeln = new TreppenFunktion(_nameTreppenFunktion);
		
		for (int j=0; j<_tableModel.length; ++j)
		{
      TableModel tm = _tableModel [j];
      
      /*
       * Von
       */
      int von = tm.getSelectedVon();

      /*
       * Bis
       */
      int bis = tm.getSelectedBis();

      /*
       * WVZ Inhalt
       */
      VerwaltungWvzInhalte verw = VerwaltungWvzInhalte.getInstance();
      WVZInhalt x = verw.getWvzInhalt(AnzeigeElementTyp.WZG_C, tm.getSelectedWVZInhalt());

      regeln.addStufe(von, bis, x);
		}

		return regeln;
  }

	/**
	 * Methode bernimmt die Daten eines anderen Folders
	 * @param tvAnzeige Tableviewer der zu bernhmenden Tabelle
	 */
	public void setTvAnzeige (TableViewer tvAnzeige)
	{
    TableModel[] modelSet = (TableModel[]) tvAnzeige.getInput();

    Vector<Stufe> stufen = new Vector<Stufe>();

    for (int i=0; i<modelSet.length; ++i)
    {
      /*
       * Von
       */
      int von = modelSet [i].getSelectedVon();

      /*
       * Bis
       */
      int bis = modelSet [i].getSelectedBis();

      /*
       * WVZ Inhalt
       */
      VerwaltungWvzInhalte verw = VerwaltungWvzInhalte.getInstance();
      WVZInhalt x = verw.getWvzInhalt(AnzeigeElementTyp.WZG_C, modelSet [i].getSelectedWVZInhalt());
      
  	  stufen.add (new Stufe (von, bis, x));
    }
    
    _aktuelleParameter.setStufen(stufen);
    updateData(_aktuelleParameter);
  }
}
