package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.entfernungsangaben;

import java.util.Observable;
import java.util.Observer;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;

/**
 * Tabellen Modell fr die Parameter Entfernungsangaben
 * @author ObertM
 *
 */
public class TableModel extends Observable implements Observer
{
	/**
	 * Merker zum Lschen der Zeile
	 */
	private Composite _loeschen;
	
	/**
	 * Von
	 */
  private Composite _von;

	/**
	 * Bis
	 */
  private Composite _bis;

	/**
	 * WVZ-Inhalt der Stufe
	 */
  private Composite _wvzInhalt;

  /**
   * Konstruktor der Klasse
   * @param loeschen Lschen
   * @param von Von
   * @param bis Bis
   * @param wvzInhalt WVZ Inhalt
   */
  public TableModel(Composite loeschen, Composite von, Composite bis, Composite wvzInhalt)
  {
  	_loeschen = loeschen;
  	
  	_von = von;
  	
  	_bis = bis;
  	
  	_wvzInhalt = wvzInhalt;
  }

  /**
   * Methode liefert den Wert des Composite Lschen
   * @return Wert des Composite
   */
  public String getSelectedLoeschen()
  {
    return getSelectedWert(_loeschen);    
  }

  /**
   * Methode setzt den Wert des Composite Lschen
   * @param selWert zu setzender Wert
   */
  public void setSelectedLoeschen(String selWert)
  {
    setSelectedWert(_loeschen, selWert, null);
  }

  /**
   * Methode liefert den Wert des Composite Von
   * @return Wert des Composite
   */
  public int getSelectedVon ()
  {
    Control control = _von.getChildren()[0];
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      return sp.getSelection();
    }
    else
      return 0;
  }

  /**
   * Methode setzt den Wert des Composite Von
   * @param selWert zu setzender Wert
   */
  public void setSelectedVon (String selWert)
  {
    setSelectedWert(_von, selWert, null);
  }

  /**
   * Methode liefert den Wert des Composite Bis
   * @return Wert des Composite
   */
  public int getSelectedBis ()
  {
    Control control = _bis.getChildren()[0];
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      return sp.getSelection();
    }
    else
      return 0;
  }

  /**
   * Methode setzt den Wert des Composite Bis
   * @param selWert zu setzender Wert
   */
  public void setSelectedBis (String selWert)
  {
    setSelectedWert(_bis, selWert, null);
  }
  
  /**
   * Methode liefert den Wert des Composite WVZInhalt
   * @return Wert des Composite
   */
  public String getSelectedWVZInhalt ()
  {
    return getSelectedWert(_wvzInhalt);    
  }

  /**
   * Methode setzt den Wert des Composite WVZInhalt
   * @param selWert zu setzender Wert
   */
  public void setSelectedWVZInhalt (String selWert)
  {
    setSelectedWert(_wvzInhalt, selWert, null);
  }

  /**
   * Methode liefert den Wert eines Comppsite
   * @param c Composite
   * @return Wert des Composite
   */
  public String getSelectedWert(Composite c)
  {
    Control control = c.getChildren()[0];
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      int spSel = sp.getSelection();
      return String.valueOf(spSel);
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      String cbSel = cb.getItem( cb.getSelectionIndex() );
      return cbSel;
    }
    else if ( control instanceof Button )
    {
      if ( (control.getStyle() & SWT.CHECK) != 0 )
      {
        Button ch = (Button)control;
        boolean chSel = ch.getSelection();
        return String.valueOf(chSel);
      }
      else
        return "";
    }
    else if ( control instanceof Label )
    {
      Label im = (Label)control;
      String imSel = (String)im.getData();
      return imSel;
    }
    else
      return "";
  }

  /**
   * Methode setzt den Wert eines Composite
   * @param c Composite
   * @param selWert Text auf den das Compisite gesetzt werden soll
   * @param selImage Image auf den das Compoiste gesetzt werden soll
   */
  public void setSelectedWert(Composite c, String selWert, Image selImage)
  {
    Control control = c.getChildren()[0];
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      try
      {
        sp.setSelection(Integer.valueOf( selWert ));
      }
      catch (NumberFormatException e)
      {
        e.printStackTrace();
      }
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      for (int k = 0; k < cb.getItemCount(); k++)
      {
        if (selWert.equals(cb.getItem( k )))
          cb.select(k);      
      }
    }
    else if ( control instanceof Button )
    {
      if ( (control.getStyle() & SWT.CHECK) != 0 )
      {
        Button ch = (Button)control;
        ch.setSelection( Boolean.valueOf( selWert ) );
      }
    }
    else if ( control instanceof Label )
    {
        Label im = (Label)control;
        im.setData( selWert );
        im.setImage( selImage ); 
    }
  }

  @Override
  public void update(Observable o, Object arg)
  {
    fireEvent();
  }

  public void fireEvent()
  {
    setChanged();
    notifyObservers();
  }

	/**
   * Methode liefert loeschen zurck
   * @return loeschen
   */
  public Composite getLoeschen()
  {
  	return _loeschen;
  }

	/**
   * Methode setzt die Variable loeschen
   * @param loeschen zu bernehmender Wert fr loeschen
   */
  public void setLoeschen(Composite loeschen)
  {
  	_loeschen = loeschen;
  }

	/**
   * Methode liefert von zurck
   * @return von
   */
  public Composite getVon()
  {
  	return _von;
  }

	/**
   * Methode setzt die Variable von
   * @param von zu bernehmender Wert fr von
   */
  public void setVon(Composite von)
  {
  	_von = von;
  }

	/**
   * Methode liefert bis zurck
   * @return bis
   */
  public Composite getBis()
  {
  	return _bis;
  }

	/**
   * Methode setzt die Variable bis
   * @param bis zu bernehmender Wert fr bis
   */
  public void setBis(Composite bis)
  {
  	_bis = bis;
  }

	/**
   * Methode liefert wvzInhalt zurck
   * @return wvzInhalt
   */
  public Composite getWvzInhalt()
  {
  	return _wvzInhalt;
  }

	/**
   * Methode setzt die Variable wvzInhalt
   * @param wvzInhalt zu bernehmender Wert fr wvzInhalt
   */
  public void setWvzInhalt(Composite wvzInhalt)
  {
  	_wvzInhalt = wvzInhalt;
  }

}
