package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.lueckenfuellen;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import javax.xml.stream.events.EndElement;

import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeElementFigure;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeInhaltAuswahlDialog;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeInhaltTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.VerwaltungWvzInhalte;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeElementTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.ProjektSpezifischeEinstellungen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.ProjektSpezifischeEinstellungen.SBA;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Darstellung des Fensters zur Versorgung der Parameter
 */
public class CompositeParameterLueckenFuellen extends Composite
{
	private final int _anzBilder = 7;
	
	/**
	 * Aktuelles Composite
	 */
  private Composite _composite;

  /**
   * Composite der Tabelle
   */
  private Composite _compositeTabelle;

  /**
   * Don't care
   */
	private WVZInhalt _dontCare = VerwaltungWvzInhalte.getInstance().getWvzInhaltDontCare();

  /**
   * Gruppe mit den Regeln
   */
  private Group _groupRegeln;

  /**
   * Tabelviewer der Tabelle
   */
  private TableViewer _tvAnzeige;

  /**
   * Aktuelle Parameter
   */
  private ListeMitRegeln _aktuelleParameter;

  /**
   * Composite fr die Buttons
   */
	private Composite _compositeButtons;

	/**
	 * Button Hinzufgen
	 */
	private Button _buttonHinzufuegen;

	/**
	 * Button Lschen
	 */
	private Button _buttonLoeschen;

	/**
	 * Tabellenmodell
	 */
	private TableModel[] _tableModel;
	
	/**
	 * Anzeigetyp fr den die Parameter versorgt werden
	 */
	private AnzeigeElementTyp _anzeigeTyp;

	private Button _buttonAlleLoeschen;

	private Button _buttonDefault;

  /**
   * Konstruktor der Klasse
   * @param parent Parent Composite
   * @param style Stil des Fenstern
   * @param aktuell Aktuelle Parameter
   * @param pidAnzeigeTyp Pid des Anzeigetyps
   */
  CompositeParameterLueckenFuellen (Composite parent, int style, ListeMitRegeln aktuell, String pidAnzeigeTyp)
  {
    super (parent, style);
    
    this._aktuelleParameter = erzeugeKopie(aktuell);
    
    _anzeigeTyp = AnzeigeElementTyp.getTypByPid(pidAnzeigeTyp);
    
    initialize();
  }

  /**
   * Methode legt eine Arbeitskopie der akutellen Daten an
   * @param aktuell aktuelle Daten
   * @return Arbeitskopie
   */
  private ListeMitRegeln erzeugeKopie(ListeMitRegeln aktuell)
  {
	  ListeMitRegeln kopie = new ListeMitRegeln();
	  
	  Vector<RegelLueckenFuellen> parameter = aktuell.getParameter();
	  for (RegelLueckenFuellen regel : parameter)
	  {
	  	List<WVZInhalt> xyz = new ArrayList<WVZInhalt>();
	  	for (WVZInhalt inhalt : regel.getAnzeigenInhaltVorgaengerNachfolger())
	  		xyz.add(inhalt);

	  	List<WVZInhalt> uvw = new ArrayList<WVZInhalt>();
	  	for (WVZInhalt inhalt : regel.getAnzeigenInhaltVorVorgaengerNachNachfolger())
	  		uvw.add(inhalt);

	  	kopie.addRegel(regel.getAnzeigenInhalt(),
	  			           xyz,
	  			           uvw,
	  			           regel.getAngeforderterAnzeigenInhalt(),
	  			           regel.getRichtung(),
	  			           regel.getSchaltgrundRegel());
	  }
	  
	  return kopie;
  }

  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    
    _composite = new Composite(this, SWT.NONE);
    _composite.setLayout(new GridLayout(1, false));
    _composite.setLayoutData(gridData1);
 
    erzeugeKommentar();
    
    //Initialisierungen der Arrays
    
    List<TableModel[]> inputList = new ArrayList<TableModel[]>();
    
    erzeugeGruppeRegeln();
    
    Table table = _tvAnzeige.getTable();
    _tableModel = createModel1(table);
    
    if ( _tableModel != null )
      _tvAnzeige.setInput(_tableModel); 
    
    createButtons();
    
    updateData(_aktuelleParameter);
  }

  /**
   * Methode blendet einen Kommentar zu dem Parametersatz ein
   */
  private void erzeugeKommentar ()
  {
  	String kommentar = "Lcken fllen:\n\n" + 
    "Wenn Anzeigeinhalt = \"X\"\n" +
    "und Inhalt des Vorgngers/Nachfolgers in Lngsrichtung in [\"X\", \"Y\", \"Z\"]\n" + 
    "und Inhalt des Vorgngers des Vorgngers / Nachfolger des Nachfolgers in Lngsrichtung in [\"U\", \"V\", \"W\"]\n" +
    "dann fr Vorgnger/Nachfolger in Lngsrichtung \"A\" anfordern.\n\n";
  	
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    
    Group groupKommentar = new Group (_composite, SWT.NONE);
    groupKommentar.setLayout(new FillLayout());
    groupKommentar.setLayoutData(gridData1);

//    groupKommentar.setText("Erklrung");
    
    Label text = new Label (groupKommentar, SWT.NONE);
    text.setText(kommentar);
  }

  /**
   * Methode bernimmt die Daten und zeigt diese in der Tabelle an
   */
  private void updateData (ListeMitRegeln parameter)
  {
  	int anzahlParameter = parameter.getParameter().size();
  	
  	erzeugeTabellenModell (anzahlParameter);

  	for (int j=0; j<anzahlParameter; ++j)
  	{
      TableModel tm = _tableModel [j];    	
    	RegelLueckenFuellen aktuell = parameter.getParameter().get(j);
    	
    	/*
    	 * Lschen
    	 */
    	tm.setSelectedLoeschen("false");
    	
    	/*
    	 * Schaltgrund
    	 */
    	
    	if (aktuell.getSchaltgrundRegel() == null)
    	{
    	  tm.setSelectedSchaltgrund("");
    	}
    	else
    	{
  	    tm.setSelectedSchaltgrund(aktuell.getSchaltgrundRegel().getName());
    	}
    	
    	/*
    	 * Richtung
    	 */
  	  tm.setSelectedRichtung(aktuell.getRichtung().getName());

  	  /*
  	   * X
  	   */
  	  Label imX = (Label) tm.getX().getChildren()[0];
  	  WVZInhalt x = aktuell.getAnzeigenInhalt();
  	  
  	  Tools.setzeBildInhalte(imX, x, _anzeigeTyp);
  	  
  	  /*
  	   * XYZ
  	   */

  	  Control[] childrenXYZ = tm.getXyz().getChildren();
  	  Label [] imXYZ = new Label [childrenXYZ.length];
  	  for (int i=0; i<childrenXYZ.length; ++i)
  	  	imXYZ [i] = (Label) childrenXYZ [i];
  	  
  	  List<WVZInhalt> xyz = aktuell.getAnzeigenInhaltVorgaengerNachfolger();
  	  for (int i=0; i<xyz.size(); ++i)
  	  	Tools.setzeBildInhalte(imXYZ[i], xyz.get(i), _anzeigeTyp);
  	  
  	  for (int i=xyz.size(); i<_anzBilder; ++i)
  	  	Tools.setzeBildInhalte(imXYZ[i], _dontCare, _anzeigeTyp);

  	  /*
  	   * UVW
  	   */
  
  	  Control[] childrenUVW = tm.getUvw().getChildren();
  	  Label [] imUVW = new Label [childrenUVW.length];
  	  for (int i=0; i<childrenUVW.length; ++i)
  	  	imUVW [i] = (Label) childrenUVW [i];
  	  
  	  List<WVZInhalt> uvw = aktuell.getAnzeigenInhaltVorVorgaengerNachNachfolger();
  	  for (int i=0; i<uvw.size(); ++i)
  	  	Tools.setzeBildInhalte(imUVW[i], uvw.get(i), _anzeigeTyp);
  	  for (int i=uvw.size(); i<_anzBilder; ++i)
  	  	Tools.setzeBildInhalte(imUVW[i], _dontCare, _anzeigeTyp);

  	  /*
  	   * A
  	   */
  	  Label imA = (Label) tm.getA().getChildren()[0];
  	  WVZInhalt a = aktuell.getAngeforderterAnzeigenInhalt();
  	  
  	  Tools.setzeBildInhalte(imA, a, _anzeigeTyp);
    }

  	_buttonAlleLoeschen.setEnabled (anzahlParameter != 0);
  	pruefeObLoeschenAngewaehlt();
  }

  /**
   * Methode erzeugt ein Tabellenmodell einer bestimmten Gre. Dabei werden bereits vorhandene
   * Composite in der Tabelle weiter verwendet
   * @param anzahlParameter Gre des erzeugten Tabellenmodells
   */
  private void erzeugeTabellenModell (int anzahlParameter)
  {
  	/*
  	 * Hat sich die Gre gendert
  	 */
  	if (_tableModel.length != anzahlParameter)
  	{
  		/*
  		 * Tabelle erweitern
  		 */
  		if (_tableModel.length < anzahlParameter)
  		{
  		  Vector<TableModel> v = new Vector<TableModel>();
  		  
  		  for (int i=0; i<_tableModel.length; ++i)
  		  	v.add(_tableModel [i]);
  		  
  		  for (int i=_tableModel.length; i<anzahlParameter; ++i)
  		  	v.add (erzeugeZeileInTabelle(_tvAnzeige.getTable()));
  		  
  		  _tableModel = new TableModel [anzahlParameter];
  		  for (int i=0; i<v.size(); ++i)
  		  	_tableModel [i] = v.get(i);
  		}
  		/*
  		 * Tabelle reduzieren
  		 */
  		else if (_tableModel.length > anzahlParameter)
  		{
  			/*
  			 * Nicht mehr bentigte Elemente lschen
  			 */
  			
  			for (int i = _tableModel.length - 1; i > anzahlParameter - 1; i--)
  			{
  				_tableModel [i].getA().dispose();
  				_tableModel [i].getUvw().dispose();
  				_tableModel [i].getXyz().dispose();
  				_tableModel [i].getX().dispose();
  				
  				_tableModel [i].getRichtung().dispose();
  				_tableModel [i].getSchaltgrund().dispose();
  				_tableModel [i].getLoeschen().dispose();
  			}
  				
  		  Vector<TableModel> v = new Vector<TableModel>();
  		  for (int i=0; i<anzahlParameter; ++i)
  		  	v.add(_tableModel [i]);

  		  _tableModel = new TableModel [anzahlParameter];
  		  for (int i=0; i<v.size(); ++i)
  		  	_tableModel [i] = v.get(i);
  		}
  		
  	  _tvAnzeige.setInput(_tableModel);
  	  _tvAnzeige.getTable().layout(true);
  	}
  }
  
  /**
   * Methode erzeugt die Gruppe mit den Regeln
   */
  private void erzeugeGruppeRegeln()
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.marginHeight = 5;
    fillLayout1.marginWidth = 5;
    fillLayout1.type = org.eclipse.swt.SWT.HORIZONTAL;
    
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    
    _groupRegeln = new Group(_composite, SWT.NONE);
    _groupRegeln.setText("Regeln");
    
    _groupRegeln.setLayout(fillLayout1);
    _groupRegeln.setLayoutData(gridData2);

    erzeugeTabelle();
  }

  /**
   * Methode erzeugt das Composite mit den Buttons
   */
  private void createButtons()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    
    _compositeButtons = new Composite( _composite, SWT.NONE );
    _compositeButtons.setLayoutData( gridData3 );
    _compositeButtons.setLayout( rowLayout );

    _buttonHinzufuegen = new Button( _compositeButtons, SWT.NONE );
    _buttonHinzufuegen.setText( "Regel hinzufgen" );
    _buttonHinzufuegen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionRegelHinzufuegen();
      }
    } );

    _buttonLoeschen = new Button( _compositeButtons, SWT.NONE );
    _buttonLoeschen.setText( "Selektierte Regeln loeschen" );
    _buttonLoeschen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionRegelLoeschen();
      }

    } );
    _buttonLoeschen.setEnabled(false);

    _buttonAlleLoeschen = new Button( _compositeButtons, SWT.NONE );
    _buttonAlleLoeschen.setText( "Alle Regeln loeschen" );
    _buttonAlleLoeschen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        aktionAlleRegelLoeschen();
      }
    } );
    
    if (_anzeigeTyp == AnzeigeElementTyp.WZG_A)
    {
	    _buttonDefault = new Button( _compositeButtons, SWT.NONE );
	    _buttonDefault.setText( "Defaultwerte" );
	    _buttonDefault.addMouseListener( new MouseAdapter()
	    {
	      @Override
	      public void mouseDown( MouseEvent e )
	      {
	        aktionDefault ();
	      }
	    } );
    }
  }

	/**
   * Methode zum Lschen aller Meldungen
   */
  protected void aktionAlleRegelLoeschen()
  {
		Vector<RegelLueckenFuellen> regeln = new Vector<RegelLueckenFuellen>();
    _aktuelleParameter.setParameter(regeln);
    
    updateData(_aktuelleParameter);
  }
  
  /**
   * Methode generiert automatisch die Defaultwerte
   */
  protected void aktionDefault()
  {
		Vector<RegelLueckenFuellen> regeln = new Vector<RegelLueckenFuellen>();

	  DefaultWerte dw = new DefaultWerte ();
	  regeln = dw.getDefaultWerte(_anzBilder);

//  	if (ProjektSpezifischeEinstellungen.getInstance().getSba() == SBA.A14)
//  	{
//  	  DefaultWerteA14 dw = new DefaultWerteA14 ();
//  	  regeln = dw.getDefaultWerte(_anzBilder);
//  	}
//
//  	if (ProjektSpezifischeEinstellungen.getInstance().getSba() == SBA.B27)
//  	{
//  	  DefaultWerteB27 dw = new DefaultWerteB27 ();
//  	  regeln = dw.getDefaultWerte(_anzBilder);
//  	}
//
//  	if (ProjektSpezifischeEinstellungen.getInstance().getSba() == SBA.A8)
//  	{
//  	  DefaultWerteA8 dw = new DefaultWerteA8 ();
//  	  regeln = dw.getDefaultWerte(_anzBilder);
//  	}

    _aktuelleParameter.setParameter(regeln);
    updateData(_aktuelleParameter);
  }

  /**
   * Methode prft ob Regeln zum Lschen ausgewhlt sind
   */
  private void pruefeObLoeschenAngewaehlt ()
  {
  	boolean angewaehlt = false;
  	
		for (TableModel tm : _tableModel)
			if (tm.getSelectedLoeschen().equals("true"))
				angewaehlt = true;
		
		_buttonLoeschen.setEnabled(angewaehlt);
  }
  
  /**
   * Methode zum Lschen der Regeln
   */
	private void aktionRegelLoeschen()
  {
		Vector<RegelLueckenFuellen> regeln = leseAktuelleEinstellungen().getParameter();
		
    Vector<RegelLueckenFuellen> loeschen = new Vector<RegelLueckenFuellen>();
    
    for (int i=0; i<_tableModel.length; ++i)
    	if (!_tableModel [i].getSelectedLoeschen().equals("false"))
    		loeschen.add(regeln.get(i));

    for (RegelLueckenFuellen r : loeschen)
    {
    	regeln.remove(r);
    }
    
    _aktuelleParameter.setParameter(regeln);
    
    updateData(_aktuelleParameter);
  }

	/**
	 * Aktion Regel hinzufgen
	 */
	private void aktionRegelHinzufuegen()
  {
		Vector<RegelLueckenFuellen> regeln = leseAktuelleEinstellungen().getParameter();
    _aktuelleParameter.setParameter(regeln);

		WVZInhalt inhalt = VerwaltungWvzInhalte.getInstance().getDefaultWvzInhalt(_anzeigeTyp);

		Aspect a = VerwaltungParaLueckenFuellen.getInstance().getSchaltgrundAllgemein();
		
		List<WVZInhalt> xyz = new ArrayList<WVZInhalt>();
		for (int i=0; i<_anzBilder; ++i)
			xyz.add(_dontCare);

		List<WVZInhalt> uvw = new ArrayList<WVZInhalt>();
		for (int i=0; i<_anzBilder; ++i)
			uvw.add(_dontCare);

		_aktuelleParameter.addRegel (inhalt, xyz, uvw, inhalt, ArbeitsRichtungVN.NACHFOLGER,  a);
		
		updateData(_aktuelleParameter);
  }

	/**
	 * Methode erzeugt das Composite mit der Tabelle
	 */
  private void erzeugeTabelle ()
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.type = org.eclipse.swt.SWT.VERTICAL;
    
    _compositeTabelle = new Composite(_groupRegeln, SWT.None);
    _compositeTabelle.setLayout(fillLayout1);
    
    _tvAnzeige = new TableViewer(_compositeTabelle, SWT.HIDE_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvAnzeige.setContentProvider(new TableContentProvider());

    TableViewerColumn column0 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column0.getColumn().setWidth(60);
    column0.getColumn().setText("Lschen");
    column0.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column1 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column1.getColumn().setWidth(200);
    column1.getColumn().setText("Schaltgrund");
    column1.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column2 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column2.getColumn().setWidth(100);
    column2.getColumn().setText("Richtung");
    column2.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column3 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column3.getColumn().setWidth(80);
    column3.getColumn().setText("X");
    column3.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column4 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column4.getColumn().setWidth(Tools.getMaxBreiteBild() * _anzBilder);
    column4.getColumn().setText("XYZ");
    column4.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column5 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column5.getColumn().setWidth(Tools.getMaxBreiteBild() * _anzBilder);
    column5.getColumn().setText("UVW");
    column5.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column6 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column6.getColumn().setWidth(80);
    column6.getColumn().setText("A");
    column6.setLabelProvider(new MyCellLabelProvider());

    // resize the row height using a MeasureItem listener
    Listener listener = new Listener() {
      @Override
      public void handleEvent(Event event)
      {
        switch (event.type)
        {
          case SWT.MeasureItem:
            event.height = Tools.getZeilenHoehe(_anzeigeTyp);
            break;

          default:
        }

      }
    };
    
    _tvAnzeige.getTable().addListener(SWT.MeasureItem, listener);
    _tvAnzeige.getTable().setLinesVisible(true);
    _tvAnzeige.getTable().setHeaderVisible(true);
  }

  /**
   * Klasse realsiert den TableContentProvider fr die Tabelle
   * @author ObertM
   */
  private class TableContentProvider implements IStructuredContentProvider, Observer
  {
  	/**
  	 * Tabelviewer
  	 */
    private TableViewer tableViewer;

    /**
     * Methode liefert die Elemente der Tabelle
     */
    public Object[] getElements(Object inputElement)
    {
			Object[] re = null;

			if (inputElement instanceof TableModel [])
			{
        re = (TableModel[]) inputElement;
			}
			
			return re;
    }

    /**
     * Dispose
     */
    public void dispose()
    {
    }

    /**
     * Methode wird aufgerufen wenn sich der Inhalt der Tabelle gendert hat
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
      tableViewer = (TableViewer)viewer;

      if (oldInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;

        if (tms != null)
        {
          for (TableModel tm : tms)
            tm.deleteObserver(this);
        }
      }
      
      if (newInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;
        if (tms != null)
        {
          for (TableModel tm : tms)
            tm.addObserver(this);
        }
      }
    }

    @Override
    public void update(Observable arg0, Object arg1)
    {
      if (tableViewer != null)
      {
        tableViewer.refresh();
      }
    }
  }

  /**
   * Erzeugt ein TabellenModell fr den Inhalt der Tabellen
   * @param table
   * @return TabellenModell
   */
  private TableModel[] createModel1 (Table table)
  {
    int tabLength = getAnzahlParameter();
    int elIndex   = 0;
    
    TableModel[] elements = new TableModel [tabLength];
    
    if (elements.length == 0)
    	return elements;

    for (int x=0; x<getAnzahlParameter(); ++x)
    {
	    elements[elIndex] = erzeugeZeileInTabelle(table);
	    elIndex++;
    }
    
    return elements;
  }

  /**
   * Methode erzeugt ein Tabellenmodell fr eine Zeile in der Tabelle
   * @param table Tabelle 
   * @return Tabellenmodell fr eine Zeile
   */
  private TableModel erzeugeZeileInTabelle (Table table)
  {
    /*
     * Layout festlegen
     */
  	
    GridData gd0 = new GridData();
    gd0.horizontalAlignment = GridData.CENTER;
    gd0.verticalAlignment = GridData.CENTER;
    gd0.grabExcessHorizontalSpace = true;
    gd0.grabExcessVerticalSpace = true;

    GridData gd1 = new GridData();
    gd1.horizontalAlignment = GridData.CENTER;
    gd1.verticalAlignment = GridData.CENTER;
    gd1.grabExcessHorizontalSpace = true;
    gd1.grabExcessVerticalSpace = true;

    GridData gd2 = new GridData();
    gd2.horizontalAlignment = GridData.CENTER;
    gd2.verticalAlignment = GridData.CENTER;
    gd2.grabExcessHorizontalSpace = true;
    gd2.grabExcessVerticalSpace = true;

    GridData gd3 = new GridData();
    gd3.horizontalAlignment = GridData.CENTER;
    gd3.verticalAlignment = GridData.CENTER;
    gd3.grabExcessHorizontalSpace = true;
    gd3.grabExcessVerticalSpace = true;

    GridData gd4 = new GridData();
    gd4.horizontalAlignment = GridData.CENTER;
    gd4.verticalAlignment = GridData.CENTER;
    gd4.grabExcessHorizontalSpace = true;
    gd4.grabExcessVerticalSpace = true;

    GridData gd5 = new GridData();
    gd5.horizontalAlignment = GridData.CENTER;
    gd5.verticalAlignment = GridData.CENTER;
    gd5.grabExcessHorizontalSpace = true;
    gd5.grabExcessVerticalSpace = true;

    GridData gd6 = new GridData();
    gd6.horizontalAlignment = GridData.CENTER;
    gd6.verticalAlignment = GridData.CENTER;
    gd6.grabExcessHorizontalSpace = true;
    gd6.grabExcessVerticalSpace = true;

    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;

    /*
     * Button Lschen
     */
    Composite comp0 = new Composite( table, SWT.NONE );
    comp0.setBackground( table.getBackground() );
    comp0.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp0.setLayoutData( gd0 );
    comp0.setLayout( new GridLayout() );
    Button b = new Button( comp0, SWT.CHECK );
    
    b.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				pruefeObLoeschenAngewaehlt();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}
		});
    
    /*
     * Combobox Schaltgrund    
     */
    Composite comp1 = new Composite( table, SWT.NONE );
    comp1.setBackground( table.getBackground() );
    comp1.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp1.setLayoutData( gd1 );
    comp1.setLayout( new GridLayout() );
    
    Combo combo1 = new Combo( comp1, SWT.BORDER );
    combo1.setLayoutData( gdTableItem );
    
    Vector<SystemObject> aspekte = Tools.getAspekte();
    String [] sa = new String [aspekte.size()];
    for (int i=0; i<aspekte.size(); ++i)
      sa [i] = aspekte.get(i).getName();
    
    combo1.setItems(sa);
    combo1.select(0);

    /*
     * Combobox Richtung    
     */
    Composite comp2 = new Composite( table, SWT.NONE );
    comp2.setBackground( table.getBackground() );
    comp2.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp2.setLayoutData( gd2 );
    comp2.setLayout( new GridLayout() );
    
    Combo combo2 = new Combo( comp2, SWT.BORDER );
    combo2.setLayoutData( gdTableItem );
    
    ArbeitsRichtungVN[] arvn = Richtung.ArbeitsRichtungVN.values();
    String[] r = new String [arvn.length];
    
    for (int i=0; i<arvn.length; ++i)
    	r [i] = arvn [i].getName();
    
    combo2.setItems(r);
    combo2.select(0);

    /*
     * X
     */
    Composite comp3 = new Composite( table, SWT.NONE );
    comp3.setBackground( table.getBackground() );
    comp3.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp3.setLayoutData( gd3 );
    comp3.setLayout( new FillLayout() );
    
    Label lbImageX = new Label( comp3, SWT.CENTER );
    lbImageX.addMouseListener( new MouseAdapter()
    {
			@Override
      public void mouseDown( MouseEvent e )
      {
      	Tools.waehleZuPruefendesBild(e, _anzeigeTyp);
      }         
    });

    /*
     * XYZ
     */
    Composite comp4 = new Composite( table, SWT.NONE );
    comp4.setBackground( table.getBackground() );
    comp4.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp4.setLayoutData( gd4 );
    comp4.setLayout( new FillLayout() );
    
    for (int i=0; i<_anzBilder; ++i)
    {
	    Label lbImageXYZ = new Label( comp4, SWT.CENTER );
	    lbImageXYZ.addMouseListener( new MouseAdapter()
	    {
				@Override
	      public void mouseDown( MouseEvent e )
	      {
	      	Tools.waehleZuPruefendesBildMitDontCare(e, _anzeigeTyp);
	      }         
	    });
    }

    /*
     * UVW
     */
    Composite comp5 = new Composite( table, SWT.NONE );
    comp5.setBackground( table.getBackground() );
    comp5.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp5.setLayoutData( gd5 );
    comp5.setLayout( new FillLayout() );
    
    for (int i=0; i<_anzBilder; ++i)
    {
	    Label lbImageXYZ = new Label( comp5, SWT.CENTER );
	    lbImageXYZ.addMouseListener( new MouseAdapter()
	    {
				@Override
	      public void mouseDown( MouseEvent e )
	      {
	      	Tools.waehleZuPruefendesBildMitDontCare(e, _anzeigeTyp);
	      }         
	    });
    }
    
    /*
     * A
     */
    Composite comp6 = new Composite( table, SWT.NONE );
    comp6.setBackground( table.getBackground() );
    comp6.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp6.setLayoutData( gd6 );
    comp6.setLayout( new FillLayout() );
    
    Label lbImageA = new Label( comp6, SWT.CENTER );
    lbImageA.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
      	Tools.waehleZuSchaltendesBild(e, _anzeigeTyp);
      }         
    });

    return new TableModel( comp0, comp1, comp2, comp3, comp4, comp5, comp6 );
  }
  
  /**
   * Methode liefert die Anzahl der Parameter
   * @return Anzahl der Parameter
   */
  public int getAnzahlParameter()
  {
    return _aktuelleParameter.getParameter().size();
  }
  
  /**
   * Methode liefert den Tabelviewer
   * @return
   */
  public TableViewer getTvAnzeige()
  {
    return _tvAnzeige;
  }

  /**
   * Renderer fr die Tabelle
   * @author ObertM
   */
  private class MyCellLabelProvider extends CellLabelProvider 
  {
    @Override
    public void update(ViewerCell cell)
    {
      TableItem item = (TableItem)cell.getItem();

//      if (item.getData("EDITOR") != null)
//      {
//        TableEditor editor = (TableEditor)item.getData("EDITOR");
//        editor.dispose();
//      }

      int cnt = cell.getColumnIndex();

      TableModel model = (TableModel)item.getData();
      Composite comp = null;
      switch (cnt)
      {
        case 0:
          comp = model.getLoeschen();
          break;
          
        case 1:
          comp = model.getSchaltgrund();
          break;
          
        case 2:
          comp = model.getRichtung();
          break;
          
        case 3:
          comp = model.getX();
          break;
          
        case 4:
          comp = model.getXyz();
          break;

        case 5:
          comp = model.getUvw();
          break;

        case 6:
          comp = model.getA();
          break;

        default:
      }
      
      TableEditor editor = new TableEditor(item.getParent());
      editor.grabHorizontal = true;
      editor.setEditor(comp, item, cnt);

    }
  }

  /**
   * Methode liefert die eingestellten Parameter
   * @return eingestellte Parameter
   */
	public ListeMitRegeln getParameter()
  {
 		aktionRegelLoeschen();
 		return leseAktuelleEinstellungen();
  }
		
	private ListeMitRegeln leseAktuelleEinstellungen ()
	{
		ListeMitRegeln regeln = new ListeMitRegeln();
		
		for (int j=0; j<_tableModel.length; ++j)
		{
      TableModel tm = _tableModel [j];
      
      /*
       * Schaltgrund
       */
      
      String nameSchaltgrund = tm.getSelectedSchaltgrund();
      Aspect schaltgrund = Tools.getAspekt(nameSchaltgrund);
      
      /*
       * Richtung 
       */
      String nameRichtung = tm.getSelectedRichtung();
      ArbeitsRichtungVN richtung = ArbeitsRichtungVN.getRichtung (nameRichtung);

      /*
       * X
       */
      VerwaltungWvzInhalte verw = VerwaltungWvzInhalte.getInstance();
      WVZInhalt x = verw.getWvzInhalt (_anzeigeTyp, tm.getSelectedX());

      /*
       * XYZ
       */
      List<String> tmp = new ArrayList<String>();
      
      for (String s : tm.getSelectedXYZ())
      {
      	if (s != null && !s.equals(AnzeigeInhaltTyp.DONT_CARE.getBildInhalt()))
      		if (!tmp.contains(s))
      			tmp.add(s);
      }

      List<WVZInhalt> xyz = new ArrayList<WVZInhalt>();
      for (String s : tmp)
      	xyz.add (verw.getWvzInhalt (_anzeigeTyp, s));

      /*
       * UVW
       */
      tmp.clear();

      for (String s : tm.getSelectedUVW())
      {
    	  if (s != null && !s.equals(AnzeigeInhaltTyp.DONT_CARE.getBildInhalt()))
    		  if (!tmp.contains(s))
    			  tmp.add(s);
      }

      List<WVZInhalt> uvw = new ArrayList<WVZInhalt>();
      for (String s : tmp)
      	uvw.add (verw.getWvzInhalt (_anzeigeTyp, s));
      
      /*
       * A
       */
      WVZInhalt a = verw.getWvzInhalt (_anzeigeTyp, tm.getSelectedA());

      regeln.addRegel(x, xyz, uvw, a, richtung, schaltgrund);
		}

		return regeln;
  }

	/**
	 * Methode bernimmt die Daten eines anderen Folders
	 * @param tvAnzeige Tableviewer der zu bernhmenden Tabelle
	 */
	public void setTvAnzeige (TableViewer tvAnzeige)
	{
    TableModel[] modelSet = (TableModel[]) tvAnzeige.getInput();

    Vector<RegelLueckenFuellen> v = new Vector<RegelLueckenFuellen>();

    for (int i=0; i<modelSet.length; ++i)
    {
      String nameSchaltgrund = modelSet [i].getSelectedSchaltgrund();
      Aspect schaltgrund = Tools.getAspekt(nameSchaltgrund);

      String nameRichtung = modelSet [i].getSelectedRichtung();
      ArbeitsRichtungVN r = ArbeitsRichtungVN.getRichtung (nameRichtung);

      VerwaltungWvzInhalte verw = VerwaltungWvzInhalte.getInstance();
      
  	  WVZInhalt x = verw.getWvzInhalt(_anzeigeTyp, modelSet [i].getSelectedX());
  	  WVZInhalt a = verw.getWvzInhalt(_anzeigeTyp, modelSet [i].getSelectedA());

  	  List<WVZInhalt> xyz = new ArrayList<WVZInhalt>();
  	  for (String s : modelSet [i].getSelectedXYZ())
  	  	xyz.add(verw.getWvzInhalt(_anzeigeTyp, s));

  	  List<WVZInhalt> uvw = new ArrayList<WVZInhalt>();
  	  for (String s : modelSet [i].getSelectedUVW())
  	  	uvw.add(verw.getWvzInhalt(_anzeigeTyp, s));
  	  
  	  v.add (new RegelLueckenFuellen (schaltgrund, r, x, xyz, uvw, a));
    }
    
    _aktuelleParameter.setParameter(v);
    updateData(_aktuelleParameter);
  }
}
