package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal;

import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.VerwaltungWvzInhalte;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Fenster zum Versorgen der Algorithmus abhngigen Parameter
 * @author ObertM
 */
public class CompositeParameterAlternativerInhalt extends Composite
{
  /**
   * Einheiten des DAV fr die Spinner
   */
  private static enum Einheit { KEIN };
  
  /**
   * ParameterElemente des DAV, die als Spinner angezeigt werden, die Namen mssen identisch sein
   * mit den Namen der Items der Attributgruppe im DaV 
   */
  private static final String[] _parItemSpin = 
  {
  };
  
  /**
   * ParameterElemente des DAV, die als ComboBox angezeigt werden, die Namen mssen identisch sein
   * mit den Namen der Items der Attributgruppe im DaV 
   */
  private static final String[] _parItemCombo = 
  {
  	"WZG Typ", "Angeforderter Inhalt",
  };

  /**
   * Gesmatcomposite
   */
  private Composite _composite;

  /**
   * Composite (Gruppe) fr die Parameter
   */
  private Group _groupParameter;

  /**
   * Composite mit der Tabelle
   */
  private Composite _compositeTabelle = null;

  /**
   * Compoiste fr die Tabelle
   */
  private Composite _composite3 = null;

  /**
   * Table Viewer
   */
  private TableViewer _tvParameter;;

  /**
   * Spinner
   */
  private Spinner[] _spinnerPar;

  /**
   * ComboBoxen
   */
  private Combo[] _comboPar;

	private RegelUniversal _aktuell;

	private List<SystemObject> _listeMoeglicherWZGTypen;

	private List<SystemObject> _listMoeglicheInhalte;

  /**
   * Konstruktor der Klasse
   * @param parent Aufrufendes Composite
   * @param style Stil
   * @param result Ergebnis vom Dav
   * @param data Datenfeld innerhalb des Ergebnisses welches ausgewertet werden soll
   */
  CompositeParameterAlternativerInhalt(Composite parent, int style, RegelUniversal regel)
  {
    super(parent, style);
    
    _aktuell = regel;

    /*
     * WZG Typen bestimmen
     */
    String pidSba = Konfig.getInstance().getPidSBA();
    _listeMoeglicherWZGTypen = Tools.bestimmeAnzeigeTypen(pidSba);

    /* 
     * Anzeigeninhalte bestimmen
     */
//    _listMoeglicheInhalte = Konfig.getInstance().getSchaltbareWVZInhalte();
    
//    _listMoeglicheInhalte = Tools.getSystemObjekte("typ.wvzInhalt");
    _listMoeglicheInhalte = new ArrayList<SystemObject>();

    initialize();
  }
  
  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

    erzeugeKommentar();
    
    createGroupParameter();

    updateData();
  }

  /**
   * Methode blendet einen Kommentar zu dem Parametersatz ein
   */
  private void erzeugeKommentar ()
  {
  	String kommentar = "Universalregel:\n\n" + 
  									   "Das alternative Schaltbild wird an Stelle der im Reiter \"Schaltbild\" definierten Inhalte immer dann verwendet,\n" + 
  									   "wenn der angeforderte Inhalt des alternativen Schaltbildes ungleich \"" + Tools._undefiniert + "\" ist.\n\n";
  	
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    
    Group groupKommentar = new Group (_composite, SWT.NONE);
    groupKommentar.setLayout(new FillLayout());
    groupKommentar.setLayoutData(gridData1);

//    groupKommentar.setText("Erklrung");
    
    Label text = new Label (groupKommentar, SWT.NONE);
    text.setText(kommentar);
  }

  /**
   * Updatefunktion fr Aufruf von aussen
   * @param result Ergebnis vom Dav
   * @param data Datenfeld innerhalb des Ergebnisses welches ausgewertet werden soll
   */
  public void update (RegelUniversal regel)
  {
    _aktuell = regel;
    
    updateData();
    
  }
  
  /**
   * Updatefunktion
   */
  private void updateData ()
  {
    Table table = _tvParameter.getTable();
    TableModel[] input = createModel(table);
    _tvParameter.setInput(input);
  	
  	/*
  	 * WZG-Typ
  	 */
  	SystemObject typ = _aktuell.getAlternativerAnzeigenTyp();
  	String kennung = Tools._undefiniert;
  	if (typ != null)
  		kennung = Tools.getName(typ);
  		
  	Combo cb = (Combo) _comboPar [0];
    for (int k = 0; k < cb.getItemCount(); k++)
    {
      if (kennung.equals(cb.getItem( k )))
        cb.select(k);      
    }
  	
    aktualisiereMoeglicheBilder(typ);
    
  	/*
  	 * Angeforderter Inhalt
  	 */
  	WVZInhalt x = _aktuell.getAlternativerAnzeigenInhalt();
  	kennung = Tools._undefiniert;
  	SystemObject so = x.getAnzeigenInhalt();
  	if (so != null)
  		kennung = Tools.getName(so);
  		
  	cb = (Combo) _comboPar [1];
    for (int k = 0; k < cb.getItemCount(); k++)
    {
      if (kennung.equals(cb.getItem( k )))
        cb.select(k);      
    }

    getTvParameter().refresh();
    layout(true);
  }

  /**
   * Methode zum Anlegen des Composite mit den Parametern
   */
  private void createGroupParameter()
  {
    FillLayout fillLayout2 = new FillLayout();
    fillLayout2.type = org.eclipse.swt.SWT.VERTICAL;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    
    _groupParameter = new Group(_composite, SWT.NONE);
    _groupParameter.setText("Parameter");
    _groupParameter.setLayout(fillLayout2);
    _groupParameter.setLayoutData(gridData2);
    
    //GridLayout mit Anzahl Spalten
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    gridLayout.marginHeight = 5;
    gridLayout.marginWidth = 5;
    
    _compositeTabelle = new Composite(_groupParameter, SWT.NONE);
    _compositeTabelle.setLayout(gridLayout);

    erzeugeTabelle();
  }

  /**
   * Tabelle fr die Parameter erzeugen
   */
  private void erzeugeTabelle()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    FillLayout fillLayout = new FillLayout();
    fillLayout.type = org.eclipse.swt.SWT.HORIZONTAL; 

    _composite3 = new Composite(_compositeTabelle, SWT.None);
    _composite3.setLayout(fillLayout);
    _composite3.setLayoutData( gridData );
    
    _tvParameter = new TableViewer(_composite3, SWT.HIDE_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvParameter.setContentProvider(new TableContentProvider());

    TableViewerColumn column1 = new TableViewerColumn(_tvParameter, SWT.NONE);
    column1.getColumn().setWidth(150);
    column1.getColumn().setText("Attribut");
    column1.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column2 = new TableViewerColumn(_tvParameter, SWT.NONE);
    column2.getColumn().setWidth(350);
    column2.getColumn().setText("Wert");
    column2.setLabelProvider(new MyCellLabelProvider());

//    TableViewerColumn column3 = new TableViewerColumn(_tvParameter, SWT.NONE);
//    column3.getColumn().setWidth(100);
//    column3.getColumn().setText("");
//    column3.setLabelProvider(new MyCellLabelProvider());

    /*
     * Zeilenhhe anpassen, nachdem die Daten eingegangen sind
     */
    _tvParameter.getTable().addListener(SWT.MeasureItem, new Listener() 
    {
      @Override
      public void handleEvent(Event event)
      {
        event.height = 29;
      }
    });

    _tvParameter.getTable().setLinesVisible(true);
    _tvParameter.getTable().setHeaderVisible(true);
  }

  /**
   * Contentprovider fr die Tabelle
   * @author ObertM
   *
   */
  private class TableContentProvider implements IStructuredContentProvider, Observer
  {
    private TableViewer tableViewer;

    /*
     * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
     */
    public Object[] getElements(Object inputElement)
    {
      return (TableModel[])inputElement;
    }

    /*
     * @see org.eclipse.jface.viewers.IContentProvider#dispose()
     */
    public void dispose()
    {

    }

    /*
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object,
     *      java.lang.Object)
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
      tableViewer = (TableViewer)viewer;

      if (oldInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;

        if (tms != null)
        {
          for (TableModel tm : tms)
          {
            tm.deleteObserver(this);
          }

        }

      }
      if (newInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;
        if (tms != null)
        {
          for (TableModel tm : tms)
          {
            tm.addObserver(this);
          }
        }

      }

    }

    @Override
    public void update(Observable arg0, Object arg1)
    {
      if (tableViewer != null)
      {
        tableViewer.refresh();

      }
    }
  }

  /**
   * Tabellenmodell fr die Parameter erzeugen
   * @param table Tabelle
   * @return TabellenModell
   */
  private TableModel[] createModel( Table table )
  {
    TableModel[] elements = new TableModel[_parItemSpin.length + _parItemCombo.length];

    /*
     * ComboBoxen erzeugen
     */
    _comboPar = new Combo [_parItemCombo.length];

    elements[0] = createComboElement(table, 0, _parItemCombo[0], Einheit.KEIN, Tools.bestimmeObjektNamen(_listeMoeglicherWZGTypen, true) );
    elements[1] = createComboElement(table, 1, _parItemCombo[1], Einheit.KEIN, Tools.bestimmeObjektNamen(_listMoeglicheInhalte, true));

    _comboPar [0].addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
		  	SystemObject typ = bestimmeWZGTyp();
		    aktualisiereMoeglicheBilder(typ);
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}
		});

    return elements;
  }

  /**
   * ein Parameter-Element des TabellenModells erzeugen
   * 
   * @param table       Tabelle
   * @param intCombo    Index der ComboBox
   * @param item        Bezeichnung des Parameters
   * @param einheit     Einheit des Parameters
   * @param anzeigeQuerschnitte       mgliche Werte der ComboBox
   * @return
   */
  private TableModel createComboElement( Table table, int intCombo, String item, Einheit einheit, List<String> anzeigeQuerschnitte )
  {
  	/*
  	 * Composite 0: Beschriftung 
  	 */
    GridData gd0 = new GridData();
    gd0.horizontalAlignment = GridData.CENTER;
    gd0.grabExcessHorizontalSpace = true;
    gd0.verticalAlignment = GridData.CENTER;
    gd0.grabExcessVerticalSpace = true;
    
    Composite comp0 = new Composite(table, SWT.NONE);
    comp0.setBackground(table.getBackground());
    comp0.setBackgroundMode(SWT.INHERIT_DEFAULT);
    comp0.setLayoutData(gd0);
    comp0.setLayout(new GridLayout());
    
    Label label = new Label(comp0, SWT.NONE);
    label.setText( item );

    /*
     * Composite 1: Combobox
     */
    GridData gd1 = new GridData();
    gd1.horizontalAlignment = GridData.CENTER;
    gd1.grabExcessHorizontalSpace = true;
    gd1.grabExcessVerticalSpace = true;
    gd1.verticalAlignment = GridData.END;

    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;

    Composite comp1 = new Composite( table, SWT.NONE );
    comp1.setBackground( table.getBackground() );
    comp1.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp1.setLayoutData( gd1 );
    comp1.setLayout( new GridLayout() );
    
    _comboPar[intCombo] = new Combo( comp1, SWT.BORDER );
    _comboPar[intCombo].setLayoutData( gdTableItem );
    
    String[] t = new String [anzeigeQuerschnitte.size()];
    for (int i=0; i<anzeigeQuerschnitte.size(); ++i)
    	t[i] = anzeigeQuerschnitte.get(i);
    	
    _comboPar[intCombo].setItems(t);
    _comboPar[intCombo].select(0);

    /*
     *  Composite 2: Einheit
     */
    
    GridData gd2 = new GridData();
    gd2.horizontalAlignment = GridData.CENTER;
    gd2.grabExcessHorizontalSpace = true;
    gd2.verticalAlignment = GridData.CENTER;
    gd2.grabExcessVerticalSpace = true;

    Composite comp2 = new Composite(table, SWT.NONE);
    comp2.setBackground(table.getBackground());
    comp2.setBackgroundMode(SWT.INHERIT_DEFAULT);
    comp2.setLayoutData(gd2);
    comp2.setLayout(new GridLayout());
    Label label2 = new Label(comp2, SWT.NONE);
    label2.setText("");
    
    return new TableModel(comp0, comp1, comp2);
  }

  /**
   * ein Parameter-Element des TabellenModells erzeugen
   * 
   * @param table       Tabelle
   * @param intSpinner  Index des Spinners
   * @param item        Bezeichnung des Parameters
   * @param einheit     Einheit des Parameters
   * @return
   */
  private TableModel createModelElement( Table table, int intSpinner, String item, Einheit einheit )
  {
    Composite comp0 = new Composite(table, SWT.NONE);
    comp0.setBackground(table.getBackground());
    comp0.setBackgroundMode(SWT.INHERIT_DEFAULT);
    GridData gd0 = new GridData();
    gd0.horizontalAlignment = GridData.CENTER;
    gd0.grabExcessHorizontalSpace = true;
    gd0.verticalAlignment = GridData.CENTER;
    gd0.grabExcessVerticalSpace = true;
    comp0.setLayoutData(gd0);
    comp0.setLayout(new GridLayout());
    Label label = new Label(comp0, SWT.NONE);
    label.setText( item );
    
    Composite comp1 = new Composite(table, SWT.NONE);
    comp1.setBackground(table.getBackground());
    comp1.setBackgroundMode(SWT.INHERIT_DEFAULT);
    GridData gd1 = new GridData();
    gd1.horizontalAlignment = GridData.CENTER;
    gd1.grabExcessHorizontalSpace = true;
    gd1.grabExcessVerticalSpace = true;
    gd1.verticalAlignment = GridData.END;
    comp1.setLayoutData(gd1);
    comp1.setLayout(new GridLayout());
    _spinnerPar[intSpinner] = new Spinner(comp1, SWT.BORDER);
    // zum Ausfllen der Zelle
    GridData gdSpinner = new GridData();
    gdSpinner.horizontalAlignment = GridData.FILL;
    gdSpinner.grabExcessHorizontalSpace = true;
    gdSpinner.verticalAlignment = GridData.CENTER;
    _spinnerPar[intSpinner].setLayoutData(gdSpinner);
    
    Composite comp2 = new Composite(table, SWT.NONE);
    comp2.setBackground(table.getBackground());
    comp2.setBackgroundMode(SWT.INHERIT_DEFAULT);
    GridData gd2 = new GridData();
    gd2.horizontalAlignment = GridData.CENTER;
    gd2.grabExcessHorizontalSpace = true;
    gd2.verticalAlignment = GridData.CENTER;
    gd2.grabExcessVerticalSpace = true;
    comp2.setLayoutData(gd2);
    comp2.setLayout(new GridLayout());
    Label label2 = new Label(comp2, SWT.NONE);

    //Erstinitialisierung, wird aus udateData heraus mit den aktuellen Daten aus dem DAV berschrieben
    switch ( einheit )
    {
//    case KEIN:
//      _spinnerPar[intSpinner].setMinimum(0);
//      _spinnerPar[intSpinner].setMaximum(100000000);
//      _spinnerPar[intSpinner].setIncrement(1000);      
//      _spinnerPar[intSpinner].setPageIncrement(10000);  
//      label2.setText( "PkwE/h" );      
//      break;
//      
//    case FZH:
//      _spinnerPar[intSpinner].setMinimum(0);
//      _spinnerPar[intSpinner].setMaximum(100000000);  
//      label2.setText( "Fz/h" );      
//      break;
      
    default:
      break;
    }
    
//    SpinnerAddOns.setSpinnerTooltip( _spinnerPar[intSpinner], null );
    
    return new TableModel(comp0, comp1, comp2);
  }

  /**
   * Methode liefert den TableViewer
   * @return TableViewer
   */
  public TableViewer getTvParameter()
  {
    return _tvParameter;
  }

  /**
   * Methode liefert den Spinner zu einem Index
   * @param index Index des Spinners
   * @return Spinner
   */
  public Spinner getSpinner(int index)
  {
    return _spinnerPar [index];
  }

  /**
   * Methode liefert die ComboBox zu einem Index
   * @param index Index des Spinners
   * @return Spinner
   */
  public Combo getComboBox (int index)
  {
    return _comboPar [index];
  }

  /**
   * Methode liefert den Namen eines Items eines Spinners zu einem Index
   * @param index Index
   * @return Name des Items
   */
  public String getItemSpinnerName (int index)
  {
  	return _parItemSpin [index];
  }

  /**
   * Methode liefert den Namen eines Items einer ComboBox zu einem Index
   * @param index Index
   * @return Name des Items
   */
  public String getItemComboBoxName (int index)
  {
  	return _parItemCombo [index];
  }

  /**
   * Labelprovider
   * @author ObertM
   */
  private class MyCellLabelProvider extends CellLabelProvider
  {
    @Override
    public void update(ViewerCell cell)
    {
      TableItem item = (TableItem)cell.getItem();

      if (item.getData("EDITOR") != null)
      {
        TableEditor editor = (TableEditor)item.getData("EDITOR");
        editor.dispose();
      }


      int cnt = cell.getColumnIndex();

      TableModel model = (TableModel)item.getData();
      Composite comp = null;
      switch (cnt)
      {
        case 0:
          comp = model.getBezeichnung();
          break;
        case 1:
          comp = model.getAttribut();
          break;
//        case 2:
//          comp = model.getEinheit();
//          break;
        default:
      }
      TableEditor editor = new TableEditor(item.getParent());
      
      editor.grabHorizontal = true;
      editor.setEditor(comp, item, cnt);
    }
  }
  
  /**
   * Methode liefert die Anzahl der verwendeten Items
   * @return Anzahl 
   */
  public int getAnzahlSpinnerItems ()
  {
  	return _parItemSpin.length;
  }

  /**
   * Methode liefert die Anzahl der verwendeten Items
   * @return Anzahl 
   */
  public int getAnzahlComboBoxItems ()
  {
  	return _parItemCombo.length;
  }

  /**
   * Methode refresht die Tabelle
   */
	public void refresh()
  {
		if (_tvParameter != null)
	    _tvParameter.refresh();
	  
  }

	public void uebernehmeDaten ()
  {
		/*
		 * WZG Typ
		 */
  	SystemObject so = bestimmeWZGTyp();
    _aktuell.setAlternativerAnzeigenTyp(so);
    
    /*
     * Anzeigeninhalt
     */
  	so = null;
    Combo cb = _comboPar [1];
    String selected = cb.getItem( cb.getSelectionIndex() );
    
    if (!selected.equals(Tools._undefiniert))
    {
      for (SystemObject so1 : _listMoeglicheInhalte)
      	if (Tools.getName(so1) != null && Tools.getName(so1).equals(selected))
      		so = so1;
    }
    
    if (so != null)
      _aktuell.setAlternativerAnzeigenInhalt(new WVZInhalt(so));
    else
      _aktuell.setAlternativerAnzeigenInhalt(new WVZInhalt());
  }
	
	/**
	 * Methode bestimmt das Systemobjekt des eingestellten WZG Typs
	 * @return Systemobjekt
	 */
	private SystemObject bestimmeWZGTyp ()
	{
  	SystemObject so = null;
    Combo cb = _comboPar [0];
    String selected = cb.getItem( cb.getSelectionIndex() );
    
    if (!selected.equals(Tools._undefiniert))
    {
      for (SystemObject so1 : _listeMoeglicherWZGTypen)
      	if (Tools.getName(so1) != null && Tools.getName(so1).equals(selected))
      		so = so1;
    }
    
    if (so != null)
      System.out.println("CompositeParameterAlternativerInhalt.bestimmeWZGTyp() " + so.getName());
    return so;
	}
	
	/**
	 * Methode zum Aktualisieren der mglichen Bilder in Abhngigkeit des WZG Typs
	 * @param wzgTyp ausgewhlter WZGTyp
	 */
	private void aktualisiereMoeglicheBilder (SystemObject wzgTyp)
	{
    _listMoeglicheInhalte = VerwaltungWvzInhalte.getInstance().getZuSchaltendeWvzInhalte(wzgTyp);
    
    List<String> namen = Tools.bestimmeObjektNamen(_listMoeglicheInhalte, true);
    String[] t = new String [namen.size()];
    for (int i=0; i<namen.size(); ++i)
    	t[i] = namen.get(i);
    _comboPar[1].setItems(t);
    _comboPar[1].select(0);
	}
}
