package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.zusammenfassungEnde;

import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeInhaltTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.VerwaltungWvzInhalte;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeElementTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.CompositeTools;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Darstellung des Fensters zur Versorgung der Parameter
 */
public class CompositeParameterZusammenfassungEnde extends Composite
{
	/**
	 * max. Anzahl versorgbarer Bilder
	 */
	private final int _anzBilder = 10;
	
	/**
	 * max. Anzahl Bilder in einer Zeile
	 */
	private final int _anzBilderProZeile = 5;
	
	/**
	 * Aktuelles Composite
	 */
  private Composite _composite;

  /**
   * Composite der Tabelle
   */
  private Composite _compositeTabelle;

  /**
   * Gruppe mit den Regeln
   */
  private Group _groupRegeln;

  /**
   * Tabelviewer der Tabelle
   */
  private TableViewer _tvAnzeige;

  /**
   * Aktuelle Parameter
   */
  private ListeMitRegeln _aktuelleParameter;

  /**
   * Composite fr die Buttons
   */
	private Composite _compositeButtons;

	/**
	 * Button Hinzufgen
	 */
	private Button _buttonHinzufuegen;

	/**
	 * Button Lschen
	 */
	private Button _buttonLoeschen;

	/**
	 * Tabellenmodell
	 */
	private TableModel[] _tableModel;
	
	/**
	 * Anzeigetyp fr den die Parameter versorgt werden
	 */
	private Button _buttonAlleLoeschen;

	private Button _buttonDefault;

  /**
   * Konstruktor der Klasse
   * @param parent Parent Composite
   * @param style Stil des Fenstern
   * @param aktuell Aktuelle Parameter
   */
  CompositeParameterZusammenfassungEnde (Composite parent, int style, ListeMitRegeln aktuell)
  {
    super (parent, style);

    this._aktuelleParameter = erzeugeKopie(aktuell);
    
    initialize();
  }

  /**
   * Methode legt eine Arbeitskopie der akutellen Daten an
   * @param aktuell aktuelle Daten
   * @return Arbeitskopie
   */
  private ListeMitRegeln erzeugeKopie(ListeMitRegeln aktuell)
  {
	  ListeMitRegeln kopie = new ListeMitRegeln();
	  
	  Vector<RegelZusammenfassungEnde> parameter = aktuell.getParameter();
	  for (RegelZusammenfassungEnde regel : parameter)
	  {
	  	List<WVZInhalt> anzeigenInhaltA = new ArrayList<WVZInhalt>();
	  	for (WVZInhalt inhalt : regel.getAnzeigenInhaltA())
	  			anzeigenInhaltA.add(inhalt);

	  	List<WVZInhalt> anzeigenInhaltB = new ArrayList<WVZInhalt>();
	  	for (WVZInhalt inhalt : regel.getAnzeigenInhaltB())
	  			anzeigenInhaltB.add(inhalt);

	  	kopie.addRegel(
	  			regel.getPrioritaetA(),
	  			regel.getPrioritaetB(),
	  			anzeigenInhaltA,
	  			anzeigenInhaltB,
	  			regel.getAngeforderterInhaltA(),
	  			regel.getAngeforderterInhaltB(),
	  			regel.getSchaltgrundRegel());
	  }
	  
	  return kopie;
  }

  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    
    _composite = new Composite(this, SWT.NONE);
    _composite.setLayout(new GridLayout(1, false));
    _composite.setLayoutData(gridData1);
 
    erzeugeKommentar();
    
    erzeugeGruppeRegeln();
    
    Table table = _tvAnzeige.getTable();
    _tableModel = createModel1(table);
    
    if ( _tableModel != null )
      _tvAnzeige.setInput(_tableModel); 
    
    createButtons();
    
    updateData(_aktuelleParameter);
  }

  /**
   * Methode blendet einen Kommentar zu dem Parametersatz ein
   */
  private void erzeugeKommentar ()
  {
  	String kommentar = "Zusammenfassung Ende:\n\n" + 
  	"Wenn die angeforderten Inhalte fr TypA und TypB in der Liste " +
  	"\"mglich TypA\" bzw. \"mglich TypB\" enthalten sind,\n" + 
  	"dann werden an diesen Anzeigen die Inhalte \"zu schalten TypA\" bzw. \"zu schalten TypB\" angefordert.\n\n" +
  	"Die Regel kann ber einen Hauptschalter abgeschaltet werden.\n\n" +
  	"";
  	
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    
    Group groupKommentar = new Group (_composite, SWT.NONE);
    groupKommentar.setLayout(new FillLayout());
    groupKommentar.setLayoutData(gridData1);

//    groupKommentar.setText("Erklrung");
    
    Label text = new Label (groupKommentar, SWT.NONE);
    text.setText(kommentar);
  }

  /**
   * Methode bernimmt die Daten und zeigt diese in der Tabelle an
   */
  private void updateData (ListeMitRegeln parameter)
  {
  	int anzahlParameter = parameter.getParameter().size();
  	
  	erzeugeTabellenModell (anzahlParameter);

  	for (int j=0; j<anzahlParameter; ++j)
  	{
      TableModel tm = _tableModel [j];    	
    	RegelZusammenfassungEnde aktuell = parameter.getParameter().get(j);
    	
    	/*
    	 * Lschen
    	 */
    	tm.setSelectedLoeschen("false");
    	
    	/*
    	 * Schaltgrund
    	 */
    	
    	if (aktuell.getSchaltgrundRegel() == null)
    	{
    	  tm.setSelectedSchaltgrund("");
    	}
    	else
    	{
  	    tm.setSelectedSchaltgrund(aktuell.getSchaltgrundRegel().getName());
    	}
    	
  	  /*
  	   * mgliche Inhalte A
  	   */
  	  erzeugeBilder (tm.getMoeglicheInhalteA(), aktuell.getAnzeigenInhaltA(), AnzeigeElementTyp.WZG_A);

  	  /*
  	   * mgliche Inhalte B
  	   */
  	  erzeugeBilder (tm.getMoeglicheInhalteB(), aktuell.getAnzeigenInhaltB(), AnzeigeElementTyp.WZG_B);

  	  /*
  	   * zu schalten A
  	   */
  	  erzeugeBildMitPrio(tm.getZuSchaltenA(), aktuell.getAngeforderterInhaltA(), aktuell.getPrioritaetA(), AnzeigeElementTyp.WZG_A);

  	  /*
  	   * zu schalten B
  	   */
  	  erzeugeBildMitPrio(tm.getZuSchaltenB(), aktuell.getAngeforderterInhaltB(), aktuell.getPrioritaetB(), AnzeigeElementTyp.WZG_B);
    }

  	_buttonAlleLoeschen.setEnabled (anzahlParameter != 0);
  	pruefeObLoeschenAngewaehlt();
  }

  /**
   * Methode erzeugt in einem Composite die versorgten Bilder
   * @param c Composite
   * @param regel
   */
  private void erzeugeBilder (Composite c, List<WVZInhalt> wi, AnzeigeElementTyp typ)
  {
	  Control[] children = c.getChildren();
	  Label [] im = new Label [children.length];
	  for (int i=0; i<children.length; ++i)
	  	im [i] = (Label) children [i];
	  
	  int max = (wi.size() > _anzBilder ? _anzBilder : wi.size());
	  
	  for (int i=0; i<max; ++i)
	  	Tools.setzeBildInhalte(im[i], wi.get(i), typ, true);
	  
		WVZInhalt dontCare = VerwaltungWvzInhalte.getInstance().getWvzInhalt(typ, AnzeigeInhaltTyp.DONT_CARE);
	  for (int i=wi.size(); i<_anzBilder; ++i)
	  	Tools.setzeBildInhalte(im[i], dontCare, typ, true);
  }
  
  /**
   * Methode erzeugt ein Composite mit einem Label fr die WVZ-Anzeige und einem
   * Spinner zum Einstellen der Prioritt
   * @param c
   * @param wi
   * @param prio
   * @param typ
   */
  private void erzeugeBildMitPrio (Composite c, WVZInhalt wi, int prio, AnzeigeElementTyp typ)
  {
	  Control[] children = c.getChildren();
	  
	  Label imA = (Label) children[0];
	  Tools.setzeBildInhalte(imA, wi, typ, true);

	  Spinner sp = (Spinner) children[2];
	  sp.setSelection(prio);
  }
  
  /**
   * Methode erzeugt ein Tabellenmodell einer bestimmten Gre. Dabei werden bereits vorhandene
   * Composite in der Tabelle weiter verwendet
   * @param anzahlParameter Gre des erzeugten Tabellenmodells
   */
  private void erzeugeTabellenModell (int anzahlParameter)
  {
  	/*
  	 * Hat sich die Gre gendert
  	 */
  	if (_tableModel.length != anzahlParameter)
  	{
  		/*
  		 * Tabelle erweitern
  		 */
  		if (_tableModel.length < anzahlParameter)
  		{
  		  Vector<TableModel> v = new Vector<TableModel>();
  		  
  		  for (int i=0; i<_tableModel.length; ++i)
  		  	v.add(_tableModel [i]);
  		  
  		  for (int i=_tableModel.length; i<anzahlParameter; ++i)
  		  	v.add (erzeugeZeileInTabelle(_tvAnzeige.getTable()));
  		  
  		  _tableModel = new TableModel [anzahlParameter];
  		  for (int i=0; i<v.size(); ++i)
  		  	_tableModel [i] = v.get(i);
  		}
  		/*
  		 * Tabelle reduzieren
  		 */
  		else if (_tableModel.length > anzahlParameter)
  		{
  			/*
  			 * Nicht mehr bentigte Elemente lschen (in umgekehrter Reihenfolge)
  			 */
  			
  			for (int i = _tableModel.length - 1; i > anzahlParameter - 1; i--)
  			{
  				_tableModel [i].getZuSchaltenB().dispose();
  				_tableModel [i].getZuSchaltenA().dispose();
  				_tableModel [i].getMoeglicheInhalteB().dispose();
  				_tableModel [i].getMoeglicheInhalteA().dispose();
  				_tableModel [i].getSchaltgrund().dispose();
  				_tableModel [i].getLoeschen().dispose();
  			}
  				
  		  Vector<TableModel> v = new Vector<TableModel>();
  		  for (int i=0; i<anzahlParameter; ++i)
  		  	v.add(_tableModel [i]);

  		  _tableModel = new TableModel [anzahlParameter];
  		  for (int i=0; i<v.size(); ++i)
  		  	_tableModel [i] = v.get(i);
  		}
  		
  	  _tvAnzeige.setInput(_tableModel);
  	  _tvAnzeige.getTable().layout(true);
  	}
  }
  
  /**
   * Methode erzeugt die Gruppe mit den Regeln
   */
  private void erzeugeGruppeRegeln()
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.marginHeight = 5;
    fillLayout1.marginWidth = 5;
    fillLayout1.type = org.eclipse.swt.SWT.HORIZONTAL;
    
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    
    _groupRegeln = new Group(_composite, SWT.NONE);
    _groupRegeln.setText("Regeln");
    
    _groupRegeln.setLayout(fillLayout1);
    _groupRegeln.setLayoutData(gridData2);

    erzeugeTabelle();
  }

  /**
   * Methode erzeugt das Composite mit den Buttons
   */
  private void createButtons()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    
    _compositeButtons = new Composite( _composite, SWT.NONE );
    _compositeButtons.setLayoutData( gridData3 );
    _compositeButtons.setLayout( rowLayout );

    _buttonHinzufuegen = new Button( _compositeButtons, SWT.NONE );
    _buttonHinzufuegen.setText( "Regel hinzufgen" );
    _buttonHinzufuegen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionRegelHinzufuegen();
      }
    } );

    _buttonLoeschen = new Button( _compositeButtons, SWT.NONE );
    _buttonLoeschen.setText( "Selektierte Regeln loeschen" );
    _buttonLoeschen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionRegelLoeschen();
      }

    } );
    _buttonLoeschen.setEnabled(false);

    _buttonAlleLoeschen = new Button( _compositeButtons, SWT.NONE );
    _buttonAlleLoeschen.setText( "Alle Regeln loeschen" );
    _buttonAlleLoeschen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        aktionAlleRegelLoeschen();
      }
    } );
    
    _buttonDefault = new Button( _compositeButtons, SWT.NONE );
    _buttonDefault.setText( "Defaultwerte" );
    _buttonDefault.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        aktionDefault ();
      }
    } );
  }

	/**
   * Methode zum Lschen aller Meldungen
   */
  protected void aktionAlleRegelLoeschen()
  {
  	Vector<RegelZusammenfassungEnde> v = new Vector<RegelZusammenfassungEnde>();
    _aktuelleParameter.setParameter(v);
    
    updateData(_aktuelleParameter);
  }
  
  /**
   * Methode generiert automatisch die Defaultwerte
   */
  protected void aktionDefault()
  {
		Vector<RegelZusammenfassungEnde> regeln = new Vector<RegelZusammenfassungEnde>();

	  DefaultWerte dw = new DefaultWerte ();
	  regeln = dw.getDefaultWerte(_anzBilder);

	  _aktuelleParameter.setParameter(regeln);
    
    updateData(_aktuelleParameter);
  }

  /**
   * Methode prft ob Regeln zum Lschen ausgewhlt sind
   */
  private void pruefeObLoeschenAngewaehlt ()
  {
  	boolean angewaehlt = false;
  	
		for (TableModel tm : _tableModel)
			if (tm.getSelectedLoeschen().equals("true"))
				angewaehlt = true;
		
		_buttonLoeschen.setEnabled(angewaehlt);
  }
  
  /**
   * Methode zum Lschen der Regeln
   */
	private void aktionRegelLoeschen()
  {
		Vector<RegelZusammenfassungEnde> regeln = leseAktuelleEinstellungen().getParameter();
		
    Vector<RegelZusammenfassungEnde> loeschen = new Vector<RegelZusammenfassungEnde>();
    
    for (int i=0; i<_tableModel.length; ++i)
    	if (!_tableModel [i].getSelectedLoeschen().equals("false"))
    		loeschen.add(regeln.get(i));

    for (RegelZusammenfassungEnde r : loeschen)
    {
    	regeln.remove(r);
    }
    
    _aktuelleParameter.setParameter(regeln);
    
    updateData(_aktuelleParameter);
  }

	/**
	 * Aktion Regel hinzufgen
	 */
	private void aktionRegelHinzufuegen()
  {
		Vector<RegelZusammenfassungEnde> regeln = leseAktuelleEinstellungen().getParameter();
    _aktuelleParameter.setParameter(regeln);

		Aspect a = VerwaltungParaZusammenfassungEnde.getInstance().getSchaltgrundAllgemein();

		List<WVZInhalt> a1 = new ArrayList<WVZInhalt>();
		List<WVZInhalt> b1 = new ArrayList<WVZInhalt>();

		WVZInhalt dontCareA = VerwaltungWvzInhalte.getInstance().getWvzInhalt(AnzeigeElementTyp.WZG_A, AnzeigeInhaltTyp.DONT_CARE);
		WVZInhalt dontCareB = VerwaltungWvzInhalte.getInstance().getWvzInhalt(AnzeigeElementTyp.WZG_B, AnzeigeInhaltTyp.DONT_CARE);

		for (int i=0; i<_anzBilder; ++i)
		{
			a1.add(dontCareA);
			b1.add(dontCareB);
		}

 	_aktuelleParameter.addRegel(
  			-1,
  			-1,
  			a1,
  			b1,
  			dontCareA,
  			dontCareB,
  			a);

		updateData(_aktuelleParameter);
  }

	/**
	 * Methode erzeugt das Composite mit der Tabelle
	 */
  private void erzeugeTabelle ()
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.type = org.eclipse.swt.SWT.VERTICAL;
    
    _compositeTabelle = new Composite(_groupRegeln, SWT.None);
    _compositeTabelle.setLayout(fillLayout1);
    
    _tvAnzeige = new TableViewer(_compositeTabelle, SWT.HIDE_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvAnzeige.setContentProvider(new TableContentProvider());
    
    int breiteWVZ = Tools.getMaxBreiteBild() * _anzBilderProZeile;
    
    String [] header = new String []{"Lschen", "Schaltgrund", "mglich TypA", "mglich TypB", "zu schalten TypA", "zu schalten TypB", };
    int    [] breite = new int [] {60, 200, breiteWVZ, breiteWVZ, 120, 120};

    for (int i=0; i<header.length; ++i)
    {
      TableViewerColumn column = new TableViewerColumn(_tvAnzeige, SWT.NONE);
      column.getColumn().setWidth(breite [i]);
      column.getColumn().setText(header [i]);
      column.setLabelProvider(new MyCellLabelProvider());
    }

    // resize the row height using a MeasureItem listener
    Listener listener = new Listener() {
      @Override
      public void handleEvent(Event event)
      {
        switch (event.type)
        {
          case SWT.MeasureItem:
          	
          	int anzZeilen = _anzBilder / _anzBilderProZeile;
          	if ((_anzBilder % _anzBilderProZeile) != 0)
          		anzZeilen++;
          	
            event.height = Tools.getZeilenHoehe(AnzeigeElementTyp.WZG_A) * anzZeilen + 50;
            break;

          default:
        }
      }
    };
    
    _tvAnzeige.getTable().addListener(SWT.MeasureItem, listener);
    _tvAnzeige.getTable().setLinesVisible(true);
    _tvAnzeige.getTable().setHeaderVisible(true);
  }

  /**
   * Klasse realsiert den TableContentProvider fr die Tabelle
   * @author ObertM
   */
  private class TableContentProvider implements IStructuredContentProvider, Observer
  {
  	/**
  	 * Tabelviewer
  	 */
    private TableViewer tableViewer;

    /**
     * Methode liefert die Elemente der Tabelle
     */
    public Object[] getElements(Object inputElement)
    {
			Object[] re = null;

			if (inputElement instanceof TableModel [])
			{
        re = (TableModel[]) inputElement;
			}
			
			return re;
    }

    /**
     * Dispose
     */
    public void dispose()
    {
    }

    /**
     * Methode wird aufgerufen wenn sich der Inhalt der Tabelle gendert hat
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
      tableViewer = (TableViewer)viewer;

      if (oldInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;

        if (tms != null)
        {
          for (TableModel tm : tms)
            tm.deleteObserver(this);
        }
      }
      
      if (newInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;
        if (tms != null)
        {
          for (TableModel tm : tms)
            tm.addObserver(this);
        }
      }
    }

    @Override
    public void update(Observable arg0, Object arg1)
    {
      if (tableViewer != null)
      {
        tableViewer.refresh();
      }
    }
  }

  /**
   * Erzeugt ein TabellenModell fr den Inhalt der Tabellen
   * @param table
   * @return TabellenModell
   */
  private TableModel[] createModel1 (Table table)
  {
    int tabLength = getAnzahlParameter();
    int elIndex   = 0;
    
    TableModel[] elements = new TableModel [tabLength];
    
    if (elements.length == 0)
    	return elements;

    for (int x=0; x<getAnzahlParameter(); ++x)
    {
	    elements[elIndex] = erzeugeZeileInTabelle(table);
	    elIndex++;
    }
    
    return elements;
  }

  /**
   * Methode erzeugt ein Tabellenmodell fr eine Zeile in der Tabelle
   * @param table Tabelle 
   * @return Tabellenmodell fr eine Zeile
   */
  private TableModel erzeugeZeileInTabelle (Table table)
  {
    /*
     * Button Lschen
     */
    Composite comp0 = CompositeTools.erzeugeCompositeLoeschen(table);
    Button b = (Button) comp0.getChildren()[0];

    b.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				pruefeObLoeschenAngewaehlt();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}
		});
    
    /*
     * Combobox Schaltgrund    
     */
    String[] namen = Tools.getNamenDerAspekte();

    Composite comp1 = CompositeTools.erzeugeCompositeCombo(table, namen); 
    
    /*
     * mgliche Inhalte A
     */
    Composite comp2 = CompositeTools.erzeugeCompositeWVZ(table, AnzeigeElementTyp.WZG_A, _anzBilder, _anzBilderProZeile);
    
    /*
     * mgliche Inhalte B
     */
    Composite comp3 = CompositeTools.erzeugeCompositeWVZ(table, AnzeigeElementTyp.WZG_B, _anzBilder, _anzBilderProZeile);

    /*
     * zu schalten A
     */
    Composite comp4 = CompositeTools.erzeugeCompositeWVZmitPrio(table, AnzeigeElementTyp.WZG_A);

    /*
     * zu schalten B
     */
    Composite comp5 = CompositeTools.erzeugeCompositeWVZmitPrio(table, AnzeigeElementTyp.WZG_B);

    return new TableModel( comp0, comp1, comp2, comp3, comp4, comp5);
  }
  
  /**
   * Methode liefert die Anzahl der Parameter
   * @return Anzahl der Parameter
   */
  public int getAnzahlParameter()
  {
    return _aktuelleParameter.getParameter().size();
  }
  
  /**
   * Methode liefert den Tabelviewer
   * @return
   */
  public TableViewer getTvAnzeige()
  {
    return _tvAnzeige;
  }

  /**
   * Renderer fr die Tabelle
   * @author ObertM
   */
  private class MyCellLabelProvider extends CellLabelProvider 
  {
    @Override
    public void update(ViewerCell cell)
    {
      TableItem item = (TableItem)cell.getItem();

//      if (item.getData("EDITOR") != null)
//      {
//        TableEditor editor = (TableEditor)item.getData("EDITOR");
//        editor.dispose();
//      }

      int cnt = cell.getColumnIndex();

      TableModel model = (TableModel)item.getData();
      Composite comp = null;
      switch (cnt)
      {
        case 0:
          comp = model.getLoeschen();
          break;
          
        case 1:
          comp = model.getSchaltgrund();
          break;
          
        case 2:
          comp = model.getMoeglicheInhalteA();
          break;

        case 3:
          comp = model.getMoeglicheInhalteB();
          break;

        case 4:
          comp = model.getZuSchaltenA();
          break;
          
        case 5:
          comp = model.getZuSchaltenB();
          break;

        default:
      }
      
      TableEditor editor = new TableEditor(item.getParent());
      editor.grabHorizontal = true;
      editor.setEditor(comp, item, cnt);

    }
  }

  /**
   * Methode liefert die eingestellten Parameter
   * @return eingestellte Parameter
   */
	public ListeMitRegeln getParameter()
  {
 		aktionRegelLoeschen();
 		return leseAktuelleEinstellungen();
  }
		
	private ListeMitRegeln leseAktuelleEinstellungen ()
	{
		ListeMitRegeln regeln = new ListeMitRegeln();

    VerwaltungWvzInhalte verw = VerwaltungWvzInhalte.getInstance();

		for (int j=0; j<_tableModel.length; ++j)
		{
      TableModel tm = _tableModel [j];
      
      /*
       * Schaltgrund
       */
      
      String nameSchaltgrund = tm.getSelectedSchaltgrund();
      Aspect schaltgrund = Tools.getAspekt(nameSchaltgrund);
      
      /*
       * mgliche Inhalte A
       */
      List<WVZInhalt> moeglicheInhalteA = leseMoeglicheInhalte(tm.getSelectedMoeglicheInhalteA(), AnzeigeElementTyp.WZG_A);

      /*
       * mgliche Inhalte B
       */
      List<WVZInhalt> moeglicheInhalteB = leseMoeglicheInhalte(tm.getSelectedMoeglicheInhalteB(), AnzeigeElementTyp.WZG_B);

      /*
       * zu schalten A
       */
      String[] selektiertA = tm.getSelectedZuSchaltenA();
      WVZInhalt zuSchaltenA = verw.getWvzInhalt (AnzeigeElementTyp.WZG_A, selektiertA [0]);
      int prioA = Integer.parseInt(selektiertA [2]);
      
      /*
       * zu schalten B
       */
      String[] selektiertB = tm.getSelectedZuSchaltenB();
      WVZInhalt zuSchaltenB = verw.getWvzInhalt (AnzeigeElementTyp.WZG_B, selektiertB [0]);
      int prioB = Integer.parseInt(selektiertB [2]);
      
      regeln.addRegel(prioA, prioB, moeglicheInhalteA, moeglicheInhalteB, zuSchaltenA, zuSchaltenB, schaltgrund);
		}

		return regeln;
  }

	/**
	 * Methode liest die Namen der Inhalte und erzeugt daraus die WVZ-Inhalte
	 * @param inhalte Namen der Inhalte
	 * @return Liste mit WVZ-Inhalten
	 */
	private List<WVZInhalt> leseMoeglicheInhalte (String[] inhalte, AnzeigeElementTyp typ)
	{
    VerwaltungWvzInhalte verw = VerwaltungWvzInhalte.getInstance();
    
    List<String> tmp = new ArrayList<String>();
    
    for (String s : inhalte)
    {
    	if (s != null && !s.equals(AnzeigeInhaltTyp.DONT_CARE.getBildInhalt()))
    		if (!tmp.contains(s))
    			tmp.add(s);
    }

    List<WVZInhalt> l = new ArrayList<WVZInhalt>();
    for (String s : tmp)
    	l.add (verw.getWvzInhalt (typ, s));
    
    return l;
	}
	
	/**
	 * Methode bernimmt die Daten eines anderen Folders
	 * @param tvAnzeige Tableviewer der zu bernhmenden Tabelle
	 */
	public void setTvAnzeige (TableViewer tvAnzeige)
	{
    VerwaltungWvzInhalte verw = VerwaltungWvzInhalte.getInstance();
    
    TableModel[] modelSet = (TableModel[]) tvAnzeige.getInput();

    Vector<RegelZusammenfassungEnde> v = new Vector<RegelZusammenfassungEnde>();

    for (int i=0; i<modelSet.length; ++i)
    {
      String nameSchaltgrund = modelSet [i].getSelectedSchaltgrund();
      Aspect schaltgrund = Tools.getAspekt(nameSchaltgrund);

  	  List<WVZInhalt> moeglichA = new ArrayList<WVZInhalt>();
  	  for (String s : modelSet [i].getSelectedMoeglicheInhalteA())
  	  	moeglichA.add(verw.getWvzInhalt(AnzeigeElementTyp.WZG_A, s));

  	  List<WVZInhalt> moeglichB = new ArrayList<WVZInhalt>();
  	  for (String s : modelSet [i].getSelectedMoeglicheInhalteB())
  	  	moeglichB.add(verw.getWvzInhalt(AnzeigeElementTyp.WZG_B, s));

  	  String s [] = modelSet [i].getSelectedZuSchaltenA();
  	  WVZInhalt zuSchaltenA = verw.getWvzInhalt(AnzeigeElementTyp.WZG_A, s [0]);
  	  
  	  int prioA = -1;
  	  
  	  s = modelSet [i].getSelectedZuSchaltenB();
  	  WVZInhalt zuSchaltenB = verw.getWvzInhalt(AnzeigeElementTyp.WZG_B, s [0]);
  	  
  	  int prioB = -1;
  	  
  	  v.add (new RegelZusammenfassungEnde(schaltgrund, moeglichA, moeglichB, zuSchaltenA, zuSchaltenB, prioA, prioB));
    }
    
    _aktuelleParameter.setParameter(v);
    updateData(_aktuelleParameter);
  }
}
