package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.prio;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse enthlt die Parameter pro Anzeigetyp (Typ A, Typ B, etc.)
 * @author Dambach Werke GmbH
 */
class RegelnProAnzeigeTyp
{
  /**
   * Parameter in einer Hashmap. Als Key dient der Anzeigetyp, als Value eine Liste mit
   * Parameterstzen
   */
  private HashMap<SystemObject, ListeMitRegeln> m_parameter = new HashMap<SystemObject, ListeMitRegeln> ();    
  
  /**
   * Parameterloser Konstruktor der Klasse
   */
  public RegelnProAnzeigeTyp ()
  {
  }
  
  /**
   * Methode liefert die Parameter fr einen Anzeigetyp
   * @param anzeigeTyp Anzeigetyp
   * @return Parameter fr diesen Anzeigetyp
   */
  public ListeMitRegeln getParameter (SystemObject anzeigeTyp)
  {
    synchronized (m_parameter)
    {
      if (!m_parameter.containsKey( anzeigeTyp ))
      {
        ListeMitRegeln liste = new ListeMitRegeln();
        m_parameter.put( anzeigeTyp, liste );
      }

      return m_parameter.get(anzeigeTyp );
    }
  }

  /**
   * Methode setzt die Variable parameter
   * @param parameter Wert fr parameter
   */
  public void setParameter(HashMap<SystemObject, ListeMitRegeln> parameter)
  {
    synchronized (m_parameter)
    {
      m_parameter = parameter; 
    }
  }

  /**
   * Methode zum Auswerten der Attributliste 'atl.sswSbaRegelPriorisierungProAnzeigeTyp'
   * @param array empfangene Daten vom DaV
   */
  public void auswerteAttributgruppe (Array array)
  {
    int anzahl = array.getLength();
    
    synchronized (m_parameter)
    {
      m_parameter.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        Data d1 = array.getItem( x );
        
        SystemObject anzeigeTyp = d1.getReferenceValue( "AnzeigeTyp" ).getSystemObject();
      
        ListeMitRegeln liste = new ListeMitRegeln();
        liste.auswerteAttributgruppe( d1 );

        m_parameter.put (anzeigeTyp, liste);
      }
    }
  }
  
  /**
   * Methode lscht die Parameter
   */
  public void clear ()
  {
    synchronized (m_parameter)
    {
      m_parameter.clear();
    }
  }
  
  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    synchronized (m_parameter)
    {
      Iterator<Entry<SystemObject, ListeMitRegeln>> it = m_parameter.entrySet().iterator();
      while (it.hasNext())
      {
        Entry<SystemObject, ListeMitRegeln> entry = it.next();
        
        SystemObject wvzTyp = (SystemObject) entry.getKey();
        ListeMitRegeln lmr = (ListeMitRegeln) entry.getValue();
        
        s += "WVZTyp: " + Tools.getObjektName(  wvzTyp ) + "\n";
        s += lmr.toString() + "\n";
      }
    }

    return s;
  }

  /**
   * Methode liefert die Variable m_parameter zurck
   * @return Variable m_parameter
   */
  public HashMap<SystemObject, ListeMitRegeln> getParameter()
  {
    return m_parameter;
  }

  /**
   * Methode fgt einen Parametersatz fr einen Anzeigetyp hinzu
   * @param anzeigeTyp Anzeigetyp
   * @param regeln Regeln fr diesen Anzeigetyp
   */
	public void addParameter(SystemObject anzeigeTyp, ListeMitRegeln regeln)
  {
	  synchronized (m_parameter)
    {
	    m_parameter.put(anzeigeTyp, regeln);
    }
  }
	
	/**
	 * Methode prft ob die Parameter versorgt sind
	 * @return
	 */
	public boolean isVersorgt ()
	{
		int anzahl = 0;
		
		Iterator<Entry<SystemObject, ListeMitRegeln>> it = m_parameter.entrySet().iterator();
		while (it.hasNext())
		{
			Entry<SystemObject, ListeMitRegeln> entry = it.next();
			
			ListeMitRegeln regeln = entry.getValue();
		
			anzahl += regeln.getParameter().size();
		}
		
		return anzahl > 0;
	}
}
