package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.steuerungskern;

import java.util.TreeMap;
import java.util.Vector;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.List;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Darstellung des Fensters zur Parameternderung ohne ButtonLeiste
 * 
 * @author Krause
 *
 */
public class CopyOfCompositeAnzeigen extends Composite
{
	/**
	 * Text keine Diagnosedaten
	 */
	private final String _keineDiagnose = "  - keine Diagnosedaten -";
	
  /**
   * Liste mit den moeglichen Anzeigen
   */
  private Vector<SystemObject> _moeglichErmittelnUndVersenden = new Vector<SystemObject>();  

  /**
   * Liste mit den Anzeigen die ermittelt und versendet werden sollen
   */
  private Vector<SystemObject> _ermittelnUndVersenden = new Vector<SystemObject>();  

  /**
   * Moegliche Anzeigen
   */
  private List _listeMoeglichErmittelnUndVersenden;

  /**
   * Anzeigen die ermittelt und versendet werden sollen
   */
  private List _listeErmittelnUndVersenden;

  /**
   * Button nach rechts
   */
  private Button buttonNachRechts;

  /**
   * Button nach links
   */
  private Button buttonNachLinks;

  /**
   * Viewer mgliche Anzeigen
   */
	private ListViewer _lvMoeglichErmittelnUndVersenden;

	/**
	 * Viewer Anzeigen die ermittelt und versendet werden sollen
	 */
	private ListViewer _lvErmittelnUndVersenden;

	/**
	 * Vektor mit den mglichen Anzeigen
	 */
	private Vector<SystemObject> _anzeigen = new Vector<SystemObject>();

	/**
	 * Aktuelle Parameter
	 */
	private ParaAnzeigen _parameter;
	
  /**
   * Konstruktor der Klasse
   * @param parent Parent Composite
   * @param style Stil
   * @param parameter aktuelle Parameter
   */
	CopyOfCompositeAnzeigen(Composite parent, int style, ParameterSteuerungsKern parameter) 
  {
    super(parent, style);
    
    _parameter = parameter.getParaAnzeigen();
   
    initialize();
  }

	/**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;

    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    
    bestimmeAnzeigen();
    
    if (_parameter.getErmittelndUndVersenden() != null)
    {
      _ermittelnUndVersenden.clear();
      for (SystemObject so :_parameter.getErmittelndUndVersenden())
      	_ermittelnUndVersenden.add(so);
    }
    
    createCompositeErmittelnUndVersenden(this);
   
    initialisiereAnzeigen();
  }

  /**
   * Methode bestimmt die im DaV definierten Anzeigen
   */
	private void bestimmeAnzeigen()
  {
		TreeMap<String, SystemObject> sorter = new TreeMap<String, SystemObject>();
		
		java.util.List<SystemObject> objekte = Konfig.getInstance().getAnzeigen();
    for (SystemObject so : objekte)
    	sorter.put(getBezeichnung(so), so);
    
    for (SystemObject so : sorter.values())
		  _anzeigen.add(so);
  }

	/**
   * Methode fgt die Anzeigen hinzu
   */
  private void initialisiereAnzeigen ()
  {
    for (SystemObject so : _anzeigen)
    {
    	if (!_moeglichErmittelnUndVersenden.contains(so) && !_ermittelnUndVersenden.contains(so))
  		  _moeglichErmittelnUndVersenden.add(so);
    }
  	
    _lvMoeglichErmittelnUndVersenden.refresh();
  }  

  /**
   * Methode legt ein Composite an in dem die Anzeigen "Ermitteln und Versenden" versorgt werden knnen
   * @param parent Parent Composite
   */
  private void createCompositeErmittelnUndVersenden (Composite parent)
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.marginHeight = 5;
    fillLayout1.marginWidth = 5;
    fillLayout1.type = org.eclipse.swt.SWT.HORIZONTAL;
    
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    
    Group eingang = new Group(parent, SWT.BORDER);
  	eingang.setLayout(fillLayout1);
  	eingang.setLayoutData(gridData2);
  	eingang.setText("Eingangsaspekte:");

    // Zeile 0
    
    GridData gdZeile0 = new GridData ();
    gdZeile0.horizontalAlignment = GridData.FILL;
    gdZeile0.verticalAlignment = GridData.FILL;
    gdZeile0.grabExcessHorizontalSpace = true;
    gdZeile0.grabExcessVerticalSpace = false;

    Group zeile0 = new Group (eingang, SWT.NONE);
    zeile0.setLayout(new GridLayout (3, false));
  	
    // Zeile 0 Spalte 1
    
    GridData gdZeile0Spalte1 = new GridData ();
    gdZeile0Spalte1.horizontalAlignment = GridData.FILL;
    gdZeile0Spalte1.verticalAlignment = GridData.FILL;
    gdZeile0Spalte1.grabExcessHorizontalSpace = true;
    gdZeile0Spalte1.grabExcessVerticalSpace = true;
    gdZeile0Spalte1.minimumWidth = 200;

    Composite zeile0Spalte1 = new Composite (zeile0, SWT.BORDER);
    zeile0Spalte1.setLayout(new FillLayout ());
    zeile0Spalte1.setLayoutData(gdZeile0Spalte1);

    // Zeile 0 Spalte 2

    GridData gdZeile0Spalte2 = new GridData ();
    gdZeile0Spalte2.horizontalAlignment = GridData.CENTER;
    gdZeile0Spalte2.grabExcessHorizontalSpace = false;
    gdZeile0Spalte2.minimumWidth = 80;

    Composite zeile0Spalte2 = new Composite (zeile0, SWT.BORDER);
    zeile0Spalte2.setLayout(new GridLayout (1, true));
    zeile0Spalte2.setLayoutData(gdZeile0Spalte2);

    // Zeile 0 Spalte 3

    GridData gdZeile0Spalte3 = new GridData ();
    gdZeile0Spalte3.horizontalAlignment = GridData.FILL;
    gdZeile0Spalte3.verticalAlignment = GridData.FILL;
    gdZeile0Spalte3.grabExcessHorizontalSpace = true;
    gdZeile0Spalte3.grabExcessVerticalSpace = true;
    gdZeile0Spalte3.minimumWidth = 200;

    Composite zeile0Spalte3 = new Composite (zeile0, SWT.BORDER);
    zeile0Spalte3.setLayout(new GridLayout(1, false));
    zeile0Spalte3.setLayoutData(gdZeile0Spalte3);

    addMoeglicheAnzeigen(zeile0Spalte1);
    
    addKopierenButtons(zeile0Spalte2);
    
    addErmittelnUndVersenden(zeile0Spalte3);
  }

  /**
   * Fgt die Liste mit den mglichen Funktionen hinzu
   * @param parent Composite in das die Liste eingefgt werden sollen
   */
  private void addMoeglicheAnzeigen (Composite parent)
  {
    Group group = new Group (parent, SWT.BORDER);
    group.setText("Mgliche Anzeigen");
    group.setLayout(new FillLayout(SWT.VERTICAL));

    _listeMoeglichErmittelnUndVersenden = new List (group, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
    _listeMoeglichErmittelnUndVersenden.addSelectionListener(new SelectionListener () 
    {
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      public void widgetSelected(SelectionEvent e)
      {
      	boolean anwahlMoeglich = true;
      	
     		anwahlMoeglich = _listeMoeglichErmittelnUndVersenden.getSelectionCount() != 0;
      	
        buttonNachRechts.setEnabled(anwahlMoeglich);
      }
    });
    
    _listeMoeglichErmittelnUndVersenden.addMouseListener(new MouseListener()
		{
			
			@Override
			public void mouseUp(MouseEvent e)
			{
			}
			
			@Override
			public void mouseDown(MouseEvent e)
			{
			}
			
			@Override
			public void mouseDoubleClick(MouseEvent e)
			{
        actionNachRechts();		
			}
		});
    
    _lvMoeglichErmittelnUndVersenden = new ListViewer(_listeMoeglichErmittelnUndVersenden);
    _lvMoeglichErmittelnUndVersenden.setContentProvider(new IStructuredContentProvider()
		{
			@Override
			public void dispose()
			{
			}
			
			@Override
			public Object[] getElements(Object inputElement)
			{
        Vector v = (Vector)inputElement;
        return v.toArray();
			}

			@Override
      public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
      {
      }
		});
    
    _lvMoeglichErmittelnUndVersenden.setInput(_moeglichErmittelnUndVersenden);
    
    _lvMoeglichErmittelnUndVersenden.setLabelProvider(new LabelProvider() 
    {
      public Image getImage(Object element) 
      {
        return null;
      }

      public String getText(Object element) 
      {
        return (getBezeichnung ((SystemObject) element));
      }
    });
    
    _lvMoeglichErmittelnUndVersenden.setSorter(new ViewerSorter()
    {
      public int compare(Viewer viewer, Object e1, Object e2) 
      {
        return getBezeichnung ((SystemObject)e1).compareTo(getBezeichnung((SystemObject)e2));
      }
    });
  }

  /**
   * Fgt die Liste mit den zugewiesenden Funktionen hinzu
   * @param parent Composite in das die Liste eingefgt werden sollen
   */
  private void addErmittelnUndVersenden (Composite parent)
  {
    Group group = new Group (parent, SWT.BORDER);
    group.setText("Ermitten und Versenden");
    group.setLayout(new FillLayout(SWT.VERTICAL));
    
    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.FILL;
    gd.verticalAlignment = GridData.FILL;
    gd.grabExcessHorizontalSpace = true;
    gd.grabExcessVerticalSpace = true;

    group.setLayoutData(gd);
   
    _listeErmittelnUndVersenden = new List (group, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
    _listeErmittelnUndVersenden.addSelectionListener(new SelectionListener () 
    {
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      public void widgetSelected(SelectionEvent e)
      {
        if (_listeErmittelnUndVersenden.getSelectionCount() != 0)
          buttonNachLinks.setEnabled(true);
        else
          buttonNachLinks.setEnabled(false);
      }
    });

    _listeErmittelnUndVersenden.addMouseListener(new MouseListener()
		{
			@Override
			public void mouseUp(MouseEvent e)
			{
			}
			
			@Override
			public void mouseDown(MouseEvent e)
			{
			}
			
			@Override
			public void mouseDoubleClick(MouseEvent e)
			{
				actionNachLinks();
			}
		});
		
    _lvErmittelnUndVersenden = new ListViewer(_listeErmittelnUndVersenden);
    _lvErmittelnUndVersenden.setContentProvider(new IStructuredContentProvider()
		{
			@Override
			public void inputChanged (Viewer viewer, Object oldInput, Object newInput)
			{
			}
			
			@Override
			public void dispose()
			{
			}
			
			@Override
			public Object[] getElements(Object inputElement)
			{
        Vector v = (Vector)inputElement;
        return v.toArray();
			}
		});
    
    _lvErmittelnUndVersenden.setInput(_ermittelnUndVersenden);
    
    _lvErmittelnUndVersenden.setLabelProvider(new LabelProvider() 
    {
      public Image getImage(Object element) 
      {
        return null;
      }

      public String getText(Object element) 
      {
        return (getBezeichnung ((SystemObject) element));
      }
    });
    
    _lvErmittelnUndVersenden.setSorter(new ViewerSorter()
    {
      public int compare(Viewer viewer, Object e1, Object e2) 
      {
        return getBezeichnung ((SystemObject)e1).compareTo(getBezeichnung((SystemObject)e2));
      }
    });

    if (_ermittelnUndVersenden != null)
	    for (SystemObject inhalt : _ermittelnUndVersenden)
	    	if (_moeglichErmittelnUndVersenden.contains(inhalt))
	        _moeglichErmittelnUndVersenden.remove(inhalt);
    
    _lvMoeglichErmittelnUndVersenden.refresh();
    _lvErmittelnUndVersenden.refresh();
  }

  /**
   * Fgt die Kopierbuttons hinzu
   * @param parent Composite in das die Buttons eingefgt werden sollen
   */
  private void addKopierenButtons (Composite parent)
  {
    Composite composite = new Composite (parent, SWT.NONE);
    
    buttonNachRechts = new Button(composite, SWT.PUSH);
    buttonNachRechts.setText(" Hinzufgen >>");
    buttonNachRechts.setEnabled(false);
    buttonNachRechts.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent e)
      {
        actionNachRechts();
      }
    });

    buttonNachLinks = new Button(composite, SWT.PUSH);
    buttonNachLinks.setText(" << Entfernen ");
    buttonNachLinks.setEnabled(false);
    buttonNachLinks.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent e)
      {
        actionNachLinks();
      }
    });

    /*
     * Layout
     */
    RowLayout rlUnten = new RowLayout(SWT.VERTICAL);
    rlUnten.pack = false;
    composite.setLayout(rlUnten);
  }

  /**
   * Kopiert die selektierten Eintrge von links nach rechts
   */
  private void actionNachRechts()
  {
    IStructuredSelection selection = (IStructuredSelection) _lvMoeglichErmittelnUndVersenden.getSelection();
    StructuredSelection ss = (StructuredSelection) selection;

  	for (int x = 0; x < ss.size(); ++x)
    {
      Object obj1 = ss.toArray()[x];

      if (obj1 instanceof SystemObject)
      {
      	SystemObject as = (SystemObject) obj1;
      	
      	if (!_ermittelnUndVersenden.contains(as))
      		_ermittelnUndVersenden.add(as);
      	
      	_moeglichErmittelnUndVersenden.remove(as);
      }
    }

  	_lvMoeglichErmittelnUndVersenden.refresh();
  	_lvErmittelnUndVersenden.refresh();
  	
    buttonNachRechts.setEnabled(false);
  }

  /**
   * Kopiert die selektierten Eintrge von rechts nach links
   */
  private void actionNachLinks()
  {
    IStructuredSelection selection = (IStructuredSelection) _lvErmittelnUndVersenden.getSelection();
    StructuredSelection ss = (StructuredSelection) selection;

  	for (int x = 0; x < ss.size(); ++x)
    {
      Object obj1 = ss.toArray()[x];

      if (obj1 instanceof SystemObject)
      {
      	SystemObject as = (SystemObject) obj1;
      	
      	if (!_moeglichErmittelnUndVersenden.contains(as))
          _moeglichErmittelnUndVersenden.add(as);
        
        _ermittelnUndVersenden.remove(as);
      }
    }

    _lvMoeglichErmittelnUndVersenden.refresh();  
    _lvErmittelnUndVersenden.refresh();
    	
    buttonNachLinks.setEnabled(false);
  }

  /**
   * Methode liefert die aktuell eingestellten Parameter
   * @return aktuell eingestellten Parameter
   */
	public ParaAnzeigen getParameter()
  {
	  return leseAktuelleEinstellungen();
  }
 
	private ParaAnzeigen leseAktuelleEinstellungen ()
	{
    ParaAnzeigen para = new ParaAnzeigen();
    
    /*
     * Ermitteln und Versenden
     */
    para.setErmittelndUndVersenden(_ermittelnUndVersenden);
    
  	return para;
  }

	/**
	 * Methode liefert eine Anzeige der ber eine Bezeichnung referenziert ist 
	 * @param bezeichnung Beszeichnung der Anzeige (siehe auch Methode getBezeichnung ())
	 * @return Aspekt
	 */
	private SystemObject getAnzeigeByBezeichnung (String bezeichnung)
  {
		if (bezeichnung.equals(_keineDiagnose))
			return null;
		
		for (SystemObject a : _anzeigen)
			if (getBezeichnung(a).equals(bezeichnung))
				return a;
		
	  return null;
  }

	/**
	 * Methode aktualisiert die View
	 */
	public void refresh()
  {
  }
	
	/**
	 * Methode liefert eine Bezeichnung fr einer Anzeige. Die Bezeichnung setzt sich
	 * zusammen aus Name + Pid
	 * @param objekt Systemobjekt der Anzeige
	 * @return Bezeichnung 
	 */
	private String getBezeichnung (SystemObject objekt)
	{
		return objekt.getName() + " (" + objekt.getPid() + ")";
	}
}
