package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemein;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.vorgaengerNachfolger.ParameterVorgaengerNachfolger;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.vorgaengerNachfolger.VerwaltungParaVorgaengerNachfolger;

/**
 * Klasse definiert eine Spalte 
 * @author ObertM
 *
 */
public class Spalte 
{
	/**
	 * Index innerhalb der Datenstruktur
	 */
	private int _index;
	
	/**
	 * Typ des Composites
	 */
	private EVA _eva;
	
	/**
	 * Spaltenberschrift
	 */
	private String _text;

	/**
	 * Indices der dazugehrenden Anzeigen aus der Querbeziehung
	 */
	private int[] _indexAnzeige;

	/**
	 * Konstruktor der Klasse
	 * @param eva Typ des Composites
	 * @param text Spaltenberschrift
	 */
	public Spalte (EVA eva, String text)
	{
	  _eva = eva;
	  
	  _text = text;
	}

	/**
   * Methode liefert index zurck
   * @return index
   */
  public int getIndex()
  {
  	return _index;
  }

	/**
   * Methode setzt die Variable index
   * @param index zu bernehmender Wert fr index
   */
  public void setIndex(int index)
  {
  	_index = index;
  }

	/**
   * Methode liefert eva zurck
   * @return eva
   */
  public EVA getEva()
  {
  	return _eva;
  }

	/**
   * Methode setzt die Variable eva
   * @param eva zu bernehmender Wert fr eva
   */
  public void setEva(EVA eva)
  {
  	_eva = eva;
  }

	/**
   * Methode liefert text zurck
   * @return text
   */
  public String getText()
  {
  	return _text;
  }

	/**
   * Methode setzt die Variable text
   * @param text zu bernehmender Wert fr text
   */
  public void setText(String text)
  {
  	_text = text;
  }

	/**
   * Methode liefert indexAnzeige zurck
   * @return indexAnzeige
   */
  public int[] getIndexAnzeige()
  {
  	return _indexAnzeige;
  }

	/**
   * Methode setzt die Variable indexAnzeige
   * @param indexAnzeige zu bernehmender Wert fr indexAnzeige
   */
  public void setIndexAnzeige(int[] indexAnzeige)
  {
  	_indexAnzeige = indexAnzeige;
  }
  
  /**
   * Methode liefert die zu dieser Spalte gehrende Anzeige
   * @param soAQ Systemobjekt des Anzeigequerschnitts
   * @param index Index innerhalb der Anzeigen der Spalte
   * @return Systemobjekt der Anzeige
   */
  public SystemObject getAnzeige (SystemObject soAQ, int index)
  {
  	if (index >= _indexAnzeige.length)
  		throw new IllegalArgumentException("Falscher Index bergeben " + index);
  	
  	VerwaltungParaVorgaengerNachfolger verw = VerwaltungParaVorgaengerNachfolger.getInstance();
  	
  	ParameterVorgaengerNachfolger para = verw.getParameter();
  	
  	SystemObject[] anzeigenAusQL = para.getQuerbeziehung(soAQ);
  	
//  	SystemObject[] anzeigenAusDefault = DefaultEinstellungen.getDefaultQuerBeziehung(soAQ);
  	
  	int indexAnzeige = _indexAnzeige [index];
  	if (indexAnzeige >= anzeigenAusQL.length)
  		throw new IllegalArgumentException("Falscher Index innehalb des AQ " + indexAnzeige);
  	
  	return anzeigenAusQL [_indexAnzeige [index]];
  }
}
