package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemein;

import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;

import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemeineVerriegelungsMatrix.CompositeA;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemeineVerriegelungsMatrix.CompositeABlinkerBC;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemeineVerriegelungsMatrix.CompositeAD;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemeineVerriegelungsMatrix.CompositeBC;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemeineVerriegelungsMatrix.CompositeBlinkerBC;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemeineVerriegelungsMatrix.CompositeBlinkerTAQ;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemeineVerriegelungsMatrix.CompositeTAQ;

/**
 * Tabellen Modell fr die Parameter Allgemeine Verriegelungsmatrix
 * @author ObertM
 */
public class TableModelVerriegelung extends Observable implements Observer
{
	/**
	 * Merker zum Lschen der Zeile
	 */
	private Composite _loeschen;
	
	/**
	 * Schaltgrund
	 */
  private Composite _schaltgrund;
  
  /**
   * Arbeitsrichtung
   */
  private Composite _richtung;

  /**
   * TAQ Zeichen
   */
  private CompositeTAQ[] _taq = new CompositeTAQ [0];

  /**
   * Blinker/TAQ Kombination
   */
  private CompositeBlinkerTAQ[] _blinkerTaq = new CompositeBlinkerTAQ [0];

  /**
   * A Zeichen 
   */
  private CompositeA[] _a = new CompositeA [0];

  /**
   * B/C Kombination
   */
  private CompositeBC[] _bc = new CompositeBC [0];

  /**
   * A/D Zeichen Kombination
   */
  private CompositeAD[] _ad = new CompositeAD [0];

  /**
   * Blinker/B/C Kombination 
   */
  private CompositeBlinkerBC[] _blinkerBC = new CompositeBlinkerBC [0];

  /**
   * Zeichen ABC (links)
   */
  private CompositeABlinkerBC[] _aBlinkerBC = new CompositeABlinkerBC [0];

  /**
   * Konstruktor der Klasse
   * @param loeschen Composite mit dem Lschen button
   * @param schaltgrund Composite mit der Schaltgrund ComboBox
   * @param richtung Composite mit der Richtung ComboBox
   * @param composites Composites mit den Labeln
   */
  public TableModelVerriegelung (Composite loeschen, Composite schaltgrund, Composite richtung,
  		Vector<Composite> composites) 
  {
  	this._loeschen = loeschen;
    this._schaltgrund = schaltgrund;
    this._richtung = richtung;
    
    initialisiereComposites (composites);
  }

  /**
   * Konstruktor der Klasse
   * @param loeschen Composite mit dem Lschen button
   * @param schaltgrund Composite mit der Schaltgrund ComboBox
   * @param composites Composites mit den Labeln
   */
  public TableModelVerriegelung (Composite loeschen, Composite schaltgrund,
  		Vector<Composite> composites) 
  {
  	this._loeschen = loeschen;
    this._schaltgrund = schaltgrund;
    this._richtung = null;
    
    initialisiereComposites (composites);
  }

  /**
   * Methode zum Anlegen der bentigten Composites
   * @param composites bergebene Composites
   */
  private void initialisiereComposites(Vector<Composite> composites)
  {
    int anzahlA          = 0;
    int anzahlBC         = 0;
    int anzahlTAQ        = 0;
    int anzahlAD         = 0;
    int anzahlBlinkerTaq = 0;
    int anzahlBlinkerBc  = 0;
    int anzahlABlinkerBc  = 0;
    
    for (Composite c : composites)
    {
    	if (c instanceof CompositeA)
    		anzahlA++;

    	if (c instanceof CompositeBC)
    		anzahlBC++;

    	if (c instanceof CompositeTAQ)
    		anzahlTAQ++;

    	if (c instanceof CompositeAD)
    		anzahlAD++;

    	if (c instanceof CompositeBlinkerTAQ)
    		anzahlBlinkerTaq++;

    	if (c instanceof CompositeBlinkerBC)
    		anzahlBlinkerBc++;

    	if (c instanceof CompositeABlinkerBC)
    		anzahlABlinkerBc++;
    }

    _a  = new CompositeA  [anzahlA];
    _bc = new CompositeBC [anzahlBC];
    _taq  = new CompositeTAQ  [anzahlTAQ];
    _ad = new CompositeAD [anzahlAD];
    _blinkerTaq = new CompositeBlinkerTAQ [anzahlBlinkerTaq];
    _blinkerBC = new CompositeBlinkerBC [anzahlBlinkerBc];
    _aBlinkerBC = new CompositeABlinkerBC [anzahlABlinkerBc];
    
    int indexA  = 0;
    int indexBC = 0;
    int indexT  = 0;
    int indexAD = 0;
    int indexBlinkerTaq = 0;
    int indexBlinkerBc = 0;
    int indexABlinkerBc = 0;
    
    for (Composite c : composites)
    {
    	if (c instanceof CompositeA)
    		_a [indexA++] = (CompositeA) c;

    	if (c instanceof CompositeBC)
    		_bc [indexBC++] = (CompositeBC) c;

    	if (c instanceof CompositeTAQ)
    		_taq [indexT++] = (CompositeTAQ) c;

    	if (c instanceof CompositeAD)
    		_ad [indexAD++] = (CompositeAD) c;

    	if (c instanceof CompositeBlinkerTAQ)
    		_blinkerTaq [indexBlinkerTaq++] = (CompositeBlinkerTAQ) c;

    	if (c instanceof CompositeBlinkerBC)
    		_blinkerBC [indexBlinkerBc++] = (CompositeBlinkerBC) c;

    	if (c instanceof CompositeABlinkerBC)
    		_aBlinkerBC [indexABlinkerBc++] = (CompositeABlinkerBC) c;
    }
  }

  /**
   * Methode zum Auslesen des eingestellten Wertes Lschen
   * @return eingestellter Wert
   */
  public String getSelectedLoeschen()
  {
    return getSelectedWert(_loeschen.getChildren()[0]);    
  }

  /**
   * Methode zum Auslesen des eingestellten Wertes Schaltgrund
   * @return eingestellter Wert
   */
  public String getSelectedSchaltgrund ()
  {
    return getSelectedWert(_schaltgrund.getChildren()[0]);    
  }

  /**
   * Methode zum Auslesen des eingestellten A Wertes
   * @param index Index der A Komponente 
   * @return eingestellter Wert
   */
  public String getSelectedA (int index)
  {
  	if (index >= _a.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
    return getSelectedWert(_a [index].getA());    
  }

  /**
   * Methode zum Auslesen des eingestellten T Wertes
   * @param index Index der T Komponente 
   * @return eingestellter Wert
   */
  public String getSelectedTAQ (int index)
  {
  	if (index >= _taq.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
    return getSelectedWert(_taq [index].getTAQ());    
  }

  /**
   * Methode zum Auslesen des eingestellten B Wertes 
   * @param index Index der BV Komponente
   * @return eingestellter Wert
   */
  public String getSelectedBC_B (int index)
  {
  	if (index >= _bc.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
    return getSelectedWert(_bc [index].getB());    
  }

  /**
   * Methode zum Auslesen des eingestellten C Wertes 
   * @param index Index der BV Komponente
   * @return eingestellter Wert
   */
  public String getSelectedBC_C (int index)
  {
  	if (index >= _bc.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
    return getSelectedWert(_bc [index].getC());    
  }

  /**
   * Methode zum Auslesen des eingestellten A Wertes 
   * @param index Index der AD Komponente
   * @return eingestellter Wert
   */
  public String getSelectedAD_A (int index)
  {
  	if (index >= _ad.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
    return getSelectedWert(_ad [index].getA());    
  }

  /**
   * Methode zum Auslesen des eingestellten D Wertes 
   * @param index Index der AD Komponente
   * @return eingestellter Wert
   */
  public String getSelectedAD_D (int index)
  {
  	if (index >= _ad.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
    return getSelectedWert(_ad [index].getD());    
  }

  /**
   * Methode zum Auslesen des eingestellten Blinker Wertes 
   * @param index Index der Blinker/TAQ Komponente
   * @return eingestellter Wert
   */
  public String getSelectedBlinkerTAQ_Blinker (int index)
  {
  	if (index >= _blinkerTaq.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
    return getSelectedWert(_blinkerTaq [index].getBlinker());    
  }

  /**
   * Methode zum Auslesen des eingestellten TAQ Wertes 
   * @param index Index der Blinker/TAQ Komponente
   * @return eingestellter Wert
   */
  public String getSelectedBlinkerTAQ_TAQ (int index)
  {
  	if (index >= _blinkerTaq.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
    return getSelectedWert(_blinkerTaq [index].getTAQ());    
  }

  /**
   * Methode zum Setzen des Wertes Lschen
   * @param selWert zu setzender Wert
   */
  public void setSelectedLoeschen(String selWert)
  {
    setSelectedWert(_loeschen.getChildren()[0], selWert, null);
  }

  /**
   * Methode zum Setzen des Wertes Schaltgrund
   * @param selWert zu setzender Wert
   */
  public void setSelectedSchaltgrund(String selWert)
  {
    setSelectedWert(_schaltgrund.getChildren()[0], selWert, null);
  }

  /**
   * Methode liest den eingestellten Wert eines Controls eines Composite
   * @param control Control eines Composite
   * @return eingestellter Wert als String
   */
  public String getSelectedWert(Control control)
  {
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      int spSel = sp.getSelection();
      return String.valueOf(spSel);
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      String cbSel = cb.getItem( cb.getSelectionIndex() );
      return cbSel;
    }
    else if ( control instanceof Button )
    {
      if ( (control.getStyle() & SWT.CHECK) != 0 )
      {
        Button ch = (Button)control;
        boolean chSel = ch.getSelection();
        return String.valueOf(chSel);
      }
      else
        return "";
    }
    else if ( control instanceof Label )
    {
      Label im = (Label)control;
      String imSel = (String)im.getData();
      return imSel;
    }
    else
      return "";
  }

  /**
   * Methode setzt den Wert eines Controls eines Composites
   * @param control Control
   * @param selWert Wert als String
   * @param selImage Wert als Image
   */
  public void setSelectedWert(Control control, String selWert, Image selImage)
  {
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      try
      {
        sp.setSelection(Integer.valueOf( selWert ));
      }
      catch (NumberFormatException e)
      {
        e.printStackTrace();
      }
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      for (int k = 0; k < cb.getItemCount(); k++)
      {
        if (selWert.equals(cb.getItem( k )))
          cb.select(k);      
      }
    }
    else if ( control instanceof Button )
    {
      if ( (control.getStyle() & SWT.CHECK) != 0 )
      {
        Button ch = (Button)control;
        ch.setSelection( Boolean.valueOf( selWert ) );
      }
    }
    else if ( control instanceof Label )
    {
        Label im = (Label)control;
        im.setData( selWert );
        im.setImage( selImage ); 
    }
  }

  @Override
  public void update(Observable o, Object arg)
  {
    fireEvent();
  }

  public void fireEvent()
  {
    setChanged();
    notifyObservers();
  }

	/**
   * Methode liefert loeschen zurck
   * @return loeschen
   */
  public Composite getLoeschen()
  {
  	return _loeschen;
  }

	/**
   * Methode setzt die Variable loeschen
   * @param loeschen zu bernehmender Wert fr loeschen
   */
  public void setLoeschen(Composite loeschen)
  {
  	_loeschen = loeschen;
  }

	/**
   * Methode liefert schaltgrund zurck
   * @return schaltgrund
   */
  public Composite getSchaltgrund()
  {
  	return _schaltgrund;
  }

	/**
   * Methode setzt die Variable schaltgrund
   * @param schaltgrund zu bernehmender Wert fr schaltgrund
   */
  public void setSchaltgrund(Composite schaltgrund)
  {
  	_schaltgrund = schaltgrund;
  }

  /**
   * Methode liefert eine A Komponente zurck
   * @param index Index der A Komponente
   * @return A Komponente
   */
  public CompositeA getA (int index)
  {
  	if (index >= _a.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
  	return _a [index];
  }

  /**
   * Methode liefert eine T Komponente zurck
   * @param index Index der T Komponente
   * @return T Komponente
   */
  public CompositeTAQ getTAQ (int index)
  {
  	if (index >= _taq.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
  	return _taq [index];
  }

  /**
   * Methode liefert eine BC Komponente zurck
   * @param index Index der BC Komponente
   * @return BC Komponente
   */
  public CompositeBC getBC (int index)
  {
  	if (index >= _bc.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
  	return _bc [index];
  }

  /**
   * Methode liefert eine AD Komponente zurck
   * @param index Index der AD Komponente
   * @return BC Komponente
   */
  public CompositeAD getAD (int index)
  {
  	if (index >= _ad.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
  	return _ad [index];
  }

  /**
   * Methode liefert eine Blinker/TAQ Komponente zurck
   * @param index Index der Blinker/TAQ Komponente
   * @return Blinker/TAQ Komponente
   */
  public CompositeBlinkerTAQ getBlinkerTAQ (int index)
  {
  	if (index >= _blinkerTaq.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
  	return _blinkerTaq [index];
  }

  /**
   * Methode liefert eine Blinker/B/C Komponente zurck
   * @param index Index der Blinker/B/C Komponente
   * @return Blinker/B/C Komponente
   */
  public CompositeBlinkerBC getBlinkerBC (int index)
  {
  	if (index >= _blinkerBC.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
  	return _blinkerBC [index];
  }

  /**
   * Methode liefert eine A/Blinker/B/C Komponente zurck
   * @param index Index der A/Blinker/B/C Komponente
   * @return A/Blinker/B/C Komponente
   */
  public CompositeABlinkerBC getABlinkerBC (int index)
  {
  	if (index >= _aBlinkerBC.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
  	return _aBlinkerBC [index];
  }

  /**
   * Methode zum Setzen des Wertes Richtung
   * @param selWert zu setzender Wert
   */
  public void setSelectedRichtung(String selWert)
  {
    setSelectedWert(_richtung.getChildren()[0], selWert, null);
  }

	/**
   * Methode liefert richtung zurck
   * @return richtung
   */
  public Composite getRichtung()
  {
  	return _richtung;
  }

  /**
   * Methode zum Auslesen des eingestellten Wertes Richtung
   * @return eingestellter Wert
   */
  public String getSelectedRichtung ()
  {
    return getSelectedWert(_richtung.getChildren()[0]);    
  }

  /**
   * Methode zum Auslesen des eingestellten A Wertes 
   * @param index Index der A Komponente
   * @return eingestellter Wert
   */
  public String getSelectedABlinkerBC_A (int index)
  {
  	if (index >= _aBlinkerBC.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
    return getSelectedWert(_aBlinkerBC [index].getA());    
  }

  /**
   * Methode zum Auslesen des eingestellten Blinker Wertes 
   * @param index Index der Blinker Komponente
   * @return eingestellter Wert
   */
  public String getSelectedABlinkerBC_Blinker (int index)
  {
  	if (index >= _aBlinkerBC.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
    return getSelectedWert(_aBlinkerBC [index].getBlinker());    
  }

  /**
   * Methode zum Auslesen des eingestellten B Wertes 
   * @param index Index der B Komponente
   * @return eingestellter Wert
   */
  public String getSelectedABlinkerBC_B (int index)
  {
  	if (index >= _aBlinkerBC.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
    return getSelectedWert(_aBlinkerBC [index].getB());    
  }
  
  /**
   * Methode zum Auslesen des eingestellten C Wertes 
   * @param index Index der C Komponente
   * @return eingestellter Wert
   */
  public String getSelectedABlinkerBC_C (int index)
  {
  	if (index >= _aBlinkerBC.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
    return getSelectedWert(_aBlinkerBC [index].getC());    
  }

  /**
   * Methode zum Auslesen des eingestellten Blinker Wertes 
   * @param index Index der Blinker Komponente
   * @return eingestellter Wert
   */
  public String getSelectedBlinkerBC_Blinker (int index)
  {
  	if (index >= _blinkerBC.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
    return getSelectedWert(_blinkerBC [index].getBlinker());    
  }

  /**
   * Methode zum Auslesen des eingestellten B Wertes 
   * @param index Index der B Komponente
   * @return eingestellter Wert
   */
  public String getSelectedBlinkerBC_B (int index)
  {
  	if (index >= _blinkerBC.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
    return getSelectedWert(_blinkerBC [index].getB());    
  }
  
  /**
   * Methode zum Auslesen des eingestellten C Wertes 
   * @param index Index der C Komponente
   * @return eingestellter Wert
   */
  public String getSelectedBlinkerBC_C (int index)
  {
  	if (index >= _blinkerBC.length)
  		throw new IllegalArgumentException("Falscher Index: " + index);
  	
    return getSelectedWert(_blinkerBC [index].getC());    
  }

}
