/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verschiebungen.verschieben;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.Map.Entry;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.DaVKommunikationSBA;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Wenn  angeforderter Inhalt = "X" und Anzeigeinhalt != "X" und<br> 
 * Schaltgrund (angeforderter Inhalt "X") nicht in ["u", "v", "w", ]:<br>
 * dann wiederhole bis Ebene "n" erreicht oder Inhalt angefordert<br>
 * Wenn Vorgnger/Nachfolger in Lngsrichtung von WZGTyp = "A" und "X" im Zeichensatz enthalten und<br>
 * Prioritt von "X" hher als dort aktueller Anzeigeinhalt<br> 
 * dann bei diesem Vorgnger/Nachfolger "X" anfordern (mit der Prioritt des ursprnglichen Schaltgrunds an dieser Anzeige).
 * Es muss angegeben werden knnen, ob abhngige Zeichen mit verschoben werden sollen.
 * @version $Revision: 1.1 $ / $Date: 2013/06/25 09:27:18 $ / ($Author: ObertM $)
 * @author Dambach-Werke GmbH
 * @author Matthias Obert
 *
 */
public class VerwaltungParaVerschieben 
{
	/**
	 * Einzige Instanz der Klasse
	 */
  private static VerwaltungParaVerschieben _instance;

  /**
   * Debug Ausgabe
   */
  private boolean _debug = false;

  /**
   * Pid der angeforderten Attributgruppe
   */
  private String _pidAtg = "atg.sswSbaVerschieben";
  
  /**
   * Allgemeiner Schaltgrund fr die Regelklasse
   */
  private Aspect m_schaltgrundAllgemein;
  
  /**
   * Parametersatz fr die Anlage
   */
  private RegelnProAnzeigeTyp m_parameterAnlage = new RegelnProAnzeigeTyp ();
  
  /**
   * Parametersatz pro Anzeigequerschnitt
   */
  private HashMap<SystemObject, RegelnProAnzeigeTyp> m_parameterProAnzeigeQuerschnitt = new HashMap<SystemObject, RegelnProAnzeigeTyp>();
  
  /**
   * Parametersatz pro Anzeige
   */
  private HashMap<SystemObject, RegelnProAnzeigeTyp> m_parameterProAnzeige = new HashMap<SystemObject, RegelnProAnzeigeTyp>();
  
  /**
   * Verwaltung der Listener
   */
  private PropertyChangeSupport m_pcs = new PropertyChangeSupport (this);

  /**
   * Mutex fr den synchronisierten Zugriff auf die Datenstruktur
   */
  private Object m_mutex = new Object();

  /**
   * Pid der Anlage
   */
  private String m_anlage;
  
  /**
   * Pid des Defaultaspekts
   */
  public static String _pidDefaultAspekt = "asp.sswSbaVerschieben";
  
  /**
   * Pid der SBA
   */
  private String _pidSBA;

  /**
   * Privater Konstruktor der Klasse
   */
  private VerwaltungParaVerschieben ()
  {
  	_pidSBA = Konfig.getInstance().getPidSBA();
    
    m_schaltgrundAllgemein = Konfig.getInstance().getAspekt(_pidDefaultAspekt);

    /*
     * Daten lesen ber DaVKommunikation
     */
  	final DaVKommunikationSBA kom = DaVKommunikationSBA.getInstanz();
  	kom.addDatenArt(_pidSBA, _pidAtg, "asp.parameterSoll", ReceiverRole.receiver());
  	
  	kom.addPropertyChangeListener(new PropertyChangeListener()
		{
			@Override
			public void propertyChange(PropertyChangeEvent evt)
			{
				if (evt.getPropertyName().equals(_pidAtg))
				{
					ResultData resultData = kom.getData(_pidAtg);
					if (resultData != null)
						auswerteAttributgruppe(resultData);
				}
			}
		});

		ResultData resultData = kom.getData(_pidAtg);
		if (resultData != null)
			auswerteAttributgruppe(resultData);
  }
  
  /**
   * Methode liefert die Singleton Instanz der Klasse
   * @return Singleton Instanz der Klasse
   */
  public static VerwaltungParaVerschieben getInstance ()
  {
    if (_instance == null)
    {
      synchronized (VerwaltungParaVerschieben.class)
      {
        if (_instance == null)
        {
          _instance = new VerwaltungParaVerschieben ();
        }
      }
    }
    
    return _instance;
  }

  /**
   * Methode liefert den Schaltgrund fr die Regelklasse
   * @return Schaltgrund fr die Regelklasse
   */
  public Aspect getSchaltgrundAllgemein()
  {
    return m_schaltgrundAllgemein;
  }

  /**
   * Methode setzt des Schaltgrund fr dei Regelklasse
   * @param schaltgrund Schaltgrund fr dei Regelklasse
   */
  public void setSchaltgrundAllgemein( Aspect schaltgrund )
  {
    m_schaltgrundAllgemein = schaltgrund;
  }
  /**
   * Methode fgt einen Listener der Klasse hinzu
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void addPropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.addPropertyChangeListener( pcl );    
  }
  
  /**
   * Methode entfernt einen Listener aus der Klasse
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void removePropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.removePropertyChangeListener( pcl );    
  }

  /**
   * Methode benachrichtigt die Listener
   */
  private void benachrichtigeListener ()
  {
    m_pcs.firePropertyChange( "Aktualisierung", null, null );
    
    if (_debug)
      System.out.println(toString());
  }

  /**
   * Methode zum Auswerten der empfangenen Attributgruppen
   * @param results empfangenen Attributgruppen
   */
  public void auswerteAttributgruppen (ResultData[] results)
  {
    int anzahl = results.length;

    // Daten aus Eingangsqueue bearbeiten      
    
    for (int i = 0; i < anzahl; i++)
    {
      ResultData dat = results[i];
  
      if (dat.getDataDescription().getAttributeGroup().getPid().equals(_pidAtg))
      {
        auswerteAttributgruppe (dat);
        
        benachrichtigeListener();
      }
    }
  }
  
  /**
   * Methode zum Auswerten der Attributgruppe 'atg.sswSbaTrichterung'
   * @param dat empfangender Datensatz
   */
  private void auswerteAttributgruppe( ResultData dat )
  {
    synchronized (m_mutex)
    {
      if (!dat.hasData())
      {
        m_schaltgrundAllgemein = null;
        m_parameterAnlage.clear();
        m_parameterProAnzeigeQuerschnitt.clear();
        m_parameterProAnzeige.clear();
        
      }
      else
      {
        Data data = dat.getData();
          
        m_schaltgrundAllgemein = (Aspect) data.getReferenceValue( "SchaltgrundAllgemein" ).getSystemObject();
        
        Array a = data.getArray( "AnlageRegeln" );
        auswerteAttributgruppeAnlage( a );
        
        a = data.getArray( "AnzeigeQuerschnitt" );
        auswerteAttributgruppeQuerschnitt (a);
        
        a = data.getArray( "Anzeige");
        auswerteAttributgruppeAnzeige (a);
      }
    }
    
    m_schaltgrundAllgemein = Konfig.getInstance().getAspekt(_pidDefaultAspekt);

  }

  /**
   * Methode zum Auswerten der fr die gesamte Anlage geltenden Parameter
   * @param array Array mit Parametern
   */
  private void auswerteAttributgruppeAnlage (Array array)
  {
    m_parameterAnlage.auswerteAttributgruppe( array, getSchaltgrundAllgemein() );
  }

  /**
   * Methode zum Auswerten der Anzeigequerschnitt spezifischen Parameter
   * @param array Array mit Parametern fr die Anzeigequerschnitte
   */
  private void auswerteAttributgruppeQuerschnitt( Array array )
  {
    int anzahl = array.getLength(); 
    
    synchronized (m_parameterProAnzeigeQuerschnitt)
    {
      m_parameterProAnzeigeQuerschnitt.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        Data d1 = array.getItem( x );
        
        SystemObject anzeigeQuerschnitt = d1.getReferenceValue( "AnzeigeQuerschnitt" ).getSystemObject();
        Array a = d1.getArray( "AnzeigeQuerschnittRegeln" );
        
        RegelnProAnzeigeTyp regeln  = new RegelnProAnzeigeTyp ();
        regeln.auswerteAttributgruppe( a, getSchaltgrundAllgemein() );
        
        m_parameterProAnzeigeQuerschnitt.put (anzeigeQuerschnitt, regeln);
      }
    }
  }

  /**
   * Methode zum Auswerten der Anzeige spezifischen Parameter
   * @param array Array mit Parametern fr die Anzeige
   */
  private void auswerteAttributgruppeAnzeige (Array array )
  {
    int anzahl = array.getLength(); 
    
    synchronized (m_parameterProAnzeige)
    {
      m_parameterProAnzeige.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        Data d1 = array.getItem( x );
        
        SystemObject anzeige = d1.getReferenceValue( "Anzeige" ).getSystemObject();
        Array a = d1.getArray( "AnzeigeRegeln" );
        
        RegelnProAnzeigeTyp regeln  = new RegelnProAnzeigeTyp ();
        regeln.auswerteAttributgruppe( a, getSchaltgrundAllgemein() );
        
        m_parameterProAnzeige.put (anzeige, regeln);
      }
    }
  }

  /**
   * Methode fgt eine Regel fr die allgemeinen Parameter der Anlage hinzu
   * @param wvzTyp WVZ Typ fr den die Regel gelten soll
   * @param richtung Arbeitet Regel in Vorgnger- oder Nachfolgerrichtung
   * @param schaltgruende zu bercksichtigende Schaltgrnde
   * @param ebene Ebende in der Baumstruktur der Vorgnger/Nachfolgerbeziehung bis zu der verschoben werden soll
   * @param anzeigeTyp WZGTyp des Vorgngers/Nachfolgers in Lngsrichtung
   * @param abhaengigeAnzeigenVerschieben Sollen abhngige Anzeigen mit verschoben werden
   * @param anzeigenInhalt zu betrachtender Anzeigeinhalt
   * @param schaltgrund Schaltgrund unter dem diese Regel publiziert wird
   */
  public void addRegelAllgemein ( SystemObject wvzTyp,
                                  ArbeitsRichtungVN richtung,
                                  List<Aspect> schaltgruende,
                                  int ebene,
                                  SystemObject anzeigeTyp,
                                  boolean abhaengigeAnzeigenVerschieben,
                                  WVZInhalt anzeigenInhalt,
                                  Aspect schaltgrund) 
  {
    ListeMitRegeln regeln = m_parameterAnlage.getParameter( wvzTyp );
    regeln.addRegel( richtung, schaltgruende, ebene, anzeigeTyp, abhaengigeAnzeigenVerschieben, anzeigenInhalt, schaltgrund);
  }
  
  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    s += "Parameter Verschieben\n";
    s += "=====================\n";
    
    s += "Parameter Anlage:\n";
    s += "=================\n";
    s += m_parameterAnlage.toString();
    
    s += "Parameter pro Anzeigequerschnitt:\n";
    s += "=================================\n";
    Iterator<Entry<SystemObject, RegelnProAnzeigeTyp>> it = m_parameterProAnzeigeQuerschnitt.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry<SystemObject, RegelnProAnzeigeTyp> entry = it.next();
      
      SystemObject anzeigeQuerschnitt = (SystemObject) entry.getKey();
      RegelnProAnzeigeTyp regeln = (RegelnProAnzeigeTyp) entry.getValue();
      
      s += Tools.getObjektName( anzeigeQuerschnitt )  + "\n";
      s += regeln.toString();
    }

    s += "Parameter pro Anzeige:\n";
    s += "======================\n";
    it = m_parameterProAnzeige.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry<SystemObject, RegelnProAnzeigeTyp> entry = it.next();
      
      SystemObject anzeige = (SystemObject) entry.getKey();
      RegelnProAnzeigeTyp regeln = (RegelnProAnzeigeTyp) entry.getValue();
      
      s += Tools.getObjektName( anzeige )  + "\n";
      s += regeln.toString();
    }

    return s;
  }

  /**
   * Methode liefert die Variable m_parameterAnlage zurck
   * @return Variable m_parameterAnlage
   */
  public RegelnProAnzeigeTyp getParameterAnlage()
  {
    return m_parameterAnlage;
  }

  /**
   * Methode liefert die Variable m_parameterProAnzeigeQuerschnitt zurck
   * @return Variable m_parameterProAnzeigeQuerschnitt
   */
  public HashMap<SystemObject, RegelnProAnzeigeTyp> getParameterProAnzeigeQuerschnitt()
  {
    return m_parameterProAnzeigeQuerschnitt;
  }

  /**
   * Methode liefert die Variable m_parameterProAnzeige zurck
   * @return Variable m_parameterProAnzeige
   */
  public HashMap<SystemObject, RegelnProAnzeigeTyp> getParameterProAnzeige()
  {
    return m_parameterProAnzeige;
  }
  
  /**
   * Methode zum Erzeugen einer Attributgruppe mit den Parametern
   * @return Attributgruppe mit den Parametern
   */
  public Data erzeugeAttributgruppe ()
  {
	  ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection == null)
      return null;

    bereinigeParameter();

    String pidAtg = _pidAtg;
    
    // Attributgruppe zusammensetzen und versenden
      
    AttributeGroup atg = (AttributeGroup) connection.getDataModel().getAttributeGroup(pidAtg);
    Data data = connection.createData(atg );

    data.getReferenceValue( "SchaltgrundAllgemein" ).setSystemObject( getSchaltgrundAllgemein() );
    
    Array anlageRegeln = data.getArray( "AnlageRegeln" );
    erzeugeAnlageRegeln (anlageRegeln);

    Array anzeigeQuerschnitt = data.getArray( "AnzeigeQuerschnitt" );
    erzeugeAnzeigeQuerschnittRegeln (anzeigeQuerschnitt);
    
    Array anzeige = data.getArray( "Anzeige" );
    erzeugeAnzeigeRegeln (anzeige);
    
    return data;
  }

  /**
   * Methode lscht leere Parameterfelder. Dies ist notwendig, da sonst die Steuerung Problem
   * mit der Bestimmung der richtigen Parameterstze hat
   */
  private void bereinigeParameter()
  {
  	bereinigeRegelProAnzeigeTyp(getParameterProAnzeigeQuerschnitt());

  	bereinigeRegelProAnzeigeTyp(getParameterProAnzeige());
  }
  
  /**
   * Methode bereinigt die Parameter. Dabei werden Anzeigetypen fr die keine Regeln definiert sind
   * aus dem Parametersatz entfern.
   * @param regelnProAnzeigeTyp zu bereinigende Regeln
   */
  private void bereinigeRegelProAnzeigeTyp (HashMap<SystemObject, RegelnProAnzeigeTyp> regelnProAnzeigeTyp)
  {
  	List<SystemObject> zuLoeschen = new ArrayList<SystemObject>();
  	
	  Iterator<Entry<SystemObject, RegelnProAnzeigeTyp>> it = regelnProAnzeigeTyp.entrySet().iterator();
	  while (it.hasNext())
	  {
	  	Entry<SystemObject,RegelnProAnzeigeTyp> entry = it.next();
	  	
	  	SystemObject anzeigeQuerschnitt = entry.getKey();
	  	RegelnProAnzeigeTyp regeln = entry.getValue();
	  	
	  	regeln.bereinigeParameter();
	  	if (regeln.getParameter().size() == 0)
	  		zuLoeschen.add (anzeigeQuerschnitt);
	  }

	  for (SystemObject so : zuLoeschen)
	  	regelnProAnzeigeTyp.remove(so);
  }

  /**
   * Methode schreibt die Anlageregeln in ein Array einer Attributgruppe
   * @param array Array fr die Regeln
   */
  private void erzeugeAnlageRegeln( Array array )
  {
    RegelnProAnzeigeTyp rpat = getParameterAnlage();
    erzeugeRegeln( array, rpat );
  }

  /**
   * Methode schreibt die Anzeigequerschnittregeln in ein Array einer Attributgruppe
   * @param array Array fr die Regeln
   */
  private void erzeugeAnzeigeQuerschnittRegeln( Array array )
  {
    HashMap<SystemObject, RegelnProAnzeigeTyp> paraAq = getParameterProAnzeigeQuerschnitt();
    int anzahlAQ = paraAq.size();
    
    array.setLength( anzahlAQ );
    
    int i=0;
    Iterator<Entry<SystemObject, RegelnProAnzeigeTyp>> it = paraAq.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<SystemObject, RegelnProAnzeigeTyp> entry = it.next();
      
      SystemObject anzeigeQuerschnitt = entry.getKey();
      RegelnProAnzeigeTyp rpat = entry.getValue();
      
      Data dataAQ = array.getItem( i++ );
      
      dataAQ.getReferenceValue( "AnzeigeQuerschnitt" ).setSystemObject( anzeigeQuerschnitt );
      Array arrayRegeln = dataAQ.getArray( "AnzeigeQuerschnittRegeln" );
      
      erzeugeRegeln (arrayRegeln, rpat);
    }
  }

  /**
   * Methode schreibt die Anzeigeregeln in ein Array einer Attributgruppe
   * @param array Array fr die Regeln
   */
  private void erzeugeAnzeigeRegeln( Array array )
  {
    HashMap<SystemObject, RegelnProAnzeigeTyp> paraAnzeige = getParameterProAnzeige();
    int anzahlAnzeigen = paraAnzeige.size();
    
    array.setLength( anzahlAnzeigen );
    
    int i=0;
    Iterator<Entry<SystemObject, RegelnProAnzeigeTyp>> it = paraAnzeige.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<SystemObject, RegelnProAnzeigeTyp> entry = it.next();
      
      SystemObject anzeige = entry.getKey();
      RegelnProAnzeigeTyp rpat = entry.getValue();
      
      Data dataAQ = array.getItem( i++ );
      
      dataAQ.getReferenceValue( "Anzeige" ).setSystemObject( anzeige );
      Array arrayRegeln = dataAQ.getArray( "AnzeigeRegeln" );
      
      erzeugeRegeln (arrayRegeln, rpat);
    }
  }

  /**
   * Methode schreibt die Regeln pro Anzeigetyp in ein Array
   * @param array Array in das die Regeln geschrieben werden solle
   * @param rpat Regeln
   */
  private void erzeugeRegeln( Array array, RegelnProAnzeigeTyp rpat )
  {
    HashMap<SystemObject, ListeMitRegeln> p = rpat.getParameter();
    int anzahl = p.size();
    
    array.setLength( anzahl );
    
    int i=0;
    Iterator<Entry<SystemObject, ListeMitRegeln>> it = p.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<SystemObject, ListeMitRegeln> entry = it.next();
      
      SystemObject anzeigeTyp = entry.getKey();
      ListeMitRegeln regeln = entry.getValue();
      
      Data aktuell = array.getItem( i++ );
      
      aktuell.getReferenceValue( "AnzeigeTyp" ).setSystemObject( anzeigeTyp );
      Array r = aktuell.getArray( "Regeln" );
      
      Vector<RegelVerschieben> parameter = regeln.getParameter();
      r.setLength( parameter.size() );
      
      for (int j=0; j<parameter.size(); ++j)
      {
        Data d = r.getItem( j );
        RegelVerschieben rv = parameter.get( j );
        
        d.getReferenceValue( "SchaltgrundRegel" ).setSystemObject( rv.getSchaltgrundRegel() );
        d.getUnscaledValue ( "VorgngerNachfolger" ).set (rv.getRichtung().getKodierung());
        
        Array schaltgrund = d.getArray( "Schaltgrund" );
        List<Aspect> schaltgruende = rv.getSchaltgruende();
        schaltgrund.setLength( schaltgruende.size() );
        for (int k=0; k<schaltgruende.size(); ++k)
          schaltgrund.getReferenceValue( k ).setSystemObject( schaltgruende.get( k ) );
        
        d.getUnscaledValue ( "Ebene" ).set ( rv.getEbene() );
        d.getReferenceValue( "AnzeigeTyp" ).setSystemObject( rv.getAnzeigeTyp() );
        d.getTextValue( "AbhngigeAnzeigenVerschieben" ).setText( rv.isAbhaengigeAnzeigenVerschieben() ? "Ja" : "Nein" );
        
        Data inhalt = d.getItem( "Anzeigeinhalt" );
        rv.getAnzeigenInhalt().erzeugeAttributgruppe( inhalt );
      }
    }
  }
  
  /**
   * Methode bernimmt die Parameter fr eine bestimmte Pid
   * @param pid Pid
   * @param regeln Regeln
   */
	public void setParameter(String pid, RegelnProAnzeigeTyp regeln)
  {
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    /*
     * Parameter Anlage
     */
    if (pid.equals(_pidSBA))
    {
      m_parameterAnlage = regeln;
    }
    else if (connection != null)
    {
    	/*
    	 * Prfen ob Anzeigequerschnitt oder Anzeige
    	 */
    	
    	SystemObject so = connection.getDataModel().getObject(pid);
    	if (so != null)
    	{
    		if (so.getType().getPid().equals("typ.anzeigeQuerschnitt"))
    			m_parameterProAnzeigeQuerschnitt.put(so, regeln);

    		if (so.getType().getPid().equals("typ.anzeige"))
    			m_parameterProAnzeige.put(so, regeln);
    	}
    }
    
    benachrichtigeListener();
  }

  /**
   * Methode liefert die Parameter als Attributgruppe fr den DaV
   * @return Attributgruppe
   */
  public Data getData ()
  {
  	return erzeugeAttributgruppe();
  }

  /**
   * Methode liefert die Parameter zu einer bestimmten Pid
   * @param pid Pid des gesuchten Objekts, dies kann die Pid der SBA, die Pid eines Anzeigequerschnitts oder
   * die Pid einer Anzeige sein.
   * @return Parameter zu der Pid
   */
	public RegelnProAnzeigeTyp getParameter(String pid)
  {
    /*
     * Parameter Anlage
     */
    if (pid.equals(_pidSBA))
      return getParameterAnlage();
    
    /*
     * Parameter Anzeigequerschnitt
     */
    for (SystemObject so : m_parameterProAnzeigeQuerschnitt.keySet())
    {
      if (pid.equals(so.getPid()))
        return m_parameterProAnzeigeQuerschnitt.get(so);
    }
    
    /*
     * Parameter Anzeige
     */
    for (SystemObject so : m_parameterProAnzeige.keySet())
    {
      if (pid.equals(so.getPid()))
        return m_parameterProAnzeige.get(so);
    }
    
    /*
     * Parametersatz noch nicht definiert
     */
   
    return new RegelnProAnzeigeTyp();
  }


}
