/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.sonstiges.statusleiste;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.eclipse.jface.action.ContributionItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Text;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.dav.daf.main.ClientDavInterface;

/**
 * Diese Klasse liefert ein {@link ContributionItem}, welches eine Uhr
 * beinhaltet und in der Statusleiste positioniert werden kann.
 * 
 * @author Pittner
 * 
 */
public class SymbolleisteDavItem extends ContributionItem implements IDaVVerbindungZuhoerer
{
  /**
   * Textfeld zur Darstellung der Zeit.
   */
  private Text _textDaV = null;
  
  /**
   * Das Parent-Composite
   */
  private Composite _parent;
  
  /**
   * Farbe Grn fr DaV verbunden
   */
  private final static Color farbeDaVOk = new Color(null, 96, 255, 96);
  
  /**
   * Farbe Rot fr DaV getrennt
   */
  private final static Color farbeDaVGestoert = new Color(null, 255, 0, 0); 
  
  /**
   * Farbe Rot fr Archiv nicht verfgbar
   */
  private final static SimpleDateFormat _sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss"); 

  /**
   * Konstruktor.
   * 
   * @param id
   *          ID des {@link ContributionItem}.
   */
  public SymbolleisteDavItem(String id)
  {
    super(id);
    DaVVerbindung.getInstanz().hinzufuegenZuhoerer(this);
  }

  public void fill(Composite parent)
  {    
    _parent = parent;    
    _textDaV = new Text(parent, SWT.BORDER);
    _textDaV.setEditable(false);
     aktualisiereStatusItem();    
  }

 

  public void dispose()
  {    
    super.dispose();
    DaVVerbindung.getInstanz().entferneZuhoerer(this);
  }

  @Override
  public void verbindungAufgebaut(DaVVerbindungEreignis e)
  {
    aktualisiereStatusItem();    
  }

  @Override
  public void verbindungAbgebaut(DaVVerbindungEreignis e)
  {    
    aktualisiereStatusItem();
  }
  
  /**
   *  Fhrt die nderung der Darstellung durch.
   * 
   * @param zustand
   *             der Zustand des DaV
   */
  private void aktualisiereStatusItem(){
    
    if ((_textDaV != null) && (!_textDaV.isDisposed()))
    {        
      ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
      if (connection != null)
      {

        String user = DaVVerbindung.getInstanz().getDaVBenutzer();
        int port = DaVVerbindung.getInstanz().getDaVPort();
        String ip = DaVVerbindung.getInstanz().getDaVIp();

        String s = user + "@" + ip + ":" + port;
        aktionTextChange(s, farbeDaVOk);
      }
      else
      {
        String s = "keine DaV-Verbindung!";
        aktionTextChange(s, farbeDaVGestoert);
      }

    }    
  }

  private void aktionTextChange(final String s, final Color color)
  {
    Display display = _textDaV.getDisplay();
    if (display != null && !display.isDisposed()){
      display.asyncExec(new Runnable() {
        
        @Override
        public void run()
        {
          _textDaV.setText(s);
          _textDaV.setBackground(color);
          
          _textDaV.setToolTipText(s + " : " + _sdf.format(new Date()));
          
          _textDaV.pack();
          _parent.layout();
        }
      });          
    }        
  }
}
