/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.actions;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.sts.buv.plugin.workflow.tsf.dlg.DialogTSFBereichSchalten;
import de.sts.buv.plugin.workflow.tsf.dlg.TSFBereichSchaltenVerwaltung;

/**
 * Hilfsklasse zum Schalten von Bereichen.
 * 
 * @author HofmannJ
 * 
 */
public class FrSchalten
{

	/**
	 * ffnet den Dialog zum Schalten eines Bereichs der temporren Seitenstreifen
	 * Freigabe. Ist die entsprechende TSF schon in Bearbeitung, erscheint ein
	 * Hinweisfenster fr den Benutzer.
	 * 
	 * @param pidTsfBereich
	 *          PID eines Objektes vom Typ
	 *          <code>typ.temporaereSeitenstreifenFreigabe</code>.
	 */
	public static void doschalten(String pidTsfBereich)
	{
		TSFBereichSchaltenVerwaltung schaltenVerwaltung = TSFBereichSchaltenVerwaltung.getInstance();
		if (!schaltenVerwaltung.isBereichInAktiverSchaltung(pidTsfBereich))
		{
			DialogTSFBereichSchalten dlg = new DialogTSFBereichSchalten(PlatformUI
					.getWorkbench().getDisplay().getActiveShell());
			dlg.oeffne(pidTsfBereich);
		} 
		else
		{
			String bereich = pidTsfBereich;
			
			ClientDavInterface con = DaVVerbindung.getInstanz().getDaVVerbindung();
			if(con!=null)
			{
				DataModel dataModel = con.getDataModel();
				SystemObject soBereich = dataModel.getObject(pidTsfBereich);
				if(soBereich!=null)
				{
					bereich = soBereich.getNameOrPidOrId();
				}
			}
			
			StringBuffer sb = new StringBuffer();
			
			sb.append("Fr den Bereich\n\n");
			sb.append("-" + bereich);
			sb.append("\n\nder temporren Seitenstreifenfreigabe");
			sb.append("\n");
			sb.append("wird gerade eine Schaltung durchgefhrt.\n");
			sb.append("Solange diese aktiv ist, kann keine weitere Schaltung durchgefhrt werden.\n");
//   		sb.append("Sie knnen jedoch ber den Progress-Monitor eine laufende Schaltungssequenz abbrechen.");

			MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.OK | SWT.ICON_INFORMATION);
			// Dialogtitel
			mb.setText("Schaltung Bereich " + bereich
					+ " z.Zt. nicht mglich.");
			
			// Message
			mb.setMessage(sb.toString());

			mb.open();
		}
	}
}
