/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.util.Date;
import java.util.HashMap;

import de.bsvrz.dav.daf.main.config.SystemObject;

public class AggregationAbschnittZustand
{
	public static AggregationAbschnittZustand _instance = null;

	private HashMap<SystemObject, Datensatz> _map = new HashMap<SystemObject, Datensatz>();

	private AggregationAbschnittZustand()
	{
	}

	public static AggregationAbschnittZustand getInstance()
	{
		if (_instance == null)
		{
			synchronized (AggregationAbschnittZustand.class)
			{
				if (_instance == null)
				{
					_instance = new AggregationAbschnittZustand();
				}
			}
		}

		return _instance;
	}

	private Datensatz getDatensatz(SystemObject soAbschnitt)
	{
		Datensatz re = null;

		if (soAbschnitt != null)
		{
			synchronized (_map)
			{
				if (!_map.containsKey(soAbschnitt))
				{
					_map.put(soAbschnitt, new Datensatz(soAbschnitt));
				}

				re = _map.get(soAbschnitt);
			}
		}

		return re;
	}

	public void setZustandSteuerung(String pidAspekt,
			M_TSFZustandSteuerung zustandSteuerung)
	{
		if (pidAspekt != null)
		{
			SystemObject soAbschnitt = zustandSteuerung.getSo();
			Datensatz datensatz = getDatensatz(soAbschnitt);
			datensatz.setZustandSteuerung(pidAspekt, zustandSteuerung);
			
			System.out.println("->" + datensatz);
		}
	}

	public void setAbschnittBefehl(String pidAspekt,
			M_TSFAbschnittBefehl abschnittBefehl)
	{
		if (pidAspekt != null)
		{
			SystemObject soAbschnitt = abschnittBefehl.getSo();
			Datensatz datensatz = getDatensatz(soAbschnitt);
			datensatz.setAbschnittBefehl(pidAspekt, abschnittBefehl);
			System.out.println("->>" + datensatz);
		}
	}

	public class Datensatz
	{
		private SystemObject _soAbschnitt = null;
		private Date _zeitstempel = null;
		private E_TSFBefehl _aktPrg = null;
		private E_TSFBefehl _zielPrg = null;
		private E_TSFProgrammZustand _prgZustandZustand = null;
		private E_TSFProgrammZustand _prgVorgabeZustand = null;
		private Long _stufe = null;
		private E_TSFSteuerungZustand _steuerungZustand;

		public Datensatz(SystemObject soAbschnitt)
		{
			setSoAbschnitt(soAbschnitt);
		}

		public void setZustandSteuerung(String pidAspekt,
				M_TSFZustandSteuerung zustandSteuerung)
		{
			if (zustandSteuerung.getTimestamp() != null)
			{
				setZeitstempel(zustandSteuerung.getTimestamp());
			}

			if (pidAspekt.equals(Defs.ASP_TSF_ZUSTAND))
			{
				// E_TSFBefehl aktPrg = zustandSteuerung.getProgramm();
				// setAktPrg(aktPrg);

				long stufe = zustandSteuerung.getStufe();
				setStufe(stufe);
				
				setSteuerungZustand(zustandSteuerung.getZustand());				
			}

			if (pidAspekt.equals(Defs.ASP_TSF_VORGABE_ZUSTAND))
			{
				E_TSFBefehl zielPrg = zustandSteuerung.getProgramm();
//				setZielPrg(zielPrg);

				long stufe = zustandSteuerung.getStufe();
				setStufe(stufe);
			}
		}

		public void setAbschnittBefehl(String pidAspekt,
				M_TSFAbschnittBefehl abschnittBefehl)
		{
			E_TSFProgrammZustand zustand = abschnittBefehl.getZustand();
			E_TSFBefehl aktion = abschnittBefehl.getAktion();

			if (pidAspekt.equals(Defs.ASP_TSF_ZUSTAND))
			{
				setPrgZustandZustand(zustand);
				setAktPrg(aktion);

			}

			if (pidAspekt.equals(Defs.ASP_TSF_VORGABE_ZUSTAND))
			{
				setPrgVorgabeZustand(zustand);
				setZielPrg(aktion);
			}

		}

		public SystemObject getSoAbschnitt()
		{
			return _soAbschnitt;
		}

		public void setSoAbschnitt(SystemObject soAbschnitt)
		{
			_soAbschnitt = soAbschnitt;
		}

		public Date getZeitstempel()
		{
			return _zeitstempel;
		}

		public void setZeitstempel(Date zeitstempel)
		{
			_zeitstempel = zeitstempel;
		}

		public E_TSFBefehl getAktPrg()
		{
			return _aktPrg;
		}

		public void setAktPrg(E_TSFBefehl aktPrg)
		{
			_aktPrg = aktPrg;
		}

		public E_TSFBefehl getZielPrg()
		{
			return _zielPrg;
		}

		public void setZielPrg(E_TSFBefehl zielPrg)
		{
			_zielPrg = zielPrg;
		}

		public E_TSFProgrammZustand getPrgZustandZustand()
		{
			return _prgZustandZustand;
		}

		public void setPrgZustandZustand(E_TSFProgrammZustand prgZustandZustand)
		{
			_prgZustandZustand = prgZustandZustand;
		}

		public E_TSFProgrammZustand getPrgVorgabeZustand()
		{
			return _prgVorgabeZustand;
		}

		public void setPrgVorgabeZustand(E_TSFProgrammZustand prgVorgabeZustand)
		{
			_prgVorgabeZustand = prgVorgabeZustand;
		}

		public Long getStufe()
		{
			return _stufe;
		}

		public void setStufe(Long stufe)
		{
			_stufe = stufe;
		}
		
		

		public E_TSFSteuerungZustand getSteuerungZustand()
		{
			return _steuerungZustand;
		}

		public void setSteuerungZustand(E_TSFSteuerungZustand steuerungZustand)
		{
			_steuerungZustand = steuerungZustand;
		}

		public boolean isZielErreicht()
		{
			boolean re = false;

			if (getPrgVorgabeZustand() != null)
			{
				if ((getPrgVorgabeZustand() != E_TSFProgrammZustand.Geschaltet)
						&& (getAktPrg() == getZielPrg()))
				{
					re = true;
				}
			} else
			{
				if ((getAktPrg() != null) && (getPrgZustandZustand() != null)
						&& (getPrgZustandZustand() == E_TSFProgrammZustand.Geschaltet))
				{
					re = true;
				}
			}

			return re;
		}

		@Override
		public String toString()
		{
			StringBuffer sb = new StringBuffer();
			sb.append("========================================\n");

			sb.append("Abschnitt: ");
			sb.append(getSoAbschnitt());
			sb.append("\n");

			sb.append("Zeitstempel: ");
			if (getZeitstempel() != null)
			{
				sb.append(Defs.SDF.format(getZeitstempel()));
			} else
			{
				sb.append("-");
			}
			sb.append("\n");

			sb.append("Aktuelles Progamm: ");
			sb.append(getAktPrg());
			sb.append("\n");

			sb.append("Ziel Progamm: ");
			sb.append(getZielPrg());
			sb.append("\n");

			sb.append("Programm Zustand Zustand: ");
			sb.append(getPrgZustandZustand());
			sb.append("\n");

			sb.append("Programm Vorgabe Zustand: ");
			sb.append(getPrgVorgabeZustand());
			sb.append("\n");

			sb.append("Stufe: ");
			sb.append(getStufe());
			sb.append("\n");
			
			sb.append("Zustand Strg.:");
			sb.append(getSteuerungZustand());
			sb.append("\n");
			
			sb.append("Zielprg. erreicht = ");
			sb.append(isZielErreicht());
			sb.append("\n");

			sb.append("========================================\n");

			return sb.toString();
		}

	}

}
