/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.util.LinkedList;
import java.util.List;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Ermglicht den Zugriff auf die Diagnose-Daten der TSF-Abschnitte. <br>
 * <br>
 * <b>Lesen</b><br>
 * <ul>
 * <li>Typ: <code>typ.tsfAbschnitt</code></li>
 * <li>Atg: <code>atg.tsfAbschnittDiagnoseDaten</code></li>
 * <li>Asp: <code>asp.tsfZustand</code></li>
 * </ul>
 * 
 * @author HofmannJ
 * 
 */
public class DaVKomTSFAbschnittDiagnoseDaten implements ClientReceiverInterface
{
	private static DaVKomTSFAbschnittDiagnoseDaten _instance = null;

	/**
	 * Verbindung DaV
	 */
	private ClientDavInterface _connection = null;

	/**
	 * {@link DataDescription} fr die Lese-Anmeldung
	 */
	private DataDescription _ddLeseAnmeldung = null;

	/**
	 * {@link SystemObject}s, auf welche die Leseanmeldung erfolgte.
	 */
	private LinkedList<SystemObject> _lSoLeseAnmeldung = new LinkedList<SystemObject>();

	/**
	 * Modell
	 */
	private GUIModelTSFAbschnittDiagnoseDaten _model = new GUIModelTSFAbschnittDiagnoseDaten();

	private DaVKomTSFAbschnittDiagnoseDaten()
	{
		initKommunikation();
	}

	public static DaVKomTSFAbschnittDiagnoseDaten getInstance()
	{
		if (_instance == null)
		{
			synchronized (DaVKomTSFAbschnittDiagnoseDaten.class)
			{
				if (_instance == null)
				{
					_instance = new DaVKomTSFAbschnittDiagnoseDaten();
				}
			}
		}

		return _instance;
	}

	private void initKommunikation()
	{
		if (_connection == null)
		{
			this._connection = DaVVerbindung.getInstanz().getDaVVerbindung();
		}

		if (_connection != null)
		{
			anmeldenEmpfang();
		} else
		{
			System.err
					.println("Es ist keine Kommunikation mglich, dar keine Verbindung zum DaV existent ist.");
		}
	}

	private void anmeldenEmpfang()
	{
		if (_ddLeseAnmeldung == null)
		{
			DataModel dataModel = _connection.getDataModel();
			SystemObjectType sot = dataModel.getType(Defs.TYP_TSF_ABSCHNITT);
			if (sot != null)
			{
				List<SystemObject> lSOTs = sot.getObjects();

				if (lSOTs.size() > 0)
				{
					AttributeGroup atg = dataModel
							.getAttributeGroup(Defs.ATG_TSF_ABSCHNITT_DIAGNOSE_DATEN);
					Aspect asp = dataModel.getAspect(Defs.ASP_TSF_ZUSTAND);

					if ((atg != null) && (asp != null))
					{
						DataDescription dd = new DataDescription(atg, asp);
						_ddLeseAnmeldung = dd;
						_lSoLeseAnmeldung.addAll(lSOTs);

						_connection.subscribeReceiver(this, lSOTs, dd, ReceiveOptions
								.normal(), ReceiverRole.receiver());

					} else
					{
						System.err.println("ATG und ASP konnten nicht bestimmt werden.");
					}
				} else
				{
					System.err.println("Es muss mindestens ein Objekt vom Typ "
							+ Defs.TYP_TSF_ABSCHNITT
							+ " existieren. Dies ist z.Zt. nicht der Fall!");
				}

			} else
			{
				System.err.println("Es konnten z.Zt. keine Objekte vom Typ "
						+ Defs.TYP_TSF_ABSCHNITT
						+ " bestimmt werden. Es ist keine Empfangsanmeldung mglich.");
			}
		} else
		{
			System.err.println("Es wurde bereits eine Leseanmeldung durchgefhrt.");
		}
	}

	private void abmeldenEmpfang()
	{
		if ((_connection != null) && (_ddLeseAnmeldung != null))
		{
			if (_lSoLeseAnmeldung.size() > 0)
			{
				_connection.unsubscribeReceiver(this, _lSoLeseAnmeldung,
						_ddLeseAnmeldung);

				this._lSoLeseAnmeldung.clear();
				this._ddLeseAnmeldung = null;
			}
		}
	}

	@Override
	public void update(ResultData[] results)
	{
		if ((results != null) && (results.length > 0))
		{
			for (ResultData resultData : results)
			{
				if (resultData != null)
				{
					if (resultData.hasData())
					{
						M_TSFAbschnittDiagnoseDaten tsfAbschnittDiagnoseDaten = new M_TSFAbschnittDiagnoseDaten(
								resultData);
						getModel().put(tsfAbschnittDiagnoseDaten);
					} else
					{
						if (resultData.getObject() != null)
						{
							getModel().remove(resultData.getObject());
						}
					}
				}
			}
		}
	}

	public GUIModelTSFAbschnittDiagnoseDaten getModel()
	{
		return _model;
	}

	public void dispose()
	{
		abmeldenEmpfang();
	}
}
