/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.ui.PlatformUI;

//import de.bsvrz.buv.rw.basislib.login.UrlasserDialog;
//import de.bsvrz.buv.rw.basislib.login.UrlasserDialog.UrlasserDialogInfo;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.koelnloevenich.dif.basislib.urlasserdialog.UrlasserDialog;
import de.koelnloevenich.dif.basislib.urlasserdialog.UrlasserDialog.UrlasserDialogInfo;

/**
 * Antwort auf die Aufforderung zum Weiterschalten. <br>
 * <b>Lesen:</b><br>
 * <ul>
 * <li>TYP <code>typ.tsfAbschnitt</code></li>,
 * <li>ATG <code>atg.tsfWeiterSchaltenAntwort</code></li>,
 * <li>ASP <code>asp.tsfVorgabe</code>.</li>
 * </ul>
 * <br>
 * <b>Schreiben:</b><br>
 * <ul>
 * <li>TYP <code>typ.tsfAbschnitt</code></li>,
 * <li>ATG <code>atg.tsfWeiterSchaltenAntwort</code></li>,
 * <li>ASP <code>asp.vorgabe</code>.</li>
 * </ul>
 * 
 * @author HofmannJ
 * 
 */
public class DaVKomTSFWeiterSchaltenAntwort implements ClientReceiverInterface,
		ClientSenderInterface
{
	private static DaVKomTSFWeiterSchaltenAntwort _instance = null;

	/**
	 * Verbindung DaV
	 */
	private ClientDavInterface _connection = null;

	/**
	 * {@link DataDescription} fr die Lese-Anmeldung
	 */
	private DataDescription _ddLeseAnmeldung = null;

	/**
	 * {@link SystemObject}s, auf welche die Leseanmeldung erfolgte.
	 */
	private LinkedList<SystemObject> _lSoLeseAnmeldung = new LinkedList<SystemObject>();

	private DaVKomTSFWeiterSchaltenAntwort()
	{
		initKommunikation();
	}

	private DaVKomTSFWeiterSchaltenAntwort(ClientDavInterface con)
	{
		this._connection = con;
		initKommunikation();
	}

	public static DaVKomTSFWeiterSchaltenAntwort getInstance()
	{
		if (_instance == null)
		{
			synchronized (DaVKomTSFWeiterSchaltenAntwort.class)
			{
				if (_instance == null)
				{
					_instance = new DaVKomTSFWeiterSchaltenAntwort();
				}
			}
		}

		return _instance;
	}

	public static DaVKomTSFWeiterSchaltenAntwort getInstance(
			ClientDavInterface con)
	{
		if (_instance == null)
		{
			synchronized (DaVKomTSFWeiterSchaltenAntwort.class)
			{
				if (_instance == null)
				{
					_instance = new DaVKomTSFWeiterSchaltenAntwort(con);
				}
			}
		}

		return _instance;
	}

	private void initKommunikation()
	{
		if (_connection == null)
		{
			this._connection = DaVVerbindung.getInstanz().getDaVVerbindung();
		}

		if (_connection != null)
		{
			anmeldenEmpfang();
		} else
		{
			System.err
					.println("Es ist keine Kommunikation mglich, dar keine Verbindung zum DaV existent ist.");
		}
	}

	private void anmeldenEmpfang()
	{
		if (_ddLeseAnmeldung == null)
		{
			DataModel dataModel = _connection.getDataModel();
			SystemObjectType sot = dataModel.getType(Defs.TYP_TSF_ABSCHNITT);
			if (sot != null)
			{
				List<SystemObject> lSOTs = sot.getObjects();

				if (lSOTs.size() > 0)
				{
					AttributeGroup atg = dataModel
							.getAttributeGroup(Defs.ATG_TSF_ABSCHNITT_BEFEHL);
					Aspect asp = dataModel.getAspect(Defs.ASP_TSF_VORGABE_ZUSTAND);

					if ((atg != null) && (asp != null))
					{
						DataDescription dd = new DataDescription(atg, asp);
						_ddLeseAnmeldung = dd;
						_lSoLeseAnmeldung.addAll(lSOTs);

						_connection.subscribeReceiver(this, lSOTs, dd, ReceiveOptions
								.normal(), ReceiverRole.receiver());

					} else
					{
						System.err.println("ATG und ASP konnten nicht bestimmt werden.");
					}
				} else
				{
					System.err.println("Es muss mindestens ein Objekt vom Typ "
							+ Defs.TYP_TSF_ABSCHNITT
							+ " existieren. Dies ist z.Zt. nicht der Fall!");
				}

			} else
			{
				System.err.println("Es konnten z.Zt. keine Objekte vom Typ "
						+ Defs.TYP_TSF_ABSCHNITT
						+ " bestimmt werden. Es ist keine Empfangsanmeldung mglich.");
			}
		} else
		{
			System.err.println("Es wurde bereits eine Leseanmeldung durchgefhrt.");
		}
	}

	private void abmeldenEmpfang()
	{
		if ((_connection != null) && (_ddLeseAnmeldung != null))
		{
			if (_lSoLeseAnmeldung.size() > 0)
			{
				_connection.unsubscribeReceiver(this, _lSoLeseAnmeldung,
						_ddLeseAnmeldung);

				this._lSoLeseAnmeldung.clear();
				this._ddLeseAnmeldung = null;
			}
		}
	}

	@Override
	public void update(ResultData[] results)
	{
		// Lesender Zugriff wird hier im Moment nicht bentigt.
		// if ((results != null) && (results.length > 0))
		// {
		// for (ResultData resultData : results)
		// {
		//
		// if (resultData.hasData())
		// {
		// SystemObject so = resultData.getObject();
		// Data data = resultData.getData();
		// if (data != null)
		// {
		// //...auswerten...
		// }
		// }
		// }
		// }
	}

	public void dispose()
	{
		abmeldenEmpfang();
	}

	/**
	 * Versendet einen Befehl (ATG <code>atg.tsfWeiterSchaltenAntwort</code>)
	 * unter dem Aspekt <code>asp.tsfVorgabe</code>.
	 * 
	 * @param befehl
	 *          Der zu versendende Befehl.
	 * @return Liefert einen {@link String} zurck mit einer Fehlerinfo, falls der
	 *         Befehl nicht versendet werden konnte. Konnte der Befehl versendet
	 *         werden, wird <code>null</code> zurckgeliefert.
	 */
	public String sendeWeiterschaltenAntwort(M_TSFWeiterSchaltenAntwort befehl)
	{
		String fehlerInfo = null;

		if (befehl != null)
		{
			if (befehl.getSo() != null)
			{
				UrlasserDialog uld = new UrlasserDialog(PlatformUI.getWorkbench()
						.getActiveWorkbenchWindow().getShell());

				DataModel dataModel = _connection.getDataModel();

				AttributeGroup atg = dataModel
						.getAttributeGroup(Defs.ATG_TSF_WEITER_SCHALTEN_ANTWORT);
				Aspect asp = dataModel.getAspect(Defs.ASP_VORGABE);

				Data data = _connection.createData(atg);

				if (befehl.getBefehl() != null)
				{
					data.getUnscaledValue("Befehl").set(befehl.getBefehl().getWert());

					if (befehl.getKennung() != null)
					{
						data.getTextValue("Kennung").setText(befehl.getKennung());
					} else
					{
						data.getTextValue("Kennung").setText("Keine Kennung eingestellt.");
					}

					// ID
					data.getTextValue("ID").setText(befehl.getId());

					DataDescription desc = new DataDescription(atg, asp);
					ResultData rd = new ResultData(befehl.getSo(), desc, System
							.currentTimeMillis(), data);
					LinkedList<ResultData> lRd = new LinkedList<ResultData>();
					lRd.add(rd);
					UrlasserDialogInfo uldInfo = null;
					if (Defs.BERECHTIGUNGEN_IGNORIEREN)
					{
						uldInfo = uld.anzeigen(lRd);
					} else
					{
						uldInfo = uld.anzeigen(lRd);
					}

					fehlerInfo = uldInfo.getFehlerInfo();

					if (!uldInfo.getEingabeOk())
					{
						if (fehlerInfo == null)
						{
							fehlerInfo = "(BUTTON ABBRECHEN WURDE GEDRCKT.)";
						} else
						{
							fehlerInfo = fehlerInfo + " (BUTTON ABBRECHEN WURDE GEDRCKT.)";
						}
					}

				} else
				{
					System.err
							.println("Im Befehl ist keine Aktion gesetzt. Kein Versenden der Daten mglich.");
				}

				// uld.anzeigen(resultDatas)
			} else
			{
				System.err.println("Der Befehl verfgt ber kein Systemobjekt vom Typ "
						+ Defs.TYP_TSF_ABSCHNITT + "!");
			}
		}

		return fehlerInfo;
	}

	@Override
	public void dataRequest(SystemObject object, DataDescription dataDescription,
			byte state)
	{

	}

	@Override
	public boolean isRequestSupported(SystemObject object,
			DataDescription dataDescription)
	{
		return false;
	}

}
