/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchListener;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.sts.buv.plugin.workflow.tsf.dlg.ViewWFEinfach;

/**
 * Empfngt Informationen des Workflows. <br>
 * Kommunikation mit dem DaV bzgl. der TSF Workflow ATG <code>atg.tsfWorkflow</code> fr alle Objekte des Typs
 * <code>typ.temporaereSeitenstreifenFreigabe</code>. <br>
 * <br>
 * <b>Lesen:</b><br>
 * <ul>
 * <li>TYP: <code>typ.temporaereSeitenstreifenFreigabe</code>,</li>
 * <li>ATG: <code>atg.tsfWorkflow</code>,</li>
 * <li>ASP: <code>atg.tsfWorkflow</code>.</li>
 * </ul>
 * 
 * @author HofmannJ
 * 
 */
public class DaVKomTSFWorkflow implements ClientReceiverInterface, IWorkbenchListener
{
  private static final boolean DEBUG = false;

  private static DaVKomTSFWorkflow _instance = null;

  /**
   * Verbindung DaV
   */
  private ClientDavInterface _connection = null;

  /**
   * {@link DataDescription} fr die Lese-Anmeldung
   */
  private DataDescription _ddLeseAnmeldung = null;

  /**
   * {@link SystemObject}s, auf welche die Leseanmeldung erfolgte.
   */
  private LinkedList<SystemObject> _lSoLeseAnmeldung = new LinkedList<SystemObject>();

  /**
   * Modell fr das GUI.
   */
  private GUIModelTSFWorkflow _model = null;

  private GUIManager_WorkflowDialog _guiwfDialogManager;

  private HashMap<SystemObject, Vector<SystemObject>> _zuordnungen = new HashMap<SystemObject, Vector<SystemObject>>();

  /**
   * In dieser Map wird gesichert, welche Abschnitte zu einem Bereich gehren.
   */
  private HashMap<SystemObject, LinkedList<SystemObject>> _mapBereichZuAbschnitten = new HashMap<SystemObject, LinkedList<SystemObject>>();

  private DaVKomTSFWorkflow()
  {
    initKommunikation();
    PlatformUI.getWorkbench().addWorkbenchListener(this);
  }

  private DaVKomTSFWorkflow(ClientDavInterface con)
  {
    this._connection = con;
    initKommunikation();
    PlatformUI.getWorkbench().addWorkbenchListener(this);
  }

  public static DaVKomTSFWorkflow getInstance()
  {
    if (_instance == null)
    {
      synchronized (DaVKomTSFWorkflow.class)
      {
        if (_instance == null)
        {
          _instance = new DaVKomTSFWorkflow();
        }
      }
    }

    return _instance;
  }

  public static DaVKomTSFWorkflow getInstance(ClientDavInterface con)
  {
    if (_instance == null)
    {
      synchronized (DaVKomTSFWorkflow.class)
      {
        if (_instance == null)
        {
          _instance = new DaVKomTSFWorkflow(con);
        }
      }
    }

    return _instance;
  }

  private void initKommunikation()
  {
    this._model = new GUIModelTSFWorkflow();
    this._guiwfDialogManager = new GUIManager_WorkflowDialog(_model);

    if (_connection == null)
    {
      this._connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    }

    if (_connection != null)
    {
      initBereichAbschnittZuordnung();
      anmeldenEmpfang();
    }
    else
    {
      System.err.println("Es ist keine Kommunikation mglich, dar keine Verbindung zum DaV existent ist.");
    }
  }

  /**
   * Bestimmt, welche Abschnitte zu einem Bereich gehren.
   */
  private void initBereichAbschnittZuordnung()
  {
    if (_connection != null)
    {
      DataModel dataModel = _connection.getDataModel();
      // SystemObjectType sot = dataModel
      // .getType("typ.temporaereSeitenstreifenFreigabe");
      SystemObjectType sot = dataModel.getType("typ.sswSbaUeSeitenstreifenFreigabe");
      if (sot != null)
      {
        List<SystemObject> lSO = sot.getObjects();

        AttributeGroup atg = dataModel.getAttributeGroup("atg.sswSbaTsfAbschnitte");
        Aspect asp = dataModel.getAspect("asp.eigenschaften");

        for (SystemObject soBereich : lSO)
        {
          Data data = soBereich.getConfigurationData(atg, asp);

          ReferenceArray referenceArray = data.getReferenceArray("Abschnitte");
          SystemObject[] aSoAbschnitte = referenceArray.getSystemObjectArray();

          synchronized (_mapBereichZuAbschnitten)
          {
            if (!_mapBereichZuAbschnitten.containsKey(soBereich))
            {
              _mapBereichZuAbschnitten.put(soBereich, new LinkedList<SystemObject>());
            }
            LinkedList<SystemObject> lSoAbs = _mapBereichZuAbschnitten.get(soBereich);
            for (SystemObject soAbs : aSoAbschnitte)
            {
              lSoAbs.add(soAbs);
              // System.out.println("=> Bereich=" + soBereich.getPid() + " ABS="
              // + soAbs);
            }

          }
        }

      }
    }
  }

  private void anmeldenEmpfang()
  {
    if (_ddLeseAnmeldung == null)
    {
      DataModel dataModel = _connection.getDataModel();
      SystemObjectType sot = dataModel.getType(Defs.TYP_TEMP_SEITENSTREIFEN_FREIGABE);
      if (sot != null)
      {
        List<SystemObject> lSOTs = sot.getObjects();

        if (lSOTs.size() > 0)
        {
          AttributeGroup atg = dataModel.getAttributeGroup(Defs.ATG_TSF_WORKFLOW);
          Aspect asp = dataModel.getAspect(Defs.ASP_TSF_ZUSTAND);

          if ((atg != null) && (asp != null))
          {
            DataDescription dd = new DataDescription(atg, asp);
            _ddLeseAnmeldung = dd;
            _lSoLeseAnmeldung.addAll(lSOTs);

            _connection.subscribeReceiver(this, lSOTs, dd, ReceiveOptions.normal(), ReceiverRole.receiver());

          }
          else
          {
            System.err.println("ATG und ASP konnten nicht bestimmt werden.");
          }
        }
        else
        {
          System.err.println("Es muss mindestens ein Objekt vom Typ " + Defs.TYP_TEMP_SEITENSTREIFEN_FREIGABE
              + " existieren. Dies ist z.Zt. nicht der Fall!");
        }

      }
      else
      {
        System.err.println("Es konnten z.Zt. keine Objekte vom Typ " + Defs.TYP_TEMP_SEITENSTREIFEN_FREIGABE
            + " bestimmt werden. Es ist keine Empfangsanmeldung mglich.");
      }
    }
    else
    {
      System.err.println("Es wurde bereits eine Leseanmeldung durchgefhrt.");
    }
  }

  private void abmeldenEmpfang()
  {
    if ((_connection != null) && (_ddLeseAnmeldung != null))
    {
      if (_lSoLeseAnmeldung.size() > 0)
      {
        _connection.unsubscribeReceiver(this, _lSoLeseAnmeldung, _ddLeseAnmeldung);

        this._lSoLeseAnmeldung.clear();
        this._ddLeseAnmeldung = null;
      }
    }
  }

  @Override
  public void update(ResultData[] results)
  {
    if ((results != null) && (results.length > 0))
    {
      for (ResultData resultData : results)
      {
        /*
         * MO 07.09.11 Die Workflow werden nicht mehr pro WorkflowObjekte (Fahrtrichtung) betrachtet, sondern pro
         * Abschnitt. Nur so ist es mit der derzeitigen SW mglich Empfehlungen fr einzelne Abschnitte einer
         * Fahrtrichtung zu ndern bzw. wegzunehmen.
         */
        SystemObject soWorkflow = resultData.getObject();

        if (!_zuordnungen.containsKey(soWorkflow))
        {
          _zuordnungen.put(soWorkflow, new Vector<SystemObject>());
        }

        if (resultData.hasData())
        {
          Data data = resultData.getData();
          if (data != null)
          {
            LinkedList<M_TSFWorkflowAbschnitt> lAbschnitte = new LinkedList<M_TSFWorkflowAbschnitt>();

            Array arrayAbschnitte = data.getArray("Abschnitte");
            for (int i = 0; i < arrayAbschnitte.getLength(); i++)
            {
              Data abs = arrayAbschnitte.getItem(i);
              SystemObject soAbschnitt = abs.getReferenceValue("Abschnitt").getSystemObject();

              if (!_zuordnungen.get(soWorkflow).contains(soAbschnitt))
                _zuordnungen.get(soWorkflow).add(soAbschnitt);

              int wertAktion = abs.getUnscaledValue("Aktion").intValue();
              E_TSFBefehl aktion = E_TSFBefehl.getBefehl(wertAktion);

              /*
               * MO 06.09.11 Bei Aktion = KEINE muss der Workflow beendet werden
               */
              if (aktion != E_TSFBefehl.Keine && aktion != E_TSFBefehl.Abbrechen)
              {
                int wertUrsache = abs.getUnscaledValue("Ursache").intValue();
                E_TSFUrsache ursache = E_TSFUrsache.getBefehl(wertUrsache);

                // System.out.println("--> DaVKomTSFWorkflow.update() "
                // + soAbschnitt.getPid() + " - " + aktion.name());

                M_TSFWorkflowAbschnitt tsfWorkflowAbschnitt = new M_TSFWorkflowAbschnitt(soAbschnitt, aktion, ursache,
                    resultData.getDataTime());
                lAbschnitte.add(tsfWorkflowAbschnitt);

                M_TSFWorkflowAbschnitt[] aAbschnitte = lAbschnitte.toArray(new M_TSFWorkflowAbschnitt[lAbschnitte
                    .size()]);
                M_TSFWorkflow wf = new M_TSFWorkflow(soAbschnitt, aAbschnitte);

                getModel().putWorkflow(wf);
              }
              else
              {
                getModel().removeWorkflow(soAbschnitt);
              }
            }

            // M_TSFWorkflowAbschnitt[] aAbschnitte = lAbschnitte.toArray(new
            // M_TSFWorkflowAbschnitt[lAbschnitte.size()]);
            //
            // /*
            // * MO 06.09.11
            // * Workflow nur ffnen wenn auch Abschnitte betroffen sind (Aktion
            // =/= KEINE)
            // */
            // if (aAbschnitte.length > 0)
            // {
            // M_TSFWorkflow wf = new M_TSFWorkflow(soWorkflow, aAbschnitte);
            // getModel().putWorkflow(wf);
            // }
            // else
            // {
            // if (soWorkflow != null)
            // {
            // getModel().removeWorkflow(soWorkflow);
            // }
            // }
          }
        }
        else
        {
          if (soWorkflow != null)
          {
            for (SystemObject so : _zuordnungen.get(soWorkflow))
            {
              System.out.println("DaVKomTSFWorkflow.update(REM1)");
              getModel().removeWorkflow(so);
            }
          }
        }
      }
    }
  }

  public void updateMO(ResultData[] results)
  {
    if ((results != null) && (results.length > 0))
    {
      for (ResultData resultData : results)
      {
        SystemObject soWorkflow = resultData.getObject();
        if (resultData.hasData())
        {
          Data data = resultData.getData();
          if (data != null)
          {
            LinkedList<M_TSFWorkflowAbschnitt> lAbschnitte = new LinkedList<M_TSFWorkflowAbschnitt>();

            Array arrayAbschnitte = data.getArray("Abschnitte");
            for (int i = 0; i < arrayAbschnitte.getLength(); i++)
            {
              Data abs = arrayAbschnitte.getItem(i);
              SystemObject soAbschnitt = abs.getReferenceValue("Abschnitt").getSystemObject();
              int wertAktion = abs.getUnscaledValue("Aktion").intValue();
              E_TSFBefehl aktion = E_TSFBefehl.getBefehl(wertAktion);

              /*
               * MO 06.09.11 Bei Aktion = KEINE muss der Workflow beendet werden
               */
              if (aktion != E_TSFBefehl.Keine)
              {
                int wertUrsache = abs.getUnscaledValue("Ursache").intValue();
                E_TSFUrsache ursache = E_TSFUrsache.getBefehl(wertUrsache);

                M_TSFWorkflowAbschnitt tsfWorkflowAbschnitt = new M_TSFWorkflowAbschnitt(soAbschnitt, aktion, ursache,
                    resultData.getDataTime());
                lAbschnitte.add(tsfWorkflowAbschnitt);
              }
            }

            M_TSFWorkflowAbschnitt[] aAbschnitte = lAbschnitte.toArray(new M_TSFWorkflowAbschnitt[lAbschnitte.size()]);

            /*
             * MO 06.09.11 Workflow nur ffnen wenn auch Abschnitte betroffen sind (Aktion =/= KEINE)
             */
            if (aAbschnitte.length > 0)
            {
              M_TSFWorkflow wf = new M_TSFWorkflow(soWorkflow, aAbschnitte);
              getModel().putWorkflow(wf);
            }
            else
            {
              if (soWorkflow != null)
              {
                System.out.println("DaVKomTSFWorkflow.update(REM3)");
                getModel().removeWorkflow(soWorkflow);
              }
            }
          }
        }
        else
        {
          if (soWorkflow != null)
          {
            System.out.println("DaVKomTSFWorkflow.update(REM4)");
            getModel().removeWorkflow(soWorkflow);
          }
        }
      }
    }
  }

  /**
   * Liefert das Modell fr das GUI.
   * 
   * @return
   */
  public GUIModelTSFWorkflow getModel()
  {
    return _model;
  }

  public GUIManager_WorkflowDialog getGuiwfDialogManager()
  {
    return _guiwfDialogManager;
  }

  public void dispose()
  {
    abmeldenEmpfang();

    if (_model != null)
    {
      _model.dispose();
    }
  }

  @Override
  public void postShutdown(IWorkbench workbench)
  {
    //Nichts
  }

  @Override
  public boolean preShutdown(IWorkbench workbench, boolean forced)
  {
    //TODO implement
    
    IViewReference[] aRefs = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getViewReferences();
    for (IViewReference ref : aRefs)
    {
      if((ref!=null) &&(ref.getView(false)!=null))
      {
        IViewPart view = ref.getView(false);
        if(view instanceof ViewWFEinfach)
        {
          ViewWFEinfach wfEinfach = (ViewWFEinfach) view;
          wfEinfach.schliesseViewMitErlaubnis();
        }
      }
    }
    
    
    
    return true;
  }

}
