/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.text.SimpleDateFormat;
import java.util.Date;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Befehl der TSF fr einen Abschnitt. Daten der ATG
 * <code>atg.tsfAbschnittBefehl</code>
 * 
 * @author HofmannJ
 * 
 */
public class M_TSFAbschnittBefehl
{
	/**
	 * ID der Oberflche.
	 */
	private String _id = null;

	/**
	 * Feld Aktion, Typ <code>att.tsfBefehl</code>.
	 */
	private E_TSFBefehl _aktion = null;

	/**
	 * Feld Zuastand, Typ <code>TSFProgrammZustand</code>.
	 */
	private E_TSFProgrammZustand _zustand = null;

	/**
	 * SystemObjekt des Abschnitts <code>typ.tsfAbschnitt</code> auf den sich
	 * dieser Befehl bezieht.
	 */
	private SystemObject _so = null;
	public static final SimpleDateFormat SDF = new SimpleDateFormat(
	"dd.MM.yyyy HH:mm:ss");

	public M_TSFAbschnittBefehl(String idGui, SystemObject so,
			E_TSFBefehl aktion, E_TSFProgrammZustand zustand)
	{
		setId(idGui);
		setSo(so);
		setAktion(aktion);
		setZustand(zustand);
	}

	/**
	 * Liefert das SystemObjekt des Abschnitts <code>typ.tsfAbschnitt</code>.
	 * 
	 * @return SystemObjekt des Abschnitts <code>typ.tsfAbschnitt</code>
	 */
	public SystemObject getSo()
	{
		return _so;
	}

	/**
	 * Setzt das SystemObjekt des Abschnitts <code>typ.tsfAbschnitt</code>.
	 * 
	 * @param so
	 *          SystemObjekt des Abschnitts <code>typ.tsfAbschnitt</code>
	 */
	public void setSo(SystemObject so)
	{
		_so = so;
	}

	public E_TSFBefehl getAktion()
	{
		return _aktion;
	}

	public void setAktion(E_TSFBefehl aktion)
	{
		_aktion = aktion;
	}

	public String getId()
	{
		return _id;
	}

	public void setId(String id)
	{
		_id = id;
	}

	public E_TSFProgrammZustand getZustand()
	{
		return _zustand;
	}

	public void setZustand(E_TSFProgrammZustand zustand)
	{
		_zustand = zustand;
	}

	@Override
	public String toString()
	{
		StringBuffer sb = new StringBuffer();

		sb.append("TSFAbschnittBefehl\n");

		sb.append("  ID GUI=");
		if (getId() != null)
		{
			sb.append(getId());
		} else
		{
			sb.append("-");
		}
		sb.append("\n");

		sb.append("  Objekt=");
		if (getSo() != null)
		{
			sb.append(getSo());
		} else
		{
			sb.append("-");
		}
		sb.append("\n");

		sb.append("  Aktion=");
		if (getAktion() != null)
		{
			sb.append(getAktion());
		} else
		{
			sb.append("-");
		}

		return sb.toString();
	}

	public String toStringSpec()
	{
		StringBuffer sb = new StringBuffer();

		if (getSo() != null)
		{
			sb.append("PID=");
			sb.append(getSo().getPidOrNameOrId());
			sb.append(", ");
		}

		if (getAktion() != null)
		{
			sb.append("AKTION=");
			sb.append(getAktion());
			sb.append(", ");
		}

		if (getZustand() != null)
		{
			sb.append("ZUSTAND=");
			sb.append(getZustand());
		}

		return sb.toString();
	}

	public M_TSFAbschnittBefehl duplicate()
	{
		SystemObject so = getSo();

		String id = null;
		if (getId() != null)
		{
			id = new String(getId());
		}

		E_TSFBefehl aktion = getAktion();
		E_TSFProgrammZustand zustand = getZustand();

		M_TSFAbschnittBefehl duplikat = new M_TSFAbschnittBefehl(id, so, aktion,
				zustand);
		return duplikat;
	}

	public Data toData(ClientDavInterface con)
	{
		Data data = null;

		if (con != null)
		{
			DataModel dataModel = con.getDataModel();
			AttributeGroup atg = dataModel
					.getAttributeGroup(Defs.ATG_TSF_ABSCHNITT_BEFEHL);
			data = con.createData(atg);
			
			long l = System.currentTimeMillis();
			
			data.getTextValue("ID").setText(getId() + "#" + SDF.format(new Date(l)));
			data.getUnscaledValue("Aktion").set(getAktion().getWert());
			data.getUnscaledValue("Zustand").set(getZustand().getWert());
		}
		
		return data;
	}
}
