/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.util.LinkedList;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.SystemObject;

public class M_TSFAbschnitte
{
	/**
	 * SystemObjekt vom Typ <code>typ.temporaereSeitenstreifenFreigabe</code>.
	 */
	private SystemObject _so = null;

	/**
	 * Feld <code>Abschnitte</code>, Referenz auf Objekte vom Typ
	 * <code>typ.tsfAbschnitt</code>.
	 */
	private LinkedList<SystemObject> _lAbschnitte = new LinkedList<SystemObject>();

	public M_TSFAbschnitte(ResultData rd)
	{
		if (rd != null)
		{
			setSo(rd.getObject());

			if ((rd.hasData()) && (rd.getData() != null))
			{
				Data data = rd.getData();
				readData(data);
			}

		}
	}

	public M_TSFAbschnitte(SystemObject soTfsAbschnitt, Data data)
	{
		setSo(soTfsAbschnitt);
		readData(data);
	}

	/**
	 * Liest das Data-Objekt ein. Die Abschnitte werden i.d. Reihenfolge des DaV
	 * gesichert, d.h. in Fahrtrichtung.
	 * 
	 * @param data
	 *          Einzulesendes Data-Objekt.
	 */
	private void readData(Data data)
	{
		ReferenceArray referenceArray = data.getReferenceArray("Abschnitte");
		if ((referenceArray != null) && (referenceArray.getLength() > 0))
		{
			SystemObject[] aSO = referenceArray.getSystemObjectArray();

			if ((aSO != null) && (aSO.length > 0))
				for (SystemObject so : aSO)
				{
					synchronized (_lAbschnitte)
					{
						_lAbschnitte.add(so);
					}
				}
		}
	}

	/**
	 * Liefert das SystemObjekt vom Typ
	 * <code>typ.temporaereSeitenstreifenFreigabe</code>.
	 * 
	 * @return SystemObjekt vom Typ
	 *         <code>typ.temporaereSeitenstreifenFreigabe</code>.
	 */
	public SystemObject getSo()
	{
		return _so;
	}

	/**
	 * Setzt das SystemObjekt vom Typ
	 * <code>typ.temporaereSeitenstreifenFreigabe</code>.
	 * 
	 * @param so
	 *          SystemObjekt vom Typ
	 *          <code>typ.temporaereSeitenstreifenFreigabe</code>.
	 */
	private void setSo(SystemObject so)
	{
		_so = so;
	}

	/**
	 * Liefert alle zugeordneten SystemObjekte vom Typ
	 * <code>typ.tsfAbschnitt</code>. Die Abschnitte werden in der Reihenfolge,
	 * wie im DaV hinterlegt, geliefert. Die Reihenfolge ist also in
	 * Fahrtrichtung.
	 * 
	 * @return Array mit SystemObjekten. Kann leer sein, wird nicht
	 *         <code>null</code>.
	 */
	public SystemObject[] getAbschnitte()
	{
		SystemObject[] aAbschnitte = null;

		synchronized (_lAbschnitte)
		{
			aAbschnitte = _lAbschnitte.toArray(new SystemObject[_lAbschnitte.size()]);
		}

		return aAbschnitte;
	}

	@Override
	public String toString()
	{
		StringBuffer sb = new StringBuffer();

		sb.append("TFS-Objekt: ");
		if (getSo() != null)
		{
			sb.append(getSo().getNameOrPidOrId());
		} else
		{
			sb.append("-");
		}
		sb.append("\n");

		SystemObject[] abschnitte = getAbschnitte();
		sb.append("Anzahl Abschnitte: " + abschnitte.length);
		sb.append("\n");

		for (int i = 0; i < abschnitte.length; i++)
		{
			SystemObject soAbs = abschnitte[i];
			sb.append("\tAbschnitt #");
			sb.append(i);
			sb.append(": ");
			if (soAbs != null)
			{
				sb.append(soAbs.getNameOrPidOrId());
			} else
			{
				sb.append("-");
			}

			if ((i + 1) < abschnitte.length)
			{
				sb.append("\n");
			}
		}

		return sb.toString();
	}

}
