/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dlg;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.sts.buv.plugin.workflow.tsf.dav.DaVKomTSFWorkflow;
import de.sts.buv.plugin.workflow.tsf.dav.GUIModelTSFWorkflow;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFWorkflow;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFWorkflowAbschnitt;

/**
 * Erzeugt eine Tabelle, in der offene Schaltempfehlungen dargestellt und ber
 * die offene Schaltempfehlungen wieder geffnet werden knne.
 * 
 * @author HofmannJ
 * 
 */
public class GeneratorCompositeWFSchaltempfehlungenUebersichtCOPY implements
		PropertyChangeListener
{
	private static final boolean DEBUG = true;

	private static final int IDX_SPALTE_BEREICH = 0;
	private static final int IDX_SPALTE_ABSCHNITTE = 1;
	private static final String[] COL_NAMES = new String[] { "Bereich",
			"Abschnitte" };

	/**
	 * Key = PID des Bereichs des Workflow, Value = {@link M_TSFWorkflow}.
	 */
	private TreeMap<String, M_TSFWorkflow> _map = new TreeMap<String, M_TSFWorkflow>();

	private TableViewer _tv;

	private ViewPart _viewPart = null;

	private Action _aWfAnzeigen;

	public GeneratorCompositeWFSchaltempfehlungenUebersichtCOPY(Composite parent,
			ViewPart viewPart)
	{
		this._viewPart = viewPart;

		if (parent != null)
		{
			parent.setLayout(new FillLayout());

			// Label l = new Label(parent, SWT.NONE);
			// l.setText("TEST BEREIT");

			createTable(parent);
			verankereKontextMenu();
		}

		DaVKomTSFWorkflow.getInstance().getModel().addListener(this);
	}

	private void createTable(Composite parent)
	{
		Table table = new Table(parent, SWT.FULL_SELECTION | SWT.SINGLE
				| SWT.H_SCROLL | SWT.V_SCROLL);
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		_tv = new TableViewer(table);
		_tv.setContentProvider(new CP());
		_tv.setLabelProvider(new LP());

		TableColumn tcBereich = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcBereich.setText(COL_NAMES[IDX_SPALTE_BEREICH]);
		tcBereich.setWidth(300);

		TableColumn tcAbschnitte = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcAbschnitte.setText(COL_NAMES[IDX_SPALTE_ABSCHNITTE]);
		tcAbschnitte.setWidth(600);
	}

	public void dispose()
	{
		DaVKomTSFWorkflow.getInstance().getModel().removeListener(this);
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt)
	{
		boolean doTableUpdate = false;

		if ((evt != null) && (evt.getSource() instanceof GUIModelTSFWorkflow))
		{
			if (evt.getNewValue() != null)
			{
				if (evt.getPropertyName() != null)
				{
					if ((evt.getPropertyName().equals(GUIModelTSFWorkflow.PROP_INIT))
							&& (evt.getNewValue() instanceof M_TSFWorkflow[]))
					{
						M_TSFWorkflow[] aWorkflows = (M_TSFWorkflow[]) evt.getNewValue();
						for (M_TSFWorkflow workflow : aWorkflows)
						{
							addWorkflow(workflow);
							doTableUpdate = true;
						}

					} else if ((evt.getPropertyName()
							.equals(GUIModelTSFWorkflow.PROP_ADDED))
							&& (evt.getNewValue() instanceof M_TSFWorkflow))
					{
						M_TSFWorkflow workflow = (M_TSFWorkflow) evt.getNewValue();
						addWorkflow(workflow);
						doTableUpdate = true;
					}
				}
			} else if (evt.getOldValue() != null)
			{
				if ((evt.getPropertyName().equals(GUIModelTSFWorkflow.PROP_REMOVED))
						&& (evt.getOldValue() instanceof M_TSFWorkflow))
				{
					M_TSFWorkflow workflow = (M_TSFWorkflow) evt.getOldValue();
					removeWorkflow(workflow);
					doTableUpdate = true;
				}
			}
		}

		if (doTableUpdate)
		{
			updateTable();
		}
	}

	/**
	 * Verankert das Kontextmen an der Tabelle mit der bersicht der offenen
	 * Schaltempfehlungen.
	 * 
	 */
	private void verankereKontextMenu()
	{
		if (_tv != null)
		{
			MenuManager menuMgr = new MenuManager("#PopupMenu");
			menuMgr.setRemoveAllWhenShown(true);
			menuMgr.addMenuListener(new IMenuListener()
			{
				public void menuAboutToShow(IMenuManager manager)
				{
					fuelleKontextMenu(manager);
				}
			});

			Menu menu = menuMgr.createContextMenu(_tv.getControl());
			_tv.getControl().setMenu(menu);

			if (_viewPart != null)
			{
				_viewPart.getSite().registerContextMenu(menuMgr, _tv);
			} else
			{
				System.err
						.println(this.getClass().getSimpleName()
								+ "::verankereKontextMenu Kontextmen lsst sich nicht an der Site des ViewParts registrieren.");
			}

		}
	}

	/**
	 * Fllt das Kontextmen.
	 * 
	 * @param manager
	 *          {@link IMenuManager}, der fr die Darstellung des Kontextmens
	 *          zustndig ist.
	 */
	private void fuelleKontextMenu(IMenuManager manager)
	{

		ISelection selection = _tv.getSelection();
		if (selection instanceof StructuredSelection)
		{
			StructuredSelection ss = (StructuredSelection) selection;
			if (ss.size() > 0)
			{
				manager.add(getActionWorkflowAnzeigen());

			}
		}

		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	/**
	 * Liefert die Aktion zum wiederanzeigen eines offenen Workflows.
	 * 
	 * @return
	 */
	private Action getActionWorkflowAnzeigen()
	{
		if (_aWfAnzeigen == null)
		{
			_aWfAnzeigen = new Action("&Workflow anzeigen")
			{
				@Override
				public void run()
				{
					if (_tv != null)
					{
						ISelection selection = _tv.getSelection();
						if (selection instanceof StructuredSelection)
						{
							StructuredSelection ss = (StructuredSelection) selection;

							if (ss.size() > 0)
							{
								Object firstElement = ss.getFirstElement();
								if (firstElement instanceof M_TSFWorkflow)
								{
									M_TSFWorkflow tsfWorkflow = (M_TSFWorkflow) firstElement;
									reopenWorkflow(tsfWorkflow);
								}
							}
						}
					}
				}
			};
			_aWfAnzeigen.setId(GeneratorCompositeWFSchaltempfehlungenUebersichtCOPY.class
					.getName()
					+ ".WfAnzeigen");
		}

		return _aWfAnzeigen;
	}

	private void addWorkflow(M_TSFWorkflow workflow)
	{
		if ((workflow != null) && (workflow.getSo() != null))
		{
			if (DEBUG)
			{
				System.out.println("=> Fge Workflow hinzu. Bereich="
						+ workflow.getSo());
			}

			synchronized (_map)
			{
				_map.put(workflow.getSo().getPid(), workflow);
			}

			M_TSFWorkflowAbschnitt[] abschnitte = workflow.getAbschnitte();
			if (abschnitte.length > 0)
			{
				for (M_TSFWorkflowAbschnitt wfAbschnitt : abschnitte)
				{
					if (DEBUG)
					{
						System.out.println("\t->Abschnitt: " + wfAbschnitt.getAbschnitt());
					}
				}
			}

		}
	}

	private void removeWorkflow(M_TSFWorkflow workflow)
	{
		if ((workflow != null) && (workflow.getSo() != null))
		{
			if (DEBUG)
			{
				System.out.println("=> Entferne Workflow. Bereich=" + workflow.getSo());
			}

			synchronized (_map)
			{
				_map.remove(workflow.getSo().getPid());
			}

			M_TSFWorkflowAbschnitt[] abschnitte = workflow.getAbschnitte();
			if (abschnitte.length > 0)
			{
				for (M_TSFWorkflowAbschnitt wfAbschnitt : abschnitte)
				{
					if (DEBUG)
					{
						System.out.println("\t->Abschnitt: " + wfAbschnitt.getAbschnitt());
					}
				}
			}
		}
	}

	private M_TSFWorkflow[] getWorkflows()
	{
		LinkedList<M_TSFWorkflow> liste = new LinkedList<M_TSFWorkflow>();

		synchronized (_map)
		{
			Iterator<Entry<String, M_TSFWorkflow>> it = _map.entrySet().iterator();
			while (it.hasNext())
			{
				Entry<String, M_TSFWorkflow> entry = it.next();
				liste.add(entry.getValue());
			}
		}

		return liste.toArray(new M_TSFWorkflow[liste.size()]);
	}

	private void reopenWorkflow(M_TSFWorkflow workflow)
	{
		DaVKomTSFWorkflow.getInstance().getGuiwfDialogManager().genFillDlgWF(
				workflow);
	}

	private void updateTable()
	{
		if (_tv != null)
		{

			Runnable run = new Runnable()
			{
				@Override
				public void run()
				{
					_tv.setInput(getWorkflows());
				}
			};

			PlatformUI.getWorkbench().getDisplay().asyncExec(run);

		}
	}

	private class CP implements IStructuredContentProvider
	{

		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = null;

			if (inputElement instanceof M_TSFWorkflow[])
			{
				re = (M_TSFWorkflow[]) inputElement;
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	private class LP implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			return null;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			String re = null;

			if ((element != null) && (element instanceof M_TSFWorkflow))
			{
				M_TSFWorkflow wf = (M_TSFWorkflow) element;

				switch (columnIndex)
				{
				case IDX_SPALTE_BEREICH:
					if (wf.getSo() != null)
					{
						re = wf.getSo().getNameOrPidOrId();
					}
					break;
				case IDX_SPALTE_ABSCHNITTE:
					M_TSFWorkflowAbschnitt[] abschnitte = wf.getAbschnitte();
					if ((abschnitte != null) && (abschnitte.length > 0))
					{
						StringBuffer sb = new StringBuffer();

						for (int i = 0; i < abschnitte.length; i++)
						{
							if ((abschnitte[i] != null)
									&& (abschnitte[i].getAbschnitt() != null))
							{
								sb.append(abschnitte[i].getAbschnitt().getNameOrPidOrId());
							} else
							{
								sb.append("-");
							}

							if ((i + 1) < abschnitte.length)
							{
								sb.append("; ");
							}
						}

						re = sb.toString();
					}

					break;
				}
			}

			if (re == null)
			{
				re = "";
			}
			return re;
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}
}
