/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dlg;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.sts.buv.plugin.workflow.tsf.ImageGetter;
import de.sts.buv.plugin.workflow.tsf.dav.Defs;
import de.sts.buv.plugin.workflow.tsf.dav.E_TSFBefehl;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFWeiterSchalten;
import de.sts.buv.plugin.workflow.tsf.dav.WiedervorlageTSFWeiterschalten;
import de.sts.buv.plugin.workflow.tsf.dav.WiedervorlageTSFWorkflowAbschnitt;

/**
 * Erzeugt eine Tabelle, in der offene Wiedervorlagen dargestellt und ber diese
 * geffnet oder abgebrochen werden knnen.
 * 
 * @author HofmannJ
 * 
 */
public class GeneratorCompositeWiedervorlagen implements PropertyChangeListener
{
	private static final boolean DEBUG = true;

//	private static final int IDX_SPALTE_ID = 0;
	private static final int IDX_SPALTE_BESCHREIBUNG = 0;
	private static final int IDX_SPALTE_TIMEOUT = 1;
	private static final int IDX_SPALTE_WIEDERVORLAGE_ZEITPUNKT = 2;
	private static final String[] COL_NAMES = new String[] { //"ID",
			"Beschreibung", "Timeout", "Wiedervorlage Zeitpunkt" };

	/**
	 * Key = ID der Wiedervorlage, Value = {@link IWiedervorlage}.
	 */
	private TreeMap<String, IWiedervorlage> _map = new TreeMap<String, IWiedervorlage>();

	private TableViewer _tv;

	private ViewPart _viewPart = null;

	private Action _aVorzeitigeWiedervorlage;

	public GeneratorCompositeWiedervorlagen(Composite parent, ViewPart viewPart)
	{
		this._viewPart = viewPart;

		if (parent != null)
		{
			parent.setLayout(new FillLayout());

			// Label l = new Label(parent, SWT.NONE);
			// l.setText("TEST BEREIT");

			createTable(parent);
			verankereKontextMenu();
		}

		RegistryWiedervorlagen.getInstance().addListener(this);
	}

	private void createTable(Composite parent)
	{
		Table table = new Table(parent, SWT.FULL_SELECTION | SWT.SINGLE
				| SWT.H_SCROLL | SWT.V_SCROLL);
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		_tv = new TableViewer(table);
		_tv.setContentProvider(new CP());
		_tv.setLabelProvider(new LP());

//		TableColumn tcBereich = new TableColumn(_tv.getTable(), SWT.LEFT);
//		tcBereich.setText(COL_NAMES[IDX_SPALTE_ID]);
//		tcBereich.setWidth(400);

		TableColumn tcAbschnitte = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcAbschnitte.setText(COL_NAMES[IDX_SPALTE_BESCHREIBUNG]);
		tcAbschnitte.setWidth(300);

		TableColumn tcTimeout = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcTimeout.setText(COL_NAMES[IDX_SPALTE_TIMEOUT]);
		tcTimeout.setWidth(100);

		TableColumn tcWiedervorlageZeitpkt = new TableColumn(_tv.getTable(),
				SWT.LEFT);
		tcWiedervorlageZeitpkt
				.setText(COL_NAMES[IDX_SPALTE_WIEDERVORLAGE_ZEITPUNKT]);
		tcWiedervorlageZeitpkt.setWidth(150);

		table.addMouseListener(new MouseAdapter()
		{

			@Override
			public void mouseDoubleClick(MouseEvent e)
			{
				if (e.button == 1)
				{
					Action actionWiedervorlageVorzeitigDarstellen = getActionWiedervorlageVorzeitigDarstellen();
					actionWiedervorlageVorzeitigDarstellen.run();
				}
			}

		});
	}

	public void dispose()
	{
		RegistryWiedervorlagen.getInstance().removeListener(this);
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt)
	{
		boolean doTableUpdate = false;

		if ((evt != null) && (evt.getSource() instanceof RegistryWiedervorlagen))
		{
			if (evt.getNewValue() != null)
			{
				if (evt.getPropertyName() != null)
				{
					if ((evt.getPropertyName().equals(RegistryWiedervorlagen.PROP_INIT))
							&& (evt.getNewValue() instanceof IWiedervorlage[]))
					{
						IWiedervorlage[] aWiedervorlagen = (IWiedervorlage[]) evt
								.getNewValue();
						for (IWiedervorlage wiedervorlage : aWiedervorlagen)
						{
							addWiedervorlage(wiedervorlage);
							doTableUpdate = true;
						}

					} else if ((evt.getPropertyName()
							.equals(RegistryWiedervorlagen.PROP_ADD))
							&& (evt.getNewValue() instanceof IWiedervorlage))
					{
						IWiedervorlage wiedervorlage = (IWiedervorlage) evt.getNewValue();
						addWiedervorlage(wiedervorlage);
						doTableUpdate = true;
					}
				}
			} else if (evt.getOldValue() != null)
			{
				if ((evt.getPropertyName().equals(RegistryWiedervorlagen.PROP_REMOVE))
						&& (evt.getOldValue() instanceof IWiedervorlage))
				{
					IWiedervorlage wiedervorlage = (IWiedervorlage) evt.getOldValue();
					removeWiedervorlage(wiedervorlage);
					doTableUpdate = true;
				}
			}
		}

		if (doTableUpdate)
		{
			updateTable();
		}
	}

	/**
	 * Verankert das Kontextmen an der Tabelle mit der bersicht der offenen
	 * Schaltempfehlungen.
	 * 
	 */
	private void verankereKontextMenu()
	{
		if (_tv != null)
		{
			MenuManager menuMgr = new MenuManager("#PopupMenu");
			menuMgr.setRemoveAllWhenShown(true);
			menuMgr.addMenuListener(new IMenuListener()
			{
				public void menuAboutToShow(IMenuManager manager)
				{
					fuelleKontextMenu(manager);
				}
			});

			Menu menu = menuMgr.createContextMenu(_tv.getControl());
			_tv.getControl().setMenu(menu);

			if (_viewPart != null)
			{
				_viewPart.getSite().registerContextMenu(menuMgr, _tv);
			} else
			{
				System.err
						.println(this.getClass().getSimpleName()
								+ "::verankereKontextMenu Kontextmen lsst sich nicht an der Site des ViewParts registrieren.");
			}

		}
	}

	/**
	 * Fllt das Kontextmen.
	 * 
	 * @param manager
	 *          {@link IMenuManager}, der fr die Darstellung des Kontextmens
	 *          zustndig ist.
	 */
	private void fuelleKontextMenu(IMenuManager manager)
	{

		ISelection selection = _tv.getSelection();
		if (selection instanceof StructuredSelection)
		{
			StructuredSelection ss = (StructuredSelection) selection;
			if (ss.size() > 0)
			{
				manager.add(getActionWiedervorlageVorzeitigDarstellen());

			}
		}

		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	/**
	 * Liefert die Aktion zum wiederanzeigen eines offenen Workflows.
	 * 
	 * @return
	 */
	private Action getActionWiedervorlageVorzeitigDarstellen()
	{
		if (_aVorzeitigeWiedervorlage == null)
		{
			_aVorzeitigeWiedervorlage = new Action(
					"&Wiedervorlage vorzeitig auslsen")
			{
				@Override
				public void run()
				{
					if (_tv != null)
					{
						ISelection selection = _tv.getSelection();
						if (selection instanceof StructuredSelection)
						{
							StructuredSelection ss = (StructuredSelection) selection;

							if (ss.size() > 0)
							{
								Object firstElement = ss.getFirstElement();
								if (firstElement instanceof IWiedervorlage)
								{
									IWiedervorlage wiedervorlage = (IWiedervorlage) firstElement;
									openWiedervorlageVorzeitig(wiedervorlage);
								}
							}
						}
					}
				}
			};
			_aVorzeitigeWiedervorlage.setId(GeneratorCompositeWiedervorlagen.class
					.getName()
					+ ".ActionVorzeitigeWiedervorlage");
			_aVorzeitigeWiedervorlage.setImageDescriptor(ImageGetter
					.getImageDescriptor("icons/refresh.gif"));
		}

		return _aVorzeitigeWiedervorlage;
	}

	private void addWiedervorlage(IWiedervorlage wiedervorlage)
	{
		if ((wiedervorlage != null) && (wiedervorlage.getId() != null))
		{
			synchronized (_map)
			{
				_map.put(wiedervorlage.getId(), wiedervorlage);
			}

		}
	}

	private void removeWiedervorlage(IWiedervorlage wiedervorlage)
	{
		if ((wiedervorlage != null) && (wiedervorlage.getId() != null))
		{

			synchronized (_map)
			{
				_map.remove(wiedervorlage.getId());
			}

		}
	}

	private IWiedervorlage[] getWiedervorlagen()
	{
		LinkedList<IWiedervorlage> liste = new LinkedList<IWiedervorlage>();

		synchronized (_map)
		{
			Iterator<Entry<String, IWiedervorlage>> it = _map.entrySet().iterator();
			while (it.hasNext())
			{
				Entry<String, IWiedervorlage> entry = it.next();
				liste.add(entry.getValue());
			}
		}

		return liste.toArray(new IWiedervorlage[liste.size()]);
	}

	private void openWiedervorlageVorzeitig(IWiedervorlage wiedervorlage)
	{
	  System.out.println("openWiedervorlageVorzeitig(): " + wiedervorlage.getBeschreibung());
	  
		if (wiedervorlage != null)
		{
			wiedervorlage.vorzeitigeWiedervorlage();
		}
	}

	private void updateTable()
	{
		if (_tv != null)
		{

			Runnable run = new Runnable()
			{
				@Override
				public void run()
				{
					_tv.setInput(getWiedervorlagen());
				}
			};

			PlatformUI.getWorkbench().getDisplay().asyncExec(run);

		}
	}

	private class CP implements IStructuredContentProvider
	{

		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = null;

			if (inputElement instanceof IWiedervorlage[])
			{
				re = (IWiedervorlage[]) inputElement;
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	private class LP implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			Image re = null;

			if (element != null)
			{
				switch (columnIndex)
				{
//				case IDX_SPALTE_ID:
				case IDX_SPALTE_BESCHREIBUNG:
					if (element instanceof WiedervorlageTSFWeiterschalten)
					{					 					 
						re = ImageGetter.getImage("icons/Camera_links_16x16.png");
						
						WiedervorlageTSFWeiterschalten wv =(WiedervorlageTSFWeiterschalten)element;
						M_TSFWeiterSchalten sw = wv.getWeiterSchalten();
						if (sw != null){
						  E_TSFBefehl prg = sw.getProgramm();
						  if (prg != null && prg.equals(E_TSFBefehl.Schnellsperrung))
						    re = ImageGetter.getImage("icons/fragezeichen_16x16.png");
						}
					} else if (element instanceof WiedervorlageTSFWorkflowAbschnitt)
					{
						re = ImageGetter.getImage("icons/achtung_16x16.png");
					}
					break;
				}
			}

			return re;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			String re = null;

			if ((element != null) && (element instanceof IWiedervorlage))
			{
				IWiedervorlage wiedervorlage = (IWiedervorlage) element;

				switch (columnIndex)
				{
//				case IDX_SPALTE_ID:
//					if (wiedervorlage.getId() != null)
//					{
//						re = wiedervorlage.getId();
//					}
//					break;
				case IDX_SPALTE_BESCHREIBUNG:
					if (wiedervorlage.getBeschreibung() != null)
					{
						re = wiedervorlage.getBeschreibung();
					}
					break;
				case IDX_SPALTE_TIMEOUT:
					re = "" + wiedervorlage.getTimeout() + " Sekunden";
					break;
				case IDX_SPALTE_WIEDERVORLAGE_ZEITPUNKT:
					if ((wiedervorlage.getStartZeitpunkt() != null)
							&& (wiedervorlage.getTimeout() > 0l))
					{
						long wiedervorlageZeitpunkt = wiedervorlage.getStartZeitpunkt()
								+ (wiedervorlage.getTimeout() * 1000l);
						Date dateWiedervorlageZeitpunkt = new Date(wiedervorlageZeitpunkt);
						re = Defs.SDF.format(dateWiedervorlageZeitpunkt);
					}
					break;
				}
			}

			if (re == null)
			{
				re = "";
			}
			return re;
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}
}
