/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dlg;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.sts.buv.plugin.workflow.tsf.Activator;
import de.sts.buv.plugin.workflow.tsf.dav.DaVKomTSFAbschnittDiagnoseDaten;
import de.sts.buv.plugin.workflow.tsf.dav.GUIModelTSFAbschnittDiagnoseDaten;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFAbschnittDiagnoseDaten;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFMQDiagnoseDaten;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFMQDiagnoseParameter;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFMQDiagnoseWerte;

/**
 * View zu Darstellung der TSF-Diagnosedaten fr einen Abschnitt. Diese View
 * lsst sich mehrfach ffnen, an Hand der Secondary-ID wird der TSF-Abschnitt
 * (Typ <code>typ.tsfAbschnitt</code>) festgelegt, fr den die Daten dargestellt
 * werden sollen.<br>
 * <br>
 * Um Daten zu erhalten, wird ein {@link PropertyChangeListener} bei der Modell
 * Klasse {@link DaVKomTSFAbschnittDiagnoseDaten} angemeldet. Das Modell wird
 * ber {@link DaVKomTSFAbschnittDiagnoseDaten#getModel()} bezogen.
 * 
 * @author HofmannJ
 * 
 */
public class ViewTsfAbschnittDiagnoseDaten extends ViewPart implements
		PropertyChangeListener
{
	public static final String ID = "de.sts.buv.plugin.workflow.tsf.dlg.ViewTsfAbschnittDiagnoseDaten";

	/**
	 * Speichert die PID des TSF-Abschnitts, fr den die Diagnosedaten dargestellt
	 * werden sollen.
	 */
	private String _pidTsfAbschnitt;

	/**
	 * Feld zur Darstellung der PID des Abschnitts.
	 */
	private Text _tAbschnitt;

	/**
	 * Darzustellende Daten
	 */
	private M_TSFAbschnittDiagnoseDaten _tsfAbschnittDiagnoseDaten = null;

	/**
	 * Feld fr <code>ZeitbereichKeineFreigabe</code>.
	 */
	private Text _tZeitbereichKeineFreigabe;

	/**
	 * Feld fr <code>PrioZeitbereichKeineFreigabe</code>.
	 */
	private Text _tPrioZeitbereichKeineFreigabe;

	/**
	 * Feld fr <code>ZeitbereichFreigabe</code>.
	 */
	private Text _tZeitbereichFreigabe;

	/**
	 * Feld fr <code>PrioZeitbereichFreigabe</code>.
	 */
	private Text _tPrioZeitbereichFreigabe;

	/**
	 * Feld fr <code>ZeitbereichFreigabeDurchAutomatik</code>.
	 */
	private Text _tZeitbereichFreigabeDurchAutomatik;

	/**
	 * Feld fr <code>PrioZeitbereichFreigabeDurchAutomatik</code>.
	 */
	private Text _tPrioZeitbereichFreigabeDurchAutomatik;

	/**
	 * Feld fr <code>TSFSollZustand</code>.
	 */
	private Text _tTSFSollZustand;

	/**
	 * Feld fr <code>TSFIstZustand</code>.
	 */
	private Text _tTSFIstZustand;

	private static final String[] COL_NAMES = new String[] { "MQ", "Empfehlung",
			"QBEin", "VKfzEin", "AnzahlMessZyklenEin", "AnzahlMessZyklenAus", "QB",
			"V", "AnzahlMessZyklenEin", "AnzahlMessZyklenAus" };

	private TableViewer _tv;

	private Composite _top;

	@Override
	public void createPartControl(Composite parent)
	{
		_pidTsfAbschnitt = getViewSite().getSecondaryId();

		if (_pidTsfAbschnitt != null)
		{
			setPartName("Diagnose TSF Abschnitt " + _pidTsfAbschnitt);
			
			ClientDavInterface con = DaVVerbindung.getInstanz().getDaVVerbindung();
			if(con!=null)
			{
				DataModel dataModel = con.getDataModel();
				if(dataModel!=null)
				{
					SystemObject soTsfAbs = dataModel.getObject(_pidTsfAbschnitt);
					if(soTsfAbs!=null)
					{
						setPartName("Diagnose TSF Abschnitt " + soTsfAbs.getNameOrPidOrId());
					}
				}
			}
		} else
		{
			setPartName("Diagnose TSF Abschnitt -");
		}

		_top = new Composite(parent, SWT.BORDER);
		
		 //Fr die F1 Taste
	    PlatformUI.getWorkbench().getHelpSystem().setHelp( _top, Activator.PLUGIN_ID + ".tsfDiagnoseAbschnitt" );
		
		_top.setLayout(new GridLayout(1, false));

		addGrpOverview(_top);
		addTable(_top);

		DaVKomTSFAbschnittDiagnoseDaten.getInstance().getModel().addListener(this);
	}

	private void addGrpOverview(Composite parent)
	{
		Group grpOverview = new Group(parent, SWT.NONE);
		grpOverview.setText("bersicht");

		// Pid Abschnitt
		Label lAbschnit = new Label(grpOverview, SWT.NONE);
		lAbschnit.setText("Abschnitt: ");

		_tAbschnitt = new Text(grpOverview, SWT.BORDER);
		_tAbschnitt.setEditable(false);

		// ZeitbereichKeineFreigabe
		Label lZeitbereichKeineFreigabe = new Label(grpOverview, SWT.NONE);
		lZeitbereichKeineFreigabe.setText("ZeitbereichKeineFreigabe: ");

		_tZeitbereichKeineFreigabe = new Text(grpOverview, SWT.BORDER);
		_tZeitbereichKeineFreigabe.setEditable(false);

		// PrioZeitbereichKeineFreigabe
		Label lPrioZeitbereichKeineFreigabe = new Label(grpOverview, SWT.NONE);
		lPrioZeitbereichKeineFreigabe.setText("PrioZeitbereichKeineFreigabe: ");

		_tPrioZeitbereichKeineFreigabe = new Text(grpOverview, SWT.BORDER);
		_tPrioZeitbereichKeineFreigabe.setEditable(false);

		// ZeitbereichFreigabe
		Label lZeitbereichFreigabe = new Label(grpOverview, SWT.NONE);
		lZeitbereichFreigabe.setText("ZeitbereichFreigabe: ");

		_tZeitbereichFreigabe = new Text(grpOverview, SWT.BORDER);
		_tZeitbereichFreigabe.setEditable(false);

		// PrioZeitbereichFreigabe
		Label lPrioZeitbereichFreigabe = new Label(grpOverview, SWT.NONE);
		lPrioZeitbereichFreigabe.setText("PrioZeitbereichFreigabe: ");

		_tPrioZeitbereichFreigabe = new Text(grpOverview, SWT.BORDER);
		_tPrioZeitbereichFreigabe.setEditable(false);

		// ZeitbereichFreigabeDurchAutomatik
		Label lZeitbereichFreigabeDurchAutomatik = new Label(grpOverview, SWT.NONE);
		lZeitbereichFreigabeDurchAutomatik
				.setText("ZeitbereichFreigabeDurchAutomatik: ");

		_tZeitbereichFreigabeDurchAutomatik = new Text(grpOverview, SWT.BORDER);
		_tZeitbereichFreigabeDurchAutomatik.setEditable(false);

		// PrioZeitbereichFreigabeDurchAutomatik
		Label lPrioZeitbereichFreigabeDurchAutomatik = new Label(grpOverview,
				SWT.NONE);
		lPrioZeitbereichFreigabeDurchAutomatik
				.setText("PrioZeitbereichFreigabeDurchAutomatik: ");

		_tPrioZeitbereichFreigabeDurchAutomatik = new Text(grpOverview, SWT.BORDER);
		_tPrioZeitbereichFreigabeDurchAutomatik.setEditable(false);

		// TSFSollZustand
		Label lTSFSollZustand = new Label(grpOverview, SWT.NONE);
		lTSFSollZustand.setText("TSFSollZustand: ");

		_tTSFSollZustand = new Text(grpOverview, SWT.BORDER);
		_tTSFSollZustand.setEditable(false);

		// TSFIstZustand
		Label lTSFIstZustand = new Label(grpOverview, SWT.NONE);
		lTSFIstZustand.setText("TSFIstZustand: ");

		_tTSFIstZustand = new Text(grpOverview, SWT.BORDER);
		_tTSFIstZustand.setEditable(false);

		/*
		 * Layout
		 */
		grpOverview.setLayout(new GridLayout(2, false));
		GridData gdGrpOverview = new GridData();
		gdGrpOverview.grabExcessHorizontalSpace = true;
		gdGrpOverview.horizontalAlignment = GridData.FILL;
		grpOverview.setLayoutData(gdGrpOverview);

		GridData gdTAbschnitt = new GridData();
		gdTAbschnitt.grabExcessHorizontalSpace = true;
		gdTAbschnitt.horizontalAlignment = GridData.FILL;
		_tAbschnitt.setLayoutData(gdTAbschnitt);

		GridData gdTZeitbereichKeineFreigabe = new GridData();
		gdTZeitbereichKeineFreigabe.grabExcessHorizontalSpace = true;
		gdTZeitbereichKeineFreigabe.horizontalAlignment = GridData.FILL;
		_tZeitbereichKeineFreigabe.setLayoutData(gdTZeitbereichKeineFreigabe);

		GridData gdTPrioZeitbereichKeineFreigabe = new GridData();
		gdTPrioZeitbereichKeineFreigabe.grabExcessHorizontalSpace = true;
		gdTPrioZeitbereichKeineFreigabe.horizontalAlignment = GridData.FILL;
		_tPrioZeitbereichKeineFreigabe
				.setLayoutData(gdTPrioZeitbereichKeineFreigabe);

		GridData gdTZeitbereichFreigabe = new GridData();
		gdTZeitbereichFreigabe.grabExcessHorizontalSpace = true;
		gdTZeitbereichFreigabe.horizontalAlignment = GridData.FILL;
		_tZeitbereichFreigabe.setLayoutData(gdTZeitbereichFreigabe);

		GridData gdTPrioZeitbereichFreigabe = new GridData();
		gdTPrioZeitbereichFreigabe.grabExcessHorizontalSpace = true;
		gdTPrioZeitbereichFreigabe.horizontalAlignment = GridData.FILL;
		_tPrioZeitbereichFreigabe.setLayoutData(gdTPrioZeitbereichFreigabe);

		GridData gdTZeitbereichFreigabeDurchAutomatik = new GridData();
		gdTZeitbereichFreigabeDurchAutomatik.grabExcessHorizontalSpace = true;
		gdTZeitbereichFreigabeDurchAutomatik.horizontalAlignment = GridData.FILL;
		_tZeitbereichFreigabeDurchAutomatik
				.setLayoutData(gdTZeitbereichFreigabeDurchAutomatik);

		GridData gdTPrioZeitbereichFreigabeDurchAutomatik = new GridData();
		gdTPrioZeitbereichFreigabeDurchAutomatik.grabExcessHorizontalSpace = true;
		gdTPrioZeitbereichFreigabeDurchAutomatik.horizontalAlignment = GridData.FILL;
		_tPrioZeitbereichFreigabeDurchAutomatik
				.setLayoutData(gdTPrioZeitbereichFreigabeDurchAutomatik);

		GridData gdTTSFSollZustand = new GridData();
		gdTTSFSollZustand.grabExcessHorizontalSpace = true;
		gdTTSFSollZustand.horizontalAlignment = GridData.FILL;
		_tTSFSollZustand.setLayoutData(gdTTSFSollZustand);

		GridData gdTTSFIstZustand = new GridData();
		gdTTSFIstZustand.grabExcessHorizontalSpace = true;
		gdTTSFIstZustand.horizontalAlignment = GridData.FILL;
		_tTSFIstZustand.setLayoutData(gdTTSFIstZustand);
	}

	private void addTable(Composite parent)
	{
		Group grpMq = new Group(parent, SWT.NONE);
		grpMq.setText("Messquerschnitte");

		Table table = new Table(grpMq, SWT.FULL_SELECTION | SWT.SINGLE
				| SWT.H_SCROLL | SWT.V_SCROLL);
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		_tv = new TableViewer(table);
		_tv.setContentProvider(new CP());
		_tv.setLabelProvider(new LP());

		TableColumn tcMQ = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcMQ.setText(COL_NAMES[0]);
		tcMQ.setWidth(100);

		TableColumn tcEmpfehlung = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcEmpfehlung.setText(COL_NAMES[1]);
		tcEmpfehlung.setWidth(75);

		TableColumn tcParaQbEin = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcParaQbEin.setText(COL_NAMES[2]);
		tcParaQbEin.setWidth(50);

		TableColumn tcParaVKfzEin = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcParaVKfzEin.setText(COL_NAMES[3]);
		tcParaVKfzEin.setWidth(50);

		TableColumn tcParaAnzMessZykEin = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcParaAnzMessZykEin.setText(COL_NAMES[4]);
		tcParaAnzMessZykEin.setWidth(100);

		TableColumn tcParaAnzMessZykAus = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcParaAnzMessZykAus.setText(COL_NAMES[5]);
		tcParaAnzMessZykAus.setWidth(100);

		TableColumn tcWerteQb = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcWerteQb.setText(COL_NAMES[6]);
		tcWerteQb.setWidth(50);

		TableColumn tcWerteV = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcWerteV.setText(COL_NAMES[7]);
		tcWerteV.setWidth(50);

		TableColumn tcWerteAnzMessZykEin = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcWerteAnzMessZykEin.setText(COL_NAMES[8]);
		tcWerteAnzMessZykEin.setWidth(100);

		TableColumn tcWerteAnzMessZykAus = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcWerteAnzMessZykAus.setText(COL_NAMES[9]);
		tcWerteAnzMessZykAus.setWidth(100);

		/*
		 * Layout
		 */
		grpMq.setLayout(new FillLayout());

		GridData gdGrpMq = new GridData();
		gdGrpMq.grabExcessHorizontalSpace = true;
		gdGrpMq.grabExcessVerticalSpace = true;
		gdGrpMq.horizontalAlignment = GridData.FILL;
		gdGrpMq.verticalAlignment = GridData.FILL;
		grpMq.setLayoutData(gdGrpMq);
	}

//	@Override
//	public void setFocus()
//	{
//	}
	
	@Override
	public void setFocus()
	{
	  //Fr die F1 Taste
	  if (_top != null)
	    _top.setFocus();
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt)
	{
		if ((evt != null) && (evt.getSource() != null)
				&& (evt.getSource() instanceof GUIModelTSFAbschnittDiagnoseDaten))
		{
			if (evt.getNewValue() != null)
			{
				if (evt.getPropertyName().equals(
						GUIModelTSFAbschnittDiagnoseDaten.PROP_INIT))
				{
					if (evt.getNewValue() instanceof M_TSFAbschnittDiagnoseDaten[])
					{
						M_TSFAbschnittDiagnoseDaten[] aTsfAbsDiagnoseDaten = (M_TSFAbschnittDiagnoseDaten[]) evt
								.getNewValue();
						for (M_TSFAbschnittDiagnoseDaten tsfAbschnittDiagnoseDaten : aTsfAbsDiagnoseDaten)
						{
							if (pruefeUndSetzeDaten(tsfAbschnittDiagnoseDaten))
							{
								break;
							}
						}
					}
				} else if (evt.getPropertyName().equals(
						GUIModelTSFAbschnittDiagnoseDaten.PROP_ADD))
				{
					if (evt.getNewValue() instanceof M_TSFAbschnittDiagnoseDaten)
					{
						M_TSFAbschnittDiagnoseDaten tsfAbschnittDiagnoseDaten = (M_TSFAbschnittDiagnoseDaten) evt
								.getNewValue();
						pruefeUndSetzeDaten(tsfAbschnittDiagnoseDaten);
					}
				}
			} else
			{
				if ((evt.getOldValue() != null)
						&& (evt.getPropertyName()
								.equals(GUIModelTSFAbschnittDiagnoseDaten.PROP_REMOVED)))
				{

				}
			}
		}

	}

	private boolean pruefeUndSetzeDaten(
			final M_TSFAbschnittDiagnoseDaten tsfAbschnittDiagnoseDaten)
	{
		boolean datenGesetzt = false;
		if ((tsfAbschnittDiagnoseDaten != null)
				&& (tsfAbschnittDiagnoseDaten.getSo() != null)
				&& (tsfAbschnittDiagnoseDaten.getSo().getPid().equals(_pidTsfAbschnitt)))
		{
			Runnable run = new Runnable()
			{
				@Override
				public void run()
				{
					setDaten(tsfAbschnittDiagnoseDaten);
				}
			};

			if (PlatformUI.isWorkbenchRunning())
			{
				if ((PlatformUI.getWorkbench() != null)
						&& (!PlatformUI.getWorkbench().isClosing()))
				{
					if ((PlatformUI.getWorkbench().getDisplay() != null)
							&& (!PlatformUI.getWorkbench().getDisplay().isDisposed()))
					{
						PlatformUI.getWorkbench().getDisplay().asyncExec(run);
					}
				}
			}

			datenGesetzt = true;
		}

		return datenGesetzt;
	}

	private void setDaten(M_TSFAbschnittDiagnoseDaten tsfAbschnittDiagnoseDaten)
	{
		this._tsfAbschnittDiagnoseDaten = tsfAbschnittDiagnoseDaten;

		if (!_top.isDisposed())
		{
			if (_tsfAbschnittDiagnoseDaten != null)
			{
				// Pid
				SystemObject so = _tsfAbschnittDiagnoseDaten.getSo();
				if (so != null)
				{
					_tAbschnitt.setText(so.getNameOrPidOrId());
				} else
				{
					_tAbschnitt.setText("-");
				}

				// ZeitbereichKeineFreigabe
				setzeBooleanTextFeld(_tZeitbereichKeineFreigabe,
						_tsfAbschnittDiagnoseDaten.getZeitbereichKeineFreigabe());

				// PrioZeitbereichKeineFreigabe
				setzeLongTextFeld(_tPrioZeitbereichKeineFreigabe,
						_tsfAbschnittDiagnoseDaten.getPrioZeitbereichKeineFreigabe());

				// ZeitbereichFreigabe
				setzeBooleanTextFeld(_tZeitbereichFreigabe, _tsfAbschnittDiagnoseDaten
						.getZeitbereichFreigabe());

				// PrioZeitbereichFreigabe
				setzeLongTextFeld(_tPrioZeitbereichFreigabe, _tsfAbschnittDiagnoseDaten
						.getPrioZeitbereichFreigabe());

				// ZeitbereichFreigabeDurchAutomatik
				setzeBooleanTextFeld(_tZeitbereichFreigabeDurchAutomatik,
						_tsfAbschnittDiagnoseDaten.getZeitbereichFreigabeDurchAutomatik());

				// PrioZeitbereichFreigabeDurchAutomatik
				setzeLongTextFeld(_tPrioZeitbereichFreigabeDurchAutomatik,
						_tsfAbschnittDiagnoseDaten
								.getPrioZeitbereichFreigabeDurchAutomatik());

				// TSFSollZustand
				if (_tsfAbschnittDiagnoseDaten.getTsfSollZustand() != null)
				{
					_tTSFSollZustand.setText(_tsfAbschnittDiagnoseDaten
							.getTsfSollZustand().toString());
				} else
				{
					_tTSFSollZustand.setText("-");
				}

				// TSFIstZustand

				if (_tsfAbschnittDiagnoseDaten.getTsfIstZustand() != null)
				{
					_tTSFIstZustand.setText(_tsfAbschnittDiagnoseDaten.getTsfIstZustand()
							.toString());
				} else
				{
					_tTSFIstZustand.setText("-");
				}
			}

			if (_tv != null)
			{
				_tv.setInput(_tsfAbschnittDiagnoseDaten);
			}
		}
	}

	private static void setzeBooleanTextFeld(Text text, Boolean bool)
	{
		if (text != null)
		{
			if (bool != null)
			{
				if (bool)
				{
					text.setText("Ja");
				} else
				{
					text.setText("Nein");
				}
			} else
			{
				text.setText("-");
			}
		}
	}

	private static void setzeLongTextFeld(Text text, Long wert)
	{
		if (text != null)
		{
			if (wert != null)
			{
				text.setText(wert.toString());
			} else
			{
				text.setText("-");
			}
		}
	}

	@Override
	public void dispose()
	{
		/*
		 * Listener Abmelden.
		 */
		DaVKomTSFAbschnittDiagnoseDaten.getInstance().getModel().removeListener(
				this);

		super.dispose();
	}

	private class CP implements IStructuredContentProvider
	{

		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = null;

			if ((inputElement != null)
					&& (inputElement instanceof M_TSFAbschnittDiagnoseDaten))
			{
				M_TSFAbschnittDiagnoseDaten tsfAbschnittDiagnoseDaten = (M_TSFAbschnittDiagnoseDaten) inputElement;
				re = tsfAbschnittDiagnoseDaten.getDiagnoseDaten();
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	private class LP implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			return null;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			String re = null;

			if (element instanceof M_TSFMQDiagnoseDaten)
			{
				M_TSFMQDiagnoseDaten tsfMqDiagnoseDaten = (M_TSFMQDiagnoseDaten) element;
				M_TSFMQDiagnoseParameter parameter = tsfMqDiagnoseDaten.getParameter();
				M_TSFMQDiagnoseWerte werte = tsfMqDiagnoseDaten.getWerte();

				switch (columnIndex)
				{
				case 0:
					if (tsfMqDiagnoseDaten.getMessQuerschnitt() != null)
					{
						re = tsfMqDiagnoseDaten.getMessQuerschnitt().getNameOrPidOrId();
					}
					break;
				case 1:
					if (tsfMqDiagnoseDaten.getEmpfehlung() != null)
					{
						re = tsfMqDiagnoseDaten.getEmpfehlung().toString();
					}
					break;
				case 2:
					if ((parameter != null) && (parameter.getQbEin() != null))
					{
						re = parameter.getQbEin().toString();
					}
					break;
				case 3:
					if ((parameter != null) && (parameter.getvKfzEin() != null))
					{
						re = parameter.getvKfzEin().toString();
					}
					break;
				case 4:
					if ((parameter != null)
							&& (parameter.getAnzahlMessZyklenEin() != null))
					{
						re = parameter.getAnzahlMessZyklenEin().toString();
					}
					break;
				case 5:
					if ((parameter != null)
							&& (parameter.getAnzahlMessZyklenAus() != null))
					{
						re = parameter.getAnzahlMessZyklenAus().toString();
					}
					break;
				case 6:
					if ((werte != null) && (werte.getQb() != null))
					{
						re = werte.getQb().toString();
					}
					break;
				case 7:
					if ((werte != null) && (werte.getV() != null))
					{
						re = werte.getV().toString();
					}
					break;
				case 8:
					if ((werte != null) && (werte.getAnzahlMessZyklenEin() != null))
					{
						re = werte.getAnzahlMessZyklenEin().toString();
					}
					break;
				case 9:
					if ((werte != null) && (werte.getAnzahlMessZyklenAus() != null))
					{
						re = werte.getAnzahlMessZyklenAus().toString();
					}
					break;
				}

			}

			if (re == null)
			{
				re = "-";
			}
			return re;
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}
}
