/* 
 * (C) by Friederich Kupzog Electronics&Software Cologne, Germany 2005
 * All rights reserved.
 */

package de.kupzog.ktools.kprint.boxes;

import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

/**
 * This is a simple extension of PLittleTextBox
 * that does not allow wrapping but therefire
 * clipping of the single-line content.
 *
 * @author Friederich Kupzog
 */

public class PClippingTextBox extends PLittleTextBox {

	public PClippingTextBox(PContainer parent) {
		super(parent);
	}

	public PClippingTextBox(PContainer parent, int style) {
		super(parent, style);
	}

	/**
	 * Creates a text box with specified width.
	 * @param parent
	 * @param style
	 * @param hWeight
	 * Spezify a number between 0 and 1 for a textbox
	 * that consumes the given fraction of the available document width.
	 * @param minWidth
	 * This allows you to specify a minimum width for the text.
	 * The text box will consume some space depending to hWeight or its text
	 * if hWeight is -1, but at least the given amount of centimeters.
	 * For a box with a fixed width for example set hWeigth = 0 and specify a non-zero
	 * minWidth.
	 */
	public PClippingTextBox(PContainer parent, int style, double hWeight,
			double minWidth) {
		super(parent, style, hWeight, minWidth);
	}
	
	protected void splitIntoLines() {
		String txt = text.replace('\n',' ');
		gc.setFont(textStyle.getFont());

		// Do sth. if we have a specified width
		if ((grabWidth > 0) || (hWeight >= 0 && hWeight <= 1))
		{
			PDocument myDoc = getDocument();
			int maxWidth;
			
			if (grabWidth > 0) maxWidth = grabWidth;
			else
			{
				double maxWidthCm = (myDoc.pageWidth-myDoc.margins[1]-myDoc.margins[3]) * hWeight;
				maxWidth = Math.max(pixelX(maxWidthCm), pixelX(minCm));
			}
			maxWidth -= pixelX(textStyle.getMarginLeft());
			maxWidth -= pixelX(textStyle.getMarginRight());
	
			// if it fits, that's it.
			int textLength = gc.stringExtent(txt).x;
			if (textLength <= maxWidth) 
			{
				textLines.add(txt);
				return;
			}
			
			// if not, substract "..."
			maxWidth -= gc.stringExtent("..").x;
			for (int i = txt.length(); i > 0 ; i--) {
				if (gc.stringExtent(txt.substring(0, i)).x < maxWidth)
				{
					textLines.add(txt.substring(0, i)+"..");
					return;
				}
			}
			textLines.add("");
		}
	}
	
	
	public void draw(int page, Point originOffset) {
		if (layoutIsOnPage(page))
		{
			Rectangle tmp = gc.getClipping();
			gc.setClipping(
					origin.x + originOffset.x, 
					origin.y + originOffset.y, 
					getWidth(), 
					getHeight());
			super.draw(page, originOffset);
			gc.setClipping(tmp);
		}
	}


}
