/*  Copyright (C) 2004 by Friederich Kupzog Elektronik & Software

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   
	Author: Friederich Kupzog  
	fkmk@kupzog.de
	www.kupzog.de/fkmk
 */

/*
 * This feature was contributed by Onsel Armagan, Istanbul, Turkey
 * Thanks a lot!
 */

package de.kupzog.ktools.kprint.examples;

import org.eclipse.swt.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.printing.*;

import de.kupzog.ktools.kprint.boxes.*;
import de.kupzog.ktools.kprint.gui.IconSource;
import de.kupzog.ktools.kprint.gui.PrintPreview;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;

/**
 * This example shows how to print data from a KTableModel. More information about this can be found in the text that is
 * produced by KPrintExample.java.
 * 
 * @author Friederich Kupzog
 * 
 */
public class PrintSWTTableExample
{
  private Display display;

  public PrintSWTTableExample()
  {

    // create some gui with a SWT table
    display = new Display();
    final Shell shell = new Shell(display);
    final org.eclipse.swt.layout.GridLayout gridLayout = new org.eclipse.swt.layout.GridLayout();
    gridLayout.numColumns = 2;
    shell.setLayout(gridLayout);
    // final Color red = display.getSystemColor(SWT.COLOR_RED);

    final Table table_swt = new Table(shell, SWT.BORDER | SWT.FULL_SELECTION);
    final GridData gridData = new GridData(GridData.FILL_BOTH);
    gridData.horizontalSpan = 2;
    table_swt.setLayoutData(gridData);
    table_swt.setLinesVisible(true);
    TableColumn column1 = new TableColumn(table_swt, SWT.NONE);
    TableColumn column2 = new TableColumn(table_swt, SWT.NONE);
    TableColumn column3 = new TableColumn(table_swt, SWT.NONE);
    column1.setText("Column1");
    column2.setText("Column2");
    column3.setText("Column3");

    for (int i = 0; i < 100; i++)
    {
      TableItem item = new TableItem(table_swt, SWT.NONE);
      item.setText(new String[] { "cell " + i + " 0", "cell " + i + " 1", "cell " + i + "2" });
    }
    column1.pack();
    column2.pack();
    column3.pack();

    final Button butPrint = new Button(shell, SWT.NONE);
    butPrint.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e)
      {
        // create a document with default settings from PageSetup
        final PDocument doc = new PDocument("SWT Table Printing Example");

        // put some header text on it
        PTextBox t;

        t = new PTextBox(doc);
        t.setText("SWT Table Printing Example");

        new PVSpace(doc, 0.1);
        new PHLine(doc, 0.02, SWT.COLOR_BLACK);
        new PVSpace(doc, 0.5);

        // create the table
        SWTPTable table = new SWTPTable(doc);
        table.setTable(table_swt);
        table.setBoxProvider(new PTableBoxProvider());

        // PrintPreview pr = new PrintPreview(null, "Test", IconSource.getImage("print"), doc);
        // pr.getPageImage(doc);
        // pr.open();

        print(doc);

      }
    });
    butPrint.setText("Print");

    final Button butClose = new Button(shell, SWT.NONE);
    butClose.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e)
      {
        shell.dispose();
      }
    });
    butClose.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
    butClose.setText("Close");

    // Show the shell
    shell.setSize(300, 400);
    shell.setText("SWT Table Printing Example");
    shell.setVisible(true);

    while (!shell.isDisposed())
    {
      if (!display.readAndDispatch())
        display.sleep();
    }

    display.dispose();
  }

  /**
   * This function would print the document witout the print preview.
   * 
   * @param doc
   */
  public void print(PDocument doc)
  {

    PrintDialog dialog = new PrintDialog(new Shell(display), SWT.BORDER);
    PrinterData data = dialog.open();
    if (data == null)
      return;
    if (data.printToFile) {
      FileDialog d = new FileDialog (new Shell(display), SWT.SAVE);
      d.setFilterNames (new String [] {"All Files (*.*)"}); //$NON-NLS-1$
      d.setFilterExtensions (new String [] {"*.*"}); //Windows wild cards //$NON-NLS-1$
      d.setFilterPath ("c:\\"); //Windows path //$NON-NLS-1$
      d.setFileName (""); //$NON-NLS-1$
      d.open ();
      data.fileName = d.getFilterPath() + "\\" + d.getFileName(); //$NON-NLS-1$
      
    }
    
    if (data.printToFile)
    {
      data.fileName = "print.out"; // you probably want to ask the user for a filename
    }

    Printer printer = new Printer(data);
    GC gc = new GC(printer);
    PBox.setParameters(gc, printer, printer.getDPI(), (int)(doc.getScale()*100));

    if (printer.startJob(doc.getTitle()))
    {
      if (data.scope == PrinterData.ALL_PAGES)
      {
        data.startPage = 1;
        data.endPage = doc.getNumOfPages();
      }
      else if (data.scope == PrinterData.SELECTION)
      {
        // data.startPage = page;
        // data.endPage = page;
      }
      else if (data.scope == PrinterData.PAGE_RANGE)
      {
        if (data.startPage > doc.getNumOfPages())
          data.startPage = doc.getNumOfPages();
        if (data.endPage > doc.getNumOfPages())
          data.endPage = doc.getNumOfPages();
      }
      printer.startPage();
      doc.layout();
      doc.getNumOfPages();
      for (int i = 0; i < doc.getNumOfPages(); i++)
      {
        if (i != 0)
          printer.startPage();
        doc.draw(i + 1);
        printer.endPage();

      }

      printer.endJob();

    }

    gc.dispose();
    printer.dispose();

  }

  public static void main(String[] args)
  {
    new PrintSWTTableExample();
  }
}